/*----------------------------------------------------------------------*\
 | Data structures and functions for determining how to add elements to	|
 | elements of compound or scalar type.									|
 |																		|
 | Peter N. Schweitzer (U.S. Geological Survey, Reston, VA 20192)		|
\*----------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "keyword.h"
#include "stricmp.h"

#define MAX_COMPOUND_COUNT	1024
#define MAX_COMPONENT_COUNT	31

struct compound {
	enum fgdc_keyword key;
	enum fgdc_keyword component[1 + MAX_COMPONENT_COUNT];
	};

static struct compound standard_element[1 + MAX_COMPOUND_COUNT] = {
	{WMetadata,{
		WIdentification_Information,
		WData_Quality_Information,
		WSpatial_Data_Organization_Information,
		WSpatial_Reference_Information,
		WEntity_and_Attribute_Information,
		WDistribution_Information,
		WMetadata_Reference_Information,
		Wnull}},
	{WIdentification_Information,{
		WCitation,
		WDescription,
		WTime_Period_of_Content,
		WStatus,
		WSpatial_Domain,
		WKeywords,
		WAccess_Constraints,
		WUse_Constraints,
		WPoint_of_Contact,
		WBrowse_Graphic,
		WData_Set_Credit,
		WSecurity_Information,
		WNative_Data_Set_Environment,
		WCross_Reference,
		Wnull}},
	{WCitation,{
		WCitation_Information,
		Wnull}},
	{WDescription,{
		WAbstract,
		WPurpose,
		WSupplemental_Information,
		Wnull}},
	{WTime_Period_of_Content,{
		WTime_Period_Information,
		WCurrentness_Reference,
		Wnull}},
	{WStatus,{
		WProgress,
		WMaintenance_and_Update_Frequency,
		Wnull}},
	{WSpatial_Domain,{
		WBounding_Coordinates,
		WData_Set_G_Polygon,
		Wnull}},
	{WBounding_Coordinates,{
		WWest_Bounding_Coordinate,
		WEast_Bounding_Coordinate,
		WNorth_Bounding_Coordinate,
		WSouth_Bounding_Coordinate,
		Wnull}},
	{WData_Set_G_Polygon,{
		WData_Set_G_Polygon_Outer_G_Ring,
		WData_Set_G_Polygon_Exclusion_G_Ring,
		Wnull}},
	{WData_Set_G_Polygon_Outer_G_Ring,{
		WG_Ring_Point,
		WG_Ring,
		Wnull}},
	{WData_Set_G_Polygon_Exclusion_G_Ring,{
		WG_Ring_Point,
		WG_Ring,
		Wnull}},
	{WG_Ring_Point,{
		WG_Ring_Latitude,
		WG_Ring_Longitude,
		Wnull}},
	{WKeywords,{
		WTheme,
		WPlace,
		WStratum,
		WTemporal,
		Wnull}},
	{WTheme,{
		WTheme_Keyword_Thesaurus,
		WTheme_Keyword,
		Wnull}},
	{WPlace,{
		WPlace_Keyword_Thesaurus,
		WPlace_Keyword,
		Wnull}},
	{WStratum,{
		WStratum_Keyword_Thesaurus,
		WStratum_Keyword,
		Wnull}},
	{WTemporal,{
		WTemporal_Keyword_Thesaurus,
		WTemporal_Keyword,
		Wnull}},
	{WPoint_of_Contact,{
		WContact_Information,
		Wnull}},
	{WBrowse_Graphic,{
		WBrowse_Graphic_File_Name,
		WBrowse_Graphic_File_Description,
		WBrowse_Graphic_File_Type,
		Wnull}},
	{WSecurity_Information,{
		WSecurity_Classification_System,
		WSecurity_Classification,
		WSecurity_Handling_Description,
		Wnull}},
	{WCross_Reference,{
		WCitation_Information,
		Wnull}},
	{WData_Quality_Information,{
		WAttribute_Accuracy,
		WLogical_Consistency_Report,
		WCompleteness_Report,
		WPositional_Accuracy,
		WLineage,
		WCloud_Cover,
		Wnull}},
	{WAttribute_Accuracy,{
		WAttribute_Accuracy_Report,
		WQuantitative_Attribute_Accuracy_Assessment,
		Wnull}},
	{WQuantitative_Attribute_Accuracy_Assessment,{
		WAttribute_Accuracy_Value,
		WAttribute_Accuracy_Explanation,
		Wnull}},
	{WPositional_Accuracy,{
		WHorizontal_Positional_Accuracy,
		WVertical_Positional_Accuracy,
		Wnull}},
	{WHorizontal_Positional_Accuracy,{
		WHorizontal_Positional_Accuracy_Report,
		WQuantitative_Horizontal_Positional_Accuracy_Assessment,
		Wnull}},
	{WQuantitative_Horizontal_Positional_Accuracy_Assessment,{
		WHorizontal_Positional_Accuracy_Value,
		WHorizontal_Positional_Accuracy_Explanation,
		Wnull}},
	{WVertical_Positional_Accuracy,{
		WVertical_Positional_Accuracy_Report,
		WQuantitative_Vertical_Positional_Accuracy_Assessment,
		Wnull}},
	{WQuantitative_Vertical_Positional_Accuracy_Assessment,{
		WVertical_Positional_Accuracy_Value,
		WVertical_Positional_Accuracy_Explanation,
		Wnull}},
	{WLineage,{
		WSource_Information,
		WProcess_Step,
		Wnull}},
	{WSource_Information,{
		WSource_Citation,
		WSource_Scale_Denominator,
		WType_of_Source_Media,
		WSource_Time_Period_of_Content,
		WSource_Citation_Abbreviation,
		WSource_Contribution,
		Wnull}},
	{WSource_Citation,{
		WCitation_Information,
		Wnull}},
	{WSource_Time_Period_of_Content,{
		WTime_Period_Information,
		WSource_Currentness_Reference,
		Wnull}},
	{WProcess_Step,{
		WProcess_Description,
		WSource_Used_Citation_Abbreviation,
		WProcess_Date,
		WProcess_Time,
		WSource_Produced_Citation_Abbreviation,
		WProcess_Contact,
		Wnull}},
	{WProcess_Contact,{
		WContact_Information,
		Wnull}},
	{WSpatial_Data_Organization_Information,{
		WIndirect_Spatial_Reference,
		WDirect_Spatial_Reference_Method,
		WPoint_and_Vector_Object_Information,
		WRaster_Object_Information,
		Wnull}},
	{WPoint_and_Vector_Object_Information,{
		WSDTS_Terms_Description,
		WVPF_Terms_Description,
		Wnull}},
	{WSDTS_Terms_Description,{
		WSDTS_Point_and_Vector_Object_Type,
		WPoint_and_Vector_Object_Count,
		Wnull}},
	{WVPF_Terms_Description,{
		WVPF_Topology_Level,
		WVPF_Point_and_Vector_Object_Information,
		Wnull}},
	{WVPF_Point_and_Vector_Object_Information,{
		WVPF_Point_and_Vector_Object_Type,
		WPoint_and_Vector_Object_Count,
		Wnull}},
	{WRaster_Object_Information,{
		WRaster_Object_Type,
		WRow_Count,
		WColumn_Count,
		WVertical_Count,
		Wnull}},
	{WSpatial_Reference_Information,{
		WHorizontal_Coordinate_System_Definition,
		WVertical_Coordinate_System_Definition,
		Wnull}},
	{WHorizontal_Coordinate_System_Definition,{
		WGeographic,
		WPlanar,
		WLocal,
		WGeodetic_Model,
		Wnull}},
	{WGeographic,{
		WLatitude_Resolution,
		WLongitude_Resolution,
		WGeographic_Coordinate_Units,
		Wnull}},
	{WPlanar,{
		WMap_Projection,
		WGrid_Coordinate_System,
		WLocal_Planar,
		WPlanar_Coordinate_Information,
		Wnull}},
	{WMap_Projection,{
		WMap_Projection_Name,
		WAlbers_Conical_Equal_Area,
		WAzimuthal_Equidistant,
		WEquidistant_Conic,
		WEquirectangular,
		WGeneral_Vertical_Near_sided_Perspective,
		WGnomonic,
		WLambert_Azimuthal_Equal_Area,
		WLambert_Conformal_Conic,
		WMercator,
		WModified_Stereographic_for_Alaska,
		WMiller_Cylindrical,
		WOblique_Mercator,
		WOrthographic,
		WPolar_Stereographic,
		WPolyconic,
		WRobinson,
		WSinusoidal,
		WSpace_Oblique_Mercator_Landsat,
		WStereographic,
		WTransverse_Mercator,
		Wvan_der_Grinten,
		WMap_Projection_Parameters,
		Wnull}},
	{WAlbers_Conical_Equal_Area,{
		WStandard_Parallel,
		WLongitude_of_Central_Meridian,
		WLatitude_of_Projection_Origin,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WAzimuthal_Equidistant,{
		WLongitude_of_Central_Meridian,
		WLatitude_of_Projection_Origin,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WEquidistant_Conic,{
		WStandard_Parallel,
		WLongitude_of_Central_Meridian,
		WLatitude_of_Projection_Origin,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WEquirectangular,{
		WStandard_Parallel,
		WLongitude_of_Central_Meridian,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WGeneral_Vertical_Near_sided_Perspective,{
		WHeight_of_Perspective_Point_Above_Surface,
		WLongitude_of_Projection_Center,
		WLatitude_of_Projection_Center,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WGnomonic,{
		WLongitude_of_Projection_Center,
		WLatitude_of_Projection_Center,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WLambert_Azimuthal_Equal_Area,{
		WLongitude_of_Projection_Center,
		WLatitude_of_Projection_Center,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WLambert_Conformal_Conic,{
		WStandard_Parallel,
		WLongitude_of_Central_Meridian,
		WLatitude_of_Projection_Origin,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WMercator,{
		WStandard_Parallel,
		WScale_Factor_at_Equator,
		WLongitude_of_Central_Meridian,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WModified_Stereographic_for_Alaska,{
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WMiller_Cylindrical,{
		WLongitude_of_Central_Meridian,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WOblique_Mercator,{
		WScale_Factor_at_Center_Line,
		WOblique_Line_Azimuth,
		WOblique_Line_Point,
		WLatitude_of_Projection_Origin,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WOblique_Line_Azimuth,{
		WAzimuthal_Angle,
		WAzimuth_Measure_Point_Longitude,
		Wnull}},
	{WOblique_Line_Point,{
		WOblique_Line_Latitude,
		WOblique_Line_Longitude,
		Wnull}},
	{WOrthographic,{
		WLongitude_of_Projection_Center,
		WLatitude_of_Projection_Center,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WPolar_Stereographic,{
		WStraight_Vertical_Longitude_from_Pole,
		WStandard_Parallel,
		WScale_Factor_at_Projection_Origin,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WPolyconic,{
		WLongitude_of_Central_Meridian,
		WLatitude_of_Projection_Origin,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WRobinson,{
		WLongitude_of_Projection_Center,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WSinusoidal,{
		WLongitude_of_Central_Meridian,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WSpace_Oblique_Mercator_Landsat,{
		WLandsat_Number,
		WPath_Number,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WStereographic,{
		WLongitude_of_Projection_Center,
		WLatitude_of_Projection_Center,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WTransverse_Mercator,{
		WScale_Factor_at_Central_Meridian,
		WLongitude_of_Central_Meridian,
		WLatitude_of_Projection_Origin,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{Wvan_der_Grinten,{
		WLongitude_of_Central_Meridian,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WMap_Projection_Parameters,{
		WAzimuth_Measure_Point_Longitude,
		WAzimuthal_Angle,
		WFalse_Easting,
		WFalse_Northing,
		WHeight_of_Perspective_Point_Above_Surface,
		WLandsat_Number,
		WLatitude_of_Projection_Center,
		WLatitude_of_Projection_Origin,
		WLongitude_of_Central_Meridian,
		WLongitude_of_Projection_Center,
		WOblique_Line_Latitude,
		WOblique_Line_Longitude,
		WPath_Number,
		WScale_Factor_at_Center_Line,
		WScale_Factor_at_Central_Meridian,
		WScale_Factor_at_Equator,
		WScale_Factor_at_Projection_Origin,
		WStandard_Parallel,
		WStraight_Vertical_Longitude_from_Pole,
		WOther_Projections_Definition,
		Wnull}},
	{WGrid_Coordinate_System,{
		WGrid_Coordinate_System_Name,
		WUniversal_Transverse_Mercator,
		WUniversal_Polar_Stereographic,
		WState_Plane_Coordinate_System,
		WARC_Coordinate_System,
		WOther_Grid_Systems_Definition,
		Wnull}},
	{WUniversal_Transverse_Mercator,{
		WUTM_Zone_Number,
		WTransverse_Mercator,
		Wnull}},
	{WUniversal_Polar_Stereographic,{
		WUPS_Zone_Identifier,
		WPolar_Stereographic,
		Wnull}},
	{WState_Plane_Coordinate_System,{
		WSPCS_Zone_Identifier,
		WLambert_Conformal_Conic,
		WTransverse_Mercator,
		WOblique_Mercator,
		WPolyconic,
		Wnull}},
	{WARC_Coordinate_System,{
		WARC_System_Zone_Identifier,
		WEquirectangular,
		WAzimuthal_Equidistant,
		Wnull}},
	{WLocal_Planar,{
		WLocal_Planar_Description,
		WLocal_Planar_Georeference_Information,
		Wnull}},
	{WPlanar_Coordinate_Information,{
		WPlanar_Coordinate_Encoding_Method,
		WCoordinate_Representation,
		WDistance_and_Bearing_Representation,
		WPlanar_Distance_Units,
		Wnull}},
	{WCoordinate_Representation,{
		WAbscissa_Resolution,
		WOrdinate_Resolution,
		Wnull}},
	{WDistance_and_Bearing_Representation,{
		WDistance_Resolution,
		WBearing_Resolution,
		WBearing_Units,
		WBearing_Reference_Direction,
		WBearing_Reference_Meridian,
		Wnull}},
	{WLocal,{
		WLocal_Description,
		WLocal_Georeference_Information,
		Wnull}},
	{WGeodetic_Model,{
		WHorizontal_Datum_Name,
		WEllipsoid_Name,
		WSemi_major_Axis,
		WDenominator_of_Flattening_Ratio,
		Wnull}},
	{WVertical_Coordinate_System_Definition,{
		WAltitude_System_Definition,
		WDepth_System_Definition,
		Wnull}},
	{WAltitude_System_Definition,{
		WAltitude_Datum_Name,
		WAltitude_Resolution,
		WAltitude_Distance_Units,
		WAltitude_Encoding_Method,
		Wnull}},
	{WDepth_System_Definition,{
		WDepth_Datum_Name,
		WDepth_Resolution,
		WDepth_Distance_Units,
		WDepth_Encoding_Method,
		Wnull}},
	{WEntity_and_Attribute_Information,{
		WDetailed_Description,
		WOverview_Description,
		Wnull}},
	{WDetailed_Description,{
		WEntity_Type,
		WAttribute,
		Wnull}},
	{WEntity_Type,{
		WEntity_Type_Label,
		WEntity_Type_Definition,
		WEntity_Type_Definition_Source,
		Wnull}},
	{WAttribute,{
		WAttribute_Label,
		WAttribute_Definition,
		WAttribute_Definition_Source,
		WAttribute_Domain_Values,
		WBeginning_Date_of_Attribute_Values,
		WEnding_Date_of_Attribute_Values,
		WAttribute_Value_Accuracy_Information,
		WAttribute_Measurement_Frequency,
		Wnull}},
	{WAttribute_Domain_Values,{
		WEnumerated_Domain,
		WRange_Domain,
		WCodeset_Domain,
		WUnrepresentable_Domain,
		Wnull}},
	{WEnumerated_Domain,{
		WEnumerated_Domain_Value,
		WEnumerated_Domain_Value_Definition,
		WEnumerated_Domain_Value_Definition_Source,
		WAttribute,
		Wnull}},
	{WRange_Domain,{
		WRange_Domain_Minimum,
		WRange_Domain_Maximum,
		WAttribute_Units_of_Measure,
		WAttribute_Measurement_Resolution,
		WAttribute,
		Wnull}},
	{WCodeset_Domain,{
		WCodeset_Name,
		WCodeset_Source,
		Wnull}},
	{WAttribute_Value_Accuracy_Information,{
		WAttribute_Value_Accuracy,
		WAttribute_Value_Accuracy_Explanation,
		Wnull}},
	{WOverview_Description,{
		WEntity_and_Attribute_Overview,
		WEntity_and_Attribute_Detail_Citation,
		Wnull}},
	{WDistribution_Information,{
		WDistributor,
		WResource_Description,
		WDistribution_Liability,
		WStandard_Order_Process,
		WCustom_Order_Process,
		WTechnical_Prerequisites,
		WAvailable_Time_Period,
		Wnull}},
	{WDistributor,{
		WContact_Information,
		Wnull}},
	{WStandard_Order_Process,{
		WNon_digital_Form,
		WDigital_Form,
		WFees,
		WOrdering_Instructions,
		WTurnaround,
		Wnull}},
	{WDigital_Form,{
		WDigital_Transfer_Information,
		WDigital_Transfer_Option,
		Wnull}},
	{WDigital_Transfer_Information,{
		WFormat_Name,
		WFormat_Version_Number,
		WFormat_Version_Date,
		WFormat_Specification ,
		WFormat_Information_Content,
		WFile_Decompression_Technique,
		WTransfer_Size,
		Wnull}},
	{WDigital_Transfer_Option,{
		WOnline_Option,
		WOffline_Option,
		Wnull}},
	{WOnline_Option,{
		WComputer_Contact_Information,
		WAccess_Instructions,
		WOnline_Computer_and_Operating_System,
		Wnull}},
	{WComputer_Contact_Information,{
		WNetwork_Address,
		WDialup_Instructions,
		Wnull}},
	{WNetwork_Address,{
		WNetwork_Resource_Name,
		Wnull}},
	{WDialup_Instructions,{
		WLowest_BPS,
		WHighest_BPS,
		WNumber_DataBits,
		WNumber_StopBits,
		WParity,
		WCompression_Support,
		WDialup_Telephone,
		WDialup_File_Name,
		Wnull}},
	{WOffline_Option,{
		WOffline_Media,
		WRecording_Capacity,
		WRecording_Format,
		WCompatibility_Information,
		Wnull}},
	{WRecording_Capacity,{
		WRecording_Density,
		WRecording_Density_Units,
		Wnull}},
	{WAvailable_Time_Period,{
		WTime_Period_Information,
		Wnull}},
	{WMetadata_Reference_Information,{
		WMetadata_Date,
		WMetadata_Review_Date,
		WMetadata_Future_Review_Date,
		WMetadata_Contact,
		WMetadata_Standard_Name,
		WMetadata_Standard_Version,
		WMetadata_Time_Convention,
		WMetadata_Access_Constraints,
		WMetadata_Use_Constraints,
		WMetadata_Security_Information,
		WMetadata_Extensions,
		WMetadata_Language,
		Wnull}},
	{WMetadata_Contact,{
		WContact_Information,
		Wnull}},
	{WMetadata_Security_Information,{
		WMetadata_Security_Classification_System,
		WMetadata_Security_Classification,
		WMetadata_Security_Handling_Description,
		Wnull}},
	{WMetadata_Extensions,{
		WOnline_Linkage,
		WProfile_Name,
		Wnull}},
	{WCitation_Information,{
		WOriginator,
		WPublication_Date,
		WPublication_Time,
		WTitle,
		WEdition,
		WGeospatial_Data_Presentation_Form,
		WSeries_Information,
		WPublication_Information,
		WOther_Citation_Details,
		WOnline_Linkage,
		WLarger_Work_Citation,
		Wnull}},
	{WSeries_Information,{
		WSeries_Name,
		WIssue_Identification,
		Wnull}},
	{WPublication_Information,{
		WPublication_Place,
		WPublisher,
		Wnull}},
	{WLarger_Work_Citation,{
		WCitation_Information,
		Wnull}},
	{WTime_Period_Information,{
		WSingle_Date_Time,
		WMultiple_Dates_Times,
		WRange_of_Dates_Times,
		Wnull}},
	{WSingle_Date_Time,{
		WCalendar_Date,
		WTime_of_Day,
		Wnull}},
	{WMultiple_Dates_Times,{
		WSingle_Date_Time,
		Wnull}},
	{WRange_of_Dates_Times,{
		WBeginning_Date,
		WBeginning_Time,
		WEnding_Date,
		WEnding_Time,
		Wnull}},
	{WContact_Information,{
		WContact_Person_Primary,
		WContact_Organization_Primary,
		WContact_Position,
		WContact_Address,
		WContact_Voice_Telephone,
		WContact_TDD_TTY_Telephone,
		WContact_Facsimile_Telephone,
		WContact_Electronic_Mail_Address,
		WHours_of_Service,
		WContact_Instructions,
		Wnull}},
	{WContact_Person_Primary,{
		WContact_Person,
		WContact_Organization,
		Wnull}},
	{WContact_Organization_Primary,{
		WContact_Organization,
		WContact_Person,
		Wnull}},
	{WContact_Address,{
		WAddress_Type,
		WAddress,
		WCity,
		WState_or_Province,
		WPostal_Code,
		WCountry,
		Wnull}},
	{Wnull,
		{Wnull}
		}
	};

/* Biological data profile */

static struct compound bioprofile_element[1 + MAX_COMPOUND_COUNT] = {
	{WMetadata,{
		WIdentification_Information,
		WData_Quality_Information,
		WSpatial_Data_Organization_Information,
		WSpatial_Reference_Information,
		WEntity_and_Attribute_Information,
		WDistribution_Information,
		WMetadata_Reference_Information,
		Wnull}},
	{WIdentification_Information,{
		WCitation,
		WDescription,
		WTime_Period_of_Content,
		WStatus,
		WSpatial_Domain,
		WKeywords,
		WTaxonomy,
		WAccess_Constraints,
		WUse_Constraints,
		WPoint_of_Contact,
		WBrowse_Graphic,
		WData_Set_Credit,
		WSecurity_Information,
		WNative_Data_Set_Environment,
		WCross_Reference,
		WAnalytical_Tool,
		Wnull}},
	{WCitation,{
		WCitation_Information,
		Wnull}},
	{WDescription,{
		WAbstract,
		WPurpose,
		WSupplemental_Information,
		Wnull}},
	{WTime_Period_of_Content,{
		WTime_Period_Information,
		WCurrentness_Reference,
		Wnull}},
	{WStatus,{
		WProgress,
		WMaintenance_and_Update_Frequency,
		Wnull}},
	{WSpatial_Domain,{
		WDescription_of_Geographic_Extent,
		WBounding_Coordinates,
		WData_Set_G_Polygon,
		Wnull}},
	{WBounding_Coordinates,{
		WWest_Bounding_Coordinate,
		WEast_Bounding_Coordinate,
		WNorth_Bounding_Coordinate,
		WSouth_Bounding_Coordinate,
		WBounding_Altitudes,
		Wnull}},
	{WBounding_Altitudes,{
		WAltitude_Minimum,
		WAltitude_Maximum,
		WAltitude_Distance_Units,
		Wnull}},
	{WData_Set_G_Polygon,{
		WData_Set_G_Polygon_Outer_G_Ring,
		WData_Set_G_Polygon_Exclusion_G_Ring,
		Wnull}},
	{WData_Set_G_Polygon_Outer_G_Ring,{
		WG_Ring_Point,
		WG_Ring,
		Wnull}},
	{WData_Set_G_Polygon_Exclusion_G_Ring,{
		WG_Ring_Point,
		WG_Ring,
		Wnull}},
	{WG_Ring_Point,{
		WG_Ring_Latitude,
		WG_Ring_Longitude,
		Wnull}},
	{WKeywords,{
		WTheme,
		WPlace,
		WStratum,
		WTemporal,
		Wnull}},
	{WTheme,{
		WTheme_Keyword_Thesaurus,
		WTheme_Keyword,
		Wnull}},
	{WPlace,{
		WPlace_Keyword_Thesaurus,
		WPlace_Keyword,
		Wnull}},
	{WStratum,{
		WStratum_Keyword_Thesaurus,
		WStratum_Keyword,
		Wnull}},
	{WTemporal,{
		WTemporal_Keyword_Thesaurus,
		WTemporal_Keyword,
		Wnull}},
	{WTaxonomy,{
		WKeywords_Taxon,
		WTaxonomic_System,
		WGeneral_Taxonomic_Coverage,
		WTaxonomic_Classification,
		Wnull}},
	{WKeywords_Taxon,{
		WTaxonomic_Keyword_Thesaurus,
		WTaxonomic_Keywords,
		Wnull}},
	{WTaxonomic_System,{
		WClassification_System_Authority,
		WIdentification_Reference,
		WIdentifier,
		WTaxonomic_Procedures,
		WTaxonomic_Completeness,
		WVouchers,
		Wnull}},
	{WClassification_System_Authority,{
		WClassification_System_Citation,
		WClassification_System_Modifications,
		Wnull}},
	{WClassification_System_Citation,{
		WCitation_Information,
		Wnull}},
	{WIdentification_Reference,{
		WCitation_Information,
		Wnull}},
	{WIdentifier,{
		WContact_Information,
		Wnull}},
	{WVouchers,{
		WSpecimen,
		WRepository,
		Wnull}},
	{WRepository,{
		WContact_Information,
		Wnull}},
	{WTaxonomic_Classification,{
		WTaxon_Rank_Name,
		WTaxon_Rank_Value,
		WApplicable_Common_Name,
		WTaxonomic_Classification,
		Wnull}},
	{WPoint_of_Contact,{
		WContact_Information,
		Wnull}},
	{WBrowse_Graphic,{
		WBrowse_Graphic_File_Name,
		WBrowse_Graphic_File_Description,
		WBrowse_Graphic_File_Type,
		Wnull}},
	{WSecurity_Information,{
		WSecurity_Classification_System,
		WSecurity_Classification,
		WSecurity_Handling_Description,
		Wnull}},
	{WCross_Reference,{
		WCitation_Information,
		Wnull}},
	{WAnalytical_Tool,{
		WAnalytical_Tool_Description,
		WTool_Access_Information,
		WTool_Contact,
		WTool_Citation,
		Wnull}},
	{WTool_Access_Information,{
		WOnline_Linkage,
		WTool_Access_Instructions,
		WTool_Computer_and_Operating_System,
		Wnull}},
	{WTool_Contact,{
		WContact_Information,
		Wnull}},
	{WTool_Citation,{
		WCitation_Information,
		Wnull}},
	{WData_Quality_Information,{
		WAttribute_Accuracy,
		WLogical_Consistency_Report,
		WCompleteness_Report,
		WPositional_Accuracy,
		WLineage,
		WCloud_Cover,
		Wnull}},
	{WAttribute_Accuracy,{
		WAttribute_Accuracy_Report,
		WQuantitative_Attribute_Accuracy_Assessment,
		Wnull}},
	{WQuantitative_Attribute_Accuracy_Assessment,{
		WAttribute_Accuracy_Value,
		WAttribute_Accuracy_Explanation,
		Wnull}},
	{WPositional_Accuracy,{
		WHorizontal_Positional_Accuracy,
		WVertical_Positional_Accuracy,
		Wnull}},
	{WHorizontal_Positional_Accuracy,{
		WHorizontal_Positional_Accuracy_Report,
		WQuantitative_Horizontal_Positional_Accuracy_Assessment,
		Wnull}},
	{WQuantitative_Horizontal_Positional_Accuracy_Assessment,{
		WHorizontal_Positional_Accuracy_Value,
		WHorizontal_Positional_Accuracy_Explanation,
		Wnull}},
	{WVertical_Positional_Accuracy,{
		WVertical_Positional_Accuracy_Report,
		WQuantitative_Vertical_Positional_Accuracy_Assessment,
		Wnull}},
	{WQuantitative_Vertical_Positional_Accuracy_Assessment,{
		WVertical_Positional_Accuracy_Value,
		WVertical_Positional_Accuracy_Explanation,
		Wnull}},
	{WLineage,{
		WMethodology,
		WSource_Information,
		WProcess_Step,
		Wnull}},
	{WMethodology,{
		WMethodology_Type,
		WMethodology_Identifier,
		WMethodology_Description,
		WMethodology_Citation,
		Wnull}},
	{WMethodology_Identifier,{
		WMethodology_Keyword_Thesaurus,
		WMethodology_Keyword,
		Wnull}},
	{WMethodology_Citation,{
		WCitation_Information,
		Wnull}},
	{WSource_Information,{
		WSource_Citation,
		WSource_Scale_Denominator,
		WType_of_Source_Media,
		WSource_Time_Period_of_Content,
		WSource_Citation_Abbreviation,
		WSource_Contribution,
		Wnull}},
	{WSource_Citation,{
		WCitation_Information,
		Wnull}},
	{WSource_Time_Period_of_Content,{
		WTime_Period_Information,
		WSource_Currentness_Reference,
		Wnull}},
	{WProcess_Step,{
		WProcess_Description,
		WSource_Used_Citation_Abbreviation,
		WProcess_Date,
		WProcess_Time,
		WSource_Produced_Citation_Abbreviation,
		WProcess_Contact,
		Wnull}},
	{WProcess_Contact,{
		WContact_Information,
		Wnull}},
	{WSpatial_Data_Organization_Information,{
		WIndirect_Spatial_Reference,
		WDirect_Spatial_Reference_Method,
		WPoint_and_Vector_Object_Information,
		WRaster_Object_Information,
		Wnull}},
	{WPoint_and_Vector_Object_Information,{
		WSDTS_Terms_Description,
		WVPF_Terms_Description,
		Wnull}},
	{WSDTS_Terms_Description,{
		WSDTS_Point_and_Vector_Object_Type,
		WPoint_and_Vector_Object_Count,
		Wnull}},
	{WVPF_Terms_Description,{
		WVPF_Topology_Level,
		WVPF_Point_and_Vector_Object_Information,
		Wnull}},
	{WVPF_Point_and_Vector_Object_Information,{
		WVPF_Point_and_Vector_Object_Type,
		WPoint_and_Vector_Object_Count,
		Wnull}},
	{WRaster_Object_Information,{
		WRaster_Object_Type,
		WRow_Count,
		WColumn_Count,
		WVertical_Count,
		Wnull}},
	{WSpatial_Reference_Information,{
		WHorizontal_Coordinate_System_Definition,
		WVertical_Coordinate_System_Definition,
		Wnull}},
	{WHorizontal_Coordinate_System_Definition,{
		WGeographic,
		WPlanar,
		WLocal,
		WGeodetic_Model,
		Wnull}},
	{WGeographic,{
		WLatitude_Resolution,
		WLongitude_Resolution,
		WGeographic_Coordinate_Units,
		Wnull}},
	{WPlanar,{
		WMap_Projection,
		WGrid_Coordinate_System,
		WLocal_Planar,
		WPlanar_Coordinate_Information,
		Wnull}},
	{WMap_Projection,{
		WMap_Projection_Name,
		WAlbers_Conical_Equal_Area,
		WAzimuthal_Equidistant,
		WEquidistant_Conic,
		WEquirectangular,
		WGeneral_Vertical_Near_sided_Perspective,
		WGnomonic,
		WLambert_Azimuthal_Equal_Area,
		WLambert_Conformal_Conic,
		WMercator,
		WModified_Stereographic_for_Alaska,
		WMiller_Cylindrical,
		WOblique_Mercator,
		WOrthographic,
		WPolar_Stereographic,
		WPolyconic,
		WRobinson,
		WSinusoidal,
		WSpace_Oblique_Mercator_Landsat,
		WStereographic,
		WTransverse_Mercator,
		Wvan_der_Grinten,
		WMap_Projection_Parameters,
		Wnull}},
	{WAlbers_Conical_Equal_Area,{
		WStandard_Parallel,
		WLongitude_of_Central_Meridian,
		WLatitude_of_Projection_Origin,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WAzimuthal_Equidistant,{
		WLongitude_of_Central_Meridian,
		WLatitude_of_Projection_Origin,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WEquidistant_Conic,{
		WStandard_Parallel,
		WLongitude_of_Central_Meridian,
		WLatitude_of_Projection_Origin,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WEquirectangular,{
		WStandard_Parallel,
		WLongitude_of_Central_Meridian,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WGeneral_Vertical_Near_sided_Perspective,{
		WHeight_of_Perspective_Point_Above_Surface,
		WLongitude_of_Projection_Center,
		WLatitude_of_Projection_Center,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WGnomonic,{
		WLongitude_of_Projection_Center,
		WLatitude_of_Projection_Center,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WLambert_Azimuthal_Equal_Area,{
		WLongitude_of_Projection_Center,
		WLatitude_of_Projection_Center,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WLambert_Conformal_Conic,{
		WStandard_Parallel,
		WLongitude_of_Central_Meridian,
		WLatitude_of_Projection_Origin,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WMercator,{
		WStandard_Parallel,
		WScale_Factor_at_Equator,
		WLongitude_of_Central_Meridian,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WModified_Stereographic_for_Alaska,{
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WMiller_Cylindrical,{
		WLongitude_of_Central_Meridian,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WOblique_Mercator,{
		WScale_Factor_at_Center_Line,
		WOblique_Line_Azimuth,
		WOblique_Line_Point,
		WLatitude_of_Projection_Origin,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WOblique_Line_Azimuth,{
		WAzimuthal_Angle,
		WAzimuth_Measure_Point_Longitude,
		Wnull}},
	{WOblique_Line_Point,{
		WOblique_Line_Latitude,
		WOblique_Line_Longitude,
		Wnull}},
	{WOrthographic,{
		WLongitude_of_Projection_Center,
		WLatitude_of_Projection_Center,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WPolar_Stereographic,{
		WStraight_Vertical_Longitude_from_Pole,
		WStandard_Parallel,
		WScale_Factor_at_Projection_Origin,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WPolyconic,{
		WLongitude_of_Central_Meridian,
		WLatitude_of_Projection_Origin,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WRobinson,{
		WLongitude_of_Projection_Center,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WSinusoidal,{
		WLongitude_of_Central_Meridian,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WSpace_Oblique_Mercator_Landsat,{
		WLandsat_Number,
		WPath_Number,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WStereographic,{
		WLongitude_of_Projection_Center,
		WLatitude_of_Projection_Center,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WTransverse_Mercator,{
		WScale_Factor_at_Central_Meridian,
		WLongitude_of_Central_Meridian,
		WLatitude_of_Projection_Origin,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{Wvan_der_Grinten,{
		WLongitude_of_Central_Meridian,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WMap_Projection_Parameters,{
		WAzimuth_Measure_Point_Longitude,
		WAzimuthal_Angle,
		WFalse_Easting,
		WFalse_Northing,
		WHeight_of_Perspective_Point_Above_Surface,
		WLandsat_Number,
		WLatitude_of_Projection_Center,
		WLatitude_of_Projection_Origin,
		WLongitude_of_Central_Meridian,
		WLongitude_of_Projection_Center,
		WOblique_Line_Latitude,
		WOblique_Line_Longitude,
		WPath_Number,
		WScale_Factor_at_Center_Line,
		WScale_Factor_at_Central_Meridian,
		WScale_Factor_at_Equator,
		WScale_Factor_at_Projection_Origin,
		WStandard_Parallel,
		WStraight_Vertical_Longitude_from_Pole,
		WOther_Projections_Definition,
		Wnull}},
	{WGrid_Coordinate_System,{
		WGrid_Coordinate_System_Name,
		WUniversal_Transverse_Mercator,
		WUniversal_Polar_Stereographic,
		WState_Plane_Coordinate_System,
		WARC_Coordinate_System,
		WOther_Grid_Systems_Definition,
		Wnull}},
	{WUniversal_Transverse_Mercator,{
		WUTM_Zone_Number,
		WTransverse_Mercator,
		Wnull}},
	{WUniversal_Polar_Stereographic,{
		WUPS_Zone_Identifier,
		WPolar_Stereographic,
		Wnull}},
	{WState_Plane_Coordinate_System,{
		WSPCS_Zone_Identifier,
		WLambert_Conformal_Conic,
		WTransverse_Mercator,
		WOblique_Mercator,
		WPolyconic,
		Wnull}},
	{WARC_Coordinate_System,{
		WARC_System_Zone_Identifier,
		WEquirectangular,
		WAzimuthal_Equidistant,
		Wnull}},
	{WLocal_Planar,{
		WLocal_Planar_Description,
		WLocal_Planar_Georeference_Information,
		Wnull}},
	{WPlanar_Coordinate_Information,{
		WPlanar_Coordinate_Encoding_Method,
		WCoordinate_Representation,
		WDistance_and_Bearing_Representation,
		WPlanar_Distance_Units,
		Wnull}},
	{WCoordinate_Representation,{
		WAbscissa_Resolution,
		WOrdinate_Resolution,
		Wnull}},
	{WDistance_and_Bearing_Representation,{
		WDistance_Resolution,
		WBearing_Resolution,
		WBearing_Units,
		WBearing_Reference_Direction,
		WBearing_Reference_Meridian,
		Wnull}},
	{WLocal,{
		WLocal_Description,
		WLocal_Georeference_Information,
		Wnull}},
	{WGeodetic_Model,{
		WHorizontal_Datum_Name,
		WEllipsoid_Name,
		WSemi_major_Axis,
		WDenominator_of_Flattening_Ratio,
		Wnull}},
	{WVertical_Coordinate_System_Definition,{
		WAltitude_System_Definition,
		WDepth_System_Definition,
		Wnull}},
	{WAltitude_System_Definition,{
		WAltitude_Datum_Name,
		WAltitude_Resolution,
		WAltitude_Distance_Units,
		WAltitude_Encoding_Method,
		Wnull}},
	{WDepth_System_Definition,{
		WDepth_Datum_Name,
		WDepth_Resolution,
		WDepth_Distance_Units,
		WDepth_Encoding_Method,
		Wnull}},
	{WEntity_and_Attribute_Information,{
		WDetailed_Description,
		WOverview_Description,
		Wnull}},
	{WDetailed_Description,{
		WEntity_Type,
		WAttribute,
		Wnull}},
	{WEntity_Type,{
		WEntity_Type_Label,
		WEntity_Type_Definition,
		WEntity_Type_Definition_Source,
		Wnull}},
	{WAttribute,{
		WAttribute_Label,
		WAttribute_Definition,
		WAttribute_Definition_Source,
		WAttribute_Domain_Values,
		WBeginning_Date_of_Attribute_Values,
		WEnding_Date_of_Attribute_Values,
		WAttribute_Value_Accuracy_Information,
		WAttribute_Measurement_Frequency,
		Wnull}},
	{WAttribute_Domain_Values,{
		WEnumerated_Domain,
		WRange_Domain,
		WCodeset_Domain,
		WUnrepresentable_Domain,
		Wnull}},
	{WEnumerated_Domain,{
		WEnumerated_Domain_Value,
		WEnumerated_Domain_Value_Definition,
		WEnumerated_Domain_Value_Definition_Source,
		WAttribute,
		Wnull}},
	{WRange_Domain,{
		WRange_Domain_Minimum,
		WRange_Domain_Maximum,
		WAttribute_Units_of_Measure,
		WAttribute_Measurement_Resolution,
		WAttribute,
		Wnull}},
	{WCodeset_Domain,{
		WCodeset_Name,
		WCodeset_Source,
		Wnull}},
	{WAttribute_Value_Accuracy_Information,{
		WAttribute_Value_Accuracy,
		WAttribute_Value_Accuracy_Explanation,
		Wnull}},
	{WOverview_Description,{
		WEntity_and_Attribute_Overview,
		WEntity_and_Attribute_Detail_Citation,
		Wnull}},
	{WDistribution_Information,{
		WDistributor,
		WResource_Description,
		WDistribution_Liability,
		WStandard_Order_Process,
		WCustom_Order_Process,
		WTechnical_Prerequisites,
		WAvailable_Time_Period,
		Wnull}},
	{WDistributor,{
		WContact_Information,
		Wnull}},
	{WStandard_Order_Process,{
		WNon_digital_Form,
		WDigital_Form,
		WFees,
		WOrdering_Instructions,
		WTurnaround,
		Wnull}},
	{WDigital_Form,{
		WDigital_Transfer_Information,
		WDigital_Transfer_Option,
		Wnull}},
	{WDigital_Transfer_Information,{
		WFormat_Name,
		WFormat_Version_Number,
		WFormat_Version_Date,
		WFormat_Specification ,
		WASCII_File_Structure,
		WFormat_Information_Content,
		WFile_Decompression_Technique,
		WTransfer_Size,
		Wnull}},
	{WASCII_File_Structure,{
		WRecord_Delimiter,
		WNumber_Header_Lines,
		WDescription_of_Header_Content,
		WOrientation,
		WCase_Sensitive,
		WAuthentication,
		WQuote_Character,
		WData_Field,
		Wnull}},
	{WData_Field,{
		WData_Field_Name,
		WMissing_Value_Code,
		WData_Field_Width_Delimiter,
		WData_Field_Width,
		Wnull}},
	{WDigital_Transfer_Option,{
		WOnline_Option,
		WOffline_Option,
		Wnull}},
	{WOnline_Option,{
		WComputer_Contact_Information,
		WAccess_Instructions,
		WOnline_Computer_and_Operating_System,
		Wnull}},
	{WComputer_Contact_Information,{
		WNetwork_Address,
		WDialup_Instructions,
		Wnull}},
	{WNetwork_Address,{
		WNetwork_Resource_Name,
		Wnull}},
	{WDialup_Instructions,{
		WLowest_BPS,
		WHighest_BPS,
		WNumber_DataBits,
		WNumber_StopBits,
		WParity,
		WCompression_Support,
		WDialup_Telephone,
		WDialup_File_Name,
		Wnull}},
	{WOffline_Option,{
		WOffline_Media,
		WRecording_Capacity,
		WRecording_Format,
		WCompatibility_Information,
		Wnull}},
	{WRecording_Capacity,{
		WRecording_Density,
		WRecording_Density_Units,
		Wnull}},
	{WAvailable_Time_Period,{
		WTime_Period_Information,
		Wnull}},
	{WMetadata_Reference_Information,{
		WMetadata_Date,
		WMetadata_Review_Date,
		WMetadata_Future_Review_Date,
		WMetadata_Contact,
		WMetadata_Standard_Name,
		WMetadata_Standard_Version,
		WMetadata_Time_Convention,
		WMetadata_Access_Constraints,
		WMetadata_Use_Constraints,
		WMetadata_Security_Information,
		WMetadata_Extensions,
		WMetadata_Language,
		Wnull}},
	{WMetadata_Contact,{
		WContact_Information,
		Wnull}},
	{WMetadata_Security_Information,{
		WMetadata_Security_Classification_System,
		WMetadata_Security_Classification,
		WMetadata_Security_Handling_Description,
		Wnull}},
	{WMetadata_Extensions,{
		WOnline_Linkage,
		WProfile_Name,
		Wnull}},
	{WCitation_Information,{
		WOriginator,
		WPublication_Date,
		WPublication_Time,
		WTitle,
		WEdition,
		WGeospatial_Data_Presentation_Form,
		WSeries_Information,
		WPublication_Information,
		WOther_Citation_Details,
		WOnline_Linkage,
		WLarger_Work_Citation,
		Wnull}},
	{WSeries_Information,{
		WSeries_Name,
		WIssue_Identification,
		Wnull}},
	{WPublication_Information,{
		WPublication_Place,
		WPublisher,
		Wnull}},
	{WLarger_Work_Citation,{
		WCitation_Information,
		Wnull}},
	{WTime_Period_Information,{
		WSingle_Date_Time,
		WMultiple_Dates_Times,
		WRange_of_Dates_Times,
		Wnull}},
	{WSingle_Date_Time,{
		WCalendar_Date,
		WTime_of_Day,
		WGeologic_Age,
		Wnull}},
	{WMultiple_Dates_Times,{
		WSingle_Date_Time,
		Wnull}},
	{WRange_of_Dates_Times,{
		WBeginning_Date,
		WBeginning_Time,
		WEnding_Date,
		WEnding_Time,
		WBeginning_Geologic_Age,
		WEnding_Geologic_Age,
		Wnull}},
	{WBeginning_Geologic_Age,{
		WGeologic_Age,
		Wnull}},
	{WEnding_Geologic_Age,{
		WGeologic_Age,
		Wnull}},
	{WGeologic_Age,{
		WGeologic_Time_Scale,
		WGeologic_Age_Estimate,
		WGeologic_Age_Uncertainty,
		WGeologic_Age_Explanation,
		WGeologic_Citation,
		Wnull}},
	{WGeologic_Citation,{
		WCitation_Information,
		Wnull}},
	{WContact_Information,{
		WContact_Person_Primary,
		WContact_Organization_Primary,
		WContact_Position,
		WContact_Address,
		WContact_Voice_Telephone,
		WContact_TDD_TTY_Telephone,
		WContact_Facsimile_Telephone,
		WContact_Electronic_Mail_Address,
		WHours_of_Service,
		WContact_Instructions,
		Wnull}},
	{WContact_Person_Primary,{
		WContact_Person,
		WContact_Organization,
		Wnull}},
	{WContact_Organization_Primary,{
		WContact_Organization,
		WContact_Person,
		Wnull}},
	{WContact_Address,{
		WAddress_Type,
		WAddress,
		WCity,
		WState_or_Province,
		WPostal_Code,
		WCountry,
		Wnull}},
	{Wnull,
		{Wnull}
		}
	};

/* Shoreline profile */

static struct compound shprofile_element[1 + MAX_COMPOUND_COUNT] = {
	{WData_Quality_Information,{
		WAttribute_Accuracy,
		WLogical_Consistency_Report,
		WCompleteness_Report,
		WPositional_Accuracy,
		WLineage,
		WCloud_Cover,
		WTidal_Information,
		WMarine_Weather_Condition,
		WEnvironmental_Event,
		Wnull}},

	{WLineage,{
		WSource_Information,
		WProcess_Step,
		WProcess_Step_Citation,
		Wnull}},

	{WTidal_Information,{
		WType_of_Tide,
		WTime_of_Tide,
		WTide_Table_Reference,
		WSupplemental_Tidal_Information,
		Wnull}},

	{WTime_of_Tide,{
		WTime_of_Low_Tide,
		WTime_of_High_Tide,
		WTidal_Datum,
		WRange_of_Tide,
		Wnull}},

	{WMarine_Weather_Condition,{
		WWind_Speed,
		WWind_Direction,
		WWave_Height,
		WBarometric_Pressure,
		Wnull}},

	{WMetadata,{
		WIdentification_Information,
		WData_Quality_Information,
		WSpatial_Data_Organization_Information,
		WSpatial_Reference_Information,
		WEntity_and_Attribute_Information,
		WDistribution_Information,
		WMetadata_Reference_Information,
		Wnull}},
	{WIdentification_Information,{
		WCitation,
		WDescription,
		WTime_Period_of_Content,
		WStatus,
		WSpatial_Domain,
		WKeywords,
		WAccess_Constraints,
		WUse_Constraints,
		WPoint_of_Contact,
		WBrowse_Graphic,
		WData_Set_Credit,
		WSecurity_Information,
		WNative_Data_Set_Environment,
		WCross_Reference,
		Wnull}},
	{WCitation,{
		WCitation_Information,
		Wnull}},
	{WDescription,{
		WAbstract,
		WPurpose,
		WSupplemental_Information,
		Wnull}},
	{WTime_Period_of_Content,{
		WTime_Period_Information,
		WCurrentness_Reference,
		Wnull}},
	{WStatus,{
		WProgress,
		WMaintenance_and_Update_Frequency,
		Wnull}},
	{WSpatial_Domain,{
		WBounding_Coordinates,
		WData_Set_G_Polygon,
		WDescription_of_Geographic_Extent,
		Wnull}},
	{WBounding_Coordinates,{
		WWest_Bounding_Coordinate,
		WEast_Bounding_Coordinate,
		WNorth_Bounding_Coordinate,
		WSouth_Bounding_Coordinate,
		Wnull}},
	{WData_Set_G_Polygon,{
		WData_Set_G_Polygon_Outer_G_Ring,
		WData_Set_G_Polygon_Exclusion_G_Ring,
		Wnull}},
	{WData_Set_G_Polygon_Outer_G_Ring,{
		WG_Ring_Point,
		WG_Ring,
		Wnull}},
	{WData_Set_G_Polygon_Exclusion_G_Ring,{
		WG_Ring_Point,
		WG_Ring,
		Wnull}},
	{WG_Ring_Point,{
		WG_Ring_Latitude,
		WG_Ring_Longitude,
		Wnull}},
	{WKeywords,{
		WTheme,
		WPlace,
		WStratum,
		WTemporal,
		Wnull}},
	{WTheme,{
		WTheme_Keyword_Thesaurus,
		WTheme_Keyword,
		Wnull}},
	{WPlace,{
		WPlace_Keyword_Thesaurus,
		WPlace_Keyword,
		Wnull}},
	{WStratum,{
		WStratum_Keyword_Thesaurus,
		WStratum_Keyword,
		Wnull}},
	{WTemporal,{
		WTemporal_Keyword_Thesaurus,
		WTemporal_Keyword,
		Wnull}},
	{WPoint_of_Contact,{
		WContact_Information,
		Wnull}},
	{WBrowse_Graphic,{
		WBrowse_Graphic_File_Name,
		WBrowse_Graphic_File_Description,
		WBrowse_Graphic_File_Type,
		Wnull}},
	{WSecurity_Information,{
		WSecurity_Classification_System,
		WSecurity_Classification,
		WSecurity_Handling_Description,
		Wnull}},
	{WCross_Reference,{
		WCitation_Information,
		Wnull}},
	{WAttribute_Accuracy,{
		WAttribute_Accuracy_Report,
		WQuantitative_Attribute_Accuracy_Assessment,
		Wnull}},
	{WQuantitative_Attribute_Accuracy_Assessment,{
		WAttribute_Accuracy_Value,
		WAttribute_Accuracy_Explanation,
		Wnull}},
	{WPositional_Accuracy,{
		WHorizontal_Positional_Accuracy,
		WVertical_Positional_Accuracy,
		Wnull}},
	{WHorizontal_Positional_Accuracy,{
		WHorizontal_Positional_Accuracy_Report,
		WQuantitative_Horizontal_Positional_Accuracy_Assessment,
		Wnull}},
	{WQuantitative_Horizontal_Positional_Accuracy_Assessment,{
		WHorizontal_Positional_Accuracy_Value,
		WHorizontal_Positional_Accuracy_Explanation,
		Wnull}},
	{WVertical_Positional_Accuracy,{
		WVertical_Positional_Accuracy_Report,
		WQuantitative_Vertical_Positional_Accuracy_Assessment,
		Wnull}},
	{WQuantitative_Vertical_Positional_Accuracy_Assessment,{
		WVertical_Positional_Accuracy_Value,
		WVertical_Positional_Accuracy_Explanation,
		Wnull}},
	{WSource_Information,{
		WSource_Citation,
		WSource_Scale_Denominator,
		WType_of_Source_Media,
		WSource_Time_Period_of_Content,
		WSource_Citation_Abbreviation,
		WSource_Contribution,
		Wnull}},
	{WSource_Citation,{
		WCitation_Information,
		Wnull}},
	{WSource_Time_Period_of_Content,{
		WTime_Period_Information,
		WSource_Currentness_Reference,
		Wnull}},
	{WProcess_Step,{
		WProcess_Description,
		WSource_Used_Citation_Abbreviation,
		WProcess_Date,
		WProcess_Time,
		WSource_Produced_Citation_Abbreviation,
		WProcess_Contact,
		Wnull}},
	{WProcess_Contact,{
		WContact_Information,
		Wnull}},
	{WSpatial_Data_Organization_Information,{
		WIndirect_Spatial_Reference,
		WDirect_Spatial_Reference_Method,
		WPoint_and_Vector_Object_Information,
		WRaster_Object_Information,
		Wnull}},
	{WPoint_and_Vector_Object_Information,{
		WSDTS_Terms_Description,
		WVPF_Terms_Description,
		Wnull}},
	{WSDTS_Terms_Description,{
		WSDTS_Point_and_Vector_Object_Type,
		WPoint_and_Vector_Object_Count,
		Wnull}},
	{WVPF_Terms_Description,{
		WVPF_Topology_Level,
		WVPF_Point_and_Vector_Object_Information,
		Wnull}},
	{WVPF_Point_and_Vector_Object_Information,{
		WVPF_Point_and_Vector_Object_Type,
		WPoint_and_Vector_Object_Count,
		Wnull}},
	{WRaster_Object_Information,{
		WRaster_Object_Type,
		WRow_Count,
		WColumn_Count,
		WVertical_Count,
		Wnull}},
	{WSpatial_Reference_Information,{
		WHorizontal_Coordinate_System_Definition,
		WVertical_Coordinate_System_Definition,
		Wnull}},
	{WHorizontal_Coordinate_System_Definition,{
		WGeographic,
		WPlanar,
		WLocal,
		WGeodetic_Model,
		Wnull}},
	{WGeographic,{
		WLatitude_Resolution,
		WLongitude_Resolution,
		WGeographic_Coordinate_Units,
		Wnull}},
	{WPlanar,{
		WMap_Projection,
		WGrid_Coordinate_System,
		WLocal_Planar,
		WPlanar_Coordinate_Information,
		Wnull}},
	{WMap_Projection,{
		WMap_Projection_Name,
		WAlbers_Conical_Equal_Area,
		WAzimuthal_Equidistant,
		WEquidistant_Conic,
		WEquirectangular,
		WGeneral_Vertical_Near_sided_Perspective,
		WGnomonic,
		WLambert_Azimuthal_Equal_Area,
		WLambert_Conformal_Conic,
		WMercator,
		WModified_Stereographic_for_Alaska,
		WMiller_Cylindrical,
		WOblique_Mercator,
		WOrthographic,
		WPolar_Stereographic,
		WPolyconic,
		WRobinson,
		WSinusoidal,
		WSpace_Oblique_Mercator_Landsat,
		WStereographic,
		WTransverse_Mercator,
		Wvan_der_Grinten,
		WMap_Projection_Parameters,
		Wnull}},
	{WAlbers_Conical_Equal_Area,{
		WStandard_Parallel,
		WLongitude_of_Central_Meridian,
		WLatitude_of_Projection_Origin,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WAzimuthal_Equidistant,{
		WLongitude_of_Central_Meridian,
		WLatitude_of_Projection_Origin,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WEquidistant_Conic,{
		WStandard_Parallel,
		WLongitude_of_Central_Meridian,
		WLatitude_of_Projection_Origin,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WEquirectangular,{
		WStandard_Parallel,
		WLongitude_of_Central_Meridian,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WGeneral_Vertical_Near_sided_Perspective,{
		WHeight_of_Perspective_Point_Above_Surface,
		WLongitude_of_Projection_Center,
		WLatitude_of_Projection_Center,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WGnomonic,{
		WLongitude_of_Projection_Center,
		WLatitude_of_Projection_Center,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WLambert_Azimuthal_Equal_Area,{
		WLongitude_of_Projection_Center,
		WLatitude_of_Projection_Center,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WLambert_Conformal_Conic,{
		WStandard_Parallel,
		WLongitude_of_Central_Meridian,
		WLatitude_of_Projection_Origin,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WMercator,{
		WStandard_Parallel,
		WScale_Factor_at_Equator,
		WLongitude_of_Central_Meridian,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WModified_Stereographic_for_Alaska,{
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WMiller_Cylindrical,{
		WLongitude_of_Central_Meridian,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WOblique_Mercator,{
		WScale_Factor_at_Center_Line,
		WOblique_Line_Azimuth,
		WOblique_Line_Point,
		WLatitude_of_Projection_Origin,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WOblique_Line_Azimuth,{
		WAzimuthal_Angle,
		WAzimuth_Measure_Point_Longitude,
		Wnull}},
	{WOblique_Line_Point,{
		WOblique_Line_Latitude,
		WOblique_Line_Longitude,
		Wnull}},
	{WOrthographic,{
		WLongitude_of_Projection_Center,
		WLatitude_of_Projection_Center,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WPolar_Stereographic,{
		WStraight_Vertical_Longitude_from_Pole,
		WStandard_Parallel,
		WScale_Factor_at_Projection_Origin,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WPolyconic,{
		WLongitude_of_Central_Meridian,
		WLatitude_of_Projection_Origin,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WRobinson,{
		WLongitude_of_Projection_Center,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WSinusoidal,{
		WLongitude_of_Central_Meridian,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WSpace_Oblique_Mercator_Landsat,{
		WLandsat_Number,
		WPath_Number,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WStereographic,{
		WLongitude_of_Projection_Center,
		WLatitude_of_Projection_Center,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WTransverse_Mercator,{
		WScale_Factor_at_Central_Meridian,
		WLongitude_of_Central_Meridian,
		WLatitude_of_Projection_Origin,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{Wvan_der_Grinten,{
		WLongitude_of_Central_Meridian,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WMap_Projection_Parameters,{
		WAzimuth_Measure_Point_Longitude,
		WAzimuthal_Angle,
		WFalse_Easting,
		WFalse_Northing,
		WHeight_of_Perspective_Point_Above_Surface,
		WLandsat_Number,
		WLatitude_of_Projection_Center,
		WLatitude_of_Projection_Origin,
		WLongitude_of_Central_Meridian,
		WLongitude_of_Projection_Center,
		WOblique_Line_Latitude,
		WOblique_Line_Longitude,
		WPath_Number,
		WScale_Factor_at_Center_Line,
		WScale_Factor_at_Central_Meridian,
		WScale_Factor_at_Equator,
		WScale_Factor_at_Projection_Origin,
		WStandard_Parallel,
		WStraight_Vertical_Longitude_from_Pole,
		WOther_Projections_Definition,
		Wnull}},
	{WGrid_Coordinate_System,{
		WGrid_Coordinate_System_Name,
		WUniversal_Transverse_Mercator,
		WUniversal_Polar_Stereographic,
		WState_Plane_Coordinate_System,
		WARC_Coordinate_System,
		WOther_Grid_Systems_Definition,
		Wnull}},
	{WUniversal_Transverse_Mercator,{
		WUTM_Zone_Number,
		WTransverse_Mercator,
		Wnull}},
	{WUniversal_Polar_Stereographic,{
		WUPS_Zone_Identifier,
		WPolar_Stereographic,
		Wnull}},
	{WState_Plane_Coordinate_System,{
		WSPCS_Zone_Identifier,
		WLambert_Conformal_Conic,
		WTransverse_Mercator,
		WOblique_Mercator,
		WPolyconic,
		Wnull}},
	{WARC_Coordinate_System,{
		WARC_System_Zone_Identifier,
		WEquirectangular,
		WAzimuthal_Equidistant,
		Wnull}},
	{WLocal_Planar,{
		WLocal_Planar_Description,
		WLocal_Planar_Georeference_Information,
		Wnull}},
	{WPlanar_Coordinate_Information,{
		WPlanar_Coordinate_Encoding_Method,
		WCoordinate_Representation,
		WDistance_and_Bearing_Representation,
		WPlanar_Distance_Units,
		Wnull}},
	{WCoordinate_Representation,{
		WAbscissa_Resolution,
		WOrdinate_Resolution,
		Wnull}},
	{WDistance_and_Bearing_Representation,{
		WDistance_Resolution,
		WBearing_Resolution,
		WBearing_Units,
		WBearing_Reference_Direction,
		WBearing_Reference_Meridian,
		Wnull}},
	{WLocal,{
		WLocal_Description,
		WLocal_Georeference_Information,
		Wnull}},
	{WGeodetic_Model,{
		WHorizontal_Datum_Name,
		WEllipsoid_Name,
		WSemi_major_Axis,
		WDenominator_of_Flattening_Ratio,
		Wnull}},
	{WVertical_Coordinate_System_Definition,{
		WAltitude_System_Definition,
		WDepth_System_Definition,
		Wnull}},
	{WAltitude_System_Definition,{
		WAltitude_Datum_Name,
		WAltitude_Resolution,
		WAltitude_Distance_Units,
		WAltitude_Encoding_Method,
		Wnull}},
	{WDepth_System_Definition,{
		WDepth_Datum_Name,
		WDepth_Resolution,
		WDepth_Distance_Units,
		WDepth_Encoding_Method,
		Wnull}},
	{WEntity_and_Attribute_Information,{
		WDetailed_Description,
		WOverview_Description,
		Wnull}},
	{WDetailed_Description,{
		WEntity_Type,
		WAttribute,
		Wnull}},
	{WEntity_Type,{
		WEntity_Type_Label,
		WEntity_Type_Definition,
		WEntity_Type_Definition_Source,
		Wnull}},
	{WAttribute,{
		WAttribute_Label,
		WAttribute_Definition,
		WAttribute_Definition_Source,
		WAttribute_Domain_Values,
		WBeginning_Date_of_Attribute_Values,
		WEnding_Date_of_Attribute_Values,
		WAttribute_Value_Accuracy_Information,
		WAttribute_Measurement_Frequency,
		Wnull}},
	{WAttribute_Domain_Values,{
		WEnumerated_Domain,
		WRange_Domain,
		WCodeset_Domain,
		WUnrepresentable_Domain,
		Wnull}},
	{WEnumerated_Domain,{
		WEnumerated_Domain_Value,
		WEnumerated_Domain_Value_Definition,
		WEnumerated_Domain_Value_Definition_Source,
		WAttribute,
		Wnull}},
	{WRange_Domain,{
		WRange_Domain_Minimum,
		WRange_Domain_Maximum,
		WAttribute_Units_of_Measure,
		WAttribute_Measurement_Resolution,
		WAttribute,
		Wnull}},
	{WCodeset_Domain,{
		WCodeset_Name,
		WCodeset_Source,
		Wnull}},
	{WAttribute_Value_Accuracy_Information,{
		WAttribute_Value_Accuracy,
		WAttribute_Value_Accuracy_Explanation,
		Wnull}},
	{WOverview_Description,{
		WEntity_and_Attribute_Overview,
		WEntity_and_Attribute_Detail_Citation,
		Wnull}},
	{WDistribution_Information,{
		WDistributor,
		WResource_Description,
		WDistribution_Liability,
		WStandard_Order_Process,
		WCustom_Order_Process,
		WTechnical_Prerequisites,
		WAvailable_Time_Period,
		Wnull}},
	{WDistributor,{
		WContact_Information,
		Wnull}},
	{WStandard_Order_Process,{
		WNon_digital_Form,
		WDigital_Form,
		WFees,
		WOrdering_Instructions,
		WTurnaround,
		Wnull}},
	{WDigital_Form,{
		WDigital_Transfer_Information,
		WDigital_Transfer_Option,
		Wnull}},
	{WDigital_Transfer_Information,{
		WFormat_Name,
		WFormat_Version_Number,
		WFormat_Version_Date,
		WFormat_Specification ,
		WFormat_Information_Content,
		WFile_Decompression_Technique,
		WTransfer_Size,
		Wnull}},
	{WDigital_Transfer_Option,{
		WOnline_Option,
		WOffline_Option,
		Wnull}},
	{WOnline_Option,{
		WComputer_Contact_Information,
		WAccess_Instructions,
		WOnline_Computer_and_Operating_System,
		Wnull}},
	{WComputer_Contact_Information,{
		WNetwork_Address,
		WDialup_Instructions,
		Wnull}},
	{WNetwork_Address,{
		WNetwork_Resource_Name,
		Wnull}},
	{WDialup_Instructions,{
		WLowest_BPS,
		WHighest_BPS,
		WNumber_DataBits,
		WNumber_StopBits,
		WParity,
		WCompression_Support,
		WDialup_Telephone,
		WDialup_File_Name,
		Wnull}},
	{WOffline_Option,{
		WOffline_Media,
		WRecording_Capacity,
		WRecording_Format,
		WCompatibility_Information,
		Wnull}},
	{WRecording_Capacity,{
		WRecording_Density,
		WRecording_Density_Units,
		Wnull}},
	{WAvailable_Time_Period,{
		WTime_Period_Information,
		Wnull}},
	{WMetadata_Reference_Information,{
		WMetadata_Date,
		WMetadata_Review_Date,
		WMetadata_Future_Review_Date,
		WMetadata_Contact,
		WMetadata_Standard_Name,
		WMetadata_Standard_Version,
		WMetadata_Time_Convention,
		WMetadata_Access_Constraints,
		WMetadata_Use_Constraints,
		WMetadata_Security_Information,
		WMetadata_Extensions,
		WMetadata_Language,
		Wnull}},
	{WMetadata_Contact,{
		WContact_Information,
		Wnull}},
	{WMetadata_Security_Information,{
		WMetadata_Security_Classification_System,
		WMetadata_Security_Classification,
		WMetadata_Security_Handling_Description,
		Wnull}},
	{WMetadata_Extensions,{
		WOnline_Linkage,
		WProfile_Name,
		Wnull}},
	{WCitation_Information,{
		WOriginator,
		WPublication_Date,
		WPublication_Time,
		WTitle,
		WEdition,
		WGeospatial_Data_Presentation_Form,
		WSeries_Information,
		WPublication_Information,
		WOther_Citation_Details,
		WOnline_Linkage,
		WLarger_Work_Citation,
		Wnull}},
	{WSeries_Information,{
		WSeries_Name,
		WIssue_Identification,
		Wnull}},
	{WPublication_Information,{
		WPublication_Place,
		WPublisher,
		Wnull}},
	{WLarger_Work_Citation,{
		WCitation_Information,
		Wnull}},
	{WTime_Period_Information,{
		WSingle_Date_Time,
		WMultiple_Dates_Times,
		WRange_of_Dates_Times,
		Wnull}},
	{WSingle_Date_Time,{
		WCalendar_Date,
		WTime_of_Day,
		Wnull}},
	{WMultiple_Dates_Times,{
		WSingle_Date_Time,
		Wnull}},
	{WRange_of_Dates_Times,{
		WBeginning_Date,
		WBeginning_Time,
		WEnding_Date,
		WEnding_Time,
		Wnull}},
	{WContact_Information,{
		WContact_Person_Primary,
		WContact_Organization_Primary,
		WContact_Position,
		WContact_Address,
		WContact_Voice_Telephone,
		WContact_TDD_TTY_Telephone,
		WContact_Facsimile_Telephone,
		WContact_Electronic_Mail_Address,
		WHours_of_Service,
		WContact_Instructions,
		Wnull}},
	{WContact_Person_Primary,{
		WContact_Person,
		WContact_Organization,
		Wnull}},
	{WContact_Organization_Primary,{
		WContact_Organization,
		WContact_Person,
		Wnull}},
	{WContact_Address,{
		WAddress_Type,
		WAddress,
		WCity,
		WState_or_Province,
		WPostal_Code,
		WCountry,
		Wnull}},
	{Wnull,
		{Wnull}
		}
	};

/* Remote-sensing profile */

static struct compound rsprofile_element[1 + MAX_COMPOUND_COUNT] = {

	/* Standard elements not redefined in the profile */

	{WCitation,{
		WCitation_Information,
		Wnull}},
	{WTime_Period_of_Content,{
		WTime_Period_Information,
		WCurrentness_Reference,
		Wnull}},
	{WStatus,{
		WProgress,
		WMaintenance_and_Update_Frequency,
		Wnull}},
	{WBounding_Coordinates,{
		WWest_Bounding_Coordinate,
		WEast_Bounding_Coordinate,
		WNorth_Bounding_Coordinate,
		WSouth_Bounding_Coordinate,
		Wnull}},
	{WData_Set_G_Polygon,{
		WData_Set_G_Polygon_Outer_G_Ring,
		WData_Set_G_Polygon_Exclusion_G_Ring,
		Wnull}},
	{WData_Set_G_Polygon_Outer_G_Ring,{
		WG_Ring_Point,
		WG_Ring,
		Wnull}},
	{WData_Set_G_Polygon_Exclusion_G_Ring,{
		WG_Ring_Point,
		WG_Ring,
		Wnull}},
	{WG_Ring_Point,{
		WG_Ring_Latitude,
		WG_Ring_Longitude,
		Wnull}},
	{WKeywords,{
		WTheme,
		WPlace,
		WStratum,
		WTemporal,
		Wnull}},
	{WTheme,{
		WTheme_Keyword_Thesaurus,
		WTheme_Keyword,
		Wnull}},
	{WPlace,{
		WPlace_Keyword_Thesaurus,
		WPlace_Keyword,
		Wnull}},
	{WStratum,{
		WStratum_Keyword_Thesaurus,
		WStratum_Keyword,
		Wnull}},
	{WTemporal,{
		WTemporal_Keyword_Thesaurus,
		WTemporal_Keyword,
		Wnull}},
	{WPoint_of_Contact,{
		WContact_Information,
		Wnull}},
	{WBrowse_Graphic,{
		WBrowse_Graphic_File_Name,
		WBrowse_Graphic_File_Description,
		WBrowse_Graphic_File_Type,
		Wnull}},
	{WSecurity_Information,{
		WSecurity_Classification_System,
		WSecurity_Classification,
		WSecurity_Handling_Description,
		Wnull}},
	{WCross_Reference,{
		WCitation_Information,
		Wnull}},
	{WAttribute_Accuracy,{
		WAttribute_Accuracy_Report,
		WQuantitative_Attribute_Accuracy_Assessment,
		Wnull}},
	{WQuantitative_Attribute_Accuracy_Assessment,{
		WAttribute_Accuracy_Value,
		WAttribute_Accuracy_Explanation,
		Wnull}},
	{WPositional_Accuracy,{
		WHorizontal_Positional_Accuracy,
		WVertical_Positional_Accuracy,
		Wnull}},
	{WHorizontal_Positional_Accuracy,{
		WHorizontal_Positional_Accuracy_Report,
		WQuantitative_Horizontal_Positional_Accuracy_Assessment,
		Wnull}},
	{WQuantitative_Horizontal_Positional_Accuracy_Assessment,{
		WHorizontal_Positional_Accuracy_Value,
		WHorizontal_Positional_Accuracy_Explanation,
		Wnull}},
	{WVertical_Positional_Accuracy,{
		WVertical_Positional_Accuracy_Report,
		WQuantitative_Vertical_Positional_Accuracy_Assessment,
		Wnull}},
	{WQuantitative_Vertical_Positional_Accuracy_Assessment,{
		WVertical_Positional_Accuracy_Value,
		WVertical_Positional_Accuracy_Explanation,
		Wnull}},
	{WSource_Information,{
		WSource_Citation,
		WSource_Scale_Denominator,
		WType_of_Source_Media,
		WSource_Time_Period_of_Content,
		WSource_Citation_Abbreviation,
		WSource_Contribution,
		Wnull}},
	{WSource_Citation,{
		WCitation_Information,
		Wnull}},
	{WSource_Time_Period_of_Content,{
		WTime_Period_Information,
		WSource_Currentness_Reference,
		Wnull}},
	{WProcess_Contact,{
		WContact_Information,
		Wnull}},
	{WPoint_and_Vector_Object_Information,{
		WSDTS_Terms_Description,
		WVPF_Terms_Description,
		Wnull}},
	{WSDTS_Terms_Description,{
		WSDTS_Point_and_Vector_Object_Type,
		WPoint_and_Vector_Object_Count,
		Wnull}},
	{WVPF_Terms_Description,{
		WVPF_Topology_Level,
		WVPF_Point_and_Vector_Object_Information,
		Wnull}},
	{WVPF_Point_and_Vector_Object_Information,{
		WVPF_Point_and_Vector_Object_Type,
		WPoint_and_Vector_Object_Count,
		Wnull}},
	{WHorizontal_Coordinate_System_Definition,{
		WGeographic,
		WPlanar,
		WLocal,
		WGeodetic_Model,
		Wnull}},
	{WGeographic,{
		WLatitude_Resolution,
		WLongitude_Resolution,
		WGeographic_Coordinate_Units,
		Wnull}},
	{WPlanar,{
		WMap_Projection,
		WGrid_Coordinate_System,
		WLocal_Planar,
		WPlanar_Coordinate_Information,
		Wnull}},
	{WMap_Projection,{
		WMap_Projection_Name,
		WAlbers_Conical_Equal_Area,
		WAzimuthal_Equidistant,
		WEquidistant_Conic,
		WEquirectangular,
		WGeneral_Vertical_Near_sided_Perspective,
		WGnomonic,
		WLambert_Azimuthal_Equal_Area,
		WLambert_Conformal_Conic,
		WMercator,
		WModified_Stereographic_for_Alaska,
		WMiller_Cylindrical,
		WOblique_Mercator,
		WOrthographic,
		WPolar_Stereographic,
		WPolyconic,
		WRobinson,
		WSinusoidal,
		WSpace_Oblique_Mercator_Landsat,
		WStereographic,
		WTransverse_Mercator,
		Wvan_der_Grinten,
		WMap_Projection_Parameters,
		Wnull}},
	{WAlbers_Conical_Equal_Area,{
		WStandard_Parallel,
		WLongitude_of_Central_Meridian,
		WLatitude_of_Projection_Origin,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WAzimuthal_Equidistant,{
		WLongitude_of_Central_Meridian,
		WLatitude_of_Projection_Origin,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WEquidistant_Conic,{
		WStandard_Parallel,
		WLongitude_of_Central_Meridian,
		WLatitude_of_Projection_Origin,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WEquirectangular,{
		WStandard_Parallel,
		WLongitude_of_Central_Meridian,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WGeneral_Vertical_Near_sided_Perspective,{
		WHeight_of_Perspective_Point_Above_Surface,
		WLongitude_of_Projection_Center,
		WLatitude_of_Projection_Center,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WGnomonic,{
		WLongitude_of_Projection_Center,
		WLatitude_of_Projection_Center,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WLambert_Azimuthal_Equal_Area,{
		WLongitude_of_Projection_Center,
		WLatitude_of_Projection_Center,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WLambert_Conformal_Conic,{
		WStandard_Parallel,
		WLongitude_of_Central_Meridian,
		WLatitude_of_Projection_Origin,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WMercator,{
		WStandard_Parallel,
		WScale_Factor_at_Equator,
		WLongitude_of_Central_Meridian,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WModified_Stereographic_for_Alaska,{
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WMiller_Cylindrical,{
		WLongitude_of_Central_Meridian,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WOblique_Mercator,{
		WScale_Factor_at_Center_Line,
		WOblique_Line_Azimuth,
		WOblique_Line_Point,
		WLatitude_of_Projection_Origin,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WOblique_Line_Azimuth,{
		WAzimuthal_Angle,
		WAzimuth_Measure_Point_Longitude,
		Wnull}},
	{WOblique_Line_Point,{
		WOblique_Line_Latitude,
		WOblique_Line_Longitude,
		Wnull}},
	{WOrthographic,{
		WLongitude_of_Projection_Center,
		WLatitude_of_Projection_Center,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WPolar_Stereographic,{
		WStraight_Vertical_Longitude_from_Pole,
		WStandard_Parallel,
		WScale_Factor_at_Projection_Origin,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WPolyconic,{
		WLongitude_of_Central_Meridian,
		WLatitude_of_Projection_Origin,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WRobinson,{
		WLongitude_of_Projection_Center,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WSinusoidal,{
		WLongitude_of_Central_Meridian,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WSpace_Oblique_Mercator_Landsat,{
		WLandsat_Number,
		WPath_Number,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WStereographic,{
		WLongitude_of_Projection_Center,
		WLatitude_of_Projection_Center,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WTransverse_Mercator,{
		WScale_Factor_at_Central_Meridian,
		WLongitude_of_Central_Meridian,
		WLatitude_of_Projection_Origin,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{Wvan_der_Grinten,{
		WLongitude_of_Central_Meridian,
		WFalse_Easting,
		WFalse_Northing,
		Wnull}},
	{WMap_Projection_Parameters,{
		WAzimuth_Measure_Point_Longitude,
		WAzimuthal_Angle,
		WFalse_Easting,
		WFalse_Northing,
		WHeight_of_Perspective_Point_Above_Surface,
		WLandsat_Number,
		WLatitude_of_Projection_Center,
		WLatitude_of_Projection_Origin,
		WLongitude_of_Central_Meridian,
		WLongitude_of_Projection_Center,
		WOblique_Line_Latitude,
		WOblique_Line_Longitude,
		WPath_Number,
		WScale_Factor_at_Center_Line,
		WScale_Factor_at_Central_Meridian,
		WScale_Factor_at_Equator,
		WScale_Factor_at_Projection_Origin,
		WStandard_Parallel,
		WStraight_Vertical_Longitude_from_Pole,
		WOther_Projections_Definition,
		Wnull}},
	{WGrid_Coordinate_System,{
		WGrid_Coordinate_System_Name,
		WUniversal_Transverse_Mercator,
		WUniversal_Polar_Stereographic,
		WState_Plane_Coordinate_System,
		WARC_Coordinate_System,
		WOther_Grid_Systems_Definition,
		Wnull}},
	{WUniversal_Transverse_Mercator,{
		WUTM_Zone_Number,
		WTransverse_Mercator,
		Wnull}},
	{WUniversal_Polar_Stereographic,{
		WUPS_Zone_Identifier,
		WPolar_Stereographic,
		Wnull}},
	{WState_Plane_Coordinate_System,{
		WSPCS_Zone_Identifier,
		WLambert_Conformal_Conic,
		WTransverse_Mercator,
		WOblique_Mercator,
		WPolyconic,
		Wnull}},
	{WARC_Coordinate_System,{
		WARC_System_Zone_Identifier,
		WEquirectangular,
		WAzimuthal_Equidistant,
		Wnull}},
	{WLocal_Planar,{
		WLocal_Planar_Description,
		WLocal_Planar_Georeference_Information,
		Wnull}},
	{WPlanar_Coordinate_Information,{
		WPlanar_Coordinate_Encoding_Method,
		WCoordinate_Representation,
		WDistance_and_Bearing_Representation,
		WPlanar_Distance_Units,
		Wnull}},
	{WCoordinate_Representation,{
		WAbscissa_Resolution,
		WOrdinate_Resolution,
		Wnull}},
	{WDistance_and_Bearing_Representation,{
		WDistance_Resolution,
		WBearing_Resolution,
		WBearing_Units,
		WBearing_Reference_Direction,
		WBearing_Reference_Meridian,
		Wnull}},
	{WLocal,{
		WLocal_Description,
		WLocal_Georeference_Information,
		Wnull}},
	{WGeodetic_Model,{
		WHorizontal_Datum_Name,
		WEllipsoid_Name,
		WSemi_major_Axis,
		WDenominator_of_Flattening_Ratio,
		Wnull}},
	{WDepth_System_Definition,{
		WDepth_Datum_Name,
		WDepth_Resolution,
		WDepth_Distance_Units,
		WDepth_Encoding_Method,
		Wnull}},
	{WEntity_Type,{
		WEntity_Type_Label,
		WEntity_Type_Definition,
		WEntity_Type_Definition_Source,
		Wnull}},
	{WEnumerated_Domain,{
		WEnumerated_Domain_Value,
		WEnumerated_Domain_Value_Definition,
		WEnumerated_Domain_Value_Definition_Source,
		WAttribute,
		Wnull}},
	{WRange_Domain,{
		WRange_Domain_Minimum,
		WRange_Domain_Maximum,
		WAttribute_Units_of_Measure,
		WAttribute_Measurement_Resolution,
		WAttribute,
		Wnull}},
	{WCodeset_Domain,{
		WCodeset_Name,
		WCodeset_Source,
		Wnull}},
	{WAttribute_Value_Accuracy_Information,{
		WAttribute_Value_Accuracy,
		WAttribute_Value_Accuracy_Explanation,
		Wnull}},
	{WOverview_Description,{
		WEntity_and_Attribute_Overview,
		WEntity_and_Attribute_Detail_Citation,
		Wnull}},
	{WDistribution_Information,{
		WDistributor,
		WResource_Description,
		WDistribution_Liability,
		WStandard_Order_Process,
		WCustom_Order_Process,
		WTechnical_Prerequisites,
		WAvailable_Time_Period,
		Wnull}},
	{WDistributor,{
		WContact_Information,
		Wnull}},
	{WStandard_Order_Process,{
		WNon_digital_Form,
		WDigital_Form,
		WFees,
		WOrdering_Instructions,
		WTurnaround,
		Wnull}},
	{WDigital_Form,{
		WDigital_Transfer_Information,
		WDigital_Transfer_Option,
		Wnull}},
	{WDigital_Transfer_Information,{
		WFormat_Name,
		WFormat_Version_Number,
		WFormat_Version_Date,
		WFormat_Specification ,
		WFormat_Information_Content,
		WFile_Decompression_Technique,
		WTransfer_Size,
		Wnull}},
	{WDigital_Transfer_Option,{
		WOnline_Option,
		WOffline_Option,
		Wnull}},
	{WOnline_Option,{
		WComputer_Contact_Information,
		WAccess_Instructions,
		WOnline_Computer_and_Operating_System,
		Wnull}},
	{WComputer_Contact_Information,{
		WNetwork_Address,
		WDialup_Instructions,
		Wnull}},
	{WNetwork_Address,{
		WNetwork_Resource_Name,
		Wnull}},
	{WDialup_Instructions,{
		WLowest_BPS,
		WHighest_BPS,
		WNumber_DataBits,
		WNumber_StopBits,
		WParity,
		WCompression_Support,
		WDialup_Telephone,
		WDialup_File_Name,
		Wnull}},
	{WOffline_Option,{
		WOffline_Media,
		WRecording_Capacity,
		WRecording_Format,
		WCompatibility_Information,
		Wnull}},
	{WRecording_Capacity,{
		WRecording_Density,
		WRecording_Density_Units,
		Wnull}},
	{WAvailable_Time_Period,{
		WTime_Period_Information,
		Wnull}},
	{WMetadata_Reference_Information,{
		WMetadata_Date,
		WMetadata_Review_Date,
		WMetadata_Future_Review_Date,
		WMetadata_Contact,
		WMetadata_Standard_Name,
		WMetadata_Standard_Version,
		WMetadata_Time_Convention,
		WMetadata_Access_Constraints,
		WMetadata_Use_Constraints,
		WMetadata_Security_Information,
		WMetadata_Extensions,
		WMetadata_Language,
		Wnull}},
	{WMetadata_Contact,{
		WContact_Information,
		Wnull}},
	{WMetadata_Security_Information,{
		WMetadata_Security_Classification_System,
		WMetadata_Security_Classification,
		WMetadata_Security_Handling_Description,
		Wnull}},
	{WMetadata_Extensions,{
		WOnline_Linkage,
		WProfile_Name,
		Wnull}},
	{WCitation_Information,{
		WOriginator,
		WPublication_Date,
		WPublication_Time,
		WTitle,
		WEdition,
		WGeospatial_Data_Presentation_Form,
		WSeries_Information,
		WPublication_Information,
		WOther_Citation_Details,
		WOnline_Linkage,
		WLarger_Work_Citation,
		Wnull}},
	{WSeries_Information,{
		WSeries_Name,
		WIssue_Identification,
		Wnull}},
	{WPublication_Information,{
		WPublication_Place,
		WPublisher,
		Wnull}},
	{WLarger_Work_Citation,{
		WCitation_Information,
		Wnull}},
	{WTime_Period_Information,{
		WSingle_Date_Time,
		WMultiple_Dates_Times,
		WRange_of_Dates_Times,
		Wnull}},
	{WSingle_Date_Time,{
		WCalendar_Date,
		WTime_of_Day,
		Wnull}},
	{WMultiple_Dates_Times,{
		WSingle_Date_Time,
		Wnull}},
	{WRange_of_Dates_Times,{
		WBeginning_Date,
		WBeginning_Time,
		WEnding_Date,
		WEnding_Time,
		Wnull}},
	{WContact_Information,{
		WContact_Person_Primary,
		WContact_Organization_Primary,
		WContact_Position,
		WContact_Address,
		WContact_Voice_Telephone,
		WContact_TDD_TTY_Telephone,
		WContact_Facsimile_Telephone,
		WContact_Electronic_Mail_Address,
		WHours_of_Service,
		WContact_Instructions,
		Wnull}},
	{WContact_Person_Primary,{
		WContact_Person,
		WContact_Organization,
		Wnull}},
	{WContact_Organization_Primary,{
		WContact_Organization,
		WContact_Person,
		Wnull}},
	{WContact_Address,{
		WAddress_Type,
		WAddress,
		WCity,
		WState_or_Province,
		WPostal_Code,
		WCountry,
		Wnull}},

	/* Elements defined specifically for the profile */

	{WMetadata,{
		WIdentification_Information,
		WData_Quality_Information,
		WSpatial_Data_Organization_Information,
		WSpatial_Reference_Information,
		WEntity_and_Attribute_Information,
		WDistribution_Information,
		WMetadata_Reference_Information,
		WPlatform_and_Mission_Information,
		WInstrument_Information,
		Wnull}},

	{WIdentification_Information,{
		WDataset_Identifier,
		WCitation,
		WDescription,
		WTime_Period_of_Content,
		WStatus,
		WSpatial_Domain,
		WProcessing_Level,
		WKeywords,
		WPlatform_and_Instrument_Identification,
		WBand_Identification,
		WThematic_Layer_Identification,
		WAccess_Constraints,
		WUse_Constraints,
		WPoint_of_Contact,
		WBrowse_Graphic,
		WData_Set_Credit,
		WSecurity_Information,
		WNative_Data_Set_Environment,
		WCross_Reference,
		WAggregation_Information,
		Wnull}},

	{WDescription,{
		WAbstract,
		WPurpose,
		WDocumentation,
		WSupplemental_Information,
		Wnull}},

	{WDocumentation,{
		WData_Dictionary_Reference,
		WUsers_Guide,
		WScience_Paper,
		Wnull}},

	{WData_Dictionary_Reference,{
		WCitation_Information,
		Wnull}},

	{WUsers_Guide,{
		WCitation_Information,
		Wnull}},

	{WScience_Paper,{
		WCitation_Information,
		Wnull}},

	{WSpatial_Domain,{
		WBounding_Coordinates,
		WData_Set_G_Polygon,
		WFrame_Area,
		WMultiple_Image_Alignment,
		WWorldwide_Reference_System,
		Wnull}},

	{WFrame_Area,{
		WFrame_Area_Value,
		WFrame_Area_Units,
		Wnull}},

	{WMultiple_Image_Alignment,{
		WID_Overlapping_Dataset,
		WType_of_Overlap,
		WPercentage_of_Overlap,
		Wnull}},

	{WID_Overlapping_Dataset,{
		WDataset_Identifier,
		Wnull}},

	{WWorldwide_Reference_System,{
		WPath,
		WRow,
		Wnull}},

	{WProcessing_Level,{
		WProcessing_Level_Identifier,
		WProcessing_Level_Authority,
		Wnull}},

	{WProcessing_Level_Authority,{
		WCitation_Information,
		Wnull}},

	{WPlatform_and_Instrument_Identification,{
		WMission_Name,
		WPlatform_Full_Name,
		WPlatform_Short_Name,
		WPlatform_Serial_Identifier,
		WInstrument_Full_Name,
		WInstrument_Short_Name,
		Wnull}},

	{WBand_Identification,{
		WNumber_of_Bands,
		WIndividual_Band_Identification,
		Wnull}},

	{WIndividual_Band_Identification,{
		WBand_ID,
		WBand_Measurement_Mode_ID,
		Wnull}},

	{WThematic_Layer_Identification,{
		WNumber_of_Thematic_Layers,
		WLayer_Name,
		Wnull}},

	{WLayer_Name,{
		WTheme,
		Wnull}},

	{WAggregation_Information,{
		WContainer_Packet_ID,
		WComponent_Information,
		Wnull}},

	{WContainer_Packet_ID,{
		WDataset_Identifier,
		Wnull}},

	{WComponent_Information,{
		WAggregation_Member_ID,
		WAggregation_Criteria,
		Wnull}},

	{WAggregation_Member_ID,{
		WDataset_Identifier,
		Wnull}},

	{WData_Quality_Information,{
		WAttribute_Accuracy,
		WLogical_Consistency_Report,
		WCompleteness_Report,
		WPositional_Accuracy,
		WLineage,
		WImage_Quality,
		WAcquisition_Information,
		WCloud_Cover,
		Wnull}},

	{WLineage,{
		WSource_Information,
		WProcess_Step,
		Wnull}},

	{WProcess_Step,{
		WProcess_Description,
		WSource_Used_Citation_Abbreviation,
		WProcess_Date,
		WProcess_Time,
		WSource_Produced_Citation_Abbreviation,
		WProcess_Contact,
		WAlgorithm_Information,
		WProcessing_Information,
		Wnull}},

	{WAlgorithm_Information,{
		WAlgorithm_Identifiers,
		WAlgorithm_Description,
		WAlgorithm_Change_History,
		WAlgorithm_Peer_Review_Information,
		Wnull}},

	{WAlgorithm_Identifiers,{
		WCitation_Information,
		Wnull}},

	{WAlgorithm_Description,{
		WAlgorithm_Text_Description,
		WAlgorithm_Reference,
		Wnull}},

	{WAlgorithm_Reference,{
		WCitation_Information,
		Wnull}},

	{WAlgorithm_Change_History,{
		WAlgorithm_Change_Description,
		WAlgorithm_Change_Reference,
		Wnull}},

	{WAlgorithm_Change_Description,{
		WProcess_Step,
		Wnull}},

	{WAlgorithm_Change_Reference,{
		WCitation_Information,
		Wnull}},

	{WAlgorithm_Peer_Review_Information,{
		WAlgorithm_Peer_Review_Description,
		WAlgorithm_Peer_Review_Reference,
		Wnull}},

	{WAlgorithm_Peer_Review_Description,{
		WProcess_Step,
		Wnull}},

	{WAlgorithm_Peer_Review_Reference,{
		WCitation_Information,
		Wnull}},

	{WProcessing_Information,{
		WProcessing_Identifiers,
		WProcessing_Input_Dataset,
		WAncillary_Dataset,
		WProcessing_Software,
		WProcessing_Procedure,
		WProcessing_Change_History,
		WProcessing_Documentation,
		Wnull}},

	{WProcessing_Identifiers,{
		WCitation_Information,
		Wnull}},

	{WProcessing_Input_Dataset,{
		WInput_Dataset_Identifier,
		WInput_Description,
		WInput_Reference,
		WInput_Level,
		Wnull}},

	{WInput_Dataset_Identifier,{
		WDataset_Identifier,
		Wnull}},

	{WInput_Reference,{
		WCitation_Information,
		Wnull}},

	{WInput_Level,{
		WProcessing_Level,
		Wnull}},

	{WAncillary_Dataset,{
		WAncillary_Dataset_Identifier,
		WAncillary_Description,
		WAncillary_Reference,
		Wnull}},

	{WAncillary_Dataset_Identifier,{
		WDataset_Identifier,
		Wnull}},

	{WAncillary_Description,{
		WAncillary_Dataset_Description,
		WCommand_Line_Processing_Parameter,
		Wnull}},

	{WAncillary_Reference,{
		WCitation_Information,
		Wnull}},

	{WProcessing_Software,{
		WProcessing_Software_Description,
		WProcessing_Software_Reference,
		Wnull}},

	{WProcessing_Software_Reference,{
		WCitation_Information,
		Wnull}},

	{WProcessing_Procedure,{
		WProcessing_Run_History,
		WProcessing_Environment,
		WProcessing_Procedure_Description,
		Wnull}},

	{WProcessing_Run_History,{
		WProcess_Step,
		Wnull}},

	{WProcessing_Environment,{
		WNative_Data_Set_Environment,
		Wnull}},

	{WProcessing_Change_History,{
		WProcess_Step,
		Wnull}},

	{WProcessing_Documentation,{
		WCitation_Information,
		Wnull}},

	{WAcquisition_Information,{
		WSatellite_Local_Zenith_Angle,
		WSatellite_Local_Azimuth_Angle,
		WSolar_Zenith_Angle,
		WSolar_Azimuth_Angle,
		WRelative_Azimuth,
		WClock_Time_Drift,
		WAscending_Descending_Indicator,
		WNadir,
		Wnull}},

	{WNadir,{
		WNadir_Latitude,
		WNadir_Longitude,
		Wnull}},

	{WSpatial_Data_Organization_Information,{
		WIndirect_Spatial_Reference,
		WDirect_Spatial_Reference_Method,
		WPoint_and_Vector_Object_Information,
		WRaster_Object_Information,
		Wnull}},

	{WRaster_Object_Information,{
		WCell_Value_Type,
		WRaster_Object_Type,
		WRow_Count,
		WColumn_Count,
		WVertical_Count,
		WDimension_Description,
		Wnull}},

	{WDimension_Description,{
		WNumber_of_Data_Dimensions,
		WDimension_Properties,
		Wnull}},

	{WDimension_Properties,{
		WName_of_Dimension,
		WDimension_Count,
		Wnull}},

	{WSpatial_Reference_Information,{
		WHorizontal_Coordinate_System_Definition,
		WVertical_Coordinate_System_Definition,
		WGeoreferencing_Information,
		Wnull}},

	{WVertical_Coordinate_System_Definition,{
		WAltitude_System_Definition,
		WDepth_System_Definition,
		Wnull}},

	{WAltitude_System_Definition,{
		WAltitude_Datum_Name,
		WAltitude_Resolution,
		WAltitude_Distance_Units,
		WAltitude_Distance_Layer,
		WAltitude_Encoding_Method,
		Wnull}},

	{WGeoreferencing_Information,{
		WGeorectified_Raster,
		WGeoreferenceable_Raster,
		Wnull}},

	{WGeorectified_Raster,{
		WPixel_Resolution,
		WGrid_First_Element,
		WGrid_Orientation,
		WPoint_Position_In_Pixel,
		WStorage_Order,
		Wnull}},

	{WPixel_Resolution,{
		WCoordinate_Representation,
		WPlanar_Distance_Units,
		Wnull}},

	{WGrid_First_Element,{
		WGrid_First_Element_Map_X_Coordinate,
		WGrid_First_Element_Map_Y_Coordinate,
		Wnull}},

	{WGrid_Orientation,{
		WRow_Delta_X,
		WRow_Delta_Y,
		WColumn_Delta_X,
		WColumn_Delta_Y,
		Wnull}},

	{WGeoreferenceable_Raster,{
		WGeoreferencing_Description,
		WAerotriangulation_Reference,
		WSwath_Track_Information,
		Wnull}},

	{WGeoreferencing_Description,{
		WGround_Control_Point_Information,
		WInstrument_Specific_Georeferencing,
		WReferencing_Polynomial,
		WOther_Georeferencing_Description,
		Wnull}},

	{WGround_Control_Point_Information,{
		WGround_Control_Point_Organization,
		WGround_Control_Point_Description,
		WGround_Control_Point_Position,
		WGround_Control_Point_Description,
		WGround_Control_Point_Position,
		Wnull}},

	{WGround_Control_Point_Description,{
		WControl_Point_Type,
		WControl_Point_Origin,
		WControl_Point_Use_Flag,
		WControl_Point_Horizontal_X_Accuracy,
		WControl_Point_Horizontal_Y_Accuracy,
		WControl_Point_Vertical_Accuracy,
		Wnull}},

	{WGround_Control_Point_Position,{
		WControl_Point_Raster_Position,
		WControl_Point_Earth_Location,
		WControl_Point_Identification,
		Wnull}},

	{WControl_Point_Raster_Position,{
		WControl_Point_Row,
		WControl_Point_Column,
		Wnull}},

	{WControl_Point_Earth_Location,{
		WControl_Point_x_Value,
		WControl_Point_y_Value,
		WControl_Point_z_Value,
		Wnull}},

	{WControl_Point_Identification,{
		WControl_Point_ID,
		WControl_Point_Authority,
		Wnull}},

	{WControl_Point_Authority,{
		WContact_Information,
		Wnull}},

	{WInstrument_Specific_Georeferencing,{
		WPositional_Information,
		WExterior_Orientation_Accuracy,
		WRotation_Sequence,
		WAxis_Rotation_Convention,
		Wnull}},

	{WPositional_Information,{
		WProjection_Center_X_Position,
		WProjection_Center_Y_Position,
		WProjection_Center_Z_Position,
		WRoll,
		WPitch,
		WYaw,
		WAttitude_Angular_Units,
		Wnull}},

	{WExterior_Orientation_Accuracy,{
		WX_Position_Accuracy,
		WY_Position_Accuracy,
		WZ_Position_Accuracy,
		WRoll_Accuracy,
		WPitch_Accuracy,
		WYaw_Accuracy,
		Wnull}},

	{WReferencing_Polynomial,{
		WPolynomial_Function,
		Wnull}},

	{WAerotriangulation_Reference,{
		WCitation_Information,
		Wnull}},

	{WSwath_Track_Information,{
		WGround_Shape,
		WCross_Track_Motion,
		Wnull}},

	{WEntity_and_Attribute_Information,{
		WDetailed_Description,
		WOverview_Description,
		WDetailed_Description,
		WOverview_Description,
		Wnull}},

	{WDetailed_Description,{
		WEntity_Type,
		WAttribute,
		Wnull}},

	{WAttribute,{
		WAttribute_Label,
		WAttribute_Definition,
		WAttribute_Definition_Source,
		WAttribute_Domain_Values,
		WBeginning_Date_of_Attribute_Values,
		WEnding_Date_of_Attribute_Values,
		WAttribute_Value_Accuracy_Information,
		WAttribute_Measurement_Frequency,
		Wnull}},

	{WAttribute_Domain_Values,{
		WEnumerated_Domain,
		WRange_Domain,
		WCodeset_Domain,
		WUnrepresentable_Domain,
		WData_Scaling_Information,
		Wnull}},

	{WData_Scaling_Information,{
		WPolynomial_Function,
		WNon_Polynomial_Scaling,
		Wnull}},

	{WPolynomial_Function,{
		WPolynomial_Numerator,
		WPolynomial_Denominator,
		Wnull}},

	{WPolynomial_Numerator,{
		WNumber_of_Numerator_Terms,
		WPolynomial_Coefficient,
		Wnull}},

	{WPolynomial_Denominator,{
		WNumber_of_Denominator_Terms,
		WPolynomial_Coefficient,
		Wnull}},

	{WPlatform_and_Mission_Information,{
		WMission_Information,
		WPlatform_Information,
		Wnull}},

	{WMission_Information,{
		WMission_Description,
		WMission_History,
		Wnull}},

	{WMission_History,{
		WMission_Start_Date,
		WMission_Significant_Event,
		WMission_Completion,
		Wnull}},

	{WMission_Start_Date,{
		WSingle_Date_Time,
		Wnull}},

	{WMission_Significant_Event,{
		WProcess_Step,
		Wnull}},

	{WMission_Completion,{
		WSingle_Date_Time,
		Wnull}},

	{WPlatform_Information,{
		WPlatform_Start_Date,
		WPlatform_Sponsor,
		WPlatform_Description,
		WPlatform_Orbit,
		WFlight_Protocol,
		Wnull}},

	{WPlatform_Start_Date,{
		WSingle_Date_Time,
		Wnull}},

	{WPlatform_Orbit,{
		WEphemeris,
		WKeplerian_Orbit,
		WNominal_Geostationary_Position,
		Wnull}},

	{WEphemeris,{
		WSingle_Date_Time,
		Wnull}},

	{WKeplerian_Orbit,{
		WSemimajor_Axis,
		WOrbit_Period,
		WSemimajor_Axis,
		WOrbit_Period,
		WEccentricity,
		WOrbit_Angle_Units,
		WInclination,
		WRight_Ascension_of_Ascending_Node,
		WArgument_of_Perigee,
		WPerigee_Passage_Time,
		Wnull}},

	{WOrbit_Period,{
		WOrbit_Period_Units,
		WOrbit_Period_Value,
		Wnull}},

	{WPerigee_Passage_Time,{
		WSingle_Date_Time,
		Wnull}},

	{WNominal_Geostationary_Position,{
		WPlatform_Nominal_Longitude,
		WPlatform_Nominal_Altitude,
		Wnull}},

	{WPlatform_Nominal_Altitude,{
		WPlatform_Nominal_Altitude_Units,
		WPlatform_Nominal_Altitude_Value,
		Wnull}},

	{WPlatform_Nominal_Altitude_Units,{
		WAltitude_Distance_Units,
		Wnull}},

	{WFlight_Protocol,{
		WFlying_Height,
		WGPS_Information_System_Availability,
		WINS_Reading_Availability,
		Wnull}},

	{WInstrument_Information,{
		WInstrument_Description,
		WInstrument_Reference,
		Wnull}},

	{WInstrument_Description,{
		WInstrument_Type,
		WOperational_Mode,
		WCollection_Type,
		WSensor_Orientation,
		WFrame_Camera,
		WScan,
		WOther_Collector_Description,
		WInstrument_Properties_Description,
		Wnull}},

	{WSensor_Orientation,{
		WAxes,
		WRotation_Description,
		WTranslation_Description,
		Wnull}},

	{WAxes,{
		WX_Axis_Definition,
		WY_Axis_Definition,
		WZ_Axis_Definition,
		Wnull}},

	{WFrame_Camera,{
		WFrame_Hardware,
		WFrame_Optics,
		WFrame_Geometric_Properties,
		WFrame_Operation,
		WFrame_Radiometric_Properties,
		WFrame_Spectral_Properties,
		Wnull}},

	{WFrame_Hardware,{
		WCamera_Type,
		WCamera_Identifier,
		WLens,
		WMagazine_Identifier,
		WFilm_Type,
		WAerial_Film_Speed,
		WEffective_Aerial_Film_Speed,
		WDeveloping_Institution,
		Wnull}},

	{WLens,{
		WLens_Type,
		WLens_Identifier,
		Wnull}},

	{WDeveloping_Institution,{
		WContact_Information,
		Wnull}},

	{WFrame_Optics,{
		WPhotographic_Resolving_Power,
		WRelative_Aperture,
		WExposure_Time,
		WCalibrated_Focal_Length,
		WQuality_of_Focal_Length,
		WLast_Calibration,
		Wnull}},

	{WPhotographic_Resolving_Power,{
		WNumber_of_Resolution_Values,
		WResolution_Value_Set,
		WArea_Weighted_Average_Resolution,
		Wnull}},

	{WResolution_Value_Set,{
		WResolving_Angle,
		WResolving_Value_Radial,
		WResolving_Value_Tangential,
		Wnull}},

	{WLast_Calibration,{
		WDate_of_Last_Calibration,
		WMethod_of_Last_Calibration,
		WInstitution_of_Last_Calibration,
		Wnull}},

	{WFrame_Geometric_Properties,{
		WImage_Size,
		WFiducial,
		WReseau,
		WSensor_System,
		WPrincipal_Point_of_Autocollimation,
		WQuality_of_Principal_Point_of_Autocollimation,
		WPrincipal_Point_of_Symmetry,
		WQuality_of_Principal_Point_of_Symmetry,
		WFiducial_Center,
		WSensor_Element_Location,
		WDistortion,
		Wnull}},

	{WImage_Size,{
		WImage_Size_x_Value,
		WImage_Size_y_Value,
		Wnull}},

	{WFiducial,{
		WLocation_Information,
		Wnull}},

	{WReseau,{
		WLocation_Information,
		Wnull}},

	{WSensor_System,{
		WSensor_Grid,
		WCalibrated_Detector_Positions,
		Wnull}},

	{WSensor_Grid,{
		WRaster_Object_Type,
		Wnull}},

	{WCalibrated_Detector_Positions,{
		WLocation_Information,
		Wnull}},

	{WPrincipal_Point_of_Autocollimation,{
		WLocation_Information,
		Wnull}},

	{WQuality_of_Principal_Point_of_Autocollimation,{
		WQuality_of_Autocollimation_Principal_Point_x_Value,
		WQuality_of_Autocollimation_Principal_Point_y_Value,
		Wnull}},

	{WPrincipal_Point_of_Symmetry,{
		WLocation_Information,
		Wnull}},

	{WQuality_of_Principal_Point_of_Symmetry,{
		WQuality_of_Symmetry_Principal_Point_x_Value,
		WQuality_of_Symmetry_Principal_Point_y_Value,
		Wnull}},

	{WFiducial_Center,{
		WLocation_Information,
		Wnull}},

	{WSensor_Element_Location,{
		WAvailability_of_Element_Locations,
		WSource_of_Element_Locations,
		Wnull}},

	{WSource_of_Element_Locations,{
		WCitation_Information,
		Wnull}},

	{WDistortion,{
		WDistortion_Type_Radial_Symmetrical,
		WDistortion_Type_Radial_Asymmetrical,
		WDistortion_Type_Affine,
		Wnull}},

	{WDistortion_Type_Radial_Symmetrical,{
		WDistance_Dependent_Distortion,
		WAngle_Dependent_Distortion,
		WRadial_Symmetrical_Distortion_Polynomial,
		Wnull}},

	{WDistance_Dependent_Distortion,{
		WRadial_Symmetrical_Distance_Interval,
		WNumber_of_Distance_Distortion_Values,
		WDistance_Distortion_Value,
		Wnull}},

	{WAngle_Dependent_Distortion,{
		WRadial_Symmetrical_Angle_Interval,
		WNumber_of_Angle_Distortion_Values,
		WAngle_Distortion_Value,
		Wnull}},

	{WRadial_Symmetrical_Distortion_Polynomial,{
		WPolynomial_Function,
		Wnull}},

	{WDistortion_Type_Radial_Asymmetrical,{
		WRadial_Asymmetrical_Coefficient_B1,
		WRadial_Asymmetrical_Coefficient_B2,
		Wnull}},

	{WDistortion_Type_Affine,{
		WAffine_Distortion_X_Prime_Coefficient,
		WAffine_Distortion_Y_Prime_Coefficient,
		Wnull}},

	{WFrame_Operation,{
		WStabilized_Mount,
		WForward_Motion_Compensation,
		Wnull}},

	{WFrame_Radiometric_Properties,{
		WFrame_Radiometric_Calibration,
		WLight_Drop,
		Wnull}},

	{WFrame_Radiometric_Calibration,{
		WData_Scaling_Information,
		Wnull}},

	{WFrame_Spectral_Properties,{
		WFrame_Spectral_Information,
		WFilter_on_Camera,
		WSpectral_Limit,
		Wnull}},

	{WFrame_Spectral_Information,{
		WSpectral_Information,
		Wnull}},

	{WFilter_on_Camera,{
		WFilter_on_Camera_Indicator,
		WFilter_Type,
		Wnull}},

	{WScan,{
		WScan_Geometric_Properties,
		WSample_Properties,
		WScan_Radiometric_Properties,
		WScan_Spectral_Properties,
		Wnull}},

	{WScan_Geometric_Properties,{
		WScan_Angle_Units,
		WScan_Time_Units,
		WScan_Distance_Units,
		WScan_Cross_Track_Properties,
		WScan_Elevation_Properties,
		WProfile_Properties,
		WScan_Timing,
		WInstantaneous_Field_of_View,
		Wnull}},

	{WScan_Cross_Track_Properties,{
		WCross_Track_Zero,
		WCross_Track_Sweep,
		WCross_Track_Fixed_Angle,
		WCross_Track_Description,
		Wnull}},

	{WCross_Track_Zero,{
		WCross_Track_Axis,
		WCross_Track_Direction,
		Wnull}},

	{WCross_Track_Sweep,{
		WNumber_of_Cross_Track_Samples,
		WCross_Track_Start_Angle,
		WCross_Track_Extent_Angle,
		WCross_Track_Step_Angle,
		Wnull}},

	{WScan_Elevation_Properties,{
		WElevation_Zero,
		WElevation_Sweep,
		WElevation_Fixed_Angle,
		WElevation_Description,
		Wnull}},

	{WElevation_Zero,{
		WElevation_Axis,
		WElevation_Direction,
		Wnull}},

	{WElevation_Sweep,{
		WNumber_of_Elevation_Samples,
		WElevation_Start_Angle,
		WElevation_Extent_Angle,
		WElevation_Step_Angle,
		Wnull}},

	{WProfile_Properties,{
		WProfile_Sounding,
		WProfile_Fixed,
		WProfile_Description,
		Wnull}},

	{WProfile_Sounding,{
		WNumber_of_Profile_Samples,
		WProfiling_Direction,
		WProfile_Start,
		WProfile_Extent,
		WProfile_Step,
		Wnull}},

	{WScan_Timing,{
		WScan_Start_Time,
		WScan_Duration,
		WScan_Step_Time,
		WScan_Repeat_Time,
		Wnull}},

	{WScan_Start_Time,{
		WSingle_Date_Time,
		Wnull}},

	{WInstantaneous_Field_of_View,{
		WIFOV_Units,
		WIFOV_x_Definition,
		WIFOV_x_Value,
		WIFOV_y_Definition,
		WIFOV_y_Value,
		Wnull}},

	{WSample_Properties,{
		WSample_Description_Units,
		WPixel_Description,
		Wnull}},

	{WSample_Description_Units,{
		WSample_Angle_Units,
		WSample_Profile_Units,
		Wnull}},

	{WSample_Profile_Units,{
		WAltitude_Distance_Units,
		Wnull}},

	{WPixel_Description,{
		WPixel_Cross_Track_Size,
		WPixel_Elevation_Size,
		WPixel_Profile_Size,
		WPixel_Height_Above_Ellipsoid,
		WPixel_Point_Spread_Function,
		Wnull}},

	{WScan_Radiometric_Properties,{
		WData_Scaling_Information,
		Wnull}},

	{WScan_Spectral_Properties,{
		WSpectral_Information,
		Wnull}},

	{WSpectral_Information,{
		WNumber_of_Wavelength_Bands,
		WWavelength_Band_Properties,
		Wnull}},

	{WNumber_of_Wavelength_Bands,{
		WNumber_of_Bands,
		Wnull}},

	{WWavelength_Band_Properties,{
		WWavelength_Units,
		WBand_Boundary_Definition,
		WMinimum_Wavelength,
		WMaximum_Wavelength,
		WPeak_Wavelength,
		WWavelength_Region,
		WNominal_Spatial_Resolution,
		WBand_Quality,
		WPolarization_Characteristics,
		WBand_Description,
		Wnull}},

	{WNominal_Spatial_Resolution,{
		WSpatial_Resolution_Units,
		WSpatial_Resolution_Value,
		Wnull}},

	{WPolarization_Characteristics,{
		WReceiver_Polarization,
		WSender_Polarization,
		Wnull}},

	{WInstrument_Reference,{
		WCitation_Information,
		Wnull}},

	{WLocation_Information,{
		WNumber_of_Points,
		WCoordinate_System,
		WCoordinate_XY_Units,
		WCoordinate_Z_Units,
		WCoordinate_Point,
		Wnull}},

	{WCoordinate_System,{
		WUnreferenced_Coordinate_System,
		WReferenced_Coordinate_System,
		Wnull}},

	{WReferenced_Coordinate_System,{
		WSpatial_Reference_Information,
		Wnull}},

	{WCoordinate_Point,{
		WCoordinate_x_Value,
		WCoordinate_y_Value,
		WCoordinate_z_Value,
		Wnull}},

	{Wnull,
		{Wnull}
		}
	};

static struct compound *compound_element = standard_element;
static int compound_element_length = 0;

void use_profile (char *name) {
	compound_element = standard_element;
	if (name && *name) {
		if (memicmp (name,"bio",3) == 0) compound_element = bioprofile_element;
		if (memicmp (name,"sh", 2) == 0) compound_element = shprofile_element;
		if (memicmp (name,"rs", 2) == 0) compound_element = rsprofile_element;
		if (memicmp (name,"remote", 6) == 0) compound_element = rsprofile_element;
		}
	compound_element_length = 0;
	}

static int compare_compound (const void *e1, const void *e2) {
	const struct compound *s1 = e1;
	const struct compound *s2 = e2;
	return (s1->key - s2->key);
	}

static void sort_compound_element (void) {
	int i;

	for (i=0; compound_element[i].key != Wnull; i++);
	compound_element_length = i;
	qsort (compound_element,compound_element_length,sizeof(struct compound),compare_compound);
	}

/*----------------------------------------------------------------------*\
 | Function to add a component to the standard list for any element		|
\*----------------------------------------------------------------------*/

static int add_child (enum fgdc_keyword parent, enum fgdc_keyword child) {
	struct compound *c;
	int n = 0;

	for (c = compound_element; c->key != Wnull; c++)
		if (c->key == parent) {
			for (n=0; c->component[n] != Wnull; n++)
				if (c->component[n] == child) return (n); /* already in list */
			if (n < MAX_COMPONENT_COUNT) {
				c->component[n++] = child;
				c->component[n] = Wnull;
				}
			}
	return (n);
	}

/*----------------------------------------------------------------------*\
 | Function to add a new compound element to the list.					|
\*----------------------------------------------------------------------*/

static int add_parent (enum fgdc_keyword parent, enum fgdc_keyword *child) {
	struct compound *c;
	int i;

	/* Do nothing if this element is already in the list. */

	for (c = compound_element; c->key != Wnull; c++)
		if (c->key == parent) return (0);

	/* c now points to the last element in the list */

	if (c - compound_element < MAX_COMPOUND_COUNT - 1) {
		compound_element_length = c - compound_element;
		c->key = parent;
		for (i=0; i < MAX_COMPONENT_COUNT; i++) {
			c->component[i] = child[i];
			if (child[i] == Wnull) break;
			}
		c++;
		c->key = Wnull;
		c->component[0] = Wnull;
		return (1);
		}
	return (0);
	}

/*----------------------------------------------------------------------*\
 | This procedure is vulnerable to changes in the values assigned the	|
 | defined constants MAX_COMPOUND_COUNT and MAX_COMPONENT_COUNT in the	|
 | file local.c.  Specifically, the constants defined here should be	|
 | equal to or greater than those defined in local.c.  If not, some		|
 | information about local extensions won't be visible in the editor.	|
\*----------------------------------------------------------------------*/

struct extension_info {
	int key;
	enum fgdc_keyword parent [1 + MAX_COMPOUND_COUNT];
	enum fgdc_keyword child [1 + MAX_COMPONENT_COUNT];
	};

extern int get_extension_count (void);
extern struct extension_info *get_extension_info (int i);

static void add_extensions (void) {
	int i,j;
	struct extension_info *e;
	struct compound *c;

	for (i=0; i < get_extension_count(); i++)
		if (e = get_extension_info (i))
			if (e->key != Wnull) {

				/* Has this extension been added already? */

				for (c = compound_element; c->key != Wnull; c++)
					if (e->key == c->key) break;

				/* If not yet added, add it. */

				if (c->key != e->key) {
					for (j=0; e->parent[j] != Wnull; j++)
						if (e->parent[j] != Wblank)
							add_child (e->parent[j],e->key);
					if (e->child[1] != Wunknown && e->child[1] != Wnull)
						add_parent (e->key,&e->child[1]);
					}
				}
	}

/*----------------------------------------------------------------------*\
 | Update the element lists to reflect extensions added.				|
\*----------------------------------------------------------------------*/

void update_element_lists (void) {
	add_extensions();
	sort_compound_element();
	}

/*----------------------------------------------------------------------*\
 | Retrieve the list of elements making up a compound type.				|
\*----------------------------------------------------------------------*/

enum fgdc_keyword *element_list_of (enum fgdc_keyword key) {
	struct compound target;
	struct compound *match;

	if (compound_element_length == 0) update_element_lists();

	target.key = key;
	match = (struct compound *) bsearch (&target,compound_element,compound_element_length,sizeof(struct compound),compare_compound);
	if (match) return (match->component);
	else return (NULL);
	}

/*----------------------------------------------------------------------*\
\*----------------------------------------------------------------------*/

void show_element_lists (FILE *fp) {
	int j;
	struct compound *c = compound_element;

	for (c = compound_element; c->key != Wnull; c++) {
		fprintf (fp,"%s:\n",text_of(c->key));
		for (j=0; c->component[j] != Wnull; j++) fprintf (fp,"\t%s\n",text_of(c->component[j]));
		}
	}

/*----------------------------------------------------------------------*\
 | Weird convenience function for mq provides, for a given element key	|
 | a list of keys that could be the immediate parent.					|
 | Answers the question "Where might element X go?"						|
 | Response is a pointer to a static buffer whose size grows as needed	|
 | but which will be overwritten on subsequent calls.					|
\*----------------------------------------------------------------------*/

static enum fgdc_keyword *parent_buffer = NULL;
static int parent_buffer_length = 0;

enum fgdc_keyword *parents_of (enum fgdc_keyword key) {
	int j;
	struct compound *c = compound_element;
	enum fgdc_keyword *p = parent_buffer;

	if (!parent_buffer)
		if (parent_buffer = (enum fgdc_keyword *) malloc (1024 * sizeof (enum fgdc_keyword))) {
			parent_buffer_length = 1024;
			p = parent_buffer;
			}
		else return (NULL);

	for (c = compound_element; c->key != Wnull; c++) {
		for (j=0; c->component[j] != Wnull; j++)
			if (c->component[j] == key) {
				if (p - parent_buffer > parent_buffer_length) {
					int k = p - parent_buffer;
					if (parent_buffer = (enum fgdc_keyword *) realloc (parent_buffer,(1024 + parent_buffer_length) * sizeof (enum fgdc_keyword))) {
						parent_buffer_length += 1024;
						p = parent_buffer + k;
						}
					else return (NULL);
					}
				*p++ = c->key;
				break;
				}
		}
	*p = Wnull;
	return (parent_buffer);
	}

/*----------------------------------------------------------------------*\
\*----------------------------------------------------------------------*/
