/*----------------------------------------------------------------------*\
 | Functions to return, for a given FGDC keyword, the ASTM tag text and	|
 | numerical code.														|
\*----------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "keyword.h"
#include "local.h"
#include "item.h"
#include "stricmp.h"

struct astm_tag {
	char *tag;
	enum fgdc_keyword key;
	};

static struct astm_tag astm_tag_table[] = {
	{"METADATA",	WMetadata},
	{"IDENTINFO",	WIdentification_Information},
	{"CITATION",	WCitation},
	{"CITATNINFO",	WCitation_Information},
	{"DESCR",		WDescription},
	{"ABSTRACT",	WAbstract},
	{"PURPOSE",		WPurpose},
	{"SUPPLINFO",	WSupplemental_Information},
	{"TIMEPDCTNT",	WTime_Period_of_Content},
	{"TIMEPDINFO",	WTime_Period_Information},
	{"CURRENTREF",	WCurrentness_Reference},
	{"STATUS",		WStatus},
	{"PROGRESS",	WProgress},
	{"MNTUPDFREQ",	WMaintenance_and_Update_Frequency},
	{"SPTLDOMAIN",	WSpatial_Domain},
	{"BNDGCOORDS",	WBounding_Coordinates},
	{"WBNDGCOORD",	WWest_Bounding_Coordinate},
	{"EBNDGCOORD",	WEast_Bounding_Coordinate},
	{"NBNDGCOORD",	WNorth_Bounding_Coordinate},
	{"SBNDGCOORD",	WSouth_Bounding_Coordinate},
	{"DSGPOLYGON",	WData_Set_G_Polygon},
	{"DSGPOGRING",	WData_Set_G_Polygon_Outer_G_Ring},
	{"GRINGLAT",	WG_Ring_Latitude},
	{"GRINGLONG",	WG_Ring_Longitude},
	{"DSGPEGRING",	WData_Set_G_Polygon_Exclusion_G_Ring},
	{"GRINGLAT",	WG_Ring_Latitude},
	{"GRINGLONG",	WG_Ring_Longitude},
	{"KEYWORDS",	WKeywords},
	{"THEME",		WTheme},
	{"THMKWTHSRS",	WTheme_Keyword_Thesaurus},
	{"THMKEYWORD",	WTheme_Keyword},
	{"GEOGPLACE",	WPlace},
	{"PLCKWTHSRS",	WPlace_Keyword_Thesaurus},
	{"PLCKEYWORD",	WPlace_Keyword},
	{"STRATUM",		WStratum},
	{"STRKWTHSRS",	WStratum_Keyword_Thesaurus},
	{"STRKEYWORD",	WStratum_Keyword},
	{"TEMPORAL",	WTemporal},
	{"TMPKWTHSRS",	WTemporal_Keyword_Thesaurus},
	{"TMPKEYWORD",	WTemporal_Keyword},
	{"ACCESSCONS",	WAccess_Constraints},
	{"USECONSTR",	WUse_Constraints},
	{"PTOFCNTACT",	WPoint_of_Contact},
	{"CNTACTINFO",	WContact_Information},
	{"BRWSEGRPHC",	WBrowse_Graphic},
	{"BRWSEFNAME",	WBrowse_Graphic_File_Name},
	{"BRWSEFNDSC",	WBrowse_Graphic_File_Description},
	{"BRWSEFNFTP",	WBrowse_Graphic_File_Type},
	{"DATACREDIT",	WData_Set_Credit},
	{"SECRTYINFO",	WSecurity_Information},
	{"SECRTYCSYS",	WSecurity_Classification_System},
	{"SECRTYCLAS",	WSecurity_Classification},
	{"SECHNDLDSC",	WSecurity_Handling_Description},
	{"NATIVEDENV",	WNative_Data_Set_Environment},
	{"CROSSREF",	WCross_Reference},
	{"CITATNINFO",	WCitation_Information},
	{"DATAQINFO",	WData_Quality_Information},
	{"ATTRIBACCY",	WAttribute_Accuracy},
	{"ATTRACCYRP",	WAttribute_Accuracy_Report},
	{"QUANATACAS",	WQuantitative_Attribute_Accuracy_Assessment},
	{"ATTRACCVAL",	WAttribute_Accuracy_Value},
	{"ATTRACCEXP",	WAttribute_Accuracy_Explanation},
	{"LOGCONSIST",	WLogical_Consistency_Report},
	{"CMPLETEREP",	WCompleteness_Report},
	{"POSITACCY",	WPositional_Accuracy},
	{"HORPOSACCY",	WHorizontal_Positional_Accuracy},
	{"HORPOSACRP",	WHorizontal_Positional_Accuracy_Report},
	{"QUANTHPAA",	WQuantitative_Horizontal_Positional_Accuracy_Assessment},
	{"HORPOSACCV",	WHorizontal_Positional_Accuracy_Value},
	{"HORPOSACCE",	WHorizontal_Positional_Accuracy_Explanation},
	{"VERTPOSACC",	WVertical_Positional_Accuracy},
	{"VERTPOSARP",	WVertical_Positional_Accuracy_Report},
	{"QUANTVPAA",	WQuantitative_Vertical_Positional_Accuracy_Assessment},
	{"VPOSACCV",	WVertical_Positional_Accuracy_Value},
	{"VPOSACCE",	WVertical_Positional_Accuracy_Explanation},
	{"LINEAGE",		WLineage},
	{"SOURCEINFO",	WSource_Information},
	{"SOURCECIT",	WSource_Citation},
	{"CITATNINFO",	WCitation_Information},
	{"SRCSCLDNOM",	WSource_Scale_Denominator},
	{"SRCMEDIA",	WType_of_Source_Media},
	{"SRCTIMEPD",	WSource_Time_Period_of_Content},
	{"TIMEPDINFO",	WTime_Period_Information},
	{"SRCCURREF",	WSource_Currentness_Reference},
	{"SRCCITABBR",	WSource_Citation_Abbreviation},
	{"SRCCONTRIB",	WSource_Contribution},
	{"PROCESSTEP",	WProcess_Step},
	{"PROCDESCR",	WProcess_Description},
	{"SRCUSEDCIT",	WSource_Used_Citation_Abbreviation},
	{"PROCDATE",	WProcess_Date},
	{"PROCTIME",	WProcess_Time},
	{"SRCPRODCAB",	WSource_Produced_Citation_Abbreviation},
	{"PROCCONTCT",	WProcess_Contact},
	{"CNTACTINFO",	WContact_Information},
	{"CLOUDCOVER",	WCloud_Cover},
	{"SDORGINFO",	WSpatial_Data_Organization_Information},
	{"INDIRSPREF",	WIndirect_Spatial_Reference},
	{"DIRSPREF",	WDirect_Spatial_Reference_Method},
	{"PTVCTROINF",	WPoint_and_Vector_Object_Information},
	{"SDTSTERMS",	WSDTS_Terms_Description},
	{"SDTSPAVOT",	WSDTS_Point_and_Vector_Object_Type},
	{"PAVOCOUNT",	WPoint_and_Vector_Object_Count},
	{"VPFTERMDSC",	WVPF_Terms_Description},
	{"VPFTOPLEV",	WVPF_Topology_Level},
	{"VPFPAVOT",	WVPF_Point_and_Vector_Object_Type},
	{"PAVOCOUNT",	WPoint_and_Vector_Object_Count},
	{"RASOBJINFO",	WRaster_Object_Information},
	{"RASTOBJTP",	WRaster_Object_Type},
	{"ROWCOUNT",	WRow_Count},
	{"COLCOUNT",	WColumn_Count},
	{"VERTCOUNT",	WVertical_Count},
	{"SPREFINFO",	WSpatial_Reference_Information},
	{"HORCORSDEF",	WHorizontal_Coordinate_System_Definition},
	{"GEOGRAPHIC",	WGeographic},
	{"LATRESOL",	WLatitude_Resolution},
	{"LONGRESOL",	WLongitude_Resolution},
	{"GEOCOORDUN",	WGeographic_Coordinate_Units},
	{"PLANAR",		WPlanar},
	{"MAPPROJECT",	WMap_Projection},
	{"MAPPRJNAME",	WMap_Projection_Name},
	{"ALBERSCEA",	WAlbers_Conical_Equal_Area},
	{"AZIMUTHAL",	WAzimuthal_Equidistant},
	{"EQUIDISTC",	WEquidistant_Conic},
	{"EQUIRECT",	WEquirectangular},
	{"GENERALVNP",	WGeneral_Vertical_Near_sided_Perspective},
	{"GNOMONIC",	WGnomonic},
	{"LAMBERTAZ",	WLambert_Azimuthal_Equal_Area},
	{"LAMBERTCC",	WLambert_Conformal_Conic},
	{"MERCATOR",	WMercator},
	{"MODSALASKA",	WModified_Stereographic_for_Alaska},
	{"MILLER",		WMiller_Cylindrical},
	{"OBMERCATOR",	WOblique_Mercator},
	{"ORTHO",		WOrthographic},
	{"PSTEREO",		WPolar_Stereographic},
	{"POLYCONIC",	WPolyconic},
	{"ROBINSON",	WRobinson},
	{"SINUSOIDAL",	WSinusoidal},
	{"SPOBLMERC",	WSpace_Oblique_Mercator_Landsat},
	{"STEREO",		WStereographic},
	{"TM",			WTransverse_Mercator},
	{"VANDERG",		Wvan_der_Grinten},
	{"STDPARALL",	WStandard_Parallel},
	{"LONGCNTMER",	WLongitude_of_Central_Meridian},
	{"LATPRJORIG",	WLatitude_of_Projection_Origin},
	{"FALSEEASTG",	WFalse_Easting},
	{"FALSENORTG",	WFalse_Northing},
	{"SCALEATEQT",	WScale_Factor_at_Equator},
	{"HTOFPERSP",	WHeight_of_Perspective_Point_Above_Surface},
	{"LONGOFPCTR",	WLongitude_of_Projection_Center},
	{"LATOFPCTR",	WLatitude_of_Projection_Center},
	{"SCALEATCTR",	WScale_Factor_at_Center_Line},
	{"OBLLINEAZM",	WOblique_Line_Azimuth},
	{"AZIMLANGLE",	WAzimuthal_Angle},
	{"AZIMMPLONG",	WAzimuth_Measure_Point_Longitude},
	{"OBLLINEPT",	WOblique_Line_Point},
	{"OBLLINELAT",	WOblique_Line_Latitude},
	{"OBLLINELON",	WOblique_Line_Longitude},
	{"STVLONGFPL",	WStraight_Vertical_Longitude_from_Pole},
	{"SCALEATPOR",	WScale_Factor_at_Projection_Origin},
	{"LANDSATNUM",	WLandsat_Number},
	{"PATHNUMBER",	WPath_Number},
	{"SCALEATCM",	WScale_Factor_at_Central_Meridian},
	{"OTHERPJDEF",	WOther_Projections_Definition},
	{"GRIDCRDSYS",	WGrid_Coordinate_System},
	{"GRIDCSYSNM",	WGrid_Coordinate_System_Name},
	{"UTMGRDINFO",	WUniversal_Transverse_Mercator},
	{"UTMZONENUM",	WUTM_Zone_Number},
	{"UPSGRDINFO",	WUniversal_Polar_Stereographic},
	{"UPSZONEID",	WUPS_Zone_Identifier},
	{"SPCSINFO",	WState_Plane_Coordinate_System},
	{"SPCSZONEID",	WSPCS_Zone_Identifier},
	{"ADRGCSYS",	WARC_Coordinate_System},
	{"ADRGZONEID",	WARC_System_Zone_Identifier},
	{"OTHRGRDDEF",	WOther_Grid_Systems_Definition},
	{"LOCALPLANR",	WLocal_Planar},
	{"LCLPLNRDES",	WLocal_Planar_Description},
	{"LCLPLNRGEO",	WLocal_Planar_Georeference_Information},
	{"PLNRCDINFO",	WPlanar_Coordinate_Information},
	{"PLNRENCMTH",	WPlanar_Coordinate_Encoding_Method},
	{"COORDREPRS",	WCoordinate_Representation},
	{"ABSCARESOL",	WAbscissa_Resolution},
	{"ORDINRESOL",	WOrdinate_Resolution},
	{"DSTBRGREPR",	WDistance_and_Bearing_Representation},
	{"DISTRESOL",	WDistance_Resolution},
	{"BRNGRESOL",	WBearing_Resolution},
	{"BRNGUNITS",	WBearing_Units},
	{"BRNGREFDIR",	WBearing_Reference_Direction},
	{"BRNGREFMER",	WBearing_Reference_Meridian},
	{"PLNRDSTUNT",	WPlanar_Distance_Units},
	{"LCLCRDSYS",	WLocal},
	{"LOCALDESCR",	WLocal_Description},
	{"LCLGEOINFO",	WLocal_Georeference_Information},
	{"GEODMODEL",	WGeodetic_Model},
	{"HORDTMNAME",	WHorizontal_Datum_Name},
	{"ELLPSDNAME",	WEllipsoid_Name},
	{"SMIMJRAXIS",	WSemi_major_Axis},
	{"DENOMFLATR",	WDenominator_of_Flattening_Ratio},
	{"VERTCSYSD",	WVertical_Coordinate_System_Definition},
	{"ALTSYSDEF",	WAltitude_System_Definition},
	{"ALTDTMNAME",	WAltitude_Datum_Name},
	{"ALTRESOL",	WAltitude_Resolution},
	{"ALTDISTUNT",	WAltitude_Distance_Units},
	{"ALTENCMTHD",	WAltitude_Encoding_Method},
	{"DPTHSYSDEF",	WDepth_System_Definition},
	{"DPTHDTMNME",	WDepth_Datum_Name},
	{"DPTHRESOL",	WDepth_Resolution},
	{"DPTHDSTUNS",	WDepth_Distance_Units},
	{"DPTHENCMTD",	WDepth_Encoding_Method},
	{"ENTATTINFO",	WEntity_and_Attribute_Information},
	{"ENTATTDESC",	WDetailed_Description},
	{"ENTITYTYPE",	WEntity_Type},
	{"ENTTYPELAB",	WEntity_Type_Label},
	{"ENTTYPEDEF",	WEntity_Type_Definition},
	{"ENTTPDFSRC",	WEntity_Type_Definition_Source},
	{"ATTRIBUTE",	WAttribute},
	{"ATTRLABEL",	WAttribute_Label},
	{"ATTRIBDEF",	WAttribute_Definition},
	{"ATTRDEFSRC",	WAttribute_Definition_Source},
	{"ATTRDOMVAL",	WAttribute_Domain_Values},
	{"ENUMDOMAIN",	WEnumerated_Domain},
	{"ENUMDOMVAL",	WEnumerated_Domain_Value},
	{"ENUMDVDEF",	WEnumerated_Domain_Value_Definition},
	{"ENUMDVDSRC",	WEnumerated_Domain_Value_Definition_Source},
	{"RANGEDOM",	WRange_Domain},
	{"RANGEDMMIN",	WRange_Domain_Minimum},
	{"RANGEDMMAX",	WRange_Domain_Maximum},
	{"CODESETDOM",	WCodeset_Domain},
	{"CODESETNAM",	WCodeset_Name},
	{"CODESETSRC",	WCodeset_Source},
	{"UNREPDMAIN",	WUnrepresentable_Domain},
	{"ATTUNITMES",	WAttribute_Units_of_Measure},
	{"ATTMEASRES",	WAttribute_Measurement_Resolution},
	{"BEGDATEATT",	WBeginning_Date_of_Attribute_Values},
	{"ENDDATEATT",	WEnding_Date_of_Attribute_Values},
	{"ATTVLAINFO",	WAttribute_Value_Accuracy_Information},
	{"ATTVALACCY",	WAttribute_Value_Accuracy},
	{"ATTVALACCE",	WAttribute_Value_Accuracy_Explanation},
	{"ATTMEASFRQ",	WAttribute_Measurement_Frequency},
	{"OVERVWDESC",	WOverview_Description},
	{"ENTATTOVRV",	WEntity_and_Attribute_Overview},
	{"ENTATTDETC",	WEntity_and_Attribute_Detail_Citation},
	{"DISTRBINFO",	WDistribution_Information},
	{"DISTRBUTOR",	WDistributor},
	{"CNTACTINFO",	WContact_Information},
	{"RESRCDESCR",	WResource_Description},
	{"DISTRLIABL",	WDistribution_Liability},
	{"STDORDPROC",	WStandard_Order_Process},
	{"NONDIGFORM",	WNon_digital_Form},
	{"DIGITLFORM",	WDigital_Form},
	{"DIGTFRINFO",	WDigital_Transfer_Information},
	{"FORMATNAME",	WFormat_Name},
	{"FMTVERSNUM",	WFormat_Version_Number},
	{"FMTVERDATE",	WFormat_Version_Date},
	{"FORMATSPEC",	WFormat_Specification},
	{"FMTINFOCON",	WFormat_Information_Content},
	{"FILEDECOMP",	WFile_Decompression_Technique},
	{"TRNSFRSIZE",	WTransfer_Size},
	{"DIGTFROPTN",	WDigital_Transfer_Option},
	{"ONLINEOPTN",	WOnline_Option},
	{"CMPCONINFO",	WComputer_Contact_Information},
	{"NETWRKADDR",	WNetwork_Address},
	{"NETWRKRNME",	WNetwork_Resource_Name},
	{"DIALUPINST",	WDialup_Instructions},
	{"LOWESTBPS",	WLowest_BPS},
	{"HIGHESTBPS",	WHighest_BPS},
	{"DATABITS",	WNumber_DataBits},
	{"STOPBITS",	WNumber_StopBits},
	{"PARITY",		WParity},
	{"COMPRAVAIL",	WCompression_Support},
	{"DUPHONENUM",	WDialup_Telephone},
	{"DUFILENAME",	WDialup_File_Name},
	{"ACCESSINST",	WAccess_Instructions},
	{"ONLNCMPTOS",	WOnline_Computer_and_Operating_System},
	{"OFFLINEOPT",	WOffline_Option},
	{"OFFLNMEDIA",	WOffline_Media},
	{"RCRDGCAPAC",	WRecording_Capacity},
	{"RCRDGDNSTY",	WRecording_Density},
	{"RCRDGDNSUN",	WRecording_Density_Units},
	{"RCRDGFORMT",	WRecording_Format},
	{"COMPTBINFO",	WCompatibility_Information},
	{"FEES",		WFees},
	{"ORDERINSTR",	WOrdering_Instructions},
	{"TURNAROUND",	WTurnaround},
	{"CUSTORDRPR",	WCustom_Order_Process},
	{"TECHPREREQ",	WTechnical_Prerequisites},
	{"AVAILTMPRD",	WAvailable_Time_Period},
	{"TIMEPDINFO",	WTime_Period_Information},
	{"METAREFRNC",	WMetadata_Reference_Information},
	{"METADATE",	WMetadata_Date},
	{"METARVWDAT",	WMetadata_Review_Date},
	{"METAFRVWDT",	WMetadata_Future_Review_Date},
	{"METACONTCT",	WMetadata_Contact},
	{"CNTACTINFO",	WContact_Information},
	{"METASTDNAM",	WMetadata_Standard_Name},
	{"METASTDVRN",	WMetadata_Standard_Version},
	{"METADATECV",	WMetadata_Time_Convention},
	{"METAACCCNS",	WMetadata_Access_Constraints},
	{"METAUSECNS",	WMetadata_Use_Constraints},
	{"METASECINF",	WMetadata_Security_Information},
	{"METASECSYS",	WMetadata_Security_Classification_System},
	{"METASECCLS",	WMetadata_Security_Classification},
	{"METASECHND",	WMetadata_Security_Handling_Description},
	{"CITATNINFO",	WCitation_Information},
	{"ORIGINATOR",	WOriginator},
	{"PUBLCNDATE",	WPublication_Date},
	{"PUBLCNTIME",	WPublication_Time},
	{"THETITLE",	WTitle},
	{"EDITION",		WEdition},
	{"TYPEOFMAP",	WGeospatial_Data_Presentation_Form},
	{"SERIESINFO",	WSeries_Information},
	{"SERIESNAME",	WSeries_Name},
	{"ISSUEIDENT",	WIssue_Identification},
	{"PUBLCNINFO",	WPublication_Information},
	{"PUBLPLACE",	WPublication_Place},
	{"PUBLISHER",	WPublisher},
	{"OTHCITINFO",	WOther_Citation_Details},
	{"ONLINELINK",	WOnline_Linkage},
	{"LGRWRKCITN",	WLarger_Work_Citation},
	{"CITATNINFO",	WCitation_Information},
	{"TIMEPDINFO",	WTime_Period_Information},
	{"SNGLDTTIME",	WSingle_Date_Time},
	{"CALNDRDATE",	WCalendar_Date},
	{"TIMEOFDAY",	WTime_of_Day},
	{"MULTDTTIME",	WMultiple_Dates_Times},
	{"RNGEDTTIME",	WRange_of_Dates_Times},
	{"BEGINDATE",	WBeginning_Date},
	{"BEGINTIME",	WBeginning_Time},
	{"ENDINGDATE",	WEnding_Date},
	{"ENDINGTIME",	WEnding_Time},
	{"CNTACTINFO",	WContact_Information},
	{"CNTCTPERSP",	WContact_Person_Primary},
	{"CNTCTPERSN",	WContact_Person},
	{"CONTACTORG",	WContact_Organization},
	{"CNTACTORGP",	WContact_Organization_Primary},
	{"CONTACTORG",	WContact_Organization},
	{"CNTCTPERSN",	WContact_Person},
	{"CONTACTPOS",	WContact_Position},
	{"CONTACTADD",	WContact_Address},
	{"ADDRESTYPE",	WAddress_Type},
	{"ADDRESS",		WAddress},
	{"CITY",		WCity},
	{"STATEPRVNC",	WState_or_Province},
	{"POSTALCODE",	WPostal_Code},
	{"COUNTRY",		WCountry},
	{"CONTACTVPH",	WContact_Voice_Telephone},
	{"CONTACTTPH",	WContact_TDD_TTY_Telephone},
	{"CONTACTFPH",	WContact_Facsimile_Telephone},
	{"CNTCTEMAIL",	WContact_Electronic_Mail_Address},
	{"HOURSOFSVC",	WHours_of_Service},
	{"CNTCTINSTR",	WContact_Instructions},
	{NULL,			Wnull}

		};
static int astm_tag_table_length = 0;

static int compare (const void *e1, const void *e2) {
	struct astm_tag *p = (struct astm_tag *) e1;
	struct astm_tag *q = (struct astm_tag *) e2;
	return (p->key - q->key);
	}

/*----------------------------------------------------------------------*/

char *astm_tag_of (enum fgdc_keyword key) {
	int i;
	struct astm_tag T,*pT;

	if (astm_tag_table_length == 0) {
		for (i=0; astm_tag_table[i].tag; i++);
		astm_tag_table_length = i;
		qsort (astm_tag_table,astm_tag_table_length,sizeof(struct astm_tag),compare);
		}

	if (key == Wnull) return ("");
	if (key == Wunknown) return ("");

	if (is_extension (key)) return (sgml_of_extension (key));

	T.key = key;
	pT = (struct astm_tag *) bsearch (&T,astm_tag_table,astm_tag_table_length,sizeof(struct astm_tag),compare);
	if (pT) return (pT->tag);
	else return ("");
	}

/*----------------------------------------------------------------------*/

enum fgdc_keyword key_of_astm_tag (char *astm_tag) {
	int i;

	if (astm_tag_table_length == 0) {
		for (i=0; astm_tag_table[i].tag; i++);
		astm_tag_table_length = i;
		qsort (astm_tag_table,astm_tag_table_length,sizeof(struct astm_tag),compare);
		}

	for (i=0; i < astm_tag_table_length; i++)
		if (stricmp (astm_tag_table[i].tag,astm_tag) == 0) return (astm_tag_table[i].key);

	i = extension_of_sgml (astm_tag);

	if (i == Unrecognized_Extension) return (Wunknown);
	else return (i);
	}

/*----------------------------------------------------------------------*\
\*----------------------------------------------------------------------*/
