/*----------------------------------------------------------------------*\
 | Code to provide minimal syntactical processing of scalar elements.	|
 | If a keyword is found as the child of a scalar element, reclassify	|
 | the child as text.													|
 |																		|
 | Peter N. Schweitzer (U.S. Geological Survey, Reston, VA 20192)		|
\*----------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "keyword.h"
#include "item.h"
#include "local.h"
#include "actions.h"

static enum fgdc_keyword scalar[] = {
	WARC_System_Zone_Identifier,
	WAbscissa_Resolution,
	WAbstract,
	WAccess_Constraints,
	WAccess_Instructions,
	WAddress,
	WAddress_Type,
	WAltitude_Datum_Name,
	WAltitude_Distance_Units,
	WAltitude_Encoding_Method,
	WAltitude_Resolution,
	WAttribute_Accuracy_Explanation,
	WAttribute_Accuracy_Report,
	WAttribute_Accuracy_Value,
	WAttribute_Definition,
	WAttribute_Definition_Source,
	WAttribute_Label,
	WAttribute_Measurement_Frequency,
	WAttribute_Measurement_Resolution,
	WAttribute_Units_of_Measure,
	WAttribute_Value_Accuracy,
	WAttribute_Value_Accuracy_Explanation,
	WAzimuth_Measure_Point_Longitude,
	WAzimuthal_Angle,
	WBearing_Reference_Direction,
	WBearing_Reference_Meridian,
	WBearing_Resolution,
	WBearing_Units,
	WBeginning_Date,
	WBeginning_Date_of_Attribute_Values,
	WBeginning_Time,
	WBrowse_Graphic_File_Description,
	WBrowse_Graphic_File_Name,
	WBrowse_Graphic_File_Type,
	WCalendar_Date,
	WCity,
	WCloud_Cover,
	WCodeset_Name,
	WCodeset_Source,
	WColumn_Count,
	WCompatibility_Information,
	WCompleteness_Report,
	WCompression_Support,
	WContact_Electronic_Mail_Address,
	WContact_Facsimile_Telephone,
	WContact_Instructions,
	WContact_Organization,
	WContact_Person,
	WContact_Position,
	WContact_TDD_TTY_Telephone,
	WContact_Voice_Telephone,
	WCountry,
	WCurrentness_Reference,
	WCustom_Order_Process,
	WData_Set_Credit,
	WDenominator_of_Flattening_Ratio,
	WDepth_Datum_Name,
	WDepth_Distance_Units,
	WDepth_Encoding_Method,
	WDepth_Resolution,
	WDialup_File_Name,
	WDialup_Telephone,
	WDirect_Spatial_Reference_Method,
	WDistance_Resolution,
	WDistribution_Liability,
	WEast_Bounding_Coordinate,
	WEdition,
	WEllipsoid_Name,
	WEnding_Date,
	WEnding_Date_of_Attribute_Values,
	WEnding_Time,
	WEntity_Type_Definition,
	WEntity_Type_Definition_Source,
	WEntity_Type_Label,
	WEntity_and_Attribute_Detail_Citation,
	WEntity_and_Attribute_Overview,
	WEnumerated_Domain_Value,
	WEnumerated_Domain_Value_Definition,
	WEnumerated_Domain_Value_Definition_Source,
	WFalse_Easting,
	WFalse_Northing,
	WFees,
	WFile_Decompression_Technique,
	WFormat_Information_Content,
	WFormat_Name,
	WFormat_Specification,
	WFormat_Version_Date,
	WFormat_Version_Number,
	WG_Ring_Latitude,
	WG_Ring_Longitude,
	WGeographic_Coordinate_Units,
	WGeospatial_Data_Presentation_Form,
	WGrid_Coordinate_System_Name,
	WHeight_of_Perspective_Point_Above_Surface,
	WHighest_BPS,
	WHorizontal_Datum_Name,
	WHorizontal_Positional_Accuracy_Explanation,
	WHorizontal_Positional_Accuracy_Report,
	WHorizontal_Positional_Accuracy_Value,
	WHours_of_Service,
	WIndirect_Spatial_Reference,
	WIssue_Identification,
	WLandsat_Number,
	WLatitude_Resolution,
	WLatitude_of_Projection_Center,
	WLatitude_of_Projection_Origin,
	WLocal_Description,
	WLocal_Georeference_Information,
	WLocal_Planar_Description,
	WLocal_Planar_Georeference_Information,
	WLogical_Consistency_Report,
	WLongitude_Resolution,
	WLongitude_of_Central_Meridian,
	WLongitude_of_Projection_Center,
	WLowest_BPS,
	WMaintenance_and_Update_Frequency,
	WMap_Projection_Name,
	WMetadata_Access_Constraints,
	WMetadata_Date,
	WMetadata_Future_Review_Date,
	WMetadata_Review_Date,
	WMetadata_Security_Classification,
	WMetadata_Security_Classification_System,
	WMetadata_Security_Handling_Description,
	WMetadata_Standard_Name,
	WMetadata_Standard_Version,
	WMetadata_Time_Convention,
	WMetadata_Use_Constraints,
	WNative_Data_Set_Environment,
	WNetwork_Resource_Name,
	WNon_digital_Form,
	WNorth_Bounding_Coordinate,
	WNumber_DataBits,
	WNumber_StopBits,
	WOblique_Line_Latitude,
	WOblique_Line_Longitude,
	WOffline_Media,
	WOnline_Computer_and_Operating_System,
	WOnline_Linkage,
	WOrdering_Instructions,
	WOrdinate_Resolution,
	WOriginator,
	WOther_Citation_Details,
	WOther_Grid_Systems_Definition,
	WOther_Projections_Definition,
	WParity,
	WPath_Number,
	WPlace_Keyword,
	WPlace_Keyword_Thesaurus,
	WPlanar_Coordinate_Encoding_Method,
	WPlanar_Distance_Units,
	WPoint_and_Vector_Object_Count,
	WPostal_Code,
	WProcess_Date,
	WProcess_Description,
	WProcess_Time,
	WProgress,
	WPublication_Date,
	WPublication_Place,
	WPublication_Time,
	WPublisher,
	WPurpose,
	WRange_Domain_Maximum,
	WRange_Domain_Minimum,
	WRaster_Object_Type,
	WRecording_Density,
	WRecording_Density_Units,
	WRecording_Format,
	WResource_Description,
	WRow_Count,
	WSDTS_Point_and_Vector_Object_Type,
	WSPCS_Zone_Identifier,
	WScale_Factor_at_Center_Line,
	WScale_Factor_at_Central_Meridian,
	WScale_Factor_at_Equator,
	WScale_Factor_at_Projection_Origin,
	WSecurity_Classification,
	WSecurity_Classification_System,
	WSecurity_Handling_Description,
	WSemi_major_Axis,
	WSeries_Name,
	WSource_Citation_Abbreviation,
	WSource_Contribution,
	WSource_Currentness_Reference,
	WSource_Produced_Citation_Abbreviation,
	WSource_Scale_Denominator,
	WSource_Used_Citation_Abbreviation,
	WSouth_Bounding_Coordinate,
	WStandard_Parallel,
	WState_or_Province,
	WStraight_Vertical_Longitude_from_Pole,
	WStratum_Keyword,
	WStratum_Keyword_Thesaurus,
	WSupplemental_Information,
	WTechnical_Prerequisites,
	WTemporal_Keyword,
	WTemporal_Keyword_Thesaurus,
	WTheme_Keyword,
	WTheme_Keyword_Thesaurus,
	WTime_of_Day,
	WTitle,
	WTransfer_Size,
	WTurnaround,
	WType_of_Source_Media,
	WUPS_Zone_Identifier,
	WUTM_Zone_Number,
	WUnrepresentable_Domain,
	WUse_Constraints,
	WVPF_Point_and_Vector_Object_Type,
	WVPF_Topology_Level,
	WVertical_Count,
	WVertical_Positional_Accuracy_Explanation,
	WVertical_Positional_Accuracy_Report,
	WVertical_Positional_Accuracy_Value,
	WWest_Bounding_Coordinate,

	/* FGDC-STD-001-1998 */

	WG_Ring,
	WProfile_Name,

	/* Extension for language */

	WMetadata_Language,

	/* Biological Data Profile */

	WDescription_of_Geographic_Extent,
	WAltitude_Minimum,
	WAltitude_Maximum,
	WTaxonomic_Keyword_Thesaurus,
	WTaxonomic_Keywords,
	WClassification_System_Modifications,
	WTaxonomic_Procedures,
	WTaxonomic_Completeness,
	WSpecimen,
	WGeneral_Taxonomic_Coverage,
	WTaxon_Rank_Name,
	WTaxon_Rank_Value,
	WApplicable_Common_Name,
	WAnalytical_Tool_Description,
	WTool_Access_Instructions,
	WTool_Computer_and_Operating_System,
	WMethodology_Type,
	WMethodology_Keyword_Thesaurus,
	WMethodology_Keyword,
	WMethodology_Description,
	WRecord_Delimiter,
	WNumber_Header_Lines,
	WDescription_of_Header_Content,
	WOrientation,
	WCase_Sensitive,
	WAuthentication,
	WQuote_Character,
	WData_Field_Name,
	WMissing_Value_Code,
	WData_Field_Width_Delimiter,
	WData_Field_Width,
	WGeologic_Time_Scale,
	WGeologic_Age_Estimate,
	WGeologic_Age_Uncertainty,
	WGeologic_Age_Explanation,

	Wnull
	};
static int scalar_length = 0;

static int compare (const void *e1, const void *e2) {
	const int *s1 = e1;
	const int *s2 = e2;
	return (*s1 - *s2);
	}

static void sort_scalars (void) {
	int i;

	if (scalar_length == 0) {
		for (i=0; scalar[i] != Wnull; i++);
		scalar_length = i;
		qsort (scalar,scalar_length,sizeof(enum fgdc_keyword),compare);
		}
	}

static int check_scalar_children (struct item *p) {
	char *s;
	int err = 0;

	/* key = (enum fgdc_keyword *) bsearch (&p->key,scalar,scalar_length,sizeof(enum fgdc_keyword),compare); */
	if (element_list_of (p->key) == NULL) {
		struct item *q = p->child;
		while (q) {
			if (q->key != Wunknown && q->key != Wblank) {

				/*----------------------------------------------------------*\
				 | A keyword has occurred at the beginning of a text value.	|
				 | Issue a warning and reclassify the node as unknown.  If	|
				 | there is a child associated with this value (i.e. more	|
				 | text on the line containing the keyword), concatenate	|
				 | the child's text with that of the keyword and delete the	|
				 | child node.												|
				\*----------------------------------------------------------*/

/*\
				fprintf (stderr,"Warning (line %d): element of type %s has child %s, expected %s; reclassified as text.\n",
					p->line_number,
					text_of(p->key),
					text_of(q->key),
					text_of(Wunknown));
\*/
				q->key = Wunknown;
				if (q->child) {

					/*------------------------------------------------------*\
					 | With the text encoding scheme, the child's text is	|
					 | certain to begin on the first non-blank character	|
					 | after the end of the keyword, so we need only remove	|
					 | the null character after the keyword to join the		|
					 | text of this node with that of its child.			|
					 |														|
					 | If some other encoding is used, the text may not be	|
					 | arranged so neatly; allocate space for the combined	|
					 | string and concatenate them with a space in between.	|
					\*------------------------------------------------------*/

					/*--------------------------------------------------*\
					 | If the data were read as XML, then q->d will be	|
					 | NULL but this situation occurs in XML only if	|
					 | we've found mixed content, and in this case the	|
					 | record is not valid.  We cannot work with this	|
					 | file without losing structural integrity.		|
					 | Consequently we return an error code.			|
					\*--------------------------------------------------*/

					if (q->d == NULL) {
						p->child = NULL;
						err = 1;
						return (err);
						}

					if (q->line_number == q->child->line_number) {
						s = q->d + strlen (q->d);
						*s = ' ';
						}
					else {
						if (q->d && q->child->d)
							if (s = (char *) malloc (strlen(q->d) + strlen(q->child->d) + 2)) {
								strcpy (s,q->d);
								strcat (s," ");
								strcat (s,q->child->d);
								q->d = s;
								}
							else {
								/* Failed to allocate space; don't mess with things */
								}
						}
					deallocate_item (q->child);
					q->child = NULL;
					}
				}
			q = q->next;
			}
		}
	return (err);
	}

/*----------------------------------------------------------------------*\
 | The function check_syntax here returns nonzero if the record cannot	|
 | be handled without loss of structural information.  This occurs only	|
 | if one of the elements has mixed content (both text and elements),	|
 | which should never occur but has at least once ;-).					|
\*----------------------------------------------------------------------*/

int check_syntax (struct item *p) {
	int n = 0;
	if (scalar_length == 0) sort_scalars();
	while (p) {
		n += check_scalar_children (p);
		if (p->child) n += check_syntax (p->child);
		p = p->next;
		}
	return (n);
	}

/*----------------------------------------------------------------------*\
\*----------------------------------------------------------------------*/
