/*----------------------------------------------------------------------*\
 | detag: remove tags bounded by <> from a text file.  Wherever we find	|
 | the combination "<dt><em>", we replace this with "@@" in the output,	|
 | so that cns can be configured to expect @@ as a prefix for element	|
 | names.																|
 |																		|
 | The output is to a file whose name is the same as the input file but	|
 | has .out appended to it.												|
 |																		|
 | Peter N. Schweitzer (U.S. Geological Survey, Reston VA 20192)		|
\*----------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "stricmp.h"

int main (int argc, char *argv[]) {
	long n;
	FILE *in,*out;
	char *input_file;
	char output_file[1024];
	char *dst,*end,*copy;
	char *s,*d,*t;
	long size,new_size;

	if (argc > 1) {
		input_file = argv[1];
		if (in = fopen (input_file,"rb")) {
			if (fseek (in,0L,SEEK_END) == 0) {
				size = 1L + ftell (in);
				if (size > 1L) {
					if (dst = (char *) malloc (size)) {
						rewind (in);
						n = fread (dst,1,size,in);
						end = dst + n;
						*end = 0;
						if (copy = (char *) malloc (size)) {
							d = copy;
							s = dst;

							/*------------------------------------------*\
							 | Try to find the beginning of the actual	|
							 | metadata, and start decoding there		|
							\*------------------------------------------*/

							for (t=s; t < end; t++)
								if (memicmp (t,"<DL>",4) == 0) {
									s = t;
									break;
									}

							/*------------------------------------------*\
							 | Try to find the end of the metadata, and	|
							 | stop decoding there.						|
							\*------------------------------------------*/

							for (t=end - 5; t >= s; t--)
								if (memicmp (t,"</DL>",5) == 0) {
									*t = 0;
									end = t;
									break;
									}

							/*------------------------------------------*\
							 | Walk through the metadata, converting	|
							 | things that look like elements to a form	|
							 | that can be reliably handled with cns.	|
							\*------------------------------------------*/

							while (s < end) {
								if (*s == '<')
									if (memicmp (s,"<dt><em>",8) == 0) {
										*d++ = '@';
										*d++ = '@';
										s += 8;
										}
									else {
										while (*s && *s != '>') s++;
										if (*s == '>') s++;
										}
								else
									*d++ = *s++;
								}
							*d = 0;
							new_size = d - copy;

							strcpy (output_file,input_file);
							strcat (output_file,".out");

							if (out = fopen (output_file,"wb")) {
								fwrite (copy,1,new_size,out);
								fclose (out);
								}
							else
								fprintf (stderr,"Error: could not create output file %s\n",output_file);
							}
						else
							fprintf (stderr,"Error: could not allocate space for results\n");
						}
					else
						fprintf (stderr,"Error: could not allocate space for input\n");
					}
				else
					fprintf (stderr,"Error: Input file is empty.\n");
				}
			else
				fprintf (stderr,"Error: could not seek to end of file\n");
			fclose (in);
			}
		else
			fprintf (stderr,"Error: could not open input file %s\n",input_file);
		}
	else
		fprintf (stderr,"Usage: %s input_file\n",argv[0]);
	}

/*----------------------------------------------------------------------*\
\*----------------------------------------------------------------------*/
