/*----------------------------------------------------------------------*\
 | Code to write DIF version of formal metadata.						|
 |																		|
 | Peter N. Schweitzer (U.S. Geological Survey, Reston, VA 20192)		|
\*----------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "keyword.h"
#include "item.h"
#include "stricmp.h"

/*----------------------------------------------------------------------*\
 | Functions to retrieve a keyword's text given its symbolic name and	|
 | vice versa.															|
\*----------------------------------------------------------------------*/

enum dif_keyword {
	DEntry_ID,
	DEntry_Title,
	DStart_date,
	DStop_date,
	DSensor_Name,
	DSource_Name,
	DInvestigator,
	DTechnical_Contact,
	DDIF_Author,
	DData_Center,
	DOriginating_Center,
	DStorage_Medium,
	DParameter,
	DDiscipline,
	DSpatial_Coverage,
	DLocation,
	DKeyword,
	DRevision_Date,
	DScience_Review_Date,
	DFuture_Review_Date,
	DReference,
	DQuality,
	DSummary,
	DGroup,
	DEnd_Group,
	DData_Center_Name,
	DDataset_ID,
	DData_Center_Contact,
	DLast_name,
	DFirst_name,
	DMiddle_name,
	DEmail,
	DPhone,
	DAddress,
	DCampaign,
	DProject,
	DSouthernmost_Latitude,
	DNorthernmost_Latitude,
	DEasternmost_Longitude,
	DWesternmost_Longitude,
	DData_Set_Citation,
	DOriginators,
	DTitle,
	DPublication,
	DIssue_Identification,
	DPublication_Date,
	DPublication_Place,
	DPublisher,
	DEdition,
	DData_Presentation_Form,
	DURL,
	DParameters,
	DCategory,
	DTopic,
	DTerm,
	DVariable,
	DDetailed_Variable,
	DData_Set_Progress,
	DMinimum_Altitude,
	DMaximum_Altitude,
	DMinimum_Depth,
	DMaximum_Depth,
	DData_Resolution,
	DLatitude_Resolution,
	DLongitude_Resolution,
	DAltitude_Resolution,
	DDepth_Resolution,
	DTemporal_Resolution,
	DAccess_Constraints,
	DUse_Constraints,
	DData_Set_Language,
	DDistribution,
	DDistribution_Media,
	DDistribution_Size,
	DDistribution_Format,
	DFees,
	DMultimedia_Sample,
	DFile,
	DFormat,
	DCaption,
	DDescription,
	DRelated_URL,
	DURL_Type,
	DParent_DIF,
	DIDN_Node,
	DDIF_Revision_Date,
	DTemporal_Coverage,
	Dunknown,
	Dblank,
	Dnull
	};

struct dif_keyword_list {
	enum dif_keyword key;
	char *text;
	};

static struct dif_keyword_list dif_keyword_list[] = {
	{DEntry_ID,					"Entry_ID"},
	{DEntry_Title,				"Entry_Title"},
	{DStart_date,				"Start_date"},
	{DStop_date,				"Stop_date"},
	{DSensor_Name,				"Sensor_Name"},
	{DSource_Name,				"Source_Name"},
	{DInvestigator,				"Investigator"},
	{DTechnical_Contact,		"Technical_Contact"},
	{DDIF_Author,				"DIF_Author"},
	{DData_Center,				"Data_Center"},
	{DOriginating_Center,		"Originating_Center"},
	{DStorage_Medium,			"Storage_Medium"},
	{DParameter,				"Parameter"},
	{DDiscipline,				"Discipline"},
	{DSpatial_Coverage,			"Spatial_Coverage"},
	{DLocation,					"Location"},
	{DKeyword,					"Keyword"},
	{DRevision_Date,			"Revision_Date"},
	{DScience_Review_Date,		"Science_Review_Date"},
	{DFuture_Review_Date,		"Future_Review_Date"},
	{DReference,				"Reference"},
	{DQuality,					"Quality"},
	{DSummary,					"Summary"},
	{DGroup,					"Group"},
	{DEnd_Group,				"End_Group"},
	{DData_Center_Name,			"Data_Center_Name"},
	{DDataset_ID,				"Dataset_ID"},
	{DData_Center_Contact,		"Data_Center_Contact"},
	{DLast_name,				"Last_name"},
	{DFirst_name,				"First_name"},
	{DMiddle_name,				"Middle_name"},
	{DEmail,					"Email"},
	{DPhone,					"Phone"},
	{DAddress,					"Address"},
	{DCampaign,					"Campaign"},
	{DProject,					"Project"},
	{DSouthernmost_Latitude,	"Southernmost_Latitude"},
	{DNorthernmost_Latitude,	"Northernmost_Latitude"},
	{DEasternmost_Longitude,	"Easternmost_Longitude"},
	{DWesternmost_Longitude,	"Westernmost_Longitude"},
	{DData_Set_Citation,		"Data_Set_Citation"},
	{DOriginators,				"Originator(s)"},
	{DTitle,					"Title"},
	{DPublication,				"Publication"},
	{DIssue_Identification,		"Issue_Identification"},
	{DPublication_Date,			"Publication_Date"},
	{DPublication_Place,		"Publication_Place"},
	{DPublisher,				"Publisher"},
	{DEdition,					"Edition"},
	{DData_Presentation_Form,	"Data_Presentation_Form"},
	{DURL,						"URL"},
	{DParameters,				"Parameters"},
	{DCategory,					"Category"},
	{DTopic,					"Topic"},
	{DTerm,						"Term"},
	{DVariable,					"Variable"},
	{DDetailed_Variable,		"Detailed_Variable"},
	{DData_Set_Progress,		"Data_Set_Progress"},
	{DMinimum_Altitude,			"Minimum_Altitude"},
	{DMaximum_Altitude,			"Maximum_Altitude"},
	{DMinimum_Depth,			"Minimum_Depth"},
	{DMaximum_Depth,			"Maximum_Depth"},
	{DData_Resolution,			"Data_Resolution"},
	{DLatitude_Resolution,		"Latitude_Resolution"},
	{DLongitude_Resolution,		"Longitude_Resolution"},
	{DAltitude_Resolution,		"Altitude_Resolution"},
	{DDepth_Resolution,			"Depth_Resolution"},
	{DTemporal_Resolution,		"Temporal_Resolution"},
	{DAccess_Constraints,		"Access_Constraints"},
	{DUse_Constraints,			"Use_Constraints"},
	{DData_Set_Language,		"Data_Set_Language"},
	{DDistribution,				"Distribution"},
	{DDistribution_Media,		"Distribution_Media"},
	{DDistribution_Size,		"Distribution_Size"},
	{DDistribution_Format,		"Distribution_Format "},
	{DFees,						"Fees"},
	{DMultimedia_Sample,		"Multimedia_Sample"},
	{DFile,						"File"},
	{DFormat,					"Format"},
	{DCaption,					"Caption"},
	{DDescription,				"Description"},
	{DRelated_URL,				"Related_URL"},
	{DURL_Type,					"URL_Type"},
	{DParent_DIF,				"Parent_DIF"},
	{DIDN_Node,					"IDN_Node"},
	{DDIF_Revision_Date,		"DIF_Revision_Date"},
	{DTemporal_Coverage,		"Temporal_Coverage"},
	{Dunknown,					"(unknown)"},
	{Dblank,					""},
	{Dnull,						NULL}
	};

static int dif_keyword_list_length = 0;

/*----------------------------------------------------------------------*/

static int compare (const void *e1, const void *e2) {
	struct dif_keyword_list *p = (struct dif_keyword_list *) e1;
	struct dif_keyword_list *q = (struct dif_keyword_list *) e2;
	return (strcmp (p->text,q->text));
	}

static enum dif_keyword dif_key_of (char *text) {
	int i;
	struct dif_keyword_list k,*key;
	char string[64];

	if (dif_keyword_list_length == 0) {
		for (i=0; dif_keyword_list[i].text; i++);
		dif_keyword_list_length = i;
		qsort (dif_keyword_list,dif_keyword_list_length,sizeof(struct dif_keyword_list),compare);
		}
	if (!text) return (Dnull);
	if (*text) {
		char *s = string;
		char *t = text;
		while (*t && !isspace(*t) && *t != ':' && *t != '=') *s++ = *t++;
		*s = 0;
		k.text = string;
		key = (struct dif_keyword_list *) bsearch (&k,dif_keyword_list,dif_keyword_list_length,sizeof(struct dif_keyword_list),compare);
		if (key) return (key->key);
		else return (Dunknown);
		}
	else return (Dblank);
	}

static char *dif_text_of (enum dif_keyword key) {
	int i;

	if (dif_keyword_list_length == 0) {
		for (i=0; dif_keyword_list[i].text; i++);
		dif_keyword_list_length = i;
		qsort (dif_keyword_list,dif_keyword_list_length,sizeof(struct dif_keyword_list),compare);
		}
	if (key == Dnull) return ("(null)");
	if (key == Dblank) return ("(blank)");
	for (i=0; i < dif_keyword_list_length; i++)
		if (dif_keyword_list[i].key == key) return (dif_keyword_list[i].text);
	return ("(unknown)");
	}

/*----------------------------------------------------------------------*\
\*----------------------------------------------------------------------*/

static struct item *parse_dif (char *name) {
	return (NULL);
	}

/*----------------------------------------------------------------------*\
\*----------------------------------------------------------------------*/

static void parse_name (char *name, char *first, char *middle, char *last) {
	char *b,*e;

	b = name;
	if (e = strchr (name,',')) {	/* Last, First Middle */
		if (e - b < 32) {
			memcpy (last,b,e-b);
			last[e-b] = 0;
			}
		else {
			memcpy (last,b,31);
			last[31] = 0;
			}
		e++;
		b = e; while (*b &&  isspace(*b)) b++;
		e = b; while (*e && !isspace(*e)) e++;
		if (e - b < 32) {
			memcpy (first,b,e-b);
			first[e-b] = 0;
			}
		else {
			memcpy (first,b,31);
			first[31] = 0;
			}
		b = e; while (*b && isspace(*b)) b++;
		if (strlen (b) < 32) {
			strcpy (middle,b);
			}
		else {
			memcpy (middle,b,31);
			middle[31] = 0;
			}
		}
	else {							/* First Middle Last */
		e = b; while (*e && !isspace(*e)) e++;
		if (e - b < 32) {
			memcpy (first,b,e-b);
			first[e-b] = 0;
			}
		else {
			memcpy (first,b,31);
			first[31] = 0;
			}
		b = e; while (*b &&  isspace(*b)) b++;
		e = b; while (*e && !isspace(*e)) e++;
		if (e - b < 32) {
			memcpy (middle,b,e-b);
			middle[e-b] = 0;
			}
		else {
			memcpy (middle,b,31);
			middle[31] = 0;
			}
		b = e; while (*b &&  isspace(*b)) b++;
		e = b; while (*e && !isspace(*e)) e++;

		/*--------------------------------------------------------------*\
		 | If no more words, what we put in middle is really the last	|
		 | name, otherwise store in the last name what remains.			|
		 | This won't properly parse names like "John Smith Jr."		|
		\*--------------------------------------------------------------*/

		if (e - b == 0) {
			strcpy (last,middle);
			*middle = 0;
			}
		else
			if (e - b < 32) {
				memcpy (last,b,e-b);
				last[e-b] = 0;
				}
			else {
				memcpy (last,b,31);
				last[31] = 0;
				}
		}
	}

static void write_contact_info (FILE *out, struct item *p, enum dif_keyword key) {
	char lname[256],fname[64],mname[64];
	struct item *q,*r;

	if (p->key == WContact_Information) {

		fprintf (out,"Group: %s\n",dif_text_of(key));

		/* Name */

		strcpy (lname,"N/A");
		strcpy (mname,"");
		strcpy (fname,"N/A");
		if (q = find_key (p,WContact_Person)) {
			if (q = q->child)
				parse_name (q->d,fname,mname,lname);
			}
		else
			if (q = find_key (p,WContact_Organization))
				if (q = q->child) strcpy (lname,q->d);

		fprintf (out,"  %s: %s\n",dif_text_of(DLast_name),lname);
		fprintf (out,"  %s: %s\n",dif_text_of(DFirst_name),fname);
		if (*mname) fprintf (out,"  %s: %s\n",dif_text_of(DMiddle_name),mname);

		/* Email */

		if (q = find_key (p,WContact_Electronic_Mail_Address)) {
			if (q->child)
				fprintf (out,"  %s: %s\n",dif_text_of(DEmail),q->child->d);
			while (q = find_next_key (q,WContact_Electronic_Mail_Address))
				if (q->child)
					fprintf (out,"  %s: %s\n",dif_text_of(DEmail),q->child->d);
			}

		/* Phone */

		if (q = find_key (p,WContact_Voice_Telephone)) {
			if (q->child)
				fprintf (out,"  %s: %s\n",dif_text_of(DPhone),q->child->d);
			if (q = find_next_key (q,WContact_Voice_Telephone))
				if (q->child)
					fprintf (out,"  %s: %s\n",dif_text_of(DPhone),q->child->d);
			}

		/* Address */

		if (q = find_key (p,WContact_Address)) {
			fprintf (out,"  Group: Address\n");
			if (r = find_key (q,WAddress)) {
				if (r = r->child)
					do {
						fprintf (out,"    %s\n",r->d);
						r = r->next;
						} while (r);
				}
			if (r = find_key (q,WCity))
				if (r = r->child)
					fprintf (out,"    %s, ",r->d);
			if (r = find_key (q,WState_or_Province))
				if (r = r->child)
					fprintf (out,"%s ",r->d);
			if (r = find_key (q,WPostal_Code))
				if (r = r->child)
					fprintf (out,"%s",r->d);
			fprintf (out,"\n");
			if (r = find_key (q,WCountry))
				if (r = r->child)
					fprintf (out,"    %s\n",r->d);
			fprintf (out,"  End_Group\n");
			}
		else {
			fprintf (out,"  Group: %s\n",dif_text_of(DAddress));
			fprintf (out,"    N/A\n");
			fprintf (out,"  End_Group\n");
			}

		fprintf (out,"End_Group\n");
		}
	}

/*----------------------------------------------------------------------*\
\*----------------------------------------------------------------------*/

static char *one_indent = "  ";	/* Set to desired indentation indicator */

static void write_text_item (FILE *fp, struct item *p, int n) {
	int i;
	if (!p) return;
	if (p->key == Wblank) fprintf (fp,"\n");
	else
		if (p->key == Wunknown) {
			for (i=0; i < n; i++) fprintf (fp,one_indent);
			fprintf (fp,"%s\n",p->d);
			}
		else {
			struct item *q;

			for (i=0; i < n; i++) fprintf (fp,one_indent);
			fprintf (fp,"%s:",text_of(p->key));
			if (p->child)
				if (p->child->key == Wunknown && p->child->next == NULL) {
					fprintf (fp," %s\n",p->child->d);
					return;
					}
			fprintf (fp,"\n");
			for (q=p->child; q; q=q->next) write_text_item (fp,q,n+1);
			}
	}

/*----------------------------------------------------------------------*\
\*----------------------------------------------------------------------*/

void write_dif (char *output_file, struct item *first) {
	FILE *out;
	struct item *p,*q,*r,*rr,*u,*v;
	char *title;
	int title_allocated = 0;
	enum fgdc_keyword key;

	if (first == NULL) return;

	if (out = fopen (output_file,"w")) {

		/*--------------------------------------------------------------*\
		 | Entry_ID
		\*--------------------------------------------------------------*/

		fprintf (out,"%s: ",dif_text_of(DEntry_ID));
		fprintf (out,"(required)");
		fprintf (out,"\n");

		/*--------------------------------------------------------------*\
		 | Entry_Title
		\*--------------------------------------------------------------*/

		title = NULL;
		if (p = find_key (first,WCitation))
			if (p = find_key (p,WTitle))
				if (p->child)
					if (p->child->key == Wunknown) {
						int n = 0;
						for (q=p->child; q; q=q->next) n += 1 + strlen (q->d);
						if (title = (char *) malloc (1 + n)) {
							char *s = title;
							for (q=p->child; q; q=q->next) {
								strcpy (s,q->d);
								s += strlen (q->d);
								if (q->next) *s++ = ' ';
								*s = 0;
								}
							title_allocated = 1;
							}
						else
							title = p->child->d;
						}

			fprintf (out,"%s: ",dif_text_of(DEntry_Title));
			if (title) fprintf (out,"%s",title);
			else fprintf (out,"(required)");
			fprintf (out,"\n");

		/*--------------------------------------------------------------*\
		 | Data_Set_Citation
		\*--------------------------------------------------------------*/

		if (p = find_key (first,WCitation)) {
			fprintf (out,"Group: %s\n",dif_text_of(DData_Set_Citation));

			if (q = find_key (p,WOriginator)) {
				int n = 0;
				fprintf (out,"  %s:",dif_text_of(DOriginators));
				do {
					if (r = q->child)
						if (r->key == Wunknown && r->d) {
							if (n) fprintf (out,";");
							fprintf (out," %s",r->d);
							n++;
							}
					} while (q = find_next_key (q,WOriginator));
				fprintf (out,"\n");
				}

			fprintf (out,"  %s:",dif_text_of(DTitle));
			if (title) fprintf (out," %s",title);
			else fprintf (out,"(required)");
			fprintf (out,"\n");

			if (q = find_key (p,WSeries_Name))
				if (q = q->child)
					if (q->key == Wunknown && q->d) {
						fprintf (out,"  %s:",dif_text_of(DPublication));
						fprintf (out," %s",q->d);
						fprintf (out,"\n");
						}

			if (q = find_key (p,WPublication_Date))
				if (q = q->child)
					if (q->key == Wunknown && q->d) {
						fprintf (out,"  %s:",dif_text_of(DPublication_Date));
						fprintf (out," %s",q->d);
						fprintf (out,"\n");
						}

			if (q = find_key (p,WPublication_Place))
				if (q = q->child)
					if (q->key == Wunknown && q->d) {
						fprintf (out,"  %s:",dif_text_of(DPublication_Place));
						fprintf (out," %s",q->d);
						fprintf (out,"\n");
						}

			if (q = find_key (p,WPublisher))
				if (q = q->child)
					if (q->key == Wunknown && q->d) {
						fprintf (out,"  %s:",dif_text_of(DPublisher));
						fprintf (out," %s",q->d);
						fprintf (out,"\n");
						}

			if (q = find_key (p,WEdition))
				if (q = q->child)
					if (q->key == Wunknown && q->d) {
						fprintf (out,"  %s:",dif_text_of(DEdition));
						fprintf (out," %s",q->d);
						fprintf (out,"\n");
						}

			if (q = find_key (p,WGeospatial_Data_Presentation_Form))
				if (q = q->child)
					if (q->key == Wunknown && q->d) {
						fprintf (out,"  %s:",dif_text_of(DData_Presentation_Form));
						fprintf (out," %s",q->d);
						fprintf (out,"\n");
						}

			if (q = find_key (p,WOnline_Linkage))
				do {
					if (r = q->child)
						if (r->key == Wunknown && r->d) {
							fprintf (out,"  %s:",dif_text_of(DURL));
							fprintf (out," %s",r->d);
							fprintf (out,"\n");
							}
					} while (q = find_next_key (q,WOnline_Linkage));

			fprintf (out,"End_Group\n");
			}

		/*--------------------------------------------------------------*\
		 | Investigator
		\*--------------------------------------------------------------*/

		if (key = key_of ("Investigator_Contact")) {
			if (p = find_key (first,key))
				do {
					if (q = p->child)
						write_contact_info (out,q,DInvestigator);
					} while (p = find_next_key (p,key));
			}
		else
			if (key = key_of ("Originator_Contact")) {
				if (p = find_key (first,key))
					do {
						if (q = p->child)
							write_contact_info (out,q,DInvestigator);
						} while (p = find_next_key (p,key));
				}
			else
				if (p = find_key (first,WCitation))
					if (p = find_key (p,WOriginator))
						do {
							if (q = p->child) {
								char fname[32],mname[32],lname[32];
								parse_name (q->d,fname,mname,lname);
								fprintf (out,"Group: %s\n",dif_text_of(DInvestigator));
								fprintf (out,"  %s: %s\n",dif_text_of(DLast_name),lname);
								fprintf (out,"  %s: %s\n",dif_text_of(DFirst_name),fname);
								if (*mname) fprintf (out,"  %s: %s\n",dif_text_of(DMiddle_name),mname);
								fprintf (out,"End_Group\n");
								}
							} while (p = find_next_key (p,WOriginator));

		/*--------------------------------------------------------------*\
		 | Technical_Contact
		\*--------------------------------------------------------------*/

		if (p = find_key (first,WCitation))
			if (p = find_key (p,WPoint_of_Contact))
				if (p = p->child)
					write_contact_info (out,p,DTechnical_Contact);


		/*--------------------------------------------------------------*\
		 | Discipline skipped here; not found in CSDGM and being phased	|
		 | out by GCMD.													|
		\*--------------------------------------------------------------*/

		/*--------------------------------------------------------------*\
		 | Parameters skipped here; these assume conformance to GCMD	|
		 | valids list and its hierarchy.								|
		\*--------------------------------------------------------------*/

		/*--------------------------------------------------------------*\
		 | Keyword: CSDGM Theme, Stratum, and Temporal
		\*--------------------------------------------------------------*/

		if (p = find_key (first,WKeywords)) {
			if (q = find_key (p,WTheme))
				if (r = find_key (q,WTheme_Keyword))
					do {
						if (r->child)
							if (r->child->key == Wunknown && r->child->d) {
								fprintf (out,"Keyword:");
								fprintf (out," %s",r->child->d);
								fprintf (out,"\n");
								}
						} while (r = find_next_key (r,WTheme_Keyword));

			if (q = find_key (p,WStratum))
				if (r = find_key (q,WStratum_Keyword))
					do {
						if (r->child)
							if (r->child->key == Wunknown && r->child->d) {
								fprintf (out,"Keyword:");
								fprintf (out," %s",r->child->d);
								fprintf (out,"\n");
								}
						} while (r = find_next_key (r,WStratum_Keyword));

			if (q = find_key (p,WTemporal))
				if (r = find_key (q,WTemporal_Keyword))
					do {
						if (r->child)
							if (r->child->key == Wunknown && r->child->d) {
								fprintf (out,"Keyword:");
								fprintf (out," %s",r->child->d);
								fprintf (out,"\n");
								}
						} while (r = find_next_key (r,WTemporal_Keyword));

			}

		/*--------------------------------------------------------------*\
		 | Sensor_Name skipped; not available in CSDGM
		fprintf (out,"%s: ",dif_text_of(DSensor_Name));
		fprintf (out,"\n");
		\*--------------------------------------------------------------*/

		/*--------------------------------------------------------------*\
		 | Source_Name skipped, not available in CSDGM
		fprintf (out,"%s: ",dif_text_of(DSource_Name));
		fprintf (out,"\n");
		\*--------------------------------------------------------------*/

		/*--------------------------------------------------------------*\
		 | Start_date and Stop_date
		\*--------------------------------------------------------------*/

		if (p = find_key (first,WIdentification_Information))
			if (p = find_key (p,WTime_Period_of_Content))
				if (p = find_key (p,WTime_Period_Information))
					if (p = p->child) {
						int n = 0;
						switch (p->key) {
							case WSingle_Date_Time:
								if (p = p->child)
									if (p->key == WCalendar_Date)
										if (p = p->child) {
											fprintf (out,"Group: %s\n",dif_text_of(DTemporal_Coverage));
											fprintf (out,"  %s: ",dif_text_of(DStart_date));
											fprintf (out,"%s",p->d);
											fprintf (out,"\n");
											fprintf (out,"  %s: ",dif_text_of(DStop_date));
											fprintf (out,"%s",p->d);
											fprintf (out,"\n");
											fprintf (out,"End_Group\n");
											}
								break;
							case WMultiple_Dates_Times:
								for (p=p->child; p; p=p->next)
									if (p->key == WCalendar_Date) {
										if (q = p->child) {
											fprintf (out,"Group: %s\n",dif_text_of(DTemporal_Coverage));
											fprintf (out,"  %s: ",dif_text_of(DStart_date));
											fprintf (out,"%s",q->d);
											fprintf (out,"\n");
											fprintf (out,"End_Group\n");
											}
										}
								break;
							case WRange_of_Dates_Times:
								p = p->child;
								n = 0;
								while (p) {
									if (p->key == WBeginning_Date)
										if (q = p->child) {
											if (n == 0) fprintf (out,"Group: %s\n",dif_text_of(DTemporal_Coverage));
											fprintf (out,"  %s: ",dif_text_of(DStart_date));
											fprintf (out,"%s",q->d);
											fprintf (out,"\n");
											n = 1;
											}
									if (p->key == WEnding_Date)
										if (q = p->child) {
											if (n == 0) fprintf (out,"Group: %s\n",dif_text_of(DTemporal_Coverage));
											fprintf (out,"  %s: ",dif_text_of(DStop_date));
											fprintf (out,"%s",q->d);
											fprintf (out,"\n");
											if (n > 0) {
												fprintf (out,"End_Group\n");
												n = 0;
												}
											}
									p = p->next;
									}
								if (n > 0) {
									fprintf (out,"End_Group\n");
									n = 0;
									}
								break;
							default:
								break;
							}
						}

		/*--------------------------------------------------------------*\
		 | Data_Set_Progress
		\*--------------------------------------------------------------*/

		if (p = find_key (first,WIdentification_Information))
			if (p = find_key (p,WProgress))
				if (p = p->child)
					if (p->key == Wunknown) {
						fprintf (out,"%s: ",dif_text_of(DData_Set_Progress));
						if (p->d) fprintf (out,"%s",p->d);
						fprintf (out,"\n");
						}

		/*--------------------------------------------------------------*\
		 | Spatial Coverage
		\*--------------------------------------------------------------*/

		if (p = find_key (first,WIdentification_Information))
			if (p = find_key (p,WBounding_Coordinates)) {
				fprintf (out,"Group: %s\n",dif_text_of(DSpatial_Coverage));
				if (q = find_key (p,WSouth_Bounding_Coordinate))
					if (q->child)
						fprintf (out,"  %s: %s\n",dif_text_of(DSouthernmost_Latitude),q->child->d);
				if (q = find_key (p,WNorth_Bounding_Coordinate))
					if (q->child)
						fprintf (out,"  %s: %s\n",dif_text_of(DNorthernmost_Latitude),q->child->d);
				if (q = find_key (p,WWest_Bounding_Coordinate))
					if (q->child)
						fprintf (out,"  %s: %s\n",dif_text_of(DWesternmost_Longitude),q->child->d);
				if (q = find_key (p,WEast_Bounding_Coordinate))
					if (q->child)
						fprintf (out,"  %s: %s\n",dif_text_of(DEasternmost_Longitude),q->child->d);
				if (key = key_of ("Minimum_Elevation"))
					if (q = find_key (p,key))
						if (q->child)
							fprintf (out,"  %s: %s\n",dif_text_of(DMinimum_Altitude),q->child->d);
				if (key = key_of ("Maximum_Elevation"))
					if (q = find_key (p,key))
						if (q->child)
							fprintf (out,"  %s: %s\n",dif_text_of(DMaximum_Altitude),q->child->d);
				fprintf (out,"End_Group\n");
				}

		/*--------------------------------------------------------------*\
		 | Location
		\*--------------------------------------------------------------*/

		if (p = find_key (first,WKeywords))
			if (q = find_key (p,WPlace))
				if (r = find_key (q,WPlace_Keyword))
					do {
						if (r->child)
							if (r->child->key == Wunknown && r->child->d) {
								fprintf (out,"Location:");
								fprintf (out," %s",r->child->d);
								fprintf (out,"\n");
								}
						} while (r = find_next_key (r,WPlace_Keyword));

		/*--------------------------------------------------------------*\
		 | Data_Resolution
		\*--------------------------------------------------------------*/

		fprintf (out,"Group: %s\n",dif_text_of(DData_Resolution));
		if (p = find_key (first,WSpatial_Reference_Information)) {
			if (q = find_key (p,WGeographic)) {
				if (r = find_key (q,WLatitude_Resolution))
					if (r = r->child) {
						fprintf (out,"  %s: ",dif_text_of(DLatitude_Resolution));
						fprintf (out,"%s",r->d);
						fprintf (out,"\n");
						}
				if (r = find_key (q,WLongitude_Resolution))
					if (r = r->child) {
						fprintf (out,"  %s: ",dif_text_of(DLongitude_Resolution));
						fprintf (out,"%s",r->d);
						fprintf (out,"\n");
						}
				}
			else
				if (q = find_key (p,WPlanar_Coordinate_Information)) {
					if (r = find_key (q,WOrdinate_Resolution))
						if (r = r->child) {
							fprintf (out,"  %s: ",dif_text_of(DLatitude_Resolution));
							fprintf (out,"%s",r->d);
							fprintf (out,"\n");
							}
					if (r = find_key (q,WAbscissa_Resolution))
						if (r = r->child) {
							fprintf (out,"  %s: ",dif_text_of(DLongitude_Resolution));
							fprintf (out,"%s",r->d);
							fprintf (out,"\n");
							}
					}
			if (q = find_key (p,WAltitude_Resolution)) {	/* Repeatable? */
				if (r = q->child) {
					fprintf (out,"  %s: ",dif_text_of(DAltitude_Resolution));
					if (r->d) fprintf (out,"%s",r->d);
					fprintf (out,"\n");
					}
				}
			if (q = find_key (p,WDepth_Resolution)) {	/* Repeatable? */
				if (r = q->child) {
					fprintf (out,"  %s: ",dif_text_of(DDepth_Resolution));
					if (r->d) fprintf (out,"%s",r->d);
					fprintf (out,"\n");
					}
				}
			}
		if (p = find_key (first,WEntity_and_Attribute_Information)) {
			if (q = find_key (p,WAttribute))
				do {
					if (r = find_key (q,WAttribute_Measurement_Frequency))
						if (r = r->child) {
							fprintf (out,"  %s:",dif_text_of(DTemporal_Resolution));
							do {
								if (r->d) fprintf (out," %s",r->d);
								} while (r = r->next);
							fprintf (out,"\n");
							}
					} while (q = find_next_key (q,WAttribute));
			}
		fprintf (out,"End_Group\n");

		/*--------------------------------------------------------------*\
		 | Project
		fprintf (out,"%s: ",dif_text_of(DProject));
		fprintf (out,"\n");
		\*--------------------------------------------------------------*/

		/*--------------------------------------------------------------*\
		 | Access_Constraints
		\*--------------------------------------------------------------*/

		if (p = find_key (first,WIdentification_Information))
			if (q = find_key (p,WAccess_Constraints))
				if (r = q->child) {
					fprintf (out,"  %s:",dif_text_of(DAccess_Constraints));
					do {
						if (r->d) fprintf (out," %s",r->d);
						} while (r = r->next);
					fprintf (out,"\n");
					}

		/*--------------------------------------------------------------*\
		 | Use_Constraints
		\*--------------------------------------------------------------*/

		if (p = find_key (first,WIdentification_Information))
			if (q = find_key (p,WUse_Constraints))
				if (r = q->child) {
					fprintf (out,"  %s:",dif_text_of(DUse_Constraints));
					do {
						if (r->d) fprintf (out," %s",r->d);
						} while (r = r->next);
					fprintf (out,"\n");
					}

		/*--------------------------------------------------------------*\
		 | Data_Set_Language
		\*--------------------------------------------------------------*/

		if (key = key_of ("Language"))
			if (p = find_key (first,key))
				if (q = p->child) {
					fprintf (out,"  %s: ",dif_text_of(DData_Set_Language));
					fprintf (out,"%s",q->d);
					fprintf (out,"\n");
					}

		/*--------------------------------------------------------------*\
		 | Originating_Center is deprecated
		\*--------------------------------------------------------------*/

		fprintf (out,"%s: ",dif_text_of(DOriginating_Center));
		fprintf (out,"(required)");
		fprintf (out,"\n");

		/*--------------------------------------------------------------*\
		 | Data_Center													|
		\*--------------------------------------------------------------*/

		if (p = find_key (first,WDistribution_Information)) {
			fprintf (out,"Group: %s\n",dif_text_of(DData_Center));

			/*----------------------------------------------------------*\
			 | A Contact_Organization will be the Data_Center.			|
			\*----------------------------------------------------------*/

			if (q = find_key (p,WContact_Organization))
				if (q = q->child)
					fprintf (out,"  %s: %s\n",dif_text_of(DData_Center_Name),q->d);
				else
					fprintf (out,"  %s: (required)\n",dif_text_of(DData_Center_Name));
			else
				fprintf (out,"  %s: (required)\n",dif_text_of(DData_Center_Name));

			/*----------------------------------------------------------*\
			 | The first Resource_Description following the Distributor	|
			 | is the Dataset_ID.										|
			\*----------------------------------------------------------*/

			if (q = find_key (p,WResource_Description))
				if (q = q->child)
					fprintf (out,"  %s: %s\n",dif_text_of(DDataset_ID),q->d);

			/*----------------------------------------------------------*\
			 | Distributor is Data_Center_Contact.						|
			\*----------------------------------------------------------*/

			if (q = find_key (p,WDistributor))
				if (q = q->child)
					write_contact_info (out,q,DData_Center_Contact);

			fprintf (out,"End_Group\n");
			}

		/*--------------------------------------------------------------*\
		 | Storage_Medium
		 | may need to truncate the value to fit in DIF catalogs
		\*--------------------------------------------------------------*/

		if (p = find_key (first,WIdentification_Information))
			if (q = find_key (p,WNative_Data_Set_Environment))
				if (r = q->child) {
					fprintf (out,"  %s:",dif_text_of(DStorage_Medium));
					do {
						if (r->d) fprintf (out," %s",r->d);
						} while (r = r->next);
					fprintf (out,"\n");
					}

		/*--------------------------------------------------------------*\
		 | Distribution
		\*--------------------------------------------------------------*/

		if (p = find_key (first,WDistribution_Information)) {
			if (q = find_key (p,WStandard_Order_Process))
				do {
					char *fees = NULL;
					int fees_allocated = 0;

					if (r = find_key (q,WFees)) {
						int n = 0;
						for (rr=r->child; rr; rr=rr->next) n += 1 + strlen (rr->d);
						if (fees = (char *) malloc (1 + n)) {
							char *s = fees;
							for (rr=r->child; rr; rr=rr->next) {
								strcpy (s,rr->d);
								s += strlen (rr->d);
								if (rr->next) *s++ = ' ';
								*s = 0;
								}
							fees_allocated = 1;
							}
						else
							fees = p->child->d;
						}

					if (r = find_key (q,WDigital_Form))
						do  {
							char *format_name = NULL;
							char *transfer_size = NULL;

							if (rr = find_key (r,WFormat_Name))
								if (rr = rr->child)
									format_name = rr->d;

							if (rr = find_key (r,WTransfer_Size))
								if (rr = rr->child)
									transfer_size = rr->d;

							/*------------------------------------------*\
							 | For each Digital_Transfer_Option, get	|
							 | "online" or the value of Offline_Media,	|
							 | and with them output a DIF Distribution	|
							\*------------------------------------------*/

							if (rr = find_key (r,WDigital_Transfer_Option))
								for (u=rr->child; u; u=u->next) {

									if (u->key == WOnline_Option) {
										fprintf (out,"Group: %s\n",dif_text_of(DDistribution));
										fprintf (out,"  %s: ",dif_text_of(DDistribution_Media));
										fprintf (out,"%s\n","online");
										if (format_name) {
											fprintf (out,"  %s: ",dif_text_of(DDistribution_Format));
											fprintf (out,"%s\n",format_name);
											}
										if (transfer_size) {
											fprintf (out,"  %s: ",dif_text_of(DDistribution_Size));
											fprintf (out,"%s\n",transfer_size);
											}
										if (fees) {
											fprintf (out,"  %s: ",dif_text_of(DFees));
											fprintf (out,"%s\n",fees);
											}
										fprintf (out,"End_Group\n");
										}

									if (u->key == WOffline_Option)
										if (v = find_key (u,WOffline_Media))
											if (v = v->child)
												if (v->d) {
													fprintf (out,"Group: %s\n",dif_text_of(DDistribution));
													fprintf (out,"  %s: ",dif_text_of(DDistribution_Media));
													fprintf (out,"%s\n",v->d);
													if (format_name) {
														fprintf (out,"  %s: ",dif_text_of(DDistribution_Format));
														fprintf (out,"%s\n",format_name);
														}
													if (transfer_size) {
														fprintf (out,"  %s: ",dif_text_of(DDistribution_Size));
														fprintf (out,"%s\n",transfer_size);
														}
													if (fees) {
														fprintf (out,"  %s: ",dif_text_of(DFees));
														fprintf (out,"%s\n",fees);
														}
													fprintf (out,"End_Group\n");
													}

									}
							} while (r = find_next_key (r,WDigital_Form));
					if (fees_allocated) free (fees);
					} while (q = find_next_key (q,WStandard_Order_Process));
			}

		/*--------------------------------------------------------------*\
		 | Multimedia Sample
		\*--------------------------------------------------------------*/

		if (p = find_key (first,WIdentification_Information)) {
			if (p = find_key (p,WBrowse_Graphic)) {
				do {
					fprintf (out,"Group: %s\n",dif_text_of(DMultimedia_Sample));
					if (q = find_key (p,WBrowse_Graphic_File_Name))
						if (q = q->child)
							if (q->d)
								if (strstr (q->d,"http")) {
									fprintf (out,"  %s: ",dif_text_of(DURL));
									fprintf (out,"%s\n",q->d);
									}
								else {
									fprintf (out,"  %s: ",dif_text_of(DFile));
									fprintf (out,"%s\n",q->d);
									}
					if (q = find_key (p,WBrowse_Graphic_File_Type))
						if (q = q->child)
							if (q->d) {
								fprintf (out,"  %s: ",dif_text_of(DFormat));
								fprintf (out,"%s\n",q->d);
								}
					if (q = find_key (p,WBrowse_Graphic_File_Description))
						if (q = q->child) {
							int n = 0;
							for (r=q; r; r=r->next)
								if (r->d) n += 1 + strlen (r->d);
							if (n < 80) {
								fprintf (out,"  %s:",dif_text_of(DCaption));
								for (r=q; r; r=r->next)
									if (r->d) fprintf (out," %s",r->d);
								fprintf (out,"\n");
								}
							else {
								fprintf (out,"  Group: %s\n",dif_text_of(DDescription));
								for (r=q; r; r=r->next)
									if (r->d) fprintf (out,"    %s\n",r->d);
								fprintf (out,"  End_Group\n");
								}
							}
					fprintf (out,"End_Group\n");
					} while (p = find_next_key (p,WBrowse_Graphic));
				}
			}

		/*--------------------------------------------------------------*\
		 | Reference
		\*--------------------------------------------------------------*/

		if (p = find_key (first,WIdentification_Information)) {
			fprintf (out,"Group: %s\n",dif_text_of(DReference));
			if (p = find_key (p,WCross_Reference)) {
				do {
					for (q=p->child; q; q=q->next) write_text_item (out,q,1);
					fprintf (out,"\n");
					} while (p = find_next_key (p,WCross_Reference));
				}
			fprintf (out,"End_Group\n");
			}

		/*--------------------------------------------------------------*\
		 | Summary
		\*--------------------------------------------------------------*/

		if (p = find_key (first,WDescription)) {
			fprintf (out,"Group: %s\n",dif_text_of(DSummary));

			if (q = find_key (p,WPurpose))
				for (q=q->child; q; q=q->next)
					fprintf (out,"  %s\n",q->d);

			if (q = find_key (p,WAbstract)) {
				fprintf (out,"  \n");
				for (q=q->child; q; q=q->next)
					fprintf (out,"  %s\n",q->d);
				}

			if (q = find_key (p,WSupplemental_Information)) {
				fprintf (out,"  \n");
				for (q=q->child; q; q=q->next)
					fprintf (out,"  %s\n",q->d);
				}

			fprintf (out,"End_Group\n");
			}

		/*--------------------------------------------------------------*\
		 | Related_URL													|
		 | Loop thru cross-references and sources looking for any		|
		 | Online_Linkage.  Output one Related_URL for each such		|
		 | element that has an Online_Linkage							|
		\*--------------------------------------------------------------*/

		if (p = find_key (first,WIdentification_Information))
			if (p = find_key (p,WCross_Reference))
				do {
					if (q = find_key (p,WOnline_Linkage))
						if (q = q->child)
							if (q->d && *q->d) {
								fprintf (out,"Group: %s\n",dif_text_of(DRelated_URL));
								fprintf (out,"  %s: ",dif_text_of(DURL_Type));
								fprintf (out,"%s\n","");
								fprintf (out,"  %s: ",dif_text_of(DURL));
								fprintf (out,"%s\n",q->d);
								fprintf (out,"  Group: %s\n",dif_text_of(DDescription));
								r = find_key (p,WCitation_Information);
								for (r=r->child; r; r=r->next) write_text_item (out,r,2);
								fprintf (out,"  End_Group\n");
								fprintf (out,"End_Group\n");
								}
					} while (p = find_next_key (p,WCross_Reference));

		if (p = find_key (first,WLineage))
			if (p = find_key (p,WSource_Information))
				do {
					if (q = find_key (p,WOnline_Linkage))
						if (q = q->child)
							if (q->d && *q->d) {
								fprintf (out,"Group: %s\n",dif_text_of(DRelated_URL));
								fprintf (out,"  %s: ",dif_text_of(DURL_Type));
								fprintf (out,"%s\n","");
								fprintf (out,"  %s: ",dif_text_of(DURL));
								fprintf (out,"%s\n",q->d);
								fprintf (out,"  Group: %s\n",dif_text_of(DDescription));
								r = find_key (p,WCitation_Information);
								for (r=r->child; r; r=r->next) write_text_item (out,r,2);
								fprintf (out,"  End_Group\n");
								fprintf (out,"End_Group\n");
								}
					} while (p = find_next_key (p,WSource_Information));

		/*--------------------------------------------------------------*\
		 | DIF_Author
		\*--------------------------------------------------------------*/

		if (p = find_key (first,WMetadata_Reference_Information))
			if (p = find_key (p,WMetadata_Contact))
				if (p = p->child)
					write_contact_info (out,p,DDIF_Author);

		/*--------------------------------------------------------------*\
		 | IDN_Node skipped
		\*--------------------------------------------------------------*/

		/*--------------------------------------------------------------*\
		 | DIF_Revision_Date
		\*--------------------------------------------------------------*/

		if (p = find_key (first,WMetadata_Reference_Information))
			if (p = find_key (p,WMetadata_Date))
				if (p = p->child) {
					fprintf (out,"%s: ",dif_text_of(DDIF_Revision_Date));
					fprintf (out,"%s",p->d);
					fprintf (out,"\n");
					}

		/*--------------------------------------------------------------*\
		 | Future_Review_Date
		\*--------------------------------------------------------------*/

		if (p = find_key (first,WMetadata_Reference_Information))
			if (p = find_key (p,WMetadata_Future_Review_Date))
				if (p = p->child) {
					fprintf (out,"%s: ",dif_text_of(DFuture_Review_Date));
					fprintf (out,"%s",p->d);
					fprintf (out,"\n");
					}

		/*--------------------------------------------------------------*\
		 | Science_Review_Date
		\*--------------------------------------------------------------*/

		fprintf (out,"%s: ",dif_text_of(DScience_Review_Date));
		fprintf (out,"\n");

		/*--------------------------------------------------------------*\
		\*--------------------------------------------------------------*/

		fclose (out);
		}
	else {
		fprintf (stderr,"Error: could not create output file %s\n",output_file);
		return;
		}
	}

/*----------------------------------------------------------------------*\
\*----------------------------------------------------------------------*/
