/*----------------------------------------------------------------------*\
 | CGI program to convert table to Enumerated_Domain elements			|
 |																		|
 | Peter N. Schweitzer (U.S. Geological Survey, Reston, VA 20192)		|
\*----------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <time.h>

#include "formio.h"
#include "buffer.h"

/*----------------------------------------------------------------------*\
\*----------------------------------------------------------------------*/

static char header[] =
"<table border=\"0\" bgcolor=\"#99CCFF\" width=\"100%\" cellpadding=\"4\"><tr><td>"
"<A HREF=\"http://www.usgs.gov/\"><img src=\"/icons/000000/usgs-mini.gif\" width=\"80\" height=\"21\" border=\"0\" alt=\"U.S. Geological Survey\"></A>"
"</td></tr></table>\n"
"<hr><p>";

static char *maintainer_line = "Maintained by <A HREF=\"/peter/\">Peter Schweitzer</A><br>";

static char *empty = "(empty)";

static char *default_delimiter = "tab";
static char *delimiter = NULL;
static char *other = NULL;
static char *stuff = NULL;

/*----------------------------------------------------------------------*\
\*----------------------------------------------------------------------*/

#ifndef HAVE_STRICMP

int stricmp (char *s, char *t) {
	int d = 0;
	do {
		d = toupper(*s) - toupper(*t);
		} while (*s++ && *t++ && !d);
	return (d);
	}

#endif

/*----------------------------------------------------------------------*\
\*----------------------------------------------------------------------*/

void decode_stuff (char *string) {
	if (string && *string) stuff = string;
	}

void decode_other (char *string) {
	if (string && *string) other = string;
	}

void decode_delimiter (char *string) {
	if (string && *string) delimiter = string;
	}

/*----------------------------------------------------------------------*/

struct keyword_table keyword[] = {
	{"stuff",		decode_stuff},
	{"other",		decode_other},
	{"delimiter",	decode_delimiter},
	{NULL,			NULL}
	};
static int keyword_count = 0;

/*----------------------------------------------------------------------*\
\*----------------------------------------------------------------------*/

main (int argc, char *argv[]) {
	char *b,*e,*s;
	char *value,*definition,*source;
	char d;

	get_form_input();

	printf ("Content-type: text/html\n");
	printf ("\n");

	printf ("<html>\n");
	printf ("<head>\n");
	printf ("<title>");
	printf ("Enumerated_Domain helper results");
	printf ("</title>\n");
	printf ("</head>\n");
	printf ("<body bgcolor=\"#FFFFFF\">\n");
	printf ("%s\n",header);

	printf ("<h3>Enumerated_Domain helper results</h3>\n");

	/* Clarify delimiter */

	if (!delimiter) {
		delimiter = default_delimiter;
		printf ("Delimiter assumed to be %s.<p>\n",default_delimiter);
		}

	if (stricmp (delimiter,"tab") == 0) d = '\t';
	else
		if (stricmp (delimiter,"space") == 0) d = ' ';
		else
			if (stricmp (delimiter,"colon") == 0) d = ':';
			else
				if (stricmp (delimiter,"equal") == 0) d = '=';
				else
					if (stricmp (delimiter,"hyphen") == 0) d = '-';
					else
						if (stricmp (delimiter,"other") == 0) d = *other;
						else
							d = '\t';

	/* Parse stuff */

	if (stuff) {
		printf ("Edit the text below to fix any mistakes, then select the text and paste into your metadata editor.<p>\n");
		printf ("<form method=\"POST\" action=\"/cgi-bin/dm/echo\">\n");
		printf ("<textarea name=\"results\" rows=\"30\" cols=\"80\" wrap=\"off\">\n");

		b = stuff;
		while (*b) {

			value = definition = source = NULL;

			/* Skip leading whitespace */

			while (*b && isspace(*b)) b++;

			/* Scan forward to the delimiter, EOL, or NUL */

			e = b;
			while (*e && *e != d && *e != '\n') e++;

			/* If you found the delimiter, look for definition */

			if (*e == d) {
				*e++ = 0;
				value = b;
				b = e;
				while (*b && isspace(*b)) b++;

				/* Scan forward to the delimiter, EOL, or NUL */

				e = b;
				while (*e && *e != d && *e != '\n') e++;
				if (*e == '\n') *e++ = 0;
				definition = b;

				/* If not EOL or NUL, a source is given on this line */

				if (*e == d) {
					*e++ = 0;
					b = e;
					while (*b && isspace(*b)) b++;
					e = b;
					while (*e && *e != '\n') e++;
					if (*e == '\n') *e++ = 0;
					source = b;
					}

				b = e;
				}
			else
				if (*e == '\n') {	/* EOL; no definition given */
					*e++ = 0;
					value = b;
					}
				else
					if (*b) value = b;	/* NUL; last value only */

			if (value) {
				printf ("Enumerated_Domain:\n");
				printf ("  Enumerated_Domain_Value: %s\n",entity_substitution (value));
				if (definition && *definition)
					printf ("  Enumerated_Domain_Value_Definition: %s\n",entity_substitution(definition));
				if (source && *source)
					printf ("  Enumerated_Domain_Value_Definition_Source: %s\n",entity_substitution(source));
				}
			value = definition = NULL;
			b = e;
			}

		printf ("</textarea>");
		printf ("</form>\n");
		}

	printf ("<hr>\n");
	printf ("This page is the result of a web form entry.<br>\n");
	printf ("%s\n",maintainer_line);
	printf ("</body>\n");
	printf ("</html>\n");
	}

/*----------------------------------------------------------------------*\
\*----------------------------------------------------------------------*/
