/*----------------------------------------------------------------------*\
 | Standalone program to convert table to Enumerated_Domain elements.	|
 |																		|
 | Peter N. Schweitzer (U.S. Geological Survey, Reston, VA 20192)		|
\*----------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "stricmp.h"

/*----------------------------------------------------------------------*\
\*----------------------------------------------------------------------*/

char option_explanation[] =
"  Options: -d specifies the character separating a value\n"
"    from its meaning.  Tab is the default, so if your\n"
"    values are separated from the meanings by a colon,\n"
"    for example, use \"-d:\" (don't include quotes).\n";

main (int argc, char *argv[]) {
	int i;
	char *default_delimiter = "tab";
	char *delimiter = NULL;
	char *other = NULL;
	char *input_file = NULL;
	FILE *in;
	long size,n;
	char *stuff,*end;
	char *b,*e,*s;
	char *value,*definition,*source;
	char d;

	if (argc <= 1) {
		fprintf (stderr,"Usage: %s [-d delimiter] input_file\n",argv[0]);
		fprintf (stderr,"%s",option_explanation);
		exit (0);
		}
	else
		for (i=1; i < argc; i++)
			if (memcmp (argv[i],"-d",2) == 0) {
				if (i+1 < argc) {
					i++;
					delimiter = argv[i];
					}
				else {
					fprintf (stderr,"Usage: %s [-d delimiter] input_file\n",argv[0]);
					fprintf (stderr,"%s",option_explanation);
					exit (0);
					}
				}
			else
				input_file = argv[i];

	if (!input_file) {
		fprintf (stderr,"Usage: %s [-d delimiter] input_file\n",argv[0]);
		fprintf (stderr,"%s",option_explanation);
		exit (0);
		}

	/* Clarify delimiter */

	if (!delimiter)
		delimiter = default_delimiter;

	if (stricmp (delimiter,"tab") == 0) d = '\t';
	else
		if (stricmp (delimiter,"space") == 0) d = ' ';
		else
			if (stricmp (delimiter,"colon") == 0) d = ':';
			else
				if (stricmp (delimiter,"equal") == 0) d = '=';
				else
					if (stricmp (delimiter,"hyphen") == 0) d = '-';
					else
						if (stricmp (delimiter,"other") == 0) d = *other;
						else
							d = '\t';

	/* Get stuff */

	if (in = fopen (input_file,"rb")) {
		if (fseek (in,0L,SEEK_END) == 0) {
			size = 1L + ftell (in);
			if (size > 1L) {
				if (stuff = (char *) malloc (size)) {
					rewind (in);
					n = fread (stuff,1,size,in);
					end = stuff + n;
					*end = 0;
					}
				}
			}
		fclose (in);
		}

	/* Parse stuff */

	if (stuff) {
		b = stuff;
		while (*b) {

			value = definition = source = NULL;

			/* Skip leading whitespace */

			while (*b && isspace(*b)) b++;

			/* Scan forward to the delimiter, EOL, or NUL */

			e = b;
			while (*e && *e != d && *e != '\n') e++;

			/* If you found the delimiter, look for definition */

			if (*e == d) {
				*e++ = 0;
				value = b;
				b = e;
				while (*b && isspace(*b)) b++;

				/* Scan forward to the delimiter, EOL, or NUL */

				e = b;
				while (*e && *e != d && *e != '\n') e++;
				if (*e == '\n') *e++ = 0;
				definition = b;

				/* If not EOL or NUL, a source is given on this line */

				if (*e == d) {
					*e++ = 0;
					b = e;
					while (*b && isspace(*b)) b++;
					e = b;
					while (*e && *e != '\n') e++;
					if (*e == '\n') *e++ = 0;
					source = b;
					}

				b = e;
				}
			else
				if (*e == '\n') {	/* EOL; no definition given */
					*e++ = 0;
					value = b;
					}
				else
					if (*b) value = b;	/* NUL; last value only */

			if (value) {
				printf ("Enumerated_Domain:\n");
				printf ("  Enumerated_Domain_Value: %s\n",value);
				if (definition && *definition)
					printf ("  Enumerated_Domain_Value_Definition: %s\n",definition);
				if (source && *source)
					printf ("  Enumerated_Domain_Value_Definition_Source: %s\n",source);
				}
			value = definition = NULL;
			b = e;
			}
		}
	}

/*----------------------------------------------------------------------*\
\*----------------------------------------------------------------------*/
