/*----------------------------------------------------------------------*\
 | Metadata element help												|
 |																		|
 | Peter N. Schweitzer (U.S. Geological Survey, Reston, VA 20192)		|
\*----------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "keyword.h"

extern char *read_text (char *name);	/* in text.c */
extern int unicode_of_utf8 (unsigned char **s);	/* in encoding.c */

struct help {
	enum fgdc_keyword key;
	char *text;
	};

static struct help help [2048];
static int help_count = 0;

static void copy_name (unsigned char *dst, unsigned char *src) {
	unsigned char *d = dst;
	unsigned char *s = src;
	int c;

	while (*s && !isspace(*s))
		if ((c = unicode_of_utf8 (&s)) < 256) *d++ = (unsigned char) c;
	*d = 0;
	}

void decode_help (char *dst) {
	char *end;
	char *b,*e,*s;
	int line_count = 0;
	char name [128];
	enum fgdc_keyword key;

	if (dst && *dst) {
		b = e = dst;
		end = dst + strlen (dst);
		*end = 0;

		for (s=dst; s < end; s++) if (*s == '\r') *s = ' ';

		b = dst;
		if (!isspace(*b)) {
			copy_name (name,b);
			key = key_of (name);
			if (key != Wunknown && key != Wblank) {
				help[help_count].key = key;
				help[help_count].text = b;
				help_count++;
				}
			}

		for (e=dst; e < end; e++)
			if (*e == '\n') {
				b = e+1;
				if (!isspace(*b)) {
					copy_name (name,b);
					key = key_of (name);
					if (key != Wunknown && key != Wblank) {
						if (b > dst) *(b-1) = 0;
						help[help_count].key = key;
						help[help_count].text = b;
						help_count++;
						}
					}
				line_count++;
				}
		}
	}

void read_help (char *help_file) {
	decode_help (read_text (help_file));
	}

char *advice_about (enum fgdc_keyword key) {
	int i;

	if (help_count)
		for (i=0; i < help_count; i++)
			if (help[i].key == key) return (help[i].text);
	return (NULL);
	}

/*----------------------------------------------------------------------*\
\*----------------------------------------------------------------------*/
