/*----------------------------------------------------------------------*\
 | Structure definition for a metadata element.							|
 |																		|
 | Peter N. Schweitzer (U.S. Geological Survey, Reston, VA 20192)		|
\*----------------------------------------------------------------------*/

#ifndef ITEM_H
#define ITEM_H	1

#include "keyword.h"

struct item {
	struct item *next;
	struct item *prev;
	struct item *parent;
	struct item *child;
	int indent;			/* number of leading whitespaces before text */
	int line_number;	/* line in source file */
	enum fgdc_keyword key;
	int original;		/* zero if member d can be released thru free() */
	int len;			/* length of bloc pointed to by d */
	char *d;			/* text of item */
	};

extern void check_topology (struct item *p);
extern struct item *allocate_item (int fail);
extern void deallocate_item (struct item *p);
extern struct item *insert_item_after (struct item *r);
extern struct item *insert_item_before (struct item *q);
extern struct item *add_child (struct item *p);
extern struct item *copy_item (struct item *p);
extern void detach_item (struct item *p);
extern void delete_item (struct item *p);
extern void delete_children (struct item *p);
extern void clear_scalars (struct item *p);
extern void indent (struct item *p);
extern struct item *find_key (struct item *p, enum fgdc_keyword key);
extern struct item *find_next_key (struct item *p, enum fgdc_keyword key);
extern int prune (struct item *p);
extern int item_count (struct item *p);
extern void list_children (FILE *tf, struct item *p);
extern int valid_node (struct item *node, struct item *root);

extern int wrap_text (struct item *p, int width);
extern int wrap_subtree (struct item *p, int width);

#endif /* ITEM_H */

/*----------------------------------------------------------------------*\
\*----------------------------------------------------------------------*/
