/*----------------------------------------------------------------------*\
 | For each standard element, return a relative hypertext link to the	|
 | point in the Standard where that element is described.				|
 |																		|
 | Peter N. Schweitzer (U.S. Geological Survey, Reston, VA 20192)		|
\*----------------------------------------------------------------------*/

#include <stdio.h>

#include "keyword.h"

struct link_t {
	enum fgdc_keyword key;
	char *link;
	};

static struct link_t link_table [] = {
	{WMetadata,
		"00.html#Metadata"},
	{WIdentification_Information,
		"01.html#Identification%20Information"},
	{WCitation,
		"01.html#Citation"},
	{WDescription,
		"01.html#Description"},
	{WAbstract,
		"01.html#Abstract"},
	{WPurpose,
		"01.html#Purpose"},
	{WSupplemental_Information,
		"01.html#Supplemental%20Information"},
	{WTime_Period_of_Content,
		"01.html#Time%20Period%20of%20Content"},
	{WCurrentness_Reference,
		"01.html#Currentness%20Reference"},
	{WStatus,
		"01.html#Status"},
	{WProgress,
		"01.html#Progress"},
	{WMaintenance_and_Update_Frequency,
		"01.html#Maintenance%20and%20Update%20Frequency"},
	{WSpatial_Domain,
		"01.html#Spatial%20Domain"},
	{WBounding_Coordinates,
		"01.html#Bounding%20Coordinates"},
	{WWest_Bounding_Coordinate,
		"01.html#West%20Bounding%20Coordinate"},
	{WEast_Bounding_Coordinate,
		"01.html#East%20Bounding%20Coordinate"},
	{WNorth_Bounding_Coordinate,
		"01.html#North%20Bounding%20Coordinate"},
	{WSouth_Bounding_Coordinate,
		"01.html#South%20Bounding%20Coordinate"},
	{WData_Set_G_Polygon,
		"01.html#Data%20Set%20G%2DPolygon"},
	{WData_Set_G_Polygon_Outer_G_Ring,
		"01.html#Data%20Set%20G%2DPolygon%20Outer%20G%2DRing"},
	{WG_Ring,
		"01.html#G%2DRing"},
	{WG_Ring_Point,
		"01.html#G%2DRing_Point"},
	{WG_Ring_Latitude,
		"01.html#G%2DRing%20Latitude"},
	{WG_Ring_Longitude,
		"01.html#G%2DRing%20Longitude"},
	{WData_Set_G_Polygon_Exclusion_G_Ring,
		"01.html#Data%20Set%20G%2DPolygon%20Exclusion%20G%2DRing"},
	{WKeywords,
		"01.html#Keywords"},
	{WTheme,
		"01.html#Theme"},
	{WTheme_Keyword_Thesaurus,
		"01.html#Theme%20Keyword%20Thesaurus"},
	{WTheme_Keyword,
		"01.html#Theme%20Keyword"},
	{WPlace,
		"01.html#Place"},
	{WPlace_Keyword_Thesaurus,
		"01.html#Place%20Keyword%20Thesaurus"},
	{WPlace_Keyword,
		"01.html#Place%20Keyword"},
	{WStratum,
		"01.html#Stratum"},
	{WStratum_Keyword_Thesaurus,
		"01.html#Stratum%20Keyword%20Thesaurus"},
	{WStratum_Keyword,
		"01.html#Stratum%20Keyword"},
	{WTemporal,
		"01.html#Temporal"},
	{WTemporal_Keyword_Thesaurus,
		"01.html#Temporal%20Keyword%20Thesaurus"},
	{WTemporal_Keyword,
		"01.html#Temporal%20Keyword"},
	{WAccess_Constraints,
		"01.html#Access%20Constraints"},
	{WUse_Constraints,
		"01.html#Use%20Constraints"},
	{WPoint_of_Contact,
		"01.html#Point%20of%20Contact"},
	{WBrowse_Graphic,
		"01.html#Browse%20Graphic"},
	{WBrowse_Graphic_File_Name,
		"01.html#Browse%20Graphic%20File%20Name"},
	{WBrowse_Graphic_File_Description,
		"01.html#Browse%20Graphic%20File%20Description"},
	{WBrowse_Graphic_File_Type,
		"01.html#Browse%20Graphic%20File%20Type"},
	{WData_Set_Credit,
		"01.html#Data%20Set%20Credit"},
	{WSecurity_Information,
		"01.html#Security%20Information"},
	{WSecurity_Classification_System,
		"01.html#Security%20Classification%20System"},
	{WSecurity_Classification,
		"01.html#Security%20Classification"},
	{WSecurity_Handling_Description,
		"01.html#Security%20Handling%20Description"},
	{WNative_Data_Set_Environment,
		"01.html#Native%20Data%20Set%20Environment"},
	{WCross_Reference,
		"01.html#Cross%20Reference"},
	{WData_Quality_Information,
		"02.html#Data%20Quality%20Information"},
	{WAttribute_Accuracy,
		"02.html#Attribute%20Accuracy"},
	{WAttribute_Accuracy_Report,
		"02.html#Attribute%20Accuracy%20Report"},
	{WQuantitative_Attribute_Accuracy_Assessment,
		"02.html#Quantitative%20Attribute%20Accuracy%20Assessment"},
	{WAttribute_Accuracy_Value,
		"02.html#Attribute%20Accuracy%20Value"},
	{WAttribute_Accuracy_Explanation,
		"02.html#Attribute%20Accuracy%20Explanation"},
	{WLogical_Consistency_Report,
		"02.html#Logical%20Consistency%20Report"},
	{WCompleteness_Report,
		"02.html#Completeness%20Report"},
	{WPositional_Accuracy,
		"02.html#Positional%20Accuracy"},
	{WHorizontal_Positional_Accuracy,
		"02.html#Horizontal%20Positional%20Accuracy"},
	{WHorizontal_Positional_Accuracy_Report,
		"02.html#Horizontal%20Positional%20Accuracy%20Report"},
	{WQuantitative_Horizontal_Positional_Accuracy_Assessment,
		"02.html#Quantitative%20Horizontal%20Positional%20Accuracy%20Assessment"},
	{WHorizontal_Positional_Accuracy_Value,
		"02.html#Horizontal%20Positional%20Accuracy%20Value"},
	{WHorizontal_Positional_Accuracy_Explanation,
		"02.html#Horizontal%20Positional%20Accuracy%20Explanation"},
	{WVertical_Positional_Accuracy,
		"02.html#Vertical%20Positional%20Accuracy"},
	{WVertical_Positional_Accuracy_Report,
		"02.html#Vertical%20Positional%20Accuracy%20Report"},
	{WQuantitative_Vertical_Positional_Accuracy_Assessment,
		"02.html#Quantitative%20Vertical%20Positional%20Accuracy%20Assessment"},
	{WVertical_Positional_Accuracy_Value,
		"02.html#Vertical%20Positional%20Accuracy%20Value"},
	{WVertical_Positional_Accuracy_Explanation,
		"02.html#Vertical%20Positional%20Accuracy%20Explanation"},
	{WLineage,
		"02.html#Lineage"},
	{WSource_Information,
		"02.html#Source%20Information"},
	{WSource_Citation,
		"02.html#Source%20Citation"},
	{WSource_Scale_Denominator,
		"02.html#Source%20Scale%20Denominator"},
	{WType_of_Source_Media,
		"02.html#Type%20of%20Source%20Media"},
	{WSource_Time_Period_of_Content,
		"02.html#Source%20Time%20Period%20of%20Content"},
	{WSource_Currentness_Reference,
		"02.html#Source%20Currentness%20Reference"},
	{WSource_Citation_Abbreviation,
		"02.html#Source%20Citation%20Abbreviation"},
	{WSource_Contribution,
		"02.html#Source%20Contribution"},
	{WProcess_Step,
		"02.html#Process%20Step"},
	{WProcess_Description,
		"02.html#Process%20Description"},
	{WSource_Used_Citation_Abbreviation,
		"02.html#Source%20Used%20Citation%20Abbreviation"},
	{WProcess_Date,
		"02.html#Process%20Date"},
	{WProcess_Time,
		"02.html#Process%20Time"},
	{WSource_Produced_Citation_Abbreviation,
		"02.html#Source%20Produced%20Citation%20Abbreviation"},
	{WProcess_Contact,
		"02.html#Process%20Contact"},
	{WCloud_Cover,
		"02.html#Cloud%20Cover"},
	{WSpatial_Data_Organization_Information,
		"03.html#Spatial%20Data%20Organization%20Information"},
	{WIndirect_Spatial_Reference,
		"03.html#Indirect%20Spatial%20Reference"},
	{WDirect_Spatial_Reference_Method,
		"03.html#Direct%20Spatial%20Reference%20Method"},
	{WPoint_and_Vector_Object_Information,
		"03.html#Point%20and%20Vector%20Object%20Information"},
	{WSDTS_Terms_Description,
		"03.html#SDTS%20Terms%20Description"},
	{WSDTS_Point_and_Vector_Object_Type,
		"03.html#SDTS%20Point%20and%20Vector%20Object%20Type"},
	{WPoint_and_Vector_Object_Count,
		"03.html#Point%20and%20Vector%20Object%20Count"},
	{WVPF_Point_and_Vector_Object_Information,
		"03.html#VPF%20Point%20and%20Vector%20Object%20Information"},
	{WVPF_Terms_Description,
		"03.html#VPF%20Terms%20Description"},
	{WVPF_Topology_Level,
		"03.html#VPF%20Topology%20Level"},
	{WVPF_Point_and_Vector_Object_Type,
		"03.html#VPF%20Point%20and%20Vector%20Object%20Type"},
	{WRaster_Object_Information,
		"03.html#Raster%20Object%20Information"},
	{WRaster_Object_Type,
		"03.html#Raster%20Object%20Type"},
	{WRow_Count,
		"03.html#Row%20Count"},
	{WColumn_Count,
		"03.html#Column%20Count"},
	{WVertical_Count,
		"03.html#Vertical%20Count"},
	{WSpatial_Reference_Information,
		"04.html#Spatial%20Reference%20Information"},
	{WHorizontal_Coordinate_System_Definition,
		"04.html#Horizontal%20Coordinate%20System%20Definition"},
	{WGeographic,
		"04.html#Geographic"},
	{WLatitude_Resolution,
		"04.html#Latitude%20Resolution"},
	{WLongitude_Resolution,
		"04.html#Longitude%20Resolution"},
	{WGeographic_Coordinate_Units,
		"04.html#Geographic%20Coordinate%20Units"},
	{WPlanar,
		"04.html#Planar"},
	{WMap_Projection,
		"04.html#Map%20Projection"},
	{WMap_Projection_Name,
		"04.html#Map%20Projection%20Name"},
	{WMap_Projection_Parameters,
		"04.html#Map%20Projection%20Parameters"},
	{WAlbers_Conical_Equal_Area,
		"04.html#Albers%20Conical%20Equal%20Area"},
	{WAzimuthal_Equidistant,
		"04.html#Azimuthal%20Equidistant"},
	{WEquidistant_Conic,
		"04.html#Equidistant%20Conic"},
	{WEquirectangular,
		"04.html#Equirectangular"},
	{WGeneral_Vertical_Near_sided_Perspective,
		"04.html#General%20Vertical%20Near%2Dsided%20Perspective"},
	{WGnomonic,
		"04.html#Gnomonic"},
	{WLambert_Azimuthal_Equal_Area,
		"04.html#Lambert%20Azimuthal%20Equal%20Area"},
	{WLambert_Conformal_Conic,
		"04.html#Lambert%20Conformal%20Conic"},
	{WMercator,
		"04.html#Mercator"},
	{WModified_Stereographic_for_Alaska,
		"04.html#Modified%20Stereographic%20for%20Alaska"},
	{WMiller_Cylindrical,
		"04.html#Miller%20Cylindrical"},
	{WOblique_Mercator,
		"04.html#Oblique%20Mercator"},
	{WOrthographic,
		"04.html#Orthographic"},
	{WPolar_Stereographic,
		"04.html#Polar%20Stereographic"},
	{WPolyconic,
		"04.html#Polyconic"},
	{WRobinson,
		"04.html#Robinson"},
	{WSinusoidal,
		"04.html#Sinusoidal"},
	{WSpace_Oblique_Mercator_Landsat,
		"04.html#Space%20Oblique%20Mercator%20(Landsat)"},
	{WStereographic,
		"04.html#Stereographic"},
	{WTransverse_Mercator,
		"04.html#Transverse%20Mercator"},
	{Wvan_der_Grinten,
		"04.html#van%20der%20Grinten"},
	{WStandard_Parallel,
		"04.html#Standard%20Parallel"},
	{WLongitude_of_Central_Meridian,
		"04.html#Longitude%20of%20Central%20Meridian"},
	{WLatitude_of_Projection_Origin,
		"04.html#Latitude%20of%20Projection%20Origin"},
	{WFalse_Easting,
		"04.html#False%20Easting"},
	{WFalse_Northing,
		"04.html#False%20Northing"},
	{WScale_Factor_at_Equator,
		"04.html#Scale%20Factor%20at%20Equator"},
	{WHeight_of_Perspective_Point_Above_Surface,
		"04.html#Height%20of%20Perspective%20Point%20Above%20Surface"},
	{WLongitude_of_Projection_Center,
		"04.html#Longitude%20of%20Projection%20Center"},
	{WLatitude_of_Projection_Center,
		"04.html#Latitude%20of%20Projection%20Center"},
	{WScale_Factor_at_Center_Line,
		"04.html#Scale%20Factor%20at%20Center%20Line"},
	{WOblique_Line_Azimuth,
		"04.html#Oblique%20Line%20Azimuth"},
	{WAzimuthal_Angle,
		"04.html#Azimuthal%20Angle"},
	{WAzimuth_Measure_Point_Longitude,
		"04.html#Azimuth%20Measure%20Point%20Longitude"},
	{WOblique_Line_Point,
		"04.html#Oblique%20Line%20Point"},
	{WOblique_Line_Latitude,
		"04.html#Oblique%20Line%20Latitude"},
	{WOblique_Line_Longitude,
		"04.html#Oblique%20Line%20Longitude"},
	{WStraight_Vertical_Longitude_from_Pole,
		"04.html#Straight%2DVertical%20Longitude%20from%20Pole"},
	{WScale_Factor_at_Projection_Origin,
		"04.html#Scale%20Factor%20at%20Projection%20Origin"},
	{WLandsat_Number,
		"04.html#Landsat%20Number"},
	{WPath_Number,
		"04.html#Path%20Number"},
	{WScale_Factor_at_Central_Meridian,
		"04.html#Scale%20Factor%20at%20Central%20Meridian"},
	{WOther_Projections_Definition,
		"04.html#Other%20Projection%27s%20Definition"},
	{WGrid_Coordinate_System,
		"04.html#Grid%20Coordinate%20System"},
	{WGrid_Coordinate_System_Name,
		"04.html#Grid%20Coordinate%20System%20Name"},
	{WUniversal_Transverse_Mercator,
		"04.html#Universal%20Transverse%20Mercator"},
	{WUTM_Zone_Number,
		"04.html#UTM%20Zone%20Number"},
	{WUniversal_Polar_Stereographic,
		"04.html#Universal%20Polar%20Stereographic"},
	{WUPS_Zone_Identifier,
		"04.html#UPS%20Zone%20Identifier"},
	{WState_Plane_Coordinate_System,
		"04.html#State%20Plane%20Coordinate%20System"},
	{WSPCS_Zone_Identifier,
		"04.html#SPCS%20Zone%20Identifier"},
	{WARC_Coordinate_System,
		"04.html#ARC%20Coordinate%20System"},
	{WARC_System_Zone_Identifier,
		"04.html#ARC%20System%20Zone%20Identifier"},
	{WOther_Grid_Systems_Definition,
		"04.html#Other%20Grid%20System%27s%20Definition"},
	{WLocal_Planar,
		"04.html#Local%20Planar"},
	{WLocal_Planar_Description,
		"04.html#Local%20Planar%20Description"},
	{WLocal_Planar_Georeference_Information,
		"04.html#Local%20Planar%20Georeference%20Information"},
	{WPlanar_Coordinate_Information,
		"04.html#Planar%20Coordinate%20Information"},
	{WPlanar_Coordinate_Encoding_Method,
		"04.html#Planar%20Coordinate%20Encoding%20Method"},
	{WCoordinate_Representation,
		"04.html#Coordinate%20Representation"},
	{WAbscissa_Resolution,
		"04.html#Abscissa%20Resolution"},
	{WOrdinate_Resolution,
		"04.html#Ordinate%20Resolution"},
	{WDistance_and_Bearing_Representation,
		"04.html#Distance%20and%20Bearing%20Representation"},
	{WDistance_Resolution,
		"04.html#Distance%20Resolution"},
	{WBearing_Resolution,
		"04.html#Bearing%20Resolution"},
	{WBearing_Units,
		"04.html#Bearing%20Units"},
	{WBearing_Reference_Direction,
		"04.html#Bearing%20Reference%20Direction"},
	{WBearing_Reference_Meridian,
		"04.html#Bearing%20Reference%20Meridian"},
	{WPlanar_Distance_Units,
		"04.html#Planar%20Distance%20Units"},
	{WLocal,
		"04.html#Local"},
	{WLocal_Description,
		"04.html#Local%20Description"},
	{WLocal_Georeference_Information,
		"04.html#Local%20Georeference%20Information"},
	{WGeodetic_Model,
		"04.html#Geodetic%20Model"},
	{WHorizontal_Datum_Name,
		"04.html#Horizontal%20Datum%20Name"},
	{WEllipsoid_Name,
		"04.html#Ellipsoid%20Name"},
	{WSemi_major_Axis,
		"04.html#Semi%2Dmajor%20Axis"},
	{WDenominator_of_Flattening_Ratio,
		"04.html#Denominator%20of%20Flattening%20Ratio"},
	{WVertical_Coordinate_System_Definition,
		"04.html#Vertical%20Coordinate%20System%20Definition"},
	{WAltitude_System_Definition,
		"04.html#Altitude%20System%20Definition"},
	{WAltitude_Datum_Name,
		"04.html#Altitude%20Datum%20Name"},
	{WAltitude_Resolution,
		"04.html#Altitude%20Resolution"},
	{WAltitude_Distance_Units,
		"04.html#Altitude%20Distance%20Units"},
	{WAltitude_Encoding_Method,
		"04.html#Altitude%20Encoding%20Method"},
	{WDepth_System_Definition,
		"04.html#Depth%20System%20Definition"},
	{WDepth_Datum_Name,
		"04.html#Depth%20Datum%20Name"},
	{WDepth_Resolution,
		"04.html#Depth%20Resolution"},
	{WDepth_Distance_Units,
		"04.html#Depth%20Distance%20Units"},
	{WDepth_Encoding_Method,
		"04.html#Depth%20Encoding%20Method"},
	{WEntity_and_Attribute_Information,
		"05.html#Entity%20and%20Attribute%20Information"},
	{WDetailed_Description,
		"05.html#Detailed%20Description"},
	{WEntity_Type,
		"05.html#Entity%20Type"},
	{WEntity_Type_Label,
		"05.html#Entity%20Type%20Label"},
	{WEntity_Type_Definition,
		"05.html#Entity%20Type%20Definition"},
	{WEntity_Type_Definition_Source,
		"05.html#Entity%20Type%20Definition%20Source"},
	{WAttribute,
		"05.html#Attribute"},
	{WAttribute_Label,
		"05.html#Attribute%20Label"},
	{WAttribute_Definition,
		"05.html#Attribute%20Definition"},
	{WAttribute_Definition_Source,
		"05.html#Attribute%20Definition%20Source"},
	{WAttribute_Domain_Values,
		"05.html#Attribute%20Domain%20Values"},
	{WEnumerated_Domain,
		"05.html#Enumerated%20Domain"},
	{WEnumerated_Domain_Value,
		"05.html#Enumerated%20Domain%20Value"},
	{WEnumerated_Domain_Value_Definition,
		"05.html#Enumerated%20Domain%20Value%20Definition"},
	{WEnumerated_Domain_Value_Definition_Source,
		"05.html#Enumerated%20Domain%20Value%20Definition%20Source"},
	{WRange_Domain,
		"05.html#Range%20Domain"},
	{WRange_Domain_Minimum,
		"05.html#Range%20Domain%20Minimum"},
	{WRange_Domain_Maximum,
		"05.html#Range%20Domain%20Maximum"},
	{WCodeset_Domain,
		"05.html#Codeset%20Domain"},
	{WCodeset_Name,
		"05.html#Codeset%20Name"},
	{WCodeset_Source,
		"05.html#Codeset%20Source"},
	{WUnrepresentable_Domain,
		"05.html#Unrepresentable%20Domain"},
	{WAttribute_Units_of_Measure,
		"05.html#Attribute%20Units%20of%20Measure"},
	{WAttribute_Measurement_Resolution,
		"05.html#Attribute%20Measurement%20Resolution"},
	{WBeginning_Date_of_Attribute_Values,
		"05.html#Beginning%20Date%20of%20Attribute%20Values"},
	{WEnding_Date_of_Attribute_Values,
		"05.html#Ending%20Date%20of%20Attribute%20Values"},
	{WAttribute_Value_Accuracy_Information,
		"05.html#Attribute%20Value%20Accuracy%20Information"},
	{WAttribute_Value_Accuracy,
		"05.html#Attribute%20Value%20Accuracy"},
	{WAttribute_Value_Accuracy_Explanation,
		"05.html#Attribute%20Value%20Accuracy%20Explanation"},
	{WAttribute_Measurement_Frequency,
		"05.html#Attribute%20Measurement%20Frequency"},
	{WOverview_Description,
		"05.html#Overview%20Description"},
	{WEntity_and_Attribute_Overview,
		"05.html#Entity%20and%20Attribute%20Overview"},
	{WEntity_and_Attribute_Detail_Citation,
		"05.html#Entity%20and%20Attribute%20Detail%20Citation"},
	{WDistribution_Information,
		"06.html#Distribution%20Information"},
	{WDistributor,
		"06.html#Distributor"},
	{WResource_Description,
		"06.html#Resource%20Description"},
	{WDistribution_Liability,
		"06.html#Distribution%20Liability"},
	{WStandard_Order_Process,
		"06.html#Standard%20Order%20Process"},
	{WNon_digital_Form,
		"06.html#Non%2Ddigital%20Form"},
	{WDigital_Form,
		"06.html#Digital%20Form"},
	{WDigital_Transfer_Information,
		"06.html#Digital%20Transfer%20Information"},
	{WFormat_Name,
		"06.html#Format%20Name"},
	{WFormat_Version_Number,
		"06.html#Format%20Version%20Number"},
	{WFormat_Version_Date,
		"06.html#Format%20Version%20Date"},
	{WFormat_Specification,
		"06.html#Format%20Specification"},
	{WFormat_Information_Content,
		"06.html#Format%20Information%20Content"},
	{WFile_Decompression_Technique,
		"06.html#File%20Decompression%20Technique"},
	{WTransfer_Size,
		"06.html#Transfer%20Size"},
	{WDigital_Transfer_Option,
		"06.html#Digital%20Transfer%20Option"},
	{WOnline_Option,
		"06.html#Online%20Option"},
	{WComputer_Contact_Information,
		"06.html#Computer%20Contact%20Information"},
	{WNetwork_Address,
		"06.html#Network%20Address"},
	{WNetwork_Resource_Name,
		"06.html#Network%20Resource%20Name"},
	{WDialup_Instructions,
		"06.html#Dialup%20Instructions"},
	{WLowest_BPS,
		"06.html#Lowest%20BPS"},
	{WHighest_BPS,
		"06.html#Highest%20BPS"},
	{WNumber_DataBits,
		"06.html#Number%20DataBits"},
	{WNumber_StopBits,
		"06.html#Number%20StopBits"},
	{WParity,
		"06.html#Parity"},
	{WCompression_Support,
		"06.html#Compression%20Support"},
	{WDialup_Telephone,
		"06.html#Dialup%20Telephone"},
	{WDialup_File_Name,
		"06.html#Dialup%20File%20Name"},
	{WAccess_Instructions,
		"06.html#Access%20Instructions"},
	{WOnline_Computer_and_Operating_System,
		"06.html#Online%20Computer%20and%20Operating%20System"},
	{WOffline_Option,
		"06.html#Offline%20Option"},
	{WOffline_Media,
		"06.html#Offline%20Media"},
	{WRecording_Capacity,
		"06.html#Recording%20Capacity"},
	{WRecording_Density,
		"06.html#Recording%20Density"},
	{WRecording_Density_Units,
		"06.html#Recording%20Density%20Units"},
	{WRecording_Format,
		"06.html#Recording%20Format"},
	{WCompatibility_Information,
		"06.html#Compatibility%20Information"},
	{WFees,
		"06.html#Fees"},
	{WOrdering_Instructions,
		"06.html#Ordering%20Instructions"},
	{WTurnaround,
		"06.html#Turnaround"},
	{WCustom_Order_Process,
		"06.html#Custom%20Order%20Process"},
	{WTechnical_Prerequisites,
		"06.html#Technical%20Prerequisites"},
	{WAvailable_Time_Period,
		"06.html#Available%20Time%20Period"},
	{WMetadata_Reference_Information,
		"07.html#Metadata%20Reference%20Information"},
	{WMetadata_Date,
		"07.html#Metadata%20Date"},
	{WMetadata_Review_Date,
		"07.html#Metadata%20Review%20Date"},
	{WMetadata_Future_Review_Date,
		"07.html#Metadata%20Future%20Review%20Date"},
	{WMetadata_Contact,
		"07.html#Metadata%20Contact"},
	{WMetadata_Standard_Name,
		"07.html#Metadata%20Standard%20Name"},
	{WMetadata_Standard_Version,
		"07.html#Metadata%20Standard%20Version"},
	{WMetadata_Time_Convention,
		"07.html#Metadata%20Time%20Convention"},
	{WMetadata_Access_Constraints,
		"07.html#Metadata%20Access%20Constraints"},
	{WMetadata_Use_Constraints,
		"07.html#Metadata%20Use%20Constraints"},
	{WMetadata_Security_Information,
		"07.html#Metadata%20Security%20Information"},
	{WMetadata_Security_Classification_System,
		"07.html#Metadata%20Security%20Classification%20System"},
	{WMetadata_Security_Classification,
		"07.html#Metadata%20Security%20Classification"},
	{WMetadata_Security_Handling_Description,
		"07.html#Metadata%20Security%20Handling%20Description"},
	{WMetadata_Extensions,
		"07.html#Metadata_Extensions"},
	{WProfile_Name,
		"07.html#Profile_Name"},
	{WCitation_Information,
		"08.html#Citation%20Information"},
	{WOriginator,
		"08.html#Originator"},
	{WPublication_Date,
		"08.html#Publication%20Date"},
	{WPublication_Time,
		"08.html#Publication%20Time"},
	{WTitle,
		"08.html#Title"},
	{WEdition,
		"08.html#Edition"},
	{WGeospatial_Data_Presentation_Form,
		"08.html#Geospatial%20Data%20Presentation%20Form"},
	{WSeries_Information,
		"08.html#Series%20Information"},
	{WSeries_Name,
		"08.html#Series%20Name"},
	{WIssue_Identification,
		"08.html#Issue%20Identification"},
	{WPublication_Information,
		"08.html#Publication%20Information"},
	{WPublication_Place,
		"08.html#Publication%20Place"},
	{WPublisher,
		"08.html#Publisher"},
	{WOther_Citation_Details,
		"08.html#Other%20Citation%20Details"},
	{WOnline_Linkage,
		"08.html#Online%20Linkage"},
	{WLarger_Work_Citation,
		"08.html#Larger%20Work%20Citation"},
	{WTime_Period_Information,
		"09.html#Time%20Period%20Information"},
	{WSingle_Date_Time,
		"09.html#Single%20Date%2FTime"},
	{WCalendar_Date,
		"09.html#Calendar%20Date"},
	{WTime_of_Day,
		"09.html#Time%20of%20Day"},
	{WMultiple_Dates_Times,
		"09.html#Multiple%20Dates%2FTimes"},
	{WRange_of_Dates_Times,
		"09.html#Range%20of%20Dates%2FTimes"},
	{WBeginning_Date,
		"09.html#Beginning%20Date"},
	{WBeginning_Time,
		"09.html#Beginning%20Time"},
	{WEnding_Date,
		"09.html#Ending%20Date"},
	{WEnding_Time,
		"09.html#Ending%20Time"},
	{WContact_Information,
		"10.html#Contact%20Information"},
	{WContact_Person_Primary,
		"10.html#Contact%20Person%20Primary"},
	{WContact_Person,
		"10.html#Contact%20Person"},
	{WContact_Organization,
		"10.html#Contact%20Organization"},
	{WContact_Organization_Primary,
		"10.html#Contact%20Organization%20Primary"},
	{WContact_Position,
		"10.html#Contact%20Position"},
	{WContact_Address,
		"10.html#Contact%20Address"},
	{WAddress_Type,
		"10.html#Address%20Type"},
	{WAddress,
		"10.html#Address"},
	{WContact_Electronic_Mail_Address,
		"10.html#Contact_Electronic_Mail_Address"},
	{WContact_Voice_Telephone,
		"10.html#Contact_Voice_Telephone"},
	{WContact_Facsimile_Telephone,
		"10.html#Contact_Facsimile_Telephone"},
	{WContact_TDD_TTY_Telephone,
		"10.html#Contact_TDD_TTY_Telephone"},
	{WContact_Instructions,
		"10.html#Contact_Instructions"},
	{WCity,
		"10.html#City"},
	{WState_or_Province,
		"10.html#State_or_Province"},
	{WPostal_Code,
		"10.html#Postal_Code"},
	{WCountry,
		"10.html#Country"},
	{WHours_of_Service,
		"10.html#Hours_of_Service"},
	{WMetadata_Language,
		"07.html#Metadata_Language"},
	{Wnull,NULL}
	};
static int link_table_len = 0;

char *link_of (enum fgdc_keyword key) {
	int i;

	if (link_table_len == 0)
		while (link_table[link_table_len].key != Wnull) link_table_len++;

	for (i=0; i < link_table_len; i++)
		if (key == link_table[i].key) return (link_table[i].link);

	return (NULL);
	}

/*----------------------------------------------------------------------*\
\*----------------------------------------------------------------------*/
