/*----------------------------------------------------------------------*\
 | Program to create appropriate code for the Version item in the Help	|
 | menus of xtme and tkme.												|
 |																		|
 | Peter N. Schweitzer (U.S. Geological Survey, Reston, VA 20192)		|
\*----------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "revision.h"

#ifdef __WIN32__

struct utsname {
	char sysname[65];
	char nodename[65];
	char release[65];
	char version[65];
	char machine[65];
	};

static int uname (struct utsname *name) {
	strcpy (name->sysname,"Microsoft Windows");
	strcpy (name->nodename,"polynices");
	sprintf (name->release,"%d.%d",_winmajor,_winminor);
	sprintf (name->version,"%d",_osver);
	strcpy (name->machine,"i586");
	return (0);
	}

#else
#include <sys/utsname.h>
#endif

#define MAX_LINE_LENGTH	4096

static FILE *in,*out;
static char line [MAX_LINE_LENGTH];
static char scratch [MAX_LINE_LENGTH];

static char *es_credit = "\
  Spanish-language element names kindly provided by\\n\
  Clearinghouse Nacional de Datos Geogrficos, Uruguay\\n\
  <http://www.clearinghouse.gub.uy/>\\n";
static char *id_credit = "\
  Indonesian-language element names kindly provided by\\n\
  the Indonesian National Coordination Agency for Surveys\\n\
  and Mapping BAKOSURTANAL\\n";
static char *fr_credit = "\
  French-language element names kindly provided by Environment\\n\
  Canada (John Cree)\\n";
static char *ca_credit = "\
  Catalan-language element names kindly provided by \\n\
  Clearinghouse Nacional de Datos Geogrficos, Uruguay\\n\
  <http://www.clearinghouse.gub.uy/>\\n";
static char *de_credit = "\
  German-language element names kindly provided by Peter Korduan\\n\
  (University of Rostock)\\n";
static char *pt_credit = "\
  Portuguese-language element names kindly provided by Luis Cavalcanti Bahiana\\n\
  (IBGE, Brazil)\\n";

static void add_file (char *name) {
	char *s,*t;
	if (in = fopen (name,"r")) {
		while (fgets (scratch,MAX_LINE_LENGTH,in)) {
			s = scratch + strlen (scratch) - 1;
			if (*s == '\n') *s-- = 0;
			if (*s == '\r') *s-- = 0;

			s = scratch;
			t = line;
			while (*s) {
				if (*s == '\"') *t++ = '\\';
				*t++ = *s++;
				}
			*t = 0;

			fprintf (out,"\"%s\\n\",\n",line);
			}
		fclose (in);
		}
	}

int main (int argc, char *argv[]) {
	int i;
	char *output_file = NULL;
	char *version_file = NULL;
	char *hints_file = NULL;
	char *s;
	time_t now;
	struct utsname u;

	for (i=1; i < argc; i++)
		if (memcmp (argv[i],"-o",2) == 0) {
			i++;
			output_file = argv[i];
			}
		else
			if (memcmp (argv[i],"-v",2) == 0) {
				i++;
				version_file = argv[i];
				}
			else
				if (memcmp (argv[i],"-h",2) == 0) {
					i++;
					hints_file = argv[i];
					}

	if (output_file)
		if (out = fopen (output_file,"w")) {
			fprintf (out,"/*----------------------------------------------------------------------*\\\n");
			fprintf (out," | This file is generated automatically while building %s.\t\t\t|\n",version_file);
			fprintf (out," | It should not be modified directly by the user.\t\t\t\t\t\t|\n");
			fprintf (out,"\\*----------------------------------------------------------------------*/\n");
			fprintf (out,"\n");
			fprintf (out,"#include <stdio.h>\t/* for definition of NULL */\n");
			fprintf (out,"\n");
			fprintf (out,"char *about_text[] = {");

			if (*version_file == 'x') {
				fprintf (out,"\"Xtme: An editor for formal metadata\\n\",\n");
				fprintf (out,"\"  by Peter N. Schweitzer (U.S. Geological Survey, Reston, VA 22092)\\n\",\n");
				fprintf (out,"\"  version %s\\n\",\n",revision.xtme);
				}
			else {
				fprintf (out,"\"Tkme: Another editor for formal metadata\\n\",\n");
				fprintf (out,"\"  by Peter N. Schweitzer (U.S. Geological Survey, Reston, VA 22092)\\n\",\n");
				fprintf (out,"\"  version %s\\n\",\n",revision.tkme);
				}

			fprintf (out,"\"\\n\",\n");
			fprintf (out,"\"%s\",\n",es_credit);
			fprintf (out,"\"%s\",\n",id_credit);
			fprintf (out,"\"%s\",\n",fr_credit);
			fprintf (out,"\"%s\",\n",ca_credit);
			fprintf (out,"\"%s\",\n",de_credit);
			fprintf (out,"\"%s\",\n",pt_credit);

			fprintf (out,"\"\\n\",\n");
			fprintf (out,"\"Compilation information:\\n\",\n");

			now = time (NULL);
			strcpy (line,ctime (&now));
			s = line + strlen (line) - 1;
			if (*s == '\n') *s-- = 0;
			if (*s == '\r') *s-- = 0;
			fprintf (out,"\"  Date: %s\\n\",\n",line);

			if (uname (&u) != -1)
				fprintf (out,"\"  System: %s %s %s %s %s\\n\",\n",u.sysname,u.nodename,u.release,u.version,u.machine);
			fprintf (out,"\"\\n\",\n");

			if (hints_file) add_file (hints_file);

			fprintf (out,"NULL};\n");
			fprintf (out,"\n");
			fprintf (out,"/*----------------------------------------------------------------------*\\\n");
			fprintf (out,"\\*----------------------------------------------------------------------*/\n");
			}
	exit (0);
	}

/*----------------------------------------------------------------------*\
\*----------------------------------------------------------------------*/

