/*----------------------------------------------------------------------*\
 | Eight-character tags for SGML output.								|
 |																		|
 | Peter N. Schweitzer (U.S. Geological Survey, Reston, VA 20192)		|
\*----------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "keyword.h"
#include "local.h"
#include "item.h"
#include "stricmp.h"

/*----------------------------------------------------------------------*\
 | Functions to return, for a given FGDC keyword, the sgml tag text.	|
\*----------------------------------------------------------------------*/

struct ps8_tag {
	char *tag;
	enum fgdc_keyword key;
	};

struct ps8_tag ps8_tag_table[] = {
	{"arcsys",		WARC_Coordinate_System},
	{"arczone",		WARC_System_Zone_Identifier},
	{"absres",		WAbscissa_Resolution},
	{"abstract",	WAbstract},
	{"accconst",	WAccess_Constraints},
	{"accinstr",	WAccess_Instructions},
	{"address",		WAddress},
	{"addrtype",	WAddress_Type},
	{"albers",		WAlbers_Conical_Equal_Area},
	{"altdatum",	WAltitude_Datum_Name},
	{"altunits",	WAltitude_Distance_Units},
	{"altenc",		WAltitude_Encoding_Method},
	{"altres",		WAltitude_Resolution},
	{"altsys",		WAltitude_System_Definition},
	{"attr",		WAttribute},
	{"attracc",		WAttribute_Accuracy},
	{"attracce",	WAttribute_Accuracy_Explanation},
	{"attraccr",	WAttribute_Accuracy_Report},
	{"attraccv",	WAttribute_Accuracy_Value},
	{"attrdef",		WAttribute_Definition},
	{"attrdefs",	WAttribute_Definition_Source},
	{"attrdomv",	WAttribute_Domain_Values},
	{"attrlabl",	WAttribute_Label},
	{"attrmfrq",	WAttribute_Measurement_Frequency},
	{"attrmres",	WAttribute_Measurement_Resolution},
	{"attrunit",	WAttribute_Units_of_Measure},
	{"attrva",		WAttribute_Value_Accuracy},
	{"attrvae",		WAttribute_Value_Accuracy_Explanation},
	{"attrvai",		WAttribute_Value_Accuracy_Information},
	{"availabl",	WAvailable_Time_Period},
	{"azimptl",		WAzimuth_Measure_Point_Longitude},
	{"azimangl",	WAzimuthal_Angle},
	{"azimequi",	WAzimuthal_Equidistant},
	{"bearrefd",	WBearing_Reference_Direction},
	{"bearrefm",	WBearing_Reference_Meridian},
	{"bearres",		WBearing_Resolution},
	{"bearunit",	WBearing_Units},
	{"begdate",		WBeginning_Date},
	{"begdatea",	WBeginning_Date_of_Attribute_Values},
	{"begtime",		WBeginning_Time},
	{"bounding",	WBounding_Coordinates},
	{"browse",		WBrowse_Graphic},
	{"browsed",		WBrowse_Graphic_File_Description},
	{"browsen",		WBrowse_Graphic_File_Name},
	{"browset",		WBrowse_Graphic_File_Type},
	{"caldate",		WCalendar_Date},
	{"citation",	WCitation},
	{"citeinfo",	WCitation_Information},
	{"city",		WCity},
	{"cloud",		WCloud_Cover},
	{"codesetd",	WCodeset_Domain},
	{"codesetn",	WCodeset_Name},
	{"codesets",	WCodeset_Source},
	{"colcount",	WColumn_Count},
	{"compat",		WCompatibility_Information},
	{"complete",	WCompleteness_Report},
	{"compress",	WCompression_Support},
	{"computer",	WComputer_Contact_Information},
	{"cntaddr",		WContact_Address},
	{"cntemail",	WContact_Electronic_Mail_Address},
	{"cntfax",		WContact_Facsimile_Telephone},
	{"cntinfo",		WContact_Information},
	{"cntinst",		WContact_Instructions},
	{"cntorg",		WContact_Organization},
	{"cntorgp",		WContact_Organization_Primary},
	{"cntper",		WContact_Person},
	{"cntperp",		WContact_Person_Primary},
	{"cntpos",		WContact_Position},
	{"cnttdd",		WContact_TDD_TTY_Telephone},
	{"cntvoice",	WContact_Voice_Telephone},
	{"coordrep",	WCoordinate_Representation},
	{"country",		WCountry},
	{"crossref",	WCross_Reference},
	{"current",		WCurrentness_Reference},
	{"custom",		WCustom_Order_Process},
	{"dataqual",	WData_Quality_Information},
	{"datacred",	WData_Set_Credit},
	{"dsgpoly",		WData_Set_G_Polygon},
	{"dsgpolyx",	WData_Set_G_Polygon_Exclusion_G_Ring},
	{"dsgpolyo",	WData_Set_G_Polygon_Outer_G_Ring},
	{"denflat",		WDenominator_of_Flattening_Ratio},
	{"depthdn",		WDepth_Datum_Name},
	{"depthdu",		WDepth_Distance_Units},
	{"depthem",		WDepth_Encoding_Method},
	{"depthres",	WDepth_Resolution},
	{"depthsys",	WDepth_System_Definition},
	{"descript",	WDescription},
	{"detailed",	WDetailed_Description},
	{"dialfile",	WDialup_File_Name},
	{"dialinst",	WDialup_Instructions},
	{"dialtel",		WDialup_Telephone},
	{"digform",		WDigital_Form},
	{"digtinfo",	WDigital_Transfer_Information},
	{"digtopt",		WDigital_Transfer_Option},
	{"direct",		WDirect_Spatial_Reference_Method},
	{"distres",		WDistance_Resolution},
	{"distbrep",	WDistance_and_Bearing_Representation},
	{"distinfo",	WDistribution_Information},
	{"distliab",	WDistribution_Liability},
	{"distrib",		WDistributor},
	{"eastbc",		WEast_Bounding_Coordinate},
	{"edition",		WEdition},
	{"ellips",		WEllipsoid_Name},
	{"enddate",		WEnding_Date},
	{"enddatea",	WEnding_Date_of_Attribute_Values},
	{"endtime",		WEnding_Time},
	{"enttyp",		WEntity_Type},
	{"enttypd",		WEntity_Type_Definition},
	{"enttypds",	WEntity_Type_Definition_Source},
	{"enttypl",		WEntity_Type_Label},
	{"eadetcit",	WEntity_and_Attribute_Detail_Citation},
	{"eainfo",		WEntity_and_Attribute_Information},
	{"eaover",		WEntity_and_Attribute_Overview},
	{"edom",		WEnumerated_Domain},
	{"edomv",		WEnumerated_Domain_Value},
	{"edomvd",		WEnumerated_Domain_Value_Definition},
	{"edomvds",		WEnumerated_Domain_Value_Definition_Source},
	{"equicon",		WEquidistant_Conic},
	{"equirect",	WEquirectangular},
	{"feast",		WFalse_Easting},
	{"fnorth",		WFalse_Northing},
	{"fees",		WFees},
	{"filedec",		WFile_Decompression_Technique},
	{"formcont",	WFormat_Information_Content},
	{"formname",	WFormat_Name},
	{"formspec",	WFormat_Specification},
	{"formverd",	WFormat_Version_Date},
	{"formvern",	WFormat_Version_Number},
	{"gringlat",	WG_Ring_Latitude},
	{"gringlon",	WG_Ring_Longitude},
	{"gvnsp",		WGeneral_Vertical_Near_sided_Perspective},
	{"geodetic",	WGeodetic_Model},
	{"geograph",	WGeographic},
	{"geogunit",	WGeographic_Coordinate_Units},
	{"geoform",		WGeospatial_Data_Presentation_Form},
	{"gnomonic",	WGnomonic},
	{"gridsys",		WGrid_Coordinate_System},
	{"gridsysn",	WGrid_Coordinate_System_Name},
	{"heightpt",	WHeight_of_Perspective_Point_Above_Surface},
	{"highbps",		WHighest_BPS},
	{"horizsys",	WHorizontal_Coordinate_System_Definition},
	{"horizdn",		WHorizontal_Datum_Name},
	{"horizpa",		WHorizontal_Positional_Accuracy},
	{"horizpae",	WHorizontal_Positional_Accuracy_Explanation},
	{"horizpar",	WHorizontal_Positional_Accuracy_Report},
	{"horizpav",	WHorizontal_Positional_Accuracy_Value},
	{"hours",		WHours_of_Service},
	{"idinfo",		WIdentification_Information},
	{"indspref",	WIndirect_Spatial_Reference},
	{"issue",		WIssue_Identification},
	{"keywords",	WKeywords},
	{"lamberta",	WLambert_Azimuthal_Equal_Area},
	{"lambertc",	WLambert_Conformal_Conic},
	{"landsat",		WLandsat_Number},
	{"lworkcit",	WLarger_Work_Citation},
	{"latres",		WLatitude_Resolution},
	{"latprjc",		WLatitude_of_Projection_Center},
	{"latprjo",		WLatitude_of_Projection_Origin},
	{"lineage",		WLineage},
	{"local",		WLocal},
	{"localdes",	WLocal_Description},
	{"localgeo",	WLocal_Georeference_Information},
	{"localp",		WLocal_Planar},
	{"localpd",		WLocal_Planar_Description},
	{"localpgi",	WLocal_Planar_Georeference_Information},
	{"logic",		WLogical_Consistency_Report},
	{"longres",		WLongitude_Resolution},
	{"longcm",		WLongitude_of_Central_Meridian},
	{"longpc",		WLongitude_of_Projection_Center},
	{"lowbps",		WLowest_BPS},
	{"update",		WMaintenance_and_Update_Frequency},
	{"mapproj",		WMap_Projection},
	{"mapprojn",	WMap_Projection_Name},
	{"mercator",	WMercator},
	{"metadata",	WMetadata},
	{"metac",		WMetadata_Access_Constraints},
	{"metc",		WMetadata_Contact},
	{"metd",		WMetadata_Date},
	{"metfrd",		WMetadata_Future_Review_Date},
	{"metainfo",	WMetadata_Reference_Information},
	{"metalang",	WMetadata_Language},
	{"metrd",		WMetadata_Review_Date},
	{"metsc",		WMetadata_Security_Classification},
	{"metscs",		WMetadata_Security_Classification_System},
	{"metshd",		WMetadata_Security_Handling_Description},
	{"metsi",		WMetadata_Security_Information},
	{"metstdn",		WMetadata_Standard_Name},
	{"metstdv",		WMetadata_Standard_Version},
	{"mettc",		WMetadata_Time_Convention},
	{"metuc",		WMetadata_Use_Constraints},
	{"miller",		WMiller_Cylindrical},
	{"modsak",		WModified_Stereographic_for_Alaska},
	{"mdattim",		WMultiple_Dates_Times},
	{"native",		WNative_Data_Set_Environment},
	{"networka",	WNetwork_Address},
	{"networkr",	WNetwork_Resource_Name},
	{"nondig",		WNon_digital_Form},
	{"northbc",		WNorth_Bounding_Coordinate},
	{"numdata",		WNumber_DataBits},
	{"numstop",		WNumber_StopBits},
	{"obqlazim",	WOblique_Line_Azimuth},
	{"obqllat",		WOblique_Line_Latitude},
	{"obqllong",	WOblique_Line_Longitude},
	{"obqlpt",		WOblique_Line_Point},
	{"obqmerc",		WOblique_Mercator},
	{"offmedia",	WOffline_Media},
	{"offoptn",		WOffline_Option},
	{"oncomp",		WOnline_Computer_and_Operating_System},
	{"onlink",		WOnline_Linkage},
	{"onlinopt",	WOnline_Option},
	{"ordering",	WOrdering_Instructions},
	{"ordres",		WOrdinate_Resolution},
	{"origin",		WOriginator},
	{"orthogr",		WOrthographic},
	{"othercit",	WOther_Citation_Details},
	{"othergrd",	WOther_Grid_Systems_Definition},
	{"otherprj",	WOther_Projections_Definition},
	{"overview",	WOverview_Description},
	{"parity",		WParity},
	{"pathnum",		WPath_Number},
	{"place",		WPlace},
	{"placekey",	WPlace_Keyword},
	{"placekt",		WPlace_Keyword_Thesaurus},
	{"planar",		WPlanar},
	{"plance",		WPlanar_Coordinate_Encoding_Method},
	{"planci",		WPlanar_Coordinate_Information},
	{"plandu",		WPlanar_Distance_Units},
	{"ptvctcnt",	WPoint_and_Vector_Object_Count},
	{"ptvctinf",	WPoint_and_Vector_Object_Information},
	{"ptcontac",	WPoint_of_Contact},
	{"polarst",		WPolar_Stereographic},
	{"polycon",		WPolyconic},
	{"posacc",		WPositional_Accuracy},
	{"postal",		WPostal_Code},
	{"proccont",	WProcess_Contact},
	{"procdate",	WProcess_Date},
	{"procdesc",	WProcess_Description},
	{"procstep",	WProcess_Step},
	{"proctime",	WProcess_Time},
	{"progress",	WProgress},
	{"pubdate",		WPublication_Date},
	{"pubinfo",		WPublication_Information},
	{"pubplace",	WPublication_Place},
	{"pubtime",		WPublication_Time},
	{"publish",		WPublisher},
	{"purpose",		WPurpose},
	{"qattracc",	WQuantitative_Attribute_Accuracy_Assessment},
	{"qhorizpa",	WQuantitative_Horizontal_Positional_Accuracy_Assessment},
	{"qvertpa",		WQuantitative_Vertical_Positional_Accuracy_Assessment},
	{"rdom",		WRange_Domain},
	{"rdommax",		WRange_Domain_Maximum},
	{"rdommin",		WRange_Domain_Minimum},
	{"rngdates",	WRange_of_Dates_Times},
	{"rastinfo",	WRaster_Object_Information},
	{"rasttype",	WRaster_Object_Type},
	{"reccap",		WRecording_Capacity},
	{"recden",		WRecording_Density},
	{"recdenu",		WRecording_Density_Units},
	{"recfmt",		WRecording_Format},
	{"resdesc",		WResource_Description},
	{"robinson",	WRobinson},
	{"rowcount",	WRow_Count},
	{"sdtstype",	WSDTS_Point_and_Vector_Object_Type},
	{"sdtsterm",	WSDTS_Terms_Description},
	{"spcszone",	WSPCS_Zone_Identifier},
	{"sfctrlin",	WScale_Factor_at_Center_Line},
	{"sfctrmer",	WScale_Factor_at_Central_Meridian},
	{"sfequat",		WScale_Factor_at_Equator},
	{"sfprjorg",	WScale_Factor_at_Projection_Origin},
	{"secclass",	WSecurity_Classification},
	{"secsys",		WSecurity_Classification_System},
	{"sechandl",	WSecurity_Handling_Description},
	{"secinfo",		WSecurity_Information},
	{"semiaxis",	WSemi_major_Axis},
	{"serinfo",		WSeries_Information},
	{"sername",		WSeries_Name},
	{"sngdate",		WSingle_Date_Time},
	{"sinusoid",	WSinusoidal},
	{"srccite",		WSource_Citation},
	{"srccitea",	WSource_Citation_Abbreviation},
	{"srccontr",	WSource_Contribution},
	{"srccurr",		WSource_Currentness_Reference},
	{"srcinfo",		WSource_Information},
	{"srcprod",		WSource_Produced_Citation_Abbreviation},
	{"srcscale",	WSource_Scale_Denominator},
	{"srctime",		WSource_Time_Period_of_Content},
	{"srcused",		WSource_Used_Citation_Abbreviation},
	{"southbc",		WSouth_Bounding_Coordinate},
	{"spaceobq",	WSpace_Oblique_Mercator_Landsat},
	{"spdoinfo",	WSpatial_Data_Organization_Information},
	{"spdom",		WSpatial_Domain},
	{"spref",		WSpatial_Reference_Information},
	{"stdorder",	WStandard_Order_Process},
	{"stdparll",	WStandard_Parallel},
	{"spcs",		WState_Plane_Coordinate_System},
	{"state",		WState_or_Province},
	{"status",		WStatus},
	{"stereo",		WStereographic},
	{"svlong",		WStraight_Vertical_Longitude_from_Pole},
	{"stratum",		WStratum},
	{"stratkey",	WStratum_Keyword},
	{"stratkt",		WStratum_Keyword_Thesaurus},
	{"supplinf",	WSupplemental_Information},
	{"techpreq",	WTechnical_Prerequisites},
	{"temporal",	WTemporal},
	{"tempkey",		WTemporal_Keyword},
	{"tempkt",		WTemporal_Keyword_Thesaurus},
	{"theme",		WTheme},
	{"themekey",	WTheme_Keyword},
	{"themekt",		WTheme_Keyword_Thesaurus},
	{"timeinfo",	WTime_Period_Information},
	{"timeperd",	WTime_Period_of_Content},
	{"time",		WTime_of_Day},
	{"title",		WTitle},
	{"transize",	WTransfer_Size},
	{"transmer",	WTransverse_Mercator},
	{"turnarnd",	WTurnaround},
	{"typesrc",		WType_of_Source_Media},
	{"upszone",		WUPS_Zone_Identifier},
	{"utmzone",		WUTM_Zone_Number},
	{"ups",			WUniversal_Polar_Stereographic},
	{"utm",			WUniversal_Transverse_Mercator},
	{"udom",		WUnrepresentable_Domain},
	{"useconst",	WUse_Constraints},
	{"vpftype",		WVPF_Point_and_Vector_Object_Type},
	{"vpfterm",		WVPF_Terms_Description},
	{"vpflevel",	WVPF_Topology_Level},
	{"vertdef",		WVertical_Coordinate_System_Definition},
	{"vrtcount",	WVertical_Count},
	{"vertacc",		WVertical_Positional_Accuracy},
	{"vertacce",	WVertical_Positional_Accuracy_Explanation},
	{"vertaccr",	WVertical_Positional_Accuracy_Report},
	{"vertaccv",	WVertical_Positional_Accuracy_Value},
	{"westbc",		WWest_Bounding_Coordinate},
	{"vdgrin",		Wvan_der_Grinten},

	/* FGDC-STD-001-1998 */

	{"gring",		WG_Ring},
	{"grngpoin",	WG_Ring_Point},
	{"vpfinfo",		WVPF_Point_and_Vector_Object_Information},
	{"mapprojp",	WMap_Projection_Parameters},
	{"metextns",	WMetadata_Extensions},
	{"metprof",		WProfile_Name},

	/* Extension for language */

	{"metalang",	WMetadata_Language},

	/* Biological Data Profile */

	{"descgeog",	WDescription_of_Geographic_Extent},
	{"boundalt",	WBounding_Altitudes},
	{"altmin",		WAltitude_Minimum},
	{"altmax",		WAltitude_Maximum},
	{"taxonomy",	WTaxonomy},
	{"keywtax",		WKeywords_Taxon},
	{"taxonkt",		WTaxonomic_Keyword_Thesaurus},
	{"taxonkey",	WTaxonomic_Keywords},
	{"taxonsys",	WTaxonomic_System},
	{"classsys",	WClassification_System_Authority},
	{"classcit",	WClassification_System_Citation},
	{"classmod",	WClassification_System_Modifications},
	{"idref",		WIdentification_Reference},
	{"ider",		WIdentifier},
	{"taxonpro",	WTaxonomic_Procedures},
	{"taxoncom",	WTaxonomic_Completeness},
	{"vouchers",	WVouchers},
	{"specimen",	WSpecimen},
	{"reposit",		WRepository},
	{"taxongen",	WGeneral_Taxonomic_Coverage},
	{"taxoncl",		WTaxonomic_Classification},
	{"taxonrn",		WTaxon_Rank_Name},
	{"taxonrv",		WTaxon_Rank_Value},
	{"common",		WApplicable_Common_Name},
	{"tool",		WAnalytical_Tool},
	{"tooldesc",	WAnalytical_Tool_Description},
	{"toolacc",		WTool_Access_Information},
	{"toolcont",	WTool_Contact},
	{"toolcite",	WTool_Citation},
	{"toolinst",	WTool_Access_Instructions},
	{"toolcomp",	WTool_Computer_and_Operating_System},
	{"method",		WMethodology},
	{"methtype",	WMethodology_Type},
	{"methodid",	WMethodology_Identifier},
	{"methkt",		WMethodology_Keyword_Thesaurus},
	{"methkey",		WMethodology_Keyword},
	{"methdesc",	WMethodology_Description},
	{"methcite",	WMethodology_Citation},
	{"asciistr",	WASCII_File_Structure},
	{"recdel",		WRecord_Delimiter},
	{"numheadl",	WNumber_Header_Lines},
	{"deschead",	WDescription_of_Header_Content},
	{"orienta",		WOrientation},
	{"casesens",	WCase_Sensitive},
	{"authent",		WAuthentication},
	{"quotech",		WQuote_Character},
	{"datafiel",	WData_Field},
	{"dfieldnm",	WData_Field_Name},
	{"missingv",	WMissing_Value_Code},
	{"dfwidthd",	WData_Field_Width_Delimiter},
	{"dfwidth",		WData_Field_Width},
	{"beggeol",		WBeginning_Geologic_Age},
	{"endgeol",		WEnding_Geologic_Age},
	{"geolage",		WGeologic_Age},
	{"geolscal",	WGeologic_Time_Scale},
	{"geolest",		WGeologic_Age_Estimate},
	{"geolun",		WGeologic_Age_Uncertainty},
	{"geolexpl",	WGeologic_Age_Explanation},
	{"geolcit",		WGeologic_Citation},

	/* Shoreline Data Profile */

	{"barpres",		WBarometric_Pressure},
	{"event",		WEnvironmental_Event},
	{"marweat",		WMarine_Weather_Condition},
	{"procite",		WProcess_Step_Citation},
	{"tidrang",		WRange_of_Tide},
	{"tidesup",		WSupplemental_Tidal_Information},
	{"tidedat",		WTidal_Datum},
	{"tidinfo",		WTidal_Information},
	{"tideref",		WTide_Table_Reference},
	{"hightime",	WTime_of_High_Tide},
	{"lowtime",		WTime_of_Low_Tide},
	{"tidtime",		WTime_of_Tide},
	{"tidtype",		WType_of_Tide},
	{"wavhite",		WWave_Height},
	{"windir",		WWind_Direction},
	{"windsp",		WWind_Speed},

	/* Remote-sensing profile */

	{"acqinfo",		WAcquisition_Information},
	{"afspeed",		WAerial_Film_Speed},
	{"aerotref",	WAerotriangulation_Reference},
	{"affc1",		WAffine_Distortion_X_Prime_Coefficient},
	{"affc2",		WAffine_Distortion_Y_Prime_Coefficient},
	{"aggcrit",		WAggregation_Criteria},
	{"agginfo",		WAggregation_Information},
	{"aggmemid",	WAggregation_Member_ID},
	{"algochde",	WAlgorithm_Change_Description},
	{"algochhi",	WAlgorithm_Change_History},
	{"algochrf",	WAlgorithm_Change_Reference},
	{"algodesc",	WAlgorithm_Description},
	{"algoid",		WAlgorithm_Identifiers},
	{"algoinfo",	WAlgorithm_Information},
	{"algprevd",	WAlgorithm_Peer_Review_Description},
	{"algprevi",	WAlgorithm_Peer_Review_Information},
	{"algprevr",	WAlgorithm_Peer_Review_Reference},
	{"algoref",		WAlgorithm_Reference},
	{"algotexd",	WAlgorithm_Text_Description},
	{"altlayer",	WAltitude_Distance_Layer},
	{"ancdata",		WAncillary_Dataset},
	{"ancdsdes",	WAncillary_Dataset_Description},
	{"ancdatid",	WAncillary_Dataset_Identifier},
	{"ancdesc",		WAncillary_Description},
	{"ancref",		WAncillary_Reference},
	{"rsaddist",	WAngle_Dependent_Distortion},
	{"rsadval",		WAngle_Distortion_Value},
	{"awar",		WArea_Weighted_Average_Resolution},
	{"argupgee",	WArgument_of_Perigee},
	{"ascdscin",	WAscending_Descending_Indicator},
	{"attanglu",	WAttitude_Angular_Units},
	{"senselav",	WAvailability_of_Element_Locations},
	{"axes",		WAxes},
	{"axrotcon",	WAxis_Rotation_Convention},
	{"bbnddef",		WBand_Boundary_Definition},
	{"banddesc",	WBand_Description},
	{"bandid",		WBand_ID},
	{"bandidnt",	WBand_Identification},
	{"bmmodid",		WBand_Measurement_Mode_ID},
	{"bandqual",	WBand_Quality},
	{"fcsscapo",	WCalibrated_Detector_Positions},
	{"calfocl",		WCalibrated_Focal_Length},
	{"camident",	WCamera_Identifier},
	{"camtype",		WCamera_Type},
	{"cvaltype",	WCell_Value_Type},
	{"clocdrft",	WClock_Time_Drift},
	{"colltype",	WCollection_Type},
	{"coldeltx",	WColumn_Delta_X},
	{"coldelty",	WColumn_Delta_Y},
	{"procpmcl",	WCommand_Line_Processing_Parameter},
	{"compinfo",	WComponent_Information},
	{"conpckid",	WContainer_Packet_ID},
	{"gcpauth",		WControl_Point_Authority},
	{"gcpcolum",	WControl_Point_Column},
	{"gcpearlc",	WControl_Point_Earth_Location},
	{"gcpxaccu",	WControl_Point_Horizontal_X_Accuracy},
	{"gcpyaccu",	WControl_Point_Horizontal_Y_Accuracy},
	{"gcpid",		WControl_Point_ID},
	{"gcpidnt",		WControl_Point_Identification},
	{"gcpori",		WControl_Point_Origin},
	{"conptrpo",	WControl_Point_Raster_Position},
	{"gcprow",		WControl_Point_Row},
	{"gcptype",		WControl_Point_Type},
	{"gcpusefl",	WControl_Point_Use_Flag},
	{"gcpzaccu",	WControl_Point_Vertical_Accuracy},
	{"gcpxval",		WControl_Point_x_Value},
	{"gcpyval",		WControl_Point_y_Value},
	{"gcpzval",		WControl_Point_z_Value},
	{"coordpt",		WCoordinate_Point},
	{"coordsys",	WCoordinate_System},
	{"coorxval",	WCoordinate_x_Value},
	{"coordxyu",	WCoordinate_XY_Units},
	{"cooryval",	WCoordinate_y_Value},
	{"coordzu",		WCoordinate_Z_Units},
	{"coorzval",	WCoordinate_z_Value},
	{"xtkaxis",		WCross_Track_Axis},
	{"xtrkdesc",	WCross_Track_Description},
	{"xtkdir",		WCross_Track_Direction},
	{"xtkexang",	WCross_Track_Extent_Angle},
	{"xtfixang",	WCross_Track_Fixed_Angle},
	{"xtrckmot",	WCross_Track_Motion},
	{"xtkstang",	WCross_Track_Start_Angle},
	{"xtstpang",	WCross_Track_Step_Angle},
	{"xtkswp",		WCross_Track_Sweep},
	{"xtkzero",		WCross_Track_Zero},
	{"datdicrf",	WData_Dictionary_Reference},
	{"datascal",	WData_Scaling_Information},
	{"datsetid",	WDataset_Identifier},
	{"datlcali",	WDate_of_Last_Calibration},
	{"develop",		WDeveloping_Institution},
	{"dimcount",	WDimension_Count},
	{"dimdesc",		WDimension_Description},
	{"dimprops",	WDimension_Properties},
	{"rsdisdis",	WDistance_Dependent_Distortion},
	{"rsddval",		WDistance_Distortion_Value},
	{"distort",		WDistortion},
	{"dsttaf",		WDistortion_Type_Affine},
	{"dsttrasy",	WDistortion_Type_Radial_Asymmetrical},
	{"dsttrs",		WDistortion_Type_Radial_Symmetrical},
	{"documnts",	WDocumentation},
	{"eccentry",	WEccentricity},
	{"eafspeed",	WEffective_Aerial_Film_Speed},
	{"elevaxis",	WElevation_Axis},
	{"elevdesc",	WElevation_Description},
	{"elevdir",		WElevation_Direction},
	{"eleexang",	WElevation_Extent_Angle},
	{"elfixang",	WElevation_Fixed_Angle},
	{"elestang",	WElevation_Start_Angle},
	{"elstpang",	WElevation_Step_Angle},
	{"elevswp",		WElevation_Sweep},
	{"elevzero",	WElevation_Zero},
	{"ephem",		WEphemeris},
	{"expotime",	WExposure_Time},
	{"accexori",	WExterior_Orientation_Accuracy},
	{"fcfid",		WFiducial},
	{"fidcent",		WFiducial_Center},
	{"filmtype",	WFilm_Type},
	{"filtonca",	WFilter_on_Camera},
	{"filtcain",	WFilter_on_Camera_Indicator},
	{"ftrtyp",		WFilter_Type},
	{"fltprot",		WFlight_Protocol},
	{"flyhite",		WFlying_Height},
	{"fmc",			WForward_Motion_Compensation},
	{"frarea",		WFrame_Area},
	{"frareaun",	WFrame_Area_Units},
	{"frareavl",	WFrame_Area_Value},
	{"frame",		WFrame_Camera},
	{"framegeo",	WFrame_Geometric_Properties},
	{"fcamhwar",	WFrame_Hardware},
	{"fcoper",		WFrame_Operation},
	{"fcamoptc",	WFrame_Optics},
	{"fcradcal",	WFrame_Radiometric_Calibration},
	{"fradprop",	WFrame_Radiometric_Properties},
	{"fcspecin",	WFrame_Spectral_Information},
	{"fcspecpr",	WFrame_Spectral_Properties},
	{"georecra",	WGeorectified_Raster},
	{"georbler",	WGeoreferenceable_Raster},
	{"georefde",	WGeoreferencing_Description},
	{"georefin",	WGeoreferencing_Information},
	{"gpsavail",	WGPS_Information_System_Availability},
	{"gridinit",	WGrid_First_Element},
	{"grinitx",		WGrid_First_Element_Map_X_Coordinate},
	{"grinity",		WGrid_First_Element_Map_Y_Coordinate},
	{"gridori",		WGrid_Orientation},
	{"gcptdesc",	WGround_Control_Point_Description},
	{"gcptinfo",	WGround_Control_Point_Information},
	{"gcporg",		WGround_Control_Point_Organization},
	{"gcptpos",		WGround_Control_Point_Position},
	{"grndshpe",	WGround_Shape},
	{"ovlpdtid",	WID_Overlapping_Dataset},
	{"ifovunit",	WIFOV_Units},
	{"ifovxdef",	WIFOV_x_Definition},
	{"ifovxval",	WIFOV_x_Value},
	{"ifovydef",	WIFOV_y_Definition},
	{"ifovyval",	WIFOV_y_Value},
	{"imagqual",	WImage_Quality},
	{"imsize",		WImage_Size},
	{"imsizex",		WImage_Size_x_Value},
	{"imsizey",		WImage_Size_y_Value},
	{"inclinat",	WInclination},
	{"inbident",	WIndividual_Band_Identification},
	{"inpdatid",	WInput_Dataset_Identifier},
	{"prindesc",	WInput_Description},
	{"prinlevl",	WInput_Level},
	{"prinref",		WInput_Reference},
	{"insavail",	WINS_Reading_Availability},
	{"ifov",		WInstantaneous_Field_of_View},
	{"inslcali",	WInstitution_of_Last_Calibration},
	{"instdesc",	WInstrument_Description},
	{"instflnm",	WInstrument_Full_Name},
	{"instinfo",	WInstrument_Information},
	{"instpdes",	WInstrument_Properties_Description},
	{"instref",		WInstrument_Reference},
	{"instshnm",	WInstrument_Short_Name},
	{"insspgeo",	WInstrument_Specific_Georeferencing},
	{"insttyp",		WInstrument_Type},
	{"kepleror",	WKeplerian_Orbit},
	{"lastcali",	WLast_Calibration},
	{"layrname",	WLayer_Name},
	{"lens",		WLens},
	{"lensidnt",	WLens_Identifier},
	{"lenstype",	WLens_Type},
	{"lightdrp",	WLight_Drop},
	{"locainfo",	WLocation_Information},
	{"magident",	WMagazine_Identifier},
	{"lambdmax",	WMaximum_Wavelength},
	{"metlcali",	WMethod_of_Last_Calibration},
	{"lambdmin",	WMinimum_Wavelength},
	{"misscomp",	WMission_Completion},
	{"missdesc",	WMission_Description},
	{"misshist",	WMission_History},
	{"missinfo",	WMission_Information},
	{"missname",	WMission_Name},
	{"misssige",	WMission_Significant_Event},
	{"missstdt",	WMission_Start_Date},
	{"multimal",	WMultiple_Image_Alignment},
	{"nadir",		WNadir},
	{"nadirlat",	WNadir_Latitude},
	{"nadirlon",	WNadir_Longitude},
	{"namedim",		WName_of_Dimension},
	{"ngeopos",		WNominal_Geostationary_Position},
	{"nomspres",	WNominal_Spatial_Resolution},
	{"npolscal",	WNon_Polynomial_Scaling},
	{"rsanumv",		WNumber_of_Angle_Distortion_Values},
	{"numbands",	WNumber_of_Bands},
	{"xtksamp",		WNumber_of_Cross_Track_Samples},
	{"nodatdim",	WNumber_of_Data_Dimensions},
	{"ndenterm",	WNumber_of_Denominator_Terms},
	{"rsdnumv",		WNumber_of_Distance_Distortion_Values},
	{"elevsamp",	WNumber_of_Elevation_Samples},
	{"nnumterm",	WNumber_of_Numerator_Terms},
	{"numpoint",	WNumber_of_Points},
	{"profsamp",	WNumber_of_Profile_Samples},
	{"nresval",		WNumber_of_Resolution_Values},
	{"numthlay",	WNumber_of_Thematic_Layers},
	{"nowvband",	WNumber_of_Wavelength_Bands},
	{"opmode",		WOperational_Mode},
	{"orbangun",	WOrbit_Angle_Units},
	{"orbitpd",		WOrbit_Period},
	{"orpdunit",	WOrbit_Period_Units},
	{"orpdval",		WOrbit_Period_Value},
	{"othcolde",	WOther_Collector_Description},
	{"othrefde",	WOther_Georeferencing_Description},
	{"wwrefpat",	WPath},
	{"pkwavlen",	WPeak_Wavelength},
	{"ovlppct",		WPercentage_of_Overlap},
	{"peripass",	WPerigee_Passage_Time},
	{"phorespo",	WPhotographic_Resolving_Power},
	{"phipitch",	WPitch},
	{"accphi",		WPitch_Accuracy},
	{"pixxtksz",	WPixel_Cross_Track_Size},
	{"pixldesc",	WPixel_Description},
	{"pixelvsz",	WPixel_Elevation_Size},
	{"pixelhgt",	WPixel_Height_Above_Ellipsoid},
	{"pixelpsf",	WPixel_Point_Spread_Function},
	{"pixprfsz",	WPixel_Profile_Size},
	{"pixlreso",	WPixel_Resolution},
	{"plainsid",	WPlatform_and_Instrument_Identification},
	{"plmiinfo",	WPlatform_and_Mission_Information},
	{"platdesc",	WPlatform_Description},
	{"platflnm",	WPlatform_Full_Name},
	{"platinfo",	WPlatform_Information},
	{"gpnalti",		WPlatform_Nominal_Altitude},
	{"gpnaltun",	WPlatform_Nominal_Altitude_Units},
	{"gpnaltva",	WPlatform_Nominal_Altitude_Value},
	{"pnomlong",	WPlatform_Nominal_Longitude},
	{"platforb",	WPlatform_Orbit},
	{"platfser",	WPlatform_Serial_Identifier},
	{"platfsnm",	WPlatform_Short_Name},
	{"platspon",	WPlatform_Sponsor},
	{"platstdt",	WPlatform_Start_Date},
	{"ptpos",		WPoint_Position_In_Pixel},
	{"polrzcha",	WPolarization_Characteristics},
	{"polycoef",	WPolynomial_Coefficient},
	{"polydeno",	WPolynomial_Denominator},
	{"polyfunc",	WPolynomial_Function},
	{"polynume",	WPolynomial_Numerator},
	{"posiinfo",	WPositional_Information},
	{"pripoaut",	WPrincipal_Point_of_Autocollimation},
	{"priposym",	WPrincipal_Point_of_Symmetry},
	{"procchhi",	WProcessing_Change_History},
	{"procdoc",		WProcessing_Documentation},
	{"procenv",		WProcessing_Environment},
	{"procidfs",	WProcessing_Identifiers},
	{"procinfo",	WProcessing_Information},
	{"procinp",		WProcessing_Input_Dataset},
	{"proclevl",	WProcessing_Level},
	{"prolevau",	WProcessing_Level_Authority},
	{"prolevid",	WProcessing_Level_Identifier},
	{"procprcd",	WProcessing_Procedure},
	{"procpdes",	WProcessing_Procedure_Description},
	{"prunhist",	WProcessing_Run_History},
	{"procsoft",	WProcessing_Software},
	{"prsodesc",	WProcessing_Software_Description},
	{"prsoref",		WProcessing_Software_Reference},
	{"profdesc",	WProfile_Description},
	{"profext",		WProfile_Extent},
	{"proffix",		WProfile_Fixed},
	{"profprop",	WProfile_Properties},
	{"profsond",	WProfile_Sounding},
	{"profst",		WProfile_Start},
	{"profstep",	WProfile_Step},
	{"profdir",		WProfiling_Direction},
	{"prjcxpos",	WProjection_Center_X_Position},
	{"prjcypos",	WProjection_Center_Y_Position},
	{"prjczpos",	WProjection_Center_Z_Position},
	{"quappax",		WQuality_of_Autocollimation_Principal_Point_x_Value},
	{"quappay",		WQuality_of_Autocollimation_Principal_Point_y_Value},
	{"quafocl",		WQuality_of_Focal_Length},
	{"quappa",		WQuality_of_Principal_Point_of_Autocollimation},
	{"quapps",		WQuality_of_Principal_Point_of_Symmetry},
	{"quappsx",		WQuality_of_Symmetry_Principal_Point_x_Value},
	{"quappsy",		WQuality_of_Symmetry_Principal_Point_y_Value},
	{"rab1",		WRadial_Asymmetrical_Coefficient_B1},
	{"rab2",		WRadial_Asymmetrical_Coefficient_B2},
	{"rsangint",	WRadial_Symmetrical_Angle_Interval},
	{"rsdmdist",	WRadial_Symmetrical_Distance_Interval},
	{"rsdispol",	WRadial_Symmetrical_Distortion_Polynomial},
	{"revrpolr",	WReceiver_Polarization},
	{"refsys",		WReferenced_Coordinate_System},
	{"refrpoly",	WReferencing_Polynomial},
	{"relaper",		WRelative_Aperture},
	{"relazi",		WRelative_Azimuth},
	{"fcres",		WReseau},
	{"phoreset",	WResolution_Value_Set},
	{"resangle",	WResolving_Angle},
	{"resrad",		WResolving_Value_Radial},
	{"restang",		WResolving_Value_Tangential},
	{"raascnod",	WRight_Ascension_of_Ascending_Node},
	{"omegarll",	WRoll},
	{"accomega",	WRoll_Accuracy},
	{"rotdesc",		WRotation_Description},
	{"pcrotseq",	WRotation_Sequence},
	{"wwrefrow",	WRow},
	{"rowdeltx",	WRow_Delta_X},
	{"rowdelty",	WRow_Delta_Y},
	{"sampangu",	WSample_Angle_Units},
	{"sampdesu",	WSample_Description_Units},
	{"samprofu",	WSample_Profile_Units},
	{"sampprop",	WSample_Properties},
	{"satlocaa",	WSatellite_Local_Azimuth_Angle},
	{"satlocza",	WSatellite_Local_Zenith_Angle},
	{"scan",		WScan},
	{"scanangu",	WScan_Angle_Units},
	{"xtrkprop",	WScan_Cross_Track_Properties},
	{"scandisu",	WScan_Distance_Units},
	{"scandur",		WScan_Duration},
	{"elevprop",	WScan_Elevation_Properties},
	{"scangeom",	WScan_Geometric_Properties},
	{"sradprop",	WScan_Radiometric_Properties},
	{"scanrptt",	WScan_Repeat_Time},
	{"scspprop",	WScan_Spectral_Properties},
	{"scanstart",	WScan_Start_Time},
	{"scanstept",	WScan_Step_Time},
	{"scantimu",	WScan_Time_Units},
	{"scantime",	WScan_Timing},
	{"scipap",		WScience_Paper},
	{"semimaax",	WSemimajor_Axis},
	{"sendpolr",	WSender_Polarization},
	{"sensello",	WSensor_Element_Location},
	{"fcssgrid",	WSensor_Grid},
	{"orininfo",	WSensor_Orientation},
	{"fcss",		WSensor_System},
	{"solaraza",	WSolar_Azimuth_Angle},
	{"solarzea",	WSolar_Zenith_Angle},
	{"senselso",	WSource_of_Element_Locations},
	{"spatresu",	WSpatial_Resolution_Units},
	{"spatresv",	WSpatial_Resolution_Value},
	{"specinfo",	WSpectral_Information},
	{"spectlim",	WSpectral_Limit},
	{"stabmnt",		WStabilized_Mount},
	{"storord",		WStorage_Order},
	{"swtrkinf",	WSwath_Track_Information},
	{"thelayid",	WThematic_Layer_Identification},
	{"trandesc",	WTranslation_Description},
	{"ovlptype",	WType_of_Overlap},
	{"unrefsys",	WUnreferenced_Coordinate_System},
	{"userguid",	WUsers_Guide},
	{"wvbandpr",	WWavelength_Band_Properties},
	{"wavlreg",		WWavelength_Region},
	{"waveunit",	WWavelength_Units},
	{"wwrefsys",	WWorldwide_Reference_System},
	{"xaxisdef",	WX_Axis_Definition},
	{"accxpos",		WX_Position_Accuracy},
	{"yaxisdef",	WY_Axis_Definition},
	{"accypos",		WY_Position_Accuracy},
	{"kappayaw",	WYaw},
	{"acckappa",	WYaw_Accuracy},
	{"zaxisdef",	WZ_Axis_Definition},
	{"acczpos",		WZ_Position_Accuracy},

#ifdef USE_ESRI_EXTENSIONS

	/* ESRI elements */

	{"altunits",	WAltitude_Units},
	{"roletype",	WAncillary_Role},
	{"rolefld",		WAncillary_Role_Attribute},
	{"attalias",	WAttribute_Alias},
	{"stflddd",		WAttribute_Defined_Domain},
	{"atindex",		WAttribute_Indexed},
	{"atnumdec",	WAttribute_Number_of_Decimals},
	{"atoutwid",	WAttribute_Output_Width},
	{"atprecis",	WAttribute_Precision},
	{"attscale",	WAttribute_Scale},
	{"attrtype",	WAttribute_Type},
	{"attwidth",	WAttribute_Width},
	{"relattr",		WAttributed_Relationship},
	{"junctid",		WAvailable_Junction_Feature_Class},
	{"junctst",		WAvailable_Junction_Subtype},
	{"rulejunc",	WAvailable_Junctions},
	{"rastnodt",	WBackground_Nodata_Value},
	{"behrman",		WBehrmann},
	{"Binary",		WBinary_Enclosures},
	{"rastbpp",		WBits_Per_Pixel},
	{"bonne",		WBonne},
	{"bottombc",	WBottom_Bounding_Coordinate},
	{"browseem",	WBrowse_Graphic_Embedded},
	{"cassini",		WCassini},
	{"rastxsz",		WCell_Size_X_Direction},
	{"rastxu",		WCell_Size_X_Units},
	{"rastysz",		WCell_Size_Y_Direction},
	{"rastyu",		WCell_Size_Y_Units},
	{"relcomp",		WComposite_Relationship},
	{"rastcomp",	WCompression_Type},
	{"connrule",	WConnectivity_Rule},
	{"cordsysn",	WCoordinate_System_Name},
	{"CreaDate",	WCreation_Date},
	{"CreaTime",	WCreation_Time},
	{"Data",		WData},
	{"framect",		WData_Frame_Count},
	{"framenam",	WData_Frame_Name},
	{"eframes",		WData_Frames},
	{"database",	WDatabase_Name},
	{"dssize",		WDataset_Size},
	{"ruledjid",	WDefault_Junction_Feature_Class},
	{"ruledjst",	WDefault_Junction_Subtype},
	{"assndesc",	WDescription_of_Association},
	{"Descript",	WDescription_of_Enclosure},
	{"reldesc",		WDescription_of_Relationship},
	{"dtfcfkey",	WDestination_Foreign_Key},
	{"dtfcname",	WDestination_Name},
	{"dtfcpkey",	WDestination_Primary_Key},
	{"domfldtp",	WDomain_Attribute_Type},
	{"domdesc",		WDomain_Description},
	{"domname",		WDomain_Name},
	{"domowner",	WDomain_Owner},
	{"domtype",		WDomain_Type},
	{"eckert1",		WEckert_I},
	{"eckert2",		WEckert_II},
	{"eckert3",		WEckert_III},
	{"eckert4",		WEckert_IV},
	{"eckert5",		WEckert_V},
	{"eckert6",		WEckert_VI},
	{"ruleeid",		WEdge_Feature_Class},
	{"ruleemxc",	WEdge_Maximum_Cardinality},
	{"ruleemnc",	WEdge_Minimum_Cardinality},
	{"ruleest",		WEdge_Subtype},
	{"enabfld",		WEnabled_Attribute},
	{"Enclosure",	WEnclosure},
	{"enttypc",		WEntity_Type_Count},
	{"enttypt",		WEntity_Type_Type},
	{"efeacnt",		WESRI_Feature_Count},
	{"efeageom",	WESRI_Feature_Geometry},
	{"efeatyp",		WESRI_Feature_Type},
	{"Esri",		WESRI_Information},
	{"esriterm",	WESRI_Terms_Description},
	{"esritopo",	WESRI_Topology},
	{"featdesc",	WFeature_Description},
	{"ftname",		WFile_or_Table_Name},
	{"rulefeid",	WFrom_Edge_Feature_Class},
	{"rulefest",	WFrom_Edge_Subtype},
	{"gallster",	WGall_Stereographic},
	{"RematchLocator",	WGeocoding_Information},
	{"geogcsn",		WGeographic_Coordinate_System_Name},
	{"netinfo",		WGeometric_Network_Information},
	{"rastcmap",	WImage_Colormap},
	{"rastifor",	WImage_Format},
	{"rastityp",	WImage_Type},
	{"instance",	WInstance_Name},
	{"rulejid",		WJunction_Feature_Class},
	{"rulejmxc",	WJunction_Maximum_Cardinality},
	{"rulejmnc",	WJunction_Minimum_Cardinality},
	{"rulejst",		WJunction_Subtype},
	{"langdata",	WLanguage_of_Dataset},
	{"langmeta",	WLanguage_of_Metadata},
	{"leftbc",		WLeft_Bounding_Coordinate},
	{"linrefer",	WLinear_Referencing},
	{"lboundng",	WLocal_Bounding_Coordinates},
	{"loximuth",	WLoximuthal},
	{"maxalti",		WMaximum_Altitude},
	{"mrgtype",		WMerge_Rule},
	{"MetaID",		WMetadata_ID},
	{"minalti",		WMinimum_Altitude},
	{"ModDate",		WModification_Date},
	{"ModTime",		WModification_Time},
	{"mollweid",	WMollweide},
	{"natvform",	WNative_Dataset_Format},
	{"elemcls",		WNetwork_Element},
	{"netwrole",	WNetwork_Role},
	{"nettype",		WNetwork_Type},
	{"relnodir",	WNotification_Direction},
	{"rastband",	WNumber_of_Raster_Bands},
	{"otfcfkey",	WOrigin_Foreign_Key},
	{"otfcname",	WOrigin_Name},
	{"otfcpkey",	WOrigin_Primary_Key},
	{"procsv",		WProcess_Software_and_Version},
	{"projcsn",		WProjected_Coordinate_System_Name},
	{"rastplyr",	WPyramid_Layers},
	{"quartic",		WQuartic_Authalic},
	{"rdommean",	WRange_Domain_Mean},
	{"rdomstdv",	WRange_Domain_Standard_Deviation},
	{"rastdtyp",	WRaster_Display_Type},
	{"rastorig",	WRaster_Origin},
	{"relblab",		WRelationship_Backward_Label},
	{"relcard",		WRelationship_Cardinality},
	{"relflab",		WRelationship_Forward_Label},
	{"relinfo",		WRelationship_Information},
	{"rightbc",		WRight_Bounding_Coordinate},
	{"rulecat",		WRule_Category},
	{"rulehelp",	WRule_Help},
	{"ruletype",	WRule_Type},
	{"sdeconn",		WSDE_Connection_Information},
	{"server",		WServer_Name},
	{"spindex",		WSpatial_Index},
	{"splttype",	WSplit_Rule},
	{"stfield",		WSubtype_Attribute},
	{"stfldnm",		WSubtype_Attribute_Name},
	{"stcode",		WSubtype_Code},
	{"stflddv",		WSubtype_Default_Value},
	{"subtype",		WSubtype_Information},
	{"stname",		WSubtype_Name},
	{"SyncDate",	WSynchronization_Date},
	{"SyncTime",	WSynchronization_Time},
	{"Sync",		WSynchronize_Automatically},
	{"SyncOnce",	WSynchronize_Once},
	{"Thumbnail",	WThumbnail},
	{"ruleteid",	WTo_Edge_Feature_Class},
	{"ruletest",	WTo_Edge_Subtype},
	{"topbc",		WTop_Bounding_Coordinate},
	{"user",		WUser_Name},
	{"version",		WVersion_Name},
	{"winkel1",		WWinkel_I},
	{"winkel2",		WWinkel_II},
	{"geoBox",		WGeographic_Box},

#endif

	{NULL,			Wnull}
	};
static int ps8_tag_table_length = 0;

static int compare (const void *e1, const void *e2) {
	struct ps8_tag *p = (struct ps8_tag *) e1;
	struct ps8_tag *q = (struct ps8_tag *) e2;
	return (p->key - q->key);
	}

/*----------------------------------------------------------------------*/

const char *ps8_tag_of (enum fgdc_keyword key) {
	int i;
	struct ps8_tag T,*pT;

	if (ps8_tag_table_length == 0) {
		for (i=0; ps8_tag_table[i].tag; i++);
		ps8_tag_table_length = i;
		qsort (ps8_tag_table,ps8_tag_table_length,sizeof(struct ps8_tag),compare);
		}

	if (key == Wnull) return ("");
	if (key == Wunknown) return ("");

	if (is_extension (key)) return (sgml_of_extension (key));

	T.key = key;
	pT = (struct ps8_tag *) bsearch (&T,ps8_tag_table,ps8_tag_table_length,sizeof(struct ps8_tag),compare);
	if (pT) return (pT->tag);
	else return ("");
	}

/*----------------------------------------------------------------------*/

enum fgdc_keyword key_of_ps8_tag (const char *ps8_tag) {
	int i;

	if (ps8_tag_table_length == 0) {
		for (i=0; ps8_tag_table[i].tag; i++);
		ps8_tag_table_length = i;
		qsort (ps8_tag_table,ps8_tag_table_length,sizeof(struct ps8_tag),compare);
		}

	for (i=0; i < ps8_tag_table_length; i++)
		if (strcmp (ps8_tag_table[i].tag,ps8_tag) == 0) return (ps8_tag_table[i].key);

	i = extension_of_sgml (ps8_tag);

	if (i == Unrecognized_Extension) return (Wunknown);
	else return (i);
	}

/*----------------------------------------------------------------------*\
\*----------------------------------------------------------------------*/

#ifdef PRINT_TAGS

char *read_text (char *name) {
	}

int main (int argc, char *argv[]) {
	struct ps8_tag *p;

	for (p=ps8_tag_table; p->tag; p++) {
		printf ("%s\t%s\n",p->tag,text_of (p->key));
		}
	}

#endif

/*----------------------------------------------------------------------*\
\*----------------------------------------------------------------------*/
