/*----------------------------------------------------------------------*\
 | Tips: Short one-line text for each element							|
\*----------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "keyword.h"

#define MAX_COMPOUND_COUNT	255
#define MAX_COMPONENT_COUNT	31

struct tip_t {
	enum fgdc_keyword parent;
	struct {
		enum fgdc_keyword key;
		char *text;
		} child [1 + MAX_COMPONENT_COUNT];
	};

static struct tip_t default_tip [1 + MAX_COMPOUND_COUNT] = {
	{WMetadata,{
		{WIdentification_Information,
			"Mandatory"},
		{WData_Quality_Information,
			"Mandatory if applicable"},
		{WSpatial_Data_Organization_Information,
			"Mandatory if applicable"},
		{WSpatial_Reference_Information,
			"Mandatory if applicable"},
		{WEntity_and_Attribute_Information,
			"Mandatory if applicable"},
		{WDistribution_Information,
			"Mandatory if applicable, Repeatable"},
		{WMetadata_Reference_Information,
			"Mandatory"},
		{Wnull,""}
		}},

	{WIdentification_Information,{
		{WCitation,
			"Mandatory"},
		{WDescription,
			"Mandatory"},
		{WTime_Period_of_Content,
			"Mandatory"},
		{WStatus,
			"Mandatory"},
		{WSpatial_Domain,
			"Mandatory"},
		{WKeywords,
			"Mandatory"},
		{WAccess_Constraints,
			"Mandatory"},
		{WUse_Constraints,
			"Mandatory"},
		{WPoint_of_Contact,
			"Optional"},
		{WBrowse_Graphic,
			"Optional, Repeatable"},
		{WData_Set_Credit,
			"Optional"},
		{WSecurity_Information,
			"Optional"},
		{WNative_Data_Set_Environment,
			"Optional"},
		{WCross_Reference,
			"Optional, Repeatable"},
		{Wnull,""}
		}},

	{WCitation,{
		{WCitation_Information,
			"Mandatory"},
		{Wnull,""}
		}},

	{WDescription,{
		{WAbstract,
			"Mandatory"},
		{WPurpose,
			"Mandatory"},
		{WSupplemental_Information,
			"Optional"},
		{Wnull,""}
		}},

	{WTime_Period_of_Content,{
		{WTime_Period_Information,
			"Mandatory"},
		{WCurrentness_Reference,
			"Mandatory"},
		{Wnull,""}
		}},

	{WStatus,{
		{WProgress,
			"Mandatory"},
		{WMaintenance_and_Update_Frequency,
			"Mandatory"},
		{Wnull,""}
		}},

	{WSpatial_Domain,{
		{WBounding_Coordinates,
			"Mandatory"},
		{WData_Set_G_Polygon,
			"Optional, Repeatable"},
		{Wnull,""}
		}},

	{WBounding_Coordinates,{
		{WWest_Bounding_Coordinate,
			"Mandatory"},
		{WEast_Bounding_Coordinate,
			"Mandatory"},
		{WNorth_Bounding_Coordinate,
			"Mandatory"},
		{WSouth_Bounding_Coordinate,
			"Mandatory"},
		{Wnull,""}
		}},

	{WData_Set_G_Polygon,{
		{WData_Set_G_Polygon_Outer_G_Ring,
			"Mandatory"},
		{WData_Set_G_Polygon_Exclusion_G_Ring,
			"Optional, Repeatable"},
		{Wnull,""}
		}},

	{WData_Set_G_Polygon_Outer_G_Ring,{
		{WG_Ring_Latitude,
			"Mandatory with G-Ring_Longitude in pairs, at least 4 pairs"},
		{WG_Ring_Longitude,
			"Mandatory with G-Ring_Latitude in pairs, at least 4 pairs"},
		{Wnull,""}
		}},

	{WData_Set_G_Polygon_Exclusion_G_Ring,{
		{WG_Ring_Latitude,
			"Mandatory with G-Ring_Longitude in pairs, at least 4 pairs"},
		{WG_Ring_Longitude,
			"Mandatory with G-Ring_Latitude in pairs, at least 4 pairs"},
		{Wnull,""}
		}},

	{WKeywords,{
		{WTheme,
			"Mandatory, Repeatable"},
		{WPlace,
			"Optional, Repeatable"},
		{WStratum,
			"Optional, Repeatable"},
		{WTemporal,
			"Optional, Repeatable"},
		{Wnull,""}
		}},

	{WTheme,{
		{WTheme_Keyword_Thesaurus,
			"Mandatory"},
		{WTheme_Keyword,
			"Mandatory, Repeatable"},
		{Wnull,""}
		}},

	{WPlace,{
		{WPlace_Keyword_Thesaurus,
			"Mandatory"},
		{WPlace_Keyword,
			"Mandatory, Repeatable"},
		{Wnull,""}
		}},

	{WStratum,{
		{WStratum_Keyword_Thesaurus,
			"Mandatory"},
		{WStratum_Keyword,
			"Mandatory, Repeatable"},
		{Wnull,""}
		}},

	{WTemporal,{
		{WTemporal_Keyword_Thesaurus,
			"Mandatory"},
		{WTemporal_Keyword,
			"Mandatory, Repeatable"},
		{Wnull,""}
		}},

	{WPoint_of_Contact,{
		{WContact_Information,
			"Mandatory"},
		{Wnull,""}
		}},

	{WBrowse_Graphic,{
		{WBrowse_Graphic_File_Name,
			"Mandatory"},
		{WBrowse_Graphic_File_Description,
			"Mandatory"},
		{WBrowse_Graphic_File_Type,
			"Mandatory"},
		{Wnull,""}
		}},

	{WSecurity_Information,{
		{WSecurity_Classification_System,
			"Mandatory"},
		{WSecurity_Classification,
			"Mandatory"},
		{WSecurity_Handling_Description,
			"Mandatory"},
		{Wnull,""}
		}},

	{WCross_Reference,{
		{WCitation_Information,
			"Mandatory"},
		{Wnull,""}
		}},

	{WData_Quality_Information,{
		{WAttribute_Accuracy,
			"Mandatory if applicable"},
		{WLogical_Consistency_Report,
			"Mandatory"},
		{WCompleteness_Report,
			"Mandatory"},
		{WPositional_Accuracy,
			"Mandatory if applicable"},
		{WLineage,
			"Mandatory"},
		{WCloud_Cover,
			"Optional"},
		{Wnull,""}
		}},

	{WAttribute_Accuracy,{
		{WAttribute_Accuracy_Report,
			"Mandatory"},
		{WQuantitative_Attribute_Accuracy_Assessment,
			"Optional, Repeatable"},
		{Wnull,""}
		}},

	{WQuantitative_Attribute_Accuracy_Assessment,{
		{WAttribute_Accuracy_Value,
			"Mandatory"},
		{WAttribute_Accuracy_Explanation,
			"Mandatory"},
		{Wnull,""}
		}},

	{WPositional_Accuracy,{
		{WHorizontal_Positional_Accuracy,
			"Mandatory if applicable"},
		{WVertical_Positional_Accuracy,
			"Mandatory if applicable"},
		{Wnull,""}
		}},

	{WHorizontal_Positional_Accuracy,{
		{WHorizontal_Positional_Accuracy_Report,
			"Mandatory"},
		{WQuantitative_Horizontal_Positional_Accuracy_Assessment,
			"Optional, Repeatable"},
		{Wnull,""}
		}},

	{WQuantitative_Horizontal_Positional_Accuracy_Assessment,{
		{WHorizontal_Positional_Accuracy_Value,
			"Mandatory"},
		{WHorizontal_Positional_Accuracy_Explanation,
			"Mandatory"},
		{Wnull,""}
		}},

	{WVertical_Positional_Accuracy,{
		{WVertical_Positional_Accuracy_Report,
			"Mandatory"},
		{WQuantitative_Vertical_Positional_Accuracy_Assessment,
			"Optional, Repeatable"},
		{Wnull,""}
		}},

	{WQuantitative_Vertical_Positional_Accuracy_Assessment,{
		{WVertical_Positional_Accuracy_Value,
			"Mandatory"},
		{WVertical_Positional_Accuracy_Explanation,
			"Mandatory"},
		{Wnull,""}
		}},

	{WLineage,{
		{WSource_Information,
			"Mandatory if applicable, Repeatable"},
		{WProcess_Step,
			"Mandatory, Repeatable"},
		{Wnull,""}
		}},

	{WSource_Information,{
		{WSource_Citation,
			"Mandatory"},
		{WSource_Scale_Denominator,
			"Mandatory if applicable"},
		{WType_of_Source_Media,
			"Mandatory"},
		{WSource_Time_Period_of_Content,
			"Mandatory"},
		{WSource_Citation_Abbreviation,
			"Mandatory"},
		{WSource_Contribution,
			"Mandatory"},
		{Wnull,""}
		}},

	{WSource_Citation,{
		{WCitation_Information,
			"Mandatory"},
		{Wnull,""}
		}},

	{WSource_Time_Period_of_Content,{
		{WTime_Period_Information,
			"Mandatory"},
		{WSource_Currentness_Reference,
			"Mandatory"},
		{Wnull,""}
		}},

	{WProcess_Step,{
		{WProcess_Description,
			"Mandatory"},
		{WSource_Used_Citation_Abbreviation,
			"Mandatory if applicable, Repeatable"},
		{WProcess_Date,
			"Mandatory"},
		{WProcess_Time,
			"Optional"},
		{WSource_Produced_Citation_Abbreviation,
			"Mandatory if applicable, Repeatable"},
		{WProcess_Contact,
			"Optional"},
		{Wnull,""}
		}},

	{WProcess_Contact,{
		{WContact_Information,
			"Mandatory"},
		{Wnull,""}
		}},

	{WSpatial_Data_Organization_Information,{
		{WIndirect_Spatial_Reference,
			"Mandatory if applicable"},
		{WDirect_Spatial_Reference_Method,
			"Mandatory if applicable"},
		{WPoint_and_Vector_Object_Information,
			"Optional; choose this or Raster_Object_Information"},
		{WRaster_Object_Information,
			"Optional; choose this or Point_and_Vector_Object_Information"},
		{Wnull,""}
		}},

	{WPoint_and_Vector_Object_Information,{
		{WSDTS_Terms_Description,
			"Mandatory, Repeatable; choose this or VPF_Terms_Description"},
		{WVPF_Terms_Description,
			"Mandatory; choose this or SDTS_Terms_Description"},
		{Wnull,""}
		}},

	{WSDTS_Terms_Description,{
		{WSDTS_Point_and_Vector_Object_Type,
			"Mandatory"},
		{WPoint_and_Vector_Object_Count,
			"Optional"},
		{Wnull,""}
		}},

	{WVPF_Terms_Description,{
		{WVPF_Topology_Level,
			"Mandatory"},
		{WVPF_Point_and_Vector_Object_Information,
			"Mandatory, Repeatable"},
		{Wnull,""}
		}},

	{WVPF_Point_and_Vector_Object_Information,{
		{WVPF_Point_and_Vector_Object_Type,
			"Mandatory"},
		{WPoint_and_Vector_Object_Count,
			"Optional"},
		{Wnull,""}
		}},

	{WRaster_Object_Information,{
		{WRaster_Object_Type,
			"Mandatory"},
		{WRow_Count,
			"Optional; if used requires Column_Count as well"},
		{WColumn_Count,
			"Optional; if used requires Row_Count as well"},
		{WVertical_Count,
			"Optional; Mandatory if applicable and Row_Count is used"},
		{Wnull,""}
		}},

	{WSpatial_Reference_Information,{
		{WHorizontal_Coordinate_System_Definition,
			"Mandatory if applicable"},
		{WVertical_Coordinate_System_Definition,
			"Mandatory if applicable"},
		{Wnull,""}
		}},

	{WHorizontal_Coordinate_System_Definition,{
		{WGeographic,
			"Mandatory; choose this, Planar, or Local"},
		{WPlanar,
			"Mandatory, Repeatable; choose this, Geographic, or Local"},
		{WLocal,
			"Mandatory; choose this, Geographic, or Planar"},
		{WGeodetic_Model,
			"Mandatory if applicable"},
		{Wnull,""}
		}},

	{WGeographic,{
		{WLatitude_Resolution,
			"Mandatory"},
		{WLongitude_Resolution,
			"Mandatory"},
		{WGeographic_Coordinate_Units,
			"Mandatory"},
		{Wnull,""}
		}},

	{WPlanar,{
		{WMap_Projection,
			"Mandatory; choose this, Grid_Coordinate_System, or Local_Planar"},
		{WGrid_Coordinate_System,
			"Mandatory; choose this, Map_Projection, or Local_Planar"},
		{WLocal_Planar,
			"Mandatory; choose this, Map_Projection, or Grid_Coordinate_System"},
		{WPlanar_Coordinate_Information,
			"Mandatory"},
		{Wnull,""}
		}},

	{WMap_Projection,{
		{WMap_Projection_Name,
			"Mandatory"},
		{WAlbers_Conical_Equal_Area,
			"Mandatory; choose this or another map projection for the parameters"},
		{WAzimuthal_Equidistant,
			"Mandatory; choose this or another map projection for the parameters"},
		{WEquidistant_Conic,
			"Mandatory; choose this or another map projection for the parameters"},
		{WEquirectangular,
			"Mandatory; choose this or another map projection for the parameters"},
		{WGeneral_Vertical_Near_sided_Perspective,
			"Mandatory; choose this or another map projection for the parameters"},
		{WGnomonic,
			"Mandatory; choose this or another map projection for the parameters"},
		{WLambert_Azimuthal_Equal_Area,
			"Mandatory; choose this or another map projection for the parameters"},
		{WLambert_Conformal_Conic,
			"Mandatory; choose this or another map projection for the parameters"},
		{WMercator,
			"Mandatory; choose this or another map projection for the parameters"},
		{WModified_Stereographic_for_Alaska,
			"Mandatory; choose this or another map projection for the parameters"},
		{WMiller_Cylindrical,
			"Mandatory; choose this or another map projection for the parameters"},
		{WOblique_Mercator,
			"Mandatory; choose this or another map projection for the parameters"},
		{WOrthographic,
			"Mandatory; choose this or another map projection for the parameters"},
		{WPolar_Stereographic,
			"Mandatory; choose this or another map projection for the parameters"},
		{WPolyconic,
			"Mandatory; choose this or another map projection for the parameters"},
		{WRobinson,
			"Mandatory; choose this or another map projection for the parameters"},
		{WSinusoidal,
			"Mandatory; choose this or another map projection for the parameters"},
		{WSpace_Oblique_Mercator_Landsat,
			"Mandatory; choose this or another map projection for the parameters"},
		{WStereographic,
			"Mandatory; choose this or another map projection for the parameters"},
		{WTransverse_Mercator,
			"Mandatory; choose this or another map projection for the parameters"},
		{Wvan_der_Grinten,
			"Mandatory; choose this or another map projection for the parameters"},
		{WOther_Projections_Definition,
			"Mandatory; choose this or another map projection for the parameters"},
		{WMap_Projection_Parameters,
			"Mandatory; choose this or a specific map projection for the parameters"},
		{Wnull,""}
		}},

	{WAlbers_Conical_Equal_Area,{
		{WStandard_Parallel,
			"Mandatory, two may be specified"},
		{WLongitude_of_Central_Meridian,
			"Mandatory"},
		{WLatitude_of_Projection_Origin,
			"Mandatory"},
		{WFalse_Easting,
			"Mandatory"},
		{WFalse_Northing,
			"Mandatory"},
		{Wnull,""}
		}},

	{WAzimuthal_Equidistant,{
		{WLongitude_of_Central_Meridian,
			"Mandatory"},
		{WLatitude_of_Projection_Origin,
			"Mandatory"},
		{WFalse_Easting,
			"Mandatory"},
		{WFalse_Northing,
			"Mandatory"},
		{Wnull,""}
		}},

	{WEquidistant_Conic,{
		{WStandard_Parallel,
			"Mandatory, two may be specified"},
		{WLongitude_of_Central_Meridian,
			"Mandatory"},
		{WLatitude_of_Projection_Origin,
			"Mandatory"},
		{WFalse_Easting,
			"Mandatory"},
		{WFalse_Northing,
			"Mandatory"},
		{Wnull,""}
		}},

	{WEquirectangular,{
		{WStandard_Parallel,
			"Mandatory"},
		{WLongitude_of_Central_Meridian,
			"Mandatory"},
		{WFalse_Easting,
			"Mandatory"},
		{WFalse_Northing,
			"Mandatory"},
		{Wnull,""}
		}},

	{WGeneral_Vertical_Near_sided_Perspective,{
		{WHeight_of_Perspective_Point_Above_Surface,
			"Mandatory"},
		{WLongitude_of_Projection_Center,
			"Mandatory"},
		{WLatitude_of_Projection_Center,
			"Mandatory"},
		{WFalse_Easting,
			"Mandatory"},
		{WFalse_Northing,
			"Mandatory"},
		{Wnull,""}
		}},

	{WGnomonic,{
		{WLongitude_of_Projection_Center,
			"Mandatory"},
		{WLatitude_of_Projection_Center,
			"Mandatory"},
		{WFalse_Easting,
			"Mandatory"},
		{WFalse_Northing,
			"Mandatory"},
		{Wnull,""}
		}},

	{WLambert_Azimuthal_Equal_Area,{
		{WLongitude_of_Projection_Center,
			"Mandatory"},
		{WLatitude_of_Projection_Center,
			"Mandatory"},
		{WFalse_Easting,
			"Mandatory"},
		{WFalse_Northing,
			"Mandatory"},
		{Wnull,""}
		}},

	{WLambert_Conformal_Conic,{
		{WStandard_Parallel,
			"Mandatory, two may be specified"},
		{WLongitude_of_Central_Meridian,
			"Mandatory"},
		{WLatitude_of_Projection_Origin,
			"Mandatory"},
		{WFalse_Easting,
			"Mandatory"},
		{WFalse_Northing,
			"Mandatory"},
		{Wnull,""}
		}},

	{WMercator,{
		{WStandard_Parallel,
			"Mandatory; choose this or Scale_Factor_at_Equator"},
		{WScale_Factor_at_Equator,
			"Mandatory; choose this or Standard_Parallel"},
		{WLongitude_of_Central_Meridian,
			"Mandatory"},
		{WFalse_Easting,
			"Mandatory"},
		{WFalse_Northing,
			"Mandatory"},
		{Wnull,""}
		}},

	{WModified_Stereographic_for_Alaska,{
		{WFalse_Easting,
			"Mandatory"},
		{WFalse_Northing,
			"Mandatory"},
		{Wnull,""}
		}},

	{WMiller_Cylindrical,{
		{WLongitude_of_Central_Meridian,
			"Mandatory"},
		{WFalse_Easting,
			"Mandatory"},
		{WFalse_Northing,
			"Mandatory"},
		{Wnull,""}
		}},

	{WOblique_Mercator,{
		{WScale_Factor_at_Center_Line,
			"Mandatory"},
		{WOblique_Line_Azimuth,
			"Mandatory; choose this or Oblique_Line_Point"},
		{WOblique_Line_Point,
			"Mandatory; choose this or Oblique_Line_Azimuth"},
		{WLatitude_of_Projection_Origin,
			"Mandatory"},
		{WFalse_Easting,
			"Mandatory"},
		{WFalse_Northing,
			"Mandatory"},
		{Wnull,""}
		}},

	{WOblique_Line_Azimuth,{
		{WAzimuthal_Angle,
			"Mandatory"},
		{WAzimuth_Measure_Point_Longitude,
			"Mandatory"},
		{Wnull,""}
		}},

	{WOblique_Line_Point,{
		{WOblique_Line_Latitude,
			"Mandatory; two are required, each with one Oblique_Line_Longitude"},
		{WOblique_Line_Longitude,
			"Mandatory; two are required, each with one Oblique_Line_Latitude"},
		{Wnull,""}
		}},

	{WOrthographic,{
		{WLongitude_of_Projection_Center,
			"Mandatory"},
		{WLatitude_of_Projection_Center,
			"Mandatory"},
		{WFalse_Easting,
			"Mandatory"},
		{WFalse_Northing,
			"Mandatory"},
		{Wnull,""}
		}},

	{WPolar_Stereographic,{
		{WStraight_Vertical_Longitude_from_Pole,
			"Mandatory"},
		{WStandard_Parallel,
			"Mandatory; choose this or Scale_Factor_at_Projection_Origin"},
		{WScale_Factor_at_Projection_Origin,
			"Mandatory; choose this or Standard_Parallel"},
		{WFalse_Easting,
			"Mandatory"},
		{WFalse_Northing,
			"Mandatory"},
		{Wnull,""}
		}},

	{WPolyconic,{
		{WLongitude_of_Central_Meridian,
			"Mandatory"},
		{WLatitude_of_Projection_Origin,
			"Mandatory"},
		{WFalse_Easting,
			"Mandatory"},
		{WFalse_Northing,
			"Mandatory"},
		{Wnull,""}
		}},

	{WRobinson,{
		{WLongitude_of_Projection_Center,
			"Mandatory"},
		{WFalse_Easting,
			"Mandatory"},
		{WFalse_Northing,
			"Mandatory"},
		{Wnull,""}
		}},

	{WSinusoidal,{
		{WLongitude_of_Central_Meridian,
			"Mandatory"},
		{WFalse_Easting,
			"Mandatory"},
		{WFalse_Northing,
			"Mandatory"},
		{Wnull,""}
		}},

	{WSpace_Oblique_Mercator_Landsat,{
		{WLandsat_Number,
			"Mandatory"},
		{WPath_Number,
			"Mandatory"},
		{WFalse_Easting,
			"Mandatory"},
		{WFalse_Northing,
			"Mandatory"},
		{Wnull,""}
		}},

	{WStereographic,{
		{WLongitude_of_Projection_Center,
			"Mandatory"},
		{WLatitude_of_Projection_Center,
			"Mandatory"},
		{WFalse_Easting,
			"Mandatory"},
		{WFalse_Northing,
			"Mandatory"},
		{Wnull,""}
		}},

	{WTransverse_Mercator,{
		{WScale_Factor_at_Central_Meridian,
			"Mandatory"},
		{WLongitude_of_Central_Meridian,
			"Mandatory"},
		{WLatitude_of_Projection_Origin,
			"Mandatory"},
		{WFalse_Easting,
			"Mandatory"},
		{WFalse_Northing,
			"Mandatory"},
		{Wnull,""}
		}},

	{Wvan_der_Grinten,{
		{WLongitude_of_Central_Meridian,
			"Mandatory"},
		{WFalse_Easting,
			"Mandatory"},
		{WFalse_Northing,
			"Mandatory"},
		{Wnull,""}
		}},

	{WMap_Projection_Parameters,{
		{WAzimuth_Measure_Point_Longitude,
			"Mandatory if applicable to your projection"},
		{WAzimuthal_Angle,
			"Mandatory if applicable to your projection"},
		{WFalse_Easting,
			"Mandatory if applicable to your projection"},
		{WFalse_Northing,
			"Mandatory if applicable to your projection"},
		{WHeight_of_Perspective_Point_Above_Surface,
			"Mandatory if applicable to your projection"},
		{WLandsat_Number,
			"Mandatory if applicable to your projection"},
		{WLatitude_of_Projection_Center,
			"Mandatory if applicable to your projection"},
		{WLatitude_of_Projection_Origin,
			"Mandatory if applicable to your projection"},
		{WLongitude_of_Central_Meridian,
			"Mandatory if applicable to your projection"},
		{WLongitude_of_Projection_Center,
			"Mandatory if applicable to your projection"},
		{WOblique_Line_Latitude,
			"Mandatory if applicable to your projection, Repeatable"},
		{WOblique_Line_Longitude,
			"Mandatory if applicable to your projection, Repeatable"},
		{WPath_Number,
			"Mandatory if applicable to your projection"},
		{WScale_Factor_at_Center_Line,
			"Mandatory if applicable to your projection"},
		{WScale_Factor_at_Central_Meridian,
			"Mandatory if applicable to your projection"},
		{WScale_Factor_at_Equator,
			"Mandatory if applicable to your projection"},
		{WScale_Factor_at_Projection_Origin,
			"Mandatory if applicable to your projection"},
		{WStandard_Parallel,
			"Mandatory if applicable to your projection, Repeatable"},
		{WStraight_Vertical_Longitude_from_Pole,
			"Mandatory if applicable to your projection"},
		{Wnull,""}
		}},

	{WGrid_Coordinate_System,{
		{WGrid_Coordinate_System_Name,
			"Mandatory"},
		{WUniversal_Transverse_Mercator,
			"Mandatory; choose this or one of the other coordinate systems"},
		{WUniversal_Polar_Stereographic,
			"Mandatory; choose this or one of the other coordinate systems"},
		{WState_Plane_Coordinate_System,
			"Mandatory; choose this or one of the other coordinate systems"},
		{WARC_Coordinate_System,
			"Mandatory; choose this or one of the other coordinate systems"},
		{WOther_Grid_Systems_Definition,
			"Mandatory; choose this or one of the other coordinate systems"},
		{Wnull,""}
		}},

	{WUniversal_Transverse_Mercator,{
		{WUTM_Zone_Number,
			"Mandatory"},
		{WTransverse_Mercator,
			"Mandatory"},
		{Wnull,""}
		}},

	{WUniversal_Polar_Stereographic,{
		{WUPS_Zone_Identifier,
			"Mandatory"},
		{WPolar_Stereographic,
			"Mandatory"},
		{Wnull,""}
		}},

	{WState_Plane_Coordinate_System,{
		{WSPCS_Zone_Identifier,
			"Mandatory"},
		{WLambert_Conformal_Conic,
			"Mandatory; choose this, Transverse_Mercator, Oblique_Mercator, or Polyconic"},
		{WTransverse_Mercator,
			"Mandatory; choose this, Lambert_Conformal_Conic, Oblique_Mercator, or Polyconic"},
		{WOblique_Mercator,
			"Mandatory; choose this, Lambert_Conformal_Conic, Transverse_Mercator, or Polyconic"},
		{WPolyconic,
			"Mandatory; choose this, Lambert_Conformal_Conic, Transverse_Mercator, or Oblique_Mercator"},
		{Wnull,""}
		}},

	{WARC_Coordinate_System,{
		{WARC_System_Zone_Identifier,
			"Mandatory"},
		{WEquirectangular,
			"Mandatory; choose this or Azimuthal_Equidistant"},
		{WAzimuthal_Equidistant,
			"Mandatory; choose this or Equirectangular"},
		{Wnull,""}
		}},

	{WLocal_Planar,{
		{WLocal_Planar_Description,
			"Mandatory"},
		{WLocal_Planar_Georeference_Information,
			"Mandatory"},
		{Wnull,""}
		}},

	{WPlanar_Coordinate_Information,{
		{WPlanar_Coordinate_Encoding_Method,
			"Mandatory"},
		{WCoordinate_Representation,
			"Mandatory; choose this or Distance_and_Bearing_Representation"},
		{WDistance_and_Bearing_Representation,
			"Mandatory; choose this or Coordinate_Representation"},
		{WPlanar_Distance_Units,
			"Mandatory"},
		{Wnull,""}
		}},

	{WCoordinate_Representation,{
		{WAbscissa_Resolution,
			"Mandatory"},
		{WOrdinate_Resolution,
			"Mandatory"},
		{Wnull,""}
		}},

	{WDistance_and_Bearing_Representation,{
		{WDistance_Resolution,
			"Mandatory"},
		{WBearing_Resolution,
			"Mandatory"},
		{WBearing_Units,
			"Mandatory"},
		{WBearing_Reference_Direction,
			"Mandatory"},
		{WBearing_Reference_Meridian,
			"Mandatory"},
		{Wnull,""}
		}},

	{WLocal,{
		{WLocal_Description,
			"Mandatory"},
		{WLocal_Georeference_Information,
			"Mandatory"},
		{Wnull,""}
		}},

	{WGeodetic_Model,{
		{WHorizontal_Datum_Name,
			"Mandatory if applicable"},
		{WEllipsoid_Name,
			"Mandatory"},
		{WSemi_major_Axis,
			"Mandatory"},
		{WDenominator_of_Flattening_Ratio,
			"Mandatory"},
		{Wnull,""}
		}},

	{WVertical_Coordinate_System_Definition,{
		{WAltitude_System_Definition,
			"Mandatory if applicable"},
		{WDepth_System_Definition,
			"Mandatory if applicable"},
		{Wnull,""}
		}},

	{WAltitude_System_Definition,{
		{WAltitude_Datum_Name,
			"Mandatory"},
		{WAltitude_Resolution,
			"Mandatory, Repeatable"},
		{WAltitude_Distance_Units,
			"Mandatory"},
		{WAltitude_Encoding_Method,
			"Mandatory"},
		{Wnull,""}
		}},

	{WDepth_System_Definition,{
		{WDepth_Datum_Name,
			"Mandatory"},
		{WDepth_Resolution,
			"Mandatory, Repeatable"},
		{WDepth_Distance_Units,
			"Mandatory"},
		{WDepth_Encoding_Method,
			"Mandatory"},
		{Wnull,""}
		}},

	{WEntity_and_Attribute_Information,{
		{WDetailed_Description,
			"Mandatory, Repeatable; use this or Overview_Description or both"},
		{WOverview_Description,
			"Mandatory, Repeatable; use this or Detailed_Description or both"},
		{Wnull,""}
		}},

	{WDetailed_Description,{
		{WEntity_Type,
			"Mandatory"},
		{WAttribute,
			"Mandatory if applicable, Repeatable"},
		{Wnull,""}
		}},

	{WEntity_Type,{
		{WEntity_Type_Label,
			"Mandatory"},
		{WEntity_Type_Definition,
			"Mandatory"},
		{WEntity_Type_Definition_Source,
			"Mandatory"},
		{Wnull,""}
		}},

	{WAttribute,{
		{WAttribute_Label,
			"Mandatory"},
		{WAttribute_Definition,
			"Mandatory"},
		{WAttribute_Definition_Source,
			"Mandatory"},
		{WAttribute_Domain_Values,
			"Mandatory, Repeatable"},
		{WBeginning_Date_of_Attribute_Values,
			"Optional, Repeatable; one required for each Ending_Date_of_Attribute_Values"},
		{WEnding_Date_of_Attribute_Values,
			"Mandatory if applicable for each Beginning_Date_of_Attribute_Values"},
		{WAttribute_Value_Accuracy_Information,
			"Optional"},
		{WAttribute_Measurement_Frequency,
			"Optional"},
		{Wnull,""}
		}},

	{WAttribute_Domain_Values,{
		{WEnumerated_Domain,
			"Mandatory, Repeatable; choose this, Range_Domain, Codeset_Domain, or Unrepresentable_Domain"},
		{WRange_Domain,
			"Mandatory; choose this, Enumerated_Domain, Codeset_Domain, or Unrepresentable_Domain"},
		{WCodeset_Domain,
			"Mandatory; choose this, Enumerated_Domain, Range_Domain, or Unrepresentable_Domain"},
		{WUnrepresentable_Domain,
			"Mandatory; choose this, Enumerated_Domain, Range_Domain, or Codeset_Domain"},
		{Wnull,""}
		}},

	{WEnumerated_Domain,{
		{WEnumerated_Domain_Value,
			"Mandatory"},
		{WEnumerated_Domain_Value_Definition,
			"Mandatory"},
		{WEnumerated_Domain_Value_Definition_Source,
			"Mandatory"},
		{WAttribute,
			"Mandatory if applicable, Repeatable"},
		{Wnull,""}
		}},

	{WRange_Domain,{
		{WRange_Domain_Minimum,
			"Mandatory"},
		{WRange_Domain_Maximum,
			"Mandatory"},
		{WAttribute_Units_of_Measure,
			"Mandatory if applicable"},
		{WAttribute_Measurement_Resolution,
			"Optional"},
		{WAttribute,
			"Mandatory if applicable, Repeatable"},
		{Wnull,""}
		}},

	{WCodeset_Domain,{
		{WCodeset_Name,
			"Mandatory"},
		{WCodeset_Source,
			"Mandatory"},
		{Wnull,""}
		}},

	{WAttribute_Value_Accuracy_Information,{
		{WAttribute_Value_Accuracy,
			"Mandatory"},
		{WAttribute_Value_Accuracy_Explanation,
			"Mandatory"},
		{Wnull,""}
		}},

	{WOverview_Description,{
		{WEntity_and_Attribute_Overview,
			"Mandatory, Repeatable"},
		{WEntity_and_Attribute_Detail_Citation,
			"Mandatory, Repeatable"},
		{Wnull,""}
		}},

	{WDistribution_Information,{
		{WDistributor,
			"Mandatory, Repeatable"},
		{WResource_Description,
			"Mandatory if applicable, one per Distributor"},
		{WDistribution_Liability,
			"Mandatory, one per Distributor"},
		{WStandard_Order_Process,
			"Mandatory if applicable, Repeatable"},
		{WCustom_Order_Process,
			"Mandatory if applicable; one per Distributor"},
		{WTechnical_Prerequisites,
			"Optional; one per Distributor"},
		{WAvailable_Time_Period,
			"Optional; one per Distributor"},
		{Wnull,""}
		}},

	{WDistributor,{
		{WContact_Information,
			"Mandatory"},
		{Wnull,""}
		}},

	{WStandard_Order_Process,{
		{WNon_digital_Form,
			"Mandatory; choose this or Digital_Form"},
		{WDigital_Form,
			"Mandatory, Repeatable; choose this or Non-digital_Form"},
		{WFees,
			"Mandatory"},
		{WOrdering_Instructions,
			"Optional"},
		{WTurnaround,
			"Optional"},
		{Wnull,""}
		}},

	{WDigital_Form,{
		{WDigital_Transfer_Information,
			"Mandatory"},
		{WDigital_Transfer_Option,
			"Mandatory"},
		{Wnull,""}
		}},

	{WDigital_Transfer_Information,{
		{WFormat_Name,
			"Mandatory"},
		{WFormat_Version_Number,
			"Optional; choose this or Format_Version_Date; Required with Format_Specification"},
		{WFormat_Version_Date,
			"Optional; choose this or Format_Version_Number; Required with Format_Specification"},
		{WFormat_Specification ,
			"Optional"},
		{WFormat_Information_Content,
			"Optional"},
		{WFile_Decompression_Technique,
			"Mandatory if applicable"},
		{WTransfer_Size,
			"Optional"},
		{Wnull,""}
		}},

	{WDigital_Transfer_Option,{
		{WOnline_Option,
			"Mandatory, Repeatable; choose this or Offline_Option"},
		{WOffline_Option,
			"Mandatory, Repeatable; choose this or Online_Option"},
		{Wnull,""}
		}},

	{WOnline_Option,{
		{WComputer_Contact_Information,
			"Mandatory, Repeatable"},
		{WAccess_Instructions,
			"Optional"},
		{WOnline_Computer_and_Operating_System,
			"Optional"},
		{Wnull,""}
		}},

	{WComputer_Contact_Information,{
		{WNetwork_Address,
			"Mandatory; choose this or Dialup_Instructions"},
		{WDialup_Instructions,
			"Mandatory; choose this or Network_Address"},
		{Wnull,""}
		}},

	{WNetwork_Address,{
		{WNetwork_Resource_Name,
			"Mandatory, Repeatable"},
		{Wnull,""}
		}},

	{WDialup_Instructions,{
		{WLowest_BPS,
			"Mandatory"},
		{WHighest_BPS,
			"Mandatory if applicable"},
		{WNumber_DataBits,
			"Mandatory"},
		{WNumber_StopBits,
			"Mandatory"},
		{WParity,
			"Mandatory"},
		{WCompression_Support,
			"Mandatory if applicable"},
		{WDialup_Telephone,
			"Mandatory, Repeatable"},
		{WDialup_File_Name,
			"Mandatory, Repeatable"},
		{Wnull,""}
		}},

	{WOffline_Option,{
		{WOffline_Media,
			"Mandatory"},
		{WRecording_Capacity,
			"Mandatory if applicable"},
		{WRecording_Format,
			"Mandatory, Repeatable"},
		{WCompatibility_Information,
			"Mandatory if applicable"},
		{Wnull,""}
		}},

	{WRecording_Capacity,{
		{WRecording_Density,
			"Mandatory, Repeatable"},
		{WRecording_Density_Units,
			"Mandatory"},
		{Wnull,""}
		}},

	{WAvailable_Time_Period,{
		{WTime_Period_Information,
			"Mandatory"},
		{Wnull,""}
		}},

	{WMetadata_Reference_Information,{
		{WMetadata_Date,
			"Mandatory"},
		{WMetadata_Review_Date,
			"Optional; required with Metadata_Future_Review_Date"},
		{WMetadata_Future_Review_Date,
			"Optional"},
		{WMetadata_Contact,
			"Mandatory"},
		{WMetadata_Standard_Name,
			"Mandatory"},
		{WMetadata_Standard_Version,
			"Mandatory"},
		{WMetadata_Time_Convention,
			"Mandatory if applicable"},
		{WMetadata_Access_Constraints,
			"Optional"},
		{WMetadata_Use_Constraints,
			"Optional"},
		{WMetadata_Security_Information,
			"Optional"},
		{WMetadata_Extensions,
			"Mandatory if applicable, Repeatable"},
		{WMetadata_Language,
			"Optional extension of the standard"},
		{Wnull,""}
		}},

	{WMetadata_Contact,{
		{WContact_Information,
			"Mandatory"},
		{Wnull,""}
		}},

	{WMetadata_Security_Information,{
		{WMetadata_Security_Classification_System,
			"Mandatory"},
		{WMetadata_Security_Classification,
			"Mandatory"},
		{WMetadata_Security_Handling_Description,
			"Mandatory"},
		{Wnull,""}
		}},

	{WMetadata_Extensions,{
		{WOnline_Linkage,
			"Mandatory if applicable, Repeatable"},
		{WProfile_Name,
			"Mandatory if applicable"},
		{Wnull,""}
		}},

	{WCitation_Information,{
		{WOriginator,
			"Mandatory, Repeatable"},
		{WPublication_Date,
			"Mandatory"},
		{WPublication_Time,
			"Optional"},
		{WTitle,
			"Mandatory"},
		{WEdition,
			"Mandatory if applicable"},
		{WGeospatial_Data_Presentation_Form,
			"Mandatory if applicable"},
		{WSeries_Information,
			"Mandatory if applicable"},
		{WPublication_Information,
			"Mandatory if applicable"},
		{WOther_Citation_Details,
			"Mandatory if applicable"},
		{WOnline_Linkage,
			"Optional, Repeatable"},
		{WLarger_Work_Citation,
			"Mandatory if applicable"},
		{Wnull,""}
		}},

	{WSeries_Information,{
		{WSeries_Name,
			"Mandatory"},
		{WIssue_Identification,
			"Mandatory"},
		{Wnull,""}
		}},

	{WPublication_Information,{
		{WPublication_Place,
			"Mandatory"},
		{WPublisher,
			"Mandatory"},
		{Wnull,""}
		}},

	{WLarger_Work_Citation,{
		{WCitation_Information,
			"Mandatory"},
		{Wnull,""}
		}},

	{WTime_Period_Information,{
		{WSingle_Date_Time,
			"Mandatory; choose this, Multiple_Dates/Times, or Range_of_Dates/Times"},
		{WMultiple_Dates_Times,
			"Mandatory; choose this, Single_Date/Time, or Range_of_Dates/Times"},
		{WRange_of_Dates_Times,
			"Mandatory; choose this, Single_Date/Time, or Multiple_Dates/Times"},
		{Wnull,""}
		}},

	{WSingle_Date_Time,{
		{WCalendar_Date,
			"Mandatory"},
		{WTime_of_Day,
			"Optional"},
		{Wnull,""}
		}},

	{WMultiple_Dates_Times,{
		{WSingle_Date_Time,
			"Mandatory, Repeatable"},
		{Wnull,""}
		}},

	{WRange_of_Dates_Times,{
		{WBeginning_Date,
			"Mandatory"},
		{WBeginning_Time,
			"Optional"},
		{WEnding_Date,
			"Mandatory"},
		{WEnding_Time,
			"Optional"},
		{Wnull,""}
		}},

	{WContact_Information,{
		{WContact_Person_Primary,
			"Mandatory; choose this or Contact_Organization_Primary"},
		{WContact_Organization_Primary,
			"Mandatory; choose this or Contact_Person_Primary"},
		{WContact_Position,
			"Optional"},
		{WContact_Address,
			"Mandatory, Repeatable"},
		{WContact_Voice_Telephone,
			"Mandatory, Repeatable"},
		{WContact_TDD_TTY_Telephone,
			"Optional, Repeatable"},
		{WContact_Facsimile_Telephone,
			"Optional, Repeatable"},
		{WContact_Electronic_Mail_Address,
			"Optional, Repeatable"},
		{WHours_of_Service,
			"Optional"},
		{WContact_Instructions,
			"Optional"},
		{Wnull,""}
		}},

	{WContact_Person_Primary,{
		{WContact_Person,
			"Mandatory"},
		{WContact_Organization,
			"Optional"},
		{Wnull,""}
		}},

	{WContact_Organization_Primary,{
		{WContact_Organization,
			"Mandatory"},
		{WContact_Person,
			"Optional"},
		{Wnull,""}
		}},

	{WContact_Address,{
		{WAddress_Type,
			"Mandatory"},
		{WAddress,
			"Mandatory if applicable, Repeatable"},
		{WCity,
			"Mandatory"},
		{WState_or_Province,
			"Mandatory"},
		{WPostal_Code,
			"Mandatory"},
		{WCountry,
			"Optional"},
		{Wnull,""}
		}},

	{Wnull,{
		{Wnull,""}
		}}
	};
static int count = 0;

static int compare (const void *e1, const void *e2) {
	const struct tip_t *t1 = (const struct tip_t *) e1;
	const struct tip_t *t2 = (const struct tip_t *) e2;
	return (t1->parent - t2->parent);
	}

static void init_tips (void) {
	for (count=0; default_tip[count].parent != Wnull; count++);
	qsort (default_tip,count,sizeof(struct tip_t),compare);
	}

/*----------------------------------------------------------------------*/

static int add_child (enum fgdc_keyword parent, enum fgdc_keyword child, char *text) {
	struct tip_t *t;
	int n = 0;

	for (t = default_tip; t->parent != Wnull; t++)
		if (t->parent == parent) {
			for (n=0; t->child[n].key != Wnull; n++)
				if (t->child[n].key == child) return (n); /* already in list */
			if (n < MAX_COMPONENT_COUNT) {
				t->child[n].key = child;
				t->child[n].text = text;
				n++;
				t->child[n].key = Wnull;
				t->child[n].text = NULL;
				}
			}
	return (n);
	}

/*----------------------------------------------------------------------*/

static int add_parent (enum fgdc_keyword parent) {
	struct tip_t *t;

	/* Do nothing if this element is already in the list. */

	for (t = default_tip; t->parent != Wnull; t++)
		if (t->parent == parent) return (0);

	/* t now points to the last element in the list */

	if (t->parent == Wnull && count < MAX_COMPOUND_COUNT) {
		t->parent = parent;
		t->child[0].key = Wnull;
		t->child[0].text = NULL;
		t++;
		t->parent = Wnull;
		t->child[0].key = Wnull;
		t->child[0].text = NULL;
		count++;
		return (1);
		}
	return (0);
	}

/*----------------------------------------------------------------------*/

void read_tips (char *input_file) {
	FILE *in;
	char line [1024];
	char *s,*text;
	enum fgdc_keyword parent,child;

	if (count == 0) init_tips();

	if (in = fopen (input_file,"r")) {
		while (fgets (line,1024,in)) {
			s = line + strlen (line) - 1;
			if (*s == '\n') *s-- = 0;
			if (*s == '\r') *s-- = 0;
			while (s > line && isspace (*s)) *s-- = 0;
			for (s = line; *s && isspace (*s); s++);
			if (*s) {
				parent = key_of (s);
				if (parent != Wunknown && parent != Wnull) {
					s += strlen (text_of (parent));
					if (*s == ':') s++;
					while (*s && isspace (*s)) s++;
					child = key_of (s);
					if (child != Wunknown && child != Wnull) {
						s += strlen (text_of (child));
						if (*s == ':') s++;
						while (*s && isspace (*s)) s++;
						if (text = (char *) malloc (1 + strlen (s))) {
							strcpy (text,s);
							add_parent (parent);
							add_child (parent,child,text);
							}
						}
					}
				}
			}
		fclose (in);
		}
	}

/*----------------------------------------------------------------------*/

char *tip_text (char *parent_name, char *child_name) {
	enum fgdc_keyword parent,child;
	struct tip_t target;
	struct tip_t *t;

	if (count == 0) init_tips();

	parent = key_of (parent_name);
	if (parent != Wnull && parent != Wblank && parent != Wunknown) {
		target.parent = parent;
		child = key_of (child_name);
		if (child != Wnull && child != Wblank && child != Wunknown)
			if (t = (struct tip_t *) bsearch (&target,default_tip,count,sizeof(struct tip_t),compare)) {
				int i;
				for (i=0; t->child[i].key != Wnull; i++)
					if (t->child[i].key == child)
						return (t->child[i].text);
				}
		}
	return (NULL);
	}

/*----------------------------------------------------------------------*\
\*----------------------------------------------------------------------*/
