#!/usr/bin/wish -f

#-------------------------------------------------------------------------
# Tkme: Tk Metadata Editor
#
# Based on Xtme, but uses Tcl/Tk for the user interface
#
# Peter N. Schweitzer (U.S. Geological Survey, Reston, VA 20192)
#-------------------------------------------------------------------------

#-------------------------------------------------------------------------
# The following variables can be modified by the user to alter the
# look and feel of Tkme.
#-------------------------------------------------------------------------

if {[string compare $tcl_platform(platform) windows] == 0} {
	set font_face(list)		terminal
	if {[string compare [root_name] Metadatos] == 0} {
		set font_face(list)		courier
		}
	set font_size(list)		9
	set font_bold(list)		0
	set font_italic(list)	0

	set font_face(value)	courier
	set font_size(value)	9
	set font_bold(value)	0
	set font_italic(value)	0

	set font_face(menu)		arial
	set font_size(menu)		9
	set font_bold(menu)		0
	set font_italic(menu)	0

	set font_face(output)	courier
	set font_size(output)	9
	set font_bold(output)	0
	set font_italic(output)	0

	set font_face(help)		arial
	set font_size(help)		9
	set font_bold(help)		0
	set font_italic(help)	0
	} \
else {
	set font_face(list)		fixed
	set font_size(list)		12
	set font_bold(list)		0
	set font_italic(list)	0

	set font_face(value)	fixed
	set font_size(value)	12
	set font_bold(value)	0
	set font_italic(value)	0

	set font_face(menu)		helvetica
	set font_size(menu)		12
	set font_bold(menu)		1
	set font_italic(menu)	0

	set font_face(output)	fixed
	set font_size(output)	12
	set font_bold(output)	0
	set font_italic(output)	0

	set font_face(help)		helvetica
	set font_size(help)		12
	set font_bold(help)		0
	set font_italic(help)	0
	}

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# Do not edit these variables or anything else below this line
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

set file_name "Untitled"
set changed 0
set current_menu ""
set current_item ""
set current_element ""
set current_index   ""

#-------------------------------------------------------------------------
# Recent files list
#-------------------------------------------------------------------------

proc add_to_recent_files_list {file_name} {
	global recent_file
	set found 0
	set file_list {}
	if {[catch "open $recent_file r" result]} {
		# do nothing
		} \
	else {
		set rf $result
		set file_list [split [read -nonewline $rf] "\n"]
		foreach f $file_list {
			if {[file exists $f]} {
				if {[string compare $f $file_name] == 0} {
					set found 1
					}
				}
			}
		close $rf
		}
	if {$found == 0} {
		set rf [open $recent_file w]
		puts $rf "$file_name"
		set i 1
		foreach f $file_list {
			if {[file exists $f]} {
				puts $rf "$f"
				incr i
				if { $i > 9 } break
				}
			}
		close $rf
		.mbar.file.menu.recent add command -label "$file_name" -command [list read_metadata "$file_name"]
		}
	}

#-------------------------------------------------------------------------
# File menu callbacks
#-------------------------------------------------------------------------
# Callback for the Open choice of the File menu
#
# calls C procedure parse_text_cmd()

proc read_metadata {name} {
	global file_name changed element_list element_value
	set w $element_list
	if {[string length $name] > 0} {
		set ptree_list [parse_text $name]
		if {[llength $ptree_list] > 0} {
			foreach item $ptree_list {
				$w insert end $item
				}
			set the_item 0
			$w selection set $the_item
			$w activate $the_item
			$w see $the_item
			set_current $w
			if {[string compare [file pathtype $name] relative] == 0} {
				set file_name [file join [pwd] $name]
				} \
			else {
				set file_name $name
				}
			wm title . "Tkme: $file_name"
			set changed 0

			# Read the list of recently-opened files.
			# If the current file is not one of them,
			# prepend the current file name to the list
			# and write the list back to disk

			add_to_recent_files_list $file_name

			# assign keyboard focus to the value window

			focus $element_value

			# disable menu items associated with opening files

			.mbar.file.menu entryconfigure 0 -state disabled
			.mbar.file.menu entryconfigure 4 -state disabled
			.mbar.file.menu entryconfigure 5 -state disabled
			}
		}
	}

proc file_open {} {
	set mf [list {Metadata files} [ext_list]]
	set types [list $mf {{Text files} {.txt .text}} {{XML files} {.xml}} {{SGML files} {.sgml .sgm}} {{All files} *}]
	set input_file [tk_getOpenFile -title "Select input file" -filetypes $types]
	if {[llength $input_file] > 0} {
		read_metadata $input_file
		}
	}

#-------------------------------------------------------------------------
# Helper function to determine whether we can write the named file
#

set reason {}

proc can_write {name} {
	global reason
	if {[file exists $name]} {
		if {[file writable $name]} {
			set q 1
			} \
		else {
			set q 0
			set reason "The file exists with Read Only attributes."
			}
		} \
	else {
		set dir [file dirname $name]
		if {[file exists $dir]} {
			if {[file writable $dir]} {
				set q 1
				} \
			else {
				set q 0
				set reason "You do not have permission to create files in this directory."
				}
			} \
		else {
			set q 0
			set reason "This directory does not exist."
			}
		}
	return $q
	}

#-------------------------------------------------------------------------
# Callback for the Save choice of the File menu
#

proc file_save {} {
	global file_name changed reason

	save_current
	if {[string compare $file_name "Untitled"] == 0 || [can_write $file_name] == 0} {
		file_save_as
		} \
	else {
		write_text 0 $file_name
		set changed 0
		}
	}

#-------------------------------------------------------------------------
# Callback for the Save As choice of the File menu
#

proc file_save_as {} {
	global file_name changed reason

	save_current
	set output_file [tk_getSaveFile -title "Select output file" -initialfile $file_name]
	if {[string length $output_file] == 0} return
	if {[can_write $output_file]} {
		set ok 1
		} \
	else {
		set ok 0
		tk_messageBox -type ok -message "$output_file\n$reason\nPlease use a different file name." -icon warning
		}
	while {$ok == 0} {
		set output_file [tk_getSaveFile -title "Select output file"]
		if {[string length $output_file] == 0} return
		if {[can_write $output_file]} {
			set ok 1
			} \
		else {
			set ok 0
			tk_messageBox -type ok -message "$output_file\n$reason\nPlease use a different file name." -icon warning
			}
		}
	write_text 0 $output_file
	set file_name $output_file
	wm title . "Tkme: $file_name"
	set changed 0
	add_to_recent_files_list $file_name
	}

#-------------------------------------------------------------------------
# Callback for the Close choice of the File menu
#

proc file_close {} {
	global element_list element_value file_name
	global changed

	save_current

	if {[string compare $changed 0] != 0} {
		set choice [tk_messageBox -type yesnocancel -default yes -message "Save changes before closing?" -icon question]
		switch -exact $choice {
			yes {
				file_save
				forget
				}
			no {
				forget
				}
			cancel {
				return
				}
			}
		} \
	else {
		forget
		}

	$element_list delete 0 end
	$element_value configure -state normal
	$element_value delete 1.0 end
	$element_value configure -state disabled
	$element_value configure -background gray
	.mbar.add.menu delete 0 end
	.mbar.add.menu add command -label "[root_name]" -command "add_element [root_name]"
	set file_name "Untitled"
	wm title . "Tkme: $file_name"

	.mbar.file.menu entryconfigure 0 -state normal
	if {[config_read]} {set state disabled} else {set state normal}
	.mbar.file.menu entryconfigure 4 -state $state
	.mbar.file.menu entryconfigure 5 -state normal
	}

#-------------------------------------------------------------------------
# Callback for the Configure choice of the File menu
#

proc file_config {} {
    set typelist {
		{"Configuration files"	{.cfg}	TEXT}
		{"Text files"			{.txt}	TEXT}
		{"All files"		*}
	    }
	set config_file [tk_getOpenFile -filetypes $typelist -title "Select config file"]
	if {[string length $config_file] > 0} {
		read_config $config_file
		.mbar.file.menu entryconfigure 4 -state disabled
		}
	}

#-------------------------------------------------------------------------
# Save window geometry
#

proc save_geometry {} {
	global recent_file
	global font_face font_size font_bold font_italic

	# Find a place to put the window geometry commands
	set geometry_file [file join [file dirname $recent_file] geometry.tcl]
	set out [open $geometry_file w]

	# Write the window geometry commands as a Tcl script, commented
	puts $out "# Tcl statements to restore the window size and position"
	puts $out "# This file is created when you exit Tkme."

	# For those windows that are displayed, get their geometry from the
	# window manager so they can be restored to the same size and positions.

	set g_main {}
	if {[string compare [wm state .] normal] == 0} {
		catch {wm geometry .} g_main
		puts $out "wm geometry . $g_main"
		}

	set g_help {}
	if {[string compare [wm state .help] normal] == 0} {
		catch {wm geometry .help} g_help
		puts $out "wm deiconify .help"
		puts $out "wm geometry .help $g_help"
		}

	set g_output {}
	if {[string compare [wm state .output] normal] == 0} {
		catch {wm geometry .output} g_output
		puts $out "wm geometry .output $g_output"
		}

	puts $out "# Font configuration"

	puts $out "set font_face(list)     \"$font_face(list)\""
	puts $out "set font_size(list)     $font_size(list)"
	puts $out "set font_bold(list)     $font_bold(list)"
	puts $out "set font_italic(list)   $font_italic(list)"

	puts $out "set font_face(value)    \"$font_face(value)\""
	puts $out "set font_size(value)    $font_size(value)"
	puts $out "set font_bold(value)    $font_bold(value)"
	puts $out "set font_italic(value)  $font_italic(value)"

	puts $out "set font_face(menu)     \"$font_face(menu)\""
	puts $out "set font_size(menu)     $font_size(menu)"
	puts $out "set font_bold(menu)     $font_bold(menu)"
	puts $out "set font_italic(menu)   $font_italic(menu)"

	puts $out "set font_face(output)   \"$font_face(output)\""
	puts $out "set font_size(output)   $font_size(output)"
	puts $out "set font_bold(output)   $font_bold(output)"
	puts $out "set font_italic(output) $font_italic(output)"

	puts $out "set font_face(help)     \"$font_face(help)\""
	puts $out "set font_size(help)     $font_size(help)"
	puts $out "set font_bold(help)     $font_bold(help)"
	puts $out "set font_italic(help)   $font_italic(help)"
	puts $out "font_apply configure"

	puts $out "# end"
	close $out
	}

#-------------------------------------------------------------------------
# Callback for the Quit choice of the File menu
#

proc file_quit {} {
	global changed
	save_current
	save_geometry
	if {[string compare $changed 0] != 0} {
		set choice [tk_messageBox -type yesnocancel -default yes -message "Save changes before quitting?" -icon question]
		switch -exact $choice {
			yes {
				file_save
				exit
				}
			no {
				exit
				}
			cancel {
				}
			}
		} \
	else {
		exit
		}
	}

#-------------------------------------------------------------------------
# Edit menu callbacks
#-------------------------------------------------------------------------
# Callback for the Cut choice of the Edit menu
#

proc edit_cut {widget} {
	global changed element_list element_value

	set is_list  [string compare $widget $element_list]
	set is_value [string compare $widget $element_value]
	set q [expr $is_list * $is_value]

	if {$q != 0} {
		# the widget is neither the list nor the value,
		# so it must be some other text widget.
		tk_textCut $widget
		} \
	else {
		# If some text was selected in the value window, cut that

		set selected_range [$element_value tag nextrange sel 1.0 end]
	    if {[string compare [$element_value tag nextrange sel 1.0 end] ""]} {
	    	tk_textCut $element_value
	    	} \
		else {
			# if no text was selected in the value window, cut the element

			set w $element_list
			save_current
			if {[string length [$w curselection]] >= 1} {
				set ptree_list [element_cut]
				if {[llength $ptree_list] > 0} {
					$w delete 0 end
					foreach item $ptree_list {
						$w insert end $item
						}
					set the_item [get_current]
					$w selection set $the_item
					$w activate $the_item
					$w see $the_item
					set_current $w
					set changed 1

					$element_value configure -insertbackground DarkRed -insertwidth 3
					}
				clipboard clear
				clipboard append "[clipped_text]"
				}
			}
		}
	}

#-------------------------------------------------------------------------
# Callback for the Copy choice of the Edit menu
#

proc edit_copy {widget} {
	global changed element_list element_value

	set is_list  [string compare $widget $element_list]
	set is_value [string compare $widget $element_value]
	set q [expr $is_list * $is_value]

	if {$q != 0} {
		# the widget is neither the list nor the value,
		# so it must be some other text widget.
		tk_textCopy $widget
		} \
	else {
		# If some text was selected in the value window, copy that

	    if {[string compare [$element_value tag nextrange sel 1.0 end] ""]} {
	    	tk_textCopy $element_value
	    	return
	    	} \
		else {
			# if no text was selected in the value window, copy the subtree

			set w $element_list
			save_current
			set w $element_list
			if {[string length [$w curselection]] >= 1} {
				element_copy
				clipboard clear
				clipboard append "[clipped_text]"
				} \
			else {
				# Find out whether text in the value window is selected,
				# and if so, copy that text to the clipboard.  If not, don't
				# do anything.
				set t $element_value
				set q [$t tag ranges sel]
				if {[llength q] > 0} {
					set b [lindex $q 0]
					set e [lindex $q 1]
					set stuff [$t get $b $e]
					clipboard clear
					clipboard append "$stuff"
					} \
				else {
					tk_messageBox -type ok -message "There doesn't seem to be anything selected right now." -icon warning
					}
				}
			}
		}
	}

#-------------------------------------------------------------------------
# Callback for the Paste choice of the Edit menu
#
# If the current element is scalar
#   If the selection is plain text,
#     paste the selection into this element's value.
#
#   If the selection is structured text,
#     Ask the user if he wants to paste it as text or as a subtree.
#
#     If as a subtree, look upwards from the current element for a
#     parent element that can take the subtree as one of its children.
#
#     If as plain text, paste the selection into this element's value.
#
# If the current element is compound
#   If the selection is plain text,
#     reject the operation.
#
#   If the selection is structured text,
#     Look upwards from the current element for a parent element that
#     can take the subtree as one of its children.
#

proc edit_paste {widget} {
	global changed element_list element_value
	set w $element_list

	set is_list  [string compare $widget $element_list]
	set is_value [string compare $widget $element_value]
	set q [expr $is_list * $is_value]

	if {$q == 0} {
		# widget is $element_list or $element_value
		# Figure out whether a subtree paste is possible
		set the_item [$w index active]
		set stuff "[selection get -selection CLIPBOARD]"
		set q [is_scalar $the_item]
		append q [recognized $stuff]

		switch -exact $q {
			00 {	# element compound selection plain
				tk_messageBox -type ok -message "Sorry, this information cannot be placed properly" -icon warning
				}

			01 {	# element compound selection subtree
				if {[paste_subtree "$stuff"] == 1} {
					set ptree_list [get_list]
					if {[llength $ptree_list] > 0} {
						$w delete 0 end
						foreach item $ptree_list {
							$w insert end $item
							}
						set the_item [get_current]
						$w selection set $the_item
						$w activate $the_item
						$w see $the_item
						set_current $w
						set changed 1
						}
					} \
				else {
					tk_messageBox -type ok -message "Sorry, this information cannot be placed properly" -icon warning
					}
				}

			10 {	# element scalar selection plain
				catch {$element_value delete sel.first sel.last}
				$element_value insert insert "$stuff"
				}

			11 {	# element scalar selection subtree
				if {[paste_subtree "$stuff"] == 1} {
					set ptree_list [get_list]
					if {[llength $ptree_list] > 0} {
						$w delete 0 end
						foreach item $ptree_list {
							$w insert end $item
							}
						set the_item [get_current]
						$w selection set $the_item
						$w activate $the_item
						$w see $the_item
						set_current $w
						set changed 1
						}
					} \
				else {
					tk_messageBox -type ok -message "Sorry, this information cannot be placed properly" -icon warning
					}
				}
			}
		} \
	else {
		# widget is neithe $element_list nor $element_value,
		# so it must be some other text widget.
		tk_textPaste $widget
		}
	}

#-------------------------------------------------------------------------
# Callback for the Clear choice of the Edit menu
#

proc edit_clear {} {
	global changed element_list
	set w $element_list
	clear_subtree [$w index active]
	set_current $w
	set changed 1
	}

#-------------------------------------------------------------------------
# Callback for the Duplicate choice of the Edit menu
#

proc edit_duplicate {} {
	global changed element_list
	set w $element_list
	save_current
	set ptree_list [duplicate]
	if {[llength $ptree_list] > 0} {
		$w delete 0 end
		foreach item $ptree_list {
			$w insert end $item
			}
		set the_item [get_current]
		$w selection set $the_item
		$w activate $the_item
		$w see $the_item
		set_current $w
		set changed 1
		}
	}

#-------------------------------------------------------------------------
# Callback for the Swap choice of the Edit menu
#

proc edit_swap {} {
	global changed element_list
	set w $element_list
	save_current
	set ptree_list [swap]
	if {[llength $ptree_list] > 0} {
		$w delete 0 end
		foreach item $ptree_list {
			$w insert end $item
			}
		set the_item [get_current]
		$w selection set $the_item
		$w activate $the_item
		$w see $the_item
		set_current $w
		set changed 1
		}
	}

#-------------------------------------------------------------------------
# Callback for the Prune choice of the Edit menu
#

proc edit_prune {} {
	global changed element_list
	set w $element_list
	save_current
	set ptree_list [prune]
	if {[llength $ptree_list] > 0} {
		$w delete 0 end
		foreach item $ptree_list {
			$w insert end $item
			}
		set the_item [get_current]
		$w selection set $the_item
		$w activate $the_item
		$w see $the_item
		set_current $w
		set changed 1
		}
	}

#-------------------------------------------------------------------------
# Revert to previous value of the current element

proc edit_revert {} {
	global element_list element_value
	global previous_value

	# If the first list element is empty, just return
	if {[string length [$element_list get 0]] == 0} {return}

	set this_index [$element_list index active]
	set this_element [$element_list get $this_index]

	#-----------------------------------------------------------------
	# Replace the value text with the new element's previous value
	#-----------------------------------------------------------------

	if {[is_scalar $this_index] == 1} {
		$element_value delete 1.0 end
		$element_value insert 1.0 "$previous_value"
		}
	}

#-------------------------------------------------------------------------
# View menu callbacks
#-------------------------------------------------------------------------
# Callback for the (number) levels choice of the View menu
#

proc view_set {number} {
	global element_list
	save_current
	set w $element_list
	set ptree_list [view_levels $number]
	$w delete 0 end
	foreach item $ptree_list {
		$w insert end $item
		}
	set the_item [get_current]
	$w selection set $the_item
	$w activate $the_item
	$w see $the_item
	set_current $w
	}

#-------------------------------------------------------------------------
# Callback for the Hide choice of the View menu
#

proc view_hide {} {
	global element_list
	save_current
	set w $element_list
	set ptree_list [hide_element]
	$w delete 0 end
	foreach item $ptree_list {
		$w insert end $item
		}
	set the_item [get_current]
	$w selection set $the_item
	$w activate $the_item
	$w see $the_item
	set_current $w
	}

#-------------------------------------------------------------------------
# Callback for the Show choice of the View menu
#

proc view_show {} {
	global element_list
	save_current
	set w $element_list
	set ptree_list [show_element]
	$w delete 0 end
	foreach item $ptree_list {
		$w insert end $item
		}
	set the_item [get_current]
	$w selection set $the_item
	$w activate $the_item
	$w see $the_item
	set_current $w
	}

#-------------------------------------------------------------------------
# Action view_toggle hides if shown, shows if hidden
#

proc view_toggle {} {
	global element_list
	set w $element_list

	set this_index [$w index active]
	set this_element [$w get $this_index]

	if {[string index $this_element 0] == "+"} {
		view_show
		} \
	else {
		view_hide
		}
	}

#-------------------------------------------------------------------------
# Callback for the Wrap choice of the View menu
#

proc view_wrap {} {
	global element_value
	set w $element_value
	set now [$w cget -wrap]
	if {[string compare $now "word"] == 0} {
		$w configure -wrap none
		.mbar.view.menu entryconfigure 6 -label "Wrap"
		.paned.value.menu entryconfigure 5 -label "Wrap"
		} \
	else {
		$w configure -wrap word
		.mbar.view.menu entryconfigure 6 -label "No Wrap"
		.paned.value.menu entryconfigure 5 -label "No Wrap"
		}
	}

proc hard_wrap {} {
	global element_value element_list

	set this_index [$element_list index active]

	if {[is_scalar $this_index] == 1} {
		save_current
		wrap_current
		$element_value configure -state normal
		$element_value delete 1.0 end
		$element_value insert 1.0 "[set_text $this_index]"
		}
	}

#-------------------------------------------------------------------------
# Add menu callbacks
#-------------------------------------------------------------------------
# Add (element)
#

proc add_element {name} {
	global changed element_list
	save_current
	set w $element_list
	set ptree_list [element_add $name]
	if {[llength $ptree_list] > 0} {
		$w delete 0 end
		foreach item $ptree_list {
			$w insert end $item
			}
		set the_item [get_current]
		$w selection set $the_item
		$w activate $the_item
		$w see $the_item
		set_current $w
		set changed 1

		# If what you just added was the root element, then
		# disable menu items associated with opening files
		if {[string compare $name [root_name]] == 0} {
			.mbar.file.menu entryconfigure 0 -state disabled
			.mbar.file.menu entryconfigure 4 -state disabled
			.mbar.file.menu entryconfigure 5 -state disabled
			}
		}
	}

#-------------------------------------------------------------------------
# Add all components
#

proc add_all {} {
	global changed element_list
	save_current
	set w $element_list
	set the_item [$w index active]
	if {[is_scalar $the_item] != 1} {
		set ptree_list [add_children]
		if {[llength $ptree_list] > 0} {
			$w delete 0 end
			foreach item $ptree_list {
				$w insert end $item
				}
			set the_item [get_current]
			$w selection set $the_item
			$w activate $the_item
			$w see $the_item
			set_current $w
			set changed 1
			}
		}
	}

#-------------------------------------------------------------------------
# Snippets menu callbacks
#-------------------------------------------------------------------------

proc snippet_store {} {
	global snippet_home

	# If $snippet_home contains a tilde, call glob to expand it

	set snippet_home_full [glob $snippet_home]

	# Prompt user to provide a file in which it can be stored

	set t [tk_getSaveFile -title "Name a file to store the snippet" -initialdir $snippet_home]
	set snippet [string map {{ } _} $t]
	set snippet_label [file tail $snippet]

	if {[string length $snippet] > 0} {
		if {[string first $snippet_home_full $snippet] < 3} {
			if {[catch "open $snippet w" fp]} {
				} \
			else {
				puts $fp [encode_current]
				close $fp

				# Insert a new item for this snippet in the appropriate menu.
				# Note that if the snippet is being stored in a subdirectory
				# below $snippet_home, then you have to walk through the
				# path elements, converting them to lower case and appending
				# their names to the menu string, because directories are
				# cascades.

				set m .mbar.snip.menu
				set h [file split $snippet_home_full]
				set lh [llength $h]
				set d [file split [file dirname $snippet]]
				set ld [llength $d]
				if {$lh < $ld} {
					for {set i $lh} {$i < $ld} {incr i} {
						set t [string tolower [lindex $d $i]]

						# If the cascade doesn't exist, create it

						if {[catch "menu $m.$t -tearoff false -font menu_font" err]} {
							# it's okay; the menu already exists, so the cascade must also
							} \
						else {
							$m add cascade -label $t -menu $m.$t
							}

						set m $m.$t
						}
					}
				$m add command -label $snippet_label -command "snippet_insert $snippet"
				}
			} \
		else {
			tk_messageBox -type ok -icon warning -message "You must store the snippet within $snippet_home"
			}
		}
	}

proc snippet_insert {snippet_name} {
	global snippet_home element_list

	if {[catch "open $snippet_name r" fp]} {
		} \
	else {
		set stuff [read -nonewline $fp]

		if {[paste_subtree "$stuff"] == 1} {
			set w $element_list
			set ptree_list [get_list]
			if {[llength $ptree_list] > 0} {
				$w delete 0 end
				foreach item $ptree_list {
					$w insert end $item
					}
				set the_item [get_current]
				$w selection set $the_item
				$w activate $the_item
				$w see $the_item
				set_current $w
				set changed 1
				}
			} \
		else {
			tk_messageBox -type ok -message "Sorry, this information cannot be placed properly" -icon warning
			}
		close $fp
		}
	}

#-------------------------------------------------------------------------
# Help menu callbacks
#-------------------------------------------------------------------------
# Callback for the Version choice of the Help menu
#

proc help_version {} {
	set w .help

	if [catch {set status [wm state $w]} ok] {
		show_help_window
		} \
	else {
		switch -exact $status {
			normal {
				}
			iconic {
				wm deiconify $w
				}
			withdrawn {
				# wm iconify $w
				wm deiconify $w
				}
			}
		.help.text.text delete 1.0 end
		.help.text.text insert end "[about_text]"
		}
	}

#-------------------------------------------------------------------------
# Callback for the Element choice of the Help menu
#

proc show_help_window {} {
	puts stdout "show_help_window"
	}

proc help_element {} {
	set w .help

	if [catch {set status [wm state $w]} ok] {
		show_help_window
		} \
	else {
		switch -exact $status {
			normal {
				}
			iconic {
				wm deiconify $w
				}
			withdrawn {
				# wm iconify $w
				wm deiconify $w
				}
			}
		.help.text.text delete 1.0 end
		.help.text.text insert end "[help_text]"
		}
	}

#-------------------------------------------------------------------------
# Callback for the Version choice of the Help menu
#

proc show_output_window {} {
	puts stdout "show_output_window"
	}

proc help_output {} {
	set w .output

	if [catch {set status [wm state $w]} ok] {
		show_output_window
		} \
	else {
		switch -exact $status {
			normal {
				}
			iconic {
				wm deiconify $w
				}
			withdrawn {
				wm deiconify $w
				}
			}
		output_update
		}
	}

#-------------------------------------------------------------------------
# Callback for the Fonts choice of the Help menu
#

proc show_font_window {} {
	global font_face font_size font_bold font_italic
	set w .fonts
	toplevel $w
	frame $w.mbar -relief flat -bd 3
	pack $w.mbar -side top -expand false -fill x
	button $w.mbar.close -text "Close" -font menu_font -command {font_close}
	button $w.mbar.apply -text "Apply" -font menu_font -command {font_apply configure}
	pack $w.mbar.close $w.mbar.apply -side left

	frame $w.f
	pack $w.f
	label $w.f.which -text "Place"
	label $w.f.face -text "Typeface"
	label $w.f.size -text "Size"
	label $w.f.bold -text "Bold"
	label $w.f.italic -text "Slanted"
	grid $w.f.which $w.f.face $w.f.size $w.f.bold $w.f.italic
	foreach part {list value menu output help} {
		label $w.f.label_$part -text "$part"

		set cmd [list tk_optionMenu $w.f.face_$part font_face($part)]
		foreach name [font families] {
			if {[string length $name] > 1} {
				if {[string compare $name nil] != 0} {
					lappend cmd $name
					}
				}
			}
		eval $cmd

		tk_optionMenu $w.f.size_$part font_size($part) 7 8 9 10 11 12 14 16 18 21 24
		checkbutton $w.f.bold_$part -variable font_bold($part)
		checkbutton $w.f.italic_$part -variable font_italic($part)
		grid $w.f.label_$part $w.f.face_$part $w.f.size_$part $w.f.bold_$part $w.f.italic_$part
		grid $w.f.label_$part $w.f.face_$part $w.f.size_$part -sticky w
		}
	}

proc help_fonts {} {
	set w .fonts

	if [catch {set status [wm state $w]} ok] {
		show_font_window
		} \
	else {
		switch -exact $status {
			normal {
				}
			iconic {
				wm deiconify $w
				}
			withdrawn {
				# wm iconify $w
				wm deiconify $w
				}
			}
		}
	}

#-------------------------------------------------------------------------
# Callback for the Close button in the Font window
#

proc font_close {} {
	wm withdraw .fonts
	}

#-------------------------------------------------------------------------
# Callback for the Apply button in the Font window
#

proc font_apply {task} {
	global font_face font_size font_bold font_italic
	set weight(0) normal
	set weight(1) bold
	set slant(0) roman
	set slant(1) italic
	foreach part {list value menu output help} {
		set font_name $part
		append font_name _font
		set cmd [list font $task $font_name -family $font_face($part) -size $font_size($part) -weight $weight($font_bold($part)) -slant $slant($font_italic($part))]
		eval $cmd
		}
	}

#-------------------------------------------------------------------------
# Virtual event actions (these could be used as menu callbacks as well)
#-------------------------------------------------------------------------
# <<list-up>> event action
#

proc list_up {} {
	global element_list
	save_current
	set w $element_list
	set the_item [$w index active]
	if {$the_item > 0} {
		set the_item [expr $the_item - 1]
		$w selection clear [get_current]
		$w selection set $the_item
		$w activate $the_item
		$w see $the_item
		set_current $w
		}
	}

#-------------------------------------------------------------------------
# <<list-down>> event action
#

proc list_down {} {
	global element_list
	save_current
	set w $element_list
	set item_max [$w index end]
	set the_item [$w index active]
	if {$the_item < [expr $item_max - 1]} {
		set the_item [expr $the_item + 1]
		$w selection clear [get_current]
		$w selection set $the_item
		$w activate $the_item
		$w see $the_item
		set_current $w
		}
	}

#-------------------------------------------------------------------------
# <<list-home>> event action
#

proc list_home {} {
	global element_list
	save_current
	set w $element_list
	set the_item [$w index active]
	if {$the_item > 0} {
		set the_item 0
		$w selection clear [get_current]
		$w selection set $the_item
		$w activate $the_item
		$w see $the_item
		set_current $w
		}
	}

#-------------------------------------------------------------------------
# <<list-end>> event action
#

proc list_end {} {
	global element_list
	save_current
	set w $element_list
	set the_item [expr [$w index end] - 1]
	$w selection clear [get_current]
	$w selection set $the_item
	$w activate $the_item
	$w see $the_item
	set_current $w
	}

#-------------------------------------------------------------------------
# <<list-pageup>> event action
#

proc list_pageup {} {
	global element_list
	save_current
	set w $element_list
	set h [$w cget -height]
	set the_item [$w index active]
	set the_item [expr $the_item - $h]
	if {$the_item < 0} {
		set the_item 0
		}
	$w selection clear [get_current]
	$w selection set $the_item
	$w activate $the_item
	$w see $the_item
	set_current $w
	}

#-------------------------------------------------------------------------
# <<list-pagedown>> event action
#

proc list_pagedown {} {
	global element_list
	save_current
	set w $element_list
	set h [$w cget -height]
	set the_item [$w index active]
	set item_max [$w index end]
	set the_item [expr $the_item + $h]
	if {$the_item > [expr $item_max - 1]} {
		set the_item [expr $item_max - 1]
		}
	$w selection clear [get_current]
	$w selection set $the_item
	$w activate $the_item
	$w see $the_item
	set_current $w
	}

#-------------------------------------------------------------------------
# <<list-expand>> action
#

proc list_expand {} {
	global element_list
	set w $element_list
	set ptree_list [expand_list]
	$w delete 0 end
	foreach item $ptree_list {
		$w insert end $item
		}
	set the_item [get_current]
	$w selection set $the_item
	$w activate $the_item
	$w see $the_item
	set_current $w
	}

#-------------------------------------------------------------------------
# <<list-collapse>> action
#

proc list_collapse {} {
	global element_list
	set w $element_list
	set ptree_list [collapse_list]
	$w delete 0 end
	foreach item $ptree_list {
		$w insert end $item
		}
	set the_item [get_current]
	$w selection set $the_item
	$w activate $the_item
	$w see $the_item
	set_current $w
	}

#-------------------------------------------------------------------------
# Callback for menu events <Enter>, <Leave>, and <Motion>: set tip text
#-------------------------------------------------------------------------

proc tip {menu_name} {
	global current_menu current_item
	set this_item [$menu_name index active]
	set is_different 1
	if {[string compare $current_menu $menu_name] == 0} {
		if {[string compare $current_item $this_item] == 0} {
			set is_different 0
			}
		}
	if {$is_different} {
		set current_menu $menu_name
		set current_item $this_item
		.tf.tip configure -text [tip_text [$menu_name cget -title] [$menu_name entrycget active -label]]
		}
	}

#-------------------------------------------------------------------------
# Callbacks for the Help and Output windows
#-------------------------------------------------------------------------
# Callback for the Close button of the Help window
#

proc help_close {} {
	wm withdraw .help
	}

#-------------------------------------------------------------------------
# Callback for the Wrap button of the Help window
#

proc help_wrap {} {
	set w .help.text.text
	set how [$w cget -wrap]
	if {[string compare $how word] == 0} {
		$w configure -wrap none
		.help.mbar.edit.menu entryconfigure 1 -label "Wrap"
		.help.text.menu entryconfigure 2 -label "Wrap"
		} \
	else {
		$w configure -wrap word
		.help.mbar.edit.menu entryconfigure 1 -label "UnWrap"
		.help.text.menu entryconfigure 2 -label "UnWrap"
		}
	}

#-------------------------------------------------------------------------
# Callback for the Close button of the Output window
#

proc output_close {} {
	wm withdraw .output
	}

#-------------------------------------------------------------------------
# Callback for the Update button of the Output window
#

proc output_update {} {
	global tcl_platform file_name
	save_current
	if {[string compare [wm state .output] "normal"] == 0} {
		wm title .output "Tkme output: $file_name"
		set w .output.text.text
		$w delete 1.0 end
		$w insert end "[output_text]"
		$w mark set insert "1.0"
		}
	}

#-------------------------------------------------------------------------
# Callback for the Wrap button of the Output window
#

proc output_wrap {} {
	set w .output.text.text
	set how [$w cget -wrap]
	if {[string compare $how word] == 0} {
		$w configure -wrap none
		.output.mbar.view.menu entryconfigure 1 -label "Wrap"
		} \
	else {
		$w configure -wrap word
		.output.mbar.view.menu entryconfigure 1 -label "No Wrap"
		}
	}

#-------------------------------------------------------------------------
# Callback for the Search button of the Output window
#

proc output_search {pattern} {
	set w .output.text.text
	focus $w
	set old [$w index insert]
	set pos [$w search -count cnt -nocase -forward "$pattern" insert end]
	if {[string compare $old $pos] == 0} {
		$w mark set insert "$old + 1 char"
		set pos [$w search -count cnt -nocase -forward "$pattern" insert end]
		}
	if {[string length $pos] > 0} {
		$w mark set insert $pos
		$w see $pos
		$w tag remove sel 1.0 end
		$w tag add sel $pos "$pos + $cnt chars"
		}
	}

#-------------------------------------------------------------------------
# Callback for the Edit button of the Output window
#

proc output_edit {} {
	save_current
	set t .output.text.text
	set s "[$t index insert]"
	set k [string first "." $s]
	if {$k > 0} {
		set the_line [string range $s 0 [expr $k - 1]]
		} \
	else {
		set the_line 0
		}

	global element_list
	set w $element_list
	set ptree_list [edit_line $the_line]
	$w delete 0 end
	foreach item $ptree_list {
		$w insert end $item
		}
	set the_item [get_current]
	$w selection set $the_item
	$w activate $the_item
	$w see $the_item
	set_current $w
	}

#-------------------------------------------------------------------------
# Insert today's date into the value text widget in ISO 8601 format
#-------------------------------------------------------------------------

proc today {} {
	global element_value
	$element_value insert insert "[clock format [clock seconds] -format "%Y%m%d"]"
	}

#-------------------------------------------------------------------------
# Auxiliary functions
#-------------------------------------------------------------------------
# Replace the list with a new list

proc replace_list {ptree_list} {
	global element_list element_color
	set w $element_list
	$w delete 0 end
	set i 0
	foreach item $ptree_list {
		$w insert end $item
		set t [string trim $item]
		set s [array names element_color $t]
		if {[llength $s] > 0} {
			if {[string length [lindex $s 0]] > 0} {
				}
			}
		incr i
		}
	}

#-------------------------------------------------------------------------
# Save current element's value

proc save_current {} {
	global changed element_value
	set changed [expr $changed + [get_text [$element_value get 1.0 end]]]
	}

#-------------------------------------------------------------------------
# Set current element
#   replaces the text in the value window
#   replaces the menu items in the add menu
#   caches the current element's list index and name

proc set_current {the_listbox} {
	global current_element current_index changed element_value
	global previous_value

	# If the first list element is empty, just return
	if {[string length [$the_listbox get 0]] == 0} {return}

	set this_index [$the_listbox index active]
	set this_element [$the_listbox get $this_index]

	#---------------------------------------------------------------------
	# Determine whether the current element is different from the one
	# that was selected the last time this procedure was called.
	#---------------------------------------------------------------------

	set is_different 1
	if {[string compare $current_element $this_element] == 0} {
		if {[string compare $current_index $this_index] == 0} {
			set is_different 0
			}
		}
	if {$is_different} {

		#-----------------------------------------------------------------
		# Replace the value text with the new element's value
		#-----------------------------------------------------------------

		$element_value configure -state normal
		$element_value delete 1.0 end
		if {[is_scalar $this_index] == 1} {
			set previous_value "[set_text $this_index]"
			$element_value insert 1.0 "$previous_value"
			$element_value configure -background white
			} \
		else {
			$element_value insert 1.0 "[set_text $this_index]"
			$element_value configure -state disabled -background gray
			}
		set current_element $this_element

		#-----------------------------------------------------------------
		# Replace the add menu's items to fit the current element
		#-----------------------------------------------------------------

		set add_menu_list [add_menu_items $current_element]
		.mbar.add.menu delete 0 end
		if {[llength $add_menu_list] > 0} {
			foreach item $add_menu_list {
				.mbar.add.menu add command -label $item -command "add_element $item"
				}
			.mbar.add.menu add command -label "(All of the above)" -command {add_all}
			bind .mbar.add.menu <<MenuSelect>> {tip .mbar.add.menu}
			} \
		else {
			.mbar.add.menu add command -label "Enter value below" -command {}
			}

		#-----------------------------------------------------------------
		# Replace the help text to fit the current element
		#-----------------------------------------------------------------

		if {[string compare [wm state .help] "normal"] == 0} {
			.help.text.text delete 1.0 end
			.help.text.text insert end "[help_text]"
			}
		}
	}

#-------------------------------------------------------------------------
# Create a scrolled listbox widget
# Modified from Brent Welch's book

proc ScrolledListbox { parent args } {
	# Create listbox attached to scrollbars, pass thru $args
	frame $parent
	eval {listbox $parent.list \
		-selectmode "single" \
		-yscrollcommand [list $parent.sy set] \
		-relief flat \
		} $args
	# Create scrollbars attached to the listbox
	scrollbar $parent.sy -orient vertical -width 10 -command [list $parent.list yview]
	# Arrange them in the parent frame
	pack $parent.sy -side right -fill y
	pack $parent.list -side left -fill both -expand true
	}

#-------------------------------------------------------------------------
# Create a scrolled text widget
# Modified from Brent Welch's book

proc ScrolledText { f width height } {
	frame $f
	# The setgrid setting allows the window to be resized.
	text $f.text -width $width -height $height \
		-setgrid true -wrap none -padx 2 \
		-yscrollcommand [list $f.yscroll set]
	scrollbar $f.yscroll -orient vertical -width 10 -command [list $f.text yview]
	pack $f.yscroll -side right -fill y
	# The fill and expand are needed when resizing.
	pack $f.text -side left -fill both -expand true
	pack $f -side top -fill both -expand true
	return $f.text
	}

#-------------------------------------------------------------------------
# Paned widget manager
# From Brent Welch's book

#
# Example 25-5
# Pane_Create sets up vertical or horizontal panes.
#

proc Pane_Create {f1 f2 args} {

	# Map optional arguments into array values
	set t(-orient) vertical
	set t(-percent) 0.333
	set t(-in) [winfo parent $f1]
	array set t $args

	# Keep state in an array associated with the master frame
	set master $t(-in)
	upvar #0 Pane$master pane
	array set pane [array get t]

	# Create the grip and set placement attributes that
	# will not change. A thin divider line is achieved by
	# making the two frames one pixel smaller in the
	# adjustable dimension and making the main frame black.

	set pane(1) $f1
	set pane(2) $f2
	set pane(grip) [frame $master.grip -background gray50 \
		-width 10 -height 10 -bd 1 -relief raised \
		-cursor crosshair]
	if {[string match vert* $pane(-orient)]} {
		set pane(D) Y		;# Adjust boundary in Y direction
		place $pane(1) -in $master -x 0 -rely 0.0 -anchor nw \
			-relwidth 1.0 -height -1
		place $pane(2) -in $master -x 0 -rely 1.0 -anchor sw \
			-relwidth 1.0 -height -1
		place $pane(grip) -in $master -anchor c -relx 0.9
	} else {
		set pane(D) X 		;# Adjust boundary in X direction
		place $pane(1) -in $master -relx 0.0 -y 0 -anchor nw \
			-relheight 1.0 -width -1
		place $pane(2) -in $master -relx 1.0 -y 0 -anchor ne \
			-relheight 1.0 -width -1
		place $pane(grip) -in $master -anchor c -rely 0.9
	}
	$master configure -background black

	# Set up bindings for resize, <Configure>, and
	# for dragging the grip.

	bind $master <Configure> [list PaneGeometry $master]
	bind $pane(grip) <ButtonPress-1> \
		[list PaneDrag $master %$pane(D)]
	bind $pane(grip) <B1-Motion> \
		[list PaneDrag $master %$pane(D)]
	bind $pane(grip) <ButtonRelease-1> \
		[list PaneStop $master]

	# Do the initial layout

	PaneGeometry $master
}

#
# Example 25-6
# PaneDrag adjusts the percentage.
#

proc PaneDrag {master D} {
	upvar #0 Pane$master pane
	if [info exists pane(lastD)] {
		set delta [expr double($pane(lastD) - $D) \
									/ $pane(size)]
		set pane(-percent) [expr $pane(-percent) - $delta]
		if {$pane(-percent) < 0.0} {
			set pane(-percent) 0.0
		} elseif {$pane(-percent) > 1.0} {
			set pane(-percent) 1.0
		}
		PaneGeometry $master
	}
	set pane(lastD) $D
}
proc PaneStop {master} {
	upvar #0 Pane$master pane
	catch {unset pane(lastD)}
}

#
# Example 25-7
# PaneGeometry updates the layout.
#

proc PaneGeometry {master} {
	upvar #0 Pane$master pane
	if {$pane(D) == "X"} {
		place $pane(1) -relwidth $pane(-percent)
		place $pane(2) -relwidth [expr 1.0 - $pane(-percent)]
		place $pane(grip) -relx $pane(-percent)
		set pane(size) [winfo width $master]
	} else {
		place $pane(1) -relheight $pane(-percent)
		place $pane(2) -relheight [expr 1.0 - $pane(-percent)]
		place $pane(grip) -rely $pane(-percent)
		set pane(size) [winfo height $master]
	}
}

#-------------------------------------------------------------------------
# Main program
#-------------------------------------------------------------------------

font_apply create

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# Create menu bar
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

frame .mbar -relief flat -bd 3
.mbar configure -background navy
pack .mbar -side top -expand false -fill x
menubutton .mbar.file -text "File" -menu .mbar.file.menu -font menu_font
menubutton .mbar.edit -text "Edit" -menu .mbar.edit.menu -font menu_font
menubutton .mbar.view -text "View" -menu .mbar.view.menu -font menu_font
menubutton .mbar.add  -text "Add"  -menu .mbar.add.menu  -font menu_font
menubutton .mbar.snip -text "Snippets" -menu .mbar.snip.menu -font menu_font
menubutton .mbar.help -text "Help" -menu .mbar.help.menu -font menu_font
.mbar.file configure -bg navy -fg white
.mbar.edit configure -bg navy -fg white
.mbar.view configure -bg navy -fg white
.mbar.add  configure -bg navy -fg white
.mbar.snip configure -bg navy -fg white
.mbar.help configure -bg navy -fg white
pack .mbar.file .mbar.edit .mbar.view .mbar.add .mbar.snip .mbar.help -side left

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# Constitute menus
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

menu .mbar.file.menu -tearoff false -font menu_font -title "File"
	.mbar.file.menu configure -bg blue -fg white
	.mbar.file.menu add command -label "Open"       -command {file_open}
	.mbar.file.menu add command -label "Save"       -command {file_save}
	.mbar.file.menu add command -label "Save As"    -command {file_save_as}
	.mbar.file.menu add command -label "Close"      -command {file_close}
	.mbar.file.menu add command -label "Configure"  -command {file_config}
	.mbar.file.menu add cascade -label "Recent"     -menu {.mbar.file.menu.recent}
	.mbar.file.menu add command -label "Quit"       -command {file_quit}

menu .mbar.file.menu.recent -tearoff false -font menu_font -title "Recent"
.mbar.file.menu.recent configure -bg blue -fg white
set recent_file [rc_file_spec]
if {[catch "open $recent_file r" rf]} {
	if {[catch {file mkdir [file dirname $recent_file]} q]} {
		if {[string equal windows $tcl_platform(platform)]} {
			tk_messageBox -type ok -message "Warning: the file $recent_file could not be opened because the folder does not exist, and the folder could not be created."
			}
		}
	} \
else {
	foreach f [split [read -nonewline $rf] "\n"] {
		set f [string trim $f]
		if {[file exists $f]} {
			.mbar.file.menu.recent add command -label "$f" -command [list read_metadata "$f"]
			}
		}
	close $rf
	}

menu .mbar.edit.menu -tearoff false -font menu_font -title "Edit"
	.mbar.edit.menu configure -bg blue -fg white
	.mbar.edit.menu add command -label "Cut"        -command {edit_cut .paned.value.text}
	.mbar.edit.menu add command -label "Copy"       -command {edit_copy .paned.value.text}
	.mbar.edit.menu add command -label "Paste"      -command {edit_paste .paned.tree.list}
	.mbar.edit.menu add command -label "Clear"      -command {edit_clear}
	.mbar.edit.menu add command -label "Duplicate"  -command {edit_duplicate}
	.mbar.edit.menu add command -label "Swap"       -command {edit_swap}
	.mbar.edit.menu add command -label "Prune"      -command {edit_prune}
	.mbar.edit.menu add command -label "Replace"    -command {wm deiconify .replace}

menu .mbar.view.menu -tearoff false -font menu_font -title "View"
	.mbar.view.menu configure -bg blue -fg white
	.mbar.view.menu add command -label "All levels" -command {view_set 0}
	.mbar.view.menu add command -label "1 level"    -command {view_set 1}
	.mbar.view.menu add command -label "2 levels"   -command {view_set 2}
	.mbar.view.menu add command -label "3 levels"   -command {view_set 3}
	.mbar.view.menu add command -label "Hide"       -command {view_hide}
	.mbar.view.menu add command -label "Show"       -command {view_show}
	.mbar.view.menu add command -label "Wrap"       -command {view_wrap}

menu .mbar.add.menu -tearoff false -font menu_font -title "Add"
	.mbar.add.menu configure -bg blue -fg white
	.mbar.add.menu add command -label "[root_name]" -command "add_element [root_name]"

menu .mbar.snip.menu -tearoff false -font menu_font -title "Snippets"
	.mbar.snip.menu configure -bg blue -fg white
	.mbar.snip.menu add command -label "Save as snippet" -command {snippet_store}
	.mbar.snip.menu add separator

	# Read the snippet directory, adding each file found there to
	# the snippets menu as an item.

	set snippet_home [file join [file dirname $recent_file] snippets]

	if {[file exists $snippet_home] == 0} {
		file mkdir $snippet_home
		} \
	else {
		if {[file isdirectory $snippet_home] == 0} {
			set snippet_home [tk_chooseDirectory -title "Choose a directory for snippets"]
			if {[file exists $snippet_home] == 0} {
				file mkdir $snippet_home
				}
			}
		}

proc add_snippets {m d} {
	if {[catch "glob [file join $d *]" unsorted]} {
		} \
	else {
		set the_list [lsort $unsorted]
		foreach snippet $the_list {
			set snippet_label [file tail $snippet]
			if {[file isdirectory $snippet]} {
				set snippet_label [string tolower $snippet_label]
				$m add cascade -label $snippet_label -menu $m.$snippet_label
				menu $m.$snippet_label -tearoff false -font menu_font
				add_snippets $m.$snippet_label $snippet
				} \
			else {
				$m add command -label $snippet_label -command "snippet_insert $snippet"
				}
			}
		}
	}

	add_snippets .mbar.snip.menu $snippet_home

menu .mbar.help.menu -tearoff false -font menu_font -title "Help"
	.mbar.help.menu configure -bg blue -fg white
	.mbar.help.menu add command -label "Version"    -command {help_version}
	.mbar.help.menu add command -label "Element"    -command {help_element}
	.mbar.help.menu add command -label "Output"     -command {help_output}
	.mbar.help.menu add command -label "Fonts"      -command {help_fonts}

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# Create a frame for the two panes

frame .paned
.paned configure -height 400 -width 640
pack .paned -fill both -expand true

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# Create scrolled list box for tree structure
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

ScrolledListbox .paned.tree -height 10 -setgrid true
.paned.tree configure
set element_list .paned.tree.list
$element_list configure -font list_font
$element_list configure -selectbackground navy -selectforeground white
$element_list configure -exportselection false
bind $element_list <ButtonPress-1> {save_current; %W activate @%x,%y; set_current %W}
bind $element_list <Double-ButtonPress-1> {view_toggle}

menu .paned.tree.menu -tearoff false -font menu_font -title "Tree" -disabledforeground Navy
	.paned.tree.menu add command -label "Element"         -command {} -state disabled
	.paned.tree.menu add command -label "Cut"             -command {edit_cut .paned.tree.list}
	.paned.tree.menu add command -label "Copy"            -command {edit_copy .paned.tree.list}
	.paned.tree.menu add command -label "Paste"           -command {edit_paste .paned.tree.list}
	.paned.tree.menu add separator
	.paned.tree.menu add command -label "Duplicate"       -command {edit_duplicate}
	.paned.tree.menu add command -label "Clear"           -command {edit_clear}
	.paned.tree.menu add command -label "Prune"           -command {edit_prune}
	.paned.tree.menu add command -label "Save as snippet" -command {snippet_store}

bind $element_list <ButtonPress-3> {tk_popup .paned.tree.menu %X %Y 0}

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# Create scrolled text box for element values
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

ScrolledText .paned.value 64 10
.paned.value configure
set element_value .paned.value.text
$element_value configure -font value_font
$element_value configure -insertbackground DarkRed
$element_value configure -insertborderwidth 0
$element_value configure -insertwidth 3

menu .paned.value.menu -tearoff false -font menu_font -title "Value" -disabledforeground Navy
	.paned.value.menu add command -label "Value"  -command {} -state disabled
	.paned.value.menu add command -label "Cut"      -command {tk_textCut .paned.value.text}
	.paned.value.menu add command -label "Copy"     -command {tk_textCopy .paned.value.text}
	.paned.value.menu add command -label "Paste"    -command {tk_textPaste .paned.value.text}
	.paned.value.menu add command -label "Revert"   -command {edit_revert}
	.paned.value.menu add command -label "Wrap"     -command {view_wrap}
	.paned.value.menu add command -label "HardWrap" -command {hard_wrap}

bind $element_value <ButtonPress-3> {tk_popup .paned.value.menu %X %Y 0}

Pane_Create .paned.tree .paned.value -in .paned -orient horizontal

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# Bind the menu widgets to the tip text command
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

foreach w {.mbar.file.menu .mbar.edit.menu .mbar.view.menu .mbar.add.menu .mbar.snip.menu .mbar.help.menu .paned.tree.menu .paned.value.menu} {
	bind $w <<MenuSelect>> {tip %W}
	}

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# Create the tip line as a framed label
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

frame .tf -relief flat -bd 1
.tf config -background navy
pack .tf -side bottom -expand false -fill x
label .tf.tip -bg navy -fg white -text "Tkme: Tk Metadata Editor by Peter Schweitzer (USGS)"
pack .tf.tip -fill both -expand false -side left

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# Create the help window as a toplevel
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

toplevel .help
frame .help.mbar -relief flat -bd 3
.help.mbar config -background navy
pack .help.mbar -side top -expand false -fill x
button .help.mbar.close -text "Close" -font menu_font -bg navy -fg white -highlightthickness 0 -relief flat -command {help_close}
menubutton .help.mbar.edit -text "Edit" -menu .help.mbar.edit.menu -bg navy -fg white -highlightthickness 0 -relief flat -font menu_font

menu .help.mbar.edit.menu -tearoff false -bg blue -fg white -font menu_font -title "Help-Edit"
	.help.mbar.edit.menu add command -label "Copy" -command {tk_textCopy .help.text.text}
	.help.mbar.edit.menu add command -label "Wrap" -command {help_wrap}

pack .help.mbar.close .help.mbar.edit -side left

ScrolledText .help.text 40 10
.help.text.text configure -font help_font
pack .help.text -fill both -expand true
.help.text.text configure -tabs {32}
.help.text.text configure -wrap none

menu .help.text.menu -tearoff false -font menu_font -title "Help-context"
	.help.text.menu add command -label "Copy" -command {tk_textCopy .help.text.text}
	.help.text.menu add command -label "Wrap" -command {help_wrap}

bind .help.text.text <ButtonPress-3> {tk_popup .help.text.menu %X %Y 0}

wm protocol .help WM_DELETE_WINDOW {wm withdraw .help}
wm withdraw .help

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# Create the output window as a toplevel
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

toplevel .output
frame .output.mbar -relief flat -bd 3
.output.mbar config -background navy
pack .output.mbar -side top -expand false -fill x
menubutton .output.mbar.view -text "View" -bg navy -fg white -menu .output.mbar.view.menu -font menu_font
button .output.mbar.edit -text "Edit"  -font menu_font -bg navy -fg white -relief flat -highlightthickness 0 -command {output_edit}
pack .output.mbar.view .output.mbar.edit -side left

menu .output.mbar.view.menu -bg blue -fg white -tearoff false -font menu_font -title "Output-View"
	.output.mbar.view.menu add command -label "Update"  -command {output_update}
	.output.mbar.view.menu add command -label "Wrap"    -command {output_wrap}
	.output.mbar.view.menu add command -label "Dismiss" -command {output_close}

ScrolledText .output.text 80 20
.output.text.text configure -font output_font
.output.text.text configure -selectbackground navy -selectforeground white
.output.text.text configure -exportselection false
pack .output.text -fill both -expand true
.output.text.text configure -tabs {32}
.output.text.text configure -wrap none
.output.text.text configure -insertbackground DarkRed
.output.text.text configure -insertborderwidth 0
.output.text.text configure -insertwidth 3

# Alter the binding order of the output text widget so that it
# appears to be basically read-only

bindtags .output.text.text {.output.text.text .output all}
bind .output.text.text <Key> {}
bind .output.text.text <<Cut>> {}
bind .output.text.text <<Copy>> {tk_textCopy %W}
bind .output.text.text <<Paste>> {}

if {[string equal $tcl_version 8.3]} {
	bind .output.text.text <1> { tkTextButton1 %W %x %y; %W tag remove sel 0.0 end }
	bind .output.text.text <B1-Motion> { set tkPriv(x) %x; set tkPriv(y) %y; tkTextSelectTo %W %x %y }
	bind .output.text.text <Double-1> { set tkPriv(selectMode) word; tkTextSelectTo %W %x %y; catch {%W mark set insert sel.first} }
	bind .output.text.text <Triple-1> { set tkPriv(selectMode) line; tkTextSelectTo %W %x %y; catch {%W mark set insert sel.first} }
	bind .output.text.text <Shift-1> { tkTextResetAnchor %W @%x,%y; set tkPriv(selectMode) char; tkTextSelectTo %W %x %y }
	bind .output.text.text <Double-Shift-1>	{ set tkPriv(selectMode) word; tkTextSelectTo %W %x %y }
	bind .output.text.text <Triple-Shift-1>	{ set tkPriv(selectMode) line; tkTextSelectTo %W %x %y }
	bind .output.text.text <B1-Leave> { set tkPriv(x) %x; set tkPriv(y) %y; tkTextAutoScan %W }
	bind .output.text.text <B1-Enter> { tkCancelRepeat }
	bind .output.text.text <ButtonRelease-1> { tkCancelRepeat }
	bind .output.text.text <Control-1> { %W mark set insert @%x,%y }
	bind .output.text.text <Left> { tkTextSetCursor %W insert-1c }
	bind .output.text.text <Right> { tkTextSetCursor %W insert+1c }
	bind .output.text.text <Up> { tkTextSetCursor %W [tkTextUpDownLine %W -1] }
	bind .output.text.text <Down> { tkTextSetCursor %W [tkTextUpDownLine %W 1] }
	bind .output.text.text <Shift-Left> { tkTextKeySelect %W [%W index {insert - 1c}] }
	bind .output.text.text <Shift-Right> { tkTextKeySelect %W [%W index {insert + 1c}] }
	bind .output.text.text <Shift-Up> { tkTextKeySelect %W [tkTextUpDownLine %W -1] }
	bind .output.text.text <Shift-Down> { tkTextKeySelect %W [tkTextUpDownLine %W 1] }
	bind .output.text.text <Control-Left> { tkTextSetCursor %W [tkTextPrevPos %W insert tcl_startOfPreviousWord] }
	bind .output.text.text <Control-Right> { tkTextSetCursor %W [tkTextNextWord %W insert] }
	bind .output.text.text <Control-Up> { tkTextSetCursor %W [tkTextPrevPara %W insert] }
	bind .output.text.text <Control-Down> { tkTextSetCursor %W [tkTextNextPara %W insert] }
	bind .output.text.text <Shift-Control-Left> { tkTextKeySelect %W [tkTextPrevPos %W insert tcl_startOfPreviousWord] }
	bind .output.text.text <Shift-Control-Right> { tkTextKeySelect %W [tkTextNextWord %W insert] }
	bind .output.text.text <Shift-Control-Up> { tkTextKeySelect %W [tkTextPrevPara %W insert] }
	bind .output.text.text <Shift-Control-Down> { tkTextKeySelect %W [tkTextNextPara %W insert] }
	bind .output.text.text <Prior> { tkTextSetCursor %W [tkTextScrollPages %W -1] }
	bind .output.text.text <Shift-Prior> { tkTextKeySelect %W [tkTextScrollPages %W -1] }
	bind .output.text.text <Next> { tkTextSetCursor %W [tkTextScrollPages %W 1] }
	bind .output.text.text <Shift-Next> { tkTextKeySelect %W [tkTextScrollPages %W 1] }
	bind .output.text.text <Control-Prior> { %W xview scroll -1 page }
	bind .output.text.text <Control-Next> { %W xview scroll 1 page }
	bind .output.text.text <Home> { tkTextSetCursor %W {insert linestart} }
	bind .output.text.text <Shift-Home> { tkTextKeySelect %W {insert linestart} }
	bind .output.text.text <End> { tkTextSetCursor %W {insert lineend} }
	bind .output.text.text <Shift-End> { tkTextKeySelect %W {insert lineend} }
	bind .output.text.text <Control-Home> { tkTextSetCursor %W 1.0 }
	bind .output.text.text <Control-Shift-Home> { tkTextKeySelect %W 1.0 }
	bind .output.text.text <Control-End> { tkTextSetCursor %W {end - 1 char} }
	bind .output.text.text <Control-Shift-End> { tkTextKeySelect %W {end - 1 char} }
	}

if {[string equal $tcl_version 8.4]} {
	bind .output.text.text <1> { tk::TextButton1 %W %x %y; %W tag remove sel 0.0 end }
	bind .output.text.text <B1-Motion> { set tk::Priv(x) %x; set tk::Priv(y) %y; tk::TextSelectTo %W %x %y }
	bind .output.text.text <Double-1> { set tk::Priv(selectMode) word; tk::TextSelectTo %W %x %y; catch {%W mark set insert sel.last} }
	bind .output.text.text <Triple-1> { set tk::Priv(selectMode) line; tk::TextSelectTo %W %x %y; catch {%W mark set insert sel.last} }
	bind .output.text.text <Shift-1> { tk::TextResetAnchor %W @%x,%y; set tk::Priv(selectMode) char; tk::TextSelectTo %W %x %y }
	bind .output.text.text <Double-Shift-1>	{ set tk::Priv(selectMode) word; tk::TextSelectTo %W %x %y 1 }
	bind .output.text.text <Triple-Shift-1>	{ set tk::Priv(selectMode) line; tk::TextSelectTo %W %x %y }
	bind .output.text.text <B1-Leave> { set tk::Priv(x) %x; set tk::Priv(y) %y; tk::TextAutoScan %W }
	bind .output.text.text <B1-Enter> { tk::CancelRepeat }
	bind .output.text.text <ButtonRelease-1> { tk::CancelRepeat }
	bind .output.text.text <Control-1> { %W mark set insert @%x,%y }
	bind .output.text.text <Left> { tk::TextSetCursor %W insert-1c }
	bind .output.text.text <Right> { tk::TextSetCursor %W insert+1c }
	bind .output.text.text <Up> { tk::TextSetCursor %W [tk::TextUpDownLine %W -1] }
	bind .output.text.text <Down> { tk::TextSetCursor %W [tk::TextUpDownLine %W 1] }
	bind .output.text.text <Shift-Left> { tk::TextKeySelect %W [%W index {insert - 1c}] }
	bind .output.text.text <Shift-Right> { tk::TextKeySelect %W [%W index {insert + 1c}] }
	bind .output.text.text <Shift-Up> { tk::TextKeySelect %W [tk::TextUpDownLine %W -1] }
	bind .output.text.text <Shift-Down> { tk::TextKeySelect %W [tk::TextUpDownLine %W 1] }
	bind .output.text.text <Control-Left> { tk::TextSetCursor %W [tk::TextPrevPos %W insert tcl_startOfPreviousWord] }
	bind .output.text.text <Control-Right> { tk::TextSetCursor %W [tk::TextNextWord %W insert] }
	bind .output.text.text <Control-Up> { tk::TextSetCursor %W [tk::TextPrevPara %W insert] }
	bind .output.text.text <Control-Down> { tk::TextSetCursor %W [tk::TextNextPara %W insert] }
	bind .output.text.text <Shift-Control-Left> { tk::TextKeySelect %W [tk::TextPrevPos %W insert tcl_startOfPreviousWord] }
	bind .output.text.text <Shift-Control-Right> { tk::TextKeySelect %W [tk::TextNextWord %W insert] }
	bind .output.text.text <Shift-Control-Up> { tk::TextKeySelect %W [tk::TextPrevPara %W insert] }
	bind .output.text.text <Shift-Control-Down> { tk::TextKeySelect %W [tk::TextNextPara %W insert] }
	bind .output.text.text <Prior> { tk::TextSetCursor %W [tk::TextScrollPages %W -1] }
	bind .output.text.text <Shift-Prior> { tk::TextKeySelect %W [tk::TextScrollPages %W -1] }
	bind .output.text.text <Next> { tk::TextSetCursor %W [tk::TextScrollPages %W 1] }
	bind .output.text.text <Shift-Next> { tk::TextKeySelect %W [tk::TextScrollPages %W 1] }
	bind .output.text.text <Control-Prior> { %W xview scroll -1 page }
	bind .output.text.text <Control-Next> { %W xview scroll 1 page }
	bind .output.text.text <Home> { tk::TextSetCursor %W {insert linestart} }
	bind .output.text.text <Shift-Home> { tk::TextKeySelect %W {insert linestart} }
	bind .output.text.text <End> { tk::TextSetCursor %W {insert lineend} }
	bind .output.text.text <Shift-End> { tk::TextKeySelect %W {insert lineend} }
	bind .output.text.text <Control-Home> { tk::TextSetCursor %W 1.0 }
	bind .output.text.text <Control-Shift-Home> { tk::TextKeySelect %W 1.0 }
	bind .output.text.text <Control-End> { tk::TextSetCursor %W {end - 1 char} }
	bind .output.text.text <Control-Shift-End> { tk::TextKeySelect %W {end - 1 char} }
	}

wm protocol .output WM_DELETE_WINDOW {wm withdraw .output}
wm withdraw .output

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# Search dialog
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

button .output.mbar.find -bg navy -fg white -relief flat -highlightthickness 0 -text "Find:" -command {output_search "$search_string"}
entry .output.mbar.entry -textvariable search_string
bind .output.mbar.entry <Return> {output_search "$search_string"}
pack .output.mbar.find .output.mbar.entry -side left

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# Dialog for global text search and replace
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

proc replace_text {find_text find_case repl_text repl_which} {
	global changed element_value element_list
	set where [$element_value index insert]
	save_current
	replace "$find_text" $find_case "$repl_text" "$repl_which"
	set_current $element_list
	$element_value mark set insert $where
	}

set repl_which this

toplevel .replace
label .replace.intro -text "Change text in element values"
radiobutton .replace.this   -text "just this element" -variable repl_which -value this
radiobutton .replace.within -text "this subtree"      -variable repl_which -value within
radiobutton .replace.all    -text "the whole file"    -variable repl_which -value all
label .replace.findlabel -text "Find what:"
entry .replace.findentry -textvariable find_text
label .replace.repllabel -text "Replace with:"
entry .replace.replentry -textvariable repl_text
checkbutton .replace.usecase -text "Match Case" -variable find_case
button .replace.exec -text "Replace All" -command {replace_text $find_text $find_case $repl_text $repl_which}
button .replace.dismiss -text "Close" -command {wm withdraw .replace}

grid .replace.intro -columnspan 2 -pady 8 -padx 4
grid .replace.findlabel .replace.findentry -padx 4 -pady 2
grid .replace.repllabel .replace.replentry -padx 4 -pady 2
grid .replace.findlabel -sticky w
grid .replace.repllabel -sticky w
grid .replace.this   .replace.usecase -padx 4 -pady 2
grid .replace.within .replace.exec -padx 4 -pady 2
grid .replace.all    .replace.dismiss -padx 4 -pady 2
grid .replace.this -sticky w
grid .replace.within -sticky w
grid .replace.all -sticky w
grid .replace.exec -sticky e
grid .replace.dismiss -sticky e

wm title .replace "Tkme: Replace"
wm protocol .replace WM_DELETE_WINDOW {wm withdraw .replace}
wm withdraw .replace

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# Key assignments of virtual events
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

event add <<list-up>>       <Alt-Key-Up>    <Alt-Key-KP_8>
event add <<list-down>>     <Alt-Key-Down>  <Alt-Key-KP_2>
event add <<list-home>>     <Alt-Key-Home>  <Alt-Key-KP_7>
event add <<list-end>>      <Alt-Key-End>   <Alt-Key-KP_1>
event add <<list-pageup>>   <Alt-Key-Prior> <Alt-Key-KP_9>
event add <<list-pagedown>> <Alt-Key-Next>  <Alt-Key-KP_3>
event add <<hide>>          <Alt-Key-Left>  <Alt-Key-KP_4>
event add <<show>>          <Alt-Key-Right> <Alt-Key-KP_6>
event add <<list-expand>>   <Alt-Key-plus>  <Alt-Key-KP_Add>
event add <<list-collapse>> <Alt-Key-minus> <Alt-Key-KP_Subtract>

event add <<open>>          <Control-Key-o>
event add <<save>>          <Control-Key-s>
event add <<quit>>          <Control-Key-q>
event add <<Cut>>           <Control-Key-x>
event add <<Copy>>          <Control-Key-c>
event add <<Paste>>         <Control-Key-v>

event add <<duplicate>>     <Alt-Key-d>
event add <<swap>>          <Alt-Key-quoteleft>
event add <<prune>>         <Alt-Key-p>
event add <<add-all>>       <Alt-Key-a>
event add <<today>>         <Alt-Key-t>
event add <<help>>          <Key-F1>
event add <<output>>        <Key-F2>

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# Bindings for virtual events
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

bind . <<list-up>>       {list_up}
bind . <<list-down>>     {list_down}
bind . <<list-home>>     {list_home}
bind . <<list-end>>      {list_end}
bind . <<list-pageup>>   {list_pageup}
bind . <<list-pagedown>> {list_pagedown}

bind . <<hide>>          {view_hide}
bind . <<show>>          {view_show}
bind . <<list-expand>>   {list_expand}
bind . <<list-collapse>> {list_collapse}

bind . <<open>>          {file_open}
bind . <<save>>          {file_save}
bind . <<quit>>          {file_quit}
bind Text <<Cut>>        {edit_cut %W}
bind Text <<Copy>>       {edit_copy %W}
bind Text <<Paste>>      {edit_paste %W}
bind . <<duplicate>>     {edit_duplicate}
bind . <<swap>>          {edit_swap}
bind . <<prune>>         {edit_prune}
bind . <<help>>          {help_element}
bind . <<output>>        {help_output}
bind . <<add-all>>       {add_all}

bind . <<today>>         {today}

if {[string equal $tcl_version 8.3]} {
	bind $element_value <Control-KP_4> { tkTextSetCursor %W [tkTextPrevPos %W insert tcl_startOfPreviousWord] }
	bind $element_value <Control-KP_6> { tkTextSetCursor %W [tkTextNextWord %W insert] }
	bind $element_value <Control-KP_8> { tkTextSetCursor %W [tkTextPrevPara %W insert] }
	bind $element_value <Control-KP_2> { tkTextSetCursor %W [tkTextNextPara %W insert] }
	bind $element_value <Shift-Control-KP_4> { tkTextKeySelect %W [tkTextPrevPos %W insert tcl_startOfPreviousWord] }
	bind $element_value <Shift-Control-KP_6> { tkTextKeySelect %W [tkTextNextWord %W insert] }
	bind $element_value <Shift-Control-KP_8> { tkTextKeySelect %W [tkTextPrevPara %W insert] }
	bind $element_value <Shift-Control-KP_2> { tkTextKeySelect %W [tkTextNextPara %W insert] }
	bind $element_value <Control-KP_7> { tkTextSetCursor %W 1.0 }
	bind $element_value <Control-Shift-KP_7> { tkTextKeySelect %W 1.0 }
	bind $element_value <Control-KP_1> { tkTextSetCursor %W {end - 1 char} }
	bind $element_value <Control-Shift-KP_1> { tkTextKeySelect %W {end - 1 char} }
	}

if {[string equal $tcl_version 8.4]} {
	bind $element_value <Control-KP_4> { tk::TextSetCursor %W [tk::TextPrevPos %W insert tcl_startOfPreviousWord] }
	bind $element_value <Control-KP_6> { tk::TextSetCursor %W [tk::TextNextWord %W insert] }
	bind $element_value <Control-KP_8> { tk::TextSetCursor %W [tk::TextPrevPara %W insert] }
	bind $element_value <Control-KP_2> { tk::TextSetCursor %W [tk::TextNextPara %W insert] }
	bind $element_value <Shift-Control-KP_4> { tk::TextKeySelect %W [tk::TextPrevPos %W insert tcl_startOfPreviousWord] }
	bind $element_value <Shift-Control-KP_6> { tk::TextKeySelect %W [tk::TextNextWord %W insert] }
	bind $element_value <Shift-Control-KP_8> { tk::TextKeySelect %W [tk::TextPrevPara %W insert] }
	bind $element_value <Shift-Control-KP_2> { tk::TextKeySelect %W [tk::TextNextPara %W insert] }
	bind $element_value <Control-KP_7> { tk::TextSetCursor %W 1.0 }
	bind $element_value <Control-Shift-KP_7> { tk::TextKeySelect %W 1.0 }
	bind $element_value <Control-KP_1> { tk::TextSetCursor %W {end - 1 char} }
	bind $element_value <Control-Shift-KP_1> { tk::TextKeySelect %W {end - 1 char} }
	}

bind . <Control-F4> {focus .paned.value.text}
bind . <Control-F5> {.paned.value.text configure -insertbackground DarkRed -insertborderwidth 0 -insertwidth 3}

if {[string equal $tcl_version 8.3]} {
	bind . <Control-F6> {tkTextSetCursor .paned.value.text insert}
	}
if {[string equal $tcl_version 8.4]} {
	bind . <Control-F6> {tk::TextSetCursor .paned.value.text insert}
	}
bind . <Control-F7> {topology}

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# Window-manager setup of the main window
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

wm protocol . WM_DELETE_WINDOW {file_quit}
wm title . "Tkme: $file_name"

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# Read the input file if one was specified on the command line
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

read_metadata [cmdline_file]

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# Gray out the "configure" menu option if a config file was specified
# on the command line
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

if {[config_read]} {
	.mbar.file.menu entryconfigure 4 -state disabled
	}

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# Show the user any error messages generated while reading the config or
# extension files.
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

if {[string length [cfg_messages]] > 0 || [string length [ext_messages]] > 0} {

	toplevel .messages
	frame .messages.mbar -relief flat -bd 3
	.messages.mbar config -background navy
	pack .messages.mbar -side top -expand false -fill x
	button .messages.mbar.close -text "Close"  -font menu_font -bg navy -fg white -relief flat -highlightthickness 0 -command {wm withdraw .messages}
	pack .messages.mbar.close -side left

	ScrolledText .messages.text 80 20
	.messages.text.text configure -font output_font
	.messages.text.text configure -selectbackground navy -selectforeground white
	.messages.text.text configure -exportselection false
	pack .messages.text -fill both -expand true
	.messages.text.text configure -wrap word

	if {[string length [cfg_messages]] > 0} {
		.messages.text.text insert end "Errors or warnings while reading the config file:\n"
		.messages.text.text insert end "[cfg_messages]"
		}

	if {[string length [ext_messages]] > 0} {
		.messages.text.text insert end "Errors or warnings while reading extension files:\n"
		.messages.text.text insert end "[ext_messages]"
		}
	}

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# If window size and position were saved, try to restore them
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

set geometry_file [file join [file dirname $recent_file] geometry.tcl]

if {[file exists $geometry_file]} {
	source $geometry_file
	}

#-------------------------------------------------------------------------
# End
#-------------------------------------------------------------------------
