/*----------------------------------------------------------------------*\
 | Code to upgrade version 1 metadata to version 2						|
 |																		|
 | Peter N. Schweitzer (U.S. Geological Survey, Reston, VA 20192)		|
\*----------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "keyword.h"
#include "item.h"
#include "stricmp.h"

extern FILE *out;

#define CSDGM2_ID	"FGDC-STD-001-1998"

/*----------------------------------------------------------------------*\
\*----------------------------------------------------------------------*/

void upgrade (struct item *root) {
	struct item *a,*b,*c;
	struct item *p,*q,*r,*u,*v;
	char *s;
	int n;

	if (root == NULL) return;

	if (out == NULL) out = stderr;

	/*------------------------------------------------------------------*\
	 | Convert G-Ring_Latitude and G-Ring_Longitude to G-Ring
	\*------------------------------------------------------------------*/

	if (p = find_key (root,WSpatial_Domain))
		if (p = find_key (p,WData_Set_G_Polygon))
			while (p) {

				if (q = find_key (p,WData_Set_G_Polygon_Outer_G_Ring)) {
					if (r = q->child)
						if (r->key == WG_Ring_Latitude ||
							r->key == WG_Ring_Longitude) {
							u = insert_item_before (r);
							u->key = WG_Ring;
							u->next = NULL;
							r->prev = NULL;

							fprintf (out,"Upgrade (line %d): %s added to contain the coordinates of %s\n",r->line_number,text_of(u->key),text_of(p->key));

							/* Size the G-Ring */

							n = 0;
							for (v=r; v; v=v->next)
								if (v->child)
									if (v->child->d)
										n += 1 + strlen (v->child->d);

							/* Allocate a buffer to hold the G-Ring */

							if (v = add_child (u)) {
								v->key = Wunknown;
								v->line_number = r->line_number;
								if (v->d = (char *) malloc (1 + n)) {
									*v->d = 0;
									while (r) {
										if ((r->key == WG_Ring_Latitude) ||
											(r->key == WG_Ring_Latitude)) {
											char *x = NULL;
											char *y = NULL;

											if (r->key == WG_Ring_Latitude)
												if (r->child) y = r->child->d;
											if (r->key == WG_Ring_Longitude)
												if (r->child) x = r->child->d;
											r = r->next;
											while (r && r->key == Wblank) r = r->next;
											if (r->key == WG_Ring_Latitude)
												if (r->child) y = r->child->d;
											if (r->key == WG_Ring_Longitude)
												if (r->child) x = r->child->d;

											if (x && y) {
												strcat (v->d,x);
												strcat (v->d,",");
												strcat (v->d,y);
												if (r->next)
													if (r->next->key != Wblank)
														strcat (v->d,",");
												}
											}
										r = r->next;
										}
									}
								if (*v->d == 0) delete_item (v);
								}
							}

					while (q = find_next_key (q,WData_Set_G_Polygon_Exclusion_G_Ring))
						if (r = q->child)
							if (r->key == WG_Ring_Latitude ||
								r->key == WG_Ring_Longitude) {
								u = insert_item_before (r);
								u->key = WG_Ring;
								u->next = NULL;
								r->prev = NULL;

								fprintf (out,"Upgrade (line %d): %s added to contain coordinates of %s\n",r->line_number,text_of(u->key),text_of(q->key));

								/* Size the G-Ring */

								n = 0;
								for (v=r; v; v=v->next)
									if (v->child)
										if (v->child->d)
											n += 1 + strlen (v->child->d);

								/* Allocate a buffer to hold the G-Ring */

								if (v = add_child (u)) {
									v->key = Wunknown;
									v->line_number = r->line_number;
									if (v->d = (char *) malloc (1 + n)) {
										*v->d = 0;
										while (r) {
											if ((r->key == WG_Ring_Latitude) ||
												(r->key == WG_Ring_Latitude)) {
												char *x = NULL;
												char *y = NULL;

												if (r->key == WG_Ring_Latitude)
													if (r->child) y = r->child->d;
												if (r->key == WG_Ring_Longitude)
													if (r->child) x = r->child->d;
												r = r->next;
												while (r && r->key == Wblank) r = r->next;
												if (r->key == WG_Ring_Latitude)
													if (r->child) y = r->child->d;
												if (r->key == WG_Ring_Longitude)
													if (r->child) x = r->child->d;

												if (x && y) {
													strcat (v->d,x);
													strcat (v->d,",");
													strcat (v->d,y);
													if (r->next)
														if (r->next->key != Wblank)
															strcat (v->d,",");
													}
												}
											r = r->next;
											}
										}
									if (*v->d == 0) delete_item (v);
									}
								}
					}

				p = find_next_key (p,WData_Set_G_Polygon);
				}

	/*------------------------------------------------------------------*\
	 | Add new Theme for each Theme_Keyword_Thesaurus
	\*------------------------------------------------------------------*/

	if (p = find_key (root,WKeywords))
		if (q = find_key (p,WTheme))
			do {
				/*------------------------------------------------------*\
				 | If there is exactly one Theme_Keyword_Thesaurus,		|
				 | then don't change anything.							|
				\*------------------------------------------------------*/

				if (r = find_key (q,WTheme_Keyword_Thesaurus))
					if (find_next_key (r,WTheme_Keyword_Thesaurus)) {

						/*----------------------------------------------*\
						 | If we found a second thesaurus, we want to	|
						 | subdivide the section, adding a new Theme	|
						 | for each thesaurus.							|
						\*----------------------------------------------*/

						for (r = q->child; r->key == Wblank; r = r->next);

						r = r->next;	/* Skip over the first non-blank element */
						while (r) {
							if (r->key == WTheme_Keyword_Thesaurus) {
								u = insert_item_after (r->parent);
								u->key = WTheme;
								r->prev->next = NULL;
								r->prev = NULL;
								r->parent = u;
								u->line_number = r->line_number;
								u->child = r;
								fprintf (out,"Upgrade (line %d): %s added for this %s\n",r->line_number,text_of(u->key),text_of(r->key));
								for (v=r; v; v=v->next) v->parent = u;
								break;
								}
							else r = r->next;
							}
						}
				} while (q = find_next_key (q,q->key));

	/*------------------------------------------------------------------*\
	 | Add new Place for each Place_Keyword_Thesaurus
	\*------------------------------------------------------------------*/

	if (p = find_key (root,WKeywords))
		if (q = find_key (p,WPlace))
			do {
				/*------------------------------------------------------*\
				 | If there is exactly one Place_Keyword_Thesaurus,		|
				 | then don't change anything.							|
				\*------------------------------------------------------*/

				if (r = find_key (q,WPlace_Keyword_Thesaurus))
					if (find_next_key (r,WPlace_Keyword_Thesaurus)) {

						/*----------------------------------------------*\
						 | If we found a second thesaurus, we want to	|
						 | subdivide the section, adding a new Place for|
						 | each thesaurus.								|
						\*----------------------------------------------*/

						for (r = q->child; r->key == Wblank; r = r->next);

						r = r->next;	/* Skip over the first non-blank element */
						while (r) {
							if (r->key == WPlace_Keyword_Thesaurus) {
								u = insert_item_after (r->parent);
								u->key = WPlace;
								r->prev->next = NULL;
								r->prev = NULL;
								r->parent = u;
								u->line_number = r->line_number;
								u->child = r;
								fprintf (out,"Upgrade (line %d): %s added for this %s\n",r->line_number,text_of(u->key),text_of(r->key));
								for (v=r; v; v=v->next) v->parent = u;
								break;
								}
							else r = r->next;
							}
						}
				} while (q = find_next_key (q,q->key));

	/*------------------------------------------------------------------*\
	 | Add new Stratum for each Stratum_Keyword_Thesaurus
	\*------------------------------------------------------------------*/

	if (p = find_key (root,WKeywords))
		if (q = find_key (p,WStratum))
			do {
				/*------------------------------------------------------*\
				 | If there is exactly one Stratum_Keyword_Thesaurus,	|
				 | then don't change anything.							|
				\*------------------------------------------------------*/

				if (r = find_key (q,WStratum_Keyword_Thesaurus))
					if (find_next_key (r,WStratum_Keyword_Thesaurus)) {

						/*----------------------------------------------*\
						 | If we found a second thesaurus, we want to	|
						 | subdivide the section, adding a new Stratum	|
						 | for each thesaurus.							|
						\*----------------------------------------------*/

						for (r = q->child; r->key == Wblank; r = r->next);

						r = r->next;	/* Skip over the first non-blank element */
						while (r) {
							if (r->key == WStratum_Keyword_Thesaurus) {
								u = insert_item_after (r->parent);
								u->key = WStratum;
								r->prev->next = NULL;
								r->prev = NULL;
								r->parent = u;
								u->line_number = r->line_number;
								u->child = r;
								fprintf (out,"Upgrade (line %d): %s added for this %s\n",r->line_number,text_of(u->key),text_of(r->key));
								for (v=r; v; v=v->next) v->parent = u;
								break;
								}
							else r = r->next;
							}
						}
				} while (q = find_next_key (q,q->key));

	/*------------------------------------------------------------------*\
	 | Add new Temporal for each Temporal_Keyword_Thesaurus
	\*------------------------------------------------------------------*/

	if (p = find_key (root,WKeywords))
		if (q = find_key (p,WTemporal))
			do {
				/*------------------------------------------------------*\
				 | If there is exactly one Temporal_Keyword_Thesaurus,	|
				 | then don't change anything.							|
				\*------------------------------------------------------*/

				if (r = find_key (q,WTemporal_Keyword_Thesaurus))
					if (find_next_key (r,WTemporal_Keyword_Thesaurus)) {

						/*----------------------------------------------*\
						 | If we found a second thesaurus, we want to	|
						 | subdivide the section, adding a new Temporal	|
						 | for each thesaurus.							|
						\*----------------------------------------------*/

						for (r = q->child; r->key == Wblank; r = r->next);

						r = r->next;	/* Skip over the first non-blank element */
						while (r) {
							if (r->key == WTemporal_Keyword_Thesaurus) {
								u = insert_item_after (r->parent);
								u->key = WTemporal;
								r->prev->next = NULL;
								r->prev = NULL;
								r->parent = u;
								u->line_number = r->line_number;
								u->child = r;
								fprintf (out,"Upgrade (line %d): %s added for this %s\n",r->line_number,text_of(u->key),text_of(r->key));
								for (v=r; v; v=v->next) v->parent = u;
								break;
								}
							else r = r->next;
							}
						}
				} while (q = find_next_key (q,q->key));

	/*------------------------------------------------------------------*\
	 | Add new SDTS_Terms_Description for each SDTS_Point_and_Vector_Object_Type
	\*------------------------------------------------------------------*/

	if (p = find_key (root,WPoint_and_Vector_Object_Information))
		if (q = find_key (p,WSDTS_Terms_Description))
			do {
				if (r = q->child)
					while (r) {
						while (r && r->key != WSDTS_Point_and_Vector_Object_Type) r=r->next;
						if (r)
							if (r == r->parent->child) r = r->next;
							else {
								u = insert_item_after (r->parent);
								u->key = WSDTS_Terms_Description;
								r->prev->next = NULL;
								r->prev = NULL;
								r->parent = u;
								u->line_number = r->line_number;
								u->child = r;
								fprintf (out,"Upgrade (line %d): %s added for this %s\n",r->line_number,text_of(u->key),text_of(r->key));
								for (v=r; v; v=v->next) v->parent = u;
								}
						}
				} while (q = find_next_key (q,WSDTS_Terms_Description));

	/*------------------------------------------------------------------*\
	 | Add new VPF_Point_and_Vector_Object_Information for each
	 | VPF_Point_and_Vector_Object_Type
	\*------------------------------------------------------------------*/

	if (p = find_key (root,WVPF_Terms_Description)) {
		if (q = p->child) {

			/* Skip over VPF_Topology_Level, if present */

			if (q->key == WVPF_Topology_Level) q = q->next;

			/* Skip over any VPF_Point_and_Vector_Object_Information */

			while (q && (q->key == Wblank || q->key == WVPF_Point_and_Vector_Object_Information)) q = q->next;

			/* Detach remaining children of p */

			if (q) {
				if (q->prev) q->prev->next = NULL;
				q->prev = NULL;
				}

			/* Scan remaining (ex-) children of p */

			while (q) {
				if (q->key == WVPF_Point_and_Vector_Object_Type) {
					u = add_child (p);
					u->key = WVPF_Point_and_Vector_Object_Information;
					u->line_number = q->line_number;
					q->parent = u;
					u->child = q;
					if (q->prev) q->prev->next = NULL;
					q->prev = NULL;
					fprintf (out,"Upgrade (line %d): %s added for this %s\n",u->line_number,text_of(u->key),text_of(q->key));
					for (v=q; v; v=v->next) v->parent = u;
					}
				q = q->next;
				}
			}
		}

	/*------------------------------------------------------------------*\
	 | If Other_Projection's_Definition is found within Map_Projection,
	 | then move it into Map_Projection_Parameters.  If the latter does
	 | not yet exist, create it.
	\*------------------------------------------------------------------*/

	if (q = find_key (root,WOther_Projections_Definition)) {
		p = q->parent;
		if (p->key == WMap_Projection) {
			detach_item (q);
			if (r = find_key (p,WMap_Projection_Parameters)) {
				if (r->child) {
					for (u=r->child; u->next; u=u->next);
					u->next = q;
					}
				else
					r->child = q;
				q->parent = r;
				fprintf (out,"Upgrade (line %d): %s moved under %s\n",r->line_number,text_of(q->key),text_of(r->key));
				}
			else {
				r = add_child (p);
				r->key = WMap_Projection_Parameters;
				r->line_number = p->line_number;
				r->child = q;
				q->parent = r;
				fprintf (out,"Upgrade (line %d): %s created to contain %s\n",q->line_number,text_of(r->key),text_of(q->key));
				}
			}
		}

	/*------------------------------------------------------------------*\
	 | Add new Detailed_Description for each Entity_Type
	\*------------------------------------------------------------------*/

	if (p = find_key (root,WEntity_and_Attribute_Information))
		if (q = find_key (p,WDetailed_Description))
			if (r = q->child)
				while (r) {
					while (r && r->key != WEntity_Type) r=r->next;
					if (r)
						if (r == r->parent->child) r = r->next;
						else {
							u = insert_item_after (r->parent);
							u->key = WDetailed_Description;
							r->prev->next = NULL;
							r->prev = NULL;
							r->parent = u;
							u->line_number = r->line_number;
							u->child = r;
							fprintf (out,"Upgrade (line %d): %s added for this %s\n",r->line_number,text_of(u->key),text_of(r->key));
							for (v=r; v; v=v->next) v->parent = u;
							}
					}

	/*------------------------------------------------------------------*\
	 | Add new Enumerated_Domain for each Enumerated_Domain_Value
	 |
	 | For each Detailed_Description
	 |   For each Attribute
	 |     For each Attribute_Domain_Values
	 |       For each Enumerated_Domain
	 |         For each child of Enumerated_Domain
	\*------------------------------------------------------------------*/

	if (a = find_key (root,WEntity_and_Attribute_Information))
		if (b = find_key (a,WDetailed_Description))
			while (b) {
				if (c = find_key (b,WAttribute))
					while (c) {
						if (p = find_key (c,WAttribute_Domain_Values))
							while (p) {
								if (q = find_key (p,WEnumerated_Domain))
									while (q) {
										if (r = q->child) {

											/* Take out blanks--they'll only cause trouble here */
											for (r=q->child; r->next; r=r->next);
											while (r) {
												u = r->prev;
												if (r->key == Wblank) delete_item (r);
												r = u;
												}

											r = q->child;
											while (r) {
												while (r && r->key != WEnumerated_Domain_Value) r=r->next;
												if (r)
													/* Don't create a new Enumerated_Domain for the first _Value; it already has one */
													if (r == r->parent->child) r = r->next;
													else {
														u = insert_item_after (r->parent);
														u->key = WEnumerated_Domain;
														r->prev->next = NULL;
														r->prev = NULL;
														r->parent = u;
														u->line_number = r->line_number;
														u->child = r;
														fprintf (out,"Upgrade (line %d): %s added for this %s\n",r->line_number,text_of(u->key),text_of(r->key));
														for (v=r; v; v=v->next) v->parent = u;
														}
												}
											}
										q = find_next_key (q,WEnumerated_Domain);
										}
								p = find_next_key (p,WAttribute_Domain_Values);
								}
						c = find_next_key (c,WAttribute);
						}
				b = find_next_key (b,WDetailed_Description);
				}

	/*------------------------------------------------------------------*\
	 | Add new Attribute_Domain_Values for each Enumerated_Domain,
	 | Range_Domain, Codeset_Domain, or Unrepresentable_Domain beyond
	 | the first.
	 |
	 | For each Detailed_Description
	 |   For each Attribute
	 |     For each Attribute_Domain_Values
	 |       For each child beyond the first,
	\*------------------------------------------------------------------*/

	if (a = find_key (root,WEntity_and_Attribute_Information))
		if (b = find_key (a,WDetailed_Description))
			while (b) {
				if (c = find_key (b,WAttribute))
					while (c) {
						if (p = find_key (c,WAttribute_Domain_Values))
							while (p) {
								if (q = p->child) {


									/* Take out blanks--they'll only cause trouble here */
									for (q=p->child; q->next; q=q->next);
									while (q) {
										r = q->prev;
										if (q->key == Wblank) delete_item (q);
										q = r;
										}

									if (q = p->child) {
										for (r=p->child; r->next; r=r->next);
										while (r != q) {
											u = r->prev;
											if (r->key == WEnumerated_Domain ||
												r->key == WRange_Domain      ||
												r->key == WCodeset_Domain    ||
												r->key == WUnrepresentable_Domain) {

												/* Create an Attribute_Domain_Values to hold r */
												if (v = insert_item_after (p)) {
													v->key = WAttribute_Domain_Values;
													v->line_number = r->line_number;
													v->child = r;
													r->parent = v;
													if (r->next) r->next->prev = r->prev;
													if (r->prev) r->prev->next = r->next;
													r->prev = NULL;
													r->next = NULL;
													fprintf (out,"Upgrade (line %d): %s added for this %s\n",r->line_number,text_of(v->key),text_of(r->key));
													}
												}
											r = u;
											}
										}
									}
								p = find_next_key (p,WAttribute_Domain_Values);
								}
						c = find_next_key (c,WAttribute);
						}
				b = find_next_key (b,WDetailed_Description);
				}

	/*------------------------------------------------------------------*\
	 | For each Attribute, look for Attribute_Units_of_Measure and		|
	 | Range_Domain.  If both are present, detach the former and attach	|
	 | it to the latter.  Same for Attribute_Measurement_Resolution.	|
	\*------------------------------------------------------------------*/

	if (a = find_key (root,WEntity_and_Attribute_Information))
		if (b = find_key (a,WDetailed_Description))
			while (b) {
				if (c = find_key (b,WAttribute))
					while (c) {
						p = find_key (c,WRange_Domain);

						/* Attribute_Units_of_Measure */

						if (u = c->child) {
							while (u) {
								q = u;
								u = u->next;
								if (q->key == WAttribute_Units_of_Measure) {

									/*----------------------------------*\
									 | If you didn't find Range_Domain,	|
									 | create a Range_Domain in a new	|
									 | Attribute_Domain_Values			|
									\*----------------------------------*/

									if (!p) {
										v = add_child (c);
										v->key = WAttribute_Domain_Values;
										p = add_child (v);
										p->key = WRange_Domain;
										fprintf (out,"Upgrade (line %d): %s and %s created to hold %s\n",q->line_number,text_of(v->key),text_of(p->key),text_of(q->key));
										}

									/*----------------------------------*\
									 | Move Attribute_Units_of_Measure	|
									 | into the Range_Domain			|
									\*----------------------------------*/

									detach_item (q);
									if (r = p->child) {
										while (r->next) r = r->next;
										r->next = q;
										q->prev = r;
										q->parent = r->parent;
										fprintf (out,"Upgrade (line %d): %s moved to %s\n",q->line_number,text_of(q->key),text_of(p->key));
										}
									else {
										p->child = q;
										q->parent = p;
										fprintf (out,"Upgrade (line %d): %s moved to %s\n",q->line_number,text_of(q->key),text_of(p->key));
										}
									}
								}
							}

						/* Attribute_Measurement_Resolution */

						if (u = c->child) {
							while (u) {
								q = u;
								u = u->next;
								if (q->key == WAttribute_Measurement_Resolution) {

									/*----------------------------------*\
									 | If you didn't find Range_Domain,	|
									 | create a Range_Domain in a new	|
									 | Attribute_Domain_Values			|
									\*----------------------------------*/

									if (!p) {
										v = add_child (c);
										v->key = WAttribute_Domain_Values;
										p = add_child (v);
										p->key = WRange_Domain;
										fprintf (out,"Upgrade (line %d): %s and %s created to hold %s\n",q->line_number,text_of(v->key),text_of(p->key),text_of(q->key));
										}

									/*----------------------------------*\
									 | Attribute_Measurement_Resolution	|
									 | goes into the Range_Domain		|
									\*----------------------------------*/

									detach_item (q);
									if (r = p->child) {
										while (r->next) r = r->next;
										r->next = q;
										q->prev = r;
										q->parent = r->parent;
										fprintf (out,"Upgrade (line %d): %s moved to %s\n",q->line_number,text_of(q->key),text_of(p->key));
										}
									else {
										p->child = q;
										q->parent = p;
										fprintf (out,"Upgrade (line %d): %s moved to %s\n",q->line_number,text_of(q->key),text_of(p->key));
										}
									}
								}
							}

					c = find_next_key (c,WAttribute);
					}
				b = find_next_key (b,WDetailed_Description);
				}

	/*------------------------------------------------------------------*\
	 | Add new Distribution_Information for each Distributor
	\*------------------------------------------------------------------*/

	n = 0;
	if (q = find_key (root,WDistribution_Information))
		if (r = q->child)
			while (r) {
				while (r && r->key != WDistributor) r = r->next;
				if (r) {
					if (n > 0) {
						u = insert_item_after (r->parent);
						u->key = WDistribution_Information;
						r->prev->next = NULL;
						r->prev = NULL;
						r->parent = u;
						u->line_number = r->line_number;
						u->child = r;
						fprintf (out,"Upgrade (line %d): %s added for this %s\n",r->line_number,text_of(u->key),text_of(r->key));
						for (v=r; v; v=v->next) v->parent = u;
						}
					n++;
					r = r->next;
					}
				}

	/*------------------------------------------------------------------*\
	 | Add new Single_Date/Time for each Calendar_Date in any
	 | Multiple_Dates/Times
	\*------------------------------------------------------------------*/

	/* One Time_Period_of_Content */

	if (a = find_key (root,WTime_Period_of_Content))
		if (p = find_key (a,WMultiple_Dates_Times))
			if (q = p->child)
				while (q)
					if (q->key == WCalendar_Date) {
						u = add_child (p);
						u->key = WSingle_Date_Time;
						u->line_number = q->line_number;
						fprintf (out,"Upgrade (line %d): %s added for this %s\n",u->line_number,text_of(u->key),text_of(q->key));
						v = q->next;
						detach_item (q);
						q->parent = u;
						u->child = q;
						if (v)
							if (v->key == WTime_of_Day) {
								q = v;
								v = v->next;
								detach_item (q);
								q->parent = u;
								u->child->next = q;
								q->prev = u->child;
								}
						q = v;
						}
					else
						q = q->next;

	/* One Source_Time_Period_of_Content in each Source_Information */

	if (c = find_key (root,WSource_Information))
		while (c) {
			if (a = find_key (c,WSource_Time_Period_of_Content))
				if (p = find_key (a,WMultiple_Dates_Times))
					if (q = p->child)
						while (q)
							if (q->key == WCalendar_Date) {
								u = add_child (p);
								u->key = WSingle_Date_Time;
								u->line_number = q->line_number;
								fprintf (out,"Upgrade (line %d): %s added for this %s\n",u->line_number,text_of(u->key),text_of(q->key));
								v = q->next;
								detach_item (q);
								q->parent = u;
								u->child = q;
								if (v)
									if (v->key == WTime_of_Day) {
										q = v;
										v = v->next;
										detach_item (q);
										q->parent = u;
										u->child->next = q;
										q->prev = u->child;
										}
								q = v;
								}
							else
								q = q->next;
			c = find_next_key (c,WSource_Information);
			}

	/* One Available_Time_Period in each Distribution_Information */

	if (c = find_key (root,WDistribution_Information))
		while (c) {
			if (a = find_key (c,WAvailable_Time_Period))
				if (p = find_key (a,WMultiple_Dates_Times))
					if (q = p->child)
						while (q)
							if (q->key == WCalendar_Date) {
								u = add_child (p);
								u->key = WSingle_Date_Time;
								u->line_number = q->line_number;
								fprintf (out,"Upgrade (line %d): %s added for this %s\n",u->line_number,text_of(u->key),text_of(q->key));
								v = q->next;
								detach_item (q);
								q->parent = u;
								u->child = q;
								if (v)
									if (v->key == WTime_of_Day) {
										q = v;
										v = v->next;
										detach_item (q);
										q->parent = u;
										u->child->next = q;
										q->prev = u->child;
										}
								q = v;
								}
							else
								q = q->next;
			c = find_next_key (c,WSource_Information);
			}

	/*------------------------------------------------------------------*\
	 | Modify the value of Metadata_Standard_Version
	\*------------------------------------------------------------------*/

	if (p = find_key (root,WMetadata_Reference_Information))
		if (p = find_key (p,WMetadata_Standard_Version))
			if (q = p->child)
				if (q->key == Wunknown)
					if (q->d)
						if (memcmp (q->d,"FGDC-STD-",9) != 0)
							if (s = (char *) malloc (1 + strlen (CSDGM2_ID))) {
								strcpy (s,CSDGM2_ID);
								if (!(q = p->child)) {
									q = add_child (p);
									q->key = Wunknown;
									q->line_number = p->line_number;
									}
								q->d = s;
								q->len = 1 + strlen (CSDGM2_ID);
								fprintf (out,"Upgrade (line %d): value of %s set to %s\n",q->line_number,text_of(p->key),s);
								while (q->next) {
									r = q->next;
									detach_item (r);
									delete_item (r);
									}
								}

	}

/*----------------------------------------------------------------------*\
\*----------------------------------------------------------------------*/
