/*----------------------------------------------------------------------*\
 | Functions to return, for a given FGDC keyword, the z39.50 code.		|
 |																		|
 | Peter N. Schweitzer (U.S. Geological Survey, Reston, VA 20192)		|
\*----------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "keyword.h"
#include "local.h"
#include "item.h"

struct z3950_tag {
	int code;
	enum fgdc_keyword key;
	};

static struct z3950_tag z3950_tag_table[] = {
	{3000,	WMetadata},
	{3100,	WIdentification_Information},
	{3101,	WCitation},
	{3800,	WCitation_Information},
	{3102,	WDescription},
	{62,	WAbstract},
	{3104,	WPurpose},
	{3105,	WSupplemental_Information},
	{3103,	WTime_Period_of_Content},
	{3900,	WTime_Period_Information},
	{3106,	WCurrentness_Reference},
	{3107,	WStatus},
	{3108,	WProgress},
	{3109,	WMaintenance_and_Update_Frequency},
	{3110,	WSpatial_Domain},
	{3111,	WBounding_Coordinates},
	{3112,	WWest_Bounding_Coordinate},
	{3113,	WEast_Bounding_Coordinate},
	{3114,	WNorth_Bounding_Coordinate},
	{3115,	WSouth_Bounding_Coordinate},
	{3116,	WData_Set_G_Polygon},
	{3117,	WData_Set_G_Polygon_Outer_G_Ring},
	{3118,	WG_Ring_Latitude},
	{3119,	WG_Ring_Longitude},
	{3120,	WData_Set_G_Polygon_Exclusion_G_Ring},
	{3118,	WG_Ring_Latitude},
	{3119,	WG_Ring_Longitude},
	{3121,	WKeywords},
	{3122,	WTheme},
	{3123,	WTheme_Keyword_Thesaurus},
	{3124,	WTheme_Keyword},
	{58,	WPlace},
	{3126,	WPlace_Keyword_Thesaurus},
	{3127,	WPlace_Keyword},
	{3128,	WStratum},
	{3129,	WStratum_Keyword_Thesaurus},
	{3130,	WStratum_Keyword},
	{3131,	WTemporal},
	{3132,	WTemporal_Keyword_Thesaurus},
	{3133,	WTemporal_Keyword},
	{3134,	WAccess_Constraints},
	{3135,	WUse_Constraints},
	{3136,	WPoint_of_Contact},
	{3000,	WContact_Information},
	{3137,	WBrowse_Graphic},
	{3138,	WBrowse_Graphic_File_Name},
	{3139,	WBrowse_Graphic_File_Description},
	{3140,	WBrowse_Graphic_File_Type},
	{3141,	WData_Set_Credit},
	{3142,	WSecurity_Information},
	{3143,	WSecurity_Classification_System},
	{3144,	WSecurity_Classification},
	{3145,	WSecurity_Handling_Description},
	{3146,	WNative_Data_Set_Environment},
	{3147,	WCross_Reference},
	{3800,	WCitation_Information},
	{3200,	WData_Quality_Information},
	{3201,	WAttribute_Accuracy},
	{3202,	WAttribute_Accuracy_Report},
	{3203,	WQuantitative_Attribute_Accuracy_Assessment},
	{3204,	WAttribute_Accuracy_Value},
	{3205,	WAttribute_Accuracy_Explanation},
	{3206,	WLogical_Consistency_Report},
	{3207,	WCompleteness_Report},
	{3208,	WPositional_Accuracy},
	{3209,	WHorizontal_Positional_Accuracy},
	{3210,	WHorizontal_Positional_Accuracy_Report},
	{3211,	WQuantitative_Horizontal_Positional_Accuracy_Assessment},
	{3212,	WHorizontal_Positional_Accuracy_Value},
	{3213,	WHorizontal_Positional_Accuracy_Explanation},
	{3214,	WVertical_Positional_Accuracy},
	{3215,	WVertical_Positional_Accuracy_Report},
	{3216,	WQuantitative_Vertical_Positional_Accuracy_Assessment},
	{3217,	WVertical_Positional_Accuracy_Value},
	{3218,	WVertical_Positional_Accuracy_Explanation},
	{3219,	WLineage},
	{3220,	WSource_Information},
	{3221,	WSource_Citation},
	{3800,	WCitation_Information},
	{1024,	WSource_Scale_Denominator},
	{1031,	WType_of_Source_Media},
	{3223,	WSource_Time_Period_of_Content},
	{3900,	WTime_Period_Information},
	{3224,	WSource_Currentness_Reference},
	{3225,	WSource_Citation_Abbreviation},
	{3226,	WSource_Contribution},
	{3227,	WProcess_Step},
	{3228,	WProcess_Description},
	{3229,	WSource_Used_Citation_Abbreviation},
	{3230,	WProcess_Date},
	{3231,	WProcess_Time},
	{3232,	WSource_Produced_Citation_Abbreviation},
	{3233,	WProcess_Contact},
	{3000,	WContact_Information},
	{3234,	WCloud_Cover},
	{3300,	WSpatial_Data_Organization_Information},
	{3301,	WIndirect_Spatial_Reference},
	{3302,	WDirect_Spatial_Reference_Method},
	{3314,	WPoint_and_Vector_Object_Information},
	{3303,	WSDTS_Terms_Description},
	{3304,	WSDTS_Point_and_Vector_Object_Type},
	{3305,	WPoint_and_Vector_Object_Count},
	{3306,	WVPF_Terms_Description},
	{3307,	WVPF_Topology_Level},
	{3308,	WVPF_Point_and_Vector_Object_Type},
	{3305,	WPoint_and_Vector_Object_Count},
	{3309,	WRaster_Object_Information},
	{3310,	WRaster_Object_Type},
	{3311,	WRow_Count},
	{3312,	WColumn_Count},
	{3313,	WVertical_Count},
	{3400,	WSpatial_Reference_Information},
	{3401,	WHorizontal_Coordinate_System_Definition},
	{3402,	WGeographic},
	{3403,	WLatitude_Resolution},
	{3404,	WLongitude_Resolution},
	{3405,	WGeographic_Coordinate_Units},
	{3406,	WPlanar},
	{3407,	WMap_Projection},
	{3408,	WMap_Projection_Name},
	{3409,	WAlbers_Conical_Equal_Area},
	{3409,	WAzimuthal_Equidistant},
	{3409,	WEquidistant_Conic},
	{3409,	WEquirectangular},
	{3409,	WGeneral_Vertical_Near_sided_Perspective},
	{3409,	WGnomonic},
	{3409,	WLambert_Azimuthal_Equal_Area},
	{3409,	WLambert_Conformal_Conic},
	{3409,	WMercator},
	{3409,	WModified_Stereographic_for_Alaska},
	{3409,	WMiller_Cylindrical},
	{3409,	WOblique_Mercator},
	{3409,	WOrthographic},
	{3409,	WPolar_Stereographic},
	{3409,	WPolyconic},
	{3409,	WRobinson},
	{3409,	WSinusoidal},
	{3409,	WSpace_Oblique_Mercator_Landsat},
	{3409,	WStereographic},
	{3409,	WTransverse_Mercator},
	{3409,	Wvan_der_Grinten},
	{3410,	WStandard_Parallel},
	{3411,	WLongitude_of_Central_Meridian},
	{3412,	WLatitude_of_Projection_Origin},
	{3413,	WFalse_Easting},
	{3414,	WFalse_Northing},
	{3415,	WScale_Factor_at_Equator},
	{3416,	WHeight_of_Perspective_Point_Above_Surface},
	{3417,	WLongitude_of_Projection_Center},
	{3418,	WLatitude_of_Projection_Center},
	{3419,	WScale_Factor_at_Center_Line},
	{3420,	WOblique_Line_Azimuth},
	{3421,	WAzimuthal_Angle},
	{3422,	WAzimuth_Measure_Point_Longitude},
	{3423,	WOblique_Line_Point},
	{3424,	WOblique_Line_Latitude},
	{3425,	WOblique_Line_Longitude},
	{3426,	WStraight_Vertical_Longitude_from_Pole},
	{3427,	WScale_Factor_at_Projection_Origin},
	{3428,	WLandsat_Number},
	{3429,	WPath_Number},
	{3430,	WScale_Factor_at_Central_Meridian},
	{3431,	WOther_Projections_Definition},
	{3432,	WGrid_Coordinate_System},
	{3433,	WGrid_Coordinate_System_Name},
	{3434,	WUniversal_Transverse_Mercator},
	{3435,	WUTM_Zone_Number},
	{3436,	WUniversal_Polar_Stereographic},
	{3437,	WUPS_Zone_Identifier},
	{3438,	WState_Plane_Coordinate_System},
	{3439,	WSPCS_Zone_Identifier},
	{3440,	WARC_Coordinate_System},
	{3441,	WARC_System_Zone_Identifier},
	{3442,	WOther_Grid_Systems_Definition},
	{3443,	WLocal_Planar},
	{3444,	WLocal_Planar_Description},
	{3445,	WLocal_Planar_Georeference_Information},
	{3446,	WPlanar_Coordinate_Information},
	{3447,	WPlanar_Coordinate_Encoding_Method},
	{3448,	WCoordinate_Representation},
	{3449,	WAbscissa_Resolution},
	{3450,	WOrdinate_Resolution},
	{3451,	WDistance_and_Bearing_Representation},
	{3452,	WDistance_Resolution},
	{3453,	WBearing_Resolution},
	{3454,	WBearing_Units},
	{3455,	WBearing_Reference_Direction},
	{3456,	WBearing_Reference_Meridian},
	{3457,	WPlanar_Distance_Units},
	{3458,	WLocal},
	{3459,	WLocal_Description},
	{3460,	WLocal_Georeference_Information},
	{3461,	WGeodetic_Model},
	{3462,	WHorizontal_Datum_Name},
	{3463,	WEllipsoid_Name},
	{3464,	WSemi_major_Axis},
	{3465,	WDenominator_of_Flattening_Ratio},
	{3466,	WVertical_Coordinate_System_Definition},
	{3467,	WAltitude_System_Definition},
	{3468,	WAltitude_Datum_Name},
	{3469,	WAltitude_Resolution},
	{3470,	WAltitude_Distance_Units},
	{3471,	WAltitude_Encoding_Method},
	{3472,	WDepth_System_Definition},
	{3473,	WDepth_Datum_Name},
	{3474,	WDepth_Resolution},
	{3475,	WDepth_Distance_Units},
	{3476,	WDepth_Encoding_Method},
	{3500,	WEntity_and_Attribute_Information},
	{3501,	WDetailed_Description},
	{3502,	WEntity_Type},
	{3503,	WEntity_Type_Label},
	{3504,	WEntity_Type_Definition},
	{3505,	WEntity_Type_Definition_Source},
	{3506,	WAttribute},
	{3507,	WAttribute_Label},
	{3508,	WAttribute_Definition},
	{3509,	WAttribute_Definition_Source},
	{3510,	WAttribute_Domain_Values},
	{3511,	WEnumerated_Domain},
	{3512,	WEnumerated_Domain_Value},
	{3513,	WEnumerated_Domain_Value_Definition},
	{3514,	WEnumerated_Domain_Value_Definition_Source},
	{3515,	WRange_Domain},
	{3516,	WRange_Domain_Minimum},
	{3517,	WRange_Domain_Maximum},
	{3518,	WCodeset_Domain},
	{3519,	WCodeset_Name},
	{3520,	WCodeset_Source},
	{3521,	WUnrepresentable_Domain},
	{3522,	WAttribute_Units_of_Measure},
	{3523,	WAttribute_Measurement_Resolution},
	{3524,	WBeginning_Date_of_Attribute_Values},
	{3525,	WEnding_Date_of_Attribute_Values},
	{3526,	WAttribute_Value_Accuracy_Information},
	{3527,	WAttribute_Value_Accuracy},
	{3528,	WAttribute_Value_Accuracy_Explanation},
	{3529,	WAttribute_Measurement_Frequency},
	{3530,	WOverview_Description},
	{3531,	WEntity_and_Attribute_Overview},
	{3532,	WEntity_and_Attribute_Detail_Citation},
	{3601,	WDistribution_Information},
	{3602,	WDistributor},
	{3000,	WContact_Information},
	{3602,	WResource_Description},
	{3603,	WDistribution_Liability},
	{3604,	WStandard_Order_Process},
	{3605,	WNon_digital_Form},
	{3606,	WDigital_Form},
	{3607,	WDigital_Transfer_Information},
	{3608,	WFormat_Name},
	{3609,	WFormat_Version_Number},
	{3610,	WFormat_Version_Date},
	{3611,	WFormat_Specification},
	{3612,	WFormat_Information_Content},
	{3613,	WFile_Decompression_Technique},
	{3614,	WTransfer_Size},
	{3615,	WDigital_Transfer_Option},
	{3616,	WOnline_Option},
	{3617,	WComputer_Contact_Information},
	{3618,	WNetwork_Address},
	{3619,	WNetwork_Resource_Name},
	{3620,	WDialup_Instructions},
	{3621,	WLowest_BPS},
	{3622,	WHighest_BPS},
	{3623,	WNumber_DataBits},
	{3624,	WNumber_StopBits},
	{3625,	WParity},
	{3626,	WCompression_Support},
	{3627,	WDialup_Telephone},
	{3628,	WDialup_File_Name},
	{3629,	WAccess_Instructions},
	{3630,	WOnline_Computer_and_Operating_System},
	{3631,	WOffline_Option},
	{3632,	WOffline_Media},
	{3633,	WRecording_Capacity},
	{3634,	WRecording_Density},
	{3635,	WRecording_Density_Units},
	{3636,	WRecording_Format},
	{3637,	WCompatibility_Information},
	{3638,	WFees},
	{3639,	WOrdering_Instructions},
	{3640,	WTurnaround},
	{3641,	WCustom_Order_Process},
	{3642,	WTechnical_Prerequisites},
	{3643,	WAvailable_Time_Period},
	{3900,	WTime_Period_Information},
	{3700,	WMetadata_Reference_Information},
	{3701,	WMetadata_Date},
	{3702,	WMetadata_Review_Date},
	{3703,	WMetadata_Future_Review_Date},
	{3704,	WMetadata_Contact},
	{3000,	WContact_Information},
	{3705,	WMetadata_Standard_Name},
	{3706,	WMetadata_Standard_Version},
	{3707,	WMetadata_Time_Convention},
	{3708,	WMetadata_Access_Constraints},
	{3709,	WMetadata_Use_Constraints},
	{3710,	WMetadata_Security_Information},
	{3711,	WMetadata_Security_Classification_System},
	{3712,	WMetadata_Security_Classification},
	{3713,	WMetadata_Security_Handling_Description},
	{3800,	WCitation_Information},
	{1003,	WOriginator},
	{31,	WPublication_Date},
	{3803,	WPublication_Time},
	{4,		WTitle},
	{3815,	WEdition},
	{3805,	WGeospatial_Data_Presentation_Form},
	{3806,	WSeries_Information},
	{5,		WSeries_Name},
	{3808,	WIssue_Identification},
	{3809,	WPublication_Information},
	{59,	WPublication_Place},
	{1018,	WPublisher},
	{3812,	WOther_Citation_Details},
	{3813,	WOnline_Linkage},
	{3814,	WLarger_Work_Citation},
	{3800,	WCitation_Information},
	{3900,	WTime_Period_Information},
	{3902,	WSingle_Date_Time},
	{3903,	WCalendar_Date},
	{3904,	WTime_of_Day},
	{3905,	WMultiple_Dates_Times},
	{3906,	WRange_of_Dates_Times},
	{3907,	WBeginning_Date},
	{3908,	WBeginning_Time},
	{3909,	WEnding_Date},
	{3910,	WEnding_Time},
	{3000,	WContact_Information},
	{3001,	WContact_Person_Primary},
	{3002,	WContact_Person},
	{3003,	WContact_Organization},
	{3004,	WContact_Organization_Primary},
	{3003,	WContact_Organization},
	{3002,	WContact_Person},
	{3005,	WContact_Position},
	{3006,	WContact_Address},
	{3007,	WAddress_Type},
	{3008,	WAddress},
	{3009,	WCity},
	{3010,	WState_or_Province},
	{3011,	WPostal_Code},
	{3012,	WCountry},
	{3013,	WContact_Voice_Telephone},
	{3014,	WContact_TDD_TTY_Telephone},
	{3015,	WContact_Facsimile_Telephone},
	{3016,	WContact_Electronic_Mail_Address},
	{3017,	WHours_of_Service},
	{3018,	WContact_Instructions},
	{0,		Wnull}
	};
static int z3950_tag_table_length = 0;

static int compare (const void *e1, const void *e2) {
	struct z3950_tag *p = (struct z3950_tag *) e1;
	struct z3950_tag *q = (struct z3950_tag *) e2;
	return (p->key - q->key);
	}

/*----------------------------------------------------------------------*/

int z3950_tag_of (enum fgdc_keyword key) {
	int i;
	struct z3950_tag T,*pT;

	if (z3950_tag_table_length == 0) {
		for (i=0; z3950_tag_table[i].code; i++);
		z3950_tag_table_length = i;
		qsort (z3950_tag_table,z3950_tag_table_length,sizeof(struct z3950_tag),compare);
		}

	if (key == Wnull) return (0);
	if (key == Wunknown) return (0);

	if (is_extension (key)) return (z3950_of_extension (key));

	T.key = key;
	pT = (struct z3950_tag *) bsearch (&T,z3950_tag_table,z3950_tag_table_length,sizeof(struct z3950_tag),compare);
	if (pT) return (pT->code);
	else return (0);
	}

/*----------------------------------------------------------------------*/

enum fgdc_keyword key_of_z3950_tag (int z3950_tag) {
	int i;

	if (z3950_tag_table_length == 0) {
		for (i=0; z3950_tag_table[i].code; i++);
		z3950_tag_table_length = i;
		qsort (z3950_tag_table,z3950_tag_table_length,sizeof(struct z3950_tag),compare);
		}

	for (i=0; i < z3950_tag_table_length; i++)
		if (z3950_tag_table[i].code == z3950_tag) return (z3950_tag_table[i].key);

	return (Wunknown);
	}

/*----------------------------------------------------------------------*\
\*----------------------------------------------------------------------*/
