/*----------------------------------------------------------------------*\
 | Functions to retrieve a keyword's text given its symbolic name and	|
 | vice versa.															|
 |																		|
 | Peter N. Schweitzer (U.S. Geological Survey, Reston, VA 20192)		|
\*----------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <ctype.h>

#include "keyword.h"

#ifndef PRINT_NAMES
	#include "local.h"
#endif

#include "stricmp.h"

struct keyword_list {
	enum fgdc_keyword key;
	char *text;
	};

/*----------------------------------------------------------------------*\
 | English
\*----------------------------------------------------------------------*/

static struct keyword_list en_std[] = {
	{WAbscissa_Resolution,
	 "Abscissa_Resolution"},
	{WAbstract,
	 "Abstract"},
	{WAccess_Constraints,
	 "Access_Constraints"},
	{WAccess_Instructions,
	 "Access_Instructions"},
	{WAddress,
	 "Address"},
	{WAddress_Type,
	 "Address_Type"},
	{WAlbers_Conical_Equal_Area,
	 "Albers_Conical_Equal_Area"},
	{WAltitude_Datum_Name,
	 "Altitude_Datum_Name"},
	{WAltitude_Distance_Units,
	 "Altitude_Distance_Units"},
	{WAltitude_Encoding_Method,
	 "Altitude_Encoding_Method"},
	{WAltitude_Resolution,
	 "Altitude_Resolution"},
	{WAltitude_System_Definition,
	 "Altitude_System_Definition"},
	{WARC_Coordinate_System,
	 "ARC_Coordinate_System"},
	{WARC_System_Zone_Identifier,
	 "ARC_System_Zone_Identifier"},
	{WAttribute,
	 "Attribute"},
	{WAttribute_Accuracy,
	 "Attribute_Accuracy"},
	{WAttribute_Accuracy_Explanation,
	 "Attribute_Accuracy_Explanation"},
	{WAttribute_Accuracy_Report,
	 "Attribute_Accuracy_Report"},
	{WAttribute_Accuracy_Value,
	 "Attribute_Accuracy_Value"},
	{WAttribute_Definition,
	 "Attribute_Definition"},
	{WAttribute_Definition_Source,
	 "Attribute_Definition_Source"},
	{WAttribute_Domain_Values,
	 "Attribute_Domain_Values"},
	{WAttribute_Label,
	 "Attribute_Label"},
	{WAttribute_Measurement_Frequency,
	 "Attribute_Measurement_Frequency"},
	{WAttribute_Measurement_Resolution,
	 "Attribute_Measurement_Resolution"},
	{WAttribute_Units_of_Measure,
	 "Attribute_Units_of_Measure"},
	{WAttribute_Value_Accuracy,
	 "Attribute_Value_Accuracy"},
	{WAttribute_Value_Accuracy_Explanation,
	 "Attribute_Value_Accuracy_Explanation"},
	{WAttribute_Value_Accuracy_Information,
	 "Attribute_Value_Accuracy_Information"},
	{WAvailable_Time_Period,
	 "Available_Time_Period"},
	{WAzimuthal_Angle,
	 "Azimuthal_Angle"},
	{WAzimuthal_Equidistant,
	 "Azimuthal_Equidistant"},
	{WAzimuth_Measure_Point_Longitude,
	 "Azimuth_Measure_Point_Longitude"},
	{WBearing_Reference_Direction,
	 "Bearing_Reference_Direction"},
	{WBearing_Reference_Meridian,
	 "Bearing_Reference_Meridian"},
	{WBearing_Resolution,
	 "Bearing_Resolution"},
	{WBearing_Units,
	 "Bearing_Units"},
	{WBeginning_Date,
	 "Beginning_Date"},
	{WBeginning_Date_of_Attribute_Values,
	 "Beginning_Date_of_Attribute_Values"},
	{WBeginning_Time,
	 "Beginning_Time"},
	{WBounding_Coordinates,
	 "Bounding_Coordinates"},
	{WBrowse_Graphic,
	 "Browse_Graphic"},
	{WBrowse_Graphic_File_Description,
	 "Browse_Graphic_File_Description"},
	{WBrowse_Graphic_File_Name,
	 "Browse_Graphic_File_Name"},
	{WBrowse_Graphic_File_Type,
	 "Browse_Graphic_File_Type"},
	{WCalendar_Date,
	 "Calendar_Date"},
	{WCitation,
	 "Citation"},
	{WCitation_Information,
	 "Citation_Information"},
	{WCity,
	 "City"},
	{WCloud_Cover,
	 "Cloud_Cover"},
	{WCodeset_Domain,
	 "Codeset_Domain"},
	{WCodeset_Name,
	 "Codeset_Name"},
	{WCodeset_Source,
	 "Codeset_Source"},
	{WColumn_Count,
	 "Column_Count"},
	{WCompatibility_Information,
	 "Compatibility_Information"},
	{WCompleteness_Report,
	 "Completeness_Report"},
	{WCompression_Support,
	 "Compression_Support"},
	{WComputer_Contact_Information,
	 "Computer_Contact_Information"},
	{WContact_Address,
	 "Contact_Address"},
	{WContact_Electronic_Mail_Address,
	 "Contact_Electronic_Mail_Address"},
	{WContact_Facsimile_Telephone,
	 "Contact_Facsimile_Telephone"},
	{WContact_Information,
	 "Contact_Information"},
	{WContact_Instructions,
	 "Contact_Instructions"},
	{WContact_Organization,
	 "Contact_Organization"},
	{WContact_Organization_Primary,
	 "Contact_Organization_Primary"},
	{WContact_Person,
	 "Contact_Person"},
	{WContact_Person_Primary,
	 "Contact_Person_Primary"},
	{WContact_Position,
	 "Contact_Position"},
	{WContact_TDD_TTY_Telephone,
	 "Contact_TDD/TTY_Telephone"},
	{WContact_Voice_Telephone,
	 "Contact_Voice_Telephone"},
	{WCoordinate_Representation,
	 "Coordinate_Representation"},
	{WCountry,
	 "Country"},
	{WCross_Reference,
	 "Cross_Reference"},
	{WCurrentness_Reference,
	 "Currentness_Reference"},
	{WCustom_Order_Process,
	 "Custom_Order_Process"},
	{WData_Quality_Information,
	 "Data_Quality_Information"},
	{WData_Set_Credit,
	 "Data_Set_Credit"},
	{WData_Set_G_Polygon,
	 "Data_Set_G-Polygon"},
	{WData_Set_G_Polygon_Exclusion_G_Ring,
	 "Data_Set_G-Polygon_Exclusion_G-Ring"},
	{WData_Set_G_Polygon_Outer_G_Ring,
	 "Data_Set_G-Polygon_Outer_G-Ring"},
	{WDenominator_of_Flattening_Ratio,
	 "Denominator_of_Flattening_Ratio"},
	{WDepth_Datum_Name,
	 "Depth_Datum_Name"},
	{WDepth_Distance_Units,
	 "Depth_Distance_Units"},
	{WDepth_Encoding_Method,
	 "Depth_Encoding_Method"},
	{WDepth_Resolution,
	 "Depth_Resolution"},
	{WDepth_System_Definition,
	 "Depth_System_Definition"},
	{WDescription,
	 "Description"},
	{WDetailed_Description,
	 "Detailed_Description"},
	{WDialup_File_Name,
	 "Dialup_File_Name"},
	{WDialup_Instructions,
	 "Dialup_Instructions"},
	{WDialup_Telephone,
	 "Dialup_Telephone"},
	{WDigital_Form,
	 "Digital_Form"},
	{WDigital_Transfer_Information,
	 "Digital_Transfer_Information"},
	{WDigital_Transfer_Option,
	 "Digital_Transfer_Option"},
	{WDirect_Spatial_Reference_Method,
	 "Direct_Spatial_Reference_Method"},
	{WDistance_and_Bearing_Representation,
	 "Distance_and_Bearing_Representation"},
	{WDistance_Resolution,
	 "Distance_Resolution"},
	{WDistribution_Information,
	 "Distribution_Information"},
	{WDistribution_Liability,
	 "Distribution_Liability"},
	{WDistributor,
	 "Distributor"},
	{WEast_Bounding_Coordinate,
	 "East_Bounding_Coordinate"},
	{WEdition,
	 "Edition"},
	{WEllipsoid_Name,
	 "Ellipsoid_Name"},
	{WEnding_Date,
	 "Ending_Date"},
	{WEnding_Date_of_Attribute_Values,
	 "Ending_Date_of_Attribute_Values"},
	{WEnding_Time,
	 "Ending_Time"},
	{WEntity_and_Attribute_Detail_Citation,
	 "Entity_and_Attribute_Detail_Citation"},
	{WEntity_and_Attribute_Information,
	 "Entity_and_Attribute_Information"},
	{WEntity_and_Attribute_Overview,
	 "Entity_and_Attribute_Overview"},
	{WEntity_Type,
	 "Entity_Type"},
	{WEntity_Type_Definition,
	 "Entity_Type_Definition"},
	{WEntity_Type_Definition_Source,
	 "Entity_Type_Definition_Source"},
	{WEntity_Type_Label,
	 "Entity_Type_Label"},
	{WEnumerated_Domain,
	 "Enumerated_Domain"},
	{WEnumerated_Domain_Value,
	 "Enumerated_Domain_Value"},
	{WEnumerated_Domain_Value_Definition,
	 "Enumerated_Domain_Value_Definition"},
	{WEnumerated_Domain_Value_Definition_Source,
	 "Enumerated_Domain_Value_Definition_Source"},
	{WEquidistant_Conic,
	 "Equidistant_Conic"},
	{WEquirectangular,
	 "Equirectangular"},
	{WFalse_Easting,
	 "False_Easting"},
	{WFalse_Northing,
	 "False_Northing"},
	{WFees,
	 "Fees"},
	{WFile_Decompression_Technique,
	 "File_Decompression_Technique"},
	{WFormat_Information_Content,
	 "Format_Information_Content"},
	{WFormat_Name,
	 "Format_Name"},
	{WFormat_Specification,
	 "Format_Specification"},
	{WFormat_Version_Date,
	 "Format_Version_Date"},
	{WFormat_Version_Number,
	 "Format_Version_Number"},
	{WG_Ring,
	 "G-Ring"},
	{WG_Ring_Latitude,
	 "G-Ring_Latitude"},
	{WG_Ring_Longitude,
	 "G-Ring_Longitude"},
	{WG_Ring_Point,
	 "G-Ring_Point"},
	{WGeneral_Vertical_Near_sided_Perspective,
	 "General_Vertical_Near-sided_Perspective"},
	{WGeodetic_Model,
	 "Geodetic_Model"},
	{WGeographic,
	 "Geographic"},
	{WGeographic_Coordinate_Units,
	 "Geographic_Coordinate_Units"},
	{WGeospatial_Data_Presentation_Form,
	 "Geospatial_Data_Presentation_Form"},
	{WGnomonic,
	 "Gnomonic"},
	{WGrid_Coordinate_System,
	 "Grid_Coordinate_System"},
	{WGrid_Coordinate_System_Name,
	 "Grid_Coordinate_System_Name"},
	{WHeight_of_Perspective_Point_Above_Surface,
	 "Height_of_Perspective_Point_Above_Surface"},
	{WHighest_BPS,
	 "Highest_BPS"},
	{WHorizontal_Coordinate_System_Definition,
	 "Horizontal_Coordinate_System_Definition"},
	{WHorizontal_Datum_Name,
	 "Horizontal_Datum_Name"},
	{WHorizontal_Positional_Accuracy,
	 "Horizontal_Positional_Accuracy"},
	{WHorizontal_Positional_Accuracy_Explanation,
	 "Horizontal_Positional_Accuracy_Explanation"},
	{WHorizontal_Positional_Accuracy_Report,
	 "Horizontal_Positional_Accuracy_Report"},
	{WHorizontal_Positional_Accuracy_Value,
	 "Horizontal_Positional_Accuracy_Value"},
	{WHours_of_Service,
	 "Hours_of_Service"},
	{WIdentification_Information,
	 "Identification_Information"},
	{WIndirect_Spatial_Reference,
	 "Indirect_Spatial_Reference"},
	{WIssue_Identification,
	 "Issue_Identification"},
	{WKeywords,
	 "Keywords"},
	{WLambert_Azimuthal_Equal_Area,
	 "Lambert_Azimuthal_Equal_Area"},
	{WLambert_Conformal_Conic,
	 "Lambert_Conformal_Conic"},
	{WLandsat_Number,
	 "Landsat_Number"},
	{WLarger_Work_Citation,
	 "Larger_Work_Citation"},
	{WLatitude_of_Projection_Center,
	 "Latitude_of_Projection_Center"},
	{WLatitude_of_Projection_Origin,
	 "Latitude_of_Projection_Origin"},
	{WLatitude_Resolution,
	 "Latitude_Resolution"},
	{WLineage,
	 "Lineage"},
	{WLocal,
	 "Local"},
	{WLocal_Description,
	 "Local_Description"},
	{WLocal_Georeference_Information,
	 "Local_Georeference_Information"},
	{WLocal_Planar,
	 "Local_Planar"},
	{WLocal_Planar_Description,
	 "Local_Planar_Description"},
	{WLocal_Planar_Georeference_Information,
	 "Local_Planar_Georeference_Information"},
	{WLogical_Consistency_Report,
	 "Logical_Consistency_Report"},
	{WLongitude_of_Central_Meridian,
	 "Longitude_of_Central_Meridian"},
	{WLongitude_of_Projection_Center,
	 "Longitude_of_Projection_Center"},
	{WLongitude_Resolution,
	 "Longitude_Resolution"},
	{WLowest_BPS,
	 "Lowest_BPS"},
	{WMaintenance_and_Update_Frequency,
	 "Maintenance_and_Update_Frequency"},
	{WMap_Projection,
	 "Map_Projection"},
	{WMap_Projection_Name,
	 "Map_Projection_Name"},
	{WMap_Projection_Parameters,
	 "Map_Projection_Parameters"},
	{WMercator,
	 "Mercator"},
	{WMetadata,
	 "Metadata"},
	{WMetadata_Access_Constraints,
	 "Metadata_Access_Constraints"},
	{WMetadata_Contact,
	 "Metadata_Contact"},
	{WMetadata_Date,
	 "Metadata_Date"},
	{WMetadata_Extensions,
	 "Metadata_Extensions"},
	{WMetadata_Future_Review_Date,
	 "Metadata_Future_Review_Date"},
	{WMetadata_Language,
	 "Metadata_Language"},
	{WMetadata_Reference_Information,
	 "Metadata_Reference_Information"},
	{WMetadata_Review_Date,
	 "Metadata_Review_Date"},
	{WMetadata_Security_Classification,
	 "Metadata_Security_Classification"},
	{WMetadata_Security_Classification_System,
	 "Metadata_Security_Classification_System"},
	{WMetadata_Security_Handling_Description,
	 "Metadata_Security_Handling_Description"},
	{WMetadata_Security_Information,
	 "Metadata_Security_Information"},
	{WMetadata_Standard_Name,
	 "Metadata_Standard_Name"},
	{WMetadata_Standard_Version,
	 "Metadata_Standard_Version"},
	{WMetadata_Time_Convention,
	 "Metadata_Time_Convention"},
	{WMetadata_Use_Constraints,
	 "Metadata_Use_Constraints"},
	{WMiller_Cylindrical,
	 "Miller_Cylindrical"},
	{WModified_Stereographic_for_Alaska,
	 "Modified_Stereographic_for_Alaska"},
	{WMultiple_Dates_Times,
	 "Multiple_Dates/Times"},
	{WNative_Data_Set_Environment,
	 "Native_Data_Set_Environment"},
	{WNetwork_Address,
	 "Network_Address"},
	{WNetwork_Resource_Name,
	 "Network_Resource_Name"},
	{WNon_digital_Form,
	 "Non-digital_Form"},
	{WNorth_Bounding_Coordinate,
	 "North_Bounding_Coordinate"},
	{WNumber_DataBits,
	 "Number_DataBits"},
	{WNumber_StopBits,
	 "Number_StopBits"},
	{WOblique_Line_Azimuth,
	 "Oblique_Line_Azimuth"},
	{WOblique_Line_Latitude,
	 "Oblique_Line_Latitude"},
	{WOblique_Line_Longitude,
	 "Oblique_Line_Longitude"},
	{WOblique_Line_Point,
	 "Oblique_Line_Point"},
	{WOblique_Mercator,
	 "Oblique_Mercator"},
	{WOffline_Media,
	 "Offline_Media"},
	{WOffline_Option,
	 "Offline_Option"},
	{WOnline_Computer_and_Operating_System,
	 "Online_Computer_and_Operating_System"},
	{WOnline_Linkage,
	 "Online_Linkage"},
	{WOnline_Option,
	 "Online_Option"},
	{WOrdering_Instructions,
	 "Ordering_Instructions"},
	{WOrdinate_Resolution,
	 "Ordinate_Resolution"},
	{WOriginator,
	 "Originator"},
	{WOrthographic,
	 "Orthographic"},
	{WOther_Citation_Details,
	 "Other_Citation_Details"},
	{WOther_Grid_Systems_Definition,
	 "Other_Grid_System's_Definition"},
	{WOther_Projections_Definition,
	 "Other_Projection's_Definition"},
	{WOverview_Description,
	 "Overview_Description"},
	{WParity,
	 "Parity"},
	{WPath_Number,
	 "Path_Number"},
	{WPlace,
	 "Place"},
	{WPlace_Keyword,
	 "Place_Keyword"},
	{WPlace_Keyword_Thesaurus,
	 "Place_Keyword_Thesaurus"},
	{WPlanar,
	 "Planar"},
	{WPlanar_Coordinate_Encoding_Method,
	 "Planar_Coordinate_Encoding_Method"},
	{WPlanar_Coordinate_Information,
	 "Planar_Coordinate_Information"},
	{WPlanar_Distance_Units,
	 "Planar_Distance_Units"},
	{WPoint_and_Vector_Object_Count,
	 "Point_and_Vector_Object_Count"},
	{WPoint_and_Vector_Object_Information,
	 "Point_and_Vector_Object_Information"},
	{WPoint_of_Contact,
	 "Point_of_Contact"},
	{WPolar_Stereographic,
	 "Polar_Stereographic"},
	{WPolyconic,
	 "Polyconic"},
	{WPositional_Accuracy,
	 "Positional_Accuracy"},
	{WPostal_Code,
	 "Postal_Code"},
	{WProcess_Contact,
	 "Process_Contact"},
	{WProcess_Date,
	 "Process_Date"},
	{WProcess_Description,
	 "Process_Description"},
	{WProcess_Step,
	 "Process_Step"},
	{WProcess_Time,
	 "Process_Time"},
	{WProfile_Name,
	 "Profile_Name"},
	{WProgress,
	 "Progress"},
	{WPublication_Date,
	 "Publication_Date"},
	{WPublication_Information,
	 "Publication_Information"},
	{WPublication_Place,
	 "Publication_Place"},
	{WPublication_Time,
	 "Publication_Time"},
	{WPublisher,
	 "Publisher"},
	{WPurpose,
	 "Purpose"},
	{WQuantitative_Attribute_Accuracy_Assessment,
	 "Quantitative_Attribute_Accuracy_Assessment"},
	{WQuantitative_Horizontal_Positional_Accuracy_Assessment,
	 "Quantitative_Horizontal_Positional_Accuracy_Assessment"},
	{WQuantitative_Vertical_Positional_Accuracy_Assessment,
	 "Quantitative_Vertical_Positional_Accuracy_Assessment"},
	{WRange_Domain,
	 "Range_Domain"},
	{WRange_Domain_Maximum,
	 "Range_Domain_Maximum"},
	{WRange_Domain_Minimum,
	 "Range_Domain_Minimum"},
	{WRange_of_Dates_Times,
	 "Range_of_Dates/Times"},
	{WRaster_Object_Information,
	 "Raster_Object_Information"},
	{WRaster_Object_Type,
	 "Raster_Object_Type"},
	{WRecording_Capacity,
	 "Recording_Capacity"},
	{WRecording_Density,
	 "Recording_Density"},
	{WRecording_Density_Units,
	 "Recording_Density_Units"},
	{WRecording_Format,
	 "Recording_Format"},
	{WResource_Description,
	 "Resource_Description"},
	{WRobinson,
	 "Robinson"},
	{WRow_Count,
	 "Row_Count"},
	{WScale_Factor_at_Center_Line,
	 "Scale_Factor_at_Center_Line"},
	{WScale_Factor_at_Central_Meridian,
	 "Scale_Factor_at_Central_Meridian"},
	{WScale_Factor_at_Equator,
	 "Scale_Factor_at_Equator"},
	{WScale_Factor_at_Projection_Origin,
	 "Scale_Factor_at_Projection_Origin"},
	{WSDTS_Point_and_Vector_Object_Type,
	 "SDTS_Point_and_Vector_Object_Type"},
	{WSDTS_Terms_Description,
	 "SDTS_Terms_Description"},
	{WSecurity_Classification,
	 "Security_Classification"},
	{WSecurity_Classification_System,
	 "Security_Classification_System"},
	{WSecurity_Handling_Description,
	 "Security_Handling_Description"},
	{WSecurity_Information,
	 "Security_Information"},
	{WSemi_major_Axis,
	 "Semi-major_Axis"},
	{WSeries_Information,
	 "Series_Information"},
	{WSeries_Name,
	 "Series_Name"},
	{WSingle_Date_Time,
	 "Single_Date/Time"},
	{WSinusoidal,
	 "Sinusoidal"},
	{WSource_Citation,
	 "Source_Citation"},
	{WSource_Citation_Abbreviation,
	 "Source_Citation_Abbreviation"},
	{WSource_Contribution,
	 "Source_Contribution"},
	{WSource_Currentness_Reference,
	 "Source_Currentness_Reference"},
	{WSource_Information,
	 "Source_Information"},
	{WSource_Produced_Citation_Abbreviation,
	 "Source_Produced_Citation_Abbreviation"},
	{WSource_Scale_Denominator,
	 "Source_Scale_Denominator"},
	{WSource_Time_Period_of_Content,
	 "Source_Time_Period_of_Content"},
	{WSource_Used_Citation_Abbreviation,
	 "Source_Used_Citation_Abbreviation"},
	{WSouth_Bounding_Coordinate,
	 "South_Bounding_Coordinate"},
	{WSpace_Oblique_Mercator_Landsat,
	 "Space_Oblique_Mercator_(Landsat)"},
	{WSpatial_Data_Organization_Information,
	 "Spatial_Data_Organization_Information"},
	{WSpatial_Domain,
	 "Spatial_Domain"},
	{WSpatial_Reference_Information,
	 "Spatial_Reference_Information"},
	{WSPCS_Zone_Identifier,
	 "SPCS_Zone_Identifier"},
	{WStandard_Order_Process,
	 "Standard_Order_Process"},
	{WStandard_Parallel,
	 "Standard_Parallel"},
	{WState_or_Province,
	 "State_or_Province"},
	{WState_Plane_Coordinate_System,
	 "State_Plane_Coordinate_System"},
	{WStatus,
	 "Status"},
	{WStereographic,
	 "Stereographic"},
	{WStraight_Vertical_Longitude_from_Pole,
	 "Straight-Vertical_Longitude_from_Pole"},
	{WStratum,
	 "Stratum"},
	{WStratum_Keyword,
	 "Stratum_Keyword"},
	{WStratum_Keyword_Thesaurus,
	 "Stratum_Keyword_Thesaurus"},
	{WSupplemental_Information,
	 "Supplemental_Information"},
	{WTechnical_Prerequisites,
	 "Technical_Prerequisites"},
	{WTemporal,
	 "Temporal"},
	{WTemporal_Keyword,
	 "Temporal_Keyword"},
	{WTemporal_Keyword_Thesaurus,
	 "Temporal_Keyword_Thesaurus"},
	{WTheme,
	 "Theme"},
	{WTheme_Keyword,
	 "Theme_Keyword"},
	{WTheme_Keyword_Thesaurus,
	 "Theme_Keyword_Thesaurus"},
	{WTime_of_Day,
	 "Time_of_Day"},
	{WTime_Period_Information,
	 "Time_Period_Information"},
	{WTime_Period_of_Content,
	 "Time_Period_of_Content"},
	{WTitle,
	 "Title"},
	{WTransfer_Size,
	 "Transfer_Size"},
	{WTransverse_Mercator,
	 "Transverse_Mercator"},
	{WTurnaround,
	 "Turnaround"},
	{WType_of_Source_Media,
	 "Type_of_Source_Media"},
	{WUniversal_Polar_Stereographic,
	 "Universal_Polar_Stereographic"},
	{WUniversal_Transverse_Mercator,
	 "Universal_Transverse_Mercator"},
	{WUnrepresentable_Domain,
	 "Unrepresentable_Domain"},
	{WUPS_Zone_Identifier,
	 "UPS_Zone_Identifier"},
	{WUse_Constraints,
	 "Use_Constraints"},
	{WUTM_Zone_Number,
	 "UTM_Zone_Number"},
	{Wvan_der_Grinten,
	 "van_der_Grinten"},
	{WVertical_Coordinate_System_Definition,
	 "Vertical_Coordinate_System_Definition"},
	{WVertical_Count,
	 "Vertical_Count"},
	{WVertical_Positional_Accuracy,
	 "Vertical_Positional_Accuracy"},
	{WVertical_Positional_Accuracy_Explanation,
	 "Vertical_Positional_Accuracy_Explanation"},
	{WVertical_Positional_Accuracy_Report,
	 "Vertical_Positional_Accuracy_Report"},
	{WVertical_Positional_Accuracy_Value,
	 "Vertical_Positional_Accuracy_Value"},
	{WVPF_Point_and_Vector_Object_Information,
	 "VPF_Point_and_Vector_Object_Information"},
	{WVPF_Point_and_Vector_Object_Type,
	 "VPF_Point_and_Vector_Object_Type"},
	{WVPF_Terms_Description,
	 "VPF_Terms_Description"},
	{WVPF_Topology_Level,
	 "VPF_Topology_Level"},
	{WWest_Bounding_Coordinate,
	 "West_Bounding_Coordinate"},

/* Biological Data Profile */

	{WDescription_of_Geographic_Extent,
		"Description_of_Geographic_Extent"},
	{WBounding_Altitudes,
		"Bounding_Altitudes"},
	{WAltitude_Minimum,
		"Altitude_Minimum"},
	{WAltitude_Maximum,
		"Altitude_Maximum"},
	{WTaxonomy,
		"Taxonomy"},
	{WKeywords_Taxon,
		"Keywords/Taxon"},
	{WTaxonomic_Keyword_Thesaurus,
		"Taxonomic_Keyword_Thesaurus"},
	{WTaxonomic_Keywords,
		"Taxonomic_Keywords"},
	{WTaxonomic_System,
		"Taxonomic_System"},
	{WClassification_System_Authority,
		"Classification_System/Authority"},
	{WClassification_System_Citation,
		"Classification_System_Citation"},
	{WClassification_System_Modifications,
		"Classification_System_Modifications"},
	{WIdentification_Reference,
		"Identification_Reference"},
	{WIdentifier,
		"Identifier"},
	{WTaxonomic_Procedures,
		"Taxonomic_Procedures"},
	{WTaxonomic_Completeness,
		"Taxonomic_Completeness"},
	{WVouchers,
		"Vouchers"},
	{WSpecimen,
		"Specimen"},
	{WRepository,
		"Repository"},
	{WGeneral_Taxonomic_Coverage,
		"General_Taxonomic_Coverage"},
	{WTaxonomic_Classification,
		"Taxonomic_Classification"},
	{WTaxon_Rank_Name,
		"Taxon_Rank_Name"},
	{WTaxon_Rank_Value,
		"Taxon_Rank_Value"},
	{WApplicable_Common_Name,
		"Applicable_Common_Name"},
	{WAnalytical_Tool,
		"Analytical_Tool"},
	{WAnalytical_Tool_Description,
		"Analytical_Tool_Description"},
	{WTool_Access_Information,
		"Tool_Access_Information"},
	{WTool_Contact,
		"Tool_Contact"},
	{WTool_Citation,
		"Tool_Citation"},
	{WTool_Access_Instructions,
		"Tool_Access_Instructions"},
	{WTool_Computer_and_Operating_System,
		"Tool_Computer_and_Operating_System"},
	{WMethodology,
		"Methodology"},
	{WMethodology_Type,
		"Methodology_Type"},
	{WMethodology_Identifier,
		"Methodology_Identifier"},
	{WMethodology_Keyword_Thesaurus,
		"Methodology_Keyword_Thesaurus"},
	{WMethodology_Keyword,
		"Methodology_Keyword"},
	{WMethodology_Description,
		"Methodology_Description"},
	{WMethodology_Citation,
		"Methodology_Citation"},
	{WASCII_File_Structure,
		"ASCII_File_Structure"},
	{WRecord_Delimiter,
		"Record_Delimiter"},
	{WNumber_Header_Lines,
		"Number_Header_Lines"},
	{WDescription_of_Header_Content,
		"Description_of_Header_Content"},
	{WOrientation,
		"Orientation"},
	{WCase_Sensitive,
		"Case_Sensitive"},
	{WAuthentication,
		"Authentication"},
	{WQuote_Character,
		"Quote_Character"},
	{WData_Field,
		"Data_Field"},
	{WData_Field_Name,
		"Data_Field_Name"},
	{WMissing_Value_Code,
		"Missing_Value_Code"},
	{WData_Field_Width_Delimiter,
		"Data_Field_Width_Delimiter"},
	{WData_Field_Width,
		"Data_Field_Width"},
	{WBeginning_Geologic_Age,
		"Beginning_Geologic_Age"},
	{WEnding_Geologic_Age,
		"Ending_Geologic_Age"},
	{WGeologic_Age,
		"Geologic_Age"},
	{WGeologic_Time_Scale,
		"Geologic_Time_Scale"},
	{WGeologic_Age_Estimate,
		"Geologic_Age_Estimate"},
	{WGeologic_Age_Uncertainty,
		"Geologic_Age_Uncertainty"},
	{WGeologic_Age_Explanation,
		"Geologic_Age_Explanation"},
	{WGeologic_Citation,
		"Geologic_Citation"},

/* Shoreline profile */

	{WBarometric_Pressure,
	 "Barometric_Pressure"},
	{WEnvironmental_Event,
	 "Environmental_Event"},
	{WMarine_Weather_Condition,
	 "Marine_Weather_Condition"},
	{WProcess_Step_Citation,
	 "Process_Step_Citation"},
	{WRange_of_Tide,
	 "Range_of_Tide"},
	{WSupplemental_Tidal_Information,
	 "Supplemental_Tidal_Information"},
	{WTidal_Datum,
	 "Tidal_Datum"},
	{WTidal_Information,
	 "Tidal_Information"},
	{WTide_Table_Reference,
	 "Tide_Table_Reference"},
	{WTime_of_High_Tide,
	 "Time_of_High_Tide"},
	{WTime_of_Low_Tide,
	 "Time_of_Low_Tide"},
	{WTime_of_Tide,
	 "Time_of_Tide"},
	{WType_of_Tide,
	 "Type_of_Tide"},
	{WWave_Height,
	 "Wave_Height"},
	{WWind_Direction,
	 "Wind_Direction"},
	{WWind_Speed,
	 "Wind_Speed"},

/* Remote-sensing profile */

	{WAcquisition_Information,
	 "Acquisition_Information"},
	{WAerial_Film_Speed,
	 "Aerial_Film_Speed"},
	{WAerotriangulation_Reference,
	 "Aerotriangulation_Reference"},
	{WAffine_Distortion_X_Prime_Coefficient,
	 "Affine_Distortion_X_Prime_Coefficient"},
	{WAffine_Distortion_Y_Prime_Coefficient,
	 "Affine_Distortion_Y_Prime_Coefficient"},
	{WAggregation_Criteria,
	 "Aggregation_Criteria"},
	{WAggregation_Information,
	 "Aggregation_Information"},
	{WAggregation_Member_ID,
	 "Aggregation_Member_ID"},
	{WAlgorithm_Change_Description,
	 "Algorithm_Change_Description"},
	{WAlgorithm_Change_History,
	 "Algorithm_Change_History"},
	{WAlgorithm_Change_Reference,
	 "Algorithm_Change_Reference"},
	{WAlgorithm_Description,
	 "Algorithm_Description"},
	{WAlgorithm_Identifiers,
	 "Algorithm_Identifiers"},
	{WAlgorithm_Information,
	 "Algorithm_Information"},
	{WAlgorithm_Peer_Review_Description,
	 "Algorithm_Peer_Review_Description"},
	{WAlgorithm_Peer_Review_Information,
	 "Algorithm_Peer_Review_Information"},
	{WAlgorithm_Peer_Review_Reference,
	 "Algorithm_Peer_Review_Reference"},
	{WAlgorithm_Reference,
	 "Algorithm_Reference"},
	{WAlgorithm_Text_Description,
	 "Algorithm_Text_Description"},
	{WAltitude_Distance_Layer,
	 "Altitude_Distance_Layer"},
	{WAncillary_Dataset,
	 "Ancillary_Dataset"},
	{WAncillary_Dataset_Description,
	 "Ancillary_Dataset_Description"},
	{WAncillary_Dataset_Identifier,
	 "Ancillary_Dataset_Identifier"},
	{WAncillary_Description,
	 "Ancillary_Description"},
	{WAncillary_Reference,
	 "Ancillary_Reference"},
	{WAngle_Dependent_Distortion,
	 "Angle_Dependent_Distortion"},
	{WAngle_Distortion_Value,
	 "Angle_Distortion_Value"},
	{WArea_Weighted_Average_Resolution,
	 "Area_Weighted_Average_Resolution"},
	{WArgument_of_Perigee,
	 "Argument_of_Perigee"},
	{WAscending_Descending_Indicator,
	 "Ascending/Descending_Indicator"},
	{WAttitude_Angular_Units,
	 "Attitude_Angular_Units"},
	{WAvailability_of_Element_Locations,
	 "Availability_of_Element_Locations"},
	{WAxes,
	 "Axes"},
	{WAxis_Rotation_Convention,
	 "Axis_Rotation_Convention"},
	{WBand_Boundary_Definition,
	 "Band_Boundary_Definition"},
	{WBand_Description,
	 "Band_Description"},
	{WBand_ID,
	 "Band_ID"},
	{WBand_Identification,
	 "Band_Identification"},
	{WBand_Measurement_Mode_ID,
	 "Band_Measurement_Mode_ID"},
	{WBand_Quality,
	 "Band_Quality"},
	{WCalibrated_Detector_Positions,
	 "Calibrated_Detector_Positions"},
	{WCalibrated_Focal_Length,
	 "Calibrated_Focal_Length"},
	{WCamera_Identifier,
	 "Camera_Identifier"},
	{WCamera_Type,
	 "Camera_Type"},
	{WCell_Value_Type,
	 "Cell_Value_Type"},
	{WClock_Time_Drift,
	 "Clock_Time/Drift"},
	{WCollection_Type,
	 "Collection_Type"},
	{WColumn_Delta_X,
	 "Column_Delta_X"},
	{WColumn_Delta_Y,
	 "Column_Delta_Y"},
	{WCommand_Line_Processing_Parameter,
	 "Command_Line_Processing_Parameter"},
	{WComponent_Information,
	 "Component_Information"},
	{WContainer_Packet_ID,
	 "Container_Packet_ID"},
	{WControl_Point_Authority,
	 "Control_Point_Authority"},
	{WControl_Point_Column,
	 "Control_Point_Column"},
	{WControl_Point_Earth_Location,
	 "Control_Point_Earth_Location"},
	{WControl_Point_Horizontal_X_Accuracy,
	 "Control_Point_Horizontal_X_Accuracy"},
	{WControl_Point_Horizontal_Y_Accuracy,
	 "Control_Point_Horizontal_Y_Accuracy"},
	{WControl_Point_ID,
	 "Control_Point_ID"},
	{WControl_Point_Identification,
	 "Control_Point_Identification"},
	{WControl_Point_Origin,
	 "Control_Point_Origin"},
	{WControl_Point_Raster_Position,
	 "Control_Point_Raster_Position"},
	{WControl_Point_Row,
	 "Control_Point_Row"},
	{WControl_Point_Type,
	 "Control_Point_Type"},
	{WControl_Point_Use_Flag,
	 "Control_Point_Use_Flag"},
	{WControl_Point_Vertical_Accuracy,
	 "Control_Point_Vertical_Accuracy"},
	{WControl_Point_x_Value,
	 "Control_Point_x_Value"},
	{WControl_Point_y_Value,
	 "Control_Point_y_Value"},
	{WControl_Point_z_Value,
	 "Control_Point_z_Value"},
	{WCoordinate_Point,
	 "Coordinate_Point"},
	{WCoordinate_System,
	 "Coordinate_System"},
	{WCoordinate_x_Value,
	 "Coordinate_x_Value"},
	{WCoordinate_XY_Units,
	 "Coordinate_XY_Units"},
	{WCoordinate_y_Value,
	 "Coordinate_y_Value"},
	{WCoordinate_Z_Units,
	 "Coordinate_Z_Units"},
	{WCoordinate_z_Value,
	 "Coordinate_z_Value"},
	{WCross_Track_Axis,
	 "Cross_Track_Axis"},
	{WCross_Track_Description,
	 "Cross_Track_Description"},
	{WCross_Track_Direction,
	 "Cross_Track_Direction"},
	{WCross_Track_Extent_Angle,
	 "Cross_Track_Extent_Angle"},
	{WCross_Track_Fixed_Angle,
	 "Cross_Track_Fixed_Angle"},
	{WCross_Track_Motion,
	 "Cross_Track_Motion"},
	{WCross_Track_Start_Angle,
	 "Cross_Track_Start_Angle"},
	{WCross_Track_Step_Angle,
	 "Cross_Track_Step_Angle"},
	{WCross_Track_Sweep,
	 "Cross_Track_Sweep"},
	{WCross_Track_Zero,
	 "Cross_Track_Zero"},
	{WData_Dictionary_Reference,
	 "Data_Dictionary_Reference"},
	{WData_Scaling_Information,
	 "Data_Scaling_Information"},
	{WDataset_Identifier,
	 "Dataset_Identifier"},
	{WDate_of_Last_Calibration,
	 "Date_of_Last_Calibration"},
	{WDeveloping_Institution,
	 "Developing_Institution"},
	{WDimension_Count,
	 "Dimension_Count"},
	{WDimension_Description,
	 "Dimension_Description"},
	{WDimension_Properties,
	 "Dimension_Properties"},
	{WDistance_Dependent_Distortion,
	 "Distance_Dependent_Distortion"},
	{WDistance_Distortion_Value,
	 "Distance_Distortion_Value"},
	{WDistortion,
	 "Distortion"},
	{WDistortion_Type_Affine,
	 "Distortion_Type_Affine"},
	{WDistortion_Type_Radial_Asymmetrical,
	 "Distortion_Type_Radial_Asymmetrical"},
	{WDistortion_Type_Radial_Symmetrical,
	 "Distortion_Type_Radial_Symmetrical"},
	{WDocumentation,
	 "Documentation"},
	{WEccentricity,
	 "Eccentricity"},
	{WEffective_Aerial_Film_Speed,
	 "Effective_Aerial_Film_Speed"},
	{WElevation_Axis,
	 "Elevation_Axis"},
	{WElevation_Description,
	 "Elevation_Description"},
	{WElevation_Direction,
	 "Elevation_Direction"},
	{WElevation_Extent_Angle,
	 "Elevation_Extent_Angle"},
	{WElevation_Fixed_Angle,
	 "Elevation_Fixed_Angle"},
	{WElevation_Start_Angle,
	 "Elevation_Start_Angle"},
	{WElevation_Step_Angle,
	 "Elevation_Step_Angle"},
	{WElevation_Sweep,
	 "Elevation_Sweep"},
	{WElevation_Zero,
	 "Elevation_Zero"},
	{WEphemeris,
	 "Ephemeris"},
	{WExposure_Time,
	 "Exposure_Time"},
	{WExterior_Orientation_Accuracy,
	 "Exterior_Orientation_Accuracy"},
	{WFiducial,
	 "Fiducial"},
	{WFiducial_Center,
	 "Fiducial_Center"},
	{WFilm_Type,
	 "Film_Type"},
	{WFilter_on_Camera,
	 "Filter_on_Camera"},
	{WFilter_on_Camera_Indicator,
	 "Filter_on_Camera_Indicator"},
	{WFilter_Type,
	 "Filter_Type"},
	{WFlight_Protocol,
	 "Flight_Protocol"},
	{WFlying_Height,
	 "Flying_Height"},
	{WForward_Motion_Compensation,
	 "Forward_Motion_Compensation"},
	{WFrame_Area,
	 "Frame_Area"},
	{WFrame_Area_Units,
	 "Frame_Area_Units"},
	{WFrame_Area_Value,
	 "Frame_Area_Value"},
	{WFrame_Camera,
	 "Frame_Camera"},
	{WFrame_Geometric_Properties,
	 "Frame_Geometric_Properties"},
	{WFrame_Hardware,
	 "Frame_Hardware"},
	{WFrame_Operation,
	 "Frame_Operation"},
	{WFrame_Optics,
	 "Frame_Optics"},
	{WFrame_Radiometric_Calibration,
	 "Frame_Radiometric_Calibration"},
	{WFrame_Radiometric_Properties,
	 "Frame_Radiometric_Properties"},
	{WFrame_Spectral_Information,
	 "Frame_Spectral_Information"},
	{WFrame_Spectral_Properties,
	 "Frame_Spectral_Properties"},
	{WGeorectified_Raster,
	 "Georectified_Raster"},
	{WGeoreferenceable_Raster,
	 "Georeferenceable_Raster"},
	{WGeoreferencing_Description,
	 "Georeferencing_Description"},
	{WGeoreferencing_Information,
	 "Georeferencing_Information"},
	{WGPS_Information_System_Availability,
	 "GPS_Information_System_Availability"},
	{WGrid_First_Element,
	 "Grid_First_Element"},
	{WGrid_First_Element_Map_X_Coordinate,
	 "Grid_First_Element_Map_X_Coordinate"},
	{WGrid_First_Element_Map_Y_Coordinate,
	 "Grid_First_Element_Map_Y_Coordinate"},
	{WGrid_Orientation,
	 "Grid_Orientation"},
	{WGround_Control_Point_Description,
	 "Ground_Control_Point_Description"},
	{WGround_Control_Point_Information,
	 "Ground_Control_Point_Information"},
	{WGround_Control_Point_Organization,
	 "Ground_Control_Point_Organization"},
	{WGround_Control_Point_Position,
	 "Ground_Control_Point_Position"},
	{WGround_Shape,
	 "Ground_Shape"},
	{WID_Overlapping_Dataset,
	 "ID_Overlapping_Dataset"},
	{WIFOV_Units,
	 "IFOV_Units"},
	{WIFOV_x_Definition,
	 "IFOV_x_Definition"},
	{WIFOV_x_Value,
	 "IFOV_x_Value"},
	{WIFOV_y_Definition,
	 "IFOV_y_Definition"},
	{WIFOV_y_Value,
	 "IFOV_y_Value"},
	{WImage_Quality,
	 "Image_Quality"},
	{WImage_Size,
	 "Image_Size"},
	{WImage_Size_x_Value,
	 "Image_Size_x_Value"},
	{WImage_Size_y_Value,
	 "Image_Size_y_Value"},
	{WInclination,
	 "Inclination"},
	{WIndividual_Band_Identification,
	 "Individual_Band_Identification"},
	{WInput_Dataset_Identifier,
	 "Input_Dataset_Identifier"},
	{WInput_Description,
	 "Input_Description"},
	{WInput_Level,
	 "Input_Level"},
	{WInput_Reference,
	 "Input_Reference"},
	{WINS_Reading_Availability,
	 "INS_Reading_Availability"},
	{WInstantaneous_Field_of_View,
	 "Instantaneous_Field_of_View"},
	{WInstitution_of_Last_Calibration,
	 "Institution_of_Last_Calibration"},
	{WInstrument_Description,
	 "Instrument_Description"},
	{WInstrument_Full_Name,
	 "Instrument_Full_Name"},
	{WInstrument_Information,
	 "Instrument_Information"},
	{WInstrument_Properties_Description,
	 "Instrument_Properties_Description"},
	{WInstrument_Reference,
	 "Instrument_Reference"},
	{WInstrument_Short_Name,
	 "Instrument_Short_Name"},
	{WInstrument_Specific_Georeferencing,
	 "Instrument_Specific_Georeferencing"},
	{WInstrument_Type,
	 "Instrument_Type"},
	{WKeplerian_Orbit,
	 "Keplerian_Orbit"},
	{WLast_Calibration,
	 "Last_Calibration"},
	{WLayer_Name,
	 "Layer_Name"},
	{WLens,
	 "Lens"},
	{WLens_Identifier,
	 "Lens_Identifier"},
	{WLens_Type,
	 "Lens_Type"},
	{WLight_Drop,
	 "Light_Drop"},
	{WLocation_Information,
	 "Location_Information"},
	{WMagazine_Identifier,
	 "Magazine_Identifier"},
	{WMaximum_Wavelength,
	 "Maximum_Wavelength"},
	{WMethod_of_Last_Calibration,
	 "Method_of_Last_Calibration"},
	{WMinimum_Wavelength,
	 "Minimum_Wavelength"},
	{WMission_Completion,
	 "Mission_Completion"},
	{WMission_Description,
	 "Mission_Description"},
	{WMission_History,
	 "Mission_History"},
	{WMission_Information,
	 "Mission_Information"},
	{WMission_Name,
	 "Mission_Name"},
	{WMission_Significant_Event,
	 "Mission_Significant_Event"},
	{WMission_Start_Date,
	 "Mission_Start_Date"},
	{WMultiple_Image_Alignment,
	 "Multiple_Image_Alignment"},
	{WNadir,
	 "Nadir"},
	{WNadir_Latitude,
	 "Nadir_Latitude"},
	{WNadir_Longitude,
	 "Nadir_Longitude"},
	{WName_of_Dimension,
	 "Name_of_Dimension"},
	{WNominal_Geostationary_Position,
	 "Nominal_Geostationary_Position"},
	{WNominal_Spatial_Resolution,
	 "Nominal_Spatial_Resolution"},
	{WNon_Polynomial_Scaling,
	 "Non_Polynomial_Scaling"},
	{WNumber_of_Angle_Distortion_Values,
	 "Number_of_Angle_Distortion_Values"},
	{WNumber_of_Bands,
	 "Number_of_Bands"},
	{WNumber_of_Cross_Track_Samples,
	 "Number_of_Cross_Track_Samples"},
	{WNumber_of_Data_Dimensions,
	 "Number_of_Data_Dimensions"},
	{WNumber_of_Denominator_Terms,
	 "Number_of_Denominator_Terms"},
	{WNumber_of_Distance_Distortion_Values,
	 "Number_of_Distance_Distortion_Values"},
	{WNumber_of_Elevation_Samples,
	 "Number_of_Elevation_Samples"},
	{WNumber_of_Numerator_Terms,
	 "Number_of_Numerator_Terms"},
	{WNumber_of_Points,
	 "Number_of_Points"},
	{WNumber_of_Profile_Samples,
	 "Number_of_Profile_Samples"},
	{WNumber_of_Resolution_Values,
	 "Number_of_Resolution_Values"},
	{WNumber_of_Thematic_Layers,
	 "Number_of_Thematic_Layers"},
	{WNumber_of_Wavelength_Bands,
	 "Number_of_Wavelength_Bands"},
	{WOperational_Mode,
	 "Operational_Mode"},
	{WOrbit_Angle_Units,
	 "Orbit_Angle_Units"},
	{WOrbit_Period,
	 "Orbit_Period"},
	{WOrbit_Period_Units,
	 "Orbit_Period_Units"},
	{WOrbit_Period_Value,
	 "Orbit_Period_Value"},
	{WOther_Collector_Description,
	 "Other_Collector_Description"},
	{WOther_Georeferencing_Description,
	 "Other_Georeferencing_Description"},
	{WPath,
	 "Path"},
	{WPeak_Wavelength,
	 "Peak_Wavelength"},
	{WPercentage_of_Overlap,
	 "Percentage_of_Overlap"},
	{WPerigee_Passage_Time,
	 "Perigee_Passage_Time"},
	{WPhotographic_Resolving_Power,
	 "Photographic_Resolving_Power"},
	{WPitch,
	 "Pitch"},
	{WPitch_Accuracy,
	 "Pitch_Accuracy"},
	{WPixel_Cross_Track_Size,
	 "Pixel_Cross_Track_Size"},
	{WPixel_Description,
	 "Pixel_Description"},
	{WPixel_Elevation_Size,
	 "Pixel_Elevation_Size"},
	{WPixel_Height_Above_Ellipsoid,
	 "Pixel_Height_Above_Ellipsoid"},
	{WPixel_Point_Spread_Function,
	 "Pixel_Point_Spread_Function"},
	{WPixel_Profile_Size,
	 "Pixel_Profile_Size"},
	{WPixel_Resolution,
	 "Pixel_Resolution"},
	{WPlatform_and_Instrument_Identification,
	 "Platform_and_Instrument_Identification"},
	{WPlatform_and_Mission_Information,
	 "Platform_and_Mission_Information"},
	{WPlatform_Description,
	 "Platform_Description"},
	{WPlatform_Full_Name,
	 "Platform_Full_Name"},
	{WPlatform_Information,
	 "Platform_Information"},
	{WPlatform_Nominal_Altitude,
	 "Platform_Nominal_Altitude"},
	{WPlatform_Nominal_Altitude_Units,
	 "Platform_Nominal_Altitude_Units"},
	{WPlatform_Nominal_Altitude_Value,
	 "Platform_Nominal_Altitude_Value"},
	{WPlatform_Nominal_Longitude,
	 "Platform_Nominal_Longitude"},
	{WPlatform_Orbit,
	 "Platform_Orbit"},
	{WPlatform_Serial_Identifier,
	 "Platform_Serial_Identifier"},
	{WPlatform_Short_Name,
	 "Platform_Short_Name"},
	{WPlatform_Sponsor,
	 "Platform_Sponsor"},
	{WPlatform_Start_Date,
	 "Platform_Start_Date"},
	{WPoint_Position_In_Pixel,
	 "Point_Position_In_Pixel"},
	{WPolarization_Characteristics,
	 "Polarization_Characteristics"},
	{WPolynomial_Coefficient,
	 "Polynomial_Coefficient"},
	{WPolynomial_Denominator,
	 "Polynomial_Denominator"},
	{WPolynomial_Function,
	 "Polynomial_Function"},
	{WPolynomial_Numerator,
	 "Polynomial_Numerator"},
	{WPositional_Information,
	 "Positional_Information"},
	{WPrincipal_Point_of_Autocollimation,
	 "Principal_Point_of_Autocollimation"},
	{WPrincipal_Point_of_Symmetry,
	 "Principal_Point_of_Symmetry"},
	{WProcessing_Change_History,
	 "Processing_Change_History"},
	{WProcessing_Documentation,
	 "Processing_Documentation"},
	{WProcessing_Environment,
	 "Processing_Environment"},
	{WProcessing_Identifiers,
	 "Processing_Identifiers"},
	{WProcessing_Information,
	 "Processing_Information"},
	{WProcessing_Input_Dataset,
	 "Processing_Input_Dataset"},
	{WProcessing_Level,
	 "Processing_Level"},
	{WProcessing_Level_Authority,
	 "Processing_Level_Authority"},
	{WProcessing_Level_Identifier,
	 "Processing_Level_Identifier"},
	{WProcessing_Procedure,
	 "Processing_Procedure"},
	{WProcessing_Procedure_Description,
	 "Processing_Procedure_Description"},
	{WProcessing_Run_History,
	 "Processing_Run_History"},
	{WProcessing_Software,
	 "Processing_Software"},
	{WProcessing_Software_Description,
	 "Processing_Software_Description"},
	{WProcessing_Software_Reference,
	 "Processing_Software_Reference"},
	{WProfile_Description,
	 "Profile_Description"},
	{WProfile_Extent,
	 "Profile_Extent"},
	{WProfile_Fixed,
	 "Profile_Fixed"},
	{WProfile_Properties,
	 "Profile_Properties"},
	{WProfile_Sounding,
	 "Profile_Sounding"},
	{WProfile_Start,
	 "Profile_Start"},
	{WProfile_Step,
	 "Profile_Step"},
	{WProfiling_Direction,
	 "Profiling_Direction"},
	{WProjection_Center_X_Position,
	 "Projection_Center_X_Position"},
	{WProjection_Center_Y_Position,
	 "Projection_Center_Y_Position"},
	{WProjection_Center_Z_Position,
	 "Projection_Center_Z_Position"},
	{WQuality_of_Autocollimation_Principal_Point_x_Value,
	 "Quality_of_Autocollimation_Principal_Point_x_Value"},
	{WQuality_of_Autocollimation_Principal_Point_y_Value,
	 "Quality_of_Autocollimation_Principal_Point_y_Value"},
	{WQuality_of_Focal_Length,
	 "Quality_of_Focal_Length"},
	{WQuality_of_Principal_Point_of_Autocollimation,
	 "Quality_of_Principal_Point_of_Autocollimation"},
	{WQuality_of_Principal_Point_of_Symmetry,
	 "Quality_of_Principal_Point_of_Symmetry"},
	{WQuality_of_Symmetry_Principal_Point_x_Value,
	 "Quality_of_Symmetry_Principal_Point_x_Value"},
	{WQuality_of_Symmetry_Principal_Point_y_Value,
	 "Quality_of_Symmetry_Principal_Point_y_Value"},
	{WRadial_Asymmetrical_Coefficient_B1,
	 "Radial_Asymmetrical_Coefficient_B1"},
	{WRadial_Asymmetrical_Coefficient_B2,
	 "Radial_Asymmetrical_Coefficient_B2"},
	{WRadial_Symmetrical_Angle_Interval,
	 "Radial_Symmetrical_Angle_Interval"},
	{WRadial_Symmetrical_Distance_Interval,
	 "Radial_Symmetrical_Distance_Interval"},
	{WRadial_Symmetrical_Distortion_Polynomial,
	 "Radial_Symmetrical_Distortion_Polynomial"},
	{WReceiver_Polarization,
	 "Receiver_Polarization"},
	{WReferenced_Coordinate_System,
	 "Referenced_Coordinate_System"},
	{WReferencing_Polynomial,
	 "Referencing_Polynomial"},
	{WRelative_Aperture,
	 "Relative_Aperture"},
	{WRelative_Azimuth,
	 "Relative_Azimuth"},
	{WReseau,
	 "Reseau"},
	{WResolution_Value_Set,
	 "Resolution_Value_Set"},
	{WResolving_Angle,
	 "Resolving_Angle"},
	{WResolving_Value_Radial,
	 "Resolving_Value_Radial"},
	{WResolving_Value_Tangential,
	 "Resolving_Value_Tangential"},
	{WRight_Ascension_of_Ascending_Node,
	 "Right_Ascension_of_Ascending_Node"},
	{WRoll,
	 "Roll"},
	{WRoll_Accuracy,
	 "Roll_Accuracy"},
	{WRotation_Description,
	 "Rotation_Description"},
	{WRotation_Sequence,
	 "Rotation_Sequence"},
	{WRow,
	 "Row"},
	{WRow_Delta_X,
	 "Row_Delta_X"},
	{WRow_Delta_Y,
	 "Row_Delta_Y"},
	{WSample_Angle_Units,
	 "Sample_Angle_Units"},
	{WSample_Description_Units,
	 "Sample_Description_Units"},
	{WSample_Profile_Units,
	 "Sample_Profile_Units"},
	{WSample_Properties,
	 "Sample_Properties"},
	{WSatellite_Local_Azimuth_Angle,
	 "Satellite/Local_Azimuth_Angle"},
	{WSatellite_Local_Zenith_Angle,
	 "Satellite/Local_Zenith_Angle"},
	{WScan,
	 "Scan"},
	{WScan_Angle_Units,
	 "Scan_Angle_Units"},
	{WScan_Cross_Track_Properties,
	 "Scan_Cross_Track_Properties"},
	{WScan_Distance_Units,
	 "Scan_Distance_Units"},
	{WScan_Duration,
	 "Scan_Duration"},
	{WScan_Elevation_Properties,
	 "Scan_Elevation_Properties"},
	{WScan_Geometric_Properties,
	 "Scan_Geometric_Properties"},
	{WScan_Radiometric_Properties,
	 "Scan_Radiometric_Properties"},
	{WScan_Repeat_Time,
	 "Scan_Repeat_Time"},
	{WScan_Spectral_Properties,
	 "Scan_Spectral_Properties"},
	{WScan_Start_Time,
	 "Scan_Start_Time"},
	{WScan_Step_Time,
	 "Scan_Step_Time"},
	{WScan_Time_Units,
	 "Scan_Time_Units"},
	{WScan_Timing,
	 "Scan_Timing"},
	{WScience_Paper,
	 "Science_Paper"},
	{WSemimajor_Axis,
	 "Semimajor_Axis"},
	{WSender_Polarization,
	 "Sender_Polarization"},
	{WSensor_Element_Location,
	 "Sensor_Element_Location"},
	{WSensor_Grid,
	 "Sensor_Grid"},
	{WSensor_Orientation,
	 "Sensor_Orientation"},
	{WSensor_System,
	 "Sensor_System"},
	{WSolar_Azimuth_Angle,
	 "Solar_Azimuth_Angle"},
	{WSolar_Zenith_Angle,
	 "Solar_Zenith_Angle"},
	{WSource_of_Element_Locations,
	 "Source_of_Element_Locations"},
	{WSpatial_Resolution_Units,
	 "Spatial_Resolution_Units"},
	{WSpatial_Resolution_Value,
	 "Spatial_Resolution_Value"},
	{WSpectral_Information,
	 "Spectral_Information"},
	{WSpectral_Limit,
	 "Spectral_Limit"},
	{WStabilized_Mount,
	 "Stabilized_Mount"},
	{WStorage_Order,
	 "Storage_Order"},
	{WSwath_Track_Information,
	 "Swath_Track_Information"},
	{WThematic_Layer_Identification,
	 "Thematic_Layer_Identification"},
	{WTranslation_Description,
	 "Translation_Description"},
	{WType_of_Overlap,
	 "Type_of_Overlap"},
	{WUnreferenced_Coordinate_System,
	 "Unreferenced_Coordinate_System"},
	{WUsers_Guide,
	 "User's_Guide"},
	{WWavelength_Band_Properties,
	 "Wavelength_Band_Properties"},
	{WWavelength_Region,
	 "Wavelength_Region"},
	{WWavelength_Units,
	 "Wavelength_Units"},
	{WWorldwide_Reference_System,
	 "Worldwide_Reference_System"},
	{WX_Axis_Definition,
	 "X_Axis_Definition"},
	{WX_Position_Accuracy,
	 "X_Position_Accuracy"},
	{WY_Axis_Definition,
	 "Y_Axis_Definition"},
	{WY_Position_Accuracy,
	 "Y_Position_Accuracy"},
	{WYaw,
	 "Yaw"},
	{WYaw_Accuracy,
	 "Yaw_Accuracy"},
	{WZ_Axis_Definition,
	 "Z_Axis_Definition"},
	{WZ_Position_Accuracy,
	 "Z_Position_Accuracy"},

#ifdef USE_ESRI_EXTENSIONS

	/* ESRI extensions */

	{WAltitude_Units,
	 "Altitude_Units"},
	{WAncillary_Role,
	 "Ancillary_Role"},
	{WAncillary_Role_Attribute,
	 "Ancillary_Role_Attribute"},
	{WAttribute_Alias,
	 "Attribute_Alias"},
	{WAttribute_Defined_Domain,
	 "Attribute_Defined_Domain"},
	{WAttribute_Indexed,
	 "Attribute_Indexed"},
	{WAttribute_Number_of_Decimals,
	 "Attribute_Number_of_Decimals"},
	{WAttribute_Output_Width,
	 "Attribute_Output_Width"},
	{WAttribute_Precision,
	 "Attribute_Precision"},
	{WAttribute_Scale,
	 "Attribute_Scale"},
	{WAttribute_Type,
	 "Attribute_Type"},
	{WAttribute_Width,
	 "Attribute_Width"},
	{WAttributed_Relationship,
	 "Attributed_Relationship"},
	{WAvailable_Junction_Feature_Class,
	 "Available_Junction_Feature_Class"},
	{WAvailable_Junction_Subtype,
	 "Available_Junction_Subtype"},
	{WAvailable_Junctions,
	 "Available_Junctions"},
	{WBackground_Nodata_Value,
	 "Background_Nodata_Value"},
	{WBehrmann,
	 "Behrmann"},
	{WBinary_Enclosures,
	 "Binary_Enclosures"},
	{WBits_Per_Pixel,
	 "Bits_Per_Pixel"},
	{WBonne,
	 "Bonne"},
	{WBottom_Bounding_Coordinate,
	 "Bottom_Bounding_Coordinate"},
	{WBrowse_Graphic_Embedded,
	 "Browse_Graphic_Embedded"},
	{WCassini,
	 "Cassini"},
	{WCell_Size_X_Direction,
	 "Cell_Size_X_Direction"},
	{WCell_Size_X_Units,
	 "Cell_Size_X_Units"},
	{WCell_Size_Y_Direction,
	 "Cell_Size_Y_Direction"},
	{WCell_Size_Y_Units,
	 "Cell_Size_Y_Units"},
	{WComposite_Relationship,
	 "Composite_Relationship"},
	{WCompression_Type,
	 "Compression_Type"},
	{WConnectivity_Rule,
	 "Connectivity_Rule"},
	{WCoordinate_System_Name,
	 "Coordinate_System_Name"},
	{WCreation_Date,
	 "Creation_Date"},
	{WCreation_Time,
	 "Creation_Time"},
	{WData,
	 "Data"},
	{WData_Frame_Count,
	 "Data_Frame_Count"},
	{WData_Frame_Name,
	 "Data_Frame_Name"},
	{WData_Frames,
	 "Data_Frames"},
	{WDatabase_Name,
	 "Database_Name"},
	{WDataset_Size,
	 "Dataset_Size"},
	{WDefault_Junction_Feature_Class,
	 "Default_Junction_Feature_Class"},
	{WDefault_Junction_Subtype,
	 "Default_Junction_Subtype"},
	{WDescription_of_Association,
	 "Description_of_Association"},
	{WDescription_of_Enclosure,
	 "Description_of_Enclosure"},
	{WDescription_of_Relationship,
	 "Description_of_Relationship"},
	{WDestination_Foreign_Key,
	 "Destination_Foreign_Key"},
	{WDestination_Name,
	 "Destination_Name"},
	{WDestination_Primary_Key,
	 "Destination_Primary_Key"},
	{WDomain_Attribute_Type,
	 "Domain_Attribute_Type"},
	{WDomain_Description,
	 "Domain_Description"},
	{WDomain_Name,
	 "Domain_Name"},
	{WDomain_Owner,
	 "Domain_Owner"},
	{WDomain_Type,
	 "Domain_Type"},
	{WEckert_I,
	 "Eckert_I"},
	{WEckert_II,
	 "Eckert_II"},
	{WEckert_III,
	 "Eckert_III"},
	{WEckert_IV,
	 "Eckert_IV"},
	{WEckert_V,
	 "Eckert_V"},
	{WEckert_VI,
	 "Eckert_VI"},
	{WEdge_Feature_Class,
	 "Edge_Feature_Class"},
	{WEdge_Maximum_Cardinality,
	 "Edge_Maximum_Cardinality"},
	{WEdge_Minimum_Cardinality,
	 "Edge_Minimum_Cardinality"},
	{WEdge_Subtype,
	 "Edge_Subtype"},
	{WEnabled_Attribute,
	 "Enabled_Attribute"},
	{WEnclosure,
	 "Enclosure"},
	{WEntity_Type_Count,
	 "Entity_Type_Count"},
	{WEntity_Type_Type,
	 "Entity_Type_Type"},
	{WESRI_Feature_Count,
	 "ESRI_Feature_Count"},
	{WESRI_Feature_Geometry,
	 "ESRI_Feature_Geometry"},
	{WESRI_Feature_Type,
	 "ESRI_Feature_Type"},
	{WESRI_Information,
	 "ESRI_Information"},
	{WESRI_Terms_Description,
	 "ESRI_Terms_Description"},
	{WESRI_Topology,
	 "ESRI_Topology"},
	{WFeature_Description,
	 "Feature_Description"},
	{WFile_or_Table_Name,
	 "File_or_Table_Name"},
	{WFrom_Edge_Feature_Class,
	 "From_Edge_Feature_Class"},
	{WFrom_Edge_Subtype,
	 "From_Edge_Subtype"},
	{WGall_Stereographic,
	 "Gall_Stereographic"},
	{WGeocoding_Information,
	 "Geocoding_Information"},
	{WGeographic_Coordinate_System_Name,
	 "Geographic_Coordinate_System_Name"},
	{WGeometric_Network_Information,
	 "Geometric_Network_Information"},
	{WImage_Colormap,
	 "Image_Colormap"},
	{WImage_Format,
	 "Image_Format"},
	{WImage_Type,
	 "Image_Type"},
	{WInstance_Name,
	 "Instance_Name"},
	{WJunction_Feature_Class,
	 "Junction_Feature_Class"},
	{WJunction_Maximum_Cardinality,
	 "Junction_Maximum_Cardinality"},
	{WJunction_Minimum_Cardinality,
	 "Junction_Minimum_Cardinality"},
	{WJunction_Subtype,
	 "Junction_Subtype"},
	{WLanguage_of_Dataset,
	 "Language_of_Dataset"},
	{WLanguage_of_Metadata,
	 "Language_of_Metadata"},
	{WLeft_Bounding_Coordinate,
	 "Left_Bounding_Coordinate"},
	{WLinear_Referencing,
	 "Linear_Referencing"},
	{WLocal_Bounding_Coordinates,
	 "Local_Bounding_Coordinates"},
	{WLoximuthal,
	 "Loximuthal"},
	{WMaximum_Altitude,
	 "Maximum_Altitude"},
	{WMerge_Rule,
	 "Merge_Rule"},
	{WMetadata_ID,
	 "Metadata_ID"},
	{WMinimum_Altitude,
	 "Minimum_Altitude"},
	{WModification_Date,
	 "Modification_Date"},
	{WModification_Time,
	 "Modification_Time"},
	{WMollweide,
	 "Mollweide"},
	{WNative_Dataset_Format,
	 "Native_Dataset_Format"},
	{WNetwork_Element,
	 "Network_Element"},
	{WNetwork_Role,
	 "Network_Role"},
	{WNetwork_Type,
	 "Network_Type"},
	{WNotification_Direction,
	 "Notification_Direction"},
	{WNumber_of_Raster_Bands,
	 "Number_of_Raster_Bands"},
	{WOrigin_Foreign_Key,
	 "Origin_Foreign_Key"},
	{WOrigin_Name,
	 "Origin_Name"},
	{WOrigin_Primary_Key,
	 "Origin_Primary_Key"},
	{WProcess_Software_and_Version,
	 "Process_Software_and_Version"},
	{WProjected_Coordinate_System_Name,
	 "Projected_Coordinate_System_Name"},
	{WPyramid_Layers,
	 "Pyramid_Layers"},
	{WQuartic_Authalic,
	 "Quartic_Authalic"},
	{WRange_Domain_Mean,
	 "Range_Domain_Mean"},
	{WRange_Domain_Standard_Deviation,
	 "Range_Domain_Standard_Deviation"},
	{WRaster_Display_Type,
	 "Raster_Display_Type"},
	{WRaster_Origin,
	 "Raster_Origin"},
	{WRelationship_Backward_Label,
	 "Relationship_Backward_Label"},
	{WRelationship_Cardinality,
	 "Relationship_Cardinality"},
	{WRelationship_Forward_Label,
	 "Relationship_Forward_Label"},
	{WRelationship_Information,
	 "Relationship_Information"},
	{WRight_Bounding_Coordinate,
	 "Right_Bounding_Coordinate"},
	{WRule_Category,
	 "Rule_Category"},
	{WRule_Help,
	 "Rule_Help"},
	{WRule_Type,
	 "Rule_Type"},
	{WSDE_Connection_Information,
	 "SDE_Connection_Information"},
	{WServer_Name,
	 "Server_Name"},
	{WSpatial_Index,
	 "Spatial_Index"},
	{WSplit_Rule,
	 "Split_Rule"},
	{WSubtype_Attribute,
	 "Subtype_Attribute"},
	{WSubtype_Attribute_Name,
	 "Subtype_Attribute_Name"},
	{WSubtype_Code,
	 "Subtype_Code"},
	{WSubtype_Default_Value,
	 "Subtype_Default_Value"},
	{WSubtype_Information,
	 "Subtype_Information"},
	{WSubtype_Name,
	 "Subtype_Name"},
	{WSynchronization_Date,
	 "Synchronization_Date"},
	{WSynchronization_Time,
	 "Synchronization_Time"},
	{WSynchronize_Automatically,
	 "Synchronize_Automatically"},
	{WSynchronize_Once,
	 "Synchronize_Once"},
	{WThumbnail,
	 "Thumbnail"},
	{WTo_Edge_Feature_Class,
	 "To_Edge_Feature_Class"},
	{WTo_Edge_Subtype,
	 "To_Edge_Subtype"},
	{WTop_Bounding_Coordinate,
	 "Top_Bounding_Coordinate"},
	{WUser_Name,
	 "User_Name"},
	{WVersion_Name,
	 "Version_Name"},
	{WWinkel_I,
	 "Winkel_I"},
	{WWinkel_II,
	 "Winkel_II"},
	{WGeographic_Box,
	 "Geographic_Box"},

#endif

	{Wunknown, NULL}
	};

/*----------------------------------------------------------------------*\
 | Espaol																|
 |																		|
 | Spanish-language element names kindly provided by					|
 | Clearinghouse Nacional de Datos Geogrficos, Uruguay					|
 | <http://www.clearinghouse.gub.uy/>									|
\*----------------------------------------------------------------------*/

static struct keyword_list es_std[] = {
	{WAbscissa_Resolution,
	 "Resolucin_de_Abscisa"},
	{WAbstract,
	 "Resumen"},
	{WAccess_Constraints,
	 "Restricciones_de_Acceso"},
	{WAccess_Instructions,
	 "Instrucciones_de_Acceso"},
	{WAddress,
	 "Direccin"},
	{WAddress_Type,
	 "Tipo_de_Direccin"},
	{WAlbers_Conical_Equal_Area,
	 "Iso_rea_Azimutal_de_Albers"},
	{WAltitude_Datum_Name,
	 "Nombre_del_Datum_de_Altitud"},
	{WAltitude_Distance_Units,
	 "Unidades_de_Distancia_en_Altitud"},
	{WAltitude_Encoding_Method,
	 "Mtodo_de_Codificacin_de_Altitud"},
	{WAltitude_Resolution,
	 "Resolucin_en_Altitud"},
	{WAltitude_System_Definition,
	 "Definicin_del_Sistema_de_Altitud"},
	{WARC_Coordinate_System,
	 "Sistema_ARC_de_Coordenadas"},
	{WARC_System_Zone_Identifier,
	 "Identificador_de_Zona_del_Sistema_ARC"},
	{WAttribute,
	 "Atributos"},
	{WAttribute_Accuracy,
	 "Atributo_de_Precisin"},
	{WAttribute_Accuracy_Explanation,
	 "Descripcin_de_la_Precisin_del_Atributo"},
	{WAttribute_Accuracy_Report,
	 "Informe_Sobre_Atributo_de_Precisin"},
	{WAttribute_Accuracy_Value,
	 "Valor_del_Atributo_de_Precisin"},
	{WAttribute_Definition,
	 "Definicin_del_Atributo"},
	{WAttribute_Definition_Source,
	 "Fuente_de_Definicin_de_Atributos"},
	{WAttribute_Domain_Values,
	 "Rango_de_Valores_de_Atributos"},
	{WAttribute_Label,
	 "Etiqueta_del_Atributo"},
	{WAttribute_Measurement_Frequency,
	 "Frecuencia_de_Medidas_del_Atributo"},
	{WAttribute_Measurement_Resolution,
	 "Resolucin_de_Medidas_del_Atributo"},
	{WAttribute_Units_of_Measure,
	 "Unidades_de_Medida_del_Atributo"},
	{WAttribute_Value_Accuracy,
	 "Precisin_de_Valores_del_Atributo"},
	{WAttribute_Value_Accuracy_Explanation,
	 "Explicacin_de_Precisin_de_Valores_del_Atributo"},
	{WAttribute_Value_Accuracy_Information,
	 "Informacin_de_Precisin_de_Valores_del_Atributo"},
	{WAvailable_Time_Period,
	 "Perodo_Con_Datos_Disponibles"},
	{WAzimuthal_Angle,
	 "ngulo_Azimutal"},
	{WAzimuthal_Equidistant,
	 "Equidistante_Azimutal"},
	{WAzimuth_Measure_Point_Longitude,
	 "Longitud_del_Azimut_del_Punto_de_Medida"},
	{WBearing_Reference_Direction,
	 "Direccin_del_Rumbo_de_Referencia"},
	{WBearing_Reference_Meridian,
	 "Meridiano_del_Rumbo_de_Referencia"},
	{WBearing_Resolution,
	 "Resolucin_de_Rumbo"},
	{WBearing_Units,
	 "Unidades_de_Rumbo"},
	{WBeginning_Date,
	 "Fecha_de_Comienzo"},
	{WBeginning_Date_of_Attribute_Values,
	 "Fecha_Inicial_de_Valores_del_Atributo"},
	{WBeginning_Time,
	 "Hora_de_Comienzo"},
	{WBounding_Coordinates,
	 "Coordenadas_Lmites"},
	{WBrowse_Graphic,
	 "Grfico_Para_Visualizar"},
	{WBrowse_Graphic_File_Description,
	 "Descripcin_del_Archivo_Grfico_Para_Visualizar"},
	{WBrowse_Graphic_File_Name,
	 "Nombre_del_Archivo_Grfico_Para_Visualizar"},
	{WBrowse_Graphic_File_Type,
	 "Tipo_del_Archivo_Grfico_Para_Visualizar"},
	{WCalendar_Date,
	 "Fecha_Calendario"},
	{WCitation,
	 "Cita"},
	{WCitation_Information,
	 "Informacin_Para_Cita"},
	{WCity,
	 "Ciudad"},
	{WCloud_Cover,
	 "Cubrimiento_por_Nubes"},
	{WCodeset_Domain,
	 "Dominio_del_Codeset"},
	{WCodeset_Name,
	 "Nombre_del_Codeset"},
	{WCodeset_Source,
	 "Fuente_del_Codeset"},
	{WColumn_Count,
	 "Nmero_de_Objetos_Raster_en_Eje_X"},
	{WCompatibility_Information,
	 "Informacin_de_Compatibilidad"},
	{WCompleteness_Report,
	 "Informe_Sobre_Completitud"},
	{WCompression_Support,
	 "Protocolos_de_Compresin_Soportados"},
	{WComputer_Contact_Information,
	 "Informacin_Para_Contacto_Con_Computador"},
	{WContact_Address,
	 "Direccin_de_Contacto"},
	{WContact_Electronic_Mail_Address,
	 "Correo_Electrnico_de_Contacto"},
	{WContact_Facsimile_Telephone,
	 "Facsmil_de_Contacto"},
	{WContact_Information,
	 "Informacin_de_Contacto"},
	{WContact_Instructions,
	 "Instrucciones_Para_Contacto"},
	{WContact_Organization,
	 "Organizacin_Para_Contacto"},
	{WContact_Organization_Primary,
	 "Contacto_Preferentemente_Con_Organizacin"},
	{WContact_Person,
	 "Persona_Para_Contacto"},
	{WContact_Person_Primary,
	 "Contacto_Preferentemente_Con_Persona"},
	{WContact_Position,
	 "Cargo_del_Contacto"},
	{WContact_TDD_TTY_Telephone,
	 "TDD_TTY_de_Contacto"},
	{WContact_Voice_Telephone,
	 "Telfono_de_Contacto"},
	{WCoordinate_Representation,
	 "Representacin_de_Coordenadas"},
	{WCountry,
	 "Pas"},
	{WCross_Reference,
	 "Referencia_Cruzada"},
	{WCurrentness_Reference,
	 "Vigencia"},
	{WCustom_Order_Process,
	 "Procesamiento_Personalizado_de_Pedidos"},
	{WData_Quality_Information,
	 "Informacin_de_Calidad_de_Datos"},
	{WData_Set_Credit,
	 "Crdito_del_Conjunto_de_Datos"},
	{WData_Set_G_Polygon,
	 "Polgono_Lmite_de_los_Datos"},
	{WData_Set_G_Polygon_Exclusion_G_Ring,
	 "Polgono_Excludo_de_los_Datos"},
	{WData_Set_G_Polygon_Outer_G_Ring,
	 "Polgono_Externo_de_los_Datos"},
	{WDenominator_of_Flattening_Ratio,
	 "Denominador_de_la_Razn_de_Aplastamiento"},
	{WDepth_Datum_Name,
	 "Nombre_del_Datum_de_Profundidad"},
	{WDepth_Distance_Units,
	 "Unidades_de_Distancia_en_Profundidad"},
	{WDepth_Encoding_Method,
	 "Mtodo_de_Codificacin_de_Profundidad"},
	{WDepth_Resolution,
	 "Resolucin_en_Profundidad"},
	{WDepth_System_Definition,
	 "Definicin_del_Sistema_de_Profundidad"},
	{WDescription,
	 "Descripcin"},
	{WDetailed_Description,
	 "Descripcin_Detallada"},
	{WDialup_File_Name,
	 "Nombre_de_Archivo_de_Discado_Automtico"},
	{WDialup_Instructions,
	 "Instrucciones_Para_Discado_Automtico"},
	{WDialup_Telephone,
	 "Nmero_Telefnico_Para_Discado_Automtico"},
	{WDigital_Form,
	 "Formato_Digital"},
	{WDigital_Transfer_Information,
	 "Informacin_Sobre_Transferencia_Digital"},
	{WDigital_Transfer_Option,
	 "Opcin_Sobre_Transferencia_Digital"},
	{WDirect_Spatial_Reference_Method,
	 "Mtodo_de_Referencia_Espacial_Directa"},
	{WDistance_and_Bearing_Representation,
	 "Representacin_de_Distancia_y_Rumbo"},
	{WDistance_Resolution,
	 "Resolucin_de_Distancia"},
	{WDistribution_Information,
	 "Informacin_de_Distribucin"},
	{WDistribution_Liability,
	 "Responsabilidades_del_Distribuidor"},
	{WDistributor,
	 "Distribuidor"},
	{WEast_Bounding_Coordinate,
	 "Coordenada_Lmite_al_Este"},
	{WEdition,
	 "Edicin"},
	{WEllipsoid_Name,
	 "Nombre_del_Elipsoide"},
	{WEnding_Date,
	 "Fecha_de_Finalizacin"},
	{WEnding_Date_of_Attribute_Values,
	 "Fecha_Final_de_Valores_del_Atributo"},
	{WEnding_Time,
	 "Hora_de_Finalizacin"},
	{WEntity_and_Attribute_Detail_Citation,
	 "Cita_Detallada_de_Entidad_y_Atributo"},
	{WEntity_and_Attribute_Information,
	 "Informacin_de_Entidad_y_Atributo"},
	{WEntity_and_Attribute_Overview,
	 "Resumen_Somero_de_Entidad_y_Atributo"},
	{WEntity_Type,
	 "Tipo_de_Entidad"},
	{WEntity_Type_Definition,
	 "Definicin_de_Tipo_de_Entidad"},
	{WEntity_Type_Definition_Source,
	 "Fuente_de_Definicin_de_Tipo_de_Entidad"},
	{WEntity_Type_Label,
	 "Etiqueta_de_Tipo_de_Entidad"},
	{WEnumerated_Domain,
	 "Dominio_Enumerado"},
	{WEnumerated_Domain_Value,
	 "Valores_del_Dominio_Enumerado"},
	{WEnumerated_Domain_Value_Definition,
	 "Definicin_del_Dominio_Enumerado"},
	{WEnumerated_Domain_Value_Definition_Source,
	 "Fuente_Para_la_Definicin_del_Dominio_Enumerado"},
	{WEquidistant_Conic,
	 "Cnica_Equidistante"},
	{WEquirectangular,
	 "Equirrectangular"},
	{WFalse_Easting,
	 "Falso_Este"},
	{WFalse_Northing,
	 "Falso_Norte"},
	{WFees,
	 "Tarifas"},
	{WFile_Decompression_Technique,
	 "Tcnica_de_Descompresin_de_Archivos"},
	{WFormat_Information_Content,
	 "Informacin_del_Formato_del_Contenido"},
	{WFormat_Name,
	 "Nombre_del_Formato"},
	{WFormat_Specification,
	 "Especificacin_del_Formato"},
	{WFormat_Version_Date,
	 "Fecha_de_Versin_del_Formato"},
	{WFormat_Version_Number,
	 "Nmero_de_Versin_del_Formato"},
	{WG_Ring,
	 "Poligonal_Geogrfica"},
	{WG_Ring_Latitude,
	 "Latitud_de_Poligonal"},
	{WG_Ring_Longitude,
	 "Longitud_de_Poligonal"},
	{WG_Ring_Point,
	 "Punto_de_Poligonal_Geogrfica"},
	{WGeneral_Vertical_Near_sided_Perspective,
	 "Perspectiva_de_Costado_General_Vertical"},
	{WGeodetic_Model,
	 "Modelo_Geodtico"},
	{WGeographic,
	 "Geogrfico"},
	{WGeographic_Coordinate_Units,
	 "Unidades_de_las_Coordenadas_Geogrficas"},
	{WGeospatial_Data_Presentation_Form,
	 "Forma_de_Presentacin_Geoespacial"},
	{WGnomonic,
	 "Nomnica"},
	{WGrid_Coordinate_System,
	 "Sistema_de_Coordenadas_de_Cuadrcula"},
	{WGrid_Coordinate_System_Name,
	 "Nombre_del_Sistema_de_Coordenadas_de_Cuadrcula"},
	{WHeight_of_Perspective_Point_Above_Surface,
	 "Altura_del_Punto_de_Perspectiva_Sobre_la_Superficie"},
	{WHighest_BPS,
	 "Mxima_BPS"},
	{WHorizontal_Coordinate_System_Definition,
	 "Definicin_del_Sistema_de_Coordenadas_Horizontales"},
	{WHorizontal_Datum_Name,
	 "Nombre_del_Datum_Horizontal"},
	{WHorizontal_Positional_Accuracy,
	 "Precisin_en_la_Horizontal"},
	{WHorizontal_Positional_Accuracy_Explanation,
	 "Interpretacin_de_Precisin_Posicional_Horizontal"},
	{WHorizontal_Positional_Accuracy_Report,
	 "Informe_Sobre_Precisin_en_la_Horizontal"},
	{WHorizontal_Positional_Accuracy_Value,
	 "Valor_de_Precisin_en_la_Horizontal"},
	{WHours_of_Service,
	 "Horario_de_Atencin"},
	{WIdentification_Information,
	 "Informacin_de_Identificacin"},
	{WIndirect_Spatial_Reference,
	 "Referencia_Espacial_Indirecta"},
	{WIssue_Identification,
	 "Identificacin_del_Nmero"},
	{WKeywords,
	 "Palabras_Clave"},
	{WLambert_Azimuthal_Equal_Area,
	 "Iso_rea_Azimutal_de_Lambert"},
	{WLambert_Conformal_Conic,
	 "Cnica_Conforme_de_Lambert"},
	{WLandsat_Number,
	 "Nmero_Landsat"},
	{WLarger_Work_Citation,
	 "Cita_a_Trabajo_Mayor"},
	{WLatitude_of_Projection_Center,
	 "Latitud_del_Centro_de_la_Proyeccin"},
	{WLatitude_of_Projection_Origin,
	 "Latitud_del_Origen_de_la_Proyeccin"},
	{WLatitude_Resolution,
	 "Resolucin_en_Latitud"},
	{WLineage,
	 "Linaje"},
	{WLocal,
	 "Local"},
	{WLocal_Description,
	 "Descripcin_Local"},
	{WLocal_Georeference_Information,
	 "Informacin_de_Georreferenciacin_Local"},
	{WLocal_Planar,
	 "Plana_Local"},
	{WLocal_Planar_Description,
	 "Descripcin_de_la_Plana_Local"},
	{WLocal_Planar_Georeference_Information,
	 "Informacin_de_la_Georreferencia_de_la_Plana_Local"},
	{WLogical_Consistency_Report,
	 "Informe_Sobre_Consistencia_Lgica"},
	{WLongitude_of_Central_Meridian,
	 "Longitud_del_Meridiano_Central"},
	{WLongitude_of_Projection_Center,
	 "Longitud_del_Centro_de_la_Proyeccin"},
	{WLongitude_Resolution,
	 "Resolucin_en_Longitud"},
	{WLowest_BPS,
	 "Mnima_BPS"},
	{WMaintenance_and_Update_Frequency,
	 "Frecuencia_de_Mantenimiento_y_Actualizacin"},
	{WMap_Projection,
	 "Proyeccin_Cartogrfica"},
	{WMap_Projection_Name,
	 "Nombre_de_la_Proyeccin_Cartogrfica"},
	{WMap_Projection_Parameters,
	 "Parmetros_de_la_Proyeccin_Cartogrfica"},
	{WMercator,
	 "Mercator"},
	{WMetadata,
	 "Metadatos"},
	{WMetadata_Access_Constraints,
	 "Restricciones_al_Acceso_a_los_Metadatos"},
	{WMetadata_Contact,
	 "Contacto_Por_Metadatos"},
	{WMetadata_Date,
	 "Fecha_de_Metadatos"},
	{WMetadata_Extensions,
	 "Extensiones_al_Estndar_de_Metadatos"},
	{WMetadata_Future_Review_Date,
	 "Fecha_de_Futura_Revisin_de_Metadatos"},
	{WMetadata_Language,
	 "Lenguaje_de_Metadatos"},
	{WMetadata_Reference_Information,
	 "Informacin_de_Referencia_de_Metadatos"},
	{WMetadata_Review_Date,
	 "Fecha_de_ltima_Revisin_de_Metadatos"},
	{WMetadata_Security_Classification,
	 "Clasificacin_de_Seguridad_de_Metadatos"},
	{WMetadata_Security_Classification_System,
	 "Sistema_de_Clasificacin_de_Seguridad_de_Metadatos"},
	{WMetadata_Security_Handling_Description,
	 "Descripcin_de_Precauciones_de_Manejo_de_Metadatos"},
	{WMetadata_Security_Information,
	 "Informacin_de_Seguridad_de_Metadatos"},
	{WMetadata_Standard_Name,
	 "Nombre_del_Estndar_de_Metadatos"},
	{WMetadata_Standard_Version,
	 "Versin_del_Estndar_de_Metadatos"},
	{WMetadata_Time_Convention,
	 "Convencin_de_Fechas_en_Metadatos"},
	{WMetadata_Use_Constraints,
	 "Restricciones_al_Uso_de_Metadatos"},
	{WMiller_Cylindrical,
	 "Cilndrica_de_Miller"},
	{WModified_Stereographic_for_Alaska,
	 "Estereogrfica_Modificada_Para_Alaska"},
	{WMultiple_Dates_Times,
	 "Mltiples_Fecha_y_Hora"},
	{WNative_Data_Set_Environment,
	 "Entorno_Original_de_los_Datos"},
	{WNetwork_Address,
	 "Direccin_de_la_Red"},
	{WNetwork_Resource_Name,
	 "Nombre_del_Recurso_de_Red"},
	{WNon_digital_Form,
	 "Formato_No_Digital"},
	{WNorth_Bounding_Coordinate,
	 "Coordenada_Lmite_al_Norte"},
	{WNumber_DataBits,
	 "Nmero_de_Bits_de_Datos"},
	{WNumber_StopBits,
	 "Nmero_de_Bits_de_Parada"},
	{WOblique_Line_Azimuth,
	 "Azimuth_de_Lnea_Oblicua"},
	{WOblique_Line_Latitude,
	 "Latitud_de_Lnea_Oblicua"},
	{WOblique_Line_Longitude,
	 "Longitud_de_Lnea_Oblicua"},
	{WOblique_Line_Point,
	 "Punto_de_Lnea_Oblicua"},
	{WOblique_Mercator,
	 "Mercator_Oblicuo"},
	{WOffline_Media,
	 "Medio_Fsico_Fuera_de_Lnea"},
	{WOffline_Option,
	 "Opcin_Fuera_de_Lnea"},
	{WOnline_Computer_and_Operating_System,
	 "Computadora_y_Sistema_Operativo_en_Lnea"},
	{WOnline_Linkage,
	 "Acceso_en_Lnea"},
	{WOnline_Option,
	 "Opcin_en_Lnea"},
	{WOrdering_Instructions,
	 "Instrucciones_Para_el_Pedido"},
	{WOrdinate_Resolution,
	 "Resolucin_de_Ordenadas"},
	{WOriginator,
	 "Productor"},
	{WOrthographic,
	 "Ortogrfico"},
	{WOther_Citation_Details,
	 "Otros_Detalles_de_la_Cita"},
	{WOther_Grid_Systems_Definition,
	 "Definicin_de_Otro_Sistema_de_Cuadrcula"},
	{WOther_Projections_Definition,
	 "Definicin_de_Otra_Proyeccin"},
	{WOverview_Description,
	 "Descripcin_Somera"},
	{WParity,
	 "Paridad"},
	{WPath_Number,
	 "Nmero_de_Camino"},
	{WPlace,
	 "Lugar"},
	{WPlace_Keyword,
	 "Palabra_Clave_de_Lugar"},
	{WPlace_Keyword_Thesaurus,
	 "Tesauro_de_Palabras_Clave_de_Lugar"},
	{WPlanar,
	 "Plano"},
	{WPlanar_Coordinate_Encoding_Method,
	 "Mtodo_Codificado_de_Coordenadas_Plana"},
	{WPlanar_Coordinate_Information,
	 "Informacin_de_Coordenadas_Plana"},
	{WPlanar_Distance_Units,
	 "Unidades_de_Distancias_Planas"},
	{WPoint_and_Vector_Object_Count,
	 "Cantidad_de_Objetos_Punto_y_Vector"},
	{WPoint_and_Vector_Object_Information,
	 "Informacin_de_Objetos_Punto_y_Vector"},
	{WPoint_of_Contact,
	 "Punto_de_Contacto"},
	{WPolar_Stereographic,
	 "Estereogrfico_Polar"},
	{WPolyconic,
	 "Policnico"},
	{WPositional_Accuracy,
	 "Precisin_de_la_Posicin"},
	{WPostal_Code,
	 "Cdigo_Postal"},
	{WProcess_Contact,
	 "Contacto_Por_el_Proceso"},
	{WProcess_Date,
	 "Fecha_del_Proceso"},
	{WProcess_Description,
	 "Descripcin_del_Proceso"},
	{WProcess_Step,
	 "Paso_del_Proceso"},
	{WProcess_Time,
	 "Hora_del_Proceso"},
	{WProfile_Name,
	 "Nombre_del_Perfil"},
	{WProgress,
	 "Avance"},
	{WPublication_Date,
	 "Fecha_de_Publicacin"},
	{WPublication_Information,
	 "Informacin_de_la_Publicacin"},
	{WPublication_Place,
	 "Lugar_de_la_Publicacin"},
	{WPublication_Time,
	 "Hora_de_Publicacin"},
	{WPublisher,
	 "Impresor"},
	{WPurpose,
	 "Propsito"},
	{WQuantitative_Attribute_Accuracy_Assessment,
	 "Determinacin_Cuantitativa_Sobre_Precisin_en_Atributos"},
	{WQuantitative_Horizontal_Positional_Accuracy_Assessment,
	 "Declaracin_de_Precisin_Posicional_Horizontal"},
	{WQuantitative_Vertical_Positional_Accuracy_Assessment,
	 "Declaracin_de_Precisin_Posicional_Vertical"},
	{WRange_Domain,
	 "Dominio_del_Rango"},
	{WRange_Domain_Maximum,
	 "Mximo_del_Dominio_del_Rango"},
	{WRange_Domain_Minimum,
	 "Mnimo_del_Dominio_del_Rango"},
	{WRange_of_Dates_Times,
	 "Rango_de_Fechas_y_Horas"},
	{WRaster_Object_Information,
	 "Informacin_Sobre_Objetos_Raster"},
	{WRaster_Object_Type,
	 "Tipo_de_Objetos_Raster"},
	{WRecording_Capacity,
	 "Capacidad_de_Grabacin"},
	{WRecording_Density,
	 "Densidad_de_Grabacin"},
	{WRecording_Density_Units,
	 "Unidades_de_Densidad_de_Grabacin"},
	{WRecording_Format,
	 "Formato_de_Grabacin"},
	{WResource_Description,
	 "Descripcin_del_Recurso"},
	{WRobinson,
	 "Robinson"},
	{WRow_Count,
	 "Nmero_de_Objetos_Raster_en_Eje_Y"},
	{WScale_Factor_at_Center_Line,
	 "Factor_de_Escala_en_la_Lnea_Central"},
	{WScale_Factor_at_Central_Meridian,
	 "Factor_de_Escala_en_el_Meridiano_Central"},
	{WScale_Factor_at_Equator,
	 "Factor_de_Escala_en_el_Ecuador"},
	{WScale_Factor_at_Projection_Origin,
	 "Factor_de_Escala_en_el_Origen_de_la_Proyeccin"},
	{WSDTS_Point_and_Vector_Object_Type,
	 "Tipo_de_Objetos_Punto_y_Vector_SDTS"},
	{WSDTS_Terms_Description,
	 "Descripcin_de_Trminos_SDTS"},
	{WSecurity_Classification,
	 "Clasificacin_de_Seguridad"},
	{WSecurity_Classification_System,
	 "Sistema_de_Clasificacin_de_Seguridad"},
	{WSecurity_Handling_Description,
	 "Descripcin_del_Manejo_de_Seguridad"},
	{WSecurity_Information,
	 "Informacin_de_Seguridad"},
	{WSemi_major_Axis,
	 "Semi-eje_Mayor"},
	{WSeries_Information,
	 "Informacin_de_la_Serie"},
	{WSeries_Name,
	 "Nombre_de_la_Serie"},
	{WSingle_Date_Time,
	 "Fecha_y_Hora"},
	{WSinusoidal,
	 "Sinusoidal"},
	{WSource_Citation,
	 "Cita_de_la_Fuente"},
	{WSource_Citation_Abbreviation,
	 "Abreviacin_de_Cita_de_la_Fuente"},
	{WSource_Contribution,
	 "Contribucin_de_la_Fuente"},
	{WSource_Currentness_Reference,
	 "Referencia_a_Vigencia_de_la_Fuente"},
	{WSource_Information,
	 "Informacin_de_la_Fuente"},
	{WSource_Produced_Citation_Abbreviation,
	 "Abreviatura_de_Cita_de_la_Fuente_Producida"},
	{WSource_Scale_Denominator,
	 "Denominador_de_Escala_de_la_Fuente"},
	{WSource_Time_Period_of_Content,
	 "Perodo_Contenido_en_la_Fuente"},
	{WSource_Used_Citation_Abbreviation,
	 "Abreviatura_Usual_de_Cita_de_la_Fuente"},
	{WSouth_Bounding_Coordinate,
	 "Coordenada_Lmite_al_Sur"},
	{WSpace_Oblique_Mercator_Landsat,
	 "Mercator_Espacial_Oblicua_(Landsat)"},
	{WSpatial_Data_Organization_Information,
	 "Informacin_de_Organizacin_de_Datos_Espaciales"},
	{WSpatial_Domain,
	 "Dominio_Espacial"},
	{WSpatial_Reference_Information,
	 "Informacin_de_Referencia_Espacial"},
	{WSPCS_Zone_Identifier,
	 "Identificador_de_Zona_SPCS"},
	{WStandard_Order_Process,
	 "Procesamiento_Estndar_de_Pedidos"},
	{WStandard_Parallel,
	 "Paralelo_Estndar"},
	{WState_or_Province,
	 "Estado_O_Provincia"},
	{WState_Plane_Coordinate_System,
	 "Sistema_Estatal_de_Coordenadas_Planas"},
	{WStatus,
	 "Estatus"},
	{WStereographic,
	 "Estereogrfica"},
	{WStraight_Vertical_Longitude_from_Pole,
	 "Longitud_Vertical-Recta_Desde_el_Polo"},
	{WStratum,
	 "Estrato"},
	{WStratum_Keyword,
	 "Palabra_Clave_de_Estrato"},
	{WStratum_Keyword_Thesaurus,
	 "Tesauro_de_Palabras_Clave_de_Estrato"},
	{WSupplemental_Information,
	 "Informacin_Suplementaria"},
	{WTechnical_Prerequisites,
	 "Prerrequisitos_Tcnicos"},
	{WTemporal,
	 "Temporal"},
	{WTemporal_Keyword,
	 "Palabra_Clave_de_Tiempo"},
	{WTemporal_Keyword_Thesaurus,
	 "Tesauro_de_Palabras_Clave_de_Tiempo"},
	{WTheme,
	 "Tema"},
	{WTheme_Keyword,
	 "Palabra_Clave_de_Tema"},
	{WTheme_Keyword_Thesaurus,
	 "Tesauro_de_Palabras_Clave_de_Tema"},
	{WTime_of_Day,
	 "Hora_del_Da"},
	{WTime_Period_Information,
	 "Informacin_de_Perodo_de_Tiempo"},
	{WTime_Period_of_Content,
	 "Perodo_Asociado_al_Contenido"},
	{WTitle,
	 "Ttulo"},
	{WTransfer_Size,
	 "Tamao_de_Transferencia"},
	{WTransverse_Mercator,
	 "Mercator_Transverso"},
	{WTurnaround,
	 "Tiempo_de_Entrega"},
	{WType_of_Source_Media,
	 "Tipo_de_Medio_Fsico_de_la_Fuente"},
	{WUniversal_Polar_Stereographic,
	 "Estereogrfica_Polar_Universal"},
	{WUniversal_Transverse_Mercator,
	 "Mercator_Transversa_Universal"},
	{WUnrepresentable_Domain,
	 "Dominio_No_Representable"},
	{WUPS_Zone_Identifier,
	 "Identificador_de_Zona_UPS"},
	{WUse_Constraints,
	 "Restricciones_de_Uso"},
	{WUTM_Zone_Number,
	 "Nmero_de_Zona_UTM"},
	{Wvan_der_Grinten,
	 "van_der_Grinten"},
	{WVertical_Coordinate_System_Definition,
	 "Definicin_del_Sistema_de_Coordenadas_Verticales"},
	{WVertical_Count,
	 "Nmero_de_Objetos_Raster_en_Eje_Z"},
	{WVertical_Positional_Accuracy,
	 "Precisin_en_la_Vertical"},
	{WVertical_Positional_Accuracy_Explanation,
	 "Interpretacin_de_Precisin_Posicional_Vertical"},
	{WVertical_Positional_Accuracy_Report,
	 "Informe_Sobre_Precisin_en_la_Vertical"},
	{WVertical_Positional_Accuracy_Value,
	 "Valor_de_Precisin_en_la_Vertical"},
	{WVPF_Point_and_Vector_Object_Information,
	 "Informacin_de_Objetos_Punto_y_Vector_VPF"},
	{WVPF_Point_and_Vector_Object_Type,
	 "Tipo_de_Objetos_Punto_y_Vector_VPF"},
	{WVPF_Terms_Description,
	 "Descripcin_de_Trminos_VPF"},
	{WVPF_Topology_Level,
	 "Nivel_de_Topologa_VPF"},
	{WWest_Bounding_Coordinate,
	 "Coordenada_Lmite_al_Oeste"},

/* Biological Data Profile */

	{WDescription_of_Geographic_Extent,
		"Description_of_Geographic_Extent"},
	{WBounding_Altitudes,
		"Bounding_Altitudes"},
	{WAltitude_Minimum,
		"Altitude_Minimum"},
	{WAltitude_Maximum,
		"Altitude_Maximum"},
	{WTaxonomy,
		"Taxonomy"},
	{WKeywords_Taxon,
		"Keywords/Taxon"},
	{WTaxonomic_Keyword_Thesaurus,
		"Taxonomic_Keyword_Thesaurus"},
	{WTaxonomic_Keywords,
		"Taxonomic_Keywords"},
	{WTaxonomic_System,
		"Taxonomic_System"},
	{WClassification_System_Authority,
		"Classification_System/Authority"},
	{WClassification_System_Citation,
		"Classification_System_Citation"},
	{WClassification_System_Modifications,
		"Classification_System_Modifications"},
	{WIdentification_Reference,
		"Identification_Reference"},
	{WIdentifier,
		"Identifier"},
	{WTaxonomic_Procedures,
		"Taxonomic_Procedures"},
	{WTaxonomic_Completeness,
		"Taxonomic_Completeness"},
	{WVouchers,
		"Vouchers"},
	{WSpecimen,
		"Specimen"},
	{WRepository,
		"Repository"},
	{WGeneral_Taxonomic_Coverage,
		"General_Taxonomic_Coverage"},
	{WTaxonomic_Classification,
		"Taxonomic_Classification"},
	{WTaxon_Rank_Name,
		"Taxon_Rank_Name"},
	{WTaxon_Rank_Value,
		"Taxon_Rank_Value"},
	{WApplicable_Common_Name,
		"Applicable_Common_Name"},
	{WAnalytical_Tool,
		"Analytical_Tool"},
	{WAnalytical_Tool_Description,
		"Analytical_Tool_Description"},
	{WTool_Access_Information,
		"Tool_Access_Information"},
	{WTool_Contact,
		"Tool_Contact"},
	{WTool_Citation,
		"Tool_Citation"},
	{WTool_Access_Instructions,
		"Tool_Access_Instructions"},
	{WTool_Computer_and_Operating_System,
		"Tool_Computer_and_Operating_System"},
	{WMethodology,
		"Methodology"},
	{WMethodology_Type,
		"Methodology_Type"},
	{WMethodology_Identifier,
		"Methodology_Identifier"},
	{WMethodology_Keyword_Thesaurus,
		"Methodology_Keyword_Thesaurus"},
	{WMethodology_Keyword,
		"Methodology_Keyword"},
	{WMethodology_Description,
		"Methodology_Description"},
	{WMethodology_Citation,
		"Methodology_Citation"},
	{WASCII_File_Structure,
		"ASCII_File_Structure"},
	{WRecord_Delimiter,
		"Record_Delimiter"},
	{WNumber_Header_Lines,
		"Number_Header_Lines"},
	{WDescription_of_Header_Content,
		"Description_of_Header_Content"},
	{WOrientation,
		"Orientation"},
	{WCase_Sensitive,
		"Case_Sensitive"},
	{WAuthentication,
		"Authentication"},
	{WQuote_Character,
		"Quote_Character"},
	{WData_Field,
		"Data_Field"},
	{WData_Field_Name,
		"Data_Field_Name"},
	{WMissing_Value_Code,
		"Missing_Value_Code"},
	{WData_Field_Width_Delimiter,
		"Data_Field_Width_Delimiter"},
	{WData_Field_Width,
		"Data_Field_Width"},
	{WBeginning_Geologic_Age,
		"Beginning_Geologic_Age"},
	{WEnding_Geologic_Age,
		"Ending_Geologic_Age"},
	{WGeologic_Age,
		"Geologic_Age"},
	{WGeologic_Time_Scale,
		"Geologic_Time_Scale"},
	{WGeologic_Age_Estimate,
		"Geologic_Age_Estimate"},
	{WGeologic_Age_Uncertainty,
		"Geologic_Age_Uncertainty"},
	{WGeologic_Age_Explanation,
		"Geologic_Age_Explanation"},
	{WGeologic_Citation,
		"Geologic_Citation"},

/* Shoreline profile */

	{WBarometric_Pressure,
	 "Barometric_Pressure"},
	{WEnvironmental_Event,
	 "Environmental_Event"},
	{WMarine_Weather_Condition,
	 "Marine_Weather_Condition"},
	{WProcess_Step_Citation,
	 "Process_Step_Citation"},
	{WRange_of_Tide,
	 "Range_of_Tide"},
	{WSupplemental_Tidal_Information,
	 "Supplemental_Tidal_Information"},
	{WTidal_Datum,
	 "Tidal_Datum"},
	{WTidal_Information,
	 "Tidal_Information"},
	{WTide_Table_Reference,
	 "Tide_Table_Reference"},
	{WTime_of_High_Tide,
	 "Time_of_High_Tide"},
	{WTime_of_Low_Tide,
	 "Time_of_Low_Tide"},
	{WTime_of_Tide,
	 "Time_of_Tide"},
	{WType_of_Tide,
	 "Type_of_Tide"},
	{WWave_Height,
	 "Wave_Height"},
	{WWind_Direction,
	 "Wind_Direction"},
	{WWind_Speed,
	 "Wind_Speed"},

/* Remote-sensing profile */

	{WAcquisition_Information,
	 "Acquisition_Information"},
	{WAerial_Film_Speed,
	 "Aerial_Film_Speed"},
	{WAerotriangulation_Reference,
	 "Aerotriangulation_Reference"},
	{WAffine_Distortion_X_Prime_Coefficient,
	 "Affine_Distortion_X_Prime_Coefficient"},
	{WAffine_Distortion_Y_Prime_Coefficient,
	 "Affine_Distortion_Y_Prime_Coefficient"},
	{WAggregation_Criteria,
	 "Aggregation_Criteria"},
	{WAggregation_Information,
	 "Aggregation_Information"},
	{WAggregation_Member_ID,
	 "Aggregation_Member_ID"},
	{WAlgorithm_Change_Description,
	 "Algorithm_Change_Description"},
	{WAlgorithm_Change_History,
	 "Algorithm_Change_History"},
	{WAlgorithm_Change_Reference,
	 "Algorithm_Change_Reference"},
	{WAlgorithm_Description,
	 "Algorithm_Description"},
	{WAlgorithm_Identifiers,
	 "Algorithm_Identifiers"},
	{WAlgorithm_Information,
	 "Algorithm_Information"},
	{WAlgorithm_Peer_Review_Description,
	 "Algorithm_Peer_Review_Description"},
	{WAlgorithm_Peer_Review_Information,
	 "Algorithm_Peer_Review_Information"},
	{WAlgorithm_Peer_Review_Reference,
	 "Algorithm_Peer_Review_Reference"},
	{WAlgorithm_Reference,
	 "Algorithm_Reference"},
	{WAlgorithm_Text_Description,
	 "Algorithm_Text_Description"},
	{WAltitude_Distance_Layer,
	 "Altitude_Distance_Layer"},
	{WAncillary_Dataset,
	 "Ancillary_Dataset"},
	{WAncillary_Dataset_Description,
	 "Ancillary_Dataset_Description"},
	{WAncillary_Dataset_Identifier,
	 "Ancillary_Dataset_Identifier"},
	{WAncillary_Description,
	 "Ancillary_Description"},
	{WAncillary_Reference,
	 "Ancillary_Reference"},
	{WAngle_Dependent_Distortion,
	 "Angle_Dependent_Distortion"},
	{WAngle_Distortion_Value,
	 "Angle_Distortion_Value"},
	{WArea_Weighted_Average_Resolution,
	 "Area_Weighted_Average_Resolution"},
	{WArgument_of_Perigee,
	 "Argument_of_Perigee"},
	{WAscending_Descending_Indicator,
	 "Ascending/Descending_Indicator"},
	{WAttitude_Angular_Units,
	 "Attitude_Angular_Units"},
	{WAvailability_of_Element_Locations,
	 "Availability_of_Element_Locations"},
	{WAxes,
	 "Axes"},
	{WAxis_Rotation_Convention,
	 "Axis_Rotation_Convention"},
	{WBand_Boundary_Definition,
	 "Band_Boundary_Definition"},
	{WBand_Description,
	 "Band_Description"},
	{WBand_ID,
	 "Band_ID"},
	{WBand_Identification,
	 "Band_Identification"},
	{WBand_Measurement_Mode_ID,
	 "Band_Measurement_Mode_ID"},
	{WBand_Quality,
	 "Band_Quality"},
	{WCalibrated_Detector_Positions,
	 "Calibrated_Detector_Positions"},
	{WCalibrated_Focal_Length,
	 "Calibrated_Focal_Length"},
	{WCamera_Identifier,
	 "Camera_Identifier"},
	{WCamera_Type,
	 "Camera_Type"},
	{WCell_Value_Type,
	 "Cell_Value_Type"},
	{WClock_Time_Drift,
	 "Clock_Time/Drift"},
	{WCollection_Type,
	 "Collection_Type"},
	{WColumn_Delta_X,
	 "Column_Delta_X"},
	{WColumn_Delta_Y,
	 "Column_Delta_Y"},
	{WCommand_Line_Processing_Parameter,
	 "Command_Line_Processing_Parameter"},
	{WComponent_Information,
	 "Component_Information"},
	{WContainer_Packet_ID,
	 "Container_Packet_ID"},
	{WControl_Point_Authority,
	 "Control_Point_Authority"},
	{WControl_Point_Column,
	 "Control_Point_Column"},
	{WControl_Point_Earth_Location,
	 "Control_Point_Earth_Location"},
	{WControl_Point_Horizontal_X_Accuracy,
	 "Control_Point_Horizontal_X_Accuracy"},
	{WControl_Point_Horizontal_Y_Accuracy,
	 "Control_Point_Horizontal_Y_Accuracy"},
	{WControl_Point_ID,
	 "Control_Point_ID"},
	{WControl_Point_Identification,
	 "Control_Point_Identification"},
	{WControl_Point_Origin,
	 "Control_Point_Origin"},
	{WControl_Point_Raster_Position,
	 "Control_Point_Raster_Position"},
	{WControl_Point_Row,
	 "Control_Point_Row"},
	{WControl_Point_Type,
	 "Control_Point_Type"},
	{WControl_Point_Use_Flag,
	 "Control_Point_Use_Flag"},
	{WControl_Point_Vertical_Accuracy,
	 "Control_Point_Vertical_Accuracy"},
	{WControl_Point_x_Value,
	 "Control_Point_x_Value"},
	{WControl_Point_y_Value,
	 "Control_Point_y_Value"},
	{WControl_Point_z_Value,
	 "Control_Point_z_Value"},
	{WCoordinate_Point,
	 "Coordinate_Point"},
	{WCoordinate_System,
	 "Coordinate_System"},
	{WCoordinate_x_Value,
	 "Coordinate_x_Value"},
	{WCoordinate_XY_Units,
	 "Coordinate_XY_Units"},
	{WCoordinate_y_Value,
	 "Coordinate_y_Value"},
	{WCoordinate_Z_Units,
	 "Coordinate_Z_Units"},
	{WCoordinate_z_Value,
	 "Coordinate_z_Value"},
	{WCross_Track_Axis,
	 "Cross_Track_Axis"},
	{WCross_Track_Description,
	 "Cross_Track_Description"},
	{WCross_Track_Direction,
	 "Cross_Track_Direction"},
	{WCross_Track_Extent_Angle,
	 "Cross_Track_Extent_Angle"},
	{WCross_Track_Fixed_Angle,
	 "Cross_Track_Fixed_Angle"},
	{WCross_Track_Motion,
	 "Cross_Track_Motion"},
	{WCross_Track_Start_Angle,
	 "Cross_Track_Start_Angle"},
	{WCross_Track_Step_Angle,
	 "Cross_Track_Step_Angle"},
	{WCross_Track_Sweep,
	 "Cross_Track_Sweep"},
	{WCross_Track_Zero,
	 "Cross_Track_Zero"},
	{WData_Dictionary_Reference,
	 "Data_Dictionary_Reference"},
	{WData_Scaling_Information,
	 "Data_Scaling_Information"},
	{WDataset_Identifier,
	 "Dataset_Identifier"},
	{WDate_of_Last_Calibration,
	 "Date_of_Last_Calibration"},
	{WDeveloping_Institution,
	 "Developing_Institution"},
	{WDimension_Count,
	 "Dimension_Count"},
	{WDimension_Description,
	 "Dimension_Description"},
	{WDimension_Properties,
	 "Dimension_Properties"},
	{WDistance_Dependent_Distortion,
	 "Distance_Dependent_Distortion"},
	{WDistance_Distortion_Value,
	 "Distance_Distortion_Value"},
	{WDistortion,
	 "Distortion"},
	{WDistortion_Type_Affine,
	 "Distortion_Type_Affine"},
	{WDistortion_Type_Radial_Asymmetrical,
	 "Distortion_Type_Radial_Asymmetrical"},
	{WDistortion_Type_Radial_Symmetrical,
	 "Distortion_Type_Radial_Symmetrical"},
	{WDocumentation,
	 "Documentation"},
	{WEccentricity,
	 "Eccentricity"},
	{WEffective_Aerial_Film_Speed,
	 "Effective_Aerial_Film_Speed"},
	{WElevation_Axis,
	 "Elevation_Axis"},
	{WElevation_Description,
	 "Elevation_Description"},
	{WElevation_Direction,
	 "Elevation_Direction"},
	{WElevation_Extent_Angle,
	 "Elevation_Extent_Angle"},
	{WElevation_Fixed_Angle,
	 "Elevation_Fixed_Angle"},
	{WElevation_Start_Angle,
	 "Elevation_Start_Angle"},
	{WElevation_Step_Angle,
	 "Elevation_Step_Angle"},
	{WElevation_Sweep,
	 "Elevation_Sweep"},
	{WElevation_Zero,
	 "Elevation_Zero"},
	{WEphemeris,
	 "Ephemeris"},
	{WExposure_Time,
	 "Exposure_Time"},
	{WExterior_Orientation_Accuracy,
	 "Exterior_Orientation_Accuracy"},
	{WFiducial,
	 "Fiducial"},
	{WFiducial_Center,
	 "Fiducial_Center"},
	{WFilm_Type,
	 "Film_Type"},
	{WFilter_on_Camera,
	 "Filter_on_Camera"},
	{WFilter_on_Camera_Indicator,
	 "Filter_on_Camera_Indicator"},
	{WFilter_Type,
	 "Filter_Type"},
	{WFlight_Protocol,
	 "Flight_Protocol"},
	{WFlying_Height,
	 "Flying_Height"},
	{WForward_Motion_Compensation,
	 "Forward_Motion_Compensation"},
	{WFrame_Area,
	 "Frame_Area"},
	{WFrame_Area_Units,
	 "Frame_Area_Units"},
	{WFrame_Area_Value,
	 "Frame_Area_Value"},
	{WFrame_Camera,
	 "Frame_Camera"},
	{WFrame_Geometric_Properties,
	 "Frame_Geometric_Properties"},
	{WFrame_Hardware,
	 "Frame_Hardware"},
	{WFrame_Operation,
	 "Frame_Operation"},
	{WFrame_Optics,
	 "Frame_Optics"},
	{WFrame_Radiometric_Calibration,
	 "Frame_Radiometric_Calibration"},
	{WFrame_Radiometric_Properties,
	 "Frame_Radiometric_Properties"},
	{WFrame_Spectral_Information,
	 "Frame_Spectral_Information"},
	{WFrame_Spectral_Properties,
	 "Frame_Spectral_Properties"},
	{WGeorectified_Raster,
	 "Georectified_Raster"},
	{WGeoreferenceable_Raster,
	 "Georeferenceable_Raster"},
	{WGeoreferencing_Description,
	 "Georeferencing_Description"},
	{WGeoreferencing_Information,
	 "Georeferencing_Information"},
	{WGPS_Information_System_Availability,
	 "GPS_Information_System_Availability"},
	{WGrid_First_Element,
	 "Grid_First_Element"},
	{WGrid_First_Element_Map_X_Coordinate,
	 "Grid_First_Element_Map_X_Coordinate"},
	{WGrid_First_Element_Map_Y_Coordinate,
	 "Grid_First_Element_Map_Y_Coordinate"},
	{WGrid_Orientation,
	 "Grid_Orientation"},
	{WGround_Control_Point_Description,
	 "Ground_Control_Point_Description"},
	{WGround_Control_Point_Information,
	 "Ground_Control_Point_Information"},
	{WGround_Control_Point_Organization,
	 "Ground_Control_Point_Organization"},
	{WGround_Control_Point_Position,
	 "Ground_Control_Point_Position"},
	{WGround_Shape,
	 "Ground_Shape"},
	{WID_Overlapping_Dataset,
	 "ID_Overlapping_Dataset"},
	{WIFOV_Units,
	 "IFOV_Units"},
	{WIFOV_x_Definition,
	 "IFOV_x_Definition"},
	{WIFOV_x_Value,
	 "IFOV_x_Value"},
	{WIFOV_y_Definition,
	 "IFOV_y_Definition"},
	{WIFOV_y_Value,
	 "IFOV_y_Value"},
	{WImage_Quality,
	 "Image_Quality"},
	{WImage_Size,
	 "Image_Size"},
	{WImage_Size_x_Value,
	 "Image_Size_x_Value"},
	{WImage_Size_y_Value,
	 "Image_Size_y_Value"},
	{WInclination,
	 "Inclination"},
	{WIndividual_Band_Identification,
	 "Individual_Band_Identification"},
	{WInput_Dataset_Identifier,
	 "Input_Dataset_Identifier"},
	{WInput_Description,
	 "Input_Description"},
	{WInput_Level,
	 "Input_Level"},
	{WInput_Reference,
	 "Input_Reference"},
	{WINS_Reading_Availability,
	 "INS_Reading_Availability"},
	{WInstantaneous_Field_of_View,
	 "Instantaneous_Field_of_View"},
	{WInstitution_of_Last_Calibration,
	 "Institution_of_Last_Calibration"},
	{WInstrument_Description,
	 "Instrument_Description"},
	{WInstrument_Full_Name,
	 "Instrument_Full_Name"},
	{WInstrument_Information,
	 "Instrument_Information"},
	{WInstrument_Properties_Description,
	 "Instrument_Properties_Description"},
	{WInstrument_Reference,
	 "Instrument_Reference"},
	{WInstrument_Short_Name,
	 "Instrument_Short_Name"},
	{WInstrument_Specific_Georeferencing,
	 "Instrument_Specific_Georeferencing"},
	{WInstrument_Type,
	 "Instrument_Type"},
	{WKeplerian_Orbit,
	 "Keplerian_Orbit"},
	{WLast_Calibration,
	 "Last_Calibration"},
	{WLayer_Name,
	 "Layer_Name"},
	{WLens,
	 "Lens"},
	{WLens_Identifier,
	 "Lens_Identifier"},
	{WLens_Type,
	 "Lens_Type"},
	{WLight_Drop,
	 "Light_Drop"},
	{WLocation_Information,
	 "Location_Information"},
	{WMagazine_Identifier,
	 "Magazine_Identifier"},
	{WMaximum_Wavelength,
	 "Maximum_Wavelength"},
	{WMethod_of_Last_Calibration,
	 "Method_of_Last_Calibration"},
	{WMinimum_Wavelength,
	 "Minimum_Wavelength"},
	{WMission_Completion,
	 "Mission_Completion"},
	{WMission_Description,
	 "Mission_Description"},
	{WMission_History,
	 "Mission_History"},
	{WMission_Information,
	 "Mission_Information"},
	{WMission_Name,
	 "Mission_Name"},
	{WMission_Significant_Event,
	 "Mission_Significant_Event"},
	{WMission_Start_Date,
	 "Mission_Start_Date"},
	{WMultiple_Image_Alignment,
	 "Multiple_Image_Alignment"},
	{WNadir,
	 "Nadir"},
	{WNadir_Latitude,
	 "Nadir_Latitude"},
	{WNadir_Longitude,
	 "Nadir_Longitude"},
	{WName_of_Dimension,
	 "Name_of_Dimension"},
	{WNominal_Geostationary_Position,
	 "Nominal_Geostationary_Position"},
	{WNominal_Spatial_Resolution,
	 "Nominal_Spatial_Resolution"},
	{WNon_Polynomial_Scaling,
	 "Non_Polynomial_Scaling"},
	{WNumber_of_Angle_Distortion_Values,
	 "Number_of_Angle_Distortion_Values"},
	{WNumber_of_Bands,
	 "Number_of_Bands"},
	{WNumber_of_Cross_Track_Samples,
	 "Number_of_Cross_Track_Samples"},
	{WNumber_of_Data_Dimensions,
	 "Number_of_Data_Dimensions"},
	{WNumber_of_Denominator_Terms,
	 "Number_of_Denominator_Terms"},
	{WNumber_of_Distance_Distortion_Values,
	 "Number_of_Distance_Distortion_Values"},
	{WNumber_of_Elevation_Samples,
	 "Number_of_Elevation_Samples"},
	{WNumber_of_Numerator_Terms,
	 "Number_of_Numerator_Terms"},
	{WNumber_of_Points,
	 "Number_of_Points"},
	{WNumber_of_Profile_Samples,
	 "Number_of_Profile_Samples"},
	{WNumber_of_Resolution_Values,
	 "Number_of_Resolution_Values"},
	{WNumber_of_Thematic_Layers,
	 "Number_of_Thematic_Layers"},
	{WNumber_of_Wavelength_Bands,
	 "Number_of_Wavelength_Bands"},
	{WOperational_Mode,
	 "Operational_Mode"},
	{WOrbit_Angle_Units,
	 "Orbit_Angle_Units"},
	{WOrbit_Period,
	 "Orbit_Period"},
	{WOrbit_Period_Units,
	 "Orbit_Period_Units"},
	{WOrbit_Period_Value,
	 "Orbit_Period_Value"},
	{WOther_Collector_Description,
	 "Other_Collector_Description"},
	{WOther_Georeferencing_Description,
	 "Other_Georeferencing_Description"},
	{WPath,
	 "Path"},
	{WPeak_Wavelength,
	 "Peak_Wavelength"},
	{WPercentage_of_Overlap,
	 "Percentage_of_Overlap"},
	{WPerigee_Passage_Time,
	 "Perigee_Passage_Time"},
	{WPhotographic_Resolving_Power,
	 "Photographic_Resolving_Power"},
	{WPitch,
	 "Pitch"},
	{WPitch_Accuracy,
	 "Pitch_Accuracy"},
	{WPixel_Cross_Track_Size,
	 "Pixel_Cross_Track_Size"},
	{WPixel_Description,
	 "Pixel_Description"},
	{WPixel_Elevation_Size,
	 "Pixel_Elevation_Size"},
	{WPixel_Height_Above_Ellipsoid,
	 "Pixel_Height_Above_Ellipsoid"},
	{WPixel_Point_Spread_Function,
	 "Pixel_Point_Spread_Function"},
	{WPixel_Profile_Size,
	 "Pixel_Profile_Size"},
	{WPixel_Resolution,
	 "Pixel_Resolution"},
	{WPlatform_and_Instrument_Identification,
	 "Platform_and_Instrument_Identification"},
	{WPlatform_and_Mission_Information,
	 "Platform_and_Mission_Information"},
	{WPlatform_Description,
	 "Platform_Description"},
	{WPlatform_Full_Name,
	 "Platform_Full_Name"},
	{WPlatform_Information,
	 "Platform_Information"},
	{WPlatform_Nominal_Altitude,
	 "Platform_Nominal_Altitude"},
	{WPlatform_Nominal_Altitude_Units,
	 "Platform_Nominal_Altitude_Units"},
	{WPlatform_Nominal_Altitude_Value,
	 "Platform_Nominal_Altitude_Value"},
	{WPlatform_Nominal_Longitude,
	 "Platform_Nominal_Longitude"},
	{WPlatform_Orbit,
	 "Platform_Orbit"},
	{WPlatform_Serial_Identifier,
	 "Platform_Serial_Identifier"},
	{WPlatform_Short_Name,
	 "Platform_Short_Name"},
	{WPlatform_Sponsor,
	 "Platform_Sponsor"},
	{WPlatform_Start_Date,
	 "Platform_Start_Date"},
	{WPoint_Position_In_Pixel,
	 "Point_Position_In_Pixel"},
	{WPolarization_Characteristics,
	 "Polarization_Characteristics"},
	{WPolynomial_Coefficient,
	 "Polynomial_Coefficient"},
	{WPolynomial_Denominator,
	 "Polynomial_Denominator"},
	{WPolynomial_Function,
	 "Polynomial_Function"},
	{WPolynomial_Numerator,
	 "Polynomial_Numerator"},
	{WPositional_Information,
	 "Positional_Information"},
	{WPrincipal_Point_of_Autocollimation,
	 "Principal_Point_of_Autocollimation"},
	{WPrincipal_Point_of_Symmetry,
	 "Principal_Point_of_Symmetry"},
	{WProcessing_Change_History,
	 "Processing_Change_History"},
	{WProcessing_Documentation,
	 "Processing_Documentation"},
	{WProcessing_Environment,
	 "Processing_Environment"},
	{WProcessing_Identifiers,
	 "Processing_Identifiers"},
	{WProcessing_Information,
	 "Processing_Information"},
	{WProcessing_Input_Dataset,
	 "Processing_Input_Dataset"},
	{WProcessing_Level,
	 "Processing_Level"},
	{WProcessing_Level_Authority,
	 "Processing_Level_Authority"},
	{WProcessing_Level_Identifier,
	 "Processing_Level_Identifier"},
	{WProcessing_Procedure,
	 "Processing_Procedure"},
	{WProcessing_Procedure_Description,
	 "Processing_Procedure_Description"},
	{WProcessing_Run_History,
	 "Processing_Run_History"},
	{WProcessing_Software,
	 "Processing_Software"},
	{WProcessing_Software_Description,
	 "Processing_Software_Description"},
	{WProcessing_Software_Reference,
	 "Processing_Software_Reference"},
	{WProfile_Description,
	 "Profile_Description"},
	{WProfile_Extent,
	 "Profile_Extent"},
	{WProfile_Fixed,
	 "Profile_Fixed"},
	{WProfile_Properties,
	 "Profile_Properties"},
	{WProfile_Sounding,
	 "Profile_Sounding"},
	{WProfile_Start,
	 "Profile_Start"},
	{WProfile_Step,
	 "Profile_Step"},
	{WProfiling_Direction,
	 "Profiling_Direction"},
	{WProjection_Center_X_Position,
	 "Projection_Center_X_Position"},
	{WProjection_Center_Y_Position,
	 "Projection_Center_Y_Position"},
	{WProjection_Center_Z_Position,
	 "Projection_Center_Z_Position"},
	{WQuality_of_Autocollimation_Principal_Point_x_Value,
	 "Quality_of_Autocollimation_Principal_Point_x_Value"},
	{WQuality_of_Autocollimation_Principal_Point_y_Value,
	 "Quality_of_Autocollimation_Principal_Point_y_Value"},
	{WQuality_of_Focal_Length,
	 "Quality_of_Focal_Length"},
	{WQuality_of_Principal_Point_of_Autocollimation,
	 "Quality_of_Principal_Point_of_Autocollimation"},
	{WQuality_of_Principal_Point_of_Symmetry,
	 "Quality_of_Principal_Point_of_Symmetry"},
	{WQuality_of_Symmetry_Principal_Point_x_Value,
	 "Quality_of_Symmetry_Principal_Point_x_Value"},
	{WQuality_of_Symmetry_Principal_Point_y_Value,
	 "Quality_of_Symmetry_Principal_Point_y_Value"},
	{WRadial_Asymmetrical_Coefficient_B1,
	 "Radial_Asymmetrical_Coefficient_B1"},
	{WRadial_Asymmetrical_Coefficient_B2,
	 "Radial_Asymmetrical_Coefficient_B2"},
	{WRadial_Symmetrical_Angle_Interval,
	 "Radial_Symmetrical_Angle_Interval"},
	{WRadial_Symmetrical_Distance_Interval,
	 "Radial_Symmetrical_Distance_Interval"},
	{WRadial_Symmetrical_Distortion_Polynomial,
	 "Radial_Symmetrical_Distortion_Polynomial"},
	{WReceiver_Polarization,
	 "Receiver_Polarization"},
	{WReferenced_Coordinate_System,
	 "Referenced_Coordinate_System"},
	{WReferencing_Polynomial,
	 "Referencing_Polynomial"},
	{WRelative_Aperture,
	 "Relative_Aperture"},
	{WRelative_Azimuth,
	 "Relative_Azimuth"},
	{WReseau,
	 "Reseau"},
	{WResolution_Value_Set,
	 "Resolution_Value_Set"},
	{WResolving_Angle,
	 "Resolving_Angle"},
	{WResolving_Value_Radial,
	 "Resolving_Value_Radial"},
	{WResolving_Value_Tangential,
	 "Resolving_Value_Tangential"},
	{WRight_Ascension_of_Ascending_Node,
	 "Right_Ascension_of_Ascending_Node"},
	{WRoll,
	 "Roll"},
	{WRoll_Accuracy,
	 "Roll_Accuracy"},
	{WRotation_Description,
	 "Rotation_Description"},
	{WRotation_Sequence,
	 "Rotation_Sequence"},
	{WRow,
	 "Row"},
	{WRow_Delta_X,
	 "Row_Delta_X"},
	{WRow_Delta_Y,
	 "Row_Delta_Y"},
	{WSample_Angle_Units,
	 "Sample_Angle_Units"},
	{WSample_Description_Units,
	 "Sample_Description_Units"},
	{WSample_Profile_Units,
	 "Sample_Profile_Units"},
	{WSample_Properties,
	 "Sample_Properties"},
	{WSatellite_Local_Azimuth_Angle,
	 "Satellite/Local_Azimuth_Angle"},
	{WSatellite_Local_Zenith_Angle,
	 "Satellite/Local_Zenith_Angle"},
	{WScan,
	 "Scan"},
	{WScan_Angle_Units,
	 "Scan_Angle_Units"},
	{WScan_Cross_Track_Properties,
	 "Scan_Cross_Track_Properties"},
	{WScan_Distance_Units,
	 "Scan_Distance_Units"},
	{WScan_Duration,
	 "Scan_Duration"},
	{WScan_Elevation_Properties,
	 "Scan_Elevation_Properties"},
	{WScan_Geometric_Properties,
	 "Scan_Geometric_Properties"},
	{WScan_Radiometric_Properties,
	 "Scan_Radiometric_Properties"},
	{WScan_Repeat_Time,
	 "Scan_Repeat_Time"},
	{WScan_Spectral_Properties,
	 "Scan_Spectral_Properties"},
	{WScan_Start_Time,
	 "Scan_Start_Time"},
	{WScan_Step_Time,
	 "Scan_Step_Time"},
	{WScan_Time_Units,
	 "Scan_Time_Units"},
	{WScan_Timing,
	 "Scan_Timing"},
	{WScience_Paper,
	 "Science_Paper"},
	{WSemimajor_Axis,
	 "Semimajor_Axis"},
	{WSender_Polarization,
	 "Sender_Polarization"},
	{WSensor_Element_Location,
	 "Sensor_Element_Location"},
	{WSensor_Grid,
	 "Sensor_Grid"},
	{WSensor_Orientation,
	 "Sensor_Orientation"},
	{WSensor_System,
	 "Sensor_System"},
	{WSolar_Azimuth_Angle,
	 "Solar_Azimuth_Angle"},
	{WSolar_Zenith_Angle,
	 "Solar_Zenith_Angle"},
	{WSource_of_Element_Locations,
	 "Source_of_Element_Locations"},
	{WSpatial_Resolution_Units,
	 "Spatial_Resolution_Units"},
	{WSpatial_Resolution_Value,
	 "Spatial_Resolution_Value"},
	{WSpectral_Information,
	 "Spectral_Information"},
	{WSpectral_Limit,
	 "Spectral_Limit"},
	{WStabilized_Mount,
	 "Stabilized_Mount"},
	{WStorage_Order,
	 "Storage_Order"},
	{WSwath_Track_Information,
	 "Swath_Track_Information"},
	{WThematic_Layer_Identification,
	 "Thematic_Layer_Identification"},
	{WTranslation_Description,
	 "Translation_Description"},
	{WType_of_Overlap,
	 "Type_of_Overlap"},
	{WUnreferenced_Coordinate_System,
	 "Unreferenced_Coordinate_System"},
	{WUsers_Guide,
	 "User's_Guide"},
	{WWavelength_Band_Properties,
	 "Wavelength_Band_Properties"},
	{WWavelength_Region,
	 "Wavelength_Region"},
	{WWavelength_Units,
	 "Wavelength_Units"},
	{WWorldwide_Reference_System,
	 "Worldwide_Reference_System"},
	{WX_Axis_Definition,
	 "X_Axis_Definition"},
	{WX_Position_Accuracy,
	 "X_Position_Accuracy"},
	{WY_Axis_Definition,
	 "Y_Axis_Definition"},
	{WY_Position_Accuracy,
	 "Y_Position_Accuracy"},
	{WYaw,
	 "Yaw"},
	{WYaw_Accuracy,
	 "Yaw_Accuracy"},
	{WZ_Axis_Definition,
	 "Z_Axis_Definition"},
	{WZ_Position_Accuracy,
	 "Z_Position_Accuracy"},

#ifdef USE_ESRI_EXTENSIONS

	/* ESRI extensions */

	{WAltitude_Units,
	 "Altitude_Units"},
	{WAncillary_Role,
	 "Ancillary_Role"},
	{WAncillary_Role_Attribute,
	 "Ancillary_Role_Attribute"},
	{WAttribute_Alias,
	 "Attribute_Alias"},
	{WAttribute_Defined_Domain,
	 "Attribute_Defined_Domain"},
	{WAttribute_Indexed,
	 "Attribute_Indexed"},
	{WAttribute_Number_of_Decimals,
	 "Attribute_Number_of_Decimals"},
	{WAttribute_Output_Width,
	 "Attribute_Output_Width"},
	{WAttribute_Precision,
	 "Attribute_Precision"},
	{WAttribute_Scale,
	 "Attribute_Scale"},
	{WAttribute_Type,
	 "Attribute_Type"},
	{WAttribute_Width,
	 "Attribute_Width"},
	{WAttributed_Relationship,
	 "Attributed_Relationship"},
	{WAvailable_Junction_Feature_Class,
	 "Available_Junction_Feature_Class"},
	{WAvailable_Junction_Subtype,
	 "Available_Junction_Subtype"},
	{WAvailable_Junctions,
	 "Available_Junctions"},
	{WBackground_Nodata_Value,
	 "Background_Nodata_Value"},
	{WBehrmann,
	 "Behrmann"},
	{WBinary_Enclosures,
	 "Binary_Enclosures"},
	{WBits_Per_Pixel,
	 "Bits_Per_Pixel"},
	{WBonne,
	 "Bonne"},
	{WBottom_Bounding_Coordinate,
	 "Bottom_Bounding_Coordinate"},
	{WBrowse_Graphic_Embedded,
	 "Browse_Graphic_Embedded"},
	{WCassini,
	 "Cassini"},
	{WCell_Size_X_Direction,
	 "Cell_Size_X_Direction"},
	{WCell_Size_X_Units,
	 "Cell_Size_X_Units"},
	{WCell_Size_Y_Direction,
	 "Cell_Size_Y_Direction"},
	{WCell_Size_Y_Units,
	 "Cell_Size_Y_Units"},
	{WComposite_Relationship,
	 "Composite_Relationship"},
	{WCompression_Type,
	 "Compression_Type"},
	{WConnectivity_Rule,
	 "Connectivity_Rule"},
	{WCoordinate_System_Name,
	 "Coordinate_System_Name"},
	{WCreation_Date,
	 "Creation_Date"},
	{WCreation_Time,
	 "Creation_Time"},
	{WData,
	 "Data"},
	{WData_Frame_Count,
	 "Data_Frame_Count"},
	{WData_Frame_Name,
	 "Data_Frame_Name"},
	{WData_Frames,
	 "Data_Frames"},
	{WDatabase_Name,
	 "Database_Name"},
	{WDataset_Size,
	 "Dataset_Size"},
	{WDefault_Junction_Feature_Class,
	 "Default_Junction_Feature_Class"},
	{WDefault_Junction_Subtype,
	 "Default_Junction_Subtype"},
	{WDescription_of_Association,
	 "Description_of_Association"},
	{WDescription_of_Enclosure,
	 "Description_of_Enclosure"},
	{WDescription_of_Relationship,
	 "Description_of_Relationship"},
	{WDestination_Foreign_Key,
	 "Destination_Foreign_Key"},
	{WDestination_Name,
	 "Destination_Name"},
	{WDestination_Primary_Key,
	 "Destination_Primary_Key"},
	{WDomain_Attribute_Type,
	 "Domain_Attribute_Type"},
	{WDomain_Description,
	 "Domain_Description"},
	{WDomain_Name,
	 "Domain_Name"},
	{WDomain_Owner,
	 "Domain_Owner"},
	{WDomain_Type,
	 "Domain_Type"},
	{WEckert_I,
	 "Eckert_I"},
	{WEckert_II,
	 "Eckert_II"},
	{WEckert_III,
	 "Eckert_III"},
	{WEckert_IV,
	 "Eckert_IV"},
	{WEckert_V,
	 "Eckert_V"},
	{WEckert_VI,
	 "Eckert_VI"},
	{WEdge_Feature_Class,
	 "Edge_Feature_Class"},
	{WEdge_Maximum_Cardinality,
	 "Edge_Maximum_Cardinality"},
	{WEdge_Minimum_Cardinality,
	 "Edge_Minimum_Cardinality"},
	{WEdge_Subtype,
	 "Edge_Subtype"},
	{WEnabled_Attribute,
	 "Enabled_Attribute"},
	{WEnclosure,
	 "Enclosure"},
	{WEntity_Type_Count,
	 "Entity_Type_Count"},
	{WEntity_Type_Type,
	 "Entity_Type_Type"},
	{WESRI_Feature_Count,
	 "ESRI_Feature_Count"},
	{WESRI_Feature_Geometry,
	 "ESRI_Feature_Geometry"},
	{WESRI_Feature_Type,
	 "ESRI_Feature_Type"},
	{WESRI_Information,
	 "ESRI_Information"},
	{WESRI_Terms_Description,
	 "ESRI_Terms_Description"},
	{WESRI_Topology,
	 "ESRI_Topology"},
	{WFeature_Description,
	 "Feature_Description"},
	{WFile_or_Table_Name,
	 "File_or_Table_Name"},
	{WFrom_Edge_Feature_Class,
	 "From_Edge_Feature_Class"},
	{WFrom_Edge_Subtype,
	 "From_Edge_Subtype"},
	{WGall_Stereographic,
	 "Gall_Stereographic"},
	{WGeocoding_Information,
	 "Geocoding_Information"},
	{WGeographic_Coordinate_System_Name,
	 "Geographic_Coordinate_System_Name"},
	{WGeometric_Network_Information,
	 "Geometric_Network_Information"},
	{WImage_Colormap,
	 "Image_Colormap"},
	{WImage_Format,
	 "Image_Format"},
	{WImage_Type,
	 "Image_Type"},
	{WInstance_Name,
	 "Instance_Name"},
	{WJunction_Feature_Class,
	 "Junction_Feature_Class"},
	{WJunction_Maximum_Cardinality,
	 "Junction_Maximum_Cardinality"},
	{WJunction_Minimum_Cardinality,
	 "Junction_Minimum_Cardinality"},
	{WJunction_Subtype,
	 "Junction_Subtype"},
	{WLanguage_of_Dataset,
	 "Language_of_Dataset"},
	{WLanguage_of_Metadata,
	 "Language_of_Metadata"},
	{WLeft_Bounding_Coordinate,
	 "Left_Bounding_Coordinate"},
	{WLinear_Referencing,
	 "Linear_Referencing"},
	{WLocal_Bounding_Coordinates,
	 "Local_Bounding_Coordinates"},
	{WLoximuthal,
	 "Loximuthal"},
	{WMaximum_Altitude,
	 "Maximum_Altitude"},
	{WMerge_Rule,
	 "Merge_Rule"},
	{WMetadata_ID,
	 "Metadata_ID"},
	{WMinimum_Altitude,
	 "Minimum_Altitude"},
	{WModification_Date,
	 "Modification_Date"},
	{WModification_Time,
	 "Modification_Time"},
	{WMollweide,
	 "Mollweide"},
	{WNative_Dataset_Format,
	 "Native_Dataset_Format"},
	{WNetwork_Element,
	 "Network_Element"},
	{WNetwork_Role,
	 "Network_Role"},
	{WNetwork_Type,
	 "Network_Type"},
	{WNotification_Direction,
	 "Notification_Direction"},
	{WNumber_of_Raster_Bands,
	 "Number_of_Raster_Bands"},
	{WOrigin_Foreign_Key,
	 "Origin_Foreign_Key"},
	{WOrigin_Name,
	 "Origin_Name"},
	{WOrigin_Primary_Key,
	 "Origin_Primary_Key"},
	{WProcess_Software_and_Version,
	 "Process_Software_and_Version"},
	{WProjected_Coordinate_System_Name,
	 "Projected_Coordinate_System_Name"},
	{WPyramid_Layers,
	 "Pyramid_Layers"},
	{WQuartic_Authalic,
	 "Quartic_Authalic"},
	{WRange_Domain_Mean,
	 "Range_Domain_Mean"},
	{WRange_Domain_Standard_Deviation,
	 "Range_Domain_Standard_Deviation"},
	{WRaster_Display_Type,
	 "Raster_Display_Type"},
	{WRaster_Origin,
	 "Raster_Origin"},
	{WRelationship_Backward_Label,
	 "Relationship_Backward_Label"},
	{WRelationship_Cardinality,
	 "Relationship_Cardinality"},
	{WRelationship_Forward_Label,
	 "Relationship_Forward_Label"},
	{WRelationship_Information,
	 "Relationship_Information"},
	{WRight_Bounding_Coordinate,
	 "Right_Bounding_Coordinate"},
	{WRule_Category,
	 "Rule_Category"},
	{WRule_Help,
	 "Rule_Help"},
	{WRule_Type,
	 "Rule_Type"},
	{WSDE_Connection_Information,
	 "SDE_Connection_Information"},
	{WServer_Name,
	 "Server_Name"},
	{WSpatial_Index,
	 "Spatial_Index"},
	{WSplit_Rule,
	 "Split_Rule"},
	{WSubtype_Attribute,
	 "Subtype_Attribute"},
	{WSubtype_Attribute_Name,
	 "Subtype_Attribute_Name"},
	{WSubtype_Code,
	 "Subtype_Code"},
	{WSubtype_Default_Value,
	 "Subtype_Default_Value"},
	{WSubtype_Information,
	 "Subtype_Information"},
	{WSubtype_Name,
	 "Subtype_Name"},
	{WSynchronization_Date,
	 "Synchronization_Date"},
	{WSynchronization_Time,
	 "Synchronization_Time"},
	{WSynchronize_Automatically,
	 "Synchronize_Automatically"},
	{WSynchronize_Once,
	 "Synchronize_Once"},
	{WThumbnail,
	 "Thumbnail"},
	{WTo_Edge_Feature_Class,
	 "To_Edge_Feature_Class"},
	{WTo_Edge_Subtype,
	 "To_Edge_Subtype"},
	{WTop_Bounding_Coordinate,
	 "Top_Bounding_Coordinate"},
	{WUser_Name,
	 "User_Name"},
	{WVersion_Name,
	 "Version_Name"},
	{WWinkel_I,
	 "Winkel_I"},
	{WWinkel_II,
	 "Winkel_II"},
	{WGeographic_Box,
	 "Geographic_Box"},

#endif

	{Wunknown, NULL}
	};

/*----------------------------------------------------------------------*\
 | French
 |
 | from John Cree
\*----------------------------------------------------------------------*/

static struct keyword_list fr_std[] = {
	{WAbscissa_Resolution,
	 "Rsolution_en_abscisse"},
	{WAbstract,
	 "Rsum"},
	{WAccess_Constraints,
	 "Contraintes_d'accs"},
	{WAccess_Instructions,
	 "Instructions_d'accs"},
	{WAddress,
	 "Adresse"},
	{WAddress_Type,
	 "Type_d'adresse"},
	{WAlbers_Conical_Equal_Area,
	 "Conique_quivalente_d'Albers"},
	{WAltitude_Datum_Name,
	 "Nom_du_plan_de_rfrence_altimtrique"},
	{WAltitude_Distance_Units,
	 "Units_de_distance_de_l'altitude"},
	{WAltitude_Encoding_Method,
	 "Mthode_de_codage_de_l'altitude"},
	{WAltitude_Resolution,
	 "Rsolution_en_altitude"},
	{WAltitude_System_Definition,
	 "Dfinition_du_systme_altimtrique"},
	{WARC_Coordinate_System,
	 "Systme_de_coordonnes_ARC"},
	{WARC_System_Zone_Identifier,
	 "Identifiant_de_zone_du_systme_ARC"},
	{WAttribute,
	 "Attribut"},
	{WAttribute_Accuracy,
	 "Prcision_d'attribut"},
	{WAttribute_Accuracy_Explanation,
	 "Explication_de_la_prcision_d'attribut"},
	{WAttribute_Accuracy_Report,
	 "Rapport_sur_la_prcision_d'attribut"},
	{WAttribute_Accuracy_Value,
	 "Valeur_de_la_prcision_d'attribut"},
	{WAttribute_Definition,
	 "Dfinition_de_l'attribut"},
	{WAttribute_Definition_Source,
	 "Source_de_dfinition_de_l'attribut"},
	{WAttribute_Domain_Values,
	 "Valeurs_du_domaine_de_l'attribut"},
	{WAttribute_Label,
	 "Annotation_d'attribut"},
	{WAttribute_Measurement_Frequency,
	 "Frquence_de_mesure_de_l'attribut"},
	{WAttribute_Measurement_Resolution,
	 "Rsolution_de_la_mesure_d'attribut"},
	{WAttribute_Units_of_Measure,
	 "Units_de_mesure_de_l'attribut"},
	{WAttribute_Value_Accuracy,
	 "Prcision_des_valeurs_d'attribut"},
	{WAttribute_Value_Accuracy_Explanation,
	 "Explication_de_la_prcision_des_valeurs_d'attribut"},
	{WAttribute_Value_Accuracy_Information,
	 "Information_sur_la_prcision_des_valeurs_d'attribut"},
	{WAvailable_Time_Period,
	 "Priode_de_temps_de_disponibilit"},
	{WAzimuthal_Angle,
	 "Angle_azimutal"},
	{WAzimuthal_Equidistant,
	 "Azimutale_quidistante"},
	{WAzimuth_Measure_Point_Longitude,
	 "Longitude_du_point_de_mesure_de_l'azimut"},
	{WBearing_Reference_Direction,
	 "Direction_de_rfrence_pour_les_angles_de_relvement"},
	{WBearing_Reference_Meridian,
	 "Mridien_de_rfrence_pour_les_angles_de_relvement"},
	{WBearing_Resolution,
	 "Rsolution_en_angle_de_relvement"},
	{WBearing_Units,
	 "Units_d'angle_de_relvement"},
	{WBeginning_Date,
	 "Date_de_dbut"},
	{WBeginning_Date_of_Attribute_Values,
	 "Date_de_dbut_des_valeurs_d'attribut"},
	{WBeginning_Time,
	 "Temps_de_dbut"},
	{WBounding_Coordinates,
	 "Coordonnes_de_dlimitation"},
	{WBrowse_Graphic,
	 "Aperu_graphique"},
	{WBrowse_Graphic_File_Description,
	 "Description_de_l'aperu_graphique"},
	{WBrowse_Graphic_File_Name,
	 "Nom_d'aperu_graphique"},
	{WBrowse_Graphic_File_Type,
	 "Type_d'aperu_graphique"},
	{WCalendar_Date,
	 "Date_du_calendrier"},
	{WCitation,
	 "Citation"},
	{WCitation_Information,
	 "Information_de_citation"},
	{WCity,
	 "Ville"},
	{WCloud_Cover,
	 "Couverture_nuageuse"},
	{WCodeset_Domain,
	 "Domaine_de_l'ensemble_de_code"},
	{WCodeset_Name,
	 "Nom_du_domaine_de_l'ensemble_de_code"},
	{WCodeset_Source,
	 "Source_de_l'ensemble_de_code"},
	{WColumn_Count,
	 "Dnombrement_des_colonnes"},
	{WCompatibility_Information,
	 "Information_sur_la_compatibilit"},
	{WCompleteness_Report,
	 "Rapport_de_compltude"},
	{WCompression_Support,
	 "Support_de_compression"},
	{WComputer_Contact_Information,
	 "Information_sur_l'ordinateur_du_contact"},
	{WContact_Address,
	 "Adresse_du_contact"},
	{WContact_Electronic_Mail_Address,
	 "Courriel_du_contact"},
	{WContact_Facsimile_Telephone,
	 "Tlcopieur_du_contact"},
	{WContact_Information,
	 "Information_sur_le_contact"},
	{WContact_Instructions,
	 "Instructions_sur_le_contact"},
	{WContact_Organization,
	 "Organisation_contact"},
	{WContact_Organization_Primary,
	 "Organisation_contact_primaire"},
	{WContact_Person,
	 "Personne_contact"},
	{WContact_Person_Primary,
	 "Personne_contact_primaire"},
	{WContact_Position,
	 "Position_du_contact"},
	{WContact_TDD_TTY_Telephone,
	 "Numro_de_tlphone_ATME_du_contact"},
	{WContact_Voice_Telephone,
	 "Numro_de_tlphone_du_contact"},
	{WCoordinate_Representation,
	 "Reprsentation_par_coordonnes"},
	{WCountry,
	 "Pays"},
	{WCross_Reference,
	 "Renvoi"},
	{WCurrentness_Reference,
	 "Rfrence__l'actualit"},
	{WCustom_Order_Process,
	 "Processus_de_commande_spciale"},
	{WData_Quality_Information,
	 "Information_sur_la_qualit_des_donnes"},
	{WData_Set_Credit,
	 "Crdit_pour_l'ensemble_des_donnes"},
	{WData_Set_G_Polygon,
	 "Polygone-G_de_l'ensemble_de_donnes"},
	{WData_Set_G_Polygon_Exclusion_G_Ring,
	 "Cercle-G_d'exclusion_du_polygone-G_de_l'ensemble_de_donnes"},
	{WData_Set_G_Polygon_Outer_G_Ring,
	 "Cercle-G_extrieur_du_polygone-G_de_l'ensemble_de_donnes"},
	{WDenominator_of_Flattening_Ratio,
	 "Dnominateur_du_rapport_d'aplatissement"},
	{WDepth_Datum_Name,
	 "Nom_du_plan_de_rfrence_des_profondeurs"},
	{WDepth_Distance_Units,
	 "Units_de_distance_en_profondeur"},
	{WDepth_Encoding_Method,
	 "Mthode_de_codage_de_la_profondeur"},
	{WDepth_Resolution,
	 "Rsolution_en_profondeur"},
	{WDepth_System_Definition,
	 "Dfinition_du_systme_de_la_profondeur"},
	{WDescription,
	 "Description"},
	{WDetailed_Description,
	 "Description_dtaille"},
	{WDialup_File_Name,
	 "Nom_du_fichier_d'accs_commut"},
	{WDialup_Instructions,
	 "Instructions_d'accs_commut"},
	{WDialup_Telephone,
	 "Numro_de_tlphone_d'accs_commut"},
	{WDigital_Form,
	 "Forme_numrique"},
	{WDigital_Transfer_Information,
	 "Information_de_transfert_numrique"},
	{WDigital_Transfer_Option,
	 "Option_de_transfert_numrique"},
	{WDirect_Spatial_Reference_Method,
	 "Mthode_de_rfrence_spatiale_directe"},
	{WDistance_and_Bearing_Representation,
	 "Reprsentation_par_distance_et_par_angle_de_relvement"},
	{WDistance_Resolution,
	 "Rsolution_en_distance"},
	{WDistribution_Information,
	 "Information_de_distribution"},
	{WDistribution_Liability,
	 "Responsabilit_de_distribution"},
	{WDistributor,
	 "Distributeur"},
	{WEast_Bounding_Coordinate,
	 "Coordonne_de_dlimitation_Est"},
	{WEdition,
	 "dition"},
	{WEllipsoid_Name,
	 "Nom_de_l'ellipsode"},
	{WEnding_Date,
	 "Date_de_fin"},
	{WEnding_Date_of_Attribute_Values,
	 "Date_de_fin_des_valeurs_d'attribut"},
	{WEnding_Time,
	 "Temps_de_fin"},
	{WEntity_and_Attribute_Detail_Citation,
	 "Citation_dtaille_sur_l'entit_et_l'attribut"},
	{WEntity_and_Attribute_Information,
	 "Information_sur_l'entit_et_l'attribut"},
	{WEntity_and_Attribute_Overview,
	 "Sommaire_des_entits_et_des_attributs"},
	{WEntity_Type,
	 "Type_d'entit"},
	{WEntity_Type_Definition,
	 "Dfinition_de_type_d'entit"},
	{WEntity_Type_Definition_Source,
	 "Source_de_la_dfinition_de_type_d'entit"},
	{WEntity_Type_Label,
	 "Annotation_du_type_d'entit"},
	{WEnumerated_Domain,
	 "Domaine_d'numrations"},
	{WEnumerated_Domain_Value,
	 "Valeur_du_domaine_d'numrations"},
	{WEnumerated_Domain_Value_Definition,
	 "Dfinition_du_domaine_d'numrations"},
	{WEnumerated_Domain_Value_Definition_Source,
	 "Source_de_dfinition_du_domaine_d'numrations"},
	{WEquidistant_Conic,
	 "Conique_quidistante"},
	{WEquirectangular,
	 "quirectangulaire"},
	{WFalse_Easting,
	 "Abscisse_fictive"},
	{WFalse_Northing,
	 "Ordonne_fictive"},
	{WFees,
	 "Frais"},
	{WFile_Decompression_Technique,
	 "Technique_de_dcompression_des_fichiers"},
	{WFormat_Information_Content,
	 "Contenu_informationnel_du_format"},
	{WFormat_Name,
	 "Nom_du_format"},
	{WFormat_Specification,
	 "Spcification_de_la_version_du_format"},
	{WFormat_Version_Date,
	 "Date_de_version_du_format"},
	{WFormat_Version_Number,
	 "Numro_de_version_de_format"},
	{WG_Ring,
	 "Cercle-G"},
	{WG_Ring_Latitude,
	 "Latitude_du_cercle-G"},
	{WG_Ring_Longitude,
	 "Longitude_du_cercle_-G"},
	{WG_Ring_Point,
	 "Point_du_cercle-G"},
	{WGeneral_Vertical_Near_sided_Perspective,
	 "Perspective_verticale_gnrale_proximale"},
	{WGeodetic_Model,
	 "Modle_godsique"},
	{WGeographic,
	 "Gographique"},
	{WGeographic_Coordinate_Units,
	 "Units_des_coordonnes_gographiques"},
	{WGeospatial_Data_Presentation_Form,
	 "Forme_de_prsentation_de_donnes_gospatiales"},
	{WGnomonic,
	 "Gnomonique"},
	{WGrid_Coordinate_System,
	 "Systme_de_coordonnes_cartsiennes_orthogonales"},
	{WGrid_Coordinate_System_Name,
	 "Nom_du_systme_de_coordonnes_cartsiennes_orthogonales"},
	{WHeight_of_Perspective_Point_Above_Surface,
	 "Hauteur_du_point_de_perspective_au-dessus_de_la_surface"},
	{WHighest_BPS,
	 "Dbit_maximal_en_BPS"},
	{WHorizontal_Coordinate_System_Definition,
	 "Dfinition_du_systme_de_coordonnes_horizontales"},
	{WHorizontal_Datum_Name,
	 "Nom_du_systme_de_rfrence"},
	{WHorizontal_Positional_Accuracy,
	 "Prcision_d'alignement_horizontal"},
	{WHorizontal_Positional_Accuracy_Explanation,
	 "Explication_de_la_prcision_d'alignement_horizontal"},
	{WHorizontal_Positional_Accuracy_Report,
	 "Rapport_sur_la_prcision_d'alignement_horizontal"},
	{WHorizontal_Positional_Accuracy_Value,
	 "Valeur_de_prcision_d'alignement_horizontal"},
	{WHours_of_Service,
	 "Heures_de_service"},
	{WIdentification_Information,
	 "Information_d'identification"},
	{WIndirect_Spatial_Reference,
	 "Rfrence_spatiale_indirecte"},
	{WIssue_Identification,
	 "Identification_du_numro"},
	{WKeywords,
	 "Mots-cls"},
	{WLambert_Azimuthal_Equal_Area,
	 "Azimutale_quivalente_de_Lambert"},
	{WLambert_Conformal_Conic,
	 "Conique_conforme_de_Lambert"},
	{WLandsat_Number,
	 "Numro_du_satellite_Landsat"},
	{WLarger_Work_Citation,
	 "Latitude_du_centre_de_la_projection"},
	{WLatitude_of_Projection_Center,
	 "Latitude_du_centre_de_la_projection"},
	{WLatitude_of_Projection_Origin,
	 "Latitude_de_l'origine_de_la_projection"},
	{WLatitude_Resolution,
	 "Rsolution_en_latitude"},
	{WLineage,
	 "Lignage"},
	{WLocal,
	 "Local_(systme_de_coordonnes)"},
	{WLocal_Description,
	 "Description_du_systme_local_de_coordonnes"},
	{WLocal_Georeference_Information,
	 "Information_gorfrence_du_systme_local"},
	{WLocal_Planar,
	 "Systme_local_de_coordonnes_planes"},
	{WLocal_Planar_Description,
	 "Description_du_systme_local_de_coordonnes_planes"},
	{WLocal_Planar_Georeference_Information,
	 "Information_gorfrence_sur_le_systme_local_de_coordonnes_planes"},
	{WLogical_Consistency_Report,
	 "Rapport_de_cohrence_logique"},
	{WLongitude_of_Central_Meridian,
	 "Longitude_du_mridien_central"},
	{WLongitude_of_Projection_Center,
	 "Longitude_du_centre_de_la_projection"},
	{WLongitude_Resolution,
	 "Rsolution_en_longitude"},
	{WLowest_BPS,
	 "Dbit_minimal_en_BPS"},
	{WMaintenance_and_Update_Frequency,
	 "Frquence_de_mise__jour_et_d'actualisation"},
	{WMap_Projection,
	 "Projection_cartographique"},
	{WMap_Projection_Name,
	 "Nom_de_la_projection_cartographique"},
	{WMap_Projection_Parameters,
	 "Paramtres_de_la_projection_cartographique"},
	{WMercator,
	 "Mercator"},
	{WMetadata,
	 "Mtadonnes"},
	{WMetadata_Access_Constraints,
	 "Contraintes_d'accs_aux_mtadonnes"},
	{WMetadata_Contact,
	 "Contact_pour_les_mtadonnes"},
	{WMetadata_Date,
	 "Date_des_mtadonnes"},
	{WMetadata_Extensions,
	 "Extensions_des_mtadonnes"},
	{WMetadata_Future_Review_Date,
	 "Date_de_la_prochaine_rvision_des_mtadonnes"},
	{WMetadata_Language,
	 "Langue_d'utilisation_pour_les_metadonnees"},
	{WMetadata_Reference_Information,
	 "Information_de_rfrence_sur_les_mtadonnes"},
	{WMetadata_Review_Date,
	 "Date_de_rvision_des_mtadonnes"},
	{WMetadata_Security_Classification,
	 "Classification_de_scurit_des_mtadonnes"},
	{WMetadata_Security_Classification_System,
	 "Systme_de_classification_de_la_scurit_des_mtadonnes"},
	{WMetadata_Security_Handling_Description,
	 "Description_des_consignes_de_scurit_pour_les_mtadonnes"},
	{WMetadata_Security_Information,
	 "Information_sur_la_scurit_des_mtadonnes"},
	{WMetadata_Standard_Name,
	 "Nom_des_normes_utilises_pour_les_mtadonnes"},
	{WMetadata_Standard_Version,
	 "Version_des_normes_des_mtadonnes"},
	{WMetadata_Time_Convention,
	 "Convention_de_temps_pour_les_mtadonnes"},
	{WMetadata_Use_Constraints,
	 "Contraintes_d'utilisation_des_mtadonnes"},
	{WMiller_Cylindrical,
	 "Cylindrique_de_Miller"},
	{WModified_Stereographic_for_Alaska,
	 "Strographique_modifie_pour_l'Alaska"},
	{WMultiple_Dates_Times,
	 "Dates/heures_multiples"},
	{WNative_Data_Set_Environment,
	 "Environnement_natif_de_l'ensemble_de_donnes"},
	{WNetwork_Address,
	 "Adresse_rseau"},
	{WNetwork_Resource_Name,
	 "Nom_de_la_ressource_rseau_(URL)"},
	{WNon_digital_Form,
	 "Forme_non_numrique"},
	{WNorth_Bounding_Coordinate,
	 "Coordonne_de_dlimitation_Nord"},
	{WNumber_DataBits,
	 "Nombre_de_bits_de_donnes"},
	{WNumber_StopBits,
	 "Nombre_de_bits_d'arrt"},
	{WOblique_Line_Azimuth,
	 "Azimut_de_ligne_oblique"},
	{WOblique_Line_Latitude,
	 "Latitude_de_dfinition_de_l'oblique"},
	{WOblique_Line_Longitude,
	 "Longitude_de_dfinition_de_l'oblique"},
	{WOblique_Line_Point,
	 "Point_de_l'oblique"},
	{WOblique_Mercator,
	 "Mercator_oblique"},
	{WOffline_Media,
	 "Mdia_hors_ligne"},
	{WOffline_Option,
	 "Option_hors_ligne"},
	{WOnline_Computer_and_Operating_System,
	 "Ordinateur_en_ligne_et_systme_d'exploitation"},
	{WOnline_Linkage,
	 "Lien_en_direct"},
	{WOnline_Option,
	 "Option_en_ligne"},
	{WOrdering_Instructions,
	 "Instructions_pour_commander"},
	{WOrdinate_Resolution,
	 "Rsolution_en_ordonne"},
	{WOriginator,
	 "Crateur"},
	{WOrthographic,
	 "Orthographique"},
	{WOther_Citation_Details,
	 "Autres_dtails_de_citation"},
	{WOther_Grid_Systems_Definition,
	 "Autre_dfinition_de_systmes_de_quadrillage"},
	{WOther_Projections_Definition,
	 "Dfinition_d'autre_projection"},
	{WOverview_Description,
	 "Description_sommaire"},
	{WParity,
	 "Parit"},
	{WPath_Number,
	 "Numro_de_trajectoire"},
	{WPlace,
	 "Lieu"},
	{WPlace_Keyword,
	 "Mot-cl_de_lieu"},
	{WPlace_Keyword_Thesaurus,
	 "Thsaurus_de_mots-cls_de_lieu"},
	{WPlanar,
	 "Planair"},
	{WPlanar_Coordinate_Encoding_Method,
	 "Mthode_de_codage_des_coordonnes_planes"},
	{WPlanar_Coordinate_Information,
	 "Information_sur_les_coordonnes_planes"},
	{WPlanar_Distance_Units,
	 "Units_de_distance_en_coordonnes_planes"},
	{WPoint_and_Vector_Object_Count,
	 "Dnombrement_d'objets_ponctuels_et_vectoriels"},
	{WPoint_and_Vector_Object_Information,
	 "Information_sur_l'objet_ponctuel_et_vectoriel"},
	{WPoint_of_Contact,
	 "Contact"},
	{WPolar_Stereographic,
	 "Strographique_polaire"},
	{WPolyconic,
	 "Polyconique"},
	{WPositional_Accuracy,
	 "Prcision_d'alignement"},
	{WPostal_Code,
	 "Code_postal"},
	{WProcess_Contact,
	 "Contact_du_processus"},
	{WProcess_Date,
	 "Date_du_processus"},
	{WProcess_Description,
	 "Description_du_processus"},
	{WProcess_Step,
	 "tape_du_processus"},
	{WProcess_Time,
	 "Temps_du_processus"},
	{WProfile_Name,
	 "Nom_du_profil"},
	{WProgress,
	 "Progrs"},
	{WPublication_Date,
	 "Date_de_publication"},
	{WPublication_Information,
	 "Information_de_publication"},
	{WPublication_Place,
	 "Lieu_de_publication"},
	{WPublication_Time,
	 "Temps_de_publication"},
	{WPublisher,
	 "diteur"},
	{WPurpose,
	 "But"},
	{WQuantitative_Attribute_Accuracy_Assessment,
	 "valuation_quantitative_de_la_prcision_d'attribut"},
	{WQuantitative_Horizontal_Positional_Accuracy_Assessment,
	 "valuation_quantitative_de_la_prcision_d'alignement_horizontal"},
	{WQuantitative_Vertical_Positional_Accuracy_Assessment,
	 "valuation_quantitative_de_la_prcision_d'alignement_vertical"},
	{WRange_Domain,
	 "Domaine_des_limites"},
	{WRange_Domain_Maximum,
	 "Maximum_du_domaine_de_limites"},
	{WRange_Domain_Minimum,
	 "Minimum_du_domaine_de_limites"},
	{WRange_of_Dates_Times,
	 "Plusieurs_dates_et_temps"},
	{WRaster_Object_Information,
	 "Information_sur_les_objets_matriciels"},
	{WRaster_Object_Type,
	 "Type_d'objet_matriciel"},
	{WRecording_Capacity,
	 "Capacit_d'enregistrement"},
	{WRecording_Density,
	 "Densit_d'enregistrement"},
	{WRecording_Density_Units,
	 "Units_de_densit_d'enregistrement"},
	{WRecording_Format,
	 "Format_d'enregistrement"},
	{WResource_Description,
	 "Description_de_la_ressource"},
	{WRobinson,
	 "Robinson"},
	{WRow_Count,
	 "Dnombrement_des_ranges"},
	{WScale_Factor_at_Center_Line,
	 "Facteur_d'chelle__la_ligne_centrale"},
	{WScale_Factor_at_Central_Meridian,
	 "Facteur_d'chelle_au_mridien_central"},
	{WScale_Factor_at_Equator,
	 "Facteur_d'chelle__l'quateur"},
	{WScale_Factor_at_Projection_Origin,
	 "Facteur_d'chelle__l'origine_de_la_projection"},
	{WSDTS_Point_and_Vector_Object_Type,
	 "Type_d'objet_vectoriel_et_ponctuel_SDTS"},
	{WSDTS_Terms_Description,
	 "Description_des_termes_SDTS"},
	{WSecurity_Classification,
	 "Scurit_de_la_classification"},
	{WSecurity_Classification_System,
	 "Systme_de_scurit_de_la_classification"},
	{WSecurity_Handling_Description,
	 "Description_des_consignes_de_scurit"},
	{WSecurity_Information,
	 "Information_sur_la_scurit"},
	{WSemi_major_Axis,
	 "Demi-grand_axe"},
	{WSeries_Information,
	 "Information_sur_la_srie"},
	{WSeries_Name,
	 "Nom_de_la_srie"},
	{WSingle_Date_Time,
	 "Date/temps_unique"},
	{WSinusoidal,
	 "Sinusodale"},
	{WSource_Citation,
	 "Citation_de_la_source"},
	{WSource_Citation_Abbreviation,
	 "Citation_abrge_de_la_source"},
	{WSource_Contribution,
	 "Contribution_de_la_source"},
	{WSource_Currentness_Reference,
	 "Rfrence__l'actualit_de_la_source"},
	{WSource_Information,
	 "Information_sur_la_source"},
	{WSource_Produced_Citation_Abbreviation,
	 "Citation_abrge_produite_par_la_source"},
	{WSource_Scale_Denominator,
	 "Dnominateur_de_l'chelle_de_la_source"},
	{WSource_Time_Period_of_Content,
	 "Priode_de_temps_du_contenu_de_la_source"},
	{WSource_Used_Citation_Abbreviation,
	 "Citation_abrge_utilise_par_la_source"},
	{WSouth_Bounding_Coordinate,
	 "Coordonne_de_dlimitation_Sud"},
	{WSpace_Oblique_Mercator_Landsat,
	 "Mercator_oblique_spatiale_(Landsat)"},
	{WSpatial_Data_Organization_Information,
	 "Information_sur_l'organisation_des_donnes_spatiales"},
	{WSpatial_Domain,
	 "Domaine_spatial"},
	{WSpatial_Reference_Information,
	 "Information_de_rfrence_spatiale"},
	{WSPCS_Zone_Identifier,
	 "Identifiant_de_zone_SPCE"},
	{WStandard_Order_Process,
	 "Processus_de_commande_standard"},
	{WStandard_Parallel,
	 "Parallle_standard"},
	{WState_or_Province,
	 "tat_ou_province"},
	{WState_Plane_Coordinate_System,
	 "Systme_de_coordonnes_planes_d'tat_(SCPE)_(State_Plane_Coordinate_System_-_SPCS)"},
	{WStatus,
	 "tat"},
	{WStereographic,
	 "Stereographic"},
	{WStraight_Vertical_Longitude_from_Pole,
	 "Longitude_rectiligne_verticale__partir_du_ple"},
	{WStratum,
	 "Strate"},
	{WStratum_Keyword,
	 "Mot-cl_de_strate"},
	{WStratum_Keyword_Thesaurus,
	 "Thsaurus_de_mots-cls_de_strate"},
	{WSupplemental_Information,
	 "Information_supplmentaire"},
	{WTechnical_Prerequisites,
	 "Exigences_techniques"},
	{WTemporal,
	 "Temporel"},
	{WTemporal_Keyword,
	 "Mot-cl_temporel"},
	{WTemporal_Keyword_Thesaurus,
	 "Thsaurus_de_mots-cls_temporels"},
	{WTheme,
	 "Thme"},
	{WTheme_Keyword,
	 "Mot-cl_du_thme"},
	{WTheme_Keyword_Thesaurus,
	 "Thsaurus_de_mots-cls_du_thme"},
	{WTime_of_Day,
	 "Temps_du_jour"},
	{WTime_Period_Information,
	 "Information_sur_la_priode_de_temps"},
	{WTime_Period_of_Content,
	 "Priode_de_temps_du_contenu"},
	{WTitle,
	 "Titre"},
	{WTransfer_Size,
	 "Taille_du_transfert"},
	{WTransverse_Mercator,
	 "Mercator_transverse"},
	{WTurnaround,
	 "Temps_de_rponse"},
	{WType_of_Source_Media,
	 "Type_de_mdia_de_la_source"},
	{WUniversal_Polar_Stereographic,
	 "Strographique_polaire_universel_(SPU)"},
	{WUniversal_Transverse_Mercator,
	 "Universelle_transverse_de_Mercator_(UTM)"},
	{WUnrepresentable_Domain,
	 "Domaine_non_reprsentable"},
	{WUPS_Zone_Identifier,
	 "Identifiant_de_zone_SPU"},
	{WUse_Constraints,
	 "Contraintes_d'utilisation"},
	{WUTM_Zone_Number,
	 "Numro_de_zone_UTM"},
	{Wvan_der_Grinten,
	 "van_der_Grinten"},
	{WVertical_Coordinate_System_Definition,
	 "Dfinition_du_systme_de_coordonnes_verticales"},
	{WVertical_Count,
	 "Dnombrement_vertical"},
	{WVertical_Positional_Accuracy,
	 "Prcision_d'alignement_vertical"},
	{WVertical_Positional_Accuracy_Explanation,
	 "Explication_de_la_prcision_d'alignement_vertical"},
	{WVertical_Positional_Accuracy_Report,
	 "Rapport_sur_la_prcision_d'alignement_vertical"},
	{WVertical_Positional_Accuracy_Value,
	 "Valeur_de_la_prcision_d'alignement_vertical"},
	{WVPF_Point_and_Vector_Object_Information,
	 "Information_sur_les_objets_ponctuels_et_vectoriels_VPF"},
	{WVPF_Point_and_Vector_Object_Type,
	 "Type_d'objet_ponctuel_et_vectoriel_VPF"},
	{WVPF_Terms_Description,
	 "Description_des_termes_VPF"},
	{WVPF_Topology_Level,
	 "Niveau_topologique_VPF"},
	{WWest_Bounding_Coordinate,
	 "Coordonne_de_dlimitation_Ouest"},

/* Biological Data Profile */

	{WDescription_of_Geographic_Extent,
	 "Description_de_l'tendue_gographique"},
	{WBounding_Altitudes,
	 "Altitudes_de_dlimitation"},
	{WAltitude_Minimum,
	 "Altitude_Minimum"},
	{WAltitude_Maximum,
	 "Altitude_Maximum"},
	{WTaxonomy,
	 "Taxonomie"},
	{WKeywords_Taxon,
	 "Mots-cls/Taxon"},
	{WTaxonomic_Keyword_Thesaurus,
	 "Thsaurus_de_mots-cls_taxonomiques"},
	{WTaxonomic_Keywords,
	 "Mots-cls_taxonomiques"},
	{WTaxonomic_System,
	 "Systme_taxonomique"},
	{WClassification_System_Authority,
	 "Systme/Autorit_de_classification"},
	{WClassification_System_Citation,
	 "Citation_du_systme_de_classification"},
	{WClassification_System_Modifications,
	 "Modifications_au_systme_de_classification"},
	{WIdentification_Reference,
	 "Rfrence_d'identification"},
	{WIdentifier,
	 "Identifiant"},
	{WTaxonomic_Procedures,
	 "Procdures_taxonomiques"},
	{WTaxonomic_Completeness,
	 "Compltude_taxonomique"},
	{WVouchers,
	 "Spcimens_tmoins"},
	{WSpecimen,
	 "Spcimen"},
	{WRepository,
	 "Dpt"},
	{WGeneral_Taxonomic_Coverage,
	 "Couverture_taxonomique_gnrale"},
	{WTaxonomic_Classification,
	 "Classification_taxonomique"},
	{WTaxon_Rank_Name,
	 "Nom_du_rang_du_taxon"},
	{WTaxon_Rank_Value,
	 "Valeur_du_rang_du_taxon"},
	{WApplicable_Common_Name,
	 "Nom_commun_pertinent"},
	{WAnalytical_Tool,
	 "Outil_analytique"},
	{WAnalytical_Tool_Description,
	 "Description_de_l'outil_analytique"},
	{WTool_Access_Information,
	 "Information_sur_l'accs__l'outil"},
	{WTool_Contact,
	 "Contact_pour_l'outil"},
	{WTool_Citation,
	 "Citations_sur_l'outil"},
	{WTool_Access_Instructions,
	 "Instructions_d'accs__l'outil"},
	{WTool_Computer_and_Operating_System,
	 "Ordinateur_et_systme_d'exploitation_utiliss_pour_l'outil"},
	{WMethodology,
	 "Mthodologie"},
	{WMethodology_Type,
	 "Type_de_mthodologie"},
	{WMethodology_Identifier,
	 "Identifiant_de_mthodologie"},
	{WMethodology_Keyword_Thesaurus,
	 "Thsaurus_de_mots-cls_mthodologiques"},
	{WMethodology_Keyword,
	 "Mots-cls_mthodologiques"},
	{WMethodology_Description,
	 "Description_de_la_mthodologie"},
	{WMethodology_Citation,
	 "Citation_de_mthodologie"},
	{WASCII_File_Structure,
	 "Structure_des_fichiers_ASCII"},
	{WRecord_Delimiter,
	 "Dlimiteur_d'enregistrement"},
	{WNumber_Header_Lines,
	 "Nombre_de_lignes_de_l'en-tte"},
	{WDescription_of_Header_Content,
	 "Description_de_l'en-tte"},
	{WOrientation,
	 "Orientation"},
	{WCase_Sensitive,
	 "Sensibilit__la_casse"},
	{WAuthentication,
	 "Authentification"},
	{WQuote_Character,
	 "Guillemets"},
	{WData_Field,
	 "Champ_de_donnes"},
	{WData_Field_Name,
	 "Nom_du_champ_de_donnes"},
	{WMissing_Value_Code,
	 "Code_de_valeur_manquante"},
	{WData_Field_Width_Delimiter,
	 "Dlimiteur_de_la_largeur_des_champs_de_donnes"},
	{WData_Field_Width,
	 "Largeur_du_champ_de_donnees"},
	{WBeginning_Geologic_Age,
	 "ge_gologique_de_dbut"},
	{WEnding_Geologic_Age,
	 "ge_gologique_de_fin"},
	{WGeologic_Age,
	 "ge_gologique"},
	{WGeologic_Time_Scale,
	 "chelle_de_temps_gologique"},
	{WGeologic_Age_Estimate,
	 "Estimation_de_l'ge_gologique"},
	{WGeologic_Age_Uncertainty,
	 "Incertitude_de_l'ge_gologique"},
	{WGeologic_Age_Explanation,
	 "Explication_de_l'ge_gologique"},
	{WGeologic_Citation,
	 "Informations_bibliographiques_gologiques"},

/* Shoreline profile */

	{WBarometric_Pressure,
	 "Barometric_Pressure"},
	{WEnvironmental_Event,
	 "Environmental_Event"},
	{WMarine_Weather_Condition,
	 "Marine_Weather_Condition"},
	{WProcess_Step_Citation,
	 "Process_Step_Citation"},
	{WRange_of_Tide,
	 "Range_of_Tide"},
	{WSupplemental_Tidal_Information,
	 "Supplemental_Tidal_Information"},
	{WTidal_Datum,
	 "Tidal_Datum"},
	{WTidal_Information,
	 "Tidal_Information"},
	{WTide_Table_Reference,
	 "Tide_Table_Reference"},
	{WTime_of_High_Tide,
	 "Time_of_High_Tide"},
	{WTime_of_Low_Tide,
	 "Time_of_Low_Tide"},
	{WTime_of_Tide,
	 "Time_of_Tide"},
	{WType_of_Tide,
	 "Type_of_Tide"},
	{WWave_Height,
	 "Wave_Height"},
	{WWind_Direction,
	 "Wind_Direction"},
	{WWind_Speed,
	 "Wind_Speed"},

/* Remote-sensing profile */

	{WAcquisition_Information,
	 "Acquisition_Information"},
	{WAerial_Film_Speed,
	 "Aerial_Film_Speed"},
	{WAerotriangulation_Reference,
	 "Aerotriangulation_Reference"},
	{WAffine_Distortion_X_Prime_Coefficient,
	 "Affine_Distortion_X_Prime_Coefficient"},
	{WAffine_Distortion_Y_Prime_Coefficient,
	 "Affine_Distortion_Y_Prime_Coefficient"},
	{WAggregation_Criteria,
	 "Aggregation_Criteria"},
	{WAggregation_Information,
	 "Aggregation_Information"},
	{WAggregation_Member_ID,
	 "Aggregation_Member_ID"},
	{WAlgorithm_Change_Description,
	 "Algorithm_Change_Description"},
	{WAlgorithm_Change_History,
	 "Algorithm_Change_History"},
	{WAlgorithm_Change_Reference,
	 "Algorithm_Change_Reference"},
	{WAlgorithm_Description,
	 "Algorithm_Description"},
	{WAlgorithm_Identifiers,
	 "Algorithm_Identifiers"},
	{WAlgorithm_Information,
	 "Algorithm_Information"},
	{WAlgorithm_Peer_Review_Description,
	 "Algorithm_Peer_Review_Description"},
	{WAlgorithm_Peer_Review_Information,
	 "Algorithm_Peer_Review_Information"},
	{WAlgorithm_Peer_Review_Reference,
	 "Algorithm_Peer_Review_Reference"},
	{WAlgorithm_Reference,
	 "Algorithm_Reference"},
	{WAlgorithm_Text_Description,
	 "Algorithm_Text_Description"},
	{WAltitude_Distance_Layer,
	 "Altitude_Distance_Layer"},
	{WAncillary_Dataset,
	 "Ancillary_Dataset"},
	{WAncillary_Dataset_Description,
	 "Ancillary_Dataset_Description"},
	{WAncillary_Dataset_Identifier,
	 "Ancillary_Dataset_Identifier"},
	{WAncillary_Description,
	 "Ancillary_Description"},
	{WAncillary_Reference,
	 "Ancillary_Reference"},
	{WAngle_Dependent_Distortion,
	 "Angle_Dependent_Distortion"},
	{WAngle_Distortion_Value,
	 "Angle_Distortion_Value"},
	{WArea_Weighted_Average_Resolution,
	 "Area_Weighted_Average_Resolution"},
	{WArgument_of_Perigee,
	 "Argument_of_Perigee"},
	{WAscending_Descending_Indicator,
	 "Ascending/Descending_Indicator"},
	{WAttitude_Angular_Units,
	 "Attitude_Angular_Units"},
	{WAvailability_of_Element_Locations,
	 "Availability_of_Element_Locations"},
	{WAxes,
	 "Axes"},
	{WAxis_Rotation_Convention,
	 "Axis_Rotation_Convention"},
	{WBand_Boundary_Definition,
	 "Band_Boundary_Definition"},
	{WBand_Description,
	 "Band_Description"},
	{WBand_ID,
	 "Band_ID"},
	{WBand_Identification,
	 "Band_Identification"},
	{WBand_Measurement_Mode_ID,
	 "Band_Measurement_Mode_ID"},
	{WBand_Quality,
	 "Band_Quality"},
	{WCalibrated_Detector_Positions,
	 "Calibrated_Detector_Positions"},
	{WCalibrated_Focal_Length,
	 "Calibrated_Focal_Length"},
	{WCamera_Identifier,
	 "Camera_Identifier"},
	{WCamera_Type,
	 "Camera_Type"},
	{WCell_Value_Type,
	 "Cell_Value_Type"},
	{WClock_Time_Drift,
	 "Clock_Time/Drift"},
	{WCollection_Type,
	 "Collection_Type"},
	{WColumn_Delta_X,
	 "Column_Delta_X"},
	{WColumn_Delta_Y,
	 "Column_Delta_Y"},
	{WCommand_Line_Processing_Parameter,
	 "Command_Line_Processing_Parameter"},
	{WComponent_Information,
	 "Component_Information"},
	{WContainer_Packet_ID,
	 "Container_Packet_ID"},
	{WControl_Point_Authority,
	 "Control_Point_Authority"},
	{WControl_Point_Column,
	 "Control_Point_Column"},
	{WControl_Point_Earth_Location,
	 "Control_Point_Earth_Location"},
	{WControl_Point_Horizontal_X_Accuracy,
	 "Control_Point_Horizontal_X_Accuracy"},
	{WControl_Point_Horizontal_Y_Accuracy,
	 "Control_Point_Horizontal_Y_Accuracy"},
	{WControl_Point_ID,
	 "Control_Point_ID"},
	{WControl_Point_Identification,
	 "Control_Point_Identification"},
	{WControl_Point_Origin,
	 "Control_Point_Origin"},
	{WControl_Point_Raster_Position,
	 "Control_Point_Raster_Position"},
	{WControl_Point_Row,
	 "Control_Point_Row"},
	{WControl_Point_Type,
	 "Control_Point_Type"},
	{WControl_Point_Use_Flag,
	 "Control_Point_Use_Flag"},
	{WControl_Point_Vertical_Accuracy,
	 "Control_Point_Vertical_Accuracy"},
	{WControl_Point_x_Value,
	 "Control_Point_x_Value"},
	{WControl_Point_y_Value,
	 "Control_Point_y_Value"},
	{WControl_Point_z_Value,
	 "Control_Point_z_Value"},
	{WCoordinate_Point,
	 "Coordinate_Point"},
	{WCoordinate_System,
	 "Coordinate_System"},
	{WCoordinate_x_Value,
	 "Coordinate_x_Value"},
	{WCoordinate_XY_Units,
	 "Coordinate_XY_Units"},
	{WCoordinate_y_Value,
	 "Coordinate_y_Value"},
	{WCoordinate_Z_Units,
	 "Coordinate_Z_Units"},
	{WCoordinate_z_Value,
	 "Coordinate_z_Value"},
	{WCross_Track_Axis,
	 "Cross_Track_Axis"},
	{WCross_Track_Description,
	 "Cross_Track_Description"},
	{WCross_Track_Direction,
	 "Cross_Track_Direction"},
	{WCross_Track_Extent_Angle,
	 "Cross_Track_Extent_Angle"},
	{WCross_Track_Fixed_Angle,
	 "Cross_Track_Fixed_Angle"},
	{WCross_Track_Motion,
	 "Cross_Track_Motion"},
	{WCross_Track_Start_Angle,
	 "Cross_Track_Start_Angle"},
	{WCross_Track_Step_Angle,
	 "Cross_Track_Step_Angle"},
	{WCross_Track_Sweep,
	 "Cross_Track_Sweep"},
	{WCross_Track_Zero,
	 "Cross_Track_Zero"},
	{WData_Dictionary_Reference,
	 "Data_Dictionary_Reference"},
	{WData_Scaling_Information,
	 "Data_Scaling_Information"},
	{WDataset_Identifier,
	 "Dataset_Identifier"},
	{WDate_of_Last_Calibration,
	 "Date_of_Last_Calibration"},
	{WDeveloping_Institution,
	 "Developing_Institution"},
	{WDimension_Count,
	 "Dimension_Count"},
	{WDimension_Description,
	 "Dimension_Description"},
	{WDimension_Properties,
	 "Dimension_Properties"},
	{WDistance_Dependent_Distortion,
	 "Distance_Dependent_Distortion"},
	{WDistance_Distortion_Value,
	 "Distance_Distortion_Value"},
	{WDistortion,
	 "Distortion"},
	{WDistortion_Type_Affine,
	 "Distortion_Type_Affine"},
	{WDistortion_Type_Radial_Asymmetrical,
	 "Distortion_Type_Radial_Asymmetrical"},
	{WDistortion_Type_Radial_Symmetrical,
	 "Distortion_Type_Radial_Symmetrical"},
	{WDocumentation,
	 "Documentation"},
	{WEccentricity,
	 "Eccentricity"},
	{WEffective_Aerial_Film_Speed,
	 "Effective_Aerial_Film_Speed"},
	{WElevation_Axis,
	 "Elevation_Axis"},
	{WElevation_Description,
	 "Elevation_Description"},
	{WElevation_Direction,
	 "Elevation_Direction"},
	{WElevation_Extent_Angle,
	 "Elevation_Extent_Angle"},
	{WElevation_Fixed_Angle,
	 "Elevation_Fixed_Angle"},
	{WElevation_Start_Angle,
	 "Elevation_Start_Angle"},
	{WElevation_Step_Angle,
	 "Elevation_Step_Angle"},
	{WElevation_Sweep,
	 "Elevation_Sweep"},
	{WElevation_Zero,
	 "Elevation_Zero"},
	{WEphemeris,
	 "Ephemeris"},
	{WExposure_Time,
	 "Exposure_Time"},
	{WExterior_Orientation_Accuracy,
	 "Exterior_Orientation_Accuracy"},
	{WFiducial,
	 "Fiducial"},
	{WFiducial_Center,
	 "Fiducial_Center"},
	{WFilm_Type,
	 "Film_Type"},
	{WFilter_on_Camera,
	 "Filter_on_Camera"},
	{WFilter_on_Camera_Indicator,
	 "Filter_on_Camera_Indicator"},
	{WFilter_Type,
	 "Filter_Type"},
	{WFlight_Protocol,
	 "Flight_Protocol"},
	{WFlying_Height,
	 "Flying_Height"},
	{WForward_Motion_Compensation,
	 "Forward_Motion_Compensation"},
	{WFrame_Area,
	 "Frame_Area"},
	{WFrame_Area_Units,
	 "Frame_Area_Units"},
	{WFrame_Area_Value,
	 "Frame_Area_Value"},
	{WFrame_Camera,
	 "Frame_Camera"},
	{WFrame_Geometric_Properties,
	 "Frame_Geometric_Properties"},
	{WFrame_Hardware,
	 "Frame_Hardware"},
	{WFrame_Operation,
	 "Frame_Operation"},
	{WFrame_Optics,
	 "Frame_Optics"},
	{WFrame_Radiometric_Calibration,
	 "Frame_Radiometric_Calibration"},
	{WFrame_Radiometric_Properties,
	 "Frame_Radiometric_Properties"},
	{WFrame_Spectral_Information,
	 "Frame_Spectral_Information"},
	{WFrame_Spectral_Properties,
	 "Frame_Spectral_Properties"},
	{WGeorectified_Raster,
	 "Georectified_Raster"},
	{WGeoreferenceable_Raster,
	 "Georeferenceable_Raster"},
	{WGeoreferencing_Description,
	 "Georeferencing_Description"},
	{WGeoreferencing_Information,
	 "Georeferencing_Information"},
	{WGPS_Information_System_Availability,
	 "GPS_Information_System_Availability"},
	{WGrid_First_Element,
	 "Grid_First_Element"},
	{WGrid_First_Element_Map_X_Coordinate,
	 "Grid_First_Element_Map_X_Coordinate"},
	{WGrid_First_Element_Map_Y_Coordinate,
	 "Grid_First_Element_Map_Y_Coordinate"},
	{WGrid_Orientation,
	 "Grid_Orientation"},
	{WGround_Control_Point_Description,
	 "Ground_Control_Point_Description"},
	{WGround_Control_Point_Information,
	 "Ground_Control_Point_Information"},
	{WGround_Control_Point_Organization,
	 "Ground_Control_Point_Organization"},
	{WGround_Control_Point_Position,
	 "Ground_Control_Point_Position"},
	{WGround_Shape,
	 "Ground_Shape"},
	{WID_Overlapping_Dataset,
	 "ID_Overlapping_Dataset"},
	{WIFOV_Units,
	 "IFOV_Units"},
	{WIFOV_x_Definition,
	 "IFOV_x_Definition"},
	{WIFOV_x_Value,
	 "IFOV_x_Value"},
	{WIFOV_y_Definition,
	 "IFOV_y_Definition"},
	{WIFOV_y_Value,
	 "IFOV_y_Value"},
	{WImage_Quality,
	 "Image_Quality"},
	{WImage_Size,
	 "Image_Size"},
	{WImage_Size_x_Value,
	 "Image_Size_x_Value"},
	{WImage_Size_y_Value,
	 "Image_Size_y_Value"},
	{WInclination,
	 "Inclination"},
	{WIndividual_Band_Identification,
	 "Individual_Band_Identification"},
	{WInput_Dataset_Identifier,
	 "Input_Dataset_Identifier"},
	{WInput_Description,
	 "Input_Description"},
	{WInput_Level,
	 "Input_Level"},
	{WInput_Reference,
	 "Input_Reference"},
	{WINS_Reading_Availability,
	 "INS_Reading_Availability"},
	{WInstantaneous_Field_of_View,
	 "Instantaneous_Field_of_View"},
	{WInstitution_of_Last_Calibration,
	 "Institution_of_Last_Calibration"},
	{WInstrument_Description,
	 "Instrument_Description"},
	{WInstrument_Full_Name,
	 "Instrument_Full_Name"},
	{WInstrument_Information,
	 "Instrument_Information"},
	{WInstrument_Properties_Description,
	 "Instrument_Properties_Description"},
	{WInstrument_Reference,
	 "Instrument_Reference"},
	{WInstrument_Short_Name,
	 "Instrument_Short_Name"},
	{WInstrument_Specific_Georeferencing,
	 "Instrument_Specific_Georeferencing"},
	{WInstrument_Type,
	 "Instrument_Type"},
	{WKeplerian_Orbit,
	 "Keplerian_Orbit"},
	{WLast_Calibration,
	 "Last_Calibration"},
	{WLayer_Name,
	 "Layer_Name"},
	{WLens,
	 "Lens"},
	{WLens_Identifier,
	 "Lens_Identifier"},
	{WLens_Type,
	 "Lens_Type"},
	{WLight_Drop,
	 "Light_Drop"},
	{WLocation_Information,
	 "Location_Information"},
	{WMagazine_Identifier,
	 "Magazine_Identifier"},
	{WMaximum_Wavelength,
	 "Maximum_Wavelength"},
	{WMethod_of_Last_Calibration,
	 "Method_of_Last_Calibration"},
	{WMinimum_Wavelength,
	 "Minimum_Wavelength"},
	{WMission_Completion,
	 "Mission_Completion"},
	{WMission_Description,
	 "Mission_Description"},
	{WMission_History,
	 "Mission_History"},
	{WMission_Information,
	 "Mission_Information"},
	{WMission_Name,
	 "Mission_Name"},
	{WMission_Significant_Event,
	 "Mission_Significant_Event"},
	{WMission_Start_Date,
	 "Mission_Start_Date"},
	{WMultiple_Image_Alignment,
	 "Multiple_Image_Alignment"},
	{WNadir,
	 "Nadir"},
	{WNadir_Latitude,
	 "Nadir_Latitude"},
	{WNadir_Longitude,
	 "Nadir_Longitude"},
	{WName_of_Dimension,
	 "Name_of_Dimension"},
	{WNominal_Geostationary_Position,
	 "Nominal_Geostationary_Position"},
	{WNominal_Spatial_Resolution,
	 "Nominal_Spatial_Resolution"},
	{WNon_Polynomial_Scaling,
	 "Non_Polynomial_Scaling"},
	{WNumber_of_Angle_Distortion_Values,
	 "Number_of_Angle_Distortion_Values"},
	{WNumber_of_Bands,
	 "Number_of_Bands"},
	{WNumber_of_Cross_Track_Samples,
	 "Number_of_Cross_Track_Samples"},
	{WNumber_of_Data_Dimensions,
	 "Number_of_Data_Dimensions"},
	{WNumber_of_Denominator_Terms,
	 "Number_of_Denominator_Terms"},
	{WNumber_of_Distance_Distortion_Values,
	 "Number_of_Distance_Distortion_Values"},
	{WNumber_of_Elevation_Samples,
	 "Number_of_Elevation_Samples"},
	{WNumber_of_Numerator_Terms,
	 "Number_of_Numerator_Terms"},
	{WNumber_of_Points,
	 "Number_of_Points"},
	{WNumber_of_Profile_Samples,
	 "Number_of_Profile_Samples"},
	{WNumber_of_Resolution_Values,
	 "Number_of_Resolution_Values"},
	{WNumber_of_Thematic_Layers,
	 "Number_of_Thematic_Layers"},
	{WNumber_of_Wavelength_Bands,
	 "Number_of_Wavelength_Bands"},
	{WOperational_Mode,
	 "Operational_Mode"},
	{WOrbit_Angle_Units,
	 "Orbit_Angle_Units"},
	{WOrbit_Period,
	 "Orbit_Period"},
	{WOrbit_Period_Units,
	 "Orbit_Period_Units"},
	{WOrbit_Period_Value,
	 "Orbit_Period_Value"},
	{WOther_Collector_Description,
	 "Other_Collector_Description"},
	{WOther_Georeferencing_Description,
	 "Other_Georeferencing_Description"},
	{WPath,
	 "Path"},
	{WPeak_Wavelength,
	 "Peak_Wavelength"},
	{WPercentage_of_Overlap,
	 "Percentage_of_Overlap"},
	{WPerigee_Passage_Time,
	 "Perigee_Passage_Time"},
	{WPhotographic_Resolving_Power,
	 "Photographic_Resolving_Power"},
	{WPitch,
	 "Pitch"},
	{WPitch_Accuracy,
	 "Pitch_Accuracy"},
	{WPixel_Cross_Track_Size,
	 "Pixel_Cross_Track_Size"},
	{WPixel_Description,
	 "Pixel_Description"},
	{WPixel_Elevation_Size,
	 "Pixel_Elevation_Size"},
	{WPixel_Height_Above_Ellipsoid,
	 "Pixel_Height_Above_Ellipsoid"},
	{WPixel_Point_Spread_Function,
	 "Pixel_Point_Spread_Function"},
	{WPixel_Profile_Size,
	 "Pixel_Profile_Size"},
	{WPixel_Resolution,
	 "Pixel_Resolution"},
	{WPlatform_and_Instrument_Identification,
	 "Platform_and_Instrument_Identification"},
	{WPlatform_and_Mission_Information,
	 "Platform_and_Mission_Information"},
	{WPlatform_Description,
	 "Platform_Description"},
	{WPlatform_Full_Name,
	 "Platform_Full_Name"},
	{WPlatform_Information,
	 "Platform_Information"},
	{WPlatform_Nominal_Altitude,
	 "Platform_Nominal_Altitude"},
	{WPlatform_Nominal_Altitude_Units,
	 "Platform_Nominal_Altitude_Units"},
	{WPlatform_Nominal_Altitude_Value,
	 "Platform_Nominal_Altitude_Value"},
	{WPlatform_Nominal_Longitude,
	 "Platform_Nominal_Longitude"},
	{WPlatform_Orbit,
	 "Platform_Orbit"},
	{WPlatform_Serial_Identifier,
	 "Platform_Serial_Identifier"},
	{WPlatform_Short_Name,
	 "Platform_Short_Name"},
	{WPlatform_Sponsor,
	 "Platform_Sponsor"},
	{WPlatform_Start_Date,
	 "Platform_Start_Date"},
	{WPoint_Position_In_Pixel,
	 "Point_Position_In_Pixel"},
	{WPolarization_Characteristics,
	 "Polarization_Characteristics"},
	{WPolynomial_Coefficient,
	 "Polynomial_Coefficient"},
	{WPolynomial_Denominator,
	 "Polynomial_Denominator"},
	{WPolynomial_Function,
	 "Polynomial_Function"},
	{WPolynomial_Numerator,
	 "Polynomial_Numerator"},
	{WPositional_Information,
	 "Positional_Information"},
	{WPrincipal_Point_of_Autocollimation,
	 "Principal_Point_of_Autocollimation"},
	{WPrincipal_Point_of_Symmetry,
	 "Principal_Point_of_Symmetry"},
	{WProcessing_Change_History,
	 "Processing_Change_History"},
	{WProcessing_Documentation,
	 "Processing_Documentation"},
	{WProcessing_Environment,
	 "Processing_Environment"},
	{WProcessing_Identifiers,
	 "Processing_Identifiers"},
	{WProcessing_Information,
	 "Processing_Information"},
	{WProcessing_Input_Dataset,
	 "Processing_Input_Dataset"},
	{WProcessing_Level,
	 "Processing_Level"},
	{WProcessing_Level_Authority,
	 "Processing_Level_Authority"},
	{WProcessing_Level_Identifier,
	 "Processing_Level_Identifier"},
	{WProcessing_Procedure,
	 "Processing_Procedure"},
	{WProcessing_Procedure_Description,
	 "Processing_Procedure_Description"},
	{WProcessing_Run_History,
	 "Processing_Run_History"},
	{WProcessing_Software,
	 "Processing_Software"},
	{WProcessing_Software_Description,
	 "Processing_Software_Description"},
	{WProcessing_Software_Reference,
	 "Processing_Software_Reference"},
	{WProfile_Description,
	 "Profile_Description"},
	{WProfile_Extent,
	 "Profile_Extent"},
	{WProfile_Fixed,
	 "Profile_Fixed"},
	{WProfile_Properties,
	 "Profile_Properties"},
	{WProfile_Sounding,
	 "Profile_Sounding"},
	{WProfile_Start,
	 "Profile_Start"},
	{WProfile_Step,
	 "Profile_Step"},
	{WProfiling_Direction,
	 "Profiling_Direction"},
	{WProjection_Center_X_Position,
	 "Projection_Center_X_Position"},
	{WProjection_Center_Y_Position,
	 "Projection_Center_Y_Position"},
	{WProjection_Center_Z_Position,
	 "Projection_Center_Z_Position"},
	{WQuality_of_Autocollimation_Principal_Point_x_Value,
	 "Quality_of_Autocollimation_Principal_Point_x_Value"},
	{WQuality_of_Autocollimation_Principal_Point_y_Value,
	 "Quality_of_Autocollimation_Principal_Point_y_Value"},
	{WQuality_of_Focal_Length,
	 "Quality_of_Focal_Length"},
	{WQuality_of_Principal_Point_of_Autocollimation,
	 "Quality_of_Principal_Point_of_Autocollimation"},
	{WQuality_of_Principal_Point_of_Symmetry,
	 "Quality_of_Principal_Point_of_Symmetry"},
	{WQuality_of_Symmetry_Principal_Point_x_Value,
	 "Quality_of_Symmetry_Principal_Point_x_Value"},
	{WQuality_of_Symmetry_Principal_Point_y_Value,
	 "Quality_of_Symmetry_Principal_Point_y_Value"},
	{WRadial_Asymmetrical_Coefficient_B1,
	 "Radial_Asymmetrical_Coefficient_B1"},
	{WRadial_Asymmetrical_Coefficient_B2,
	 "Radial_Asymmetrical_Coefficient_B2"},
	{WRadial_Symmetrical_Angle_Interval,
	 "Radial_Symmetrical_Angle_Interval"},
	{WRadial_Symmetrical_Distance_Interval,
	 "Radial_Symmetrical_Distance_Interval"},
	{WRadial_Symmetrical_Distortion_Polynomial,
	 "Radial_Symmetrical_Distortion_Polynomial"},
	{WReceiver_Polarization,
	 "Receiver_Polarization"},
	{WReferenced_Coordinate_System,
	 "Referenced_Coordinate_System"},
	{WReferencing_Polynomial,
	 "Referencing_Polynomial"},
	{WRelative_Aperture,
	 "Relative_Aperture"},
	{WRelative_Azimuth,
	 "Relative_Azimuth"},
	{WReseau,
	 "Reseau"},
	{WResolution_Value_Set,
	 "Resolution_Value_Set"},
	{WResolving_Angle,
	 "Resolving_Angle"},
	{WResolving_Value_Radial,
	 "Resolving_Value_Radial"},
	{WResolving_Value_Tangential,
	 "Resolving_Value_Tangential"},
	{WRight_Ascension_of_Ascending_Node,
	 "Right_Ascension_of_Ascending_Node"},
	{WRoll,
	 "Roll"},
	{WRoll_Accuracy,
	 "Roll_Accuracy"},
	{WRotation_Description,
	 "Rotation_Description"},
	{WRotation_Sequence,
	 "Rotation_Sequence"},
	{WRow,
	 "Row"},
	{WRow_Delta_X,
	 "Row_Delta_X"},
	{WRow_Delta_Y,
	 "Row_Delta_Y"},
	{WSample_Angle_Units,
	 "Sample_Angle_Units"},
	{WSample_Description_Units,
	 "Sample_Description_Units"},
	{WSample_Profile_Units,
	 "Sample_Profile_Units"},
	{WSample_Properties,
	 "Sample_Properties"},
	{WSatellite_Local_Azimuth_Angle,
	 "Satellite/Local_Azimuth_Angle"},
	{WSatellite_Local_Zenith_Angle,
	 "Satellite/Local_Zenith_Angle"},
	{WScan,
	 "Scan"},
	{WScan_Angle_Units,
	 "Scan_Angle_Units"},
	{WScan_Cross_Track_Properties,
	 "Scan_Cross_Track_Properties"},
	{WScan_Distance_Units,
	 "Scan_Distance_Units"},
	{WScan_Duration,
	 "Scan_Duration"},
	{WScan_Elevation_Properties,
	 "Scan_Elevation_Properties"},
	{WScan_Geometric_Properties,
	 "Scan_Geometric_Properties"},
	{WScan_Radiometric_Properties,
	 "Scan_Radiometric_Properties"},
	{WScan_Repeat_Time,
	 "Scan_Repeat_Time"},
	{WScan_Spectral_Properties,
	 "Scan_Spectral_Properties"},
	{WScan_Start_Time,
	 "Scan_Start_Time"},
	{WScan_Step_Time,
	 "Scan_Step_Time"},
	{WScan_Time_Units,
	 "Scan_Time_Units"},
	{WScan_Timing,
	 "Scan_Timing"},
	{WScience_Paper,
	 "Science_Paper"},
	{WSemimajor_Axis,
	 "Semimajor_Axis"},
	{WSender_Polarization,
	 "Sender_Polarization"},
	{WSensor_Element_Location,
	 "Sensor_Element_Location"},
	{WSensor_Grid,
	 "Sensor_Grid"},
	{WSensor_Orientation,
	 "Sensor_Orientation"},
	{WSensor_System,
	 "Sensor_System"},
	{WSolar_Azimuth_Angle,
	 "Solar_Azimuth_Angle"},
	{WSolar_Zenith_Angle,
	 "Solar_Zenith_Angle"},
	{WSource_of_Element_Locations,
	 "Source_of_Element_Locations"},
	{WSpatial_Resolution_Units,
	 "Spatial_Resolution_Units"},
	{WSpatial_Resolution_Value,
	 "Spatial_Resolution_Value"},
	{WSpectral_Information,
	 "Spectral_Information"},
	{WSpectral_Limit,
	 "Spectral_Limit"},
	{WStabilized_Mount,
	 "Stabilized_Mount"},
	{WStorage_Order,
	 "Storage_Order"},
	{WSwath_Track_Information,
	 "Swath_Track_Information"},
	{WThematic_Layer_Identification,
	 "Thematic_Layer_Identification"},
	{WTranslation_Description,
	 "Translation_Description"},
	{WType_of_Overlap,
	 "Type_of_Overlap"},
	{WUnreferenced_Coordinate_System,
	 "Unreferenced_Coordinate_System"},
	{WUsers_Guide,
	 "User's_Guide"},
	{WWavelength_Band_Properties,
	 "Wavelength_Band_Properties"},
	{WWavelength_Region,
	 "Wavelength_Region"},
	{WWavelength_Units,
	 "Wavelength_Units"},
	{WWorldwide_Reference_System,
	 "Worldwide_Reference_System"},
	{WX_Axis_Definition,
	 "X_Axis_Definition"},
	{WX_Position_Accuracy,
	 "X_Position_Accuracy"},
	{WY_Axis_Definition,
	 "Y_Axis_Definition"},
	{WY_Position_Accuracy,
	 "Y_Position_Accuracy"},
	{WYaw,
	 "Yaw"},
	{WYaw_Accuracy,
	 "Yaw_Accuracy"},
	{WZ_Axis_Definition,
	 "Z_Axis_Definition"},
	{WZ_Position_Accuracy,
	 "Z_Position_Accuracy"},


#ifdef USE_ESRI_EXTENSIONS

	/* ESRI extensions */

	{WAltitude_Units,
	 "Altitude_Units"},
	{WAncillary_Role,
	 "Ancillary_Role"},
	{WAncillary_Role_Attribute,
	 "Ancillary_Role_Attribute"},
	{WAttribute_Alias,
	 "Attribute_Alias"},
	{WAttribute_Defined_Domain,
	 "Attribute_Defined_Domain"},
	{WAttribute_Indexed,
	 "Attribute_Indexed"},
	{WAttribute_Number_of_Decimals,
	 "Attribute_Number_of_Decimals"},
	{WAttribute_Output_Width,
	 "Attribute_Output_Width"},
	{WAttribute_Precision,
	 "Attribute_Precision"},
	{WAttribute_Scale,
	 "Attribute_Scale"},
	{WAttribute_Type,
	 "Attribute_Type"},
	{WAttribute_Width,
	 "Attribute_Width"},
	{WAttributed_Relationship,
	 "Attributed_Relationship"},
	{WAvailable_Junction_Feature_Class,
	 "Available_Junction_Feature_Class"},
	{WAvailable_Junction_Subtype,
	 "Available_Junction_Subtype"},
	{WAvailable_Junctions,
	 "Available_Junctions"},
	{WBackground_Nodata_Value,
	 "Background_Nodata_Value"},
	{WBehrmann,
	 "Behrmann"},
	{WBinary_Enclosures,
	 "Binary_Enclosures"},
	{WBits_Per_Pixel,
	 "Bits_Per_Pixel"},
	{WBonne,
	 "Bonne"},
	{WBottom_Bounding_Coordinate,
	 "Bottom_Bounding_Coordinate"},
	{WBrowse_Graphic_Embedded,
	 "Browse_Graphic_Embedded"},
	{WCassini,
	 "Cassini"},
	{WCell_Size_X_Direction,
	 "Cell_Size_X_Direction"},
	{WCell_Size_X_Units,
	 "Cell_Size_X_Units"},
	{WCell_Size_Y_Direction,
	 "Cell_Size_Y_Direction"},
	{WCell_Size_Y_Units,
	 "Cell_Size_Y_Units"},
	{WComposite_Relationship,
	 "Composite_Relationship"},
	{WCompression_Type,
	 "Compression_Type"},
	{WConnectivity_Rule,
	 "Connectivity_Rule"},
	{WCoordinate_System_Name,
	 "Coordinate_System_Name"},
	{WCreation_Date,
	 "Creation_Date"},
	{WCreation_Time,
	 "Creation_Time"},
	{WData,
	 "Data"},
	{WData_Frame_Count,
	 "Data_Frame_Count"},
	{WData_Frame_Name,
	 "Data_Frame_Name"},
	{WData_Frames,
	 "Data_Frames"},
	{WDatabase_Name,
	 "Database_Name"},
	{WDataset_Size,
	 "Dataset_Size"},
	{WDefault_Junction_Feature_Class,
	 "Default_Junction_Feature_Class"},
	{WDefault_Junction_Subtype,
	 "Default_Junction_Subtype"},
	{WDescription_of_Association,
	 "Description_of_Association"},
	{WDescription_of_Enclosure,
	 "Description_of_Enclosure"},
	{WDescription_of_Relationship,
	 "Description_of_Relationship"},
	{WDestination_Foreign_Key,
	 "Destination_Foreign_Key"},
	{WDestination_Name,
	 "Destination_Name"},
	{WDestination_Primary_Key,
	 "Destination_Primary_Key"},
	{WDomain_Attribute_Type,
	 "Domain_Attribute_Type"},
	{WDomain_Description,
	 "Domain_Description"},
	{WDomain_Name,
	 "Domain_Name"},
	{WDomain_Owner,
	 "Domain_Owner"},
	{WDomain_Type,
	 "Domain_Type"},
	{WEckert_I,
	 "Eckert_I"},
	{WEckert_II,
	 "Eckert_II"},
	{WEckert_III,
	 "Eckert_III"},
	{WEckert_IV,
	 "Eckert_IV"},
	{WEckert_V,
	 "Eckert_V"},
	{WEckert_VI,
	 "Eckert_VI"},
	{WEdge_Feature_Class,
	 "Edge_Feature_Class"},
	{WEdge_Maximum_Cardinality,
	 "Edge_Maximum_Cardinality"},
	{WEdge_Minimum_Cardinality,
	 "Edge_Minimum_Cardinality"},
	{WEdge_Subtype,
	 "Edge_Subtype"},
	{WEnabled_Attribute,
	 "Enabled_Attribute"},
	{WEnclosure,
	 "Enclosure"},
	{WEntity_Type_Count,
	 "Entity_Type_Count"},
	{WEntity_Type_Type,
	 "Entity_Type_Type"},
	{WESRI_Feature_Count,
	 "ESRI_Feature_Count"},
	{WESRI_Feature_Geometry,
	 "ESRI_Feature_Geometry"},
	{WESRI_Feature_Type,
	 "ESRI_Feature_Type"},
	{WESRI_Information,
	 "ESRI_Information"},
	{WESRI_Terms_Description,
	 "ESRI_Terms_Description"},
	{WESRI_Topology,
	 "ESRI_Topology"},
	{WFeature_Description,
	 "Feature_Description"},
	{WFile_or_Table_Name,
	 "File_or_Table_Name"},
	{WFrom_Edge_Feature_Class,
	 "From_Edge_Feature_Class"},
	{WFrom_Edge_Subtype,
	 "From_Edge_Subtype"},
	{WGall_Stereographic,
	 "Gall_Stereographic"},
	{WGeocoding_Information,
	 "Geocoding_Information"},
	{WGeographic_Coordinate_System_Name,
	 "Geographic_Coordinate_System_Name"},
	{WGeometric_Network_Information,
	 "Geometric_Network_Information"},
	{WImage_Colormap,
	 "Image_Colormap"},
	{WImage_Format,
	 "Image_Format"},
	{WImage_Type,
	 "Image_Type"},
	{WInstance_Name,
	 "Instance_Name"},
	{WJunction_Feature_Class,
	 "Junction_Feature_Class"},
	{WJunction_Maximum_Cardinality,
	 "Junction_Maximum_Cardinality"},
	{WJunction_Minimum_Cardinality,
	 "Junction_Minimum_Cardinality"},
	{WJunction_Subtype,
	 "Junction_Subtype"},
	{WLanguage_of_Dataset,
	 "Language_of_Dataset"},
	{WLanguage_of_Metadata,
	 "Language_of_Metadata"},
	{WLeft_Bounding_Coordinate,
	 "Left_Bounding_Coordinate"},
	{WLinear_Referencing,
	 "Linear_Referencing"},
	{WLocal_Bounding_Coordinates,
	 "Local_Bounding_Coordinates"},
	{WLoximuthal,
	 "Loximuthal"},
	{WMaximum_Altitude,
	 "Maximum_Altitude"},
	{WMerge_Rule,
	 "Merge_Rule"},
	{WMetadata_ID,
	 "Metadata_ID"},
	{WMinimum_Altitude,
	 "Minimum_Altitude"},
	{WModification_Date,
	 "Modification_Date"},
	{WModification_Time,
	 "Modification_Time"},
	{WMollweide,
	 "Mollweide"},
	{WNative_Dataset_Format,
	 "Native_Dataset_Format"},
	{WNetwork_Element,
	 "Network_Element"},
	{WNetwork_Role,
	 "Network_Role"},
	{WNetwork_Type,
	 "Network_Type"},
	{WNotification_Direction,
	 "Notification_Direction"},
	{WNumber_of_Raster_Bands,
	 "Number_of_Raster_Bands"},
	{WOrigin_Foreign_Key,
	 "Origin_Foreign_Key"},
	{WOrigin_Name,
	 "Origin_Name"},
	{WOrigin_Primary_Key,
	 "Origin_Primary_Key"},
	{WProcess_Software_and_Version,
	 "Process_Software_and_Version"},
	{WProjected_Coordinate_System_Name,
	 "Projected_Coordinate_System_Name"},
	{WPyramid_Layers,
	 "Pyramid_Layers"},
	{WQuartic_Authalic,
	 "Quartic_Authalic"},
	{WRange_Domain_Mean,
	 "Range_Domain_Mean"},
	{WRange_Domain_Standard_Deviation,
	 "Range_Domain_Standard_Deviation"},
	{WRaster_Display_Type,
	 "Raster_Display_Type"},
	{WRaster_Origin,
	 "Raster_Origin"},
	{WRelationship_Backward_Label,
	 "Relationship_Backward_Label"},
	{WRelationship_Cardinality,
	 "Relationship_Cardinality"},
	{WRelationship_Forward_Label,
	 "Relationship_Forward_Label"},
	{WRelationship_Information,
	 "Relationship_Information"},
	{WRight_Bounding_Coordinate,
	 "Right_Bounding_Coordinate"},
	{WRule_Category,
	 "Rule_Category"},
	{WRule_Help,
	 "Rule_Help"},
	{WRule_Type,
	 "Rule_Type"},
	{WSDE_Connection_Information,
	 "SDE_Connection_Information"},
	{WServer_Name,
	 "Server_Name"},
	{WSpatial_Index,
	 "Spatial_Index"},
	{WSplit_Rule,
	 "Split_Rule"},
	{WSubtype_Attribute,
	 "Subtype_Attribute"},
	{WSubtype_Attribute_Name,
	 "Subtype_Attribute_Name"},
	{WSubtype_Code,
	 "Subtype_Code"},
	{WSubtype_Default_Value,
	 "Subtype_Default_Value"},
	{WSubtype_Information,
	 "Subtype_Information"},
	{WSubtype_Name,
	 "Subtype_Name"},
	{WSynchronization_Date,
	 "Synchronization_Date"},
	{WSynchronization_Time,
	 "Synchronization_Time"},
	{WSynchronize_Automatically,
	 "Synchronize_Automatically"},
	{WSynchronize_Once,
	 "Synchronize_Once"},
	{WThumbnail,
	 "Thumbnail"},
	{WTo_Edge_Feature_Class,
	 "To_Edge_Feature_Class"},
	{WTo_Edge_Subtype,
	 "To_Edge_Subtype"},
	{WTop_Bounding_Coordinate,
	 "Top_Bounding_Coordinate"},
	{WUser_Name,
	 "User_Name"},
	{WVersion_Name,
	 "Version_Name"},
	{WWinkel_I,
	 "Winkel_I"},
	{WWinkel_II,
	 "Winkel_II"},
	{WGeographic_Box,
	 "Geographic_Box"},

#endif

	{Wunknown, NULL}
	};

/*----------------------------------------------------------------------*\
 | Indonesian															|
 |																		|
 | Indonesian-language element names kindly provided by the Indonesian	|
 | National Coordination Agency for Surveys and Mapping BAKOSURTANAL	|
\*----------------------------------------------------------------------*/

static struct keyword_list id_std[] = {
	{WAbscissa_Resolution,
	 "Resolusi_Abscissa"},
	{WAbstract,
	 "Abstrak"},
	{WAccess_Constraints,
	 "Batasan_Akses_ke_Data"},
	{WAccess_Instructions,
	 "Akses_Instruksi"},
	{WAddress,
	 "Alamat"},
	{WAddress_Type,
	 "Tipe_Alamat"},
	{WAlbers_Conical_Equal_Area,
	 "Albers_Conical_Equal_Area"},
	{WAltitude_Datum_Name,
	 "Altitude_Datum_Name"},
	{WAltitude_Distance_Units,
	 "Altitude_Distance_Units"},
	{WAltitude_Encoding_Method,
	 "Altitude_Encoding_Method"},
	{WAltitude_Resolution,
	 "Altitude_Resolution"},
	{WAltitude_System_Definition,
	 "Altitude_System_Definition"},
	{WARC_Coordinate_System,
	 "ARC_Coordinate_System"},
	{WARC_System_Zone_Identifier,
	 "ARC_System_Zone_Identifier"},
	{WAttribute,
	 "Atribut"},
	{WAttribute_Accuracy,
	 "Ketelitian_Atribut"},
	{WAttribute_Accuracy_Explanation,
	 "Keterangan_Ketelitian_Atribut"},
	{WAttribute_Accuracy_Report,
	 "Laporan_Ketelitian_Atribut"},
	{WAttribute_Accuracy_Value,
	 "Nilai_Ketelitian_Atribut"},
	{WAttribute_Definition,
	 "Definisi_Atribut"},
	{WAttribute_Definition_Source,
	 "Sumber_Definisi_Atribut"},
	{WAttribute_Domain_Values,
	 "Nilai_Atribut"},
	{WAttribute_Label,
	 "Label_Atribut"},
	{WAttribute_Measurement_Frequency,
	 "Frekuensi_Ukuran_Atribut"},
	{WAttribute_Measurement_Resolution,
	 "Resolusi_Ukuran_Atribut"},
	{WAttribute_Units_of_Measure,
	 "Satuan_Atribut"},
	{WAttribute_Value_Accuracy,
	 "Ketelitian_Nilai_Atribut"},
	{WAttribute_Value_Accuracy_Explanation,
	 "Keterangan_Ketelitian_Nilai_Atribut"},
	{WAttribute_Value_Accuracy_Information,
	 "Informasi_Ketelitian_Nilai_Atribut"},
	{WAvailable_Time_Period,
	 "Waktu_Yang_Tersedia"},
	{WAzimuthal_Angle,
	 "Sudut_Azimut"},
	{WAzimuthal_Equidistant,
	 "Jarak_Azimut"},
	{WAzimuth_Measure_Point_Longitude,
	 "Azimuth_Measure_Point_Longitude"},
	{WBearing_Reference_Direction,
	 "Bearing_Reference_Direction"},
	{WBearing_Reference_Meridian,
	 "Bearing_Reference_Meridian"},
	{WBearing_Resolution,
	 "Bearing_Resolution"},
	{WBearing_Units,
	 "Bearing_Units"},
	{WBeginning_Date,
	 "Tanggal_Mulai"},
	{WBeginning_Date_of_Attribute_Values,
	 "Tanggal_Mulai_Penilaian_Atribut"},
	{WBeginning_Time,
	 "Waktu_Mulai"},
	{WBounding_Coordinates,
	 "Batas_Koordinat"},
	{WBrowse_Graphic,
	 "Gambaran_Data"},
	{WBrowse_Graphic_File_Description,
	 "Deskripsi_File_Gambaran_Data"},
	{WBrowse_Graphic_File_Name,
	 "Nama_File_Gambaran_Data"},
	{WBrowse_Graphic_File_Type,
	 "Tipe_File_Gambaran_Data"},
	{WCalendar_Date,
	 "Tanggal"},
	{WCitation,
	 "Sitasi"},
	{WCitation_Information,
	 "Informasi_Sitasi"},
	{WCity,
	 "Kota"},
	{WCloud_Cover,
	 "Tutupan_Awan"},
	{WCodeset_Domain,
	 "Kode_Letak"},
	{WCodeset_Name,
	 "Kode_Nama"},
	{WCodeset_Source,
	 "Kode_Sumber"},
	{WColumn_Count,
	 "Jumlah_Kolom"},
	{WCompatibility_Information,
	 "Informasi_Kesesuaian"},
	{WCompleteness_Report,
	 "Kelengkapan_Laporan"},
	{WCompression_Support,
	 "Compression_Support"},
	{WComputer_Contact_Information,
	 "Kontak_Informasi_Komputer"},
	{WContact_Address,
	 "Kontak_Alamat"},
	{WContact_Electronic_Mail_Address,
	 "E-mail"},
	{WContact_Facsimile_Telephone,
	 "Faksimili"},
	{WContact_Information,
	 "Informasi_Kontak"},
	{WContact_Instructions,
	 "Kontak_Instruksi"},
	{WContact_Organization,
	 "Kontak_Organisasi"},
	{WContact_Organization_Primary,
	 "Kontak_Organisasi_Utama"},
	{WContact_Person,
	 "Kontak_Personil"},
	{WContact_Person_Primary,
	 "Kontak_Personil_Utama"},
	{WContact_Position,
	 "Kontak_Jabatan"},
	{WContact_TDD_TTY_Telephone,
	 "Contact_TDD/TTY_Telephone"},
	{WContact_Voice_Telephone,
	 "Telepon"},
	{WCoordinate_Representation,
	 "Koordinat_Yang_Mewakili"},
	{WCountry,
	 "Negara"},
	{WCross_Reference,
	 "Acuan_Lain"},
	{WCurrentness_Reference,
	 "Acuan_Terbaru"},
	{WCustom_Order_Process,
	 "Tatacara_Pemesanan"},
	{WData_Quality_Information,
	 "Informasi_Kualitas_Data"},
	{WData_Set_Credit,
	 "Penghargaan_Data"},
	{WData_Set_G_Polygon,
	 "Data_Set_G-Polygon"},
	{WData_Set_G_Polygon_Exclusion_G_Ring,
	 "Data_Set_G-Polygon_Exclusion_G-Ring"},
	{WData_Set_G_Polygon_Outer_G_Ring,
	 "Data_Set_G-Polygon_Outer_G-Ring"},
	{WDenominator_of_Flattening_Ratio,
	 "Denominator_of_Flattening_Ratio"},
	{WDepth_Datum_Name,
	 "Nama_Detil_Datum"},
	{WDepth_Distance_Units,
	 "Satuan_Detil_Jarak"},
	{WDepth_Encoding_Method,
	 "Metode_Detil_Kodifikasi"},
	{WDepth_Resolution,
	 "Resolusi_Detil"},
	{WDepth_System_Definition,
	 "Detil_Definisi_Sistim"},
	{WDescription,
	 "Diskripsi"},
	{WDetailed_Description,
	 "Diskripsi_Detil"},
	{WDialup_File_Name,
	 "Dialup_File_Name"},
	{WDialup_Instructions,
	 "Instruksi_Dialup"},
	{WDialup_Telephone,
	 "Telephone_Dialup"},
	{WDigital_Form,
	 "Bentuk_Dijital"},
	{WDigital_Transfer_Information,
	 "Informasi_Transfer_Dijital"},
	{WDigital_Transfer_Option,
	 "Pilihan_Transfer_Dijital"},
	{WDirect_Spatial_Reference_Method,
	 "Metode_Referensi_Spasial_Langsung"},
	{WDistance_and_Bearing_Representation,
	 "Distance_and_Bearing_Representation"},
	{WDistance_Resolution,
	 "Resolusi_Jarak"},
	{WDistribution_Information,
	 "Informasi_Distribusi"},
	{WDistribution_Liability,
	 "Jaminan_Distribusi"},
	{WDistributor,
	 "Distribusi"},
	{WEast_Bounding_Coordinate,
	 "Koordinat_Batas_Timur"},
	{WEdition,
	 "Edisi"},
	{WEllipsoid_Name,
	 "Nama_Elipsoid"},
	{WEnding_Date,
	 "Tanggal_Akhir"},
	{WEnding_Date_of_Attribute_Values,
	 "Tanggal_Akhir_Nilai_Atribut"},
	{WEnding_Time,
	 "Waktu_Akhir"},
	{WEntity_and_Attribute_Detail_Citation,
	 "Sitasi_Detil_Entitas_dan_Atribut"},
	{WEntity_and_Attribute_Information,
	 "Informasi_Entitas_dan_Atribut"},
	{WEntity_and_Attribute_Overview,
	 "Gambaran_Umum_Entitas_dan_Atribut"},
	{WEntity_Type,
	 "Tipe_Entitas"},
	{WEntity_Type_Definition,
	 "Definisi_Tipe_Entitas"},
	{WEntity_Type_Definition_Source,
	 "Sumber_Definisi_Tipe_Entitas"},
	{WEntity_Type_Label,
	 "Label_Tipe_Entitas"},
	{WEnumerated_Domain,
	 "Daftar_Kategori"},
	{WEnumerated_Domain_Value,
	 "Label_Kategori"},
	{WEnumerated_Domain_Value_Definition,
	 "Definisi_Label_Kategori"},
	{WEnumerated_Domain_Value_Definition_Source,
	 "Sumber_Definisi_Label_Kategori"},
	{WEquidistant_Conic,
	 "Equidistant_Conic"},
	{WEquirectangular,
	 "Equirectangular"},
	{WFalse_Easting,
	 "False_Easting"},
	{WFalse_Northing,
	 "False_Northing"},
	{WFees,
	 "Biaya"},
	{WFile_Decompression_Technique,
	 "Tehnik_Pengkompresan_File"},
	{WFormat_Information_Content,
	 "Isi_Informasi_Format"},
	{WFormat_Name,
	 "Nama_Format"},
	{WFormat_Specification,
	 "Spesifikasi_Format"},
	{WFormat_Version_Date,
	 "Tanggal_Versi_Format"},
	{WFormat_Version_Number,
	 "Nomor_Versi_Format"},
	{WG_Ring,
	 "G-Ring"},
	{WG_Ring_Latitude,
	 "G-Ring_Latitude"},
	{WG_Ring_Longitude,
	 "G-Ring_Longitude"},
	{WG_Ring_Point,
	 "G-Ring_Point"},
	{WGeneral_Vertical_Near_sided_Perspective,
	 "General_Vertical_Near-sided_Perspective"},
	{WGeodetic_Model,
	 "Model_Geodetik"},
	{WGeographic,
	 "Geografi"},
	{WGeographic_Coordinate_Units,
	 "Unit_Koordinat_Geografi"},
	{WGeospatial_Data_Presentation_Form,
	 "Bentuk_Tampilan_Data_Geospasial"},
	{WGnomonic,
	 "Gnomonic"},
	{WGrid_Coordinate_System,
	 "Sistim_Koordinat_Grid"},
	{WGrid_Coordinate_System_Name,
	 "Nama_Sistim_Koordinat_Grid"},
	{WHeight_of_Perspective_Point_Above_Surface,
	 "Height_of_Perspective_Point_Above_Surface"},
	{WHighest_BPS,
	 "Highest_BPS"},
	{WHorizontal_Coordinate_System_Definition,
	 "Definisi_Sistim_Koordinat_Horizontal"},
	{WHorizontal_Datum_Name,
	 "Nama_Datum_Horizontal"},
	{WHorizontal_Positional_Accuracy,
	 "Ketelitian_Posisi_Horizontal"},
	{WHorizontal_Positional_Accuracy_Explanation,
	 "Keterangan_Ketelitian_Posisi_Horizontal"},
	{WHorizontal_Positional_Accuracy_Report,
	 "Laporan_Ketelitian_Posisi_Horizontal"},
	{WHorizontal_Positional_Accuracy_Value,
	 "Nilai_Ketelitian_Posisi_Horizontal"},
	{WHours_of_Service,
	 "Waktu_Pelayanan"},
	{WIdentification_Information,
	 "Informasi_Identifikasi"},
	{WIndirect_Spatial_Reference,
	 "Acuan_Spasial_Tidak_Langsung"},
	{WIssue_Identification,
	 "Identitas_Terbitan"},
	{WKeywords,
	 "Kata_Kunci"},
	{WLambert_Azimuthal_Equal_Area,
	 "Lambert_Azimuthal_Equal_Area"},
	{WLambert_Conformal_Conic,
	 "Lambert_Conformal_Conic"},
	{WLandsat_Number,
	 "Nomor_Landsat"},
	{WLarger_Work_Citation,
	 "Larger_Work_Citation"},
	{WLatitude_of_Projection_Center,
	 "Lintang_Titik_Pusat_Proyeksi"},
	{WLatitude_of_Projection_Origin,
	 "Lintang_Proyeksi_Asli"},
	{WLatitude_Resolution,
	 "Resolusi_Lintang"},
	{WLineage,
	 "Silsilah"},
	{WLocal,
	 "Lokal"},
	{WLocal_Description,
	 "Diskripsi_Lokal"},
	{WLocal_Georeference_Information,
	 "Informasi_Georeferensi_Lokal"},
	{WLocal_Planar,
	 "Planar_Lokal"},
	{WLocal_Planar_Description,
	 "Diskripsi_Planar_Lokal"},
	{WLocal_Planar_Georeference_Information,
	 "Informasi_Georeferensi_Planar_Lokal"},
	{WLogical_Consistency_Report,
	 "Keterangan_Konsistensi_Data"},
	{WLongitude_of_Central_Meridian,
	 "Bujur_dari_Titik_Pusat_Meridian"},
	{WLongitude_of_Projection_Center,
	 "Bujur_dari_Titik_Pusat_Proyeksi"},
	{WLongitude_Resolution,
	 "Resolusi_Bujur"},
	{WLowest_BPS,
	 "BPS_Terendah"},
	{WMaintenance_and_Update_Frequency,
	 "Frekuensi_Perubahan_dan_Penambahan_Data"},
	{WMap_Projection,
	 "Proyeksi_Peta"},
	{WMap_Projection_Name,
	 "Nama_Proyeksi_Peta"},
	{WMap_Projection_Parameters,
	 "Map_Projection_Parameters"},
	{WMercator,
	 "Mercator"},
	{WMetadata,
	 "Metadata"},
	{WMetadata_Access_Constraints,
	 "Batasan_Akses_ke_Metadata"},
	{WMetadata_Contact,
	 "Kontak_Metadata"},
	{WMetadata_Date,
	 "Tanggal_Metadata"},
	{WMetadata_Extensions,
	 "Metadata_Extensions"},
	{WMetadata_Future_Review_Date,
	 "Tanggal_Pembaharuan_Ulang_Metadata"},
	{WMetadata_Language,
	 "Metadata_Language"},
	{WMetadata_Reference_Information,
	 "Informasi_Acuan_Metadata"},
	{WMetadata_Review_Date,
	 "Tanggal_Pembaharuan_Metadata"},
	{WMetadata_Security_Classification,
	 "Klasifikasi_Keamanan_Metadata"},
	{WMetadata_Security_Classification_System,
	 "Sistim_Klasifikasi_Keamanan_Metadata"},
	{WMetadata_Security_Handling_Description,
	 "Diskripsi_Penanganan_Keamanan_Metadata"},
	{WMetadata_Security_Information,
	 "Informasi_Keamanan_Metadata"},
	{WMetadata_Standard_Name,
	 "Nama_Standar_Metadata"},
	{WMetadata_Standard_Version,
	 "Versi_Standar_Metadata"},
	{WMetadata_Time_Convention,
	 "Waktu_Ketentuan_Metadata"},
	{WMetadata_Use_Constraints,
	 "Batasan_Penggunaan_Metadata"},
	{WMiller_Cylindrical,
	 "Miller_Cylindrical"},
	{WModified_Stereographic_for_Alaska,
	 "Modified_Stereographic_for_Alaska"},
	{WMultiple_Dates_Times,
	 "Multi_Waktu"},
	{WNative_Data_Set_Environment,
	 "Lingkup_Pengolahan_Data"},
	{WNetwork_Address,
	 "Alamat_Network"},
	{WNetwork_Resource_Name,
	 "Nama_Sumber_Network"},
	{WNon_digital_Form,
	 "Bentuk_Non_Dijital"},
	{WNorth_Bounding_Coordinate,
	 "Koordinat_Batas_Utara"},
	{WNumber_DataBits,
	 "Number_DataBits"},
	{WNumber_StopBits,
	 "Number_StopBits"},
	{WOblique_Line_Azimuth,
	 "Oblique_Line_Azimuth"},
	{WOblique_Line_Latitude,
	 "Oblique_Line_Latitude"},
	{WOblique_Line_Longitude,
	 "Oblique_Line_Longitude"},
	{WOblique_Line_Point,
	 "Oblique_Line_Point"},
	{WOblique_Mercator,
	 "Oblique_Mercator"},
	{WOffline_Media,
	 "Media_Offline"},
	{WOffline_Option,
	 "Pilihan_Offline"},
	{WOnline_Computer_and_Operating_System,
	 "Online_Komputer_dan_Sistim_Operasi"},
	{WOnline_Linkage,
	 "Jaringan_Online"},
	{WOnline_Option,
	 "Pilihan_Online"},
	{WOrdering_Instructions,
	 "Instruksi_Pemesanan"},
	{WOrdinate_Resolution,
	 "Resolusi_Ordinat"},
	{WOriginator,
	 "Pemilik_Data"},
	{WOrthographic,
	 "Orthographi"},
	{WOther_Citation_Details,
	 "Detil_Sitasi_Lainnya"},
	{WOther_Grid_Systems_Definition,
	 "Definisi_Sistim_Grid_Lainnya"},
	{WOther_Projections_Definition,
	 "Definisi_Proyeksi_Lainnya"},
	{WOverview_Description,
	 "Diskripsi_Gambaran_Umum"},
	{WParity,
	 "Parity"},
	{WPath_Number,
	 "Nomor_Path"},
	{WPlace,
	 "Tempat"},
	{WPlace_Keyword,
	 "Kata_Kunci_Tempat"},
	{WPlace_Keyword_Thesaurus,
	 "Kata_Kunci_Tempat_Thesaurus"},
	{WPlanar,
	 "Planar"},
	{WPlanar_Coordinate_Encoding_Method,
	 "Metode_Pengkodean_Koordinat_Planar"},
	{WPlanar_Coordinate_Information,
	 "Informasi_Koordinat_Planar"},
	{WPlanar_Distance_Units,
	 "Satuan_Jarak_Planar"},
	{WPoint_and_Vector_Object_Count,
	 "Jumlah_Obyek_Titik_dan_Vektor"},
	{WPoint_and_Vector_Object_Information,
	 "Informasi_Obyek_Titik_dan_Vektor"},
	{WPoint_of_Contact,
	 "Titik_Kontak"},
	{WPolar_Stereographic,
	 "Stereografi_Polar"},
	{WPolyconic,
	 "Polyconic"},
	{WPositional_Accuracy,
	 "Ketelitian_Posisi"},
	{WPostal_Code,
	 "Kode_Pos"},
	{WProcess_Contact,
	 "Kontak_Proses"},
	{WProcess_Date,
	 "Tanggal_Proses"},
	{WProcess_Description,
	 "Diskripsi_Kegiatan"},
	{WProcess_Step,
	 "Tahap_Kegiatan"},
	{WProcess_Time,
	 "Waktu_Kegiatan"},
	{WProfile_Name,
	 "Profile_Name"},
	{WProgress,
	 "Kemajuan_Data"},
	{WPublication_Date,
	 "Tanggal_Penerbitan"},
	{WPublication_Information,
	 "Informasi_Penerbitan"},
	{WPublication_Place,
	 "Tempat_Penerbitan"},
	{WPublication_Time,
	 "Waktu_Penerbitan"},
	{WPublisher,
	 "Penerbit"},
	{WPurpose,
	 "Tujuan"},
	{WQuantitative_Attribute_Accuracy_Assessment,
	 "Kajian_Kuantitatif_Ketelitian_Atribut"},
	{WQuantitative_Horizontal_Positional_Accuracy_Assessment,
	 "Kajian_Kuantitatif_Ketelitian_Posisi_Horizontal"},
	{WQuantitative_Vertical_Positional_Accuracy_Assessment,
	 "Kajian_Kuantitatif_Ketelitian_Posisi_Vertikal"},
	{WRange_Domain,
	 "Interval"},
	{WRange_Domain_Maximum,
	 "Kisaran_Maximum"},
	{WRange_Domain_Minimum,
	 "Kisaran_Minimum"},
	{WRange_of_Dates_Times,
	 "Kisaran_Waktu"},
	{WRaster_Object_Information,
	 "Informasi_Obyek_Raster"},
	{WRaster_Object_Type,
	 "Tipe_Obyek_Raster"},
	{WRecording_Capacity,
	 "Kapasitas_Rekaman"},
	{WRecording_Density,
	 "Kepadatan_Rekaman"},
	{WRecording_Density_Units,
	 "Satuan_Kepadatan_Rekaman"},
	{WRecording_Format,
	 "Format_Rekaman"},
	{WResource_Description,
	 "Diskripsi_Sumber"},
	{WRobinson,
	 "Robinson"},
	{WRow_Count,
	 "Jumlah_Baris"},
	{WScale_Factor_at_Center_Line,
	 "Faktor_Skala_pada_Garis_Pusat"},
	{WScale_Factor_at_Central_Meridian,
	 "Faktor_Skala_pada_Pusat_Meridian"},
	{WScale_Factor_at_Equator,
	 "Faktor_Skala_di_Ekuator"},
	{WScale_Factor_at_Projection_Origin,
	 "Faktor_Skala_Pada_Proyeksi_Asli"},
	{WSDTS_Point_and_Vector_Object_Type,
	 "SDTS_Point_and_Vector_Object_Type"},
	{WSDTS_Terms_Description,
	 "SDTS_Terms_Description"},
	{WSecurity_Classification,
	 "Klasifikasi_Keamanan"},
	{WSecurity_Classification_System,
	 "Sistim_Klasifikasi_Keamanan"},
	{WSecurity_Handling_Description,
	 "Diskripsi_Penanganan_Keamanan"},
	{WSecurity_Information,
	 "Informasi_Keamanan"},
	{WSemi_major_Axis,
	 "Sumbu_Semi_Mayor"},
	{WSeries_Information,
	 "Informasi_Serial"},
	{WSeries_Name,
	 "Nama_Serial"},
	{WSingle_Date_Time,
	 "Tanggal/Waktu_Tunggal"},
	{WSinusoidal,
	 "Sinusoidal"},
	{WSource_Citation,
	 "Sumber_Sitasi"},
	{WSource_Citation_Abbreviation,
	 "Singkatan_Sumber_Sitasi"},
	{WSource_Contribution,
	 "Kontribusi_Sumber"},
	{WSource_Currentness_Reference,
	 "Sumber_Acuan_Terbaru"},
	{WSource_Information,
	 "Informasi_Sumber"},
	{WSource_Produced_Citation_Abbreviation,
	 "Singkatan_Sumber_Produk_Sitasi"},
	{WSource_Scale_Denominator,
	 "Skala_Sumber_Data"},
	{WSource_Time_Period_of_Content,
	 "Periode_Waktu_Pengambilan_Data_Sumber"},
	{WSource_Used_Citation_Abbreviation,
	 "Singkatan_Sumber_Sitasi_yang_Digunakan"},
	{WSouth_Bounding_Coordinate,
	 "Koordinat_Batas_Selatan"},
	{WSpace_Oblique_Mercator_Landsat,
	 "Space_Oblique_Mercator_Landsat"},
	{WSpatial_Data_Organization_Information,
	 "Informasi_Organisasi_Data_Spasial"},
	{WSpatial_Domain,
	 "Letak_Geografis"},
	{WSpatial_Reference_Information,
	 "Informasi_Acuan_Spasial"},
	{WSPCS_Zone_Identifier,
	 "Identitas_Zone_SPCS"},
	{WStandard_Order_Process,
	 "Standar_Proses_Pemesanan"},
	{WStandard_Parallel,
	 "Standard_Paralel"},
	{WState_or_Province,
	 "Propinsi"},
	{WState_Plane_Coordinate_System,
	 "Sistim_Koordinat_State_Plane"},
	{WStatus,
	 "Status"},
	{WStereographic,
	 "Stereografi"},
	{WStraight_Vertical_Longitude_from_Pole,
	 "Straight-Vertical_Longitude_from_Pole"},
	{WStratum,
	 "Strata"},
	{WStratum_Keyword,
	 "Kata_Kunci_Strata"},
	{WStratum_Keyword_Thesaurus,
	 "Kata_Kunci_Strata_Thesaurus"},
	{WSupplemental_Information,
	 "Informasi_Tambahan"},
	{WTechnical_Prerequisites,
	 "Persyaratan_Teknis"},
	{WTemporal,
	 "Waktu"},
	{WTemporal_Keyword,
	 "Kata_Kunci_Waktu"},
	{WTemporal_Keyword_Thesaurus,
	 "Kata_Kunci_Waktu_Thesaurus"},
	{WTheme,
	 "Tema"},
	{WTheme_Keyword,
	 "Kata_Kunci_Tema"},
	{WTheme_Keyword_Thesaurus,
	 "Kata_Kunci_Tema_Thesaurus"},
	{WTime_of_Day,
	 "Hari"},
	{WTime_Period_Information,
	 "Informasi_Waktu"},
	{WTime_Period_of_Content,
	 "Waktu_Pengambilan_Data"},
	{WTitle,
	 "Judul"},
	{WTransfer_Size,
	 "Ukuran_Transfer"},
	{WTransverse_Mercator,
	 "Transverse_Mercator"},
	{WTurnaround,
	 "Turnaround"},
	{WType_of_Source_Media,
	 "Media_Sumber_Data"},
	{WUniversal_Polar_Stereographic,
	 "Universal_Polar_Stereographic"},
	{WUniversal_Transverse_Mercator,
	 "Universal_Transverse_Mercator"},
	{WUnrepresentable_Domain,
	 "Unrepresentable_Domain"},
	{WUPS_Zone_Identifier,
	 "Identitas_Zone_UPS"},
	{WUse_Constraints,
	 "Batasan_Penggunaan_Data"},
	{WUTM_Zone_Number,
	 "Nomor_Zone_UTM"},
	{Wvan_der_Grinten,
	 "van_der_Grinten"},
	{WVertical_Coordinate_System_Definition,
	 "Definisi_Sistim_Koordinat_Vertikal"},
	{WVertical_Count,
	 "Jumlah_Vertikal"},
	{WVertical_Positional_Accuracy,
	 "Ketelitian_Posisi_Vertikal"},
	{WVertical_Positional_Accuracy_Explanation,
	 "Keterangan_Ketelitian_Posisi_Vertikal"},
	{WVertical_Positional_Accuracy_Report,
	 "Laporan_Ketelitian_Posisi_Vertikal"},
	{WVertical_Positional_Accuracy_Value,
	 "Nilai_Ketelitian_Posisi_Vertikal"},
	{WVPF_Point_and_Vector_Object_Information,
	 "VPF_Point_and_Vector_Object_Information"},
	{WVPF_Point_and_Vector_Object_Type,
	 "Tipe_Obyek_Vektor_dan_Titik_VPF"},
	{WVPF_Terms_Description,
	 "VPF_Terms_Description"},
	{WVPF_Topology_Level,
	 "Tingkat_Topologi_VPF"},
	{WWest_Bounding_Coordinate,
	 "Koordinat_Batas_Barat"},

/* Biological Data Profile */

	{WDescription_of_Geographic_Extent,
		"Description_of_Geographic_Extent"},
	{WBounding_Altitudes,
		"Bounding_Altitudes"},
	{WAltitude_Minimum,
		"Altitude_Minimum"},
	{WAltitude_Maximum,
		"Altitude_Maximum"},
	{WTaxonomy,
		"Taxonomy"},
	{WKeywords_Taxon,
		"Keywords/Taxon"},
	{WTaxonomic_Keyword_Thesaurus,
		"Taxonomic_Keyword_Thesaurus"},
	{WTaxonomic_Keywords,
		"Taxonomic_Keywords"},
	{WTaxonomic_System,
		"Taxonomic_System"},
	{WClassification_System_Authority,
		"Classification_System/Authority"},
	{WClassification_System_Citation,
		"Classification_System_Citation"},
	{WClassification_System_Modifications,
		"Classification_System_Modifications"},
	{WIdentification_Reference,
		"Identification_Reference"},
	{WIdentifier,
		"Identifier"},
	{WTaxonomic_Procedures,
		"Taxonomic_Procedures"},
	{WTaxonomic_Completeness,
		"Taxonomic_Completeness"},
	{WVouchers,
		"Vouchers"},
	{WSpecimen,
		"Specimen"},
	{WRepository,
		"Repository"},
	{WGeneral_Taxonomic_Coverage,
		"General_Taxonomic_Coverage"},
	{WTaxonomic_Classification,
		"Taxonomic_Classification"},
	{WTaxon_Rank_Name,
		"Taxon_Rank_Name"},
	{WTaxon_Rank_Value,
		"Taxon_Rank_Value"},
	{WApplicable_Common_Name,
		"Applicable_Common_Name"},
	{WAnalytical_Tool,
		"Analytical_Tool"},
	{WAnalytical_Tool_Description,
		"Analytical_Tool_Description"},
	{WTool_Access_Information,
		"Tool_Access_Information"},
	{WTool_Contact,
		"Tool_Contact"},
	{WTool_Citation,
		"Tool_Citation"},
	{WTool_Access_Instructions,
		"Tool_Access_Instructions"},
	{WTool_Computer_and_Operating_System,
		"Tool_Computer_and_Operating_System"},
	{WMethodology,
		"Methodology"},
	{WMethodology_Type,
		"Methodology_Type"},
	{WMethodology_Identifier,
		"Methodology_Identifier"},
	{WMethodology_Keyword_Thesaurus,
		"Methodology_Keyword_Thesaurus"},
	{WMethodology_Keyword,
		"Methodology_Keyword"},
	{WMethodology_Description,
		"Methodology_Description"},
	{WMethodology_Citation,
		"Methodology_Citation"},
	{WASCII_File_Structure,
		"ASCII_File_Structure"},
	{WRecord_Delimiter,
		"Record_Delimiter"},
	{WNumber_Header_Lines,
		"Number_Header_Lines"},
	{WDescription_of_Header_Content,
		"Description_of_Header_Content"},
	{WOrientation,
		"Orientation"},
	{WCase_Sensitive,
		"Case_Sensitive"},
	{WAuthentication,
		"Authentication"},
	{WQuote_Character,
		"Quote_Character"},
	{WData_Field,
		"Data_Field"},
	{WData_Field_Name,
		"Data_Field_Name"},
	{WMissing_Value_Code,
		"Missing_Value_Code"},
	{WData_Field_Width_Delimiter,
		"Data_Field_Width_Delimiter"},
	{WData_Field_Width,
		"Data_Field_Width"},
	{WBeginning_Geologic_Age,
		"Beginning_Geologic_Age"},
	{WEnding_Geologic_Age,
		"Ending_Geologic_Age"},
	{WGeologic_Age,
		"Geologic_Age"},
	{WGeologic_Time_Scale,
		"Geologic_Time_Scale"},
	{WGeologic_Age_Estimate,
		"Geologic_Age_Estimate"},
	{WGeologic_Age_Uncertainty,
		"Geologic_Age_Uncertainty"},
	{WGeologic_Age_Explanation,
		"Geologic_Age_Explanation"},
	{WGeologic_Citation,
		"Geologic_Citation"},

/* Shoreline profile */

	{WBarometric_Pressure,
	 "Barometric_Pressure"},
	{WEnvironmental_Event,
	 "Environmental_Event"},
	{WMarine_Weather_Condition,
	 "Marine_Weather_Condition"},
	{WProcess_Step_Citation,
	 "Process_Step_Citation"},
	{WRange_of_Tide,
	 "Range_of_Tide"},
	{WSupplemental_Tidal_Information,
	 "Supplemental_Tidal_Information"},
	{WTidal_Datum,
	 "Tidal_Datum"},
	{WTidal_Information,
	 "Tidal_Information"},
	{WTide_Table_Reference,
	 "Tide_Table_Reference"},
	{WTime_of_High_Tide,
	 "Time_of_High_Tide"},
	{WTime_of_Low_Tide,
	 "Time_of_Low_Tide"},
	{WTime_of_Tide,
	 "Time_of_Tide"},
	{WType_of_Tide,
	 "Type_of_Tide"},
	{WWave_Height,
	 "Wave_Height"},
	{WWind_Direction,
	 "Wind_Direction"},
	{WWind_Speed,
	 "Wind_Speed"},

/* Remote-sensing profile */

	{WAcquisition_Information,
	 "Acquisition_Information"},
	{WAerial_Film_Speed,
	 "Aerial_Film_Speed"},
	{WAerotriangulation_Reference,
	 "Aerotriangulation_Reference"},
	{WAffine_Distortion_X_Prime_Coefficient,
	 "Affine_Distortion_X_Prime_Coefficient"},
	{WAffine_Distortion_Y_Prime_Coefficient,
	 "Affine_Distortion_Y_Prime_Coefficient"},
	{WAggregation_Criteria,
	 "Aggregation_Criteria"},
	{WAggregation_Information,
	 "Aggregation_Information"},
	{WAggregation_Member_ID,
	 "Aggregation_Member_ID"},
	{WAlgorithm_Change_Description,
	 "Algorithm_Change_Description"},
	{WAlgorithm_Change_History,
	 "Algorithm_Change_History"},
	{WAlgorithm_Change_Reference,
	 "Algorithm_Change_Reference"},
	{WAlgorithm_Description,
	 "Algorithm_Description"},
	{WAlgorithm_Identifiers,
	 "Algorithm_Identifiers"},
	{WAlgorithm_Information,
	 "Algorithm_Information"},
	{WAlgorithm_Peer_Review_Description,
	 "Algorithm_Peer_Review_Description"},
	{WAlgorithm_Peer_Review_Information,
	 "Algorithm_Peer_Review_Information"},
	{WAlgorithm_Peer_Review_Reference,
	 "Algorithm_Peer_Review_Reference"},
	{WAlgorithm_Reference,
	 "Algorithm_Reference"},
	{WAlgorithm_Text_Description,
	 "Algorithm_Text_Description"},
	{WAltitude_Distance_Layer,
	 "Altitude_Distance_Layer"},
	{WAncillary_Dataset,
	 "Ancillary_Dataset"},
	{WAncillary_Dataset_Description,
	 "Ancillary_Dataset_Description"},
	{WAncillary_Dataset_Identifier,
	 "Ancillary_Dataset_Identifier"},
	{WAncillary_Description,
	 "Ancillary_Description"},
	{WAncillary_Reference,
	 "Ancillary_Reference"},
	{WAngle_Dependent_Distortion,
	 "Angle_Dependent_Distortion"},
	{WAngle_Distortion_Value,
	 "Angle_Distortion_Value"},
	{WArea_Weighted_Average_Resolution,
	 "Area_Weighted_Average_Resolution"},
	{WArgument_of_Perigee,
	 "Argument_of_Perigee"},
	{WAscending_Descending_Indicator,
	 "Ascending/Descending_Indicator"},
	{WAttitude_Angular_Units,
	 "Attitude_Angular_Units"},
	{WAvailability_of_Element_Locations,
	 "Availability_of_Element_Locations"},
	{WAxes,
	 "Axes"},
	{WAxis_Rotation_Convention,
	 "Axis_Rotation_Convention"},
	{WBand_Boundary_Definition,
	 "Band_Boundary_Definition"},
	{WBand_Description,
	 "Band_Description"},
	{WBand_ID,
	 "Band_ID"},
	{WBand_Identification,
	 "Band_Identification"},
	{WBand_Measurement_Mode_ID,
	 "Band_Measurement_Mode_ID"},
	{WBand_Quality,
	 "Band_Quality"},
	{WCalibrated_Detector_Positions,
	 "Calibrated_Detector_Positions"},
	{WCalibrated_Focal_Length,
	 "Calibrated_Focal_Length"},
	{WCamera_Identifier,
	 "Camera_Identifier"},
	{WCamera_Type,
	 "Camera_Type"},
	{WCell_Value_Type,
	 "Cell_Value_Type"},
	{WClock_Time_Drift,
	 "Clock_Time/Drift"},
	{WCollection_Type,
	 "Collection_Type"},
	{WColumn_Delta_X,
	 "Column_Delta_X"},
	{WColumn_Delta_Y,
	 "Column_Delta_Y"},
	{WCommand_Line_Processing_Parameter,
	 "Command_Line_Processing_Parameter"},
	{WComponent_Information,
	 "Component_Information"},
	{WContainer_Packet_ID,
	 "Container_Packet_ID"},
	{WControl_Point_Authority,
	 "Control_Point_Authority"},
	{WControl_Point_Column,
	 "Control_Point_Column"},
	{WControl_Point_Earth_Location,
	 "Control_Point_Earth_Location"},
	{WControl_Point_Horizontal_X_Accuracy,
	 "Control_Point_Horizontal_X_Accuracy"},
	{WControl_Point_Horizontal_Y_Accuracy,
	 "Control_Point_Horizontal_Y_Accuracy"},
	{WControl_Point_ID,
	 "Control_Point_ID"},
	{WControl_Point_Identification,
	 "Control_Point_Identification"},
	{WControl_Point_Origin,
	 "Control_Point_Origin"},
	{WControl_Point_Raster_Position,
	 "Control_Point_Raster_Position"},
	{WControl_Point_Row,
	 "Control_Point_Row"},
	{WControl_Point_Type,
	 "Control_Point_Type"},
	{WControl_Point_Use_Flag,
	 "Control_Point_Use_Flag"},
	{WControl_Point_Vertical_Accuracy,
	 "Control_Point_Vertical_Accuracy"},
	{WControl_Point_x_Value,
	 "Control_Point_x_Value"},
	{WControl_Point_y_Value,
	 "Control_Point_y_Value"},
	{WControl_Point_z_Value,
	 "Control_Point_z_Value"},
	{WCoordinate_Point,
	 "Coordinate_Point"},
	{WCoordinate_System,
	 "Coordinate_System"},
	{WCoordinate_x_Value,
	 "Coordinate_x_Value"},
	{WCoordinate_XY_Units,
	 "Coordinate_XY_Units"},
	{WCoordinate_y_Value,
	 "Coordinate_y_Value"},
	{WCoordinate_Z_Units,
	 "Coordinate_Z_Units"},
	{WCoordinate_z_Value,
	 "Coordinate_z_Value"},
	{WCross_Track_Axis,
	 "Cross_Track_Axis"},
	{WCross_Track_Description,
	 "Cross_Track_Description"},
	{WCross_Track_Direction,
	 "Cross_Track_Direction"},
	{WCross_Track_Extent_Angle,
	 "Cross_Track_Extent_Angle"},
	{WCross_Track_Fixed_Angle,
	 "Cross_Track_Fixed_Angle"},
	{WCross_Track_Motion,
	 "Cross_Track_Motion"},
	{WCross_Track_Start_Angle,
	 "Cross_Track_Start_Angle"},
	{WCross_Track_Step_Angle,
	 "Cross_Track_Step_Angle"},
	{WCross_Track_Sweep,
	 "Cross_Track_Sweep"},
	{WCross_Track_Zero,
	 "Cross_Track_Zero"},
	{WData_Dictionary_Reference,
	 "Data_Dictionary_Reference"},
	{WData_Scaling_Information,
	 "Data_Scaling_Information"},
	{WDataset_Identifier,
	 "Dataset_Identifier"},
	{WDate_of_Last_Calibration,
	 "Date_of_Last_Calibration"},
	{WDeveloping_Institution,
	 "Developing_Institution"},
	{WDimension_Count,
	 "Dimension_Count"},
	{WDimension_Description,
	 "Dimension_Description"},
	{WDimension_Properties,
	 "Dimension_Properties"},
	{WDistance_Dependent_Distortion,
	 "Distance_Dependent_Distortion"},
	{WDistance_Distortion_Value,
	 "Distance_Distortion_Value"},
	{WDistortion,
	 "Distortion"},
	{WDistortion_Type_Affine,
	 "Distortion_Type_Affine"},
	{WDistortion_Type_Radial_Asymmetrical,
	 "Distortion_Type_Radial_Asymmetrical"},
	{WDistortion_Type_Radial_Symmetrical,
	 "Distortion_Type_Radial_Symmetrical"},
	{WDocumentation,
	 "Documentation"},
	{WEccentricity,
	 "Eccentricity"},
	{WEffective_Aerial_Film_Speed,
	 "Effective_Aerial_Film_Speed"},
	{WElevation_Axis,
	 "Elevation_Axis"},
	{WElevation_Description,
	 "Elevation_Description"},
	{WElevation_Direction,
	 "Elevation_Direction"},
	{WElevation_Extent_Angle,
	 "Elevation_Extent_Angle"},
	{WElevation_Fixed_Angle,
	 "Elevation_Fixed_Angle"},
	{WElevation_Start_Angle,
	 "Elevation_Start_Angle"},
	{WElevation_Step_Angle,
	 "Elevation_Step_Angle"},
	{WElevation_Sweep,
	 "Elevation_Sweep"},
	{WElevation_Zero,
	 "Elevation_Zero"},
	{WEphemeris,
	 "Ephemeris"},
	{WExposure_Time,
	 "Exposure_Time"},
	{WExterior_Orientation_Accuracy,
	 "Exterior_Orientation_Accuracy"},
	{WFiducial,
	 "Fiducial"},
	{WFiducial_Center,
	 "Fiducial_Center"},
	{WFilm_Type,
	 "Film_Type"},
	{WFilter_on_Camera,
	 "Filter_on_Camera"},
	{WFilter_on_Camera_Indicator,
	 "Filter_on_Camera_Indicator"},
	{WFilter_Type,
	 "Filter_Type"},
	{WFlight_Protocol,
	 "Flight_Protocol"},
	{WFlying_Height,
	 "Flying_Height"},
	{WForward_Motion_Compensation,
	 "Forward_Motion_Compensation"},
	{WFrame_Area,
	 "Frame_Area"},
	{WFrame_Area_Units,
	 "Frame_Area_Units"},
	{WFrame_Area_Value,
	 "Frame_Area_Value"},
	{WFrame_Camera,
	 "Frame_Camera"},
	{WFrame_Geometric_Properties,
	 "Frame_Geometric_Properties"},
	{WFrame_Hardware,
	 "Frame_Hardware"},
	{WFrame_Operation,
	 "Frame_Operation"},
	{WFrame_Optics,
	 "Frame_Optics"},
	{WFrame_Radiometric_Calibration,
	 "Frame_Radiometric_Calibration"},
	{WFrame_Radiometric_Properties,
	 "Frame_Radiometric_Properties"},
	{WFrame_Spectral_Information,
	 "Frame_Spectral_Information"},
	{WFrame_Spectral_Properties,
	 "Frame_Spectral_Properties"},
	{WGeorectified_Raster,
	 "Georectified_Raster"},
	{WGeoreferenceable_Raster,
	 "Georeferenceable_Raster"},
	{WGeoreferencing_Description,
	 "Georeferencing_Description"},
	{WGeoreferencing_Information,
	 "Georeferencing_Information"},
	{WGPS_Information_System_Availability,
	 "GPS_Information_System_Availability"},
	{WGrid_First_Element,
	 "Grid_First_Element"},
	{WGrid_First_Element_Map_X_Coordinate,
	 "Grid_First_Element_Map_X_Coordinate"},
	{WGrid_First_Element_Map_Y_Coordinate,
	 "Grid_First_Element_Map_Y_Coordinate"},
	{WGrid_Orientation,
	 "Grid_Orientation"},
	{WGround_Control_Point_Description,
	 "Ground_Control_Point_Description"},
	{WGround_Control_Point_Information,
	 "Ground_Control_Point_Information"},
	{WGround_Control_Point_Organization,
	 "Ground_Control_Point_Organization"},
	{WGround_Control_Point_Position,
	 "Ground_Control_Point_Position"},
	{WGround_Shape,
	 "Ground_Shape"},
	{WID_Overlapping_Dataset,
	 "ID_Overlapping_Dataset"},
	{WIFOV_Units,
	 "IFOV_Units"},
	{WIFOV_x_Definition,
	 "IFOV_x_Definition"},
	{WIFOV_x_Value,
	 "IFOV_x_Value"},
	{WIFOV_y_Definition,
	 "IFOV_y_Definition"},
	{WIFOV_y_Value,
	 "IFOV_y_Value"},
	{WImage_Quality,
	 "Image_Quality"},
	{WImage_Size,
	 "Image_Size"},
	{WImage_Size_x_Value,
	 "Image_Size_x_Value"},
	{WImage_Size_y_Value,
	 "Image_Size_y_Value"},
	{WInclination,
	 "Inclination"},
	{WIndividual_Band_Identification,
	 "Individual_Band_Identification"},
	{WInput_Dataset_Identifier,
	 "Input_Dataset_Identifier"},
	{WInput_Description,
	 "Input_Description"},
	{WInput_Level,
	 "Input_Level"},
	{WInput_Reference,
	 "Input_Reference"},
	{WINS_Reading_Availability,
	 "INS_Reading_Availability"},
	{WInstantaneous_Field_of_View,
	 "Instantaneous_Field_of_View"},
	{WInstitution_of_Last_Calibration,
	 "Institution_of_Last_Calibration"},
	{WInstrument_Description,
	 "Instrument_Description"},
	{WInstrument_Full_Name,
	 "Instrument_Full_Name"},
	{WInstrument_Information,
	 "Instrument_Information"},
	{WInstrument_Properties_Description,
	 "Instrument_Properties_Description"},
	{WInstrument_Reference,
	 "Instrument_Reference"},
	{WInstrument_Short_Name,
	 "Instrument_Short_Name"},
	{WInstrument_Specific_Georeferencing,
	 "Instrument_Specific_Georeferencing"},
	{WInstrument_Type,
	 "Instrument_Type"},
	{WKeplerian_Orbit,
	 "Keplerian_Orbit"},
	{WLast_Calibration,
	 "Last_Calibration"},
	{WLayer_Name,
	 "Layer_Name"},
	{WLens,
	 "Lens"},
	{WLens_Identifier,
	 "Lens_Identifier"},
	{WLens_Type,
	 "Lens_Type"},
	{WLight_Drop,
	 "Light_Drop"},
	{WLocation_Information,
	 "Location_Information"},
	{WMagazine_Identifier,
	 "Magazine_Identifier"},
	{WMaximum_Wavelength,
	 "Maximum_Wavelength"},
	{WMethod_of_Last_Calibration,
	 "Method_of_Last_Calibration"},
	{WMinimum_Wavelength,
	 "Minimum_Wavelength"},
	{WMission_Completion,
	 "Mission_Completion"},
	{WMission_Description,
	 "Mission_Description"},
	{WMission_History,
	 "Mission_History"},
	{WMission_Information,
	 "Mission_Information"},
	{WMission_Name,
	 "Mission_Name"},
	{WMission_Significant_Event,
	 "Mission_Significant_Event"},
	{WMission_Start_Date,
	 "Mission_Start_Date"},
	{WMultiple_Image_Alignment,
	 "Multiple_Image_Alignment"},
	{WNadir,
	 "Nadir"},
	{WNadir_Latitude,
	 "Nadir_Latitude"},
	{WNadir_Longitude,
	 "Nadir_Longitude"},
	{WName_of_Dimension,
	 "Name_of_Dimension"},
	{WNominal_Geostationary_Position,
	 "Nominal_Geostationary_Position"},
	{WNominal_Spatial_Resolution,
	 "Nominal_Spatial_Resolution"},
	{WNon_Polynomial_Scaling,
	 "Non_Polynomial_Scaling"},
	{WNumber_of_Angle_Distortion_Values,
	 "Number_of_Angle_Distortion_Values"},
	{WNumber_of_Bands,
	 "Number_of_Bands"},
	{WNumber_of_Cross_Track_Samples,
	 "Number_of_Cross_Track_Samples"},
	{WNumber_of_Data_Dimensions,
	 "Number_of_Data_Dimensions"},
	{WNumber_of_Denominator_Terms,
	 "Number_of_Denominator_Terms"},
	{WNumber_of_Distance_Distortion_Values,
	 "Number_of_Distance_Distortion_Values"},
	{WNumber_of_Elevation_Samples,
	 "Number_of_Elevation_Samples"},
	{WNumber_of_Numerator_Terms,
	 "Number_of_Numerator_Terms"},
	{WNumber_of_Points,
	 "Number_of_Points"},
	{WNumber_of_Profile_Samples,
	 "Number_of_Profile_Samples"},
	{WNumber_of_Resolution_Values,
	 "Number_of_Resolution_Values"},
	{WNumber_of_Thematic_Layers,
	 "Number_of_Thematic_Layers"},
	{WNumber_of_Wavelength_Bands,
	 "Number_of_Wavelength_Bands"},
	{WOperational_Mode,
	 "Operational_Mode"},
	{WOrbit_Angle_Units,
	 "Orbit_Angle_Units"},
	{WOrbit_Period,
	 "Orbit_Period"},
	{WOrbit_Period_Units,
	 "Orbit_Period_Units"},
	{WOrbit_Period_Value,
	 "Orbit_Period_Value"},
	{WOther_Collector_Description,
	 "Other_Collector_Description"},
	{WOther_Georeferencing_Description,
	 "Other_Georeferencing_Description"},
	{WPath,
	 "Path"},
	{WPeak_Wavelength,
	 "Peak_Wavelength"},
	{WPercentage_of_Overlap,
	 "Percentage_of_Overlap"},
	{WPerigee_Passage_Time,
	 "Perigee_Passage_Time"},
	{WPhotographic_Resolving_Power,
	 "Photographic_Resolving_Power"},
	{WPitch,
	 "Pitch"},
	{WPitch_Accuracy,
	 "Pitch_Accuracy"},
	{WPixel_Cross_Track_Size,
	 "Pixel_Cross_Track_Size"},
	{WPixel_Description,
	 "Pixel_Description"},
	{WPixel_Elevation_Size,
	 "Pixel_Elevation_Size"},
	{WPixel_Height_Above_Ellipsoid,
	 "Pixel_Height_Above_Ellipsoid"},
	{WPixel_Point_Spread_Function,
	 "Pixel_Point_Spread_Function"},
	{WPixel_Profile_Size,
	 "Pixel_Profile_Size"},
	{WPixel_Resolution,
	 "Pixel_Resolution"},
	{WPlatform_and_Instrument_Identification,
	 "Platform_and_Instrument_Identification"},
	{WPlatform_and_Mission_Information,
	 "Platform_and_Mission_Information"},
	{WPlatform_Description,
	 "Platform_Description"},
	{WPlatform_Full_Name,
	 "Platform_Full_Name"},
	{WPlatform_Information,
	 "Platform_Information"},
	{WPlatform_Nominal_Altitude,
	 "Platform_Nominal_Altitude"},
	{WPlatform_Nominal_Altitude_Units,
	 "Platform_Nominal_Altitude_Units"},
	{WPlatform_Nominal_Altitude_Value,
	 "Platform_Nominal_Altitude_Value"},
	{WPlatform_Nominal_Longitude,
	 "Platform_Nominal_Longitude"},
	{WPlatform_Orbit,
	 "Platform_Orbit"},
	{WPlatform_Serial_Identifier,
	 "Platform_Serial_Identifier"},
	{WPlatform_Short_Name,
	 "Platform_Short_Name"},
	{WPlatform_Sponsor,
	 "Platform_Sponsor"},
	{WPlatform_Start_Date,
	 "Platform_Start_Date"},
	{WPoint_Position_In_Pixel,
	 "Point_Position_In_Pixel"},
	{WPolarization_Characteristics,
	 "Polarization_Characteristics"},
	{WPolynomial_Coefficient,
	 "Polynomial_Coefficient"},
	{WPolynomial_Denominator,
	 "Polynomial_Denominator"},
	{WPolynomial_Function,
	 "Polynomial_Function"},
	{WPolynomial_Numerator,
	 "Polynomial_Numerator"},
	{WPositional_Information,
	 "Positional_Information"},
	{WPrincipal_Point_of_Autocollimation,
	 "Principal_Point_of_Autocollimation"},
	{WPrincipal_Point_of_Symmetry,
	 "Principal_Point_of_Symmetry"},
	{WProcessing_Change_History,
	 "Processing_Change_History"},
	{WProcessing_Documentation,
	 "Processing_Documentation"},
	{WProcessing_Environment,
	 "Processing_Environment"},
	{WProcessing_Identifiers,
	 "Processing_Identifiers"},
	{WProcessing_Information,
	 "Processing_Information"},
	{WProcessing_Input_Dataset,
	 "Processing_Input_Dataset"},
	{WProcessing_Level,
	 "Processing_Level"},
	{WProcessing_Level_Authority,
	 "Processing_Level_Authority"},
	{WProcessing_Level_Identifier,
	 "Processing_Level_Identifier"},
	{WProcessing_Procedure,
	 "Processing_Procedure"},
	{WProcessing_Procedure_Description,
	 "Processing_Procedure_Description"},
	{WProcessing_Run_History,
	 "Processing_Run_History"},
	{WProcessing_Software,
	 "Processing_Software"},
	{WProcessing_Software_Description,
	 "Processing_Software_Description"},
	{WProcessing_Software_Reference,
	 "Processing_Software_Reference"},
	{WProfile_Description,
	 "Profile_Description"},
	{WProfile_Extent,
	 "Profile_Extent"},
	{WProfile_Fixed,
	 "Profile_Fixed"},
	{WProfile_Properties,
	 "Profile_Properties"},
	{WProfile_Sounding,
	 "Profile_Sounding"},
	{WProfile_Start,
	 "Profile_Start"},
	{WProfile_Step,
	 "Profile_Step"},
	{WProfiling_Direction,
	 "Profiling_Direction"},
	{WProjection_Center_X_Position,
	 "Projection_Center_X_Position"},
	{WProjection_Center_Y_Position,
	 "Projection_Center_Y_Position"},
	{WProjection_Center_Z_Position,
	 "Projection_Center_Z_Position"},
	{WQuality_of_Autocollimation_Principal_Point_x_Value,
	 "Quality_of_Autocollimation_Principal_Point_x_Value"},
	{WQuality_of_Autocollimation_Principal_Point_y_Value,
	 "Quality_of_Autocollimation_Principal_Point_y_Value"},
	{WQuality_of_Focal_Length,
	 "Quality_of_Focal_Length"},
	{WQuality_of_Principal_Point_of_Autocollimation,
	 "Quality_of_Principal_Point_of_Autocollimation"},
	{WQuality_of_Principal_Point_of_Symmetry,
	 "Quality_of_Principal_Point_of_Symmetry"},
	{WQuality_of_Symmetry_Principal_Point_x_Value,
	 "Quality_of_Symmetry_Principal_Point_x_Value"},
	{WQuality_of_Symmetry_Principal_Point_y_Value,
	 "Quality_of_Symmetry_Principal_Point_y_Value"},
	{WRadial_Asymmetrical_Coefficient_B1,
	 "Radial_Asymmetrical_Coefficient_B1"},
	{WRadial_Asymmetrical_Coefficient_B2,
	 "Radial_Asymmetrical_Coefficient_B2"},
	{WRadial_Symmetrical_Angle_Interval,
	 "Radial_Symmetrical_Angle_Interval"},
	{WRadial_Symmetrical_Distance_Interval,
	 "Radial_Symmetrical_Distance_Interval"},
	{WRadial_Symmetrical_Distortion_Polynomial,
	 "Radial_Symmetrical_Distortion_Polynomial"},
	{WReceiver_Polarization,
	 "Receiver_Polarization"},
	{WReferenced_Coordinate_System,
	 "Referenced_Coordinate_System"},
	{WReferencing_Polynomial,
	 "Referencing_Polynomial"},
	{WRelative_Aperture,
	 "Relative_Aperture"},
	{WRelative_Azimuth,
	 "Relative_Azimuth"},
	{WReseau,
	 "Reseau"},
	{WResolution_Value_Set,
	 "Resolution_Value_Set"},
	{WResolving_Angle,
	 "Resolving_Angle"},
	{WResolving_Value_Radial,
	 "Resolving_Value_Radial"},
	{WResolving_Value_Tangential,
	 "Resolving_Value_Tangential"},
	{WRight_Ascension_of_Ascending_Node,
	 "Right_Ascension_of_Ascending_Node"},
	{WRoll,
	 "Roll"},
	{WRoll_Accuracy,
	 "Roll_Accuracy"},
	{WRotation_Description,
	 "Rotation_Description"},
	{WRotation_Sequence,
	 "Rotation_Sequence"},
	{WRow,
	 "Row"},
	{WRow_Delta_X,
	 "Row_Delta_X"},
	{WRow_Delta_Y,
	 "Row_Delta_Y"},
	{WSample_Angle_Units,
	 "Sample_Angle_Units"},
	{WSample_Description_Units,
	 "Sample_Description_Units"},
	{WSample_Profile_Units,
	 "Sample_Profile_Units"},
	{WSample_Properties,
	 "Sample_Properties"},
	{WSatellite_Local_Azimuth_Angle,
	 "Satellite/Local_Azimuth_Angle"},
	{WSatellite_Local_Zenith_Angle,
	 "Satellite/Local_Zenith_Angle"},
	{WScan,
	 "Scan"},
	{WScan_Angle_Units,
	 "Scan_Angle_Units"},
	{WScan_Cross_Track_Properties,
	 "Scan_Cross_Track_Properties"},
	{WScan_Distance_Units,
	 "Scan_Distance_Units"},
	{WScan_Duration,
	 "Scan_Duration"},
	{WScan_Elevation_Properties,
	 "Scan_Elevation_Properties"},
	{WScan_Geometric_Properties,
	 "Scan_Geometric_Properties"},
	{WScan_Radiometric_Properties,
	 "Scan_Radiometric_Properties"},
	{WScan_Repeat_Time,
	 "Scan_Repeat_Time"},
	{WScan_Spectral_Properties,
	 "Scan_Spectral_Properties"},
	{WScan_Start_Time,
	 "Scan_Start_Time"},
	{WScan_Step_Time,
	 "Scan_Step_Time"},
	{WScan_Time_Units,
	 "Scan_Time_Units"},
	{WScan_Timing,
	 "Scan_Timing"},
	{WScience_Paper,
	 "Science_Paper"},
	{WSemimajor_Axis,
	 "Semimajor_Axis"},
	{WSender_Polarization,
	 "Sender_Polarization"},
	{WSensor_Element_Location,
	 "Sensor_Element_Location"},
	{WSensor_Grid,
	 "Sensor_Grid"},
	{WSensor_Orientation,
	 "Sensor_Orientation"},
	{WSensor_System,
	 "Sensor_System"},
	{WSolar_Azimuth_Angle,
	 "Solar_Azimuth_Angle"},
	{WSolar_Zenith_Angle,
	 "Solar_Zenith_Angle"},
	{WSource_of_Element_Locations,
	 "Source_of_Element_Locations"},
	{WSpatial_Resolution_Units,
	 "Spatial_Resolution_Units"},
	{WSpatial_Resolution_Value,
	 "Spatial_Resolution_Value"},
	{WSpectral_Information,
	 "Spectral_Information"},
	{WSpectral_Limit,
	 "Spectral_Limit"},
	{WStabilized_Mount,
	 "Stabilized_Mount"},
	{WStorage_Order,
	 "Storage_Order"},
	{WSwath_Track_Information,
	 "Swath_Track_Information"},
	{WThematic_Layer_Identification,
	 "Thematic_Layer_Identification"},
	{WTranslation_Description,
	 "Translation_Description"},
	{WType_of_Overlap,
	 "Type_of_Overlap"},
	{WUnreferenced_Coordinate_System,
	 "Unreferenced_Coordinate_System"},
	{WUsers_Guide,
	 "User's_Guide"},
	{WWavelength_Band_Properties,
	 "Wavelength_Band_Properties"},
	{WWavelength_Region,
	 "Wavelength_Region"},
	{WWavelength_Units,
	 "Wavelength_Units"},
	{WWorldwide_Reference_System,
	 "Worldwide_Reference_System"},
	{WX_Axis_Definition,
	 "X_Axis_Definition"},
	{WX_Position_Accuracy,
	 "X_Position_Accuracy"},
	{WY_Axis_Definition,
	 "Y_Axis_Definition"},
	{WY_Position_Accuracy,
	 "Y_Position_Accuracy"},
	{WYaw,
	 "Yaw"},
	{WYaw_Accuracy,
	 "Yaw_Accuracy"},
	{WZ_Axis_Definition,
	 "Z_Axis_Definition"},
	{WZ_Position_Accuracy,
	 "Z_Position_Accuracy"},

#ifdef USE_ESRI_EXTENSIONS

	/* ESRI extensions */

	{WAltitude_Units,
	 "Altitude_Units"},
	{WAncillary_Role,
	 "Ancillary_Role"},
	{WAncillary_Role_Attribute,
	 "Ancillary_Role_Attribute"},
	{WAttribute_Alias,
	 "Attribute_Alias"},
	{WAttribute_Defined_Domain,
	 "Attribute_Defined_Domain"},
	{WAttribute_Indexed,
	 "Attribute_Indexed"},
	{WAttribute_Number_of_Decimals,
	 "Attribute_Number_of_Decimals"},
	{WAttribute_Output_Width,
	 "Attribute_Output_Width"},
	{WAttribute_Precision,
	 "Attribute_Precision"},
	{WAttribute_Scale,
	 "Attribute_Scale"},
	{WAttribute_Type,
	 "Attribute_Type"},
	{WAttribute_Width,
	 "Attribute_Width"},
	{WAttributed_Relationship,
	 "Attributed_Relationship"},
	{WAvailable_Junction_Feature_Class,
	 "Available_Junction_Feature_Class"},
	{WAvailable_Junction_Subtype,
	 "Available_Junction_Subtype"},
	{WAvailable_Junctions,
	 "Available_Junctions"},
	{WBackground_Nodata_Value,
	 "Background_Nodata_Value"},
	{WBehrmann,
	 "Behrmann"},
	{WBinary_Enclosures,
	 "Binary_Enclosures"},
	{WBits_Per_Pixel,
	 "Bits_Per_Pixel"},
	{WBonne,
	 "Bonne"},
	{WBottom_Bounding_Coordinate,
	 "Bottom_Bounding_Coordinate"},
	{WBrowse_Graphic_Embedded,
	 "Browse_Graphic_Embedded"},
	{WCassini,
	 "Cassini"},
	{WCell_Size_X_Direction,
	 "Cell_Size_X_Direction"},
	{WCell_Size_X_Units,
	 "Cell_Size_X_Units"},
	{WCell_Size_Y_Direction,
	 "Cell_Size_Y_Direction"},
	{WCell_Size_Y_Units,
	 "Cell_Size_Y_Units"},
	{WComposite_Relationship,
	 "Composite_Relationship"},
	{WCompression_Type,
	 "Compression_Type"},
	{WConnectivity_Rule,
	 "Connectivity_Rule"},
	{WCoordinate_System_Name,
	 "Coordinate_System_Name"},
	{WCreation_Date,
	 "Creation_Date"},
	{WCreation_Time,
	 "Creation_Time"},
	{WData,
	 "Data"},
	{WData_Frame_Count,
	 "Data_Frame_Count"},
	{WData_Frame_Name,
	 "Data_Frame_Name"},
	{WData_Frames,
	 "Data_Frames"},
	{WDatabase_Name,
	 "Database_Name"},
	{WDataset_Size,
	 "Dataset_Size"},
	{WDefault_Junction_Feature_Class,
	 "Default_Junction_Feature_Class"},
	{WDefault_Junction_Subtype,
	 "Default_Junction_Subtype"},
	{WDescription_of_Association,
	 "Description_of_Association"},
	{WDescription_of_Enclosure,
	 "Description_of_Enclosure"},
	{WDescription_of_Relationship,
	 "Description_of_Relationship"},
	{WDestination_Foreign_Key,
	 "Destination_Foreign_Key"},
	{WDestination_Name,
	 "Destination_Name"},
	{WDestination_Primary_Key,
	 "Destination_Primary_Key"},
	{WDomain_Attribute_Type,
	 "Domain_Attribute_Type"},
	{WDomain_Description,
	 "Domain_Description"},
	{WDomain_Name,
	 "Domain_Name"},
	{WDomain_Owner,
	 "Domain_Owner"},
	{WDomain_Type,
	 "Domain_Type"},
	{WEckert_I,
	 "Eckert_I"},
	{WEckert_II,
	 "Eckert_II"},
	{WEckert_III,
	 "Eckert_III"},
	{WEckert_IV,
	 "Eckert_IV"},
	{WEckert_V,
	 "Eckert_V"},
	{WEckert_VI,
	 "Eckert_VI"},
	{WEdge_Feature_Class,
	 "Edge_Feature_Class"},
	{WEdge_Maximum_Cardinality,
	 "Edge_Maximum_Cardinality"},
	{WEdge_Minimum_Cardinality,
	 "Edge_Minimum_Cardinality"},
	{WEdge_Subtype,
	 "Edge_Subtype"},
	{WEnabled_Attribute,
	 "Enabled_Attribute"},
	{WEnclosure,
	 "Enclosure"},
	{WEntity_Type_Count,
	 "Entity_Type_Count"},
	{WEntity_Type_Type,
	 "Entity_Type_Type"},
	{WESRI_Feature_Count,
	 "ESRI_Feature_Count"},
	{WESRI_Feature_Geometry,
	 "ESRI_Feature_Geometry"},
	{WESRI_Feature_Type,
	 "ESRI_Feature_Type"},
	{WESRI_Information,
	 "ESRI_Information"},
	{WESRI_Terms_Description,
	 "ESRI_Terms_Description"},
	{WESRI_Topology,
	 "ESRI_Topology"},
	{WFeature_Description,
	 "Feature_Description"},
	{WFile_or_Table_Name,
	 "File_or_Table_Name"},
	{WFrom_Edge_Feature_Class,
	 "From_Edge_Feature_Class"},
	{WFrom_Edge_Subtype,
	 "From_Edge_Subtype"},
	{WGall_Stereographic,
	 "Gall_Stereographic"},
	{WGeocoding_Information,
	 "Geocoding_Information"},
	{WGeographic_Coordinate_System_Name,
	 "Geographic_Coordinate_System_Name"},
	{WGeometric_Network_Information,
	 "Geometric_Network_Information"},
	{WImage_Colormap,
	 "Image_Colormap"},
	{WImage_Format,
	 "Image_Format"},
	{WImage_Type,
	 "Image_Type"},
	{WInstance_Name,
	 "Instance_Name"},
	{WJunction_Feature_Class,
	 "Junction_Feature_Class"},
	{WJunction_Maximum_Cardinality,
	 "Junction_Maximum_Cardinality"},
	{WJunction_Minimum_Cardinality,
	 "Junction_Minimum_Cardinality"},
	{WJunction_Subtype,
	 "Junction_Subtype"},
	{WLanguage_of_Dataset,
	 "Language_of_Dataset"},
	{WLanguage_of_Metadata,
	 "Language_of_Metadata"},
	{WLeft_Bounding_Coordinate,
	 "Left_Bounding_Coordinate"},
	{WLinear_Referencing,
	 "Linear_Referencing"},
	{WLocal_Bounding_Coordinates,
	 "Local_Bounding_Coordinates"},
	{WLoximuthal,
	 "Loximuthal"},
	{WMaximum_Altitude,
	 "Maximum_Altitude"},
	{WMerge_Rule,
	 "Merge_Rule"},
	{WMetadata_ID,
	 "Metadata_ID"},
	{WMinimum_Altitude,
	 "Minimum_Altitude"},
	{WModification_Date,
	 "Modification_Date"},
	{WModification_Time,
	 "Modification_Time"},
	{WMollweide,
	 "Mollweide"},
	{WNative_Dataset_Format,
	 "Native_Dataset_Format"},
	{WNetwork_Element,
	 "Network_Element"},
	{WNetwork_Role,
	 "Network_Role"},
	{WNetwork_Type,
	 "Network_Type"},
	{WNotification_Direction,
	 "Notification_Direction"},
	{WNumber_of_Raster_Bands,
	 "Number_of_Raster_Bands"},
	{WOrigin_Foreign_Key,
	 "Origin_Foreign_Key"},
	{WOrigin_Name,
	 "Origin_Name"},
	{WOrigin_Primary_Key,
	 "Origin_Primary_Key"},
	{WProcess_Software_and_Version,
	 "Process_Software_and_Version"},
	{WProjected_Coordinate_System_Name,
	 "Projected_Coordinate_System_Name"},
	{WPyramid_Layers,
	 "Pyramid_Layers"},
	{WQuartic_Authalic,
	 "Quartic_Authalic"},
	{WRange_Domain_Mean,
	 "Range_Domain_Mean"},
	{WRange_Domain_Standard_Deviation,
	 "Range_Domain_Standard_Deviation"},
	{WRaster_Display_Type,
	 "Raster_Display_Type"},
	{WRaster_Origin,
	 "Raster_Origin"},
	{WRelationship_Backward_Label,
	 "Relationship_Backward_Label"},
	{WRelationship_Cardinality,
	 "Relationship_Cardinality"},
	{WRelationship_Forward_Label,
	 "Relationship_Forward_Label"},
	{WRelationship_Information,
	 "Relationship_Information"},
	{WRight_Bounding_Coordinate,
	 "Right_Bounding_Coordinate"},
	{WRule_Category,
	 "Rule_Category"},
	{WRule_Help,
	 "Rule_Help"},
	{WRule_Type,
	 "Rule_Type"},
	{WSDE_Connection_Information,
	 "SDE_Connection_Information"},
	{WServer_Name,
	 "Server_Name"},
	{WSpatial_Index,
	 "Spatial_Index"},
	{WSplit_Rule,
	 "Split_Rule"},
	{WSubtype_Attribute,
	 "Subtype_Attribute"},
	{WSubtype_Attribute_Name,
	 "Subtype_Attribute_Name"},
	{WSubtype_Code,
	 "Subtype_Code"},
	{WSubtype_Default_Value,
	 "Subtype_Default_Value"},
	{WSubtype_Information,
	 "Subtype_Information"},
	{WSubtype_Name,
	 "Subtype_Name"},
	{WSynchronization_Date,
	 "Synchronization_Date"},
	{WSynchronization_Time,
	 "Synchronization_Time"},
	{WSynchronize_Automatically,
	 "Synchronize_Automatically"},
	{WSynchronize_Once,
	 "Synchronize_Once"},
	{WThumbnail,
	 "Thumbnail"},
	{WTo_Edge_Feature_Class,
	 "To_Edge_Feature_Class"},
	{WTo_Edge_Subtype,
	 "To_Edge_Subtype"},
	{WTop_Bounding_Coordinate,
	 "Top_Bounding_Coordinate"},
	{WUser_Name,
	 "User_Name"},
	{WVersion_Name,
	 "Version_Name"},
	{WWinkel_I,
	 "Winkel_I"},
	{WWinkel_II,
	 "Winkel_II"},
	{WGeographic_Box,
	 "Geographic_Box"},

#endif

	{Wunknown, NULL}
	};

/*----------------------------------------------------------------------*\
 | Cataln																|
 |																		|
 | Cataln-language element names kindly provided by					|
 | Clearinghouse Nacional de Datos Geogrficos, Uruguay					|
 | <http://www.clearinghouse.gub.uy/>									|
\*----------------------------------------------------------------------*/

static struct keyword_list ca_std[] = {
	{WAbscissa_Resolution,
	 "Resoluci_d'Abscisa"},
	{WAbstract,
	 "Resum"},
	{WAccess_Constraints,
	 "Restriccions_d'Accs"},
	{WAccess_Instructions,
	 "Instruccions_d'Accs"},
	{WAddress,
	 "Direcci"},
	{WAddress_Type,
	 "Tipologia_de_Direcci"},
	{WAlbers_Conical_Equal_Area,
	 "Iso_rea_Azimutal_d'Albers"},
	{WAltitude_Datum_Name,
	 "Nom_del_Dtum_d'Altitud"},
	{WAltitude_Distance_Units,
	 "Unitats_de_Distncia_de_l'Altitud"},
	{WAltitude_Encoding_Method,
	 "Mtode_de_Codificaci_de_l'Altitud"},
	{WAltitude_Resolution,
	 "Resoluci_de_l'Altitud"},
	{WAltitude_System_Definition,
	 "Definici_del_Sistema_d'Altitud"},
	{WARC_Coordinate_System,
	 "Sistema_ARC_de_Coordenades"},
	{WARC_System_Zone_Identifier,
	 "Identificador_de_Zona_del_Sistema_ARC"},
	{WAttribute,
	 "Atributs"},
	{WAttribute_Accuracy,
	 "Atribut_de_Precisi"},
	{WAttribute_Accuracy_Explanation,
	 "Descripci_de_la_Precisi_de_l'Atribut"},
	{WAttribute_Accuracy_Report,
	 "Informe_Sobre_Atribut_de_Precisi"},
	{WAttribute_Accuracy_Value,
	 "Valor_de_l'Atribut_de_Precisi"},
	{WAttribute_Definition,
	 "Definici_de_l'Atribut"},
	{WAttribute_Definition_Source,
	 "Font_de_Definici_d'Atributs"},
	{WAttribute_Domain_Values,
	 "Rang_de_Valors_d'Atributs"},
	{WAttribute_Label,
	 "Etiqueta_de_l'Atribut"},
	{WAttribute_Measurement_Frequency,
	 "Freqncia_de_Mesures_de_l'Atribut"},
	{WAttribute_Measurement_Resolution,
	 "Resoluci_de_Mesures_de_l'Atribut"},
	{WAttribute_Units_of_Measure,
	 "Unitats_de_Mesura_de_l'Atribut"},
	{WAttribute_Value_Accuracy,
	 "Precisi_de_Valors_de_l'Atribut"},
	{WAttribute_Value_Accuracy_Explanation,
	 "Explicaci_de_Precisi_de_Valors_de_l'Atribut"},
	{WAttribute_Value_Accuracy_Information,
	 "Informaci_de_Precisi_de_Valors_de_l'Atribut"},
	{WAvailable_Time_Period,
	 "Perode_Amb_Dades_Disponibles"},
	{WAzimuthal_Angle,
	 "Angle_Azimutal"},
	{WAzimuthal_Equidistant,
	 "Equidistant_Azimutal"},
	{WAzimuth_Measure_Point_Longitude,
	 "Longitud_de_l'Azimut_del_Punt_de_Mesura"},
	{WBearing_Reference_Direction,
	 "Direcci_del_Rumb_de_Referncia"},
	{WBearing_Reference_Meridian,
	 "Meridi_del_Rumb_de_Referncia"},
	{WBearing_Resolution,
	 "Resoluci_de_Rumb"},
	{WBearing_Units,
	 "Unitats_de_Rumb"},
	{WBeginning_Date,
	 "Data_de_Comenament"},
	{WBeginning_Date_of_Attribute_Values,
	 "Data_Inicial_de_Valors_de_l'Atribut"},
	{WBeginning_Time,
	 "Hora_de_Comenament"},
	{WBounding_Coordinates,
	 "Coordenades_Lmits"},
	{WBrowse_Graphic,
	 "Grfic_Per_a_Visualitzar"},
	{WBrowse_Graphic_File_Description,
	 "Descripci_de_l'Arxiu_Grfic_Per_a_Visualitzar"},
	{WBrowse_Graphic_File_Name,
	 "Nom_de_l'Arxiu_Grfic_Per_a_Visualitzar"},
	{WBrowse_Graphic_File_Type,
	 "Tipus_d'Arxiu_Grfic_Per_a_Visualitzar"},
	{WCalendar_Date,
	 "Data_Calendari"},
	{WCitation,
	 "Cita"},
	{WCitation_Information,
	 "Informaci_Per_a_la_Cita"},
	{WCity,
	 "Ciutat"},
	{WCloud_Cover,
	 "Cobriment_per_Nvols"},
	{WCodeset_Domain,
	 "Domini_del_Codeset"},
	{WCodeset_Name,
	 "Nom_del_Codeset"},
	{WCodeset_Source,
	 "Font_del_Codeset"},
	{WColumn_Count,
	 "Nmero_d'Objectes_Raster_a_l'Eix_X"},
	{WCompatibility_Information,
	 "Informaci_de_Compatibilitat"},
	{WCompleteness_Report,
	 "Informe_Sobre_Completesa"},
	{WCompression_Support,
	 "Protocols_de_Compressi_Soportats"},
	{WComputer_Contact_Information,
	 "Informaci_Per_a_Contacte_Amb_Computadora"},
	{WContact_Address,
	 "Direcci_del_Contacte"},
	{WContact_Electronic_Mail_Address,
	 "Correu_Electrnic_de_Contacte"},
	{WContact_Facsimile_Telephone,
	 "Facsmil_de_Contacte"},
	{WContact_Information,
	 "Informaci_de_Contacte"},
	{WContact_Instructions,
	 "Instruccions_Per_a_Contacte"},
	{WContact_Organization,
	 "Organizaci_Per_a_Contacte"},
	{WContact_Organization_Primary,
	 "Contacte_Preferentement_Amb_Organizaci"},
	{WContact_Person,
	 "Persona_Per_a_Contacte"},
	{WContact_Person_Primary,
	 "Contacte_Preferentment_Amb_Persona"},
	{WContact_Position,
	 "Crrec_del_Contacte"},
	{WContact_TDD_TTY_Telephone,
	 "TDD_TTY_de_Contacte"},
	{WContact_Voice_Telephone,
	 "Telfon_de_Contacte"},
	{WCoordinate_Representation,
	 "Representaci_de_Coordenades"},
	{WCountry,
	 "Pas"},
	{WCross_Reference,
	 "Referencia_Creuada"},
	{WCurrentness_Reference,
	 "Vigncia"},
	{WCustom_Order_Process,
	 "Processament_Personalitzat_de_Comandes"},
	{WData_Quality_Information,
	 "Informaci_de_Qualitat_de_les_Dades"},
	{WData_Set_Credit,
	 "Crdit_del_Conjunt_de_Dades"},
	{WData_Set_G_Polygon,
	 "Polgon_Lmit_de_les_Dades"},
	{WData_Set_G_Polygon_Exclusion_G_Ring,
	 "Polgon_Excls_de_les_Dades"},
	{WData_Set_G_Polygon_Outer_G_Ring,
	 "Polgon_Extern_de_les_Dades"},
	{WDenominator_of_Flattening_Ratio,
	 "Denominador_de_la_Ra_d'Aixafament"},
	{WDepth_Datum_Name,
	 "Nom_del_Dtum_de_Profunditat"},
	{WDepth_Distance_Units,
	 "Unitats_de_Distncia_de_Profunditat"},
	{WDepth_Encoding_Method,
	 "Mtode_de_Codificaci_de_Profunditat"},
	{WDepth_Resolution,
	 "Resoluci_de_Profunditat"},
	{WDepth_System_Definition,
	 "Definici_del_Sistema_de_Profunditat"},
	{WDescription,
	 "Descripci"},
	{WDetailed_Description,
	 "Descripci_Detallada"},
	{WDialup_File_Name,
	 "Nom_d'Arxiu_de_Discat_Automtic"},
	{WDialup_Instructions,
	 "Instruccions_Per_a_Discat_Automtic"},
	{WDialup_Telephone,
	 "Nmero_Telefnic_Per_a_Discat_Automtic"},
	{WDigital_Form,
	 "Format_Digital"},
	{WDigital_Transfer_Information,
	 "Informaci_Sobre_Transferncia_Digital"},
	{WDigital_Transfer_Option,
	 "Opci_Sobre_Transferncia_Digital"},
	{WDirect_Spatial_Reference_Method,
	 "Mtode_de_Referncia_Espacial_Directa"},
	{WDistance_and_Bearing_Representation,
	 "Representaci_de_Distncia_i_Rumb"},
	{WDistance_Resolution,
	 "Resoluci_de_Distncia"},
	{WDistribution_Information,
	 "Informaci_de_Distribuci"},
	{WDistribution_Liability,
	 "Responsabilitats_del_Distribuidor"},
	{WDistributor,
	 "Distribuidor"},
	{WEast_Bounding_Coordinate,
	 "Coordenada_Lmit_a_l'Est"},
	{WEdition,
	 "Edici"},
	{WEllipsoid_Name,
	 "Nom_de_l'Ellipsoide"},
	{WEnding_Date,
	 "Data_de_Finalitzaci"},
	{WEnding_Date_of_Attribute_Values,
	 "Data_Final_de_Valors_de_l'Atribut"},
	{WEnding_Time,
	 "Hora_de_Finalitzaci"},
	{WEntity_and_Attribute_Detail_Citation,
	 "Cita_Detallada_d'Entidad_i_Atribut"},
	{WEntity_and_Attribute_Information,
	 "Informaci_d'Entitat_i_Atribut"},
	{WEntity_and_Attribute_Overview,
	 "Resum_General_d'Entitat_i_Atribut"},
	{WEntity_Type,
	 "Tipus_d'Entitat"},
	{WEntity_Type_Definition,
	 "Definici_de_Tipus_d'Entitat"},
	{WEntity_Type_Definition_Source,
	 "Font_de_Definici_de_Tipus_d'Entitat"},
	{WEntity_Type_Label,
	 "Etiqueta_de_Tipus_d'Entitat"},
	{WEnumerated_Domain,
	 "Domini_Enumerat"},
	{WEnumerated_Domain_Value,
	 "Valors_del_Domini_Enumerat"},
	{WEnumerated_Domain_Value_Definition,
	 "Definici_del_Domini_Enumerat"},
	{WEnumerated_Domain_Value_Definition_Source,
	 "Font_Per_a_la_Definici_del_Domini_Enumerat"},
	{WEquidistant_Conic,
	 "Cnica_Equidistant"},
	{WEquirectangular,
	 "Equirrectangular"},
	{WFalse_Easting,
	 "Fals_Est"},
	{WFalse_Northing,
	 "Fals_Nord"},
	{WFees,
	 "Tarifes"},
	{WFile_Decompression_Technique,
	 "Tcnica_de_Descompressi_d'Arxius"},
	{WFormat_Information_Content,
	 "Informaci_del_Format_del_Contingut"},
	{WFormat_Name,
	 "Nom_del_Format"},
	{WFormat_Specification,
	 "Especificaci_del_Format"},
	{WFormat_Version_Date,
	 "Data_de_Versi_del_Format"},
	{WFormat_Version_Number,
	 "Nmero_de_Versi_del_Format"},
	{WG_Ring,
	 "Poligonal_Geogrfica"},
	{WG_Ring_Latitude,
	 "Latitud_de_Poligonal"},
	{WG_Ring_Longitude,
	 "Longitud_de_Poligonal"},
	{WG_Ring_Point,
	 "Punt_de_Poligonal_Geogrfica"},
	{WGeneral_Vertical_Near_sided_Perspective,
	 "Perspectiva_de_Costat_General_Vertical"},
	{WGeodetic_Model,
	 "Model_Geodsic"},
	{WGeographic,
	 "Geogrfic"},
	{WGeographic_Coordinate_Units,
	 "Unitats_de_les_Coordenades_Geogrfiques"},
	{WGeospatial_Data_Presentation_Form,
	 "Forma_de_Presentaci_Geoespacial"},
	{WGnomonic,
	 "Nomnica"},
	{WGrid_Coordinate_System,
	 "Sistema_de_Coordenades_de_Quadrcula"},
	{WGrid_Coordinate_System_Name,
	 "Nom_del_Sistema_de_Coordenades_de_Quadrcula"},
	{WHeight_of_Perspective_Point_Above_Surface,
	 "Altura_del_Punt_de_Perspectiva_Sobre_la_Superfcie"},
	{WHighest_BPS,
	 "Mxima_BPS"},
	{WHorizontal_Coordinate_System_Definition,
	 "Definici_del_Sistema_de_Coordenades_Horitzontals"},
	{WHorizontal_Datum_Name,
	 "Nom_del_Dtum_Horitzontal"},
	{WHorizontal_Positional_Accuracy,
	 "Precisi_a_la_Horitzontal"},
	{WHorizontal_Positional_Accuracy_Explanation,
	 "Interpretaci_de_Precisi_Posicional_Horitzontal"},
	{WHorizontal_Positional_Accuracy_Report,
	 "Informe_Sobre_Precisi_a_la_Horitzontal"},
	{WHorizontal_Positional_Accuracy_Value,
	 "Valor_de_Precisi_a_la_Horitzontal"},
	{WHours_of_Service,
	 "Horari_d'Atenci"},
	{WIdentification_Information,
	 "Informaci_de_Identificaci"},
	{WIndirect_Spatial_Reference,
	 "Referncia_Espacial_Indirecta"},
	{WIssue_Identification,
	 "Identificaci_del_Nmero"},
	{WKeywords,
	 "Paraules_Clau"},
	{WLambert_Azimuthal_Equal_Area,
	 "Iso_rea_Azimutal_de_Lambert"},
	{WLambert_Conformal_Conic,
	 "Cnica_Conforme_de_Lambert"},
	{WLandsat_Number,
	 "Nmero_Landsat"},
	{WLarger_Work_Citation,
	 "Cita_a_Treball_Major"},
	{WLatitude_of_Projection_Center,
	 "Latitud_del_Centre_de_la_Projecci"},
	{WLatitude_of_Projection_Origin,
	 "Latitud_de_l'Origen_de_la_Projecci"},
	{WLatitude_Resolution,
	 "Resoluci_de_Latitud"},
	{WLineage,
	 "Llinatge"},
	{WLocal,
	 "Local"},
	{WLocal_Description,
	 "Descripci_Local"},
	{WLocal_Georeference_Information,
	 "Informaci_de_Georreferenciaci_Local"},
	{WLocal_Planar,
	 "Plana_Local"},
	{WLocal_Planar_Description,
	 "Descripci_de_la_Plana_Local"},
	{WLocal_Planar_Georeference_Information,
	 "Informaci_de_la_Georreferncia_de_la_Plana_Local"},
	{WLogical_Consistency_Report,
	 "Informe_Sobre_Consistncia_Lgica"},
	{WLongitude_of_Central_Meridian,
	 "Longitud_del_Meridi_Central"},
	{WLongitude_of_Projection_Center,
	 "Longitud_del_Centre_de_la_Projecci"},
	{WLongitude_Resolution,
	 "Resoluci_de_Longitud"},
	{WLowest_BPS,
	 "Mnima_BPS"},
	{WMaintenance_and_Update_Frequency,
	 "Freqncia_de_Manteniment_i_Actualitzaci"},
	{WMap_Projection,
	 "Projecci_Cartogrfica"},
	{WMap_Projection_Name,
	 "Nom_de_la_Projecci_Cartogrfica"},
	{WMap_Projection_Parameters,
	 "Parmetres_de_la_Projecci_Cartogrfica"},
	{WMercator,
	 "Mercator"},
	{WMetadata,
	 "Metadades"},
	{WMetadata_Access_Constraints,
	 "Restriccions_d'Accs_a_les_Metadades"},
	{WMetadata_Contact,
	 "Contacte_Per_a_Metadades"},
	{WMetadata_Date,
	 "Data_de_les_Metadades"},
	{WMetadata_Extensions,
	 "Extensions_a_l'Estndard_de_Metadades"},
	{WMetadata_Future_Review_Date,
	 "Data_de_la_Futura_Revisi_de_Metadades"},
	{WMetadata_Language,
	 "Metadata_Language"},
	{WMetadata_Reference_Information,
	 "Informaci_de_Referncia_de_Metadades"},
	{WMetadata_Review_Date,
	 "Data_de_l'ltima_Revisi_de_Metadades"},
	{WMetadata_Security_Classification,
	 "Classificaci_de_Seguretat_de_Metadades"},
	{WMetadata_Security_Classification_System,
	 "Sistema_de_Classificaci_de_Seguretat_de_Metadades"},
	{WMetadata_Security_Handling_Description,
	 "Descripci_de_Precaucions_de_Gesti_de_Metadades"},
	{WMetadata_Security_Information,
	 "Informaci_de_Seguretat_de_Metadades"},
	{WMetadata_Standard_Name,
	 "Nom_de_l'Estndard_de_Metadades"},
	{WMetadata_Standard_Version,
	 "Versi_de_l'Estndard_de_Metadades"},
	{WMetadata_Time_Convention,
	 "Convenci_de_Dates_de_Metadades"},
	{WMetadata_Use_Constraints,
	 "Restriccions_a_l's_de_Metadades"},
	{WMiller_Cylindrical,
	 "Cilndrica_de_Miller"},
	{WModified_Stereographic_for_Alaska,
	 "Estereogrfica_Modificada_Per_a_Alaska"},
	{WMultiple_Dates_Times,
	 "Mltiples_Dates/Hores"},
	{WNative_Data_Set_Environment,
	 "Entorn_Original_de_les_Dades"},
	{WNetwork_Address,
	 "Direcci_de_la_Xarxa"},
	{WNetwork_Resource_Name,
	 "Nom_del_Recurs_de_la_Xarxa"},
	{WNon_digital_Form,
	 "Format_No_Digital"},
	{WNorth_Bounding_Coordinate,
	 "Coordenada_Lmit_al_Nord"},
	{WNumber_DataBits,
	 "Nmero_de_Bits_de_Dades"},
	{WNumber_StopBits,
	 "Nmero_de_Bits_de_Parada"},
	{WOblique_Line_Azimuth,
	 "Azimut_de_Lnia_Obliqua"},
	{WOblique_Line_Latitude,
	 "Latitud_de_Lnia_Obliqua"},
	{WOblique_Line_Longitude,
	 "Longitud_de_Lnia_Obliqua"},
	{WOblique_Line_Point,
	 "Punt_de_Lnia_Obliqua"},
	{WOblique_Mercator,
	 "Mercator_Oblic"},
	{WOffline_Media,
	 "Medi_Fsic_Fora_de_Lnia"},
	{WOffline_Option,
	 "Opci_Fora_de_Lnia"},
	{WOnline_Computer_and_Operating_System,
	 "Computadora_i_Sistema_Operatiu_en_Lnia"},
	{WOnline_Linkage,
	 "Accs_en_Lnia"},
	{WOnline_Option,
	 "Opci_en_Lnia"},
	{WOrdering_Instructions,
	 "Instruccions_Per_a_la_Comanda"},
	{WOrdinate_Resolution,
	 "Resoluci_de_les_Ordenades"},
	{WOriginator,
	 "Productor"},
	{WOrthographic,
	 "Ortogrfic"},
	{WOther_Citation_Details,
	 "Altres_Detalls_de_la_Cita"},
	{WOther_Grid_Systems_Definition,
	 "Definici_d'un_Altre_Sistema_de_Quadrcula"},
	{WOther_Projections_Definition,
	 "Definici_d'una_Altra_Projecci"},
	{WOverview_Description,
	 "Descripci_General"},
	{WParity,
	 "Paritat"},
	{WPath_Number,
	 "Nmero_de_Cam"},
	{WPlace,
	 "Lloc"},
	{WPlace_Keyword,
	 "Paraules_Clau_de_Lloc"},
	{WPlace_Keyword_Thesaurus,
	 "Tesaure_de_Paraules_Clau_de_Lloc"},
	{WPlanar,
	 "Pla"},
	{WPlanar_Coordinate_Encoding_Method,
	 "Mtode_Codificat_de_Coordenades_Planes"},
	{WPlanar_Coordinate_Information,
	 "Informaci_de_Coordenades_Planes"},
	{WPlanar_Distance_Units,
	 "Unitats_de_Distncia_Planes"},
	{WPoint_and_Vector_Object_Count,
	 "Quantitat_d'Objectes_Punt_i_Vector"},
	{WPoint_and_Vector_Object_Information,
	 "Informaci_d'Objectes_Punt_i_Vector"},
	{WPoint_of_Contact,
	 "Punt_de_Contacte"},
	{WPolar_Stereographic,
	 "Estereogrfic_Polar"},
	{WPolyconic,
	 "Policnic"},
	{WPositional_Accuracy,
	 "Precisi_de_la_Posici"},
	{WPostal_Code,
	 "Codi_Postal"},
	{WProcess_Contact,
	 "Contacte_del_Procs"},
	{WProcess_Date,
	 "Data_del_Procs"},
	{WProcess_Description,
	 "Descripci_del_Procs"},
	{WProcess_Step,
	 "Pas_del_Procs"},
	{WProcess_Time,
	 "Hora_del_Procs"},
	{WProfile_Name,
	 "Nom_del_Perfil"},
	{WProgress,
	 "Aven"},
	{WPublication_Date,
	 "Data_de_Publicaci"},
	{WPublication_Information,
	 "Informaci_sobre_la_Publicaci"},
	{WPublication_Place,
	 "Lloc_de_Publicaci"},
	{WPublication_Time,
	 "Hora_de_Publicaci"},
	{WPublisher,
	 "Impresor"},
	{WPurpose,
	 "Propsit"},
	{WQuantitative_Attribute_Accuracy_Assessment,
	 "Determinaci_Quantitativa_Sobre_Precisi_d'Atributs"},
	{WQuantitative_Horizontal_Positional_Accuracy_Assessment,
	 "Declaraci_de_Precisi_Posicional_Horitzontal"},
	{WQuantitative_Vertical_Positional_Accuracy_Assessment,
	 "Declaraci_de_Precisi_Posicional_Vertical"},
	{WRange_Domain,
	 "Domini_del_Rang"},
	{WRange_Domain_Maximum,
	 "Mxim_del_Domini_del_Rang"},
	{WRange_Domain_Minimum,
	 "Mnim_del_Domini_del_Rang"},
	{WRange_of_Dates_Times,
	 "Rang_de_Dates/Hores"},
	{WRaster_Object_Information,
	 "Informaci_Sobre_Objectes_Raster"},
	{WRaster_Object_Type,
	 "Tipologia_d'Objectes_Raster"},
	{WRecording_Capacity,
	 "Capacitat_de_Gravaci"},
	{WRecording_Density,
	 "Densitat_de_Gravaci"},
	{WRecording_Density_Units,
	 "Unitats_de_Densitat_de_Grabaci"},
	{WRecording_Format,
	 "Format_de_Gravaci"},
	{WResource_Description,
	 "Descripci_del_Recurs"},
	{WRobinson,
	 "Robinson"},
	{WRow_Count,
	 "Nmero_d'Objectes_Raster_a_l'Eix_Y"},
	{WScale_Factor_at_Center_Line,
	 "Factor_d'Escala_a_la_Lnia_Central"},
	{WScale_Factor_at_Central_Meridian,
	 "Factor_d'Escala_al_Meridi_Central"},
	{WScale_Factor_at_Equator,
	 "Factor_d'Escala_a_l'Ecuador"},
	{WScale_Factor_at_Projection_Origin,
	 "Factor_d'Escala_a_l'Origen_de_la_Projecci"},
	{WSDTS_Point_and_Vector_Object_Type,
	 "Tipologia_d'Objectes_Punt_i_Vector_SDTS"},
	{WSDTS_Terms_Description,
	 "Descripci_de_Termes_SDTS"},
	{WSecurity_Classification,
	 "Classificaci_de_la_Seguretat"},
	{WSecurity_Classification_System,
	 "Sistema_de_Classificaci_de_la_Seguretat"},
	{WSecurity_Handling_Description,
	 "Descripci_de_Gesti_de_la_Seguretat"},
	{WSecurity_Information,
	 "Informaci_de_la_Seguretat"},
	{WSemi_major_Axis,
	 "Semieix_Major"},
	{WSeries_Information,
	 "Informaci_de_la_Srie"},
	{WSeries_Name,
	 "Nom_de_la_Srie"},
	{WSingle_Date_Time,
	 "Data_i_Hora"},
	{WSinusoidal,
	 "Sinusoidal"},
	{WSource_Citation,
	 "Cita_de_la_Font"},
	{WSource_Citation_Abbreviation,
	 "Abreviaci_de_la_Cita_de_la_Font"},
	{WSource_Contribution,
	 "Contribuci_de_la_Font"},
	{WSource_Currentness_Reference,
	 "Referncia_sobre_la_Vigncia_de_la_Font"},
	{WSource_Information,
	 "Informaci_de_la_Font"},
	{WSource_Produced_Citation_Abbreviation,
	 "Abreviatura_de_la_Cita_de_la_Font_Produda"},
	{WSource_Scale_Denominator,
	 "Denominador_d'Escala_de_la_Font"},
	{WSource_Time_Period_of_Content,
	 "Perode_Comprs_a_la_Font"},
	{WSource_Used_Citation_Abbreviation,
	 "Abreviatura_Usual_de_la_Cita_de_la_Font"},
	{WSouth_Bounding_Coordinate,
	 "Coordenada_Lmit_al_Sud"},
	{WSpace_Oblique_Mercator_Landsat,
	 "Mercator_Espacial_Obliqua_(Landsat)"},
	{WSpatial_Data_Organization_Information,
	 "Informaci_d'Organitzaci_de_Dades_Espacials"},
	{WSpatial_Domain,
	 "Domini_Espacial"},
	{WSpatial_Reference_Information,
	 "Informaci_de_Referncia_Espacial"},
	{WSPCS_Zone_Identifier,
	 "Identificador_de_Zona_SPCS"},
	{WStandard_Order_Process,
	 "Processament_Estndard_de_Comandes"},
	{WStandard_Parallel,
	 "Parallel_Estndard"},
	{WState_or_Province,
	 "Estat_o_Provncia"},
	{WState_Plane_Coordinate_System,
	 "Sistema_Estatal_de_Coordenades_Planes"},
	{WStatus,
	 "Estatus"},
	{WStereographic,
	 "Estereogrfica"},
	{WStraight_Vertical_Longitude_from_Pole,
	 "Longitud_Vertical-Recta_Des_del_Pol"},
	{WStratum,
	 "Estrat"},
	{WStratum_Keyword,
	 "Paraula_Clau_d'Estrat"},
	{WStratum_Keyword_Thesaurus,
	 "Tesaure_de_Paraules_Clau_d'Estrat"},
	{WSupplemental_Information,
	 "Informaci_Suplementria"},
	{WTechnical_Prerequisites,
	 "Prerrequisits_Tcnics"},
	{WTemporal,
	 "Temporal"},
	{WTemporal_Keyword,
	 "Paraules_Clau_de_Temps"},
	{WTemporal_Keyword_Thesaurus,
	 "Tesaure_de_Paraules_Clau_de_Temps"},
	{WTheme,
	 "Tema"},
	{WTheme_Keyword,
	 "Paraula_Clau_de_Tema"},
	{WTheme_Keyword_Thesaurus,
	 "Tesaure_de_Paraules_Clau_de_Tema"},
	{WTime_of_Day,
	 "Hora_del_Dia"},
	{WTime_Period_Information,
	 "Informaci_del_Perode_de_Temps"},
	{WTime_Period_of_Content,
	 "Perode_Asociat_al_Contingut"},
	{WTitle,
	 "Ttol"},
	{WTransfer_Size,
	 "Grandria_de_la_Transfrencia"},
	{WTransverse_Mercator,
	 "Transversal_Mercator"},
	{WTurnaround,
	 "Temps_de_Lliurement"},
	{WType_of_Source_Media,
	 "Tipologia_del_Medi_Fsic_de_la_Font"},
	{WUniversal_Polar_Stereographic,
	 "Estereogrfica_Polar_Universal"},
	{WUniversal_Transverse_Mercator,
	 "Transversal_Universal_Mercator"},
	{WUnrepresentable_Domain,
	 "Domini_No_Representable"},
	{WUPS_Zone_Identifier,
	 "Identificador_de_Zona_UPS"},
	{WUse_Constraints,
	 "Restriccions_d's"},
	{WUTM_Zone_Number,
	 "Nmero_de_Zona_UTM"},
	{Wvan_der_Grinten,
	 "van_der_Grinten"},
	{WVertical_Coordinate_System_Definition,
	 "Definici_del_Sistema_de_Coordenades_Verticals"},
	{WVertical_Count,
	 "Nmero_d'Objectes_Raster_a_l'Eix_Z"},
	{WVertical_Positional_Accuracy,
	 "Precisi_a_la_Vertical"},
	{WVertical_Positional_Accuracy_Explanation,
	 "Interpretaci_de_la_Precisi_Posicional_Vertical"},
	{WVertical_Positional_Accuracy_Report,
	 "Informe_Sobre_Precisi_a_la_Vertical"},
	{WVertical_Positional_Accuracy_Value,
	 "Valor_de_Precisi_a_la_Vertical"},
	{WVPF_Point_and_Vector_Object_Information,
	 "Informaci_d'Objectes_Punt_i_Vector_VPF"},
	{WVPF_Point_and_Vector_Object_Type,
	 "Tipologia_d'Objectes_Punt_i_Vector_VPF"},
	{WVPF_Terms_Description,
	 "Descripci_de_Termes_VPF"},
	{WVPF_Topology_Level,
	 "Nivell_de_Topologia_VPF"},
	{WWest_Bounding_Coordinate,
	 "Coordenada_Lmit_a_l'Oest"},

/* Biological Data Profile */

	{WDescription_of_Geographic_Extent,
		"Description_of_Geographic_Extent"},
	{WBounding_Altitudes,
		"Bounding_Altitudes"},
	{WAltitude_Minimum,
		"Altitude_Minimum"},
	{WAltitude_Maximum,
		"Altitude_Maximum"},
	{WTaxonomy,
		"Taxonomy"},
	{WKeywords_Taxon,
		"Keywords/Taxon"},
	{WTaxonomic_Keyword_Thesaurus,
		"Taxonomic_Keyword_Thesaurus"},
	{WTaxonomic_Keywords,
		"Taxonomic_Keywords"},
	{WTaxonomic_System,
		"Taxonomic_System"},
	{WClassification_System_Authority,
		"Classification_System/Authority"},
	{WClassification_System_Citation,
		"Classification_System_Citation"},
	{WClassification_System_Modifications,
		"Classification_System_Modifications"},
	{WIdentification_Reference,
		"Identification_Reference"},
	{WIdentifier,
		"Identifier"},
	{WTaxonomic_Procedures,
		"Taxonomic_Procedures"},
	{WTaxonomic_Completeness,
		"Taxonomic_Completeness"},
	{WVouchers,
		"Vouchers"},
	{WSpecimen,
		"Specimen"},
	{WRepository,
		"Repository"},
	{WGeneral_Taxonomic_Coverage,
		"General_Taxonomic_Coverage"},
	{WTaxonomic_Classification,
		"Taxonomic_Classification"},
	{WTaxon_Rank_Name,
		"Taxon_Rank_Name"},
	{WTaxon_Rank_Value,
		"Taxon_Rank_Value"},
	{WApplicable_Common_Name,
		"Applicable_Common_Name"},
	{WAnalytical_Tool,
		"Analytical_Tool"},
	{WAnalytical_Tool_Description,
		"Analytical_Tool_Description"},
	{WTool_Access_Information,
		"Tool_Access_Information"},
	{WTool_Contact,
		"Tool_Contact"},
	{WTool_Citation,
		"Tool_Citation"},
	{WTool_Access_Instructions,
		"Tool_Access_Instructions"},
	{WTool_Computer_and_Operating_System,
		"Tool_Computer_and_Operating_System"},
	{WMethodology,
		"Methodology"},
	{WMethodology_Type,
		"Methodology_Type"},
	{WMethodology_Identifier,
		"Methodology_Identifier"},
	{WMethodology_Keyword_Thesaurus,
		"Methodology_Keyword_Thesaurus"},
	{WMethodology_Keyword,
		"Methodology_Keyword"},
	{WMethodology_Description,
		"Methodology_Description"},
	{WMethodology_Citation,
		"Methodology_Citation"},
	{WASCII_File_Structure,
		"ASCII_File_Structure"},
	{WRecord_Delimiter,
		"Record_Delimiter"},
	{WNumber_Header_Lines,
		"Number_Header_Lines"},
	{WDescription_of_Header_Content,
		"Description_of_Header_Content"},
	{WOrientation,
		"Orientation"},
	{WCase_Sensitive,
		"Case_Sensitive"},
	{WAuthentication,
		"Authentication"},
	{WQuote_Character,
		"Quote_Character"},
	{WData_Field,
		"Data_Field"},
	{WData_Field_Name,
		"Data_Field_Name"},
	{WMissing_Value_Code,
		"Missing_Value_Code"},
	{WData_Field_Width_Delimiter,
		"Data_Field_Width_Delimiter"},
	{WData_Field_Width,
		"Data_Field_Width"},
	{WBeginning_Geologic_Age,
		"Beginning_Geologic_Age"},
	{WEnding_Geologic_Age,
		"Ending_Geologic_Age"},
	{WGeologic_Age,
		"Geologic_Age"},
	{WGeologic_Time_Scale,
		"Geologic_Time_Scale"},
	{WGeologic_Age_Estimate,
		"Geologic_Age_Estimate"},
	{WGeologic_Age_Uncertainty,
		"Geologic_Age_Uncertainty"},
	{WGeologic_Age_Explanation,
		"Geologic_Age_Explanation"},
	{WGeologic_Citation,
		"Geologic_Citation"},

/* Shoreline profile */

	{WBarometric_Pressure,
	 "Barometric_Pressure"},
	{WEnvironmental_Event,
	 "Environmental_Event"},
	{WMarine_Weather_Condition,
	 "Marine_Weather_Condition"},
	{WProcess_Step_Citation,
	 "Process_Step_Citation"},
	{WRange_of_Tide,
	 "Range_of_Tide"},
	{WSupplemental_Tidal_Information,
	 "Supplemental_Tidal_Information"},
	{WTidal_Datum,
	 "Tidal_Datum"},
	{WTidal_Information,
	 "Tidal_Information"},
	{WTide_Table_Reference,
	 "Tide_Table_Reference"},
	{WTime_of_High_Tide,
	 "Time_of_High_Tide"},
	{WTime_of_Low_Tide,
	 "Time_of_Low_Tide"},
	{WTime_of_Tide,
	 "Time_of_Tide"},
	{WType_of_Tide,
	 "Type_of_Tide"},
	{WWave_Height,
	 "Wave_Height"},
	{WWind_Direction,
	 "Wind_Direction"},
	{WWind_Speed,
	 "Wind_Speed"},

/* Remote-sensing profile */

	{WAcquisition_Information,
	 "Acquisition_Information"},
	{WAerial_Film_Speed,
	 "Aerial_Film_Speed"},
	{WAerotriangulation_Reference,
	 "Aerotriangulation_Reference"},
	{WAffine_Distortion_X_Prime_Coefficient,
	 "Affine_Distortion_X_Prime_Coefficient"},
	{WAffine_Distortion_Y_Prime_Coefficient,
	 "Affine_Distortion_Y_Prime_Coefficient"},
	{WAggregation_Criteria,
	 "Aggregation_Criteria"},
	{WAggregation_Information,
	 "Aggregation_Information"},
	{WAggregation_Member_ID,
	 "Aggregation_Member_ID"},
	{WAlgorithm_Change_Description,
	 "Algorithm_Change_Description"},
	{WAlgorithm_Change_History,
	 "Algorithm_Change_History"},
	{WAlgorithm_Change_Reference,
	 "Algorithm_Change_Reference"},
	{WAlgorithm_Description,
	 "Algorithm_Description"},
	{WAlgorithm_Identifiers,
	 "Algorithm_Identifiers"},
	{WAlgorithm_Information,
	 "Algorithm_Information"},
	{WAlgorithm_Peer_Review_Description,
	 "Algorithm_Peer_Review_Description"},
	{WAlgorithm_Peer_Review_Information,
	 "Algorithm_Peer_Review_Information"},
	{WAlgorithm_Peer_Review_Reference,
	 "Algorithm_Peer_Review_Reference"},
	{WAlgorithm_Reference,
	 "Algorithm_Reference"},
	{WAlgorithm_Text_Description,
	 "Algorithm_Text_Description"},
	{WAltitude_Distance_Layer,
	 "Altitude_Distance_Layer"},
	{WAncillary_Dataset,
	 "Ancillary_Dataset"},
	{WAncillary_Dataset_Description,
	 "Ancillary_Dataset_Description"},
	{WAncillary_Dataset_Identifier,
	 "Ancillary_Dataset_Identifier"},
	{WAncillary_Description,
	 "Ancillary_Description"},
	{WAncillary_Reference,
	 "Ancillary_Reference"},
	{WAngle_Dependent_Distortion,
	 "Angle_Dependent_Distortion"},
	{WAngle_Distortion_Value,
	 "Angle_Distortion_Value"},
	{WArea_Weighted_Average_Resolution,
	 "Area_Weighted_Average_Resolution"},
	{WArgument_of_Perigee,
	 "Argument_of_Perigee"},
	{WAscending_Descending_Indicator,
	 "Ascending/Descending_Indicator"},
	{WAttitude_Angular_Units,
	 "Attitude_Angular_Units"},
	{WAvailability_of_Element_Locations,
	 "Availability_of_Element_Locations"},
	{WAxes,
	 "Axes"},
	{WAxis_Rotation_Convention,
	 "Axis_Rotation_Convention"},
	{WBand_Boundary_Definition,
	 "Band_Boundary_Definition"},
	{WBand_Description,
	 "Band_Description"},
	{WBand_ID,
	 "Band_ID"},
	{WBand_Identification,
	 "Band_Identification"},
	{WBand_Measurement_Mode_ID,
	 "Band_Measurement_Mode_ID"},
	{WBand_Quality,
	 "Band_Quality"},
	{WCalibrated_Detector_Positions,
	 "Calibrated_Detector_Positions"},
	{WCalibrated_Focal_Length,
	 "Calibrated_Focal_Length"},
	{WCamera_Identifier,
	 "Camera_Identifier"},
	{WCamera_Type,
	 "Camera_Type"},
	{WCell_Value_Type,
	 "Cell_Value_Type"},
	{WClock_Time_Drift,
	 "Clock_Time/Drift"},
	{WCollection_Type,
	 "Collection_Type"},
	{WColumn_Delta_X,
	 "Column_Delta_X"},
	{WColumn_Delta_Y,
	 "Column_Delta_Y"},
	{WCommand_Line_Processing_Parameter,
	 "Command_Line_Processing_Parameter"},
	{WComponent_Information,
	 "Component_Information"},
	{WContainer_Packet_ID,
	 "Container_Packet_ID"},
	{WControl_Point_Authority,
	 "Control_Point_Authority"},
	{WControl_Point_Column,
	 "Control_Point_Column"},
	{WControl_Point_Earth_Location,
	 "Control_Point_Earth_Location"},
	{WControl_Point_Horizontal_X_Accuracy,
	 "Control_Point_Horizontal_X_Accuracy"},
	{WControl_Point_Horizontal_Y_Accuracy,
	 "Control_Point_Horizontal_Y_Accuracy"},
	{WControl_Point_ID,
	 "Control_Point_ID"},
	{WControl_Point_Identification,
	 "Control_Point_Identification"},
	{WControl_Point_Origin,
	 "Control_Point_Origin"},
	{WControl_Point_Raster_Position,
	 "Control_Point_Raster_Position"},
	{WControl_Point_Row,
	 "Control_Point_Row"},
	{WControl_Point_Type,
	 "Control_Point_Type"},
	{WControl_Point_Use_Flag,
	 "Control_Point_Use_Flag"},
	{WControl_Point_Vertical_Accuracy,
	 "Control_Point_Vertical_Accuracy"},
	{WControl_Point_x_Value,
	 "Control_Point_x_Value"},
	{WControl_Point_y_Value,
	 "Control_Point_y_Value"},
	{WControl_Point_z_Value,
	 "Control_Point_z_Value"},
	{WCoordinate_Point,
	 "Coordinate_Point"},
	{WCoordinate_System,
	 "Coordinate_System"},
	{WCoordinate_x_Value,
	 "Coordinate_x_Value"},
	{WCoordinate_XY_Units,
	 "Coordinate_XY_Units"},
	{WCoordinate_y_Value,
	 "Coordinate_y_Value"},
	{WCoordinate_Z_Units,
	 "Coordinate_Z_Units"},
	{WCoordinate_z_Value,
	 "Coordinate_z_Value"},
	{WCross_Track_Axis,
	 "Cross_Track_Axis"},
	{WCross_Track_Description,
	 "Cross_Track_Description"},
	{WCross_Track_Direction,
	 "Cross_Track_Direction"},
	{WCross_Track_Extent_Angle,
	 "Cross_Track_Extent_Angle"},
	{WCross_Track_Fixed_Angle,
	 "Cross_Track_Fixed_Angle"},
	{WCross_Track_Motion,
	 "Cross_Track_Motion"},
	{WCross_Track_Start_Angle,
	 "Cross_Track_Start_Angle"},
	{WCross_Track_Step_Angle,
	 "Cross_Track_Step_Angle"},
	{WCross_Track_Sweep,
	 "Cross_Track_Sweep"},
	{WCross_Track_Zero,
	 "Cross_Track_Zero"},
	{WData_Dictionary_Reference,
	 "Data_Dictionary_Reference"},
	{WData_Scaling_Information,
	 "Data_Scaling_Information"},
	{WDataset_Identifier,
	 "Dataset_Identifier"},
	{WDate_of_Last_Calibration,
	 "Date_of_Last_Calibration"},
	{WDeveloping_Institution,
	 "Developing_Institution"},
	{WDimension_Count,
	 "Dimension_Count"},
	{WDimension_Description,
	 "Dimension_Description"},
	{WDimension_Properties,
	 "Dimension_Properties"},
	{WDistance_Dependent_Distortion,
	 "Distance_Dependent_Distortion"},
	{WDistance_Distortion_Value,
	 "Distance_Distortion_Value"},
	{WDistortion,
	 "Distortion"},
	{WDistortion_Type_Affine,
	 "Distortion_Type_Affine"},
	{WDistortion_Type_Radial_Asymmetrical,
	 "Distortion_Type_Radial_Asymmetrical"},
	{WDistortion_Type_Radial_Symmetrical,
	 "Distortion_Type_Radial_Symmetrical"},
	{WDocumentation,
	 "Documentation"},
	{WEccentricity,
	 "Eccentricity"},
	{WEffective_Aerial_Film_Speed,
	 "Effective_Aerial_Film_Speed"},
	{WElevation_Axis,
	 "Elevation_Axis"},
	{WElevation_Description,
	 "Elevation_Description"},
	{WElevation_Direction,
	 "Elevation_Direction"},
	{WElevation_Extent_Angle,
	 "Elevation_Extent_Angle"},
	{WElevation_Fixed_Angle,
	 "Elevation_Fixed_Angle"},
	{WElevation_Start_Angle,
	 "Elevation_Start_Angle"},
	{WElevation_Step_Angle,
	 "Elevation_Step_Angle"},
	{WElevation_Sweep,
	 "Elevation_Sweep"},
	{WElevation_Zero,
	 "Elevation_Zero"},
	{WEphemeris,
	 "Ephemeris"},
	{WExposure_Time,
	 "Exposure_Time"},
	{WExterior_Orientation_Accuracy,
	 "Exterior_Orientation_Accuracy"},
	{WFiducial,
	 "Fiducial"},
	{WFiducial_Center,
	 "Fiducial_Center"},
	{WFilm_Type,
	 "Film_Type"},
	{WFilter_on_Camera,
	 "Filter_on_Camera"},
	{WFilter_on_Camera_Indicator,
	 "Filter_on_Camera_Indicator"},
	{WFilter_Type,
	 "Filter_Type"},
	{WFlight_Protocol,
	 "Flight_Protocol"},
	{WFlying_Height,
	 "Flying_Height"},
	{WForward_Motion_Compensation,
	 "Forward_Motion_Compensation"},
	{WFrame_Area,
	 "Frame_Area"},
	{WFrame_Area_Units,
	 "Frame_Area_Units"},
	{WFrame_Area_Value,
	 "Frame_Area_Value"},
	{WFrame_Camera,
	 "Frame_Camera"},
	{WFrame_Geometric_Properties,
	 "Frame_Geometric_Properties"},
	{WFrame_Hardware,
	 "Frame_Hardware"},
	{WFrame_Operation,
	 "Frame_Operation"},
	{WFrame_Optics,
	 "Frame_Optics"},
	{WFrame_Radiometric_Calibration,
	 "Frame_Radiometric_Calibration"},
	{WFrame_Radiometric_Properties,
	 "Frame_Radiometric_Properties"},
	{WFrame_Spectral_Information,
	 "Frame_Spectral_Information"},
	{WFrame_Spectral_Properties,
	 "Frame_Spectral_Properties"},
	{WGeorectified_Raster,
	 "Georectified_Raster"},
	{WGeoreferenceable_Raster,
	 "Georeferenceable_Raster"},
	{WGeoreferencing_Description,
	 "Georeferencing_Description"},
	{WGeoreferencing_Information,
	 "Georeferencing_Information"},
	{WGPS_Information_System_Availability,
	 "GPS_Information_System_Availability"},
	{WGrid_First_Element,
	 "Grid_First_Element"},
	{WGrid_First_Element_Map_X_Coordinate,
	 "Grid_First_Element_Map_X_Coordinate"},
	{WGrid_First_Element_Map_Y_Coordinate,
	 "Grid_First_Element_Map_Y_Coordinate"},
	{WGrid_Orientation,
	 "Grid_Orientation"},
	{WGround_Control_Point_Description,
	 "Ground_Control_Point_Description"},
	{WGround_Control_Point_Information,
	 "Ground_Control_Point_Information"},
	{WGround_Control_Point_Organization,
	 "Ground_Control_Point_Organization"},
	{WGround_Control_Point_Position,
	 "Ground_Control_Point_Position"},
	{WGround_Shape,
	 "Ground_Shape"},
	{WID_Overlapping_Dataset,
	 "ID_Overlapping_Dataset"},
	{WIFOV_Units,
	 "IFOV_Units"},
	{WIFOV_x_Definition,
	 "IFOV_x_Definition"},
	{WIFOV_x_Value,
	 "IFOV_x_Value"},
	{WIFOV_y_Definition,
	 "IFOV_y_Definition"},
	{WIFOV_y_Value,
	 "IFOV_y_Value"},
	{WImage_Quality,
	 "Image_Quality"},
	{WImage_Size,
	 "Image_Size"},
	{WImage_Size_x_Value,
	 "Image_Size_x_Value"},
	{WImage_Size_y_Value,
	 "Image_Size_y_Value"},
	{WInclination,
	 "Inclination"},
	{WIndividual_Band_Identification,
	 "Individual_Band_Identification"},
	{WInput_Dataset_Identifier,
	 "Input_Dataset_Identifier"},
	{WInput_Description,
	 "Input_Description"},
	{WInput_Level,
	 "Input_Level"},
	{WInput_Reference,
	 "Input_Reference"},
	{WINS_Reading_Availability,
	 "INS_Reading_Availability"},
	{WInstantaneous_Field_of_View,
	 "Instantaneous_Field_of_View"},
	{WInstitution_of_Last_Calibration,
	 "Institution_of_Last_Calibration"},
	{WInstrument_Description,
	 "Instrument_Description"},
	{WInstrument_Full_Name,
	 "Instrument_Full_Name"},
	{WInstrument_Information,
	 "Instrument_Information"},
	{WInstrument_Properties_Description,
	 "Instrument_Properties_Description"},
	{WInstrument_Reference,
	 "Instrument_Reference"},
	{WInstrument_Short_Name,
	 "Instrument_Short_Name"},
	{WInstrument_Specific_Georeferencing,
	 "Instrument_Specific_Georeferencing"},
	{WInstrument_Type,
	 "Instrument_Type"},
	{WKeplerian_Orbit,
	 "Keplerian_Orbit"},
	{WLast_Calibration,
	 "Last_Calibration"},
	{WLayer_Name,
	 "Layer_Name"},
	{WLens,
	 "Lens"},
	{WLens_Identifier,
	 "Lens_Identifier"},
	{WLens_Type,
	 "Lens_Type"},
	{WLight_Drop,
	 "Light_Drop"},
	{WLocation_Information,
	 "Location_Information"},
	{WMagazine_Identifier,
	 "Magazine_Identifier"},
	{WMaximum_Wavelength,
	 "Maximum_Wavelength"},
	{WMethod_of_Last_Calibration,
	 "Method_of_Last_Calibration"},
	{WMinimum_Wavelength,
	 "Minimum_Wavelength"},
	{WMission_Completion,
	 "Mission_Completion"},
	{WMission_Description,
	 "Mission_Description"},
	{WMission_History,
	 "Mission_History"},
	{WMission_Information,
	 "Mission_Information"},
	{WMission_Name,
	 "Mission_Name"},
	{WMission_Significant_Event,
	 "Mission_Significant_Event"},
	{WMission_Start_Date,
	 "Mission_Start_Date"},
	{WMultiple_Image_Alignment,
	 "Multiple_Image_Alignment"},
	{WNadir,
	 "Nadir"},
	{WNadir_Latitude,
	 "Nadir_Latitude"},
	{WNadir_Longitude,
	 "Nadir_Longitude"},
	{WName_of_Dimension,
	 "Name_of_Dimension"},
	{WNominal_Geostationary_Position,
	 "Nominal_Geostationary_Position"},
	{WNominal_Spatial_Resolution,
	 "Nominal_Spatial_Resolution"},
	{WNon_Polynomial_Scaling,
	 "Non_Polynomial_Scaling"},
	{WNumber_of_Angle_Distortion_Values,
	 "Number_of_Angle_Distortion_Values"},
	{WNumber_of_Bands,
	 "Number_of_Bands"},
	{WNumber_of_Cross_Track_Samples,
	 "Number_of_Cross_Track_Samples"},
	{WNumber_of_Data_Dimensions,
	 "Number_of_Data_Dimensions"},
	{WNumber_of_Denominator_Terms,
	 "Number_of_Denominator_Terms"},
	{WNumber_of_Distance_Distortion_Values,
	 "Number_of_Distance_Distortion_Values"},
	{WNumber_of_Elevation_Samples,
	 "Number_of_Elevation_Samples"},
	{WNumber_of_Numerator_Terms,
	 "Number_of_Numerator_Terms"},
	{WNumber_of_Points,
	 "Number_of_Points"},
	{WNumber_of_Profile_Samples,
	 "Number_of_Profile_Samples"},
	{WNumber_of_Resolution_Values,
	 "Number_of_Resolution_Values"},
	{WNumber_of_Thematic_Layers,
	 "Number_of_Thematic_Layers"},
	{WNumber_of_Wavelength_Bands,
	 "Number_of_Wavelength_Bands"},
	{WOperational_Mode,
	 "Operational_Mode"},
	{WOrbit_Angle_Units,
	 "Orbit_Angle_Units"},
	{WOrbit_Period,
	 "Orbit_Period"},
	{WOrbit_Period_Units,
	 "Orbit_Period_Units"},
	{WOrbit_Period_Value,
	 "Orbit_Period_Value"},
	{WOther_Collector_Description,
	 "Other_Collector_Description"},
	{WOther_Georeferencing_Description,
	 "Other_Georeferencing_Description"},
	{WPath,
	 "Path"},
	{WPeak_Wavelength,
	 "Peak_Wavelength"},
	{WPercentage_of_Overlap,
	 "Percentage_of_Overlap"},
	{WPerigee_Passage_Time,
	 "Perigee_Passage_Time"},
	{WPhotographic_Resolving_Power,
	 "Photographic_Resolving_Power"},
	{WPitch,
	 "Pitch"},
	{WPitch_Accuracy,
	 "Pitch_Accuracy"},
	{WPixel_Cross_Track_Size,
	 "Pixel_Cross_Track_Size"},
	{WPixel_Description,
	 "Pixel_Description"},
	{WPixel_Elevation_Size,
	 "Pixel_Elevation_Size"},
	{WPixel_Height_Above_Ellipsoid,
	 "Pixel_Height_Above_Ellipsoid"},
	{WPixel_Point_Spread_Function,
	 "Pixel_Point_Spread_Function"},
	{WPixel_Profile_Size,
	 "Pixel_Profile_Size"},
	{WPixel_Resolution,
	 "Pixel_Resolution"},
	{WPlatform_and_Instrument_Identification,
	 "Platform_and_Instrument_Identification"},
	{WPlatform_and_Mission_Information,
	 "Platform_and_Mission_Information"},
	{WPlatform_Description,
	 "Platform_Description"},
	{WPlatform_Full_Name,
	 "Platform_Full_Name"},
	{WPlatform_Information,
	 "Platform_Information"},
	{WPlatform_Nominal_Altitude,
	 "Platform_Nominal_Altitude"},
	{WPlatform_Nominal_Altitude_Units,
	 "Platform_Nominal_Altitude_Units"},
	{WPlatform_Nominal_Altitude_Value,
	 "Platform_Nominal_Altitude_Value"},
	{WPlatform_Nominal_Longitude,
	 "Platform_Nominal_Longitude"},
	{WPlatform_Orbit,
	 "Platform_Orbit"},
	{WPlatform_Serial_Identifier,
	 "Platform_Serial_Identifier"},
	{WPlatform_Short_Name,
	 "Platform_Short_Name"},
	{WPlatform_Sponsor,
	 "Platform_Sponsor"},
	{WPlatform_Start_Date,
	 "Platform_Start_Date"},
	{WPoint_Position_In_Pixel,
	 "Point_Position_In_Pixel"},
	{WPolarization_Characteristics,
	 "Polarization_Characteristics"},
	{WPolynomial_Coefficient,
	 "Polynomial_Coefficient"},
	{WPolynomial_Denominator,
	 "Polynomial_Denominator"},
	{WPolynomial_Function,
	 "Polynomial_Function"},
	{WPolynomial_Numerator,
	 "Polynomial_Numerator"},
	{WPositional_Information,
	 "Positional_Information"},
	{WPrincipal_Point_of_Autocollimation,
	 "Principal_Point_of_Autocollimation"},
	{WPrincipal_Point_of_Symmetry,
	 "Principal_Point_of_Symmetry"},
	{WProcessing_Change_History,
	 "Processing_Change_History"},
	{WProcessing_Documentation,
	 "Processing_Documentation"},
	{WProcessing_Environment,
	 "Processing_Environment"},
	{WProcessing_Identifiers,
	 "Processing_Identifiers"},
	{WProcessing_Information,
	 "Processing_Information"},
	{WProcessing_Input_Dataset,
	 "Processing_Input_Dataset"},
	{WProcessing_Level,
	 "Processing_Level"},
	{WProcessing_Level_Authority,
	 "Processing_Level_Authority"},
	{WProcessing_Level_Identifier,
	 "Processing_Level_Identifier"},
	{WProcessing_Procedure,
	 "Processing_Procedure"},
	{WProcessing_Procedure_Description,
	 "Processing_Procedure_Description"},
	{WProcessing_Run_History,
	 "Processing_Run_History"},
	{WProcessing_Software,
	 "Processing_Software"},
	{WProcessing_Software_Description,
	 "Processing_Software_Description"},
	{WProcessing_Software_Reference,
	 "Processing_Software_Reference"},
	{WProfile_Description,
	 "Profile_Description"},
	{WProfile_Extent,
	 "Profile_Extent"},
	{WProfile_Fixed,
	 "Profile_Fixed"},
	{WProfile_Properties,
	 "Profile_Properties"},
	{WProfile_Sounding,
	 "Profile_Sounding"},
	{WProfile_Start,
	 "Profile_Start"},
	{WProfile_Step,
	 "Profile_Step"},
	{WProfiling_Direction,
	 "Profiling_Direction"},
	{WProjection_Center_X_Position,
	 "Projection_Center_X_Position"},
	{WProjection_Center_Y_Position,
	 "Projection_Center_Y_Position"},
	{WProjection_Center_Z_Position,
	 "Projection_Center_Z_Position"},
	{WQuality_of_Autocollimation_Principal_Point_x_Value,
	 "Quality_of_Autocollimation_Principal_Point_x_Value"},
	{WQuality_of_Autocollimation_Principal_Point_y_Value,
	 "Quality_of_Autocollimation_Principal_Point_y_Value"},
	{WQuality_of_Focal_Length,
	 "Quality_of_Focal_Length"},
	{WQuality_of_Principal_Point_of_Autocollimation,
	 "Quality_of_Principal_Point_of_Autocollimation"},
	{WQuality_of_Principal_Point_of_Symmetry,
	 "Quality_of_Principal_Point_of_Symmetry"},
	{WQuality_of_Symmetry_Principal_Point_x_Value,
	 "Quality_of_Symmetry_Principal_Point_x_Value"},
	{WQuality_of_Symmetry_Principal_Point_y_Value,
	 "Quality_of_Symmetry_Principal_Point_y_Value"},
	{WRadial_Asymmetrical_Coefficient_B1,
	 "Radial_Asymmetrical_Coefficient_B1"},
	{WRadial_Asymmetrical_Coefficient_B2,
	 "Radial_Asymmetrical_Coefficient_B2"},
	{WRadial_Symmetrical_Angle_Interval,
	 "Radial_Symmetrical_Angle_Interval"},
	{WRadial_Symmetrical_Distance_Interval,
	 "Radial_Symmetrical_Distance_Interval"},
	{WRadial_Symmetrical_Distortion_Polynomial,
	 "Radial_Symmetrical_Distortion_Polynomial"},
	{WReceiver_Polarization,
	 "Receiver_Polarization"},
	{WReferenced_Coordinate_System,
	 "Referenced_Coordinate_System"},
	{WReferencing_Polynomial,
	 "Referencing_Polynomial"},
	{WRelative_Aperture,
	 "Relative_Aperture"},
	{WRelative_Azimuth,
	 "Relative_Azimuth"},
	{WReseau,
	 "Reseau"},
	{WResolution_Value_Set,
	 "Resolution_Value_Set"},
	{WResolving_Angle,
	 "Resolving_Angle"},
	{WResolving_Value_Radial,
	 "Resolving_Value_Radial"},
	{WResolving_Value_Tangential,
	 "Resolving_Value_Tangential"},
	{WRight_Ascension_of_Ascending_Node,
	 "Right_Ascension_of_Ascending_Node"},
	{WRoll,
	 "Roll"},
	{WRoll_Accuracy,
	 "Roll_Accuracy"},
	{WRotation_Description,
	 "Rotation_Description"},
	{WRotation_Sequence,
	 "Rotation_Sequence"},
	{WRow,
	 "Row"},
	{WRow_Delta_X,
	 "Row_Delta_X"},
	{WRow_Delta_Y,
	 "Row_Delta_Y"},
	{WSample_Angle_Units,
	 "Sample_Angle_Units"},
	{WSample_Description_Units,
	 "Sample_Description_Units"},
	{WSample_Profile_Units,
	 "Sample_Profile_Units"},
	{WSample_Properties,
	 "Sample_Properties"},
	{WSatellite_Local_Azimuth_Angle,
	 "Satellite/Local_Azimuth_Angle"},
	{WSatellite_Local_Zenith_Angle,
	 "Satellite/Local_Zenith_Angle"},
	{WScan,
	 "Scan"},
	{WScan_Angle_Units,
	 "Scan_Angle_Units"},
	{WScan_Cross_Track_Properties,
	 "Scan_Cross_Track_Properties"},
	{WScan_Distance_Units,
	 "Scan_Distance_Units"},
	{WScan_Duration,
	 "Scan_Duration"},
	{WScan_Elevation_Properties,
	 "Scan_Elevation_Properties"},
	{WScan_Geometric_Properties,
	 "Scan_Geometric_Properties"},
	{WScan_Radiometric_Properties,
	 "Scan_Radiometric_Properties"},
	{WScan_Repeat_Time,
	 "Scan_Repeat_Time"},
	{WScan_Spectral_Properties,
	 "Scan_Spectral_Properties"},
	{WScan_Start_Time,
	 "Scan_Start_Time"},
	{WScan_Step_Time,
	 "Scan_Step_Time"},
	{WScan_Time_Units,
	 "Scan_Time_Units"},
	{WScan_Timing,
	 "Scan_Timing"},
	{WScience_Paper,
	 "Science_Paper"},
	{WSemimajor_Axis,
	 "Semimajor_Axis"},
	{WSender_Polarization,
	 "Sender_Polarization"},
	{WSensor_Element_Location,
	 "Sensor_Element_Location"},
	{WSensor_Grid,
	 "Sensor_Grid"},
	{WSensor_Orientation,
	 "Sensor_Orientation"},
	{WSensor_System,
	 "Sensor_System"},
	{WSolar_Azimuth_Angle,
	 "Solar_Azimuth_Angle"},
	{WSolar_Zenith_Angle,
	 "Solar_Zenith_Angle"},
	{WSource_of_Element_Locations,
	 "Source_of_Element_Locations"},
	{WSpatial_Resolution_Units,
	 "Spatial_Resolution_Units"},
	{WSpatial_Resolution_Value,
	 "Spatial_Resolution_Value"},
	{WSpectral_Information,
	 "Spectral_Information"},
	{WSpectral_Limit,
	 "Spectral_Limit"},
	{WStabilized_Mount,
	 "Stabilized_Mount"},
	{WStorage_Order,
	 "Storage_Order"},
	{WSwath_Track_Information,
	 "Swath_Track_Information"},
	{WThematic_Layer_Identification,
	 "Thematic_Layer_Identification"},
	{WTranslation_Description,
	 "Translation_Description"},
	{WType_of_Overlap,
	 "Type_of_Overlap"},
	{WUnreferenced_Coordinate_System,
	 "Unreferenced_Coordinate_System"},
	{WUsers_Guide,
	 "User's_Guide"},
	{WWavelength_Band_Properties,
	 "Wavelength_Band_Properties"},
	{WWavelength_Region,
	 "Wavelength_Region"},
	{WWavelength_Units,
	 "Wavelength_Units"},
	{WWorldwide_Reference_System,
	 "Worldwide_Reference_System"},
	{WX_Axis_Definition,
	 "X_Axis_Definition"},
	{WX_Position_Accuracy,
	 "X_Position_Accuracy"},
	{WY_Axis_Definition,
	 "Y_Axis_Definition"},
	{WY_Position_Accuracy,
	 "Y_Position_Accuracy"},
	{WYaw,
	 "Yaw"},
	{WYaw_Accuracy,
	 "Yaw_Accuracy"},
	{WZ_Axis_Definition,
	 "Z_Axis_Definition"},
	{WZ_Position_Accuracy,
	 "Z_Position_Accuracy"},

#ifdef USE_ESRI_EXTENSIONS

	/* ESRI extensions */

	{WAltitude_Units,
	 "Altitude_Units"},
	{WAncillary_Role,
	 "Ancillary_Role"},
	{WAncillary_Role_Attribute,
	 "Ancillary_Role_Attribute"},
	{WAttribute_Alias,
	 "Attribute_Alias"},
	{WAttribute_Defined_Domain,
	 "Attribute_Defined_Domain"},
	{WAttribute_Indexed,
	 "Attribute_Indexed"},
	{WAttribute_Number_of_Decimals,
	 "Attribute_Number_of_Decimals"},
	{WAttribute_Output_Width,
	 "Attribute_Output_Width"},
	{WAttribute_Precision,
	 "Attribute_Precision"},
	{WAttribute_Scale,
	 "Attribute_Scale"},
	{WAttribute_Type,
	 "Attribute_Type"},
	{WAttribute_Width,
	 "Attribute_Width"},
	{WAttributed_Relationship,
	 "Attributed_Relationship"},
	{WAvailable_Junction_Feature_Class,
	 "Available_Junction_Feature_Class"},
	{WAvailable_Junction_Subtype,
	 "Available_Junction_Subtype"},
	{WAvailable_Junctions,
	 "Available_Junctions"},
	{WBackground_Nodata_Value,
	 "Background_Nodata_Value"},
	{WBehrmann,
	 "Behrmann"},
	{WBinary_Enclosures,
	 "Binary_Enclosures"},
	{WBits_Per_Pixel,
	 "Bits_Per_Pixel"},
	{WBonne,
	 "Bonne"},
	{WBottom_Bounding_Coordinate,
	 "Bottom_Bounding_Coordinate"},
	{WBrowse_Graphic_Embedded,
	 "Browse_Graphic_Embedded"},
	{WCassini,
	 "Cassini"},
	{WCell_Size_X_Direction,
	 "Cell_Size_X_Direction"},
	{WCell_Size_X_Units,
	 "Cell_Size_X_Units"},
	{WCell_Size_Y_Direction,
	 "Cell_Size_Y_Direction"},
	{WCell_Size_Y_Units,
	 "Cell_Size_Y_Units"},
	{WComposite_Relationship,
	 "Composite_Relationship"},
	{WCompression_Type,
	 "Compression_Type"},
	{WConnectivity_Rule,
	 "Connectivity_Rule"},
	{WCoordinate_System_Name,
	 "Coordinate_System_Name"},
	{WCreation_Date,
	 "Creation_Date"},
	{WCreation_Time,
	 "Creation_Time"},
	{WData,
	 "Data"},
	{WData_Frame_Count,
	 "Data_Frame_Count"},
	{WData_Frame_Name,
	 "Data_Frame_Name"},
	{WData_Frames,
	 "Data_Frames"},
	{WDatabase_Name,
	 "Database_Name"},
	{WDataset_Size,
	 "Dataset_Size"},
	{WDefault_Junction_Feature_Class,
	 "Default_Junction_Feature_Class"},
	{WDefault_Junction_Subtype,
	 "Default_Junction_Subtype"},
	{WDescription_of_Association,
	 "Description_of_Association"},
	{WDescription_of_Enclosure,
	 "Description_of_Enclosure"},
	{WDescription_of_Relationship,
	 "Description_of_Relationship"},
	{WDestination_Foreign_Key,
	 "Destination_Foreign_Key"},
	{WDestination_Name,
	 "Destination_Name"},
	{WDestination_Primary_Key,
	 "Destination_Primary_Key"},
	{WDomain_Attribute_Type,
	 "Domain_Attribute_Type"},
	{WDomain_Description,
	 "Domain_Description"},
	{WDomain_Name,
	 "Domain_Name"},
	{WDomain_Owner,
	 "Domain_Owner"},
	{WDomain_Type,
	 "Domain_Type"},
	{WEckert_I,
	 "Eckert_I"},
	{WEckert_II,
	 "Eckert_II"},
	{WEckert_III,
	 "Eckert_III"},
	{WEckert_IV,
	 "Eckert_IV"},
	{WEckert_V,
	 "Eckert_V"},
	{WEckert_VI,
	 "Eckert_VI"},
	{WEdge_Feature_Class,
	 "Edge_Feature_Class"},
	{WEdge_Maximum_Cardinality,
	 "Edge_Maximum_Cardinality"},
	{WEdge_Minimum_Cardinality,
	 "Edge_Minimum_Cardinality"},
	{WEdge_Subtype,
	 "Edge_Subtype"},
	{WEnabled_Attribute,
	 "Enabled_Attribute"},
	{WEnclosure,
	 "Enclosure"},
	{WEntity_Type_Count,
	 "Entity_Type_Count"},
	{WEntity_Type_Type,
	 "Entity_Type_Type"},
	{WESRI_Feature_Count,
	 "ESRI_Feature_Count"},
	{WESRI_Feature_Geometry,
	 "ESRI_Feature_Geometry"},
	{WESRI_Feature_Type,
	 "ESRI_Feature_Type"},
	{WESRI_Information,
	 "ESRI_Information"},
	{WESRI_Terms_Description,
	 "ESRI_Terms_Description"},
	{WESRI_Topology,
	 "ESRI_Topology"},
	{WFeature_Description,
	 "Feature_Description"},
	{WFile_or_Table_Name,
	 "File_or_Table_Name"},
	{WFrom_Edge_Feature_Class,
	 "From_Edge_Feature_Class"},
	{WFrom_Edge_Subtype,
	 "From_Edge_Subtype"},
	{WGall_Stereographic,
	 "Gall_Stereographic"},
	{WGeocoding_Information,
	 "Geocoding_Information"},
	{WGeographic_Coordinate_System_Name,
	 "Geographic_Coordinate_System_Name"},
	{WGeometric_Network_Information,
	 "Geometric_Network_Information"},
	{WImage_Colormap,
	 "Image_Colormap"},
	{WImage_Format,
	 "Image_Format"},
	{WImage_Type,
	 "Image_Type"},
	{WInstance_Name,
	 "Instance_Name"},
	{WJunction_Feature_Class,
	 "Junction_Feature_Class"},
	{WJunction_Maximum_Cardinality,
	 "Junction_Maximum_Cardinality"},
	{WJunction_Minimum_Cardinality,
	 "Junction_Minimum_Cardinality"},
	{WJunction_Subtype,
	 "Junction_Subtype"},
	{WLanguage_of_Dataset,
	 "Language_of_Dataset"},
	{WLanguage_of_Metadata,
	 "Language_of_Metadata"},
	{WLeft_Bounding_Coordinate,
	 "Left_Bounding_Coordinate"},
	{WLinear_Referencing,
	 "Linear_Referencing"},
	{WLocal_Bounding_Coordinates,
	 "Local_Bounding_Coordinates"},
	{WLoximuthal,
	 "Loximuthal"},
	{WMaximum_Altitude,
	 "Maximum_Altitude"},
	{WMerge_Rule,
	 "Merge_Rule"},
	{WMetadata_ID,
	 "Metadata_ID"},
	{WMinimum_Altitude,
	 "Minimum_Altitude"},
	{WModification_Date,
	 "Modification_Date"},
	{WModification_Time,
	 "Modification_Time"},
	{WMollweide,
	 "Mollweide"},
	{WNative_Dataset_Format,
	 "Native_Dataset_Format"},
	{WNetwork_Element,
	 "Network_Element"},
	{WNetwork_Role,
	 "Network_Role"},
	{WNetwork_Type,
	 "Network_Type"},
	{WNotification_Direction,
	 "Notification_Direction"},
	{WNumber_of_Raster_Bands,
	 "Number_of_Raster_Bands"},
	{WOrigin_Foreign_Key,
	 "Origin_Foreign_Key"},
	{WOrigin_Name,
	 "Origin_Name"},
	{WOrigin_Primary_Key,
	 "Origin_Primary_Key"},
	{WProcess_Software_and_Version,
	 "Process_Software_and_Version"},
	{WProjected_Coordinate_System_Name,
	 "Projected_Coordinate_System_Name"},
	{WPyramid_Layers,
	 "Pyramid_Layers"},
	{WQuartic_Authalic,
	 "Quartic_Authalic"},
	{WRange_Domain_Mean,
	 "Range_Domain_Mean"},
	{WRange_Domain_Standard_Deviation,
	 "Range_Domain_Standard_Deviation"},
	{WRaster_Display_Type,
	 "Raster_Display_Type"},
	{WRaster_Origin,
	 "Raster_Origin"},
	{WRelationship_Backward_Label,
	 "Relationship_Backward_Label"},
	{WRelationship_Cardinality,
	 "Relationship_Cardinality"},
	{WRelationship_Forward_Label,
	 "Relationship_Forward_Label"},
	{WRelationship_Information,
	 "Relationship_Information"},
	{WRight_Bounding_Coordinate,
	 "Right_Bounding_Coordinate"},
	{WRule_Category,
	 "Rule_Category"},
	{WRule_Help,
	 "Rule_Help"},
	{WRule_Type,
	 "Rule_Type"},
	{WSDE_Connection_Information,
	 "SDE_Connection_Information"},
	{WServer_Name,
	 "Server_Name"},
	{WSpatial_Index,
	 "Spatial_Index"},
	{WSplit_Rule,
	 "Split_Rule"},
	{WSubtype_Attribute,
	 "Subtype_Attribute"},
	{WSubtype_Attribute_Name,
	 "Subtype_Attribute_Name"},
	{WSubtype_Code,
	 "Subtype_Code"},
	{WSubtype_Default_Value,
	 "Subtype_Default_Value"},
	{WSubtype_Information,
	 "Subtype_Information"},
	{WSubtype_Name,
	 "Subtype_Name"},
	{WSynchronization_Date,
	 "Synchronization_Date"},
	{WSynchronization_Time,
	 "Synchronization_Time"},
	{WSynchronize_Automatically,
	 "Synchronize_Automatically"},
	{WSynchronize_Once,
	 "Synchronize_Once"},
	{WThumbnail,
	 "Thumbnail"},
	{WTo_Edge_Feature_Class,
	 "To_Edge_Feature_Class"},
	{WTo_Edge_Subtype,
	 "To_Edge_Subtype"},
	{WTop_Bounding_Coordinate,
	 "Top_Bounding_Coordinate"},
	{WUser_Name,
	 "User_Name"},
	{WVersion_Name,
	 "Version_Name"},
	{WWinkel_I,
	 "Winkel_I"},
	{WWinkel_II,
	 "Winkel_II"},
	{WGeographic_Box,
	 "Geographic_Box"},

#endif

	{Wunknown, NULL}
	};

static struct keyword_list de_std[] = {
	{WAbscissa_Resolution,
	 "Abszisse_Auflsung"},
	{WAbstract,
	 "Zusammenfassung"},
	{WAccess_Constraints,
	 "Zugriffs_Beschrnkungen"},
	{WAccess_Instructions,
	 "Zugriffs_Anweisungen"},
	{WAddress,
	 "Adresse"},
	{WAddress_Type,
	 "Adressentyp"},
	{WAlbers_Conical_Equal_Area,
	 "Albers_Konisch_Flchentreu"},
	{WAltitude_Datum_Name,
	 "Hhen_Datum_Name"},
	{WAltitude_Distance_Units,
	 "Hhen_Abstands_Einheit"},
	{WAltitude_Encoding_Method,
	 "Hhen_Kodierung_Methode"},
	{WAltitude_Resolution,
	 "Hhen_Auflsung"},
	{WAltitude_System_Definition,
	 "Hhen_System_Definition"},
	{WARC_Coordinate_System,
	 "ARC_Koordinatensystem"},
	{WARC_System_Zone_Identifier,
	 "ARC_System_Zonenidentifikator"},
	{WAttribute,
	 "Attribute"},
	{WAttribute_Accuracy,
	 "Attribut_Genauigkeit"},
	{WAttribute_Accuracy_Explanation,
	 "Attribut_Genauigkeit_Erklrung"},
	{WAttribute_Accuracy_Report,
	 "Attribut_Genauigkeits_Bericht"},
	{WAttribute_Accuracy_Value,
	 "Attribut_Genauigkeits_Wert"},
	{WAttribute_Definition,
	 "Attribut_Definition"},
	{WAttribute_Definition_Source,
	 "Attribut_Definition_Quelle"},
	{WAttribute_Domain_Values,
	 "Attribute_Domne_Werte"},
	{WAttribute_Label,
	 "Attribut_Bezeichnung"},
	{WAttribute_Measurement_Frequency,
	 "Attribut_Messungs_Frequenz"},
	{WAttribute_Measurement_Resolution,
	 "Attribut_Messung_Genauigkeit"},
	{WAttribute_Units_of_Measure,
	 "Attribut_MessEinheiten"},
	{WAttribute_Value_Accuracy,
	 "Attribut_Wert_Genauigkeit"},
	{WAttribute_Value_Accuracy_Explanation,
	 "Attribut_Wert_Genauigkeits_Beschreibung"},
	{WAttribute_Value_Accuracy_Information,
	 "Attribut_Werte_Genauigkeit_Information"},
	{WAvailable_Time_Period,
	 "Verfgbarkeit_Zeit_Periode"},
	{WAzimuthal_Angle,
	 "Azimuthal_Winkel"},
	{WAzimuthal_Equidistant,
	 "Azimuthal_quidistant"},
	{WAzimuth_Measure_Point_Longitude,
	 "Azimuth_Mepunkt_Lngengrad"},
	{WBearing_Reference_Direction,
	 "Winkel_Bezug_Richtung"},
	{WBearing_Reference_Meridian,
	 "Winkel_Bezug_Meridian"},
	{WBearing_Resolution,
	 "Winkel_Auflsung"},
	{WBearing_Units,
	 "Winkel_Einheit"},
	{WBeginning_Date,
	 "Anfangsdatum"},
	{WBeginning_Date_of_Attribute_Values,
	 "AnfangsDatum_fr_Attribut_Werte"},
	{WBeginning_Time,
	 "Anfangszeit"},
	{WBounding_Coordinates,
	 "Begrenzungskoordinaten"},
	{WBrowse_Graphic,
	 "Vorschau_Graphik"},
	{WBrowse_Graphic_File_Description,
	 "Vorschau_Graphik_Datei_Beschreibung"},
	{WBrowse_Graphic_File_Name,
	 "Vorschau_Graphik_Datei_Name"},
	{WBrowse_Graphic_File_Type,
	 "Vorschau_Graphik_Datei_Typ"},
	{WCalendar_Date,
	 "Kalender_Datum"},
	{WCitation,
	 "Zitatangaben"},
	{WCitation_Information,
	 "Zitat_Information"},
	{WCity,
	 "Stadt"},
	{WCloud_Cover,
	 "Wolkenbedeckung"},
	{WCodeset_Domain,
	 "Codeset_Domain"},
	{WCodeset_Name,
	 "Codeset_Name"},
	{WCodeset_Source,
	 "Codeset_Quelle"},
	{WColumn_Count,
	 "Spalten_Anzahl"},
	{WCompatibility_Information,
	 "Kompatibilitts_Information"},
	{WCompleteness_Report,
	 "VollstnigkeitsBericht"},
	{WCompression_Support,
	 "VerdichtungsHilfe"},
	{WComputer_Contact_Information,
	 "Computer_Kontakt_Information"},
	{WContact_Address,
	 "Kontaktadresse"},
	{WContact_Electronic_Mail_Address,
	 "KontaktEmailAdresse"},
	{WContact_Facsimile_Telephone,
	 "Contact_FAX"},
	{WContact_Information,
	 "Kontakt_Information"},
	{WContact_Instructions,
	 "Kontaktanweisungen"},
	{WContact_Organization,
	 "Kontakt_Organisation"},
	{WContact_Organization_Primary,
	 "Kontakt_Organisation_primr"},
	{WContact_Person,
	 "Kontakt_Person"},
	{WContact_Person_Primary,
	 "Kontakt_Person_Primr"},
	{WContact_Position,
	 "Kontaktstellung"},
	{WContact_TDD_TTY_Telephone,
	 "Kontakt_TDD/TTY_Telefon"},
	{WContact_Voice_Telephone,
	 "Contact_Mitsprache_Telefon"},
	{WCoordinate_Representation,
	 "Koordinaten_Representation"},
	{WCountry,
	 "Land"},
	{WCross_Reference,
	 "Kreuz_Verweis"},
	{WCurrentness_Reference,
	 "Aktualitt_Referenz"},
	{WCustom_Order_Process,
	 "Kunden_Bestellung_Prozess"},
	{WData_Quality_Information,
	 "Daten_Qualitt_Information"},
	{WData_Set_Credit,
	 "Datensatz_Danksagung"},
	{WData_Set_G_Polygon,
	 "Datensatz_G-Polygon"},
	{WData_Set_G_Polygon_Exclusion_G_Ring,
	 "Datensatz_G-Polygon_Ausschlieender_G-Ring"},
	{WData_Set_G_Polygon_Outer_G_Ring,
	 "Datensatz_G-Polygon_uerer_G-Ring"},
	{WDenominator_of_Flattening_Ratio,
	 "Nenner_der_Abplattung"},
	{WDepth_Datum_Name,
	 "Tiefen_Datum_Name"},
	{WDepth_Distance_Units,
	 "Tiefen_Abstands_Einheit"},
	{WDepth_Encoding_Method,
	 "Tiefen_Kodierungs_Methode"},
	{WDepth_Resolution,
	 "Tiefen_Auflsung"},
	{WDepth_System_Definition,
	 "Tiefen_System_Definition"},
	{WDescription,
	 "Beschreibung"},
	{WDetailed_Description,
	 "Detaillierte_Beschreibung"},
	{WDialup_File_Name,
	 "Wahl_File_Name"},
	{WDialup_Instructions,
	 "Einwahl_Anweisungen"},
	{WDialup_Telephone,
	 "Einwahl_Telephon"},
	{WDigital_Form,
	 "Digitale_Form"},
	{WDigital_Transfer_Information,
	 "Digitale_Austausch_Information"},
	{WDigital_Transfer_Option,
	 "Digitale_Austausch_Optionen"},
	{WDirect_Spatial_Reference_Method,
	 "Direkte_Rumliche_Referenz_Methode"},
	{WDistance_and_Bearing_Representation,
	 "Entfernung_und_Winkel_Darstellung"},
	{WDistance_Resolution,
	 "Abstands_Auflsung"},
	{WDistribution_Information,
	 "Verteilung_Information"},
	{WDistribution_Liability,
	 "Verteiler_Haftungserklrung"},
	{WDistributor,
	 "Verteiler"},
	{WEast_Bounding_Coordinate,
	 "Ost_Begrenzungs_Koordinate"},
	{WEdition,
	 "Version"},
	{WEllipsoid_Name,
	 "Ellipsoid_Name"},
	{WEnding_Date,
	 "Enddatum"},
	{WEnding_Date_of_Attribute_Values,
	 "EndDatum_fr_Attribut_Werte"},
	{WEnding_Time,
	 "Endzeit"},
	{WEntity_and_Attribute_Detail_Citation,
	 "Entity_und_Attribut_Detail_Zitierung"},
	{WEntity_and_Attribute_Information,
	 "Entity_und_Attribut_Information"},
	{WEntity_and_Attribute_Overview,
	 "Entity_und_Attribut_bersicht"},
	{WEntity_Type,
	 "Entity_Typ"},
	{WEntity_Type_Definition,
	 "Entity_Typ_Definition"},
	{WEntity_Type_Definition_Source,
	 "Entity_Typ_Definition_Quelle"},
	{WEntity_Type_Label,
	 "Entity_Typ_Bezeichnung"},
	{WEnumerated_Domain,
	 "Aufgezhlte"},
	{WEnumerated_Domain_Value,
	 "Aufgezhlter_Funktionswert"},
	{WEnumerated_Domain_Value_Definition,
	 "Aufgezhlter_Funktionswert_Definition"},
	{WEnumerated_Domain_Value_Definition_Source,
	 "Aufgezhlter_Funktionswert_Definitionsquelle"},
	{WEquidistant_Conic,
	 "quidistant_Konisch"},
	{WEquirectangular,
	 "Equirectangular"},
	{WFalse_Easting,
	 "Addition_Rechtswert"},
	{WFalse_Northing,
	 "Addition_Hochwert"},
	{WFees,
	 "Gebhren"},
	{WFile_Decompression_Technique,
	 "Datei_Dekompressions_Technik"},
	{WFormat_Information_Content,
	 "Format_InformationsGehalt"},
	{WFormat_Name,
	 "Format_Name"},
	{WFormat_Specification,
	 "Format_Spezifizierung"},
	{WFormat_Version_Date,
	 "Format_Versions_Datum"},
	{WFormat_Version_Number,
	 "Format_Versions_Nummer"},
	{WG_Ring,
	 "G-Ring"},
	{WG_Ring_Latitude,
	 "G-Ring_Lnge"},
	{WG_Ring_Longitude,
	 "G-Ring_Breite"},
	{WG_Ring_Point,
	 "G-Ring_Punkt"},
	{WGeneral_Vertical_Near_sided_Perspective,
	 "Allgemeine_Vertikale_Ortsnahe_ Perspektive"},
	{WGeodetic_Model,
	 "Geodtisches_Modell"},
	{WGeographic,
	 "Geographisch"},
	{WGeographic_Coordinate_Units,
	 "Geographische_KoordinatenEinheiten"},
	{WGeospatial_Data_Presentation_Form,
	 "Georumliche_Daten_Darstellung_Form"},
	{WGnomonic,
	 "Gnomoic"},
	{WGrid_Coordinate_System,
	 "Raster_Koordinatensystem"},
	{WGrid_Coordinate_System_Name,
	 "Raster_Koordinatensystem_Name"},
	{WHeight_of_Perspective_Point_Above_Surface,
	 "Hhe_des_Blickpunktes_ber_Oberflche"},
	{WHighest_BPS,
	 "Hchste_BPS"},
	{WHorizontal_Coordinate_System_Definition,
	 "Horizontales_Koordinatensystem_Definition"},
	{WHorizontal_Datum_Name,
	 "Horizontales_Datum_Name"},
	{WHorizontal_Positional_Accuracy,
	 "Horizontale_Position_Genauigkeit"},
	{WHorizontal_Positional_Accuracy_Explanation,
	 "Horizontale_Position_Genauigkeit_Erklrung"},
	{WHorizontal_Positional_Accuracy_Report,
	 "Horizontale_Position_Genauigkeit_Bericht"},
	{WHorizontal_Positional_Accuracy_Value,
	 "Horizontale_Position_Genauigkeit_Wert"},
	{WHours_of_Service,
	 "Sprechstunden"},
	{WIdentification_Information,
	 "Identifikation_Information"},
	{WIndirect_Spatial_Reference,
	 "Indirekte_Rumliche_Referenzen"},
	{WIssue_Identification,
	 "Themen_Identifikation"},
	{WKeywords,
	 "Schlagworte"},
	{WLambert_Azimuthal_Equal_Area,
	 "Lambert_Azimuthal_Flchentreu"},
	{WLambert_Conformal_Conic,
	 "Lambert_Konform_Konisch"},
	{WLandsat_Number,
	 "Landsat_Nummer"},
	{WLarger_Work_Citation,
	 "bergeordnete_Arbeit_Quelle"},
	{WLatitude_of_Projection_Center,
	 "Breitengrad_des_Projektionszentrums"},
	{WLatitude_of_Projection_Origin,
	 "Breite_Projektion_Ursprung"},
	{WLatitude_Resolution,
	 "Breitengrad_Auflsung"},
	{WLineage,
	 "Abstammung"},
	{WLocal,
	 "Lokal"},
	{WLocal_Description,
	 "Lokal_Beschreibung"},
	{WLocal_Georeference_Information,
	 "Lokal_Georeferenzierung_Information"},
	{WLocal_Planar,
	 "Lokal_Planar"},
	{WLocal_Planar_Description,
	 "Lokal_Planar_Beschreibung"},
	{WLocal_Planar_Georeference_Information,
	 "Lokale_Planare_Georeferenz_Information"},
	{WLogical_Consistency_Report,
	 "Logischer_Konsistenz_Report"},
	{WLongitude_of_Central_Meridian,
	 "Lngengrad_des_Zentralen_Meridians"},
	{WLongitude_of_Projection_Center,
	 "Lngengrad_des_Projektionszentrums"},
	{WLongitude_Resolution,
	 "Lngengrad_Auflsung"},
	{WLowest_BPS,
	 "Geringste_BPS"},
	{WMaintenance_and_Update_Frequency,
	 "Instandhaltungs_und_AktualisierungsFrequenz"},
	{WMap_Projection,
	 "KartenProjektion"},
	{WMap_Projection_Name,
	 "KartenProjektionsName"},
	{WMap_Projection_Parameters,
	 "Karten_Projektion_Parameter"},
	{WMercator,
	 "Mercator"},
	{WMetadata,
	 "Metadaten"},
	{WMetadata_Access_Constraints,
	 "Metadaten_Zugriffs_Bedingungen"},
	{WMetadata_Contact,
	 "Metadaten_Kontakt"},
	{WMetadata_Date,
	 "Metadata_Datum"},
	{WMetadata_Extensions,
	 "Metadaten_Erweiterung"},
	{WMetadata_Future_Review_Date,
	 "Metadata_Nchstes_Prfung_Datum"},
	{WMetadata_Language,
	 "Metadata_Sprache"},
	{WMetadata_Reference_Information,
	 "Metadata_Referenz_Information"},
	{WMetadata_Review_Date,
	 "Metadata_Prfung_Datum"},
	{WMetadata_Security_Classification,
	 "Metadaten_Sicherheit_Klassifikation"},
	{WMetadata_Security_Classification_System,
	 "Metadaten_Sicherheit_Klassifikation_System"},
	{WMetadata_Security_Handling_Description,
	 "Metadaten_Sicherheit_Handhabung_Beschreibung"},
	{WMetadata_Security_Information,
	 "Metadaten_Sicherheits_Informationen"},
	{WMetadata_Standard_Name,
	 "Metadaten_Standard_Name"},
	{WMetadata_Standard_Version,
	 "Metadaten_Standard_Version"},
	{WMetadata_Time_Convention,
	 "Metadaten_Zeit_Konvention"},
	{WMetadata_Use_Constraints,
	 "Metadaten_Nutzungs_Einschrnkungen"},
	{WMiller_Cylindrical,
	 "Miller_Zylindrisch"},
	{WModified_Stereographic_for_Alaska,
	 "Modifiziert_Stereographisch_fr_Alaska"},
	{WMultiple_Dates_Times,
	 "Mehrfaches_Datum/Zeit"},
	{WNative_Data_Set_Environment,
	 "Gewhnliche_Datensatz_Umgebung"},
	{WNetwork_Address,
	 "Netzwerk_Adresse"},
	{WNetwork_Resource_Name,
	 "NetzwerkQuelle_Name"},
	{WNon_digital_Form,
	 "Nicht-Digitale_Form"},
	{WNorth_Bounding_Coordinate,
	 "Nord_Begrenzungs_Koordinate"},
	{WNumber_DataBits,
	 "Anzahl_DatenStcke"},
	{WNumber_StopBits,
	 "Anzahl_Stopstcke"},
	{WOblique_Line_Azimuth,
	 "Schiefe_Linie_Azimuth"},
	{WOblique_Line_Latitude,
	 "Schiefe_Linie_Breite"},
	{WOblique_Line_Longitude,
	 "Schiefe_Linie_Lnge"},
	{WOblique_Line_Point,
	 "Schiefe_Linie_Punkt"},
	{WOblique_Mercator,
	 "Schiefe_Mercator"},
	{WOffline_Media,
	 "Offline_Medium"},
	{WOffline_Option,
	 "Offline_Option"},
	{WOnline_Computer_and_Operating_System,
	 "Online_Computer_und_Betriebssystem"},
	{WOnline_Linkage,
	 "Online_Verlinkung"},
	{WOnline_Option,
	 "Online_Option"},
	{WOrdering_Instructions,
	 "Bestell_Anleitung"},
	{WOrdinate_Resolution,
	 "Ordinate_Auflsung"},
	{WOriginator,
	 "Urheber"},
	{WOrthographic,
	 "Orthographisch"},
	{WOther_Citation_Details,
	 "Andere_Zitat_Details"},
	{WOther_Grid_Systems_Definition,
	 "Definition_Anderer_Rastersysteme"},
	{WOther_Projections_Definition,
	 "Other_Projection's_Definition"},
	{WOverview_Description,
	 "berblick_Beschreibung"},
	{WParity,
	 "Paritt"},
	{WPath_Number,
	 "Bahn_Nummer"},
	{WPlace,
	 "Ort"},
	{WPlace_Keyword,
	 "Ort_Schlagwort"},
	{WPlace_Keyword_Thesaurus,
	 "Ort_Schlagwort_Thesaurus"},
	{WPlanar,
	 "Eben"},
	{WPlanar_Coordinate_Encoding_Method,
	 "Planar_Koordinate_Verschlsselung_Methode"},
	{WPlanar_Coordinate_Information,
	 "Planar_Koordinate_Information"},
	{WPlanar_Distance_Units,
	 "Planar_Entfernung"},
	{WPoint_and_Vector_Object_Count,
	 "Punkt_und_Vektor_Objekt_Anzahl"},
	{WPoint_and_Vector_Object_Information,
	 "Punkt_und_Vektor_Objekt_Information"},
	{WPoint_of_Contact,
	 "Kontakt_Punkt"},
	{WPolar_Stereographic,
	 "Polar_Stereographisch"},
	{WPolyconic,
	 "Polykonisch"},
	{WPositional_Accuracy,
	 "Position_Genauigkeit"},
	{WPostal_Code,
	 "Postleitzahl"},
	{WProcess_Contact,
	 "Prozess_Kontakt"},
	{WProcess_Date,
	 "Prozess_Datum"},
	{WProcess_Description,
	 "Prozess_Beschreibung"},
	{WProcess_Step,
	 "Prozess_Schritt"},
	{WProcess_Time,
	 "Prozess_Zeit"},
	{WProfile_Name,
	 "Profil_Name"},
	{WProgress,
	 "Fortschritt"},
	{WPublication_Date,
	 "Herausgabe_Datum"},
	{WPublication_Information,
	 "Herausgabe_Information"},
	{WPublication_Place,
	 "Publikation_Ort"},
	{WPublication_Time,
	 "Herausgabe_Zeit"},
	{WPublisher,
	 "Herausgeber"},
	{WPurpose,
	 "Zweck"},
	{WQuantitative_Attribute_Accuracy_Assessment,
	 "Quantitative_Attribut_Genauigkeit_Abschtzung"},
	{WQuantitative_Horizontal_Positional_Accuracy_Assessment,
	 "Quantitative_Horizontale_Position_Genauigkeit"},
	{WQuantitative_Vertical_Positional_Accuracy_Assessment,
	 "Quantitative_Vertical_Positional_Genauigkeit_Einschtzung"},
	{WRange_Domain,
	 "Funktionsbereich"},
	{WRange_Domain_Maximum,
	 "FunktionsBereich_Maximum"},
	{WRange_Domain_Minimum,
	 "FunktionsBereich_Minimum"},
	{WRange_of_Dates_Times,
	 "Daten/Zeit_Spanne"},
	{WRaster_Object_Information,
	 "Raster_Objekt_Information"},
	{WRaster_Object_Type,
	 "Raster_Objekt_Typ"},
	{WRecording_Capacity,
	 "Speicher_Kapazitt"},
	{WRecording_Density,
	 "Aufnahme_Dichte"},
	{WRecording_Density_Units,
	 "Aufnahme_Einheiten_Dichte"},
	{WRecording_Format,
	 "Aufnahme_Format"},
	{WResource_Description,
	 "Quellenbeschreibung"},
	{WRobinson,
	 "Robinson"},
	{WRow_Count,
	 "Reihen_Anzahl"},
	{WScale_Factor_at_Center_Line,
	 "Mastabsfaktor_an_Mittellinie"},
	{WScale_Factor_at_Central_Meridian,
	 "Mastabsfaktor_Am_Mittleren_Meridian"},
	{WScale_Factor_at_Equator,
	 "Mastabsfaktor_Am_quator"},
	{WScale_Factor_at_Projection_Origin,
	 "Mastabsfaktor_Am_Projektionsursprung"},
	{WSDTS_Point_and_Vector_Object_Type,
	 "SDTS_Punkt_und_Vektor_ObjektTyp"},
	{WSDTS_Terms_Description,
	 "SDTS_Termini_Erluterung"},
	{WSecurity_Classification,
	 "Sicherheits_Klassifizierung"},
	{WSecurity_Classification_System,
	 "Sicherheits_Klassiefizierungs_System"},
	{WSecurity_Handling_Description,
	 "Sicherheit_Beschrnkung_Beschreibung"},
	{WSecurity_Information,
	 "Sicherheits_Informationen"},
	{WSemi_major_Axis,
	 "Groe_Halbachse"},
	{WSeries_Information,
	 "Serien_Information"},
	{WSeries_Name,
	 "Serien_Name"},
	{WSingle_Date_Time,
	 "Einzeln_Datum/Zeit"},
	{WSinusoidal,
	 "Sinusfrmig"},
	{WSource_Citation,
	 "Quellen_Zitatangaben"},
	{WSource_Citation_Abbreviation,
	 "Abkrzung_Quelle"},
	{WSource_Contribution,
	 "Quellen_Beitrag"},
	{WSource_Currentness_Reference,
	 "Quellen_Aktualitt_Referenz"},
	{WSource_Information,
	 "Quellen_Information"},
	{WSource_Produced_Citation_Abbreviation,
	 "Abkrzung_Produzierte_Quelle"},
	{WSource_Scale_Denominator,
	 "Quellen_Masstab_Nenner"},
	{WSource_Time_Period_of_Content,
	 "Quelle_Zeitspanne_des_Inhaltes"},
	{WSource_Used_Citation_Abbreviation,
	 "Abkrzung_Genutzte_Quelle"},
	{WSouth_Bounding_Coordinate,
	 "Sd_Begrenzungs_Koordinate"},
	{WSpace_Oblique_Mercator_Landsat,
	 "Raumabhngiger_Mercator_(Landsat)"},
	{WSpatial_Data_Organization_Information,
	 "Rumliche_Daten_Organisation_Information"},
	{WSpatial_Domain,
	 "Rumliche_Ausdehnung"},
	{WSpatial_Reference_Information,
	 "Rumliche_Referenz_Information"},
	{WSPCS_Zone_Identifier,
	 "SPCS_Zonen_Identifikator"},
	{WStandard_Order_Process,
	 "Standard_Bestell_Prozess"},
	{WStandard_Parallel,
	 "Standard_Parallel"},
	{WState_or_Province,
	 "Bundesland_oder_Region"},
	{WState_Plane_Coordinate_System,
	 "State_Plane_Coordinate_System_(SPCS)"},
	{WStatus,
	 "Status"},
	{WStereographic,
	 "Stereographisch"},
	{WStraight_Vertical_Longitude_from_Pole,
	 "Gerader_Vertikaler_Lngengrad_Vom_Pol"},
	{WStratum,
	 "Ebene"},
	{WStratum_Keyword,
	 "Ebene_Schlagwort"},
	{WStratum_Keyword_Thesaurus,
	 "Ebene_Schlagwort_Thesaurus"},
	{WSupplemental_Information,
	 "Ergnzende_Information"},
	{WTechnical_Prerequisites,
	 "Technische_Grundvoraussetungen"},
	{WTemporal,
	 "Zeitlich"},
	{WTemporal_Keyword,
	 "Zeitliches_Schlagwort"},
	{WTemporal_Keyword_Thesaurus,
	 "Zeitlicher_Schlagwort_Thesaurus"},
	{WTheme,
	 "Thema"},
	{WTheme_Keyword,
	 "Thema_Schlagwort"},
	{WTheme_Keyword_Thesaurus,
	 "Thema_Schlagwort_Thesaurus"},
	{WTime_of_Day,
	 "Zeit_des_Tages"},
	{WTime_Period_Information,
	 "Zeit_Periode_Information"},
	{WTime_Period_of_Content,
	 "Zeit_Periode_des_Inhalts"},
	{WTitle,
	 "Titel"},
	{WTransfer_Size,
	 "Transfer_Groesse"},
	{WTransverse_Mercator,
	 "Transversaler_Mercator"},
	{WTurnaround,
	 "Bearbeitungszeit"},
	{WType_of_Source_Media,
	 "Quellen_Medium_Typ"},
	{WUniversal_Polar_Stereographic,
	 "Universal_Polar_Stereographisch_(UPS)"},
	{WUniversal_Transverse_Mercator,
	 "Universal_Transversaler_Mercator_(UTM)"},
	{WUnrepresentable_Domain,
	 "Nicht_Darzustellende_Funktion"},
	{WUPS_Zone_Identifier,
	 "UPS_Zonen_Identifikator"},
	{WUse_Constraints,
	 "Nutzungs_Beschrnkungen"},
	{WUTM_Zone_Number,
	 "UTM_Zonen_Nummer"},
	{Wvan_der_Grinten,
	 "van_der_Grinten"},
	{WVertical_Coordinate_System_Definition,
	 "Vertikale_Koordinatensystem_Definition"},
	{WVertical_Count,
	 "Vertikal_Anzahl"},
	{WVertical_Positional_Accuracy,
	 "Vertikale_Position_Genauigkeit"},
	{WVertical_Positional_Accuracy_Explanation,
	 "Vertikale_Position_Genauigkeit_Erklrung"},
	{WVertical_Positional_Accuracy_Report,
	 "Vertikale_Positions_Genauigkeit_Bericht"},
	{WVertical_Positional_Accuracy_Value,
	 "Vertikale_Position_Genauigkeit_Wert"},
	{WVPF_Point_and_Vector_Object_Information,
	 "VPF_Punkt_und_Vektor_Objekt_Information"},
	{WVPF_Point_and_Vector_Object_Type,
	 "VPF_Punkt_und_Vektor_Objekt_Typ"},
	{WVPF_Terms_Description,
	 "VPF_Werte_Beschreibung"},
	{WVPF_Topology_Level,
	 "VPF_Struktur_Level"},
	{WWest_Bounding_Coordinate,
	 "West_Begrenzungs_Koordinate"},

/* Biological Data Profile */

	{WDescription_of_Geographic_Extent,
		"Description_of_Geographic_Extent"},
	{WBounding_Altitudes,
		"Bounding_Altitudes"},
	{WAltitude_Minimum,
		"Altitude_Minimum"},
	{WAltitude_Maximum,
		"Altitude_Maximum"},
	{WTaxonomy,
		"Taxonomy"},
	{WKeywords_Taxon,
		"Keywords/Taxon"},
	{WTaxonomic_Keyword_Thesaurus,
		"Taxonomic_Keyword_Thesaurus"},
	{WTaxonomic_Keywords,
		"Taxonomic_Keywords"},
	{WTaxonomic_System,
		"Taxonomic_System"},
	{WClassification_System_Authority,
		"Classification_System/Authority"},
	{WClassification_System_Citation,
		"Classification_System_Citation"},
	{WClassification_System_Modifications,
		"Classification_System_Modifications"},
	{WIdentification_Reference,
		"Identification_Reference"},
	{WIdentifier,
		"Identifier"},
	{WTaxonomic_Procedures,
		"Taxonomic_Procedures"},
	{WTaxonomic_Completeness,
		"Taxonomic_Completeness"},
	{WVouchers,
		"Vouchers"},
	{WSpecimen,
		"Specimen"},
	{WRepository,
		"Repository"},
	{WGeneral_Taxonomic_Coverage,
		"General_Taxonomic_Coverage"},
	{WTaxonomic_Classification,
		"Taxonomic_Classification"},
	{WTaxon_Rank_Name,
		"Taxon_Rank_Name"},
	{WTaxon_Rank_Value,
		"Taxon_Rank_Value"},
	{WApplicable_Common_Name,
		"Applicable_Common_Name"},
	{WAnalytical_Tool,
		"Analytical_Tool"},
	{WAnalytical_Tool_Description,
		"Analytical_Tool_Description"},
	{WTool_Access_Information,
		"Tool_Access_Information"},
	{WTool_Contact,
		"Tool_Contact"},
	{WTool_Citation,
		"Tool_Citation"},
	{WTool_Access_Instructions,
		"Tool_Access_Instructions"},
	{WTool_Computer_and_Operating_System,
		"Tool_Computer_and_Operating_System"},
	{WMethodology,
		"Methodology"},
	{WMethodology_Type,
		"Methodology_Type"},
	{WMethodology_Identifier,
		"Methodology_Identifier"},
	{WMethodology_Keyword_Thesaurus,
		"Methodology_Keyword_Thesaurus"},
	{WMethodology_Keyword,
		"Methodology_Keyword"},
	{WMethodology_Description,
		"Methodology_Description"},
	{WMethodology_Citation,
		"Methodology_Citation"},
	{WASCII_File_Structure,
		"ASCII_File_Structure"},
	{WRecord_Delimiter,
		"Record_Delimiter"},
	{WNumber_Header_Lines,
		"Number_Header_Lines"},
	{WDescription_of_Header_Content,
		"Description_of_Header_Content"},
	{WOrientation,
		"Orientation"},
	{WCase_Sensitive,
		"Case_Sensitive"},
	{WAuthentication,
		"Authentication"},
	{WQuote_Character,
		"Quote_Character"},
	{WData_Field,
		"Data_Field"},
	{WData_Field_Name,
		"Data_Field_Name"},
	{WMissing_Value_Code,
		"Missing_Value_Code"},
	{WData_Field_Width_Delimiter,
		"Data_Field_Width_Delimiter"},
	{WData_Field_Width,
		"Data_Field_Width"},
	{WBeginning_Geologic_Age,
		"Beginning_Geologic_Age"},
	{WEnding_Geologic_Age,
		"Ending_Geologic_Age"},
	{WGeologic_Age,
		"Geologic_Age"},
	{WGeologic_Time_Scale,
		"Geologic_Time_Scale"},
	{WGeologic_Age_Estimate,
		"Geologic_Age_Estimate"},
	{WGeologic_Age_Uncertainty,
		"Geologic_Age_Uncertainty"},
	{WGeologic_Age_Explanation,
		"Geologic_Age_Explanation"},
	{WGeologic_Citation,
		"Geologic_Citation"},

/* Shoreline profile */

	{WBarometric_Pressure,
	 "Barometric_Pressure"},
	{WEnvironmental_Event,
	 "Environmental_Event"},
	{WMarine_Weather_Condition,
	 "Marine_Weather_Condition"},
	{WProcess_Step_Citation,
	 "Process_Step_Citation"},
	{WRange_of_Tide,
	 "Range_of_Tide"},
	{WSupplemental_Tidal_Information,
	 "Supplemental_Tidal_Information"},
	{WTidal_Datum,
	 "Tidal_Datum"},
	{WTidal_Information,
	 "Tidal_Information"},
	{WTide_Table_Reference,
	 "Tide_Table_Reference"},
	{WTime_of_High_Tide,
	 "Time_of_High_Tide"},
	{WTime_of_Low_Tide,
	 "Time_of_Low_Tide"},
	{WTime_of_Tide,
	 "Time_of_Tide"},
	{WType_of_Tide,
	 "Type_of_Tide"},
	{WWave_Height,
	 "Wave_Height"},
	{WWind_Direction,
	 "Wind_Direction"},
	{WWind_Speed,
	 "Wind_Speed"},

/* Remote-sensing profile */

	{WAcquisition_Information,
	 "Acquisition_Information"},
	{WAerial_Film_Speed,
	 "Aerial_Film_Speed"},
	{WAerotriangulation_Reference,
	 "Aerotriangulation_Reference"},
	{WAffine_Distortion_X_Prime_Coefficient,
	 "Affine_Distortion_X_Prime_Coefficient"},
	{WAffine_Distortion_Y_Prime_Coefficient,
	 "Affine_Distortion_Y_Prime_Coefficient"},
	{WAggregation_Criteria,
	 "Aggregation_Criteria"},
	{WAggregation_Information,
	 "Aggregation_Information"},
	{WAggregation_Member_ID,
	 "Aggregation_Member_ID"},
	{WAlgorithm_Change_Description,
	 "Algorithm_Change_Description"},
	{WAlgorithm_Change_History,
	 "Algorithm_Change_History"},
	{WAlgorithm_Change_Reference,
	 "Algorithm_Change_Reference"},
	{WAlgorithm_Description,
	 "Algorithm_Description"},
	{WAlgorithm_Identifiers,
	 "Algorithm_Identifiers"},
	{WAlgorithm_Information,
	 "Algorithm_Information"},
	{WAlgorithm_Peer_Review_Description,
	 "Algorithm_Peer_Review_Description"},
	{WAlgorithm_Peer_Review_Information,
	 "Algorithm_Peer_Review_Information"},
	{WAlgorithm_Peer_Review_Reference,
	 "Algorithm_Peer_Review_Reference"},
	{WAlgorithm_Reference,
	 "Algorithm_Reference"},
	{WAlgorithm_Text_Description,
	 "Algorithm_Text_Description"},
	{WAltitude_Distance_Layer,
	 "Altitude_Distance_Layer"},
	{WAncillary_Dataset,
	 "Ancillary_Dataset"},
	{WAncillary_Dataset_Description,
	 "Ancillary_Dataset_Description"},
	{WAncillary_Dataset_Identifier,
	 "Ancillary_Dataset_Identifier"},
	{WAncillary_Description,
	 "Ancillary_Description"},
	{WAncillary_Reference,
	 "Ancillary_Reference"},
	{WAngle_Dependent_Distortion,
	 "Angle_Dependent_Distortion"},
	{WAngle_Distortion_Value,
	 "Angle_Distortion_Value"},
	{WArea_Weighted_Average_Resolution,
	 "Area_Weighted_Average_Resolution"},
	{WArgument_of_Perigee,
	 "Argument_of_Perigee"},
	{WAscending_Descending_Indicator,
	 "Ascending/Descending_Indicator"},
	{WAttitude_Angular_Units,
	 "Attitude_Angular_Units"},
	{WAvailability_of_Element_Locations,
	 "Availability_of_Element_Locations"},
	{WAxes,
	 "Axes"},
	{WAxis_Rotation_Convention,
	 "Axis_Rotation_Convention"},
	{WBand_Boundary_Definition,
	 "Band_Boundary_Definition"},
	{WBand_Description,
	 "Band_Description"},
	{WBand_ID,
	 "Band_ID"},
	{WBand_Identification,
	 "Band_Identification"},
	{WBand_Measurement_Mode_ID,
	 "Band_Measurement_Mode_ID"},
	{WBand_Quality,
	 "Band_Quality"},
	{WCalibrated_Detector_Positions,
	 "Calibrated_Detector_Positions"},
	{WCalibrated_Focal_Length,
	 "Calibrated_Focal_Length"},
	{WCamera_Identifier,
	 "Camera_Identifier"},
	{WCamera_Type,
	 "Camera_Type"},
	{WCell_Value_Type,
	 "Cell_Value_Type"},
	{WClock_Time_Drift,
	 "Clock_Time/Drift"},
	{WCollection_Type,
	 "Collection_Type"},
	{WColumn_Delta_X,
	 "Column_Delta_X"},
	{WColumn_Delta_Y,
	 "Column_Delta_Y"},
	{WCommand_Line_Processing_Parameter,
	 "Command_Line_Processing_Parameter"},
	{WComponent_Information,
	 "Component_Information"},
	{WContainer_Packet_ID,
	 "Container_Packet_ID"},
	{WControl_Point_Authority,
	 "Control_Point_Authority"},
	{WControl_Point_Column,
	 "Control_Point_Column"},
	{WControl_Point_Earth_Location,
	 "Control_Point_Earth_Location"},
	{WControl_Point_Horizontal_X_Accuracy,
	 "Control_Point_Horizontal_X_Accuracy"},
	{WControl_Point_Horizontal_Y_Accuracy,
	 "Control_Point_Horizontal_Y_Accuracy"},
	{WControl_Point_ID,
	 "Control_Point_ID"},
	{WControl_Point_Identification,
	 "Control_Point_Identification"},
	{WControl_Point_Origin,
	 "Control_Point_Origin"},
	{WControl_Point_Raster_Position,
	 "Control_Point_Raster_Position"},
	{WControl_Point_Row,
	 "Control_Point_Row"},
	{WControl_Point_Type,
	 "Control_Point_Type"},
	{WControl_Point_Use_Flag,
	 "Control_Point_Use_Flag"},
	{WControl_Point_Vertical_Accuracy,
	 "Control_Point_Vertical_Accuracy"},
	{WControl_Point_x_Value,
	 "Control_Point_x_Value"},
	{WControl_Point_y_Value,
	 "Control_Point_y_Value"},
	{WControl_Point_z_Value,
	 "Control_Point_z_Value"},
	{WCoordinate_Point,
	 "Coordinate_Point"},
	{WCoordinate_System,
	 "Coordinate_System"},
	{WCoordinate_x_Value,
	 "Coordinate_x_Value"},
	{WCoordinate_XY_Units,
	 "Coordinate_XY_Units"},
	{WCoordinate_y_Value,
	 "Coordinate_y_Value"},
	{WCoordinate_Z_Units,
	 "Coordinate_Z_Units"},
	{WCoordinate_z_Value,
	 "Coordinate_z_Value"},
	{WCross_Track_Axis,
	 "Cross_Track_Axis"},
	{WCross_Track_Description,
	 "Cross_Track_Description"},
	{WCross_Track_Direction,
	 "Cross_Track_Direction"},
	{WCross_Track_Extent_Angle,
	 "Cross_Track_Extent_Angle"},
	{WCross_Track_Fixed_Angle,
	 "Cross_Track_Fixed_Angle"},
	{WCross_Track_Motion,
	 "Cross_Track_Motion"},
	{WCross_Track_Start_Angle,
	 "Cross_Track_Start_Angle"},
	{WCross_Track_Step_Angle,
	 "Cross_Track_Step_Angle"},
	{WCross_Track_Sweep,
	 "Cross_Track_Sweep"},
	{WCross_Track_Zero,
	 "Cross_Track_Zero"},
	{WData_Dictionary_Reference,
	 "Data_Dictionary_Reference"},
	{WData_Scaling_Information,
	 "Data_Scaling_Information"},
	{WDataset_Identifier,
	 "Dataset_Identifier"},
	{WDate_of_Last_Calibration,
	 "Date_of_Last_Calibration"},
	{WDeveloping_Institution,
	 "Developing_Institution"},
	{WDimension_Count,
	 "Dimension_Count"},
	{WDimension_Description,
	 "Dimension_Description"},
	{WDimension_Properties,
	 "Dimension_Properties"},
	{WDistance_Dependent_Distortion,
	 "Distance_Dependent_Distortion"},
	{WDistance_Distortion_Value,
	 "Distance_Distortion_Value"},
	{WDistortion,
	 "Distortion"},
	{WDistortion_Type_Affine,
	 "Distortion_Type_Affine"},
	{WDistortion_Type_Radial_Asymmetrical,
	 "Distortion_Type_Radial_Asymmetrical"},
	{WDistortion_Type_Radial_Symmetrical,
	 "Distortion_Type_Radial_Symmetrical"},
	{WDocumentation,
	 "Documentation"},
	{WEccentricity,
	 "Eccentricity"},
	{WEffective_Aerial_Film_Speed,
	 "Effective_Aerial_Film_Speed"},
	{WElevation_Axis,
	 "Elevation_Axis"},
	{WElevation_Description,
	 "Elevation_Description"},
	{WElevation_Direction,
	 "Elevation_Direction"},
	{WElevation_Extent_Angle,
	 "Elevation_Extent_Angle"},
	{WElevation_Fixed_Angle,
	 "Elevation_Fixed_Angle"},
	{WElevation_Start_Angle,
	 "Elevation_Start_Angle"},
	{WElevation_Step_Angle,
	 "Elevation_Step_Angle"},
	{WElevation_Sweep,
	 "Elevation_Sweep"},
	{WElevation_Zero,
	 "Elevation_Zero"},
	{WEphemeris,
	 "Ephemeris"},
	{WExposure_Time,
	 "Exposure_Time"},
	{WExterior_Orientation_Accuracy,
	 "Exterior_Orientation_Accuracy"},
	{WFiducial,
	 "Fiducial"},
	{WFiducial_Center,
	 "Fiducial_Center"},
	{WFilm_Type,
	 "Film_Type"},
	{WFilter_on_Camera,
	 "Filter_on_Camera"},
	{WFilter_on_Camera_Indicator,
	 "Filter_on_Camera_Indicator"},
	{WFilter_Type,
	 "Filter_Type"},
	{WFlight_Protocol,
	 "Flight_Protocol"},
	{WFlying_Height,
	 "Flying_Height"},
	{WForward_Motion_Compensation,
	 "Forward_Motion_Compensation"},
	{WFrame_Area,
	 "Frame_Area"},
	{WFrame_Area_Units,
	 "Frame_Area_Units"},
	{WFrame_Area_Value,
	 "Frame_Area_Value"},
	{WFrame_Camera,
	 "Frame_Camera"},
	{WFrame_Geometric_Properties,
	 "Frame_Geometric_Properties"},
	{WFrame_Hardware,
	 "Frame_Hardware"},
	{WFrame_Operation,
	 "Frame_Operation"},
	{WFrame_Optics,
	 "Frame_Optics"},
	{WFrame_Radiometric_Calibration,
	 "Frame_Radiometric_Calibration"},
	{WFrame_Radiometric_Properties,
	 "Frame_Radiometric_Properties"},
	{WFrame_Spectral_Information,
	 "Frame_Spectral_Information"},
	{WFrame_Spectral_Properties,
	 "Frame_Spectral_Properties"},
	{WGeorectified_Raster,
	 "Georectified_Raster"},
	{WGeoreferenceable_Raster,
	 "Georeferenceable_Raster"},
	{WGeoreferencing_Description,
	 "Georeferencing_Description"},
	{WGeoreferencing_Information,
	 "Georeferencing_Information"},
	{WGPS_Information_System_Availability,
	 "GPS_Information_System_Availability"},
	{WGrid_First_Element,
	 "Grid_First_Element"},
	{WGrid_First_Element_Map_X_Coordinate,
	 "Grid_First_Element_Map_X_Coordinate"},
	{WGrid_First_Element_Map_Y_Coordinate,
	 "Grid_First_Element_Map_Y_Coordinate"},
	{WGrid_Orientation,
	 "Grid_Orientation"},
	{WGround_Control_Point_Description,
	 "Ground_Control_Point_Description"},
	{WGround_Control_Point_Information,
	 "Ground_Control_Point_Information"},
	{WGround_Control_Point_Organization,
	 "Ground_Control_Point_Organization"},
	{WGround_Control_Point_Position,
	 "Ground_Control_Point_Position"},
	{WGround_Shape,
	 "Ground_Shape"},
	{WID_Overlapping_Dataset,
	 "ID_Overlapping_Dataset"},
	{WIFOV_Units,
	 "IFOV_Units"},
	{WIFOV_x_Definition,
	 "IFOV_x_Definition"},
	{WIFOV_x_Value,
	 "IFOV_x_Value"},
	{WIFOV_y_Definition,
	 "IFOV_y_Definition"},
	{WIFOV_y_Value,
	 "IFOV_y_Value"},
	{WImage_Quality,
	 "Image_Quality"},
	{WImage_Size,
	 "Image_Size"},
	{WImage_Size_x_Value,
	 "Image_Size_x_Value"},
	{WImage_Size_y_Value,
	 "Image_Size_y_Value"},
	{WInclination,
	 "Inclination"},
	{WIndividual_Band_Identification,
	 "Individual_Band_Identification"},
	{WInput_Dataset_Identifier,
	 "Input_Dataset_Identifier"},
	{WInput_Description,
	 "Input_Description"},
	{WInput_Level,
	 "Input_Level"},
	{WInput_Reference,
	 "Input_Reference"},
	{WINS_Reading_Availability,
	 "INS_Reading_Availability"},
	{WInstantaneous_Field_of_View,
	 "Instantaneous_Field_of_View"},
	{WInstitution_of_Last_Calibration,
	 "Institution_of_Last_Calibration"},
	{WInstrument_Description,
	 "Instrument_Description"},
	{WInstrument_Full_Name,
	 "Instrument_Full_Name"},
	{WInstrument_Information,
	 "Instrument_Information"},
	{WInstrument_Properties_Description,
	 "Instrument_Properties_Description"},
	{WInstrument_Reference,
	 "Instrument_Reference"},
	{WInstrument_Short_Name,
	 "Instrument_Short_Name"},
	{WInstrument_Specific_Georeferencing,
	 "Instrument_Specific_Georeferencing"},
	{WInstrument_Type,
	 "Instrument_Type"},
	{WKeplerian_Orbit,
	 "Keplerian_Orbit"},
	{WLast_Calibration,
	 "Last_Calibration"},
	{WLayer_Name,
	 "Layer_Name"},
	{WLens,
	 "Lens"},
	{WLens_Identifier,
	 "Lens_Identifier"},
	{WLens_Type,
	 "Lens_Type"},
	{WLight_Drop,
	 "Light_Drop"},
	{WLocation_Information,
	 "Location_Information"},
	{WMagazine_Identifier,
	 "Magazine_Identifier"},
	{WMaximum_Wavelength,
	 "Maximum_Wavelength"},
	{WMethod_of_Last_Calibration,
	 "Method_of_Last_Calibration"},
	{WMinimum_Wavelength,
	 "Minimum_Wavelength"},
	{WMission_Completion,
	 "Mission_Completion"},
	{WMission_Description,
	 "Mission_Description"},
	{WMission_History,
	 "Mission_History"},
	{WMission_Information,
	 "Mission_Information"},
	{WMission_Name,
	 "Mission_Name"},
	{WMission_Significant_Event,
	 "Mission_Significant_Event"},
	{WMission_Start_Date,
	 "Mission_Start_Date"},
	{WMultiple_Image_Alignment,
	 "Multiple_Image_Alignment"},
	{WNadir,
	 "Nadir"},
	{WNadir_Latitude,
	 "Nadir_Latitude"},
	{WNadir_Longitude,
	 "Nadir_Longitude"},
	{WName_of_Dimension,
	 "Name_of_Dimension"},
	{WNominal_Geostationary_Position,
	 "Nominal_Geostationary_Position"},
	{WNominal_Spatial_Resolution,
	 "Nominal_Spatial_Resolution"},
	{WNon_Polynomial_Scaling,
	 "Non_Polynomial_Scaling"},
	{WNumber_of_Angle_Distortion_Values,
	 "Number_of_Angle_Distortion_Values"},
	{WNumber_of_Bands,
	 "Number_of_Bands"},
	{WNumber_of_Cross_Track_Samples,
	 "Number_of_Cross_Track_Samples"},
	{WNumber_of_Data_Dimensions,
	 "Number_of_Data_Dimensions"},
	{WNumber_of_Denominator_Terms,
	 "Number_of_Denominator_Terms"},
	{WNumber_of_Distance_Distortion_Values,
	 "Number_of_Distance_Distortion_Values"},
	{WNumber_of_Elevation_Samples,
	 "Number_of_Elevation_Samples"},
	{WNumber_of_Numerator_Terms,
	 "Number_of_Numerator_Terms"},
	{WNumber_of_Points,
	 "Number_of_Points"},
	{WNumber_of_Profile_Samples,
	 "Number_of_Profile_Samples"},
	{WNumber_of_Resolution_Values,
	 "Number_of_Resolution_Values"},
	{WNumber_of_Thematic_Layers,
	 "Number_of_Thematic_Layers"},
	{WNumber_of_Wavelength_Bands,
	 "Number_of_Wavelength_Bands"},
	{WOperational_Mode,
	 "Operational_Mode"},
	{WOrbit_Angle_Units,
	 "Orbit_Angle_Units"},
	{WOrbit_Period,
	 "Orbit_Period"},
	{WOrbit_Period_Units,
	 "Orbit_Period_Units"},
	{WOrbit_Period_Value,
	 "Orbit_Period_Value"},
	{WOther_Collector_Description,
	 "Other_Collector_Description"},
	{WOther_Georeferencing_Description,
	 "Other_Georeferencing_Description"},
	{WPath,
	 "Path"},
	{WPeak_Wavelength,
	 "Peak_Wavelength"},
	{WPercentage_of_Overlap,
	 "Percentage_of_Overlap"},
	{WPerigee_Passage_Time,
	 "Perigee_Passage_Time"},
	{WPhotographic_Resolving_Power,
	 "Photographic_Resolving_Power"},
	{WPitch,
	 "Pitch"},
	{WPitch_Accuracy,
	 "Pitch_Accuracy"},
	{WPixel_Cross_Track_Size,
	 "Pixel_Cross_Track_Size"},
	{WPixel_Description,
	 "Pixel_Description"},
	{WPixel_Elevation_Size,
	 "Pixel_Elevation_Size"},
	{WPixel_Height_Above_Ellipsoid,
	 "Pixel_Height_Above_Ellipsoid"},
	{WPixel_Point_Spread_Function,
	 "Pixel_Point_Spread_Function"},
	{WPixel_Profile_Size,
	 "Pixel_Profile_Size"},
	{WPixel_Resolution,
	 "Pixel_Resolution"},
	{WPlatform_and_Instrument_Identification,
	 "Platform_and_Instrument_Identification"},
	{WPlatform_and_Mission_Information,
	 "Platform_and_Mission_Information"},
	{WPlatform_Description,
	 "Platform_Description"},
	{WPlatform_Full_Name,
	 "Platform_Full_Name"},
	{WPlatform_Information,
	 "Platform_Information"},
	{WPlatform_Nominal_Altitude,
	 "Platform_Nominal_Altitude"},
	{WPlatform_Nominal_Altitude_Units,
	 "Platform_Nominal_Altitude_Units"},
	{WPlatform_Nominal_Altitude_Value,
	 "Platform_Nominal_Altitude_Value"},
	{WPlatform_Nominal_Longitude,
	 "Platform_Nominal_Longitude"},
	{WPlatform_Orbit,
	 "Platform_Orbit"},
	{WPlatform_Serial_Identifier,
	 "Platform_Serial_Identifier"},
	{WPlatform_Short_Name,
	 "Platform_Short_Name"},
	{WPlatform_Sponsor,
	 "Platform_Sponsor"},
	{WPlatform_Start_Date,
	 "Platform_Start_Date"},
	{WPoint_Position_In_Pixel,
	 "Point_Position_In_Pixel"},
	{WPolarization_Characteristics,
	 "Polarization_Characteristics"},
	{WPolynomial_Coefficient,
	 "Polynomial_Coefficient"},
	{WPolynomial_Denominator,
	 "Polynomial_Denominator"},
	{WPolynomial_Function,
	 "Polynomial_Function"},
	{WPolynomial_Numerator,
	 "Polynomial_Numerator"},
	{WPositional_Information,
	 "Positional_Information"},
	{WPrincipal_Point_of_Autocollimation,
	 "Principal_Point_of_Autocollimation"},
	{WPrincipal_Point_of_Symmetry,
	 "Principal_Point_of_Symmetry"},
	{WProcessing_Change_History,
	 "Processing_Change_History"},
	{WProcessing_Documentation,
	 "Processing_Documentation"},
	{WProcessing_Environment,
	 "Processing_Environment"},
	{WProcessing_Identifiers,
	 "Processing_Identifiers"},
	{WProcessing_Information,
	 "Processing_Information"},
	{WProcessing_Input_Dataset,
	 "Processing_Input_Dataset"},
	{WProcessing_Level,
	 "Processing_Level"},
	{WProcessing_Level_Authority,
	 "Processing_Level_Authority"},
	{WProcessing_Level_Identifier,
	 "Processing_Level_Identifier"},
	{WProcessing_Procedure,
	 "Processing_Procedure"},
	{WProcessing_Procedure_Description,
	 "Processing_Procedure_Description"},
	{WProcessing_Run_History,
	 "Processing_Run_History"},
	{WProcessing_Software,
	 "Processing_Software"},
	{WProcessing_Software_Description,
	 "Processing_Software_Description"},
	{WProcessing_Software_Reference,
	 "Processing_Software_Reference"},
	{WProfile_Description,
	 "Profile_Description"},
	{WProfile_Extent,
	 "Profile_Extent"},
	{WProfile_Fixed,
	 "Profile_Fixed"},
	{WProfile_Properties,
	 "Profile_Properties"},
	{WProfile_Sounding,
	 "Profile_Sounding"},
	{WProfile_Start,
	 "Profile_Start"},
	{WProfile_Step,
	 "Profile_Step"},
	{WProfiling_Direction,
	 "Profiling_Direction"},
	{WProjection_Center_X_Position,
	 "Projection_Center_X_Position"},
	{WProjection_Center_Y_Position,
	 "Projection_Center_Y_Position"},
	{WProjection_Center_Z_Position,
	 "Projection_Center_Z_Position"},
	{WQuality_of_Autocollimation_Principal_Point_x_Value,
	 "Quality_of_Autocollimation_Principal_Point_x_Value"},
	{WQuality_of_Autocollimation_Principal_Point_y_Value,
	 "Quality_of_Autocollimation_Principal_Point_y_Value"},
	{WQuality_of_Focal_Length,
	 "Quality_of_Focal_Length"},
	{WQuality_of_Principal_Point_of_Autocollimation,
	 "Quality_of_Principal_Point_of_Autocollimation"},
	{WQuality_of_Principal_Point_of_Symmetry,
	 "Quality_of_Principal_Point_of_Symmetry"},
	{WQuality_of_Symmetry_Principal_Point_x_Value,
	 "Quality_of_Symmetry_Principal_Point_x_Value"},
	{WQuality_of_Symmetry_Principal_Point_y_Value,
	 "Quality_of_Symmetry_Principal_Point_y_Value"},
	{WRadial_Asymmetrical_Coefficient_B1,
	 "Radial_Asymmetrical_Coefficient_B1"},
	{WRadial_Asymmetrical_Coefficient_B2,
	 "Radial_Asymmetrical_Coefficient_B2"},
	{WRadial_Symmetrical_Angle_Interval,
	 "Radial_Symmetrical_Angle_Interval"},
	{WRadial_Symmetrical_Distance_Interval,
	 "Radial_Symmetrical_Distance_Interval"},
	{WRadial_Symmetrical_Distortion_Polynomial,
	 "Radial_Symmetrical_Distortion_Polynomial"},
	{WReceiver_Polarization,
	 "Receiver_Polarization"},
	{WReferenced_Coordinate_System,
	 "Referenced_Coordinate_System"},
	{WReferencing_Polynomial,
	 "Referencing_Polynomial"},
	{WRelative_Aperture,
	 "Relative_Aperture"},
	{WRelative_Azimuth,
	 "Relative_Azimuth"},
	{WReseau,
	 "Reseau"},
	{WResolution_Value_Set,
	 "Resolution_Value_Set"},
	{WResolving_Angle,
	 "Resolving_Angle"},
	{WResolving_Value_Radial,
	 "Resolving_Value_Radial"},
	{WResolving_Value_Tangential,
	 "Resolving_Value_Tangential"},
	{WRight_Ascension_of_Ascending_Node,
	 "Right_Ascension_of_Ascending_Node"},
	{WRoll,
	 "Roll"},
	{WRoll_Accuracy,
	 "Roll_Accuracy"},
	{WRotation_Description,
	 "Rotation_Description"},
	{WRotation_Sequence,
	 "Rotation_Sequence"},
	{WRow,
	 "Row"},
	{WRow_Delta_X,
	 "Row_Delta_X"},
	{WRow_Delta_Y,
	 "Row_Delta_Y"},
	{WSample_Angle_Units,
	 "Sample_Angle_Units"},
	{WSample_Description_Units,
	 "Sample_Description_Units"},
	{WSample_Profile_Units,
	 "Sample_Profile_Units"},
	{WSample_Properties,
	 "Sample_Properties"},
	{WSatellite_Local_Azimuth_Angle,
	 "Satellite/Local_Azimuth_Angle"},
	{WSatellite_Local_Zenith_Angle,
	 "Satellite/Local_Zenith_Angle"},
	{WScan,
	 "Scan"},
	{WScan_Angle_Units,
	 "Scan_Angle_Units"},
	{WScan_Cross_Track_Properties,
	 "Scan_Cross_Track_Properties"},
	{WScan_Distance_Units,
	 "Scan_Distance_Units"},
	{WScan_Duration,
	 "Scan_Duration"},
	{WScan_Elevation_Properties,
	 "Scan_Elevation_Properties"},
	{WScan_Geometric_Properties,
	 "Scan_Geometric_Properties"},
	{WScan_Radiometric_Properties,
	 "Scan_Radiometric_Properties"},
	{WScan_Repeat_Time,
	 "Scan_Repeat_Time"},
	{WScan_Spectral_Properties,
	 "Scan_Spectral_Properties"},
	{WScan_Start_Time,
	 "Scan_Start_Time"},
	{WScan_Step_Time,
	 "Scan_Step_Time"},
	{WScan_Time_Units,
	 "Scan_Time_Units"},
	{WScan_Timing,
	 "Scan_Timing"},
	{WScience_Paper,
	 "Science_Paper"},
	{WSemimajor_Axis,
	 "Semimajor_Axis"},
	{WSender_Polarization,
	 "Sender_Polarization"},
	{WSensor_Element_Location,
	 "Sensor_Element_Location"},
	{WSensor_Grid,
	 "Sensor_Grid"},
	{WSensor_Orientation,
	 "Sensor_Orientation"},
	{WSensor_System,
	 "Sensor_System"},
	{WSolar_Azimuth_Angle,
	 "Solar_Azimuth_Angle"},
	{WSolar_Zenith_Angle,
	 "Solar_Zenith_Angle"},
	{WSource_of_Element_Locations,
	 "Source_of_Element_Locations"},
	{WSpatial_Resolution_Units,
	 "Spatial_Resolution_Units"},
	{WSpatial_Resolution_Value,
	 "Spatial_Resolution_Value"},
	{WSpectral_Information,
	 "Spectral_Information"},
	{WSpectral_Limit,
	 "Spectral_Limit"},
	{WStabilized_Mount,
	 "Stabilized_Mount"},
	{WStorage_Order,
	 "Storage_Order"},
	{WSwath_Track_Information,
	 "Swath_Track_Information"},
	{WThematic_Layer_Identification,
	 "Thematic_Layer_Identification"},
	{WTranslation_Description,
	 "Translation_Description"},
	{WType_of_Overlap,
	 "Type_of_Overlap"},
	{WUnreferenced_Coordinate_System,
	 "Unreferenced_Coordinate_System"},
	{WUsers_Guide,
	 "User's_Guide"},
	{WWavelength_Band_Properties,
	 "Wavelength_Band_Properties"},
	{WWavelength_Region,
	 "Wavelength_Region"},
	{WWavelength_Units,
	 "Wavelength_Units"},
	{WWorldwide_Reference_System,
	 "Worldwide_Reference_System"},
	{WX_Axis_Definition,
	 "X_Axis_Definition"},
	{WX_Position_Accuracy,
	 "X_Position_Accuracy"},
	{WY_Axis_Definition,
	 "Y_Axis_Definition"},
	{WY_Position_Accuracy,
	 "Y_Position_Accuracy"},
	{WYaw,
	 "Yaw"},
	{WYaw_Accuracy,
	 "Yaw_Accuracy"},
	{WZ_Axis_Definition,
	 "Z_Axis_Definition"},
	{WZ_Position_Accuracy,
	 "Z_Position_Accuracy"},

#ifdef USE_ESRI_EXTENSIONS

	/* ESRI extensions */

	{WAltitude_Units,
	 "Altitude_Units"},
	{WAncillary_Role,
	 "Ancillary_Role"},
	{WAncillary_Role_Attribute,
	 "Ancillary_Role_Attribute"},
	{WAttribute_Alias,
	 "Attribute_Alias"},
	{WAttribute_Defined_Domain,
	 "Attribute_Defined_Domain"},
	{WAttribute_Indexed,
	 "Attribute_Indexed"},
	{WAttribute_Number_of_Decimals,
	 "Attribute_Number_of_Decimals"},
	{WAttribute_Output_Width,
	 "Attribute_Output_Width"},
	{WAttribute_Precision,
	 "Attribute_Precision"},
	{WAttribute_Scale,
	 "Attribute_Scale"},
	{WAttribute_Type,
	 "Attribute_Type"},
	{WAttribute_Width,
	 "Attribute_Width"},
	{WAttributed_Relationship,
	 "Attributed_Relationship"},
	{WAvailable_Junction_Feature_Class,
	 "Available_Junction_Feature_Class"},
	{WAvailable_Junction_Subtype,
	 "Available_Junction_Subtype"},
	{WAvailable_Junctions,
	 "Available_Junctions"},
	{WBackground_Nodata_Value,
	 "Background_Nodata_Value"},
	{WBehrmann,
	 "Behrmann"},
	{WBinary_Enclosures,
	 "Binary_Enclosures"},
	{WBits_Per_Pixel,
	 "Bits_Per_Pixel"},
	{WBonne,
	 "Bonne"},
	{WBottom_Bounding_Coordinate,
	 "Bottom_Bounding_Coordinate"},
	{WBrowse_Graphic_Embedded,
	 "Browse_Graphic_Embedded"},
	{WCassini,
	 "Cassini"},
	{WCell_Size_X_Direction,
	 "Cell_Size_X_Direction"},
	{WCell_Size_X_Units,
	 "Cell_Size_X_Units"},
	{WCell_Size_Y_Direction,
	 "Cell_Size_Y_Direction"},
	{WCell_Size_Y_Units,
	 "Cell_Size_Y_Units"},
	{WComposite_Relationship,
	 "Composite_Relationship"},
	{WCompression_Type,
	 "Compression_Type"},
	{WConnectivity_Rule,
	 "Connectivity_Rule"},
	{WCoordinate_System_Name,
	 "Coordinate_System_Name"},
	{WCreation_Date,
	 "Creation_Date"},
	{WCreation_Time,
	 "Creation_Time"},
	{WData,
	 "Data"},
	{WData_Frame_Count,
	 "Data_Frame_Count"},
	{WData_Frame_Name,
	 "Data_Frame_Name"},
	{WData_Frames,
	 "Data_Frames"},
	{WDatabase_Name,
	 "Database_Name"},
	{WDataset_Size,
	 "Dataset_Size"},
	{WDefault_Junction_Feature_Class,
	 "Default_Junction_Feature_Class"},
	{WDefault_Junction_Subtype,
	 "Default_Junction_Subtype"},
	{WDescription_of_Association,
	 "Description_of_Association"},
	{WDescription_of_Enclosure,
	 "Description_of_Enclosure"},
	{WDescription_of_Relationship,
	 "Description_of_Relationship"},
	{WDestination_Foreign_Key,
	 "Destination_Foreign_Key"},
	{WDestination_Name,
	 "Destination_Name"},
	{WDestination_Primary_Key,
	 "Destination_Primary_Key"},
	{WDomain_Attribute_Type,
	 "Domain_Attribute_Type"},
	{WDomain_Description,
	 "Domain_Description"},
	{WDomain_Name,
	 "Domain_Name"},
	{WDomain_Owner,
	 "Domain_Owner"},
	{WDomain_Type,
	 "Domain_Type"},
	{WEckert_I,
	 "Eckert_I"},
	{WEckert_II,
	 "Eckert_II"},
	{WEckert_III,
	 "Eckert_III"},
	{WEckert_IV,
	 "Eckert_IV"},
	{WEckert_V,
	 "Eckert_V"},
	{WEckert_VI,
	 "Eckert_VI"},
	{WEdge_Feature_Class,
	 "Edge_Feature_Class"},
	{WEdge_Maximum_Cardinality,
	 "Edge_Maximum_Cardinality"},
	{WEdge_Minimum_Cardinality,
	 "Edge_Minimum_Cardinality"},
	{WEdge_Subtype,
	 "Edge_Subtype"},
	{WEnabled_Attribute,
	 "Enabled_Attribute"},
	{WEnclosure,
	 "Enclosure"},
	{WEntity_Type_Count,
	 "Entity_Type_Count"},
	{WEntity_Type_Type,
	 "Entity_Type_Type"},
	{WESRI_Feature_Count,
	 "ESRI_Feature_Count"},
	{WESRI_Feature_Geometry,
	 "ESRI_Feature_Geometry"},
	{WESRI_Feature_Type,
	 "ESRI_Feature_Type"},
	{WESRI_Information,
	 "ESRI_Information"},
	{WESRI_Terms_Description,
	 "ESRI_Terms_Description"},
	{WESRI_Topology,
	 "ESRI_Topology"},
	{WFeature_Description,
	 "Feature_Description"},
	{WFile_or_Table_Name,
	 "File_or_Table_Name"},
	{WFrom_Edge_Feature_Class,
	 "From_Edge_Feature_Class"},
	{WFrom_Edge_Subtype,
	 "From_Edge_Subtype"},
	{WGall_Stereographic,
	 "Gall_Stereographic"},
	{WGeocoding_Information,
	 "Geocoding_Information"},
	{WGeographic_Coordinate_System_Name,
	 "Geographic_Coordinate_System_Name"},
	{WGeometric_Network_Information,
	 "Geometric_Network_Information"},
	{WImage_Colormap,
	 "Image_Colormap"},
	{WImage_Format,
	 "Image_Format"},
	{WImage_Type,
	 "Image_Type"},
	{WInstance_Name,
	 "Instance_Name"},
	{WJunction_Feature_Class,
	 "Junction_Feature_Class"},
	{WJunction_Maximum_Cardinality,
	 "Junction_Maximum_Cardinality"},
	{WJunction_Minimum_Cardinality,
	 "Junction_Minimum_Cardinality"},
	{WJunction_Subtype,
	 "Junction_Subtype"},
	{WLanguage_of_Dataset,
	 "Language_of_Dataset"},
	{WLanguage_of_Metadata,
	 "Language_of_Metadata"},
	{WLeft_Bounding_Coordinate,
	 "Left_Bounding_Coordinate"},
	{WLinear_Referencing,
	 "Linear_Referencing"},
	{WLocal_Bounding_Coordinates,
	 "Local_Bounding_Coordinates"},
	{WLoximuthal,
	 "Loximuthal"},
	{WMaximum_Altitude,
	 "Maximum_Altitude"},
	{WMerge_Rule,
	 "Merge_Rule"},
	{WMetadata_ID,
	 "Metadata_ID"},
	{WMinimum_Altitude,
	 "Minimum_Altitude"},
	{WModification_Date,
	 "Modification_Date"},
	{WModification_Time,
	 "Modification_Time"},
	{WMollweide,
	 "Mollweide"},
	{WNative_Dataset_Format,
	 "Native_Dataset_Format"},
	{WNetwork_Element,
	 "Network_Element"},
	{WNetwork_Role,
	 "Network_Role"},
	{WNetwork_Type,
	 "Network_Type"},
	{WNotification_Direction,
	 "Notification_Direction"},
	{WNumber_of_Raster_Bands,
	 "Number_of_Raster_Bands"},
	{WOrigin_Foreign_Key,
	 "Origin_Foreign_Key"},
	{WOrigin_Name,
	 "Origin_Name"},
	{WOrigin_Primary_Key,
	 "Origin_Primary_Key"},
	{WProcess_Software_and_Version,
	 "Process_Software_and_Version"},
	{WProjected_Coordinate_System_Name,
	 "Projected_Coordinate_System_Name"},
	{WPyramid_Layers,
	 "Pyramid_Layers"},
	{WQuartic_Authalic,
	 "Quartic_Authalic"},
	{WRange_Domain_Mean,
	 "Range_Domain_Mean"},
	{WRange_Domain_Standard_Deviation,
	 "Range_Domain_Standard_Deviation"},
	{WRaster_Display_Type,
	 "Raster_Display_Type"},
	{WRaster_Origin,
	 "Raster_Origin"},
	{WRelationship_Backward_Label,
	 "Relationship_Backward_Label"},
	{WRelationship_Cardinality,
	 "Relationship_Cardinality"},
	{WRelationship_Forward_Label,
	 "Relationship_Forward_Label"},
	{WRelationship_Information,
	 "Relationship_Information"},
	{WRight_Bounding_Coordinate,
	 "Right_Bounding_Coordinate"},
	{WRule_Category,
	 "Rule_Category"},
	{WRule_Help,
	 "Rule_Help"},
	{WRule_Type,
	 "Rule_Type"},
	{WSDE_Connection_Information,
	 "SDE_Connection_Information"},
	{WServer_Name,
	 "Server_Name"},
	{WSpatial_Index,
	 "Spatial_Index"},
	{WSplit_Rule,
	 "Split_Rule"},
	{WSubtype_Attribute,
	 "Subtype_Attribute"},
	{WSubtype_Attribute_Name,
	 "Subtype_Attribute_Name"},
	{WSubtype_Code,
	 "Subtype_Code"},
	{WSubtype_Default_Value,
	 "Subtype_Default_Value"},
	{WSubtype_Information,
	 "Subtype_Information"},
	{WSubtype_Name,
	 "Subtype_Name"},
	{WSynchronization_Date,
	 "Synchronization_Date"},
	{WSynchronization_Time,
	 "Synchronization_Time"},
	{WSynchronize_Automatically,
	 "Synchronize_Automatically"},
	{WSynchronize_Once,
	 "Synchronize_Once"},
	{WThumbnail,
	 "Thumbnail"},
	{WTo_Edge_Feature_Class,
	 "To_Edge_Feature_Class"},
	{WTo_Edge_Subtype,
	 "To_Edge_Subtype"},
	{WTop_Bounding_Coordinate,
	 "Top_Bounding_Coordinate"},
	{WUser_Name,
	 "User_Name"},
	{WVersion_Name,
	 "Version_Name"},
	{WWinkel_I,
	 "Winkel_I"},
	{WWinkel_II,
	 "Winkel_II"},
	{WGeographic_Box,
	 "Geographic_Box"},

#endif

	{Wunknown, NULL}
	};

/*----------------------------------------------------------------------*\
 | Portuguese
\*----------------------------------------------------------------------*/

static struct keyword_list pt_std[] = {
	{WAbscissa_Resolution,
	 "Resoluo_de_Abscissa"},
	{WAbstract,
	 "Resumo"},
	{WAccess_Constraints,
	 "Restries_de_Acesso"},
	{WAccess_Instructions,
	 "Instrues_de_Acesso"},
	{WAddress,
	 "Endereo"},
	{WAddress_Type,
	 "Tipo_de_Endereo"},
	{WAlbers_Conical_Equal_Area,
	 "Projeo_Equivalente_de_Albers "},
	{WAltitude_Datum_Name,
	 "Nome_do_Datum_Altimtrico"},
	{WAltitude_Distance_Units,
	 "Unidades_de_Distncia_Altimticas"},
	{WAltitude_Encoding_Method,
	 "Mtodos_de_Codificao_da_Altitude"},
	{WAltitude_Resolution,
	 "Resoluo_Altimtrica"},
	{WAltitude_System_Definition,
	 "Definio_do_Sistema_de_Altitude"},
	{WARC_Coordinate_System,
	 "Sistema_Arc_de_Coordenadas"},
	{WARC_System_Zone_Identifier,
	 "Identificador_de_Zona_do_Sistema_Arc"},
	{WAttribute,
	 "Artibutos"},
	{WAttribute_Accuracy,
	 "Atributo_de_Preciso"},
	{WAttribute_Accuracy_Explanation,
	 "Descrio_da_Preciso_da_Altitude"},
	{WAttribute_Accuracy_Report,
	 "Informe_sobre_Preciso_de_Altitude"},
	{WAttribute_Accuracy_Value,
	 "Valor_do_Atributo_de_Preciso"},
	{WAttribute_Definition,
	 "Definio_do_Atributo"},
	{WAttribute_Definition_Source,
	 "Fonte_de_Definio_de_Atributos"},
	{WAttribute_Domain_Values,
	 "Valores_vlidos_de_atributo"},
	{WAttribute_Label,
	 "Rtulo_de_Atributo"},
	{WAttribute_Measurement_Frequency,
	 "Frequencia_de_Medidas_do_Atributo"},
	{WAttribute_Measurement_Resolution,
	 "Resoluo_de_Medidas_do_Atributo"},
	{WAttribute_Units_of_Measure,
	 "Unidades_de_Medida_do_Atributo"},
	{WAttribute_Value_Accuracy,
	 "Valor_da_Acurcia_do_Atributo"},
	{WAttribute_Value_Accuracy_Explanation,
	 "Explicao_do_Valor_da_Acurcia_do_Atributo"},
	{WAttribute_Value_Accuracy_Information,
	 "Informao_do_Valor_da_Acurcia_do_Atributo"},
	{WAvailable_Time_Period,
	 "Perodo_com_dados_disponveis"},
	{WAzimuthal_Angle,
	 "ngulo_Azimutal"},
	{WAzimuthal_Equidistant,
	 "Equidistante_Azimutal"},
	{WAzimuth_Measure_Point_Longitude,
	 "Longitude_do_Azimute_do_Ponto_de_Medida"},
	{WBearing_Reference_Direction,
	 "Direo_do_Rumo_de_Referncia"},
	{WBearing_Reference_Meridian,
	 "Meridiano_do_Rumo_de_Referncia"},
	{WBearing_Resolution,
	 "Resoluo_do_Rumo"},
	{WBearing_Units,
	 "Unidades_de_Rumo"},
	{WBeginning_Date,
	 "Data_de_Incio"},
	{WBeginning_Date_of_Attribute_Values,
	 "Data_de_Incio_dos_Valores_de_Atributo"},
	{WBeginning_Time,
	 "Hora_de_Incio"},
	{WBounding_Coordinates,
	 "Coordenadas_Limites"},
	{WBrowse_Graphic,
	 "Grfico_de_Visualizao"},
	{WBrowse_Graphic_File_Description,
	 "Descrio_do_Grfico_de_Visualizao"},
	{WBrowse_Graphic_File_Name,
	 "Nome_do_Arquivo_Grfico_para_Visualizao"},
	{WBrowse_Graphic_File_Type,
	 "Formato_do_Arquivo_Grfico_para_Visualizao"},
	{WCalendar_Date,
	 "Data_Calendrio"},
	{WCitation,
	 "Citao"},
	{WCitation_Information,
	 "Informao_de_Citao"},
	{WCity,
	 "Cidade"},
	{WCloud_Cover,
	 "Cobertura_de_Nuvens"},
	{WCodeset_Domain,
	 "Codeset_Domain"},
	{WCodeset_Name,
	 "Codeset_Name"},
	{WCodeset_Source,
	 "Codeset_Source"},
	{WColumn_Count,
	 "Nmero_de_Objetos_Raster_no_Eixo_X"},
	{WCompatibility_Information,
	 "Informao_de_Compatibilidade"},
	{WCompleteness_Report,
	 "Relatrio_de_Completitude"},
	{WCompression_Support,
	 "Protocolos_de_Compresso"},
	{WComputer_Contact_Information,
	 "Informaco_de_Contato_com_Computador"},
	{WContact_Address,
	 "Endereo_de_Contato"},
	{WContact_Electronic_Mail_Address,
	 "Endereo_Eletrnico_do__Contato"},
	{WContact_Facsimile_Telephone,
	 "Nmero_de_Fax_do_Contato"},
	{WContact_Information,
	 "Informao_do_Contato"},
	{WContact_Instructions,
	 "Instrues_para_Contato"},
	{WContact_Organization,
	 "Organizao_para_Contato"},
	{WContact_Organization_Primary,
	 "Organizao_de_Preferncia_para_Contato"},
	{WContact_Person,
	 "Pessoa_de_Contato"},
	{WContact_Person_Primary,
	 "Pessoa_Preferencial_para_Contato"},
	{WContact_Position,
	 "Cargo_da_Pessoa_de_Contato"},
	{WContact_TDD_TTY_Telephone,
	 "Telefone_da_TDD/TTY_de_Contato"},
	{WContact_Voice_Telephone,
	 "Telefone_da_Pessoa_de_Contato"},
	{WCoordinate_Representation,
	 "Representaao_de_Coordenadas"},
	{WCountry,
	 "Pas"},
	{WCross_Reference,
	 "Referncia_Cruzada"},
	{WCurrentness_Reference,
	 "Vigncia"},
	{WCustom_Order_Process,
	 "Processamento_Personalizado_de_Produtos"},
	{WData_Quality_Information,
	 "Informao_sobre_qualidade_de_dados"},
	{WData_Set_Credit,
	 "Crdito_do_Conjunto_de_Dados"},
	{WData_Set_G_Polygon,
	 "Polgono_Limite_dos_dados"},
	{WData_Set_G_Polygon_Exclusion_G_Ring,
	 "Polgono_Excluso_dos_Dados"},
	{WData_Set_G_Polygon_Outer_G_Ring,
	 "Polgono_Externo_aos_Dados"},
	{WDenominator_of_Flattening_Ratio,
	 "Denominador_da_Razo_de_Achatamento"},
	{WDepth_Datum_Name,
	 "Nome_do_Datum_de_Profundidade"},
	{WDepth_Distance_Units,
	 "Unidades_de_Distncia_de_Profundidade"},
	{WDepth_Encoding_Method,
	 "Mtodo_de_Codificao_de_Profundidade"},
	{WDepth_Resolution,
	 "Resoluo_da_Profundidade"},
	{WDepth_System_Definition,
	 "Definio_do_Sistema_de_Profundidade"},
	{WDescription,
	 "Descrio "},
	{WDetailed_Description,
	 "Descrio_Detalhada"},
	{WDialup_File_Name,
	 "Nome_de_Arquivo_Discado_Automtico"},
	{WDialup_Instructions,
	 "Instruo_para_Arquivo_Discado_Automtico"},
	{WDialup_Telephone,
	 "Nmero_para_Arquivo_Discado_Automtico"},
	{WDigital_Form,
	 "Formato_Digital"},
	{WDigital_Transfer_Information,
	 "Informao_sobre_Transferncia_Digital"},
	{WDigital_Transfer_Option,
	 "Opo_de_Transferncia_Digital"},
	{WDirect_Spatial_Reference_Method,
	 "Mtodo_de_Referencia_Espacial_Direta"},
	{WDistance_and_Bearing_Representation,
	 "Representaco_de_Distancia_y_Rumo"},
	{WDistance_Resolution,
	 "Resoluo_de_Distncia"},
	{WDistribution_Information,
	 "Informao_sobre_Distribuio"},
	{WDistribution_Liability,
	 "Responsabilidades_do_Distibuidor"},
	{WDistributor,
	 "Distribuidor"},
	{WEast_Bounding_Coordinate,
	 "Coordenada_Limite_Leste"},
	{WEdition,
	 "Edio"},
	{WEllipsoid_Name,
	 "Nome_do_Elipside"},
	{WEnding_Date,
	 "Data_de_Trmino"},
	{WEnding_Date_of_Attribute_Values,
	 "Data_de_Trmino_dos_Valores_de_Atributo"},
	{WEnding_Time,
	 "Data_de_Finalizao"},
	{WEntity_and_Attribute_Detail_Citation,
	 "Citao_Detalhada_de_Entidade_e_Atributo"},
	{WEntity_and_Attribute_Information,
	 "Informao_de_Entidades_e_Atributos"},
	{WEntity_and_Attribute_Overview,
	 "Resumo_de_Entidades_e_Atributos"},
	{WEntity_Type,
	 "Tipo_de_Entidade"},
	{WEntity_Type_Definition,
	 "Definio_do_Tipo_de_Entidades"},
	{WEntity_Type_Definition_Source,
	 "Fonte_de_Definio_do_Tipo_de_Entidade"},
	{WEntity_Type_Label,
	 "Rtulo_de_Tipo_de_Entidade"},
	{WEnumerated_Domain,
	 "Valores_Vlidos_de_Campo_Numrico"},
	{WEnumerated_Domain_Value,
	 "Valores_Vlidos_de_Campo_Numrico"},
	{WEnumerated_Domain_Value_Definition,
	 "Definio_Valores_Vlidos_de_Campo_Numrico"},
	{WEnumerated_Domain_Value_Definition_Source,
	 "Fonte_Definio_Valores_Vlidos_de_Campo_Numrico"},
	{WEquidistant_Conic,
	 "Cnica_Equidistante"},
	{WEquirectangular,
	 "Equirretangular"},
	{WFalse_Easting,
	 "Falso_leste"},
	{WFalse_Northing,
	 "Falso_norte"},
	{WFees,
	 "Tarifas"},
	{WFile_Decompression_Technique,
	 "Tcnica_de_descompresso_de_arquivos"},
	{WFormat_Information_Content,
	 "Informao_de_Formato_de_Contedo"},
	{WFormat_Name,
	 "Nome_do_Formato"},
	{WFormat_Specification,
	 "Especificao_do_Formato"},
	{WFormat_Version_Date,
	 "Data_de_verso_do_Formato"},
	{WFormat_Version_Number,
	 "Nemro_de_verso_do_Formato"},
	{WG_Ring,
	 "Poligonal_Geogrfica"},
	{WG_Ring_Latitude,
	 "Latitude_da_Poligonal"},
	{WG_Ring_Longitude,
	 "Longitude_da_Poligonal"},
	{WG_Ring_Point,
	 "Ponto_de_Poligonal_Geogrfica"},
	{WGeneral_Vertical_Near_sided_Perspective,
	 "General_Vertical_Near-sided_Perspective"},
	{WGeodetic_Model,
	 "Modelo_Geodsico"},
	{WGeographic,
	 "Geogrfico"},
	{WGeographic_Coordinate_Units,
	 "Unidades_das_Coordenadas_Geogrficas"},
	{WGeospatial_Data_Presentation_Form,
	 "Forma_de_Apresentao_Geoespacial"},
	{WGnomonic,
	 "Gnomnica"},
	{WGrid_Coordinate_System,
	 "Sistema_de_Coordenadas_da_Quadrcula"},
	{WGrid_Coordinate_System_Name,
	 "Nome_do_Sistema_de_Coordenadas_da_Quadrcula"},
	{WHeight_of_Perspective_Point_Above_Surface,
	 "Altura_do_Ponto_de_Perspectiva_sobre_a _superfcie"},
	{WHighest_BPS,
	 "Mximo_BPS"},
	{WHorizontal_Coordinate_System_Definition,
	 "Definio_do_Sistema_de_Coordenadas_Horizontais"},
	{WHorizontal_Datum_Name,
	 "Nome_do_Datum_Horizontal"},
	{WHorizontal_Positional_Accuracy,
	 "Acurcia_Horizontal"},
	{WHorizontal_Positional_Accuracy_Explanation,
	 "Interpretao_da_Acurcia_Horizontal "},
	{WHorizontal_Positional_Accuracy_Report,
	 "Relatrio_sobre_Acurcia_Horizontal"},
	{WHorizontal_Positional_Accuracy_Value,
	 "Valor_da_Acurcia_Horizontal"},
	{WHours_of_Service,
	 "Expediente"},
	{WIdentification_Information,
	 "Informao_de_Identificao"},
	{WIndirect_Spatial_Reference,
	 "Referncia_Espacial_Indireta"},
	{WIssue_Identification,
	 "Identificao_do_Volume"},
	{WKeywords,
	 "Palavras_Chave"},
	{WLambert_Azimuthal_Equal_Area,
	 "Projeo_Azimutal_de_Lambert"},
	{WLambert_Conformal_Conic,
	 "Projeo_Conforme_Cnica_de_Lambert"},
	{WLandsat_Number,
	 "Nmero_do_Satlite_Landsat"},
	{WLarger_Work_Citation,
	 "Citao_do_Trabalho_maior"},
	{WLatitude_of_Projection_Center,
	 "Latitude_do_Centro_da_Projeo"},
	{WLatitude_of_Projection_Origin,
	 "Latitude_da_Origem_da_Projeo"},
	{WLatitude_Resolution,
	 "Resoluo_da_latitude"},
	{WLineage,
	 "Genealogia"},
	{WLocal,
	 "Local"},
	{WLocal_Description,
	 "Descrio_Local"},
	{WLocal_Georeference_Information,
	 "Informao_Local_de_Georrefenciamento"},
	{WLocal_Planar,
	 "Planar_local"},
	{WLocal_Planar_Description,
	 "Descrio_da_Planar_Local"},
	{WLocal_Planar_Georeference_Information,
	 "Informao_de_Georreferenciao_da_Planar_Local"},
	{WLogical_Consistency_Report,
	 "Relatrio_de_Consistncia_Lgica"},
	{WLongitude_of_Central_Meridian,
	 "Longitude_do_Meridiano_Central"},
	{WLongitude_of_Projection_Center,
	 "Longitude_do_Centro_da_Projeo"},
	{WLongitude_Resolution,
	 "Resoluo_da_longitude"},
	{WLowest_BPS,
	 "Mnimo_BPS"},
	{WMaintenance_and_Update_Frequency,
	 "Frequencia_de_manuteno_e_atualizao"},
	{WMap_Projection,
	 "Projeo_Cartogrfica"},
	{WMap_Projection_Name,
	 "Nome_da_Projeo_Cartogrfica"},
	{WMap_Projection_Parameters,
	 "Parmetros_da_Projeo_Cartogrfica"},
	{WMercator,
	 "Mercator"},
	{WMetadata,
	 "Metadados"},
	{WMetadata_Access_Constraints,
	 "Restries_de_Acesso_a_Metadados"},
	{WMetadata_Contact,
	 "Contato_para_Metadados"},
	{WMetadata_Date,
	 "Data_dos_Metadados"},
	{WMetadata_Extensions,
	 "Extenses_ao_Padro_de_Metadados"},
	{WMetadata_Future_Review_Date,
	 "Data_da_Futura_Reviso_dos_Metadados"},
	{WMetadata_Language,
	 "Linguagem_dos_Metadados"},
	{WMetadata_Reference_Information,
	 "Informao_de_Referncia_dos_Metadados"},
	{WMetadata_Review_Date,
	 "Data_da_ltima_Reviso_dos_Metadados"},
	{WMetadata_Security_Classification,
	 "Classificao_de_Segurana_dos_Metadados"},
	{WMetadata_Security_Classification_System,
	 "Sistema_de_Classificao_de_Segurana_dos_Metadados"},
	{WMetadata_Security_Handling_Description,
	 "Precaues_de_Manuseio_dos_Metadados"},
	{WMetadata_Security_Information,
	 "Informao_de_Segurana_dos_Metadados"},
	{WMetadata_Standard_Name,
	 "Nome_do_Padro_de_Metadados"},
	{WMetadata_Standard_Version,
	 "Verso_do_Padro_de_Metadados"},
	{WMetadata_Time_Convention,
	 "Convenes_Temporais_dos_Metadados"},
	{WMetadata_Use_Constraints,
	 "Restrio_de_Uso_dos_Metadados"},
	{WMiller_Cylindrical,
	 "Projeo_Cilndrica_de_Miller"},
	{WModified_Stereographic_for_Alaska,
	 "Projeo_Estereogrfica_Modificada_para_o_Alaska"},
	{WMultiple_Dates_Times,
	 "Multiplas_Datas_e_Horas"},
	{WNative_Data_Set_Environment,
	 "Ambiente_Original_dos_Dados"},
	{WNetwork_Address,
	 "Endereo_de_rede"},
	{WNetwork_Resource_Name,
	 "Nome_do_Recurso_de_Rede"},
	{WNon_digital_Form,
	 "Formato_no_digital"},
	{WNorth_Bounding_Coordinate,
	 "Coordenada_limite_norte"},
	{WNumber_DataBits,
	 "Nmero_de_Bits_de_Dados"},
	{WNumber_StopBits,
	 "Nmero_de_Stop_bits"},
	{WOblique_Line_Azimuth,
	 "Azimute_de_Linha_Oblqua"},
	{WOblique_Line_Latitude,
	 "Latitude_de_Linha_Oblqua"},
	{WOblique_Line_Longitude,
	 "Longitude_de_Linha_Oblqua"},
	{WOblique_Line_Point,
	 "Ponto_de_Linha_Obqua"},
	{WOblique_Mercator,
	 "Projeo_Mercator_Oblqua"},
	{WOffline_Media,
	 "Meio_Fsico_Fora_de_Linha"},
	{WOffline_Option,
	 "Opo_Fora_de_linha"},
	{WOnline_Computer_and_Operating_System,
	 "Computador_e_Sistema_Operacional"},
	{WOnline_Linkage,
	 "Acesso_em_rede"},
	{WOnline_Option,
	 "Opo_em _rede"},
	{WOrdering_Instructions,
	 "Instrues_para_encomenda"},
	{WOrdinate_Resolution,
	 "Resoluo_das_Ordenadas"},
	{WOriginator,
	 "Produtor"},
	{WOrthographic,
	 "Ortogrfico"},
	{WOther_Citation_Details,
	 "Outros_Detalhes_de_Citao"},
	{WOther_Grid_Systems_Definition,
	 "Definio_de_Outro_sistema_de_quadrcula"},
	{WOther_Projections_Definition,
	 "Definio_de_Outra_Projeo"},
	{WOverview_Description,
	 "Descrio_Sumria"},
	{WParity,
	 "Paridade"},
	{WPath_Number,
	 "Caminho_de_Diretrio"},
	{WPlace,
	 "Lugar"},
	{WPlace_Keyword,
	 "Palavra_Chave_de_lugar"},
	{WPlace_Keyword_Thesaurus,
	 "Glossrio_de_Palavras_Chave_de_Lugar"},
	{WPlanar,
	 "Coordenadas_planas"},
	{WPlanar_Coordinate_Encoding_Method,
	 "Mtodo_de_Codificao_de_Coordenadas_Planas"},
	{WPlanar_Coordinate_Information,
	 "Informao_de_Coordenadas_Planas"},
	{WPlanar_Distance_Units,
	 "Unidades_de_Distncias_Planas"},
	{WPoint_and_Vector_Object_Count,
	 "Quantidade_de_Objetos_Pontuais_e_Vetoriais"},
	{WPoint_and_Vector_Object_Information,
	 "Informao_sobre_Objetos_Pontuais_e_Vetoriais"},
	{WPoint_of_Contact,
	 "Ponto_de_Contato"},
	{WPolar_Stereographic,
	 "Projeo_Polar_Estereogrfica"},
	{WPolyconic,
	 "Projeo_Policnica"},
	{WPositional_Accuracy,
	 "Acurcia_de_Posio"},
	{WPostal_Code,
	 "Cdigo Postal"},
	{WProcess_Contact,
	 "Contato_sobre_processo"},
	{WProcess_Date,
	 "Data_de_Processo"},
	{WProcess_Description,
	 "Descrio_de_Processo"},
	{WProcess_Step,
	 "Passo_de_Processo"},
	{WProcess_Time,
	 "Hora_de_Processo"},
	{WProfile_Name,
	 "Nome_de_Perfil"},
	{WProgress,
	 "Progresso"},
	{WPublication_Date,
	 "Data_de_Publicao"},
	{WPublication_Information,
	 "Informao_de_Publicao"},
	{WPublication_Place,
	 "Local_de_Publicao"},
	{WPublication_Time,
	 "Hora_da_Publicao"},
	{WPublisher,
	 "Editor"},
	{WPurpose,
	 "Objetivo"},
	{WQuantitative_Attribute_Accuracy_Assessment,
	 "Determinao_Quantitativa_sobre_Acurcia_de_Atributos"},
	{WQuantitative_Horizontal_Positional_Accuracy_Assessment,
	 "Avaliao_da_Acurcia_Horizontal"},
	{WQuantitative_Vertical_Positional_Accuracy_Assessment,
	 "Avaliao_da_Acurcia_Vertical"},
	{WRange_Domain,
	 "Gama_de_Valores_Possveis"},
	{WRange_Domain_Maximum,
	 "Valor_Possvel_Mximo"},
	{WRange_Domain_Minimum,
	 "Valor_Possvel_Mnimo"},
	{WRange_of_Dates_Times,
	 "Gama_de_Datas_e_Horas"},
	{WRaster_Object_Information,
	 "Informao_sobre_Objetos_Raster"},
	{WRaster_Object_Type,
	 "Tipos_de_Objetos_Raster"},
	{WRecording_Capacity,
	 "Capacidade_de_Gravao"},
	{WRecording_Density,
	 "Densidade_de_Gravao"},
	{WRecording_Density_Units,
	 "Unidades_de_Densidade_de_Gravao"},
	{WRecording_Format,
	 "Formato_de_Gravao"},
	{WResource_Description,
	 "Descrio_de_Recursos"},
	{WRobinson,
	 "Projeo_de_Robinson"},
	{WRow_Count,
	 "Nmero_de_Clulas_ no_eixo_das_ordenadas"},
	{WScale_Factor_at_Center_Line,
	 "Fator_de_Escala_na_Linha_Central"},
	{WScale_Factor_at_Central_Meridian,
	 "Fator_de_Escala_no_Meridiano_Central"},
	{WScale_Factor_at_Equator,
	 "Fator_de_Escala_no_Equador"},
	{WScale_Factor_at_Projection_Origin,
	 "Fator_de_Escala_na_Origem_da_Projeo"},
	{WSDTS_Point_and_Vector_Object_Type,
	 "Tipos_de_Objetos_Ponto_e_Vetor_SDTS"},
	{WSDTS_Terms_Description,
	 "Descrio_de_Termos_SDTS"},
	{WSecurity_Classification,
	 "Classificao_de_Segurana"},
	{WSecurity_Classification_System,
	 "Sistema_de_Classificao_de_Segurana"},
	{WSecurity_Handling_Description,
	 "Descrio_do_manuseio_de_segurana"},
	{WSecurity_Information,
	 "Informao_sobre_Segurana"},
	{WSemi_major_Axis,
	 "Semi_eixo_maior"},
	{WSeries_Information,
	 "Informao_sobre_a_srie"},
	{WSeries_Name,
	 "Nome_da_Srie"},
	{WSingle_Date_Time,
	 "Data_e_Hora"},
	{WSinusoidal,
	 "Projeo Sinusoidal"},
	{WSource_Citation,
	 "Citao_da_Fonte"},
	{WSource_Citation_Abbreviation,
	 "Abreviao_de_Citao_da_Fonte"},
	{WSource_Contribution,
	 "Contrribuio_da_Fonte"},
	{WSource_Currentness_Reference,
	 "Referncia_da_Fonte"},
	{WSource_Information,
	 "Informao_da_Fonte"},
	{WSource_Produced_Citation_Abbreviation,
	 "Abreviatura_de_Citaao_da_Fonte"},
	{WSource_Scale_Denominator,
	 "Denominao_da_Escala_da_Fonte"},
	{WSource_Time_Period_of_Content,
	 "Perodo_Contido_na_Fonte"},
	{WSource_Used_Citation_Abbreviation,
	 "Abreviatura_Usual_de_Citao_da_Fonte"},
	{WSouth_Bounding_Coordinate,
	 "Coordenada_Limite_Sul"},
	{WSpace_Oblique_Mercator_Landsat,
	 "Projeo_Mercator_Espacial_Obliqua_(LANDSAT)"},
	{WSpatial_Data_Organization_Information,
	 "Organizao_de_Organizao_dos_Dados_Espaciais"},
	{WSpatial_Domain,
	 "Domnio_Espacial"},
	{WSpatial_Reference_Information,
	 "Informao_de_Referncia_Espacial"},
	{WSPCS_Zone_Identifier,
	 "Identificador_de_Zona_SPCS"},
	{WStandard_Order_Process,
	 "Processo_Padro_de_Requisio"},
	{WStandard_Parallel,
	 "Paralelo_Padro"},
	{WState_or_Province,
	 "Estado_ou_Provncia"},
	{WState_Plane_Coordinate_System,
	 "Sistema_Estadual_de_Coordenadas_(EUA)"},
	{WStatus,
	 "Status"},
	{WStereographic,
	 "Projeo Estereogrfica"},
	{WStraight_Vertical_Longitude_from_Pole,
	 "Longitude_Vertical_Reta_a_partir_do_Polo"},
	{WStratum,
	 "Estrato"},
	{WStratum_Keyword,
	 "Palavra_Chave_de_Estrato"},
	{WStratum_Keyword_Thesaurus,
	 "Relao_de_PalavraChaves_de_Estrato"},
	{WSupplemental_Information,
	 "Informao_Suplementar"},
	{WTechnical_Prerequisites,
	 "Pre-Requisitos_Tecnicos"},
	{WTemporal,
	 "Temporal"},
	{WTemporal_Keyword,
	 "Palavra_Chave_de_Tempo"},
	{WTemporal_Keyword_Thesaurus,
	 "Relao_de_PalavraChaves_de_Tempo"},
	{WTheme,
	 "Tema"},
	{WTheme_Keyword,
	 "Palavra_Chave_de_Tempo"},
	{WTheme_Keyword_Thesaurus,
	 "Relao_de_Palavras_Chaves_de_Tema"},
	{WTime_of_Day,
	 "Hora_do_Dia"},
	{WTime_Period_Information,
	 "Informao_de_Perodo_de_Tenpo"},
	{WTime_Period_of_Content,
	 "Perodo_Associado_ao_Contedo"},
	{WTitle,
	 "Ttulo"},
	{WTransfer_Size,
	 "Tamanho_de_Transferncia"},
	{WTransverse_Mercator,
	 "Projeo_Transversa_de_Mercator"},
	{WTurnaround,
	 "Tempo_de_Entrega"},
	{WType_of_Source_Media,
	 "Tipo_de_Meio_Fsico_da_Fonte"},
	{WUniversal_Polar_Stereographic,
	 "Projeo_Estereogrfica_Polar_Universal"},
	{WUniversal_Transverse_Mercator,
	 "Projeo_Universal_Transversa_de_Mercator"},
	{WUnrepresentable_Domain,
	 "Domnio_no_representvel"},
	{WUPS_Zone_Identifier,
	 "Identificador_de_Zona_UPS"},
	{WUse_Constraints,
	 "Restries_de_Uso"},
	{WUTM_Zone_Number,
	 "Nmero_de_Zona_UTM"},
	{Wvan_der_Grinten,
	 "Projeo_Van_der_Grinten"},
	{WVertical_Coordinate_System_Definition,
	 "Definio_do_Sistema_de_Coordenadas_Vertical"},
	{WVertical_Count,
	 "Nmero_de_Objetos_Raster_no_Eixo_Z"},
	{WVertical_Positional_Accuracy,
	 "Acurcia_vertical"},
	{WVertical_Positional_Accuracy_Explanation,
	 "Interpretao_da_Acurcia_Vertical"},
	{WVertical_Positional_Accuracy_Report,
	 "Informao_sobre_Acurcia_Vertical"},
	{WVertical_Positional_Accuracy_Value,
	 "Valor_da_Acurcia_Vertical"},
	{WVPF_Point_and_Vector_Object_Information,
	 "Informao_sobre_Objetos_Ponto_e_Vetor_VPF"},
	{WVPF_Point_and_Vector_Object_Type,
	 "Tipo_de_Objetos_Ponto_e_Vetor_VPF"},
	{WVPF_Terms_Description,
	 "Descrio_de_Termos_VPF"},
	{WVPF_Topology_Level,
	 "Nvel_de_Topologia_VPF"},
	{WWest_Bounding_Coordinate,
	 "Coordenada_Limite_Oeste"},

/* Biological Data Profile */

	{WDescription_of_Geographic_Extent,
		"Description_of_Geographic_Extent"},
	{WBounding_Altitudes,
		"Bounding_Altitudes"},
	{WAltitude_Minimum,
		"Altitude_Minimum"},
	{WAltitude_Maximum,
		"Altitude_Maximum"},
	{WTaxonomy,
		"Taxonomy"},
	{WKeywords_Taxon,
		"Keywords/Taxon"},
	{WTaxonomic_Keyword_Thesaurus,
		"Taxonomic_Keyword_Thesaurus"},
	{WTaxonomic_Keywords,
		"Taxonomic_Keywords"},
	{WTaxonomic_System,
		"Taxonomic_System"},
	{WClassification_System_Authority,
		"Classification_System/Authority"},
	{WClassification_System_Citation,
		"Classification_System_Citation"},
	{WClassification_System_Modifications,
		"Classification_System_Modifications"},
	{WIdentification_Reference,
		"Identification_Reference"},
	{WIdentifier,
		"Identifier"},
	{WTaxonomic_Procedures,
		"Taxonomic_Procedures"},
	{WTaxonomic_Completeness,
		"Taxonomic_Completeness"},
	{WVouchers,
		"Vouchers"},
	{WSpecimen,
		"Specimen"},
	{WRepository,
		"Repository"},
	{WGeneral_Taxonomic_Coverage,
		"General_Taxonomic_Coverage"},
	{WTaxonomic_Classification,
		"Taxonomic_Classification"},
	{WTaxon_Rank_Name,
		"Taxon_Rank_Name"},
	{WTaxon_Rank_Value,
		"Taxon_Rank_Value"},
	{WApplicable_Common_Name,
		"Applicable_Common_Name"},
	{WAnalytical_Tool,
		"Analytical_Tool"},
	{WAnalytical_Tool_Description,
		"Analytical_Tool_Description"},
	{WTool_Access_Information,
		"Tool_Access_Information"},
	{WTool_Contact,
		"Tool_Contact"},
	{WTool_Citation,
		"Tool_Citation"},
	{WTool_Access_Instructions,
		"Tool_Access_Instructions"},
	{WTool_Computer_and_Operating_System,
		"Tool_Computer_and_Operating_System"},
	{WMethodology,
		"Methodology"},
	{WMethodology_Type,
		"Methodology_Type"},
	{WMethodology_Identifier,
		"Methodology_Identifier"},
	{WMethodology_Keyword_Thesaurus,
		"Methodology_Keyword_Thesaurus"},
	{WMethodology_Keyword,
		"Methodology_Keyword"},
	{WMethodology_Description,
		"Methodology_Description"},
	{WMethodology_Citation,
		"Methodology_Citation"},
	{WASCII_File_Structure,
		"ASCII_File_Structure"},
	{WRecord_Delimiter,
		"Record_Delimiter"},
	{WNumber_Header_Lines,
		"Number_Header_Lines"},
	{WDescription_of_Header_Content,
		"Description_of_Header_Content"},
	{WOrientation,
		"Orientation"},
	{WCase_Sensitive,
		"Case_Sensitive"},
	{WAuthentication,
		"Authentication"},
	{WQuote_Character,
		"Quote_Character"},
	{WData_Field,
		"Data_Field"},
	{WData_Field_Name,
		"Data_Field_Name"},
	{WMissing_Value_Code,
		"Missing_Value_Code"},
	{WData_Field_Width_Delimiter,
		"Data_Field_Width_Delimiter"},
	{WData_Field_Width,
		"Data_Field_Width"},
	{WBeginning_Geologic_Age,
		"Beginning_Geologic_Age"},
	{WEnding_Geologic_Age,
		"Ending_Geologic_Age"},
	{WGeologic_Age,
		"Geologic_Age"},
	{WGeologic_Time_Scale,
		"Geologic_Time_Scale"},
	{WGeologic_Age_Estimate,
		"Geologic_Age_Estimate"},
	{WGeologic_Age_Uncertainty,
		"Geologic_Age_Uncertainty"},
	{WGeologic_Age_Explanation,
		"Geologic_Age_Explanation"},
	{WGeologic_Citation,
		"Geologic_Citation"},

/* Shoreline profile */

	{WBarometric_Pressure,
	 "Barometric_Pressure"},
	{WEnvironmental_Event,
	 "Environmental_Event"},
	{WMarine_Weather_Condition,
	 "Marine_Weather_Condition"},
	{WProcess_Step_Citation,
	 "Process_Step_Citation"},
	{WRange_of_Tide,
	 "Range_of_Tide"},
	{WSupplemental_Tidal_Information,
	 "Supplemental_Tidal_Information"},
	{WTidal_Datum,
	 "Tidal_Datum"},
	{WTidal_Information,
	 "Tidal_Information"},
	{WTide_Table_Reference,
	 "Tide_Table_Reference"},
	{WTime_of_High_Tide,
	 "Time_of_High_Tide"},
	{WTime_of_Low_Tide,
	 "Time_of_Low_Tide"},
	{WTime_of_Tide,
	 "Time_of_Tide"},
	{WType_of_Tide,
	 "Type_of_Tide"},
	{WWave_Height,
	 "Wave_Height"},
	{WWind_Direction,
	 "Wind_Direction"},
	{WWind_Speed,
	 "Wind_Speed"},

/* Remote-sensing profile */

	{WAcquisition_Information,
	 "Acquisition_Information"},
	{WAerial_Film_Speed,
	 "Aerial_Film_Speed"},
	{WAerotriangulation_Reference,
	 "Aerotriangulation_Reference"},
	{WAffine_Distortion_X_Prime_Coefficient,
	 "Affine_Distortion_X_Prime_Coefficient"},
	{WAffine_Distortion_Y_Prime_Coefficient,
	 "Affine_Distortion_Y_Prime_Coefficient"},
	{WAggregation_Criteria,
	 "Aggregation_Criteria"},
	{WAggregation_Information,
	 "Aggregation_Information"},
	{WAggregation_Member_ID,
	 "Aggregation_Member_ID"},
	{WAlgorithm_Change_Description,
	 "Algorithm_Change_Description"},
	{WAlgorithm_Change_History,
	 "Algorithm_Change_History"},
	{WAlgorithm_Change_Reference,
	 "Algorithm_Change_Reference"},
	{WAlgorithm_Description,
	 "Algorithm_Description"},
	{WAlgorithm_Identifiers,
	 "Algorithm_Identifiers"},
	{WAlgorithm_Information,
	 "Algorithm_Information"},
	{WAlgorithm_Peer_Review_Description,
	 "Algorithm_Peer_Review_Description"},
	{WAlgorithm_Peer_Review_Information,
	 "Algorithm_Peer_Review_Information"},
	{WAlgorithm_Peer_Review_Reference,
	 "Algorithm_Peer_Review_Reference"},
	{WAlgorithm_Reference,
	 "Algorithm_Reference"},
	{WAlgorithm_Text_Description,
	 "Algorithm_Text_Description"},
	{WAltitude_Distance_Layer,
	 "Altitude_Distance_Layer"},
	{WAncillary_Dataset,
	 "Ancillary_Dataset"},
	{WAncillary_Dataset_Description,
	 "Ancillary_Dataset_Description"},
	{WAncillary_Dataset_Identifier,
	 "Ancillary_Dataset_Identifier"},
	{WAncillary_Description,
	 "Ancillary_Description"},
	{WAncillary_Reference,
	 "Ancillary_Reference"},
	{WAngle_Dependent_Distortion,
	 "Angle_Dependent_Distortion"},
	{WAngle_Distortion_Value,
	 "Angle_Distortion_Value"},
	{WArea_Weighted_Average_Resolution,
	 "Area_Weighted_Average_Resolution"},
	{WArgument_of_Perigee,
	 "Argument_of_Perigee"},
	{WAscending_Descending_Indicator,
	 "Ascending/Descending_Indicator"},
	{WAttitude_Angular_Units,
	 "Attitude_Angular_Units"},
	{WAvailability_of_Element_Locations,
	 "Availability_of_Element_Locations"},
	{WAxes,
	 "Axes"},
	{WAxis_Rotation_Convention,
	 "Axis_Rotation_Convention"},
	{WBand_Boundary_Definition,
	 "Band_Boundary_Definition"},
	{WBand_Description,
	 "Band_Description"},
	{WBand_ID,
	 "Band_ID"},
	{WBand_Identification,
	 "Band_Identification"},
	{WBand_Measurement_Mode_ID,
	 "Band_Measurement_Mode_ID"},
	{WBand_Quality,
	 "Band_Quality"},
	{WCalibrated_Detector_Positions,
	 "Calibrated_Detector_Positions"},
	{WCalibrated_Focal_Length,
	 "Calibrated_Focal_Length"},
	{WCamera_Identifier,
	 "Camera_Identifier"},
	{WCamera_Type,
	 "Camera_Type"},
	{WCell_Value_Type,
	 "Cell_Value_Type"},
	{WClock_Time_Drift,
	 "Clock_Time/Drift"},
	{WCollection_Type,
	 "Collection_Type"},
	{WColumn_Delta_X,
	 "Column_Delta_X"},
	{WColumn_Delta_Y,
	 "Column_Delta_Y"},
	{WCommand_Line_Processing_Parameter,
	 "Command_Line_Processing_Parameter"},
	{WComponent_Information,
	 "Component_Information"},
	{WContainer_Packet_ID,
	 "Container_Packet_ID"},
	{WControl_Point_Authority,
	 "Control_Point_Authority"},
	{WControl_Point_Column,
	 "Control_Point_Column"},
	{WControl_Point_Earth_Location,
	 "Control_Point_Earth_Location"},
	{WControl_Point_Horizontal_X_Accuracy,
	 "Control_Point_Horizontal_X_Accuracy"},
	{WControl_Point_Horizontal_Y_Accuracy,
	 "Control_Point_Horizontal_Y_Accuracy"},
	{WControl_Point_ID,
	 "Control_Point_ID"},
	{WControl_Point_Identification,
	 "Control_Point_Identification"},
	{WControl_Point_Origin,
	 "Control_Point_Origin"},
	{WControl_Point_Raster_Position,
	 "Control_Point_Raster_Position"},
	{WControl_Point_Row,
	 "Control_Point_Row"},
	{WControl_Point_Type,
	 "Control_Point_Type"},
	{WControl_Point_Use_Flag,
	 "Control_Point_Use_Flag"},
	{WControl_Point_Vertical_Accuracy,
	 "Control_Point_Vertical_Accuracy"},
	{WControl_Point_x_Value,
	 "Control_Point_x_Value"},
	{WControl_Point_y_Value,
	 "Control_Point_y_Value"},
	{WControl_Point_z_Value,
	 "Control_Point_z_Value"},
	{WCoordinate_Point,
	 "Coordinate_Point"},
	{WCoordinate_System,
	 "Coordinate_System"},
	{WCoordinate_x_Value,
	 "Coordinate_x_Value"},
	{WCoordinate_XY_Units,
	 "Coordinate_XY_Units"},
	{WCoordinate_y_Value,
	 "Coordinate_y_Value"},
	{WCoordinate_Z_Units,
	 "Coordinate_Z_Units"},
	{WCoordinate_z_Value,
	 "Coordinate_z_Value"},
	{WCross_Track_Axis,
	 "Cross_Track_Axis"},
	{WCross_Track_Description,
	 "Cross_Track_Description"},
	{WCross_Track_Direction,
	 "Cross_Track_Direction"},
	{WCross_Track_Extent_Angle,
	 "Cross_Track_Extent_Angle"},
	{WCross_Track_Fixed_Angle,
	 "Cross_Track_Fixed_Angle"},
	{WCross_Track_Motion,
	 "Cross_Track_Motion"},
	{WCross_Track_Start_Angle,
	 "Cross_Track_Start_Angle"},
	{WCross_Track_Step_Angle,
	 "Cross_Track_Step_Angle"},
	{WCross_Track_Sweep,
	 "Cross_Track_Sweep"},
	{WCross_Track_Zero,
	 "Cross_Track_Zero"},
	{WData_Dictionary_Reference,
	 "Data_Dictionary_Reference"},
	{WData_Scaling_Information,
	 "Data_Scaling_Information"},
	{WDataset_Identifier,
	 "Dataset_Identifier"},
	{WDate_of_Last_Calibration,
	 "Date_of_Last_Calibration"},
	{WDeveloping_Institution,
	 "Developing_Institution"},
	{WDimension_Count,
	 "Dimension_Count"},
	{WDimension_Description,
	 "Dimension_Description"},
	{WDimension_Properties,
	 "Dimension_Properties"},
	{WDistance_Dependent_Distortion,
	 "Distance_Dependent_Distortion"},
	{WDistance_Distortion_Value,
	 "Distance_Distortion_Value"},
	{WDistortion,
	 "Distortion"},
	{WDistortion_Type_Affine,
	 "Distortion_Type_Affine"},
	{WDistortion_Type_Radial_Asymmetrical,
	 "Distortion_Type_Radial_Asymmetrical"},
	{WDistortion_Type_Radial_Symmetrical,
	 "Distortion_Type_Radial_Symmetrical"},
	{WDocumentation,
	 "Documentation"},
	{WEccentricity,
	 "Eccentricity"},
	{WEffective_Aerial_Film_Speed,
	 "Effective_Aerial_Film_Speed"},
	{WElevation_Axis,
	 "Elevation_Axis"},
	{WElevation_Description,
	 "Elevation_Description"},
	{WElevation_Direction,
	 "Elevation_Direction"},
	{WElevation_Extent_Angle,
	 "Elevation_Extent_Angle"},
	{WElevation_Fixed_Angle,
	 "Elevation_Fixed_Angle"},
	{WElevation_Start_Angle,
	 "Elevation_Start_Angle"},
	{WElevation_Step_Angle,
	 "Elevation_Step_Angle"},
	{WElevation_Sweep,
	 "Elevation_Sweep"},
	{WElevation_Zero,
	 "Elevation_Zero"},
	{WEphemeris,
	 "Ephemeris"},
	{WExposure_Time,
	 "Exposure_Time"},
	{WExterior_Orientation_Accuracy,
	 "Exterior_Orientation_Accuracy"},
	{WFiducial,
	 "Fiducial"},
	{WFiducial_Center,
	 "Fiducial_Center"},
	{WFilm_Type,
	 "Film_Type"},
	{WFilter_on_Camera,
	 "Filter_on_Camera"},
	{WFilter_on_Camera_Indicator,
	 "Filter_on_Camera_Indicator"},
	{WFilter_Type,
	 "Filter_Type"},
	{WFlight_Protocol,
	 "Flight_Protocol"},
	{WFlying_Height,
	 "Flying_Height"},
	{WForward_Motion_Compensation,
	 "Forward_Motion_Compensation"},
	{WFrame_Area,
	 "Frame_Area"},
	{WFrame_Area_Units,
	 "Frame_Area_Units"},
	{WFrame_Area_Value,
	 "Frame_Area_Value"},
	{WFrame_Camera,
	 "Frame_Camera"},
	{WFrame_Geometric_Properties,
	 "Frame_Geometric_Properties"},
	{WFrame_Hardware,
	 "Frame_Hardware"},
	{WFrame_Operation,
	 "Frame_Operation"},
	{WFrame_Optics,
	 "Frame_Optics"},
	{WFrame_Radiometric_Calibration,
	 "Frame_Radiometric_Calibration"},
	{WFrame_Radiometric_Properties,
	 "Frame_Radiometric_Properties"},
	{WFrame_Spectral_Information,
	 "Frame_Spectral_Information"},
	{WFrame_Spectral_Properties,
	 "Frame_Spectral_Properties"},
	{WGeorectified_Raster,
	 "Georectified_Raster"},
	{WGeoreferenceable_Raster,
	 "Georeferenceable_Raster"},
	{WGeoreferencing_Description,
	 "Georeferencing_Description"},
	{WGeoreferencing_Information,
	 "Georeferencing_Information"},
	{WGPS_Information_System_Availability,
	 "GPS_Information_System_Availability"},
	{WGrid_First_Element,
	 "Grid_First_Element"},
	{WGrid_First_Element_Map_X_Coordinate,
	 "Grid_First_Element_Map_X_Coordinate"},
	{WGrid_First_Element_Map_Y_Coordinate,
	 "Grid_First_Element_Map_Y_Coordinate"},
	{WGrid_Orientation,
	 "Grid_Orientation"},
	{WGround_Control_Point_Description,
	 "Ground_Control_Point_Description"},
	{WGround_Control_Point_Information,
	 "Ground_Control_Point_Information"},
	{WGround_Control_Point_Organization,
	 "Ground_Control_Point_Organization"},
	{WGround_Control_Point_Position,
	 "Ground_Control_Point_Position"},
	{WGround_Shape,
	 "Ground_Shape"},
	{WID_Overlapping_Dataset,
	 "ID_Overlapping_Dataset"},
	{WIFOV_Units,
	 "IFOV_Units"},
	{WIFOV_x_Definition,
	 "IFOV_x_Definition"},
	{WIFOV_x_Value,
	 "IFOV_x_Value"},
	{WIFOV_y_Definition,
	 "IFOV_y_Definition"},
	{WIFOV_y_Value,
	 "IFOV_y_Value"},
	{WImage_Quality,
	 "Image_Quality"},
	{WImage_Size,
	 "Image_Size"},
	{WImage_Size_x_Value,
	 "Image_Size_x_Value"},
	{WImage_Size_y_Value,
	 "Image_Size_y_Value"},
	{WInclination,
	 "Inclination"},
	{WIndividual_Band_Identification,
	 "Individual_Band_Identification"},
	{WInput_Dataset_Identifier,
	 "Input_Dataset_Identifier"},
	{WInput_Description,
	 "Input_Description"},
	{WInput_Level,
	 "Input_Level"},
	{WInput_Reference,
	 "Input_Reference"},
	{WINS_Reading_Availability,
	 "INS_Reading_Availability"},
	{WInstantaneous_Field_of_View,
	 "Instantaneous_Field_of_View"},
	{WInstitution_of_Last_Calibration,
	 "Institution_of_Last_Calibration"},
	{WInstrument_Description,
	 "Instrument_Description"},
	{WInstrument_Full_Name,
	 "Instrument_Full_Name"},
	{WInstrument_Information,
	 "Instrument_Information"},
	{WInstrument_Properties_Description,
	 "Instrument_Properties_Description"},
	{WInstrument_Reference,
	 "Instrument_Reference"},
	{WInstrument_Short_Name,
	 "Instrument_Short_Name"},
	{WInstrument_Specific_Georeferencing,
	 "Instrument_Specific_Georeferencing"},
	{WInstrument_Type,
	 "Instrument_Type"},
	{WKeplerian_Orbit,
	 "Keplerian_Orbit"},
	{WLast_Calibration,
	 "Last_Calibration"},
	{WLayer_Name,
	 "Layer_Name"},
	{WLens,
	 "Lens"},
	{WLens_Identifier,
	 "Lens_Identifier"},
	{WLens_Type,
	 "Lens_Type"},
	{WLight_Drop,
	 "Light_Drop"},
	{WLocation_Information,
	 "Location_Information"},
	{WMagazine_Identifier,
	 "Magazine_Identifier"},
	{WMaximum_Wavelength,
	 "Maximum_Wavelength"},
	{WMethod_of_Last_Calibration,
	 "Method_of_Last_Calibration"},
	{WMinimum_Wavelength,
	 "Minimum_Wavelength"},
	{WMission_Completion,
	 "Mission_Completion"},
	{WMission_Description,
	 "Mission_Description"},
	{WMission_History,
	 "Mission_History"},
	{WMission_Information,
	 "Mission_Information"},
	{WMission_Name,
	 "Mission_Name"},
	{WMission_Significant_Event,
	 "Mission_Significant_Event"},
	{WMission_Start_Date,
	 "Mission_Start_Date"},
	{WMultiple_Image_Alignment,
	 "Multiple_Image_Alignment"},
	{WNadir,
	 "Nadir"},
	{WNadir_Latitude,
	 "Nadir_Latitude"},
	{WNadir_Longitude,
	 "Nadir_Longitude"},
	{WName_of_Dimension,
	 "Name_of_Dimension"},
	{WNominal_Geostationary_Position,
	 "Nominal_Geostationary_Position"},
	{WNominal_Spatial_Resolution,
	 "Nominal_Spatial_Resolution"},
	{WNon_Polynomial_Scaling,
	 "Non_Polynomial_Scaling"},
	{WNumber_of_Angle_Distortion_Values,
	 "Number_of_Angle_Distortion_Values"},
	{WNumber_of_Bands,
	 "Number_of_Bands"},
	{WNumber_of_Cross_Track_Samples,
	 "Number_of_Cross_Track_Samples"},
	{WNumber_of_Data_Dimensions,
	 "Number_of_Data_Dimensions"},
	{WNumber_of_Denominator_Terms,
	 "Number_of_Denominator_Terms"},
	{WNumber_of_Distance_Distortion_Values,
	 "Number_of_Distance_Distortion_Values"},
	{WNumber_of_Elevation_Samples,
	 "Number_of_Elevation_Samples"},
	{WNumber_of_Numerator_Terms,
	 "Number_of_Numerator_Terms"},
	{WNumber_of_Points,
	 "Number_of_Points"},
	{WNumber_of_Profile_Samples,
	 "Number_of_Profile_Samples"},
	{WNumber_of_Resolution_Values,
	 "Number_of_Resolution_Values"},
	{WNumber_of_Thematic_Layers,
	 "Number_of_Thematic_Layers"},
	{WNumber_of_Wavelength_Bands,
	 "Number_of_Wavelength_Bands"},
	{WOperational_Mode,
	 "Operational_Mode"},
	{WOrbit_Angle_Units,
	 "Orbit_Angle_Units"},
	{WOrbit_Period,
	 "Orbit_Period"},
	{WOrbit_Period_Units,
	 "Orbit_Period_Units"},
	{WOrbit_Period_Value,
	 "Orbit_Period_Value"},
	{WOther_Collector_Description,
	 "Other_Collector_Description"},
	{WOther_Georeferencing_Description,
	 "Other_Georeferencing_Description"},
	{WPath,
	 "Path"},
	{WPeak_Wavelength,
	 "Peak_Wavelength"},
	{WPercentage_of_Overlap,
	 "Percentage_of_Overlap"},
	{WPerigee_Passage_Time,
	 "Perigee_Passage_Time"},
	{WPhotographic_Resolving_Power,
	 "Photographic_Resolving_Power"},
	{WPitch,
	 "Pitch"},
	{WPitch_Accuracy,
	 "Pitch_Accuracy"},
	{WPixel_Cross_Track_Size,
	 "Pixel_Cross_Track_Size"},
	{WPixel_Description,
	 "Pixel_Description"},
	{WPixel_Elevation_Size,
	 "Pixel_Elevation_Size"},
	{WPixel_Height_Above_Ellipsoid,
	 "Pixel_Height_Above_Ellipsoid"},
	{WPixel_Point_Spread_Function,
	 "Pixel_Point_Spread_Function"},
	{WPixel_Profile_Size,
	 "Pixel_Profile_Size"},
	{WPixel_Resolution,
	 "Pixel_Resolution"},
	{WPlatform_and_Instrument_Identification,
	 "Platform_and_Instrument_Identification"},
	{WPlatform_and_Mission_Information,
	 "Platform_and_Mission_Information"},
	{WPlatform_Description,
	 "Platform_Description"},
	{WPlatform_Full_Name,
	 "Platform_Full_Name"},
	{WPlatform_Information,
	 "Platform_Information"},
	{WPlatform_Nominal_Altitude,
	 "Platform_Nominal_Altitude"},
	{WPlatform_Nominal_Altitude_Units,
	 "Platform_Nominal_Altitude_Units"},
	{WPlatform_Nominal_Altitude_Value,
	 "Platform_Nominal_Altitude_Value"},
	{WPlatform_Nominal_Longitude,
	 "Platform_Nominal_Longitude"},
	{WPlatform_Orbit,
	 "Platform_Orbit"},
	{WPlatform_Serial_Identifier,
	 "Platform_Serial_Identifier"},
	{WPlatform_Short_Name,
	 "Platform_Short_Name"},
	{WPlatform_Sponsor,
	 "Platform_Sponsor"},
	{WPlatform_Start_Date,
	 "Platform_Start_Date"},
	{WPoint_Position_In_Pixel,
	 "Point_Position_In_Pixel"},
	{WPolarization_Characteristics,
	 "Polarization_Characteristics"},
	{WPolynomial_Coefficient,
	 "Polynomial_Coefficient"},
	{WPolynomial_Denominator,
	 "Polynomial_Denominator"},
	{WPolynomial_Function,
	 "Polynomial_Function"},
	{WPolynomial_Numerator,
	 "Polynomial_Numerator"},
	{WPositional_Information,
	 "Positional_Information"},
	{WPrincipal_Point_of_Autocollimation,
	 "Principal_Point_of_Autocollimation"},
	{WPrincipal_Point_of_Symmetry,
	 "Principal_Point_of_Symmetry"},
	{WProcessing_Change_History,
	 "Processing_Change_History"},
	{WProcessing_Documentation,
	 "Processing_Documentation"},
	{WProcessing_Environment,
	 "Processing_Environment"},
	{WProcessing_Identifiers,
	 "Processing_Identifiers"},
	{WProcessing_Information,
	 "Processing_Information"},
	{WProcessing_Input_Dataset,
	 "Processing_Input_Dataset"},
	{WProcessing_Level,
	 "Processing_Level"},
	{WProcessing_Level_Authority,
	 "Processing_Level_Authority"},
	{WProcessing_Level_Identifier,
	 "Processing_Level_Identifier"},
	{WProcessing_Procedure,
	 "Processing_Procedure"},
	{WProcessing_Procedure_Description,
	 "Processing_Procedure_Description"},
	{WProcessing_Run_History,
	 "Processing_Run_History"},
	{WProcessing_Software,
	 "Processing_Software"},
	{WProcessing_Software_Description,
	 "Processing_Software_Description"},
	{WProcessing_Software_Reference,
	 "Processing_Software_Reference"},
	{WProfile_Description,
	 "Profile_Description"},
	{WProfile_Extent,
	 "Profile_Extent"},
	{WProfile_Fixed,
	 "Profile_Fixed"},
	{WProfile_Properties,
	 "Profile_Properties"},
	{WProfile_Sounding,
	 "Profile_Sounding"},
	{WProfile_Start,
	 "Profile_Start"},
	{WProfile_Step,
	 "Profile_Step"},
	{WProfiling_Direction,
	 "Profiling_Direction"},
	{WProjection_Center_X_Position,
	 "Projection_Center_X_Position"},
	{WProjection_Center_Y_Position,
	 "Projection_Center_Y_Position"},
	{WProjection_Center_Z_Position,
	 "Projection_Center_Z_Position"},
	{WQuality_of_Autocollimation_Principal_Point_x_Value,
	 "Quality_of_Autocollimation_Principal_Point_x_Value"},
	{WQuality_of_Autocollimation_Principal_Point_y_Value,
	 "Quality_of_Autocollimation_Principal_Point_y_Value"},
	{WQuality_of_Focal_Length,
	 "Quality_of_Focal_Length"},
	{WQuality_of_Principal_Point_of_Autocollimation,
	 "Quality_of_Principal_Point_of_Autocollimation"},
	{WQuality_of_Principal_Point_of_Symmetry,
	 "Quality_of_Principal_Point_of_Symmetry"},
	{WQuality_of_Symmetry_Principal_Point_x_Value,
	 "Quality_of_Symmetry_Principal_Point_x_Value"},
	{WQuality_of_Symmetry_Principal_Point_y_Value,
	 "Quality_of_Symmetry_Principal_Point_y_Value"},
	{WRadial_Asymmetrical_Coefficient_B1,
	 "Radial_Asymmetrical_Coefficient_B1"},
	{WRadial_Asymmetrical_Coefficient_B2,
	 "Radial_Asymmetrical_Coefficient_B2"},
	{WRadial_Symmetrical_Angle_Interval,
	 "Radial_Symmetrical_Angle_Interval"},
	{WRadial_Symmetrical_Distance_Interval,
	 "Radial_Symmetrical_Distance_Interval"},
	{WRadial_Symmetrical_Distortion_Polynomial,
	 "Radial_Symmetrical_Distortion_Polynomial"},
	{WReceiver_Polarization,
	 "Receiver_Polarization"},
	{WReferenced_Coordinate_System,
	 "Referenced_Coordinate_System"},
	{WReferencing_Polynomial,
	 "Referencing_Polynomial"},
	{WRelative_Aperture,
	 "Relative_Aperture"},
	{WRelative_Azimuth,
	 "Relative_Azimuth"},
	{WReseau,
	 "Reseau"},
	{WResolution_Value_Set,
	 "Resolution_Value_Set"},
	{WResolving_Angle,
	 "Resolving_Angle"},
	{WResolving_Value_Radial,
	 "Resolving_Value_Radial"},
	{WResolving_Value_Tangential,
	 "Resolving_Value_Tangential"},
	{WRight_Ascension_of_Ascending_Node,
	 "Right_Ascension_of_Ascending_Node"},
	{WRoll,
	 "Roll"},
	{WRoll_Accuracy,
	 "Roll_Accuracy"},
	{WRotation_Description,
	 "Rotation_Description"},
	{WRotation_Sequence,
	 "Rotation_Sequence"},
	{WRow,
	 "Row"},
	{WRow_Delta_X,
	 "Row_Delta_X"},
	{WRow_Delta_Y,
	 "Row_Delta_Y"},
	{WSample_Angle_Units,
	 "Sample_Angle_Units"},
	{WSample_Description_Units,
	 "Sample_Description_Units"},
	{WSample_Profile_Units,
	 "Sample_Profile_Units"},
	{WSample_Properties,
	 "Sample_Properties"},
	{WSatellite_Local_Azimuth_Angle,
	 "Satellite/Local_Azimuth_Angle"},
	{WSatellite_Local_Zenith_Angle,
	 "Satellite/Local_Zenith_Angle"},
	{WScan,
	 "Scan"},
	{WScan_Angle_Units,
	 "Scan_Angle_Units"},
	{WScan_Cross_Track_Properties,
	 "Scan_Cross_Track_Properties"},
	{WScan_Distance_Units,
	 "Scan_Distance_Units"},
	{WScan_Duration,
	 "Scan_Duration"},
	{WScan_Elevation_Properties,
	 "Scan_Elevation_Properties"},
	{WScan_Geometric_Properties,
	 "Scan_Geometric_Properties"},
	{WScan_Radiometric_Properties,
	 "Scan_Radiometric_Properties"},
	{WScan_Repeat_Time,
	 "Scan_Repeat_Time"},
	{WScan_Spectral_Properties,
	 "Scan_Spectral_Properties"},
	{WScan_Start_Time,
	 "Scan_Start_Time"},
	{WScan_Step_Time,
	 "Scan_Step_Time"},
	{WScan_Time_Units,
	 "Scan_Time_Units"},
	{WScan_Timing,
	 "Scan_Timing"},
	{WScience_Paper,
	 "Science_Paper"},
	{WSemimajor_Axis,
	 "Semimajor_Axis"},
	{WSender_Polarization,
	 "Sender_Polarization"},
	{WSensor_Element_Location,
	 "Sensor_Element_Location"},
	{WSensor_Grid,
	 "Sensor_Grid"},
	{WSensor_Orientation,
	 "Sensor_Orientation"},
	{WSensor_System,
	 "Sensor_System"},
	{WSolar_Azimuth_Angle,
	 "Solar_Azimuth_Angle"},
	{WSolar_Zenith_Angle,
	 "Solar_Zenith_Angle"},
	{WSource_of_Element_Locations,
	 "Source_of_Element_Locations"},
	{WSpatial_Resolution_Units,
	 "Spatial_Resolution_Units"},
	{WSpatial_Resolution_Value,
	 "Spatial_Resolution_Value"},
	{WSpectral_Information,
	 "Spectral_Information"},
	{WSpectral_Limit,
	 "Spectral_Limit"},
	{WStabilized_Mount,
	 "Stabilized_Mount"},
	{WStorage_Order,
	 "Storage_Order"},
	{WSwath_Track_Information,
	 "Swath_Track_Information"},
	{WThematic_Layer_Identification,
	 "Thematic_Layer_Identification"},
	{WTranslation_Description,
	 "Translation_Description"},
	{WType_of_Overlap,
	 "Type_of_Overlap"},
	{WUnreferenced_Coordinate_System,
	 "Unreferenced_Coordinate_System"},
	{WUsers_Guide,
	 "User's_Guide"},
	{WWavelength_Band_Properties,
	 "Wavelength_Band_Properties"},
	{WWavelength_Region,
	 "Wavelength_Region"},
	{WWavelength_Units,
	 "Wavelength_Units"},
	{WWorldwide_Reference_System,
	 "Worldwide_Reference_System"},
	{WX_Axis_Definition,
	 "X_Axis_Definition"},
	{WX_Position_Accuracy,
	 "X_Position_Accuracy"},
	{WY_Axis_Definition,
	 "Y_Axis_Definition"},
	{WY_Position_Accuracy,
	 "Y_Position_Accuracy"},
	{WYaw,
	 "Yaw"},
	{WYaw_Accuracy,
	 "Yaw_Accuracy"},
	{WZ_Axis_Definition,
	 "Z_Axis_Definition"},
	{WZ_Position_Accuracy,
	 "Z_Position_Accuracy"},

#ifdef USE_ESRI_EXTENSIONS

	/* ESRI extensions */

	{WAltitude_Units,
	 "Altitude_Units"},
	{WAncillary_Role,
	 "Ancillary_Role"},
	{WAncillary_Role_Attribute,
	 "Ancillary_Role_Attribute"},
	{WAttribute_Alias,
	 "Attribute_Alias"},
	{WAttribute_Defined_Domain,
	 "Attribute_Defined_Domain"},
	{WAttribute_Indexed,
	 "Attribute_Indexed"},
	{WAttribute_Number_of_Decimals,
	 "Attribute_Number_of_Decimals"},
	{WAttribute_Output_Width,
	 "Attribute_Output_Width"},
	{WAttribute_Precision,
	 "Attribute_Precision"},
	{WAttribute_Scale,
	 "Attribute_Scale"},
	{WAttribute_Type,
	 "Attribute_Type"},
	{WAttribute_Width,
	 "Attribute_Width"},
	{WAttributed_Relationship,
	 "Attributed_Relationship"},
	{WAvailable_Junction_Feature_Class,
	 "Available_Junction_Feature_Class"},
	{WAvailable_Junction_Subtype,
	 "Available_Junction_Subtype"},
	{WAvailable_Junctions,
	 "Available_Junctions"},
	{WBackground_Nodata_Value,
	 "Background_Nodata_Value"},
	{WBehrmann,
	 "Behrmann"},
	{WBinary_Enclosures,
	 "Binary_Enclosures"},
	{WBits_Per_Pixel,
	 "Bits_Per_Pixel"},
	{WBonne,
	 "Bonne"},
	{WBottom_Bounding_Coordinate,
	 "Bottom_Bounding_Coordinate"},
	{WBrowse_Graphic_Embedded,
	 "Browse_Graphic_Embedded"},
	{WCassini,
	 "Cassini"},
	{WCell_Size_X_Direction,
	 "Cell_Size_X_Direction"},
	{WCell_Size_X_Units,
	 "Cell_Size_X_Units"},
	{WCell_Size_Y_Direction,
	 "Cell_Size_Y_Direction"},
	{WCell_Size_Y_Units,
	 "Cell_Size_Y_Units"},
	{WComposite_Relationship,
	 "Composite_Relationship"},
	{WCompression_Type,
	 "Compression_Type"},
	{WConnectivity_Rule,
	 "Connectivity_Rule"},
	{WCoordinate_System_Name,
	 "Coordinate_System_Name"},
	{WCreation_Date,
	 "Creation_Date"},
	{WCreation_Time,
	 "Creation_Time"},
	{WData,
	 "Data"},
	{WData_Frame_Count,
	 "Data_Frame_Count"},
	{WData_Frame_Name,
	 "Data_Frame_Name"},
	{WData_Frames,
	 "Data_Frames"},
	{WDatabase_Name,
	 "Database_Name"},
	{WDataset_Size,
	 "Dataset_Size"},
	{WDefault_Junction_Feature_Class,
	 "Default_Junction_Feature_Class"},
	{WDefault_Junction_Subtype,
	 "Default_Junction_Subtype"},
	{WDescription_of_Association,
	 "Description_of_Association"},
	{WDescription_of_Enclosure,
	 "Description_of_Enclosure"},
	{WDescription_of_Relationship,
	 "Description_of_Relationship"},
	{WDestination_Foreign_Key,
	 "Destination_Foreign_Key"},
	{WDestination_Name,
	 "Destination_Name"},
	{WDestination_Primary_Key,
	 "Destination_Primary_Key"},
	{WDomain_Attribute_Type,
	 "Domain_Attribute_Type"},
	{WDomain_Description,
	 "Domain_Description"},
	{WDomain_Name,
	 "Domain_Name"},
	{WDomain_Owner,
	 "Domain_Owner"},
	{WDomain_Type,
	 "Domain_Type"},
	{WEckert_I,
	 "Eckert_I"},
	{WEckert_II,
	 "Eckert_II"},
	{WEckert_III,
	 "Eckert_III"},
	{WEckert_IV,
	 "Eckert_IV"},
	{WEckert_V,
	 "Eckert_V"},
	{WEckert_VI,
	 "Eckert_VI"},
	{WEdge_Feature_Class,
	 "Edge_Feature_Class"},
	{WEdge_Maximum_Cardinality,
	 "Edge_Maximum_Cardinality"},
	{WEdge_Minimum_Cardinality,
	 "Edge_Minimum_Cardinality"},
	{WEdge_Subtype,
	 "Edge_Subtype"},
	{WEnabled_Attribute,
	 "Enabled_Attribute"},
	{WEnclosure,
	 "Enclosure"},
	{WEntity_Type_Count,
	 "Entity_Type_Count"},
	{WEntity_Type_Type,
	 "Entity_Type_Type"},
	{WESRI_Feature_Count,
	 "ESRI_Feature_Count"},
	{WESRI_Feature_Geometry,
	 "ESRI_Feature_Geometry"},
	{WESRI_Feature_Type,
	 "ESRI_Feature_Type"},
	{WESRI_Information,
	 "ESRI_Information"},
	{WESRI_Terms_Description,
	 "ESRI_Terms_Description"},
	{WESRI_Topology,
	 "ESRI_Topology"},
	{WFeature_Description,
	 "Feature_Description"},
	{WFile_or_Table_Name,
	 "File_or_Table_Name"},
	{WFrom_Edge_Feature_Class,
	 "From_Edge_Feature_Class"},
	{WFrom_Edge_Subtype,
	 "From_Edge_Subtype"},
	{WGall_Stereographic,
	 "Gall_Stereographic"},
	{WGeocoding_Information,
	 "Geocoding_Information"},
	{WGeographic_Coordinate_System_Name,
	 "Geographic_Coordinate_System_Name"},
	{WGeometric_Network_Information,
	 "Geometric_Network_Information"},
	{WImage_Colormap,
	 "Image_Colormap"},
	{WImage_Format,
	 "Image_Format"},
	{WImage_Type,
	 "Image_Type"},
	{WInstance_Name,
	 "Instance_Name"},
	{WJunction_Feature_Class,
	 "Junction_Feature_Class"},
	{WJunction_Maximum_Cardinality,
	 "Junction_Maximum_Cardinality"},
	{WJunction_Minimum_Cardinality,
	 "Junction_Minimum_Cardinality"},
	{WJunction_Subtype,
	 "Junction_Subtype"},
	{WLanguage_of_Dataset,
	 "Language_of_Dataset"},
	{WLanguage_of_Metadata,
	 "Language_of_Metadata"},
	{WLeft_Bounding_Coordinate,
	 "Left_Bounding_Coordinate"},
	{WLinear_Referencing,
	 "Linear_Referencing"},
	{WLocal_Bounding_Coordinates,
	 "Local_Bounding_Coordinates"},
	{WLoximuthal,
	 "Loximuthal"},
	{WMaximum_Altitude,
	 "Maximum_Altitude"},
	{WMerge_Rule,
	 "Merge_Rule"},
	{WMetadata_ID,
	 "Metadata_ID"},
	{WMinimum_Altitude,
	 "Minimum_Altitude"},
	{WModification_Date,
	 "Modification_Date"},
	{WModification_Time,
	 "Modification_Time"},
	{WMollweide,
	 "Mollweide"},
	{WNative_Dataset_Format,
	 "Native_Dataset_Format"},
	{WNetwork_Element,
	 "Network_Element"},
	{WNetwork_Role,
	 "Network_Role"},
	{WNetwork_Type,
	 "Network_Type"},
	{WNotification_Direction,
	 "Notification_Direction"},
	{WNumber_of_Raster_Bands,
	 "Number_of_Raster_Bands"},
	{WOrigin_Foreign_Key,
	 "Origin_Foreign_Key"},
	{WOrigin_Name,
	 "Origin_Name"},
	{WOrigin_Primary_Key,
	 "Origin_Primary_Key"},
	{WProcess_Software_and_Version,
	 "Process_Software_and_Version"},
	{WProjected_Coordinate_System_Name,
	 "Projected_Coordinate_System_Name"},
	{WPyramid_Layers,
	 "Pyramid_Layers"},
	{WQuartic_Authalic,
	 "Quartic_Authalic"},
	{WRange_Domain_Mean,
	 "Range_Domain_Mean"},
	{WRange_Domain_Standard_Deviation,
	 "Range_Domain_Standard_Deviation"},
	{WRaster_Display_Type,
	 "Raster_Display_Type"},
	{WRaster_Origin,
	 "Raster_Origin"},
	{WRelationship_Backward_Label,
	 "Relationship_Backward_Label"},
	{WRelationship_Cardinality,
	 "Relationship_Cardinality"},
	{WRelationship_Forward_Label,
	 "Relationship_Forward_Label"},
	{WRelationship_Information,
	 "Relationship_Information"},
	{WRight_Bounding_Coordinate,
	 "Right_Bounding_Coordinate"},
	{WRule_Category,
	 "Rule_Category"},
	{WRule_Help,
	 "Rule_Help"},
	{WRule_Type,
	 "Rule_Type"},
	{WSDE_Connection_Information,
	 "SDE_Connection_Information"},
	{WServer_Name,
	 "Server_Name"},
	{WSpatial_Index,
	 "Spatial_Index"},
	{WSplit_Rule,
	 "Split_Rule"},
	{WSubtype_Attribute,
	 "Subtype_Attribute"},
	{WSubtype_Attribute_Name,
	 "Subtype_Attribute_Name"},
	{WSubtype_Code,
	 "Subtype_Code"},
	{WSubtype_Default_Value,
	 "Subtype_Default_Value"},
	{WSubtype_Information,
	 "Subtype_Information"},
	{WSubtype_Name,
	 "Subtype_Name"},
	{WSynchronization_Date,
	 "Synchronization_Date"},
	{WSynchronization_Time,
	 "Synchronization_Time"},
	{WSynchronize_Automatically,
	 "Synchronize_Automatically"},
	{WSynchronize_Once,
	 "Synchronize_Once"},
	{WThumbnail,
	 "Thumbnail"},
	{WTo_Edge_Feature_Class,
	 "To_Edge_Feature_Class"},
	{WTo_Edge_Subtype,
	 "To_Edge_Subtype"},
	{WTop_Bounding_Coordinate,
	 "Top_Bounding_Coordinate"},
	{WUser_Name,
	 "User_Name"},
	{WVersion_Name,
	 "Version_Name"},
	{WWinkel_I,
	 "Winkel_I"},
	{WWinkel_II,
	 "Winkel_II"},
	{WGeographic_Box,
	 "Geographic_Box"},

#endif

	{Wunknown, NULL}
	};

/*----------------------------------------------------------------------*/

#ifndef PRINT_NAMES

static struct keyword_list **keyword_list = NULL;
static int keyword_list_length = 0;

static int compare (const void *e1, const void *e2) {
	struct keyword_list *p = *(struct keyword_list **) e1;
	struct keyword_list *q = *(struct keyword_list **) e2;
	return (stricmp (p->text,q->text));
	}

void use_element_names (char *language, char *profile) {
	int i,j,m,n;
	struct keyword_list *standard_list = NULL;

	/* Choose standard elements based on language */

	if (! language) language = "en";
	if (!*language) language = "en";

	standard_list = en_std;

	if (stricmp (language,"en") == 0) standard_list = en_std;
	if (stricmp (language,"es") == 0) standard_list = es_std;
	if (stricmp (language,"ca") == 0) standard_list = ca_std;
	if (stricmp (language,"id") == 0) standard_list = id_std;
	if (stricmp (language,"fr") == 0) standard_list = fr_std;
	if (stricmp (language,"de") == 0) standard_list = de_std;
	if (stricmp (language,"pt") == 0) standard_list = pt_std;

	for (n=0; standard_list[n].text; n++);

	/*------------------------------------------------------------------*\
	 | Allocate an array of pointers to the list elements.  Fill these	|
	 | with the addresses from the appropriate lists based on language	|
	 | and profile.														|
	\*------------------------------------------------------------------*/

	if (keyword_list = (struct keyword_list **) malloc ((1 + n) * sizeof (struct keyword_list *))) {
		for (i=0; i < n; i++)
			keyword_list[i] = &standard_list[i];
		keyword_list[i] = &standard_list[n];

		/* Sort the list of elements*/

		keyword_list_length = n;
		qsort (keyword_list,keyword_list_length,sizeof(struct keyword_list *),compare);
		}
	else {
		fprintf (stderr,"Error: could not allocate %d pointers for element name lookup\n",1+n);
		exit (1);
		}
	}

/*----------------------------------------------------------------------*\
 | This routine is used by cns to convert the keyword list, which is an	|
 | array of pointers to struct keyword, into an array of struct keyword.|
\*----------------------------------------------------------------------*/

struct keyword_list *get_keyword_list (void) {
	int i;
	struct keyword_list *the_list;

	if (keyword_list == NULL) use_element_names ("en",NULL);

	if (the_list = (struct keyword_list *) malloc ((1 + keyword_list_length) * sizeof (struct keyword_list))) {
		for (i=0; i < keyword_list_length; i++)
			the_list[i] = *keyword_list[i];
		the_list[i].key = Wunknown;
		the_list[i].text = NULL;
		}
	return (the_list);
	}

/*----------------------------------------------------------------------*/

enum fgdc_keyword key_of (const char *text) {
	int i,n;
	struct keyword_list k,*key,*target,**result;
	char string[128];

	if (keyword_list == NULL) use_element_names ("en",NULL);

	if (!text) return (Wnull);
	if (*text) {
		n = strcspn (text," \t\n\v\f\r:=");
		if (n > 127) n = 127;
		memcpy (string,text,n);
		string[n] = 0;
		k.text = string;
		target = &k;
		result = (struct keyword_list **) bsearch (&target,keyword_list,keyword_list_length,sizeof(struct keyword_list *),compare);
		if (result) {
			key = *result;
			return (key->key);
			}
		else {
			if ((i = extension_of (string)) != Unrecognized_Extension) {
				return (i);
				}
			else return (Wunknown);
			}
		}
	else return (Wblank);
	}

char *text_of (enum fgdc_keyword key) {
	int i;

	if (keyword_list == NULL) use_element_names ("en",NULL);
	if (key == Wnull) return ("(null)");
	if (key == Wblank) return ("(blank)");
	for (i=0; i < keyword_list_length; i++)
		if (keyword_list[i]->key == key) return (keyword_list[i]->text);
	if (is_extension (key)) return (text_of_extension (key));
	return ("(unknown)");
	}

/* Identify CSDGM elements by their numbers */

int is_standard (enum fgdc_keyword key) {
	if (((int)key) < 500) return (1);
	else return (0);
	}

#endif	/* PRINT_NAMES not defined */

/*----------------------------------------------------------------------*\
 | Print the element names in each of the languages for which we have	|
 | some translations.  These are put in tab-delimited columns so they	|
 | can be revised using a spreadsheet.  Note that element names must	|
 | not contain spaces, tabs, or double-quote marks.  All spaces should	|
 | be represented by underscores in the names.							|
\*----------------------------------------------------------------------*/

#ifdef PRINT_NAMES


/*----------------------------------------------------------------------*\
 | Code to translate Latin-1 characters into HTML entities				|
\*----------------------------------------------------------------------*/

extern char *iso8859_1[];

static char *buffer = NULL;
static int buffer_length = 0;

static void clear_buffer (void) {
	if (buffer) *buffer = 0;
	}

static char *add_to_buffer (char *string, int j) {
	int k,n;
	char *t;

	if (!string || j == 0) {
		t = buffer + strlen (buffer);
		return (t);
		}

	if (!buffer)
		if (buffer = (char *) malloc (4096)) {
			memset (buffer,0,4096);
			buffer_length = 4096;
			}
		else {
			fprintf (stderr,"Error: failed to allocate 4096 bytes for html translation buffer\n");
			exit (1);
			}

	k = strlen (buffer);
	n = ((j < 4096) ? 4096 : j+1);

	if (k + j + 1 >= buffer_length)
		if (buffer = (char *) realloc (buffer,buffer_length+n)) {
			buffer_length += n;
			}
		else {
			fprintf (stderr,"Error: failed to reallocate %d bytes for html translation buffer\n",buffer_length + n);
			exit (1);
			}

	t = buffer + k;
	memcpy (t,string,j);
	t += j;
	*t = 0;
	return (t);
	}

static char *htmlentities (char *string) {
	char *s,*t;
	char entity [16];

	clear_buffer();
	s = string;
	while (*s) {
		if (t = iso8859_1[*(unsigned char *)s]) {
			sprintf (entity,"&%s;",t);
			add_to_buffer (entity,strlen (entity));
			s++;
			}
		else
			add_to_buffer (s++,1);
		}
	return (buffer);
	}

/*----------------------------------------------------------------------*\
\*----------------------------------------------------------------------*/

static char html_doctype[] = "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\"\n   \"http://www.w3.org/TR/html4/loose.dtd\">";

static char *base_href = "http://www.fgdc.gov/metadata/csdgm/";
extern char *link_of (enum fgdc_keyword key);

int main (int argc, char *argv[]) {
	int i;
	int en_count;
	int es_count;
	int id_count;
	int fr_count;
	int ca_count;
	int de_count;
	int pt_count;
	char *s;
    int do_text = 0;
    int do_html = 0;
    int needs_work = 0;

    if (argc > 1)
    	for (i=0; i < argc; i++)
    		if (memcmp (argv[i],"-html",5) == 0) do_html = 1;

	if (do_html == 0) do_text = 1;

	/*------------------------------------------------------------------*\
	 | Check list consistency.  If you want the list ordering to vary	|
	 | from language to language, you have to change the code that		|
	 | the lists out (below this for loop).  But that's probably not a	|
	 | good idea.														|
	\*------------------------------------------------------------------*/

	en_count = 0; for (i=0; en_std[i].text; i++) en_count++;
	es_count = 0; for (i=0; es_std[i].text; i++) es_count++;
	id_count = 0; for (i=0; id_std[i].text; i++) id_count++;
	fr_count = 0; for (i=0; fr_std[i].text; i++) fr_count++;
	ca_count = 0; for (i=0; ca_std[i].text; i++) ca_count++;
	de_count = 0; for (i=0; de_std[i].text; i++) de_count++;
	pt_count = 0; for (i=0; pt_std[i].text; i++) pt_count++;

	if (es_count != en_count)
		fprintf (stderr,"Warning: es_std[] contains %d elements; en_std[] has %d\n",es_count,en_count);
	if (id_count != en_count)
		fprintf (stderr,"Warning: id_std[] contains %d elements; en_std[] has %d\n",id_count,en_count);
	if (fr_count != en_count)
		fprintf (stderr,"Warning: fr_std[] contains %d elements; en_std[] has %d\n",fr_count,en_count);
	if (ca_count != en_count)
		fprintf (stderr,"Warning: ca_std[] contains %d elements; en_std[] has %d\n",ca_count,en_count);
	if (de_count != en_count)
		fprintf (stderr,"Warning: de_std[] contains %d elements; en_std[] has %d\n",de_count,en_count);
	if (pt_count != en_count)
		fprintf (stderr,"Warning: pt_std[] contains %d elements; en_std[] has %d\n",pt_count,en_count);


	for (i=0; en_std[i].text; i++)
		if (en_std[i].key != es_std[i].key) {
			fprintf (stderr,"Error: Inconsistency in language-specific list ordering.\n");
			fprintf (stderr,"       es_std[%d] differs from en_list at %s\n",i,en_std[i].text);
			fprintf (stderr,"       This program cannot continue.\n");
			exit (1);
			}

	for (i=0; en_std[i].text; i++)
		if (en_std[i].key != fr_std[i].key) {
			fprintf (stderr,"Error: Inconsistency in language-specific list ordering.\n");
			fprintf (stderr,"       fr_std[%d] differs from en_list at %s\n",i,en_std[i].text);
			fprintf (stderr,"       This program cannot continue.\n");
			exit (1);
			}

	for (i=0; en_std[i].text; i++)
		if (en_std[i].key != id_std[i].key) {
			fprintf (stderr,"Error: Inconsistency in language-specific list ordering.\n");
			fprintf (stderr,"       id_std[%d] differs from en_list at %s\n",i,en_std[i].text);
			fprintf (stderr,"       This program cannot continue.\n");
			exit (1);
			}

	for (i=0; en_std[i].text; i++)
		if (en_std[i].key != ca_std[i].key) {
			fprintf (stderr,"Error: Inconsistency in language-specific list ordering.\n");
			fprintf (stderr,"       ca_std[%d] differs from en_list at %s\n",i,en_std[i].text);
			fprintf (stderr,"       This program cannot continue.\n");
			exit (1);
			}

	for (i=0; en_std[i].text; i++)
		if (en_std[i].key != de_std[i].key) {
			fprintf (stderr,"Error: Inconsistency in language-specific list ordering.\n");
			fprintf (stderr,"       de_std[%d] differs from en_list at %s\n",i,en_std[i].text);
			fprintf (stderr,"       This program cannot continue.\n");
			exit (1);
			}

	for (i=0; en_std[i].text; i++)
		if (en_std[i].key != pt_std[i].key) {
			fprintf (stderr,"Error: Inconsistency in language-specific list ordering.\n");
			fprintf (stderr,"       pt_std[%d] differs from en_list at %s\n",i,en_std[i].text);
			fprintf (stderr,"       This program cannot continue.\n");
			exit (1);
			}

	if (do_text) {

		/*------------------------------------------------------------------*\
		 | Write out each element, with languages in separate columns, tab	|
		 | delimited.  For each language other than English, make another	|
		 | column after the element name.  If the non-English translation	|
		 | is an exact match for the English element name, put a 1 into		|
		 | this extra column to indicate that a translation is needed.		|
		\*------------------------------------------------------------------*/

		printf ("English\tEspaol\tes\tIndonesian\tid\tFranais\tfr\tCataln\tca\tDeutsch\tde\tPortuguese\tpt\n");
		for (i=0; en_std[i].text; i++) {
			printf ("%s",en_std[i].text);

			printf ("\t%s\t",es_std[i].text);
			if (strcmp (en_std[i].text,es_std[i].text) == 0) printf ("1");

			printf ("\t%s\t",id_std[i].text);
			if (strcmp (en_std[i].text,id_std[i].text) == 0) printf ("1");

			printf ("\t%s\t",fr_std[i].text);
			if (strcmp (en_std[i].text,fr_std[i].text) == 0) printf ("1");

			printf ("\t%s\t",ca_std[i].text);
			if (strcmp (en_std[i].text,ca_std[i].text) == 0) printf ("1");

			printf ("\t%s\t",de_std[i].text);
			if (strcmp (en_std[i].text,de_std[i].text) == 0) printf ("1");

			printf ("\t%s\t",pt_std[i].text);
			if (strcmp (en_std[i].text,pt_std[i].text) == 0) printf ("1");

			printf ("\n");
			}

		}

	if (do_html) {
		time_t now;

		printf ("%s\n",html_doctype);
		printf ("<html>\n");
		printf ("<head>\n");
		printf ("  <title>Status of FGDC metadata element name translation</title>\n");
		printf ("</head>\n");
		printf ("<body bgcolor=\"#FFFFFF\">\n");

		printf ("<h1>Status of FGDC metadata element name translation</h1>\n");
		printf ("<p>This table presents the current languages supported by\n");
		printf ("mp, cns, xtme, tkme, err2html, and mq.  Note that only\n");
		printf ("element names are translated into these languages, not\n");
		printf ("error messages or additional texts.  Obviously much work\n");
		printf ("must be done to make reading metadata more comfortable\n");
		printf ("for our friends in other countries.  However this is a\n");
		printf ("task for which I am not especially well equipped.  Hence\n");
		printf ("I invite those who are capable of providing translations\n");
		printf ("to do so.  Comments are welcome on any part of this table.\n");
		printf ("If you are able to assist with translation, please send\n");
		printf ("email to <a href=\"mailto:pschweitzer@usgs.gov\"><tt>pschweitzer@usgs.gov</tt></a>\n");
		printf ("</p>\n");

		printf ("<p>In the table that follows, each column describes one\n");
		printf ("language, beginning with its name and 2-letter code.\n");
		printf ("Each row begins (on the left) with the element name as\n");
		printf ("presented in FGDC-STD-001-1998, with the corresponding\n");
		printf ("translations in subsequent cells of the same row.  Where\n");
		printf ("an element name is enclosed in parentheses, its text\n");
		printf ("matches the english exactly (which is to say that I have\n");
		printf ("used the english either because I have no translation or\n");
		printf ("because the words are really the same.</p>\n");

		/*--------------------------------------------------------------*\
		 | Write out each element, with languages in separate columns,	|
		 | as a table.  For each language, if the translation matches	|
		 | the English element name exactly, shade the cell and write	|
		 | the element name in parentheses to indicate that a revised	|
		 | translation may be needed.									|
		\*--------------------------------------------------------------*/

		printf ("<table border=\"1\" cellpadding=\"2\" cellspacing=\"0\" summary=\"Each row shows the translations of a single standard metadata element name.\">\n");
		printf ("<tr>");
		printf ("<th scope=\"col\">%s</th>",htmlentities("English"));
		printf ("<th scope=\"col\">%s</th>",htmlentities("Espaol"));
		printf ("<th scope=\"col\">%s</th>",htmlentities("Indonesian"));
		printf ("<th scope=\"col\">%s</th>",htmlentities("Franais"));
		printf ("<th scope=\"col\">%s</th>",htmlentities("Cataln"));
		printf ("<th scope=\"col\">%s</th>",htmlentities("Deutsch"));
		printf ("<th scope=\"col\">%s</th>",htmlentities("Portuguese"));
		printf ("</tr>\n");
		printf ("<tr>");
		printf ("<th>en</th>");
		printf ("<th>es</th>");
		printf ("<th>id</th>");
		printf ("<th>fr</th>");
		printf ("<th>ca</th>");
		printf ("<th>de</th>");
		printf ("<th>pt</th>");
		printf ("</tr>\n");
		for (i=0; en_std[i].text; i++) {
			printf ("<tr>");
			printf ("<th scope=\"row\" align=\"left\">");
			if (s = link_of (en_std[i].key)) {
				printf ("<a href=\"%s%s\">",base_href,s);
				printf ("%s",en_std[i].text);
				printf ("</a>");
				}
			else
				printf ("%s",en_std[i].text);
			printf ("</th>");

			s = es_std[i].text;
			needs_work = (strcmp (en_std[i].text,s) == 0);
			if (needs_work)
				printf ("<td bgcolor=\"#CCCCCC\">(%s)</td>",htmlentities(s));
			else
				printf ("<td>%s</td>",htmlentities(s));

			s = id_std[i].text;
			needs_work = (strcmp (en_std[i].text,s) == 0);
			if (needs_work)
				printf ("<td bgcolor=\"#CCCCCC\">(%s)</td>",htmlentities(s));
			else
				printf ("<td>%s</td>",htmlentities(s));

			s = fr_std[i].text;
			needs_work = (strcmp (en_std[i].text,s) == 0);
			if (needs_work)
				printf ("<td bgcolor=\"#CCCCCC\">(%s)</td>",htmlentities(s));
			else
				printf ("<td>%s</td>",htmlentities(s));

			s = ca_std[i].text;
			needs_work = (strcmp (en_std[i].text,s) == 0);
			if (needs_work)
				printf ("<td bgcolor=\"#CCCCCC\">(%s)</td>",htmlentities(s));
			else
				printf ("<td>%s</td>",htmlentities(s));

			s = de_std[i].text;
			needs_work = (strcmp (en_std[i].text,s) == 0);
			if (needs_work)
				printf ("<td bgcolor=\"#CCCCCC\">(%s)</td>",htmlentities(s));
			else
				printf ("<td>%s</td>",htmlentities(s));

			s = pt_std[i].text;
			needs_work = (strcmp (en_std[i].text,s) == 0);
			if (needs_work)
				printf ("<td bgcolor=\"#CCCCCC\">(%s)</td>",htmlentities(s));
			else
				printf ("<td>%s</td>",htmlentities(s));

			printf ("</tr>\n");
			}

		printf ("</table>\n");

		now = time (NULL);
		printf ("<p>Generated by <tt>print_names</tt> %s</p>\n",asctime(localtime(&now)));

		printf ("</body>\n");
		printf ("</html>\n");
		}

	}

#endif

/*----------------------------------------------------------------------*\
\*----------------------------------------------------------------------*/
